/* config/config.h.  Generated from config.h.in by configure.  */
/* config/config.h.in.  Generated from configure.ac by autoheader.  */

/* Define to 1 to call abort() on panics (internal errors), for debugging. */
/* #undef ABORT_ON_PANIC */

/* Define if building universal (internal helper macro) */
/* #undef AC_APPLE_UNIVERSAL_BUILD */

/* Define to 1 if compiled with -fdata-sections in CFLAGS */
#define CFLAGS_FDATA_SECTIONS 1

/* Define to 1 if compiled with -ffunction-sections in CFLAGS */
#define CFLAGS_FFUNCTION_SECTIONS 1

/* Define to 1 if compiled with -fgnu89-inline in CFLAGS */
/* #undef CFLAGS_FGNU89_INLINE */

/* Define to 1 if compiled with -fno-common in CFLAGS */
#define CFLAGS_FNO_COMMON 1

/* Define to 1 if compiled with -fno-omit-frame-pointer in CFLAGS */
/* #undef CFLAGS_FNO_OMIT_FRAME_POINTER */

/* Define to 1 if compiled with -fsanitize=address in CFLAGS */
/* #undef CFLAGS_FSANITIZE_ADDRESS */

/* Define to 1 if compiled with -fsanitize=undefined in CFLAGS */
/* #undef CFLAGS_FSANITIZE_UNDEFINED */

/* Define to 1 if compiled with -ftrivial-auto-var-init=zero in CFLAGS */
#define CFLAGS_FTRIVIAL_AUTO_VAR_INIT_ZERO 1

/* Define to 1 if compiled with -fvisibility=hidden in CFLAGS */
#define CFLAGS_FVISIBILITY_HIDDEN 1

/* Define to 1 if compiled with -fwrapv in CFLAGS */
#define CFLAGS_FWRAPV 1

/* Define to 1 if compiled with -g3 in CFLAGS */
#define CFLAGS_G3 1

/* Define to 1 if compiled with -ggdb in CFLAGS */
#define CFLAGS_GGDB 1

/* Define to 1 if compiled with -O0 in CFLAGS */
/* #undef CFLAGS_O0 */

/* Define to 1 if compiled with -Og in CFLAGS */
/* #undef CFLAGS_OG */

/* Define to 1 if compiled with -pedantic in CFLAGS */
#define CFLAGS_PEDANTIC 1

/* Define to 1 if compiled with -pg in CFLAGS */
/* #undef CFLAGS_PG */

/* Define to 1 if compiled with -std=c11 in CFLAGS */
/* #undef CFLAGS_STD_C11 */

/* Define to 1 if compiled with -std=c17 in CFLAGS */
/* #undef CFLAGS_STD_C17 */

/* Define to 1 if compiled with -std=c23 in CFLAGS */
#define CFLAGS_STD_C23 1

/* Define to 1 if compiled with -std=c99 in CFLAGS */
/* #undef CFLAGS_STD_C99 */

/* Define to 1 if compiled with -U__STRICT_ANSI__ in CFLAGS */
#define CFLAGS_U_STRICT_ANSI_ 1

/* Define to 1 if compiled with -W in CFLAGS */
#define CFLAGS_W 1

/* Define to 1 if compiled with -Wall in CFLAGS */
#define CFLAGS_WALL 1

/* Define to 1 if compiled with -Werror in CFLAGS */
/* #undef CFLAGS_WERROR */

/* Define to 1 if compiled with -Werror=comment in CFLAGS */
#define CFLAGS_WERROR_COMMENT 1

/* Define to 1 if compiled with -Werror=implicit in CFLAGS */
#define CFLAGS_WERROR_IMPLICIT 1

/* Define to 1 if compiled with -Werror=missing-braces in CFLAGS */
#define CFLAGS_WERROR_MISSING_BRACES 1

/* Define to 1 if compiled with -Werror=missing-declarations in CFLAGS */
#define CFLAGS_WERROR_MISSING_DECLARATIONS 1

/* Define to 1 if compiled with -Werror=missing-prototypes in CFLAGS */
#define CFLAGS_WERROR_MISSING_PROTOTYPES 1

/* Define to 1 if compiled with -Werror=pointer-arith in CFLAGS */
#define CFLAGS_WERROR_POINTER_ARITH 1

/* Define to 1 if compiled with -Werror=return-type in CFLAGS */
#define CFLAGS_WERROR_RETURN_TYPE 1

/* Define to 1 if compiled with -Werror=strict-prototypes in CFLAGS */
#define CFLAGS_WERROR_STRICT_PROTOTYPES 1

/* Define to 1 if compiled with -Werror=trigraphs in CFLAGS */
#define CFLAGS_WERROR_TRIGRAPHS 1

/* Define to 1 if compiled with -Werror=unknown-warning-option in CFLAGS */
/* #undef CFLAGS_WERROR_UNKNOWN_WARNING_OPTION */

/* Define to 1 if compiled with -Werror=vla in CFLAGS */
#define CFLAGS_WERROR_VLA 1

/* Define to 1 if compiled with -Wno-long-long in CFLAGS */
#define CFLAGS_WNO_LONG_LONG 1

/* Define to 1 if compiled with -Wno-pedantic-ms-format in CFLAGS */
/* #undef CFLAGS_WNO_PEDANTIC_MS_FORMAT */

/* Define to 1 if compiled with -Wno-shift-negative-value in CFLAGS */
#define CFLAGS_WNO_SHIFT_NEGATIVE_VALUE 1

/* Define to 1 if compiled with -Wno-stringop-truncation in CFLAGS */
#define CFLAGS_WNO_STRINGOP_TRUNCATION 1

/* Define to 1 if compiled with -Wno-variadic-macros in CFLAGS */
#define CFLAGS_WNO_VARIADIC_MACROS 1

/* Define to 1 if compiled with -Wsuggest-attribute=cold in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_COLD */

/* Define to 1 if compiled with -Wsuggest-attribute=const in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_CONST */

/* Define to 1 if compiled with -Wsuggest-attribute=format in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_FORMAT */

/* Define to 1 if compiled with -Wsuggest-attribute=malloc in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_MALLOC */

/* Define to 1 if compiled with -Wsuggest-attribute=noreturn in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_NORETURN */

/* Define to 1 if compiled with -Wsuggest-attribute=pure in CFLAGS */
/* #undef CFLAGS_WSUGGEST_ATTRIBUTE_PURE */

/* Define to 1 if compiled with -Werror=attributes in CPPFLAGS */
#define CPPFLAGS_WERROR_ATTRIBUTES 1

/* Define to 1 if you have the 'access' function. */
#define HAVE_ACCESS 1

/* Define to 1 if you have the <arpa/inet.h> header file. */
#define HAVE_ARPA_INET_H 1

/* "Define to 1 if you have the `bswap_16' intrinsic function." */
/* #undef HAVE_BSWAP_16 */

/* "Define to 1 if you have the `bswap_32' intrinsic function." */
/* #undef HAVE_BSWAP_32 */

/* "Define to 1 if you have the `bswap_64' intrinsic function." */
/* #undef HAVE_BSWAP_64 */

/* Define to 1 if you have the <byteswap.h> header file. */
#define HAVE_BYTESWAP_H 1

/* Define to 1 if you have the 'canonicalize_file_name' function. */
#define HAVE_CANONICALIZE_FILE_NAME 1

/* "Define to 1 if you have the `cpu_to_be16' intrinsic function." */
/* #undef HAVE_CPU_TO_BE16 */

/* "Define to 1 if you have the `cpu_to_be32' intrinsic function." */
/* #undef HAVE_CPU_TO_BE32 */

/* "Define to 1 if you have the `cpu_to_be64' intrinsic function." */
/* #undef HAVE_CPU_TO_BE64 */

/* "Define to 1 if you have the `cpu_to_le16' intrinsic function." */
/* #undef HAVE_CPU_TO_LE16 */

/* "Define to 1 if you have the `cpu_to_le32' intrinsic function." */
/* #undef HAVE_CPU_TO_LE32 */

/* "Define to 1 if you have the `cpu_to_le64' intrinsic function." */
/* #undef HAVE_CPU_TO_LE64 */

/* Define to 1 if you have the declaration of 'strcasecmp', and to 0 if you
   don't. */
#define HAVE_DECL_STRCASECMP 1

/* Define to 1 if you have the declaration of 'stricmp', and to 0 if you
   don't. */
#define HAVE_DECL_STRICMP 0

/* Define to 1 if you have the declaration of 'strlcpy', and to 0 if you
   don't. */
#define HAVE_DECL_STRLCPY 1

/* Define to 1 if you have the declaration of 'strncasecmp', and to 0 if you
   don't. */
#define HAVE_DECL_STRNCASECMP 1

/* Define to 1 if you have the declaration of 'strnicmp', and to 0 if you
   don't. */
#define HAVE_DECL_STRNICMP 0

/* Define to 1 if you have the declaration of 'strnlen', and to 0 if you
   don't. */
#define HAVE_DECL_STRNLEN 1

/* Define to 1 if you have the declaration of 'strsep', and to 0 if you don't.
   */
#define HAVE_DECL_STRSEP 1

/* Define to 1 if you have the <endian.h> header file. */
#define HAVE_ENDIAN_H 1

/* Define to 1 if you have the 'faccessat' function. */
#define HAVE_FACCESSAT 1

/* Define to 1 if you have the <fcntl.h> header file. */
#define HAVE_FCNTL_H 1

/* Define to 1 if you have the 'fileno' function. */
#define HAVE_FILENO 1

/* Define to 1 if fseeko (and ftello) are declared in stdio.h. */
#define HAVE_FSEEKO 1

/* Define to 1 if you have the 'fstat' function. */
#define HAVE_FSTAT 1

/* Define to 1 if you have the 'ftruncate' function. */
#define HAVE_FTRUNCATE 1

/* Define to 1 if your compiler supports the alloc_size function attribute */
#define HAVE_FUNC_ATTRIBUTE_1_ALLOC_SIZE 1

/* Define to 1 if your compiler supports the alloc_size function attribute */
#define HAVE_FUNC_ATTRIBUTE_2_ALLOC_SIZE 1

/* Define to 1 if your compiler supports the format function attribute */
#define HAVE_FUNC_ATTRIBUTE_3_FORMAT 1

/* Define to 1 if your compiler supports the cold function attribute */
#define HAVE_FUNC_ATTRIBUTE_COLD 1

/* Define to 1 if your compiler supports the const function attribute */
#define HAVE_FUNC_ATTRIBUTE_CONST 1

/* Define to 1 if your compiler supports the error attribute on functions */
#define HAVE_FUNC_ATTRIBUTE_ERROR 1

/* Define to 1 if your compiler supports the malloc function attribute */
#define HAVE_FUNC_ATTRIBUTE_MALLOC 1

/* Define to 1 if your compiler supports the maybe_unused function attribute
   */
/* #undef HAVE_FUNC_ATTRIBUTE_MAYBE_UNUSED */

/* Define to 1 if your compiler supports the noreturn function attribute */
#define HAVE_FUNC_ATTRIBUTE_NORETURN 1

/* Define to 1 if your compiler supports the pure function attribute */
#define HAVE_FUNC_ATTRIBUTE_PURE 1

/* Define to 1 if your compiler supports the reproducible function attribute
   */
#define HAVE_FUNC_ATTRIBUTE_REPRODUCIBLE 1

/* Define to 1 if your compiler supports the returns_nonnull function
   attribute */
#define HAVE_FUNC_ATTRIBUTE_RETURNS_NONNULL 1

/* Define to 1 if your compiler supports the sentinel function attribute */
#define HAVE_FUNC_ATTRIBUTE_SENTINEL 1

/* Define to 1 if your compiler supports the unsequenced function attribute */
#define HAVE_FUNC_ATTRIBUTE_UNSEQUENCED 1

/* Define to 1 if your compiler supports the unused function attribute */
#define HAVE_FUNC_ATTRIBUTE_UNUSED 1

/* Define to 1 if the compiler supports __func__ or equivalent. */
#define HAVE_FUNC_NAME 1

/* Define to 1 if your compiler supports the alloc_size attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_1_ALLOC_SIZE 1

/* Define to 1 if your compiler supports the alloc_size attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_2_ALLOC_SIZE 1

/* Define to 1 if your compiler supports the format attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_3_FORMAT 1

/* Define to 1 if your compiler supports the cold attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_COLD 1

/* Define to 1 if your compiler supports the const attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_CONST 1

/* Define to 1 if your compiler supports the malloc attribute on function
   pointers */
/* #undef HAVE_FUNC_PTR_ATTRIBUTE_MALLOC */

/* Define to 1 if your compiler supports the maybe_unused attribute on
   function pointers */
/* #undef HAVE_FUNC_PTR_ATTRIBUTE_MAYBE_UNUSED */

/* Define to 1 if your compiler supports the noreturn attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_NORETURN 1

/* Define to 1 if your compiler supports the pure attribute on function
   pointers */
/* #undef HAVE_FUNC_PTR_ATTRIBUTE_PURE */

/* Define to 1 if your compiler supports the reproducible attribute on
   function pointers */
/* #undef HAVE_FUNC_PTR_ATTRIBUTE_REPRODUCIBLE */

/* Define to 1 if your compiler supports the returns_nonnull attribute on
   function pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_RETURNS_NONNULL 1

/* Define to 1 if your compiler supports the sentinel attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_SENTINEL 1

/* Define to 1 if your compiler supports the unsequenced attribute on function
   pointers */
/* #undef HAVE_FUNC_PTR_ATTRIBUTE_UNSEQUENCED */

/* Define to 1 if your compiler supports the unused attribute on function
   pointers */
#define HAVE_FUNC_PTR_ATTRIBUTE_UNUSED 1

/* Define to 1 if you have the 'getgid' function. */
#define HAVE_GETGID 1

/* Define to 1 if you have the 'getpagesize' function. */
#define HAVE_GETPAGESIZE 1

/* Define to 1 if you have the 'getrlimit' function. */
#define HAVE_GETRLIMIT 1

/* Define to 1 if you have the 'getuid' function. */
#define HAVE_GETUID 1

/* "Define to 1 if you have the `htobe16' intrinsic function." */
#define HAVE_HTOBE16 1

/* "Define to 1 if you have the `htobe32' intrinsic function." */
#define HAVE_HTOBE32 1

/* "Define to 1 if you have the `htobe64' intrinsic function." */
#define HAVE_HTOBE64 1

/* "Define to 1 if you have the `htole16' intrinsic function." */
#define HAVE_HTOLE16 1

/* "Define to 1 if you have the `htole32' intrinsic function." */
#define HAVE_HTOLE32 1

/* "Define to 1 if you have the `htole64' intrinsic function." */
#define HAVE_HTOLE64 1

/* "Define to 1 if you have the `htonl' intrinsic function." */
/* #undef HAVE_HTONL */

/* "Define to 1 if you have the `htonq' intrinsic function." */
/* #undef HAVE_HTONQ */

/* "Define to 1 if you have the `htons' intrinsic function." */
/* #undef HAVE_HTONS */

/* Define to 1 if you have the <intrin.h> header file. */
/* #undef HAVE_INTRIN_H */

/* Define to 1 if you have the <inttypes.h> header file. */
#define HAVE_INTTYPES_H 1

/* Define to 1 if you have the <io.h> header file. */
/* #undef HAVE_IO_H */

/* Define to 1 if you have the 'isascii' function. */
#define HAVE_ISASCII 1

/* Define to 1 if you have the 'iscntrl' function. */
#define HAVE_ISCNTRL 1

/* Define to 1 if you have the <machine/endian.h> header file. */
/* #undef HAVE_MACHINE_ENDIAN_H */

/* Define to 1 if you have the 'mempcpy' function. */
#define HAVE_MEMPCPY 1

/* Define to 1 if you have the 'mempset' function. */
/* #undef HAVE_MEMPSET */

/* Define to 1 if you have the <minix/config.h> header file. */
/* #undef HAVE_MINIX_CONFIG_H */

/* Define to 1 if you have a working 'mmap' system call. */
#define HAVE_MMAP 1

/* Define to 1 if you have the 'pathconf' function. */
#define HAVE_PATHCONF 1

/* Define to 1 if you have the 'realpath' function. */
#define HAVE_REALPATH 1

/* Define to 1 if you have some version of the snprintf function. */
#define HAVE_SNPRINTF 1

/* Define to 1 if you have the 'stat' function. */
#define HAVE_STAT 1

/* Define to 1 if you have the <stdarg.h> header file. */
#define HAVE_STDARG_H 1

/* Define to 1 if you have the <stdbit.h> header file. */
#define HAVE_STDBIT_H 1

/* Define to 1 if stdbool.h conforms to C99. */
#define HAVE_STDBOOL_H 1

/* Define to 1 if your compiler supports C99 extern inline */
#define HAVE_STDC_INLINE 1

/* "Define to 1 if you have the `stdc_leading_zeros' intrinsic function." */
#define HAVE_STDC_LEADING_ZEROS 1

/* Define to 1 if you have the <stdint.h> header file. */
#define HAVE_STDINT_H 1

/* Define to 1 if you have the <stdio.h> header file. */
#define HAVE_STDIO_H 1

/* Define to 1 if you have the <stdlib.h> header file. */
#define HAVE_STDLIB_H 1

/* Define to 1 if you have the <stdnoreturn.h> header file. */
#define HAVE_STDNORETURN_H 1

/* Define to 1 if you have the 'strcasecmp' function. */
#define HAVE_STRCASECMP 1

/* Define to 1 if you have the 'stricmp' function. */
/* #undef HAVE_STRICMP */

/* Define to 1 if you have the <strings.h> header file. */
#define HAVE_STRINGS_H 1

/* Define to 1 if you have the <string.h> header file. */
#define HAVE_STRING_H 1

/* Define to 1 if you have the 'strlcpy' function. */
#define HAVE_STRLCPY 1

/* Define to 1 if you have the 'strncasecmp' function. */
#define HAVE_STRNCASECMP 1

/* Define to 1 if you have the 'strnicmp' function. */
/* #undef HAVE_STRNICMP */

/* Define to 1 if you have the 'strnlen' function. */
#define HAVE_STRNLEN 1

/* Define to 1 if you have the 'strsep' function. */
#define HAVE_STRSEP 1

/* Define to 1 if the system has the type 'struct stat'. */
#define HAVE_STRUCT_STAT 1

/* Define to 1 if the system has the type 'struct _stati64'. */
/* #undef HAVE_STRUCT__STATI64 */

/* Define to 1 if you have the 'sysconf' function. */
#define HAVE_SYSCONF 1

/* Define to 1 if you have the <sys/endian.h> header file. */
/* #undef HAVE_SYS_ENDIAN_H */

/* Define to 1 if you have the <sys/mman.h> header file. */
#define HAVE_SYS_MMAN_H 1

/* Define to 1 if you have the <sys/param.h> header file. */
#define HAVE_SYS_PARAM_H 1

/* Define to 1 if you have the <sys/resource.h> header file. */
#define HAVE_SYS_RESOURCE_H 1

/* Define to 1 if you have the <sys/stat.h> header file. */
#define HAVE_SYS_STAT_H 1

/* Define to 1 if you have the <sys/types.h> header file. */
#define HAVE_SYS_TYPES_H 1

/* "Define to 1 if you have the `S_ISREG' intrinsic function." */
#define HAVE_S_ISREG 1

/* Define to 1 if you have some version of the typeof operator. */
#define HAVE_TYPEOF 1

/* Define to 1 if the system has the type 'uintmax_t'. */
#define HAVE_UINTMAX_T 1

/* Define to 1 if the system has the type 'uintptr_t'. */
#define HAVE_UINTPTR_T 1

/* Define to 1 if you have the <unistd.h> header file. */
#define HAVE_UNISTD_H 1

/* Define to 1 if the system has the type 'unsigned long long int'. */
#define HAVE_UNSIGNED_LONG_LONG_INT 1

/* define to 1 if your compiler supports the , ## __VA_ARGS__ hack. */
/* #undef HAVE_VARADIC_MACROS_COMMA_HACK */

/* define to 1 if your compiler supports C99 __VA_ARGS__ variadic macros. */
#define HAVE_VARIADIC_MACROS 1

/* Define to 1 if you have some version of the vsnprintf function. */
#define HAVE_VSNPRINTF 1

/* Define to 1 if you have the <wchar.h> header file. */
#define HAVE_WCHAR_H 1

/* Define to 1 if you have the '_access' function. */
/* #undef HAVE__ACCESS */

/* "Define to 1 if you have the `_BitScanReverse' intrinsic function." */
/* #undef HAVE__BITSCANREVERSE */

/* "Define to 1 if you have the `_BitScanReverse64' intrinsic function." */
/* #undef HAVE__BITSCANREVERSE64 */

/* Define to 1 if the system has the type '_Bool'. */
#define HAVE__BOOL 1

/* "Define to 1 if you have the `_byteswap_uint64' intrinsic function." */
/* #undef HAVE__BYTESWAP_UINT64 */

/* "Define to 1 if you have the `_byteswap_ulong' intrinsic function." */
/* #undef HAVE__BYTESWAP_ULONG */

/* "Define to 1 if you have the `_byteswap_ushort' intrinsic function." */
/* #undef HAVE__BYTESWAP_USHORT */

/* Define to 1 if you have the '_chsize' function. */
/* #undef HAVE__CHSIZE */

/* Define to 1 if you have the '_chsize_s' function. */
/* #undef HAVE__CHSIZE_S */

/* Define to 1 if you have the '_fileno' function. */
/* #undef HAVE__FILENO */

/* Define to 1 if you have the '_fseeki64' function. */
/* #undef HAVE__FSEEKI64 */

/* Define to 1 if you have the '_fstati64' function. */
/* #undef HAVE__FSTATI64 */

/* Define to 1 if you have the '_fullpath' function. */
/* #undef HAVE__FULLPATH */

/* Define to 1 if you have the '_stati64' function. */
/* #undef HAVE__STATI64 */

/* "Define to 1 if you have the `__builtin_bswap16' intrinsic function." */
/* #undef HAVE___BUILTIN_BSWAP16 */

/* "Define to 1 if you have the `__builtin_bswap32' intrinsic function." */
/* #undef HAVE___BUILTIN_BSWAP32 */

/* "Define to 1 if you have the `__builtin_bswap64' intrinsic function." */
/* #undef HAVE___BUILTIN_BSWAP64 */

/* "Define to 1 if you have the `__builtin_choose_expr' intrinsic function."
   */
#define HAVE___BUILTIN_CHOOSE_EXPR 1

/* "Define to 1 if you have the `__builtin_clz' intrinsic function." */
/* #undef HAVE___BUILTIN_CLZ */

/* "Define to 1 if you have the `__builtin_clzll' intrinsic function." */
/* #undef HAVE___BUILTIN_CLZLL */

/* "Define to 1 if you have the `__builtin_constant_p' intrinsic function." */
#define HAVE___BUILTIN_CONSTANT_P 1

/* "Define to 1 if you have the `__builtin_expect' intrinsic function." */
#define HAVE___BUILTIN_EXPECT 1

/* "Define to 1 if you have the `__builtin_prefetch' intrinsic function." */
#define HAVE___BUILTIN_PREFETCH 1

/* "Define to 1 if you have the `__builtin_unreachable' intrinsic function."
   */
#define HAVE___BUILTIN_UNREACHABLE 1

/* "Define to 1 if you have the `__cpu_to_be16' intrinsic function." */
/* #undef HAVE___CPU_TO_BE16 */

/* "Define to 1 if you have the `__cpu_to_be64' intrinsic function." */
/* #undef HAVE___CPU_TO_BE64 */

/* "Define to 1 if you have the `__cpu_to_le16' intrinsic function." */
/* #undef HAVE___CPU_TO_LE16 */

/* "Define to 1 if you have the `__cpu_to_le32' intrinsic function." */
/* #undef HAVE___CPU_TO_LE32 */

/* "Define to 1 if you have the `__cpu_to_le64' intrinsic function." */
/* #undef HAVE___CPU_TO_LE64 */

/* Define to 1 if compiled with -Wl */
#define LDFLAGS_WL_AS_NEEDED 1

/* Define to 1 if compiled with -Wl */
#define LDFLAGS_WL_GC_SECTIONS 1

/* Define to the address where bug reports for this package should be sent. */
#define PACKAGE_BUGREPORT "https://bugs.nasm.us/"

/* Define to the full name of this package. */
#define PACKAGE_NAME "nasm"

/* Define to the full name and version of this package. */
#define PACKAGE_STRING "nasm 3.02"

/* Define to the one symbol short name of this package. */
#define PACKAGE_TARNAME "nasm"

/* Define to the home page for this package. */
#define PACKAGE_URL "https://www.nasm.us/"

/* Define to the version of this package. */
#define PACKAGE_VERSION "3.02"

/* Define to 1 if all of the C89 standard headers exist (not just the ones
   required in a freestanding environment). This macro is provided for
   backward compatibility; new code need not use it. */
#define STDC_HEADERS 1

/* Enable extensions on AIX, Interix, z/OS.  */
#ifndef _ALL_SOURCE
# define _ALL_SOURCE 1
#endif
/* Enable general extensions on macOS.  */
#ifndef _DARWIN_C_SOURCE
# define _DARWIN_C_SOURCE 1
#endif
/* Enable general extensions on Solaris.  */
#ifndef __EXTENSIONS__
# define __EXTENSIONS__ 1
#endif
/* Enable GNU extensions on systems that have them.  */
#ifndef _GNU_SOURCE
# define _GNU_SOURCE 1
#endif
/* Enable X/Open compliant socket functions that do not require linking
   with -lxnet on HP-UX 11.11.  */
#ifndef _HPUX_ALT_XOPEN_SOCKET_API
# define _HPUX_ALT_XOPEN_SOCKET_API 1
#endif
/* Identify the host operating system as Minix.
   This macro does not affect the system headers' behavior.
   A future release of Autoconf may stop defining this macro.  */
#ifndef _MINIX
/* # undef _MINIX */
#endif
/* Enable general extensions on NetBSD.
   Enable NetBSD compatibility extensions on Minix.  */
#ifndef _NETBSD_SOURCE
# define _NETBSD_SOURCE 1
#endif
/* Enable OpenBSD compatibility extensions on NetBSD.
   Oddly enough, this does nothing on OpenBSD.  */
#ifndef _OPENBSD_SOURCE
# define _OPENBSD_SOURCE 1
#endif
/* Define to 1 if needed for POSIX-compatible behavior.  */
#ifndef _POSIX_SOURCE
/* # undef _POSIX_SOURCE */
#endif
/* Define to 2 if needed for POSIX-compatible behavior.  */
#ifndef _POSIX_1_SOURCE
/* # undef _POSIX_1_SOURCE */
#endif
/* Enable POSIX-compatible threading on Solaris.  */
#ifndef _POSIX_PTHREAD_SEMANTICS
# define _POSIX_PTHREAD_SEMANTICS 1
#endif
/* Enable extensions specified by ISO/IEC TS 18661-5:2014.  */
#ifndef __STDC_WANT_IEC_60559_ATTRIBS_EXT__
# define __STDC_WANT_IEC_60559_ATTRIBS_EXT__ 1
#endif
/* Enable extensions specified by ISO/IEC TS 18661-1:2014.  */
#ifndef __STDC_WANT_IEC_60559_BFP_EXT__
# define __STDC_WANT_IEC_60559_BFP_EXT__ 1
#endif
/* Enable extensions specified by ISO/IEC TS 18661-2:2015.  */
#ifndef __STDC_WANT_IEC_60559_DFP_EXT__
# define __STDC_WANT_IEC_60559_DFP_EXT__ 1
#endif
/* Enable extensions specified by C23 Annex F.  */
#ifndef __STDC_WANT_IEC_60559_EXT__
# define __STDC_WANT_IEC_60559_EXT__ 1
#endif
/* Enable extensions specified by ISO/IEC TS 18661-4:2015.  */
#ifndef __STDC_WANT_IEC_60559_FUNCS_EXT__
# define __STDC_WANT_IEC_60559_FUNCS_EXT__ 1
#endif
/* Enable extensions specified by C23 Annex H and ISO/IEC TS 18661-3:2015.  */
#ifndef __STDC_WANT_IEC_60559_TYPES_EXT__
# define __STDC_WANT_IEC_60559_TYPES_EXT__ 1
#endif
/* Enable extensions specified by ISO/IEC TR 24731-2:2010.  */
#ifndef __STDC_WANT_LIB_EXT2__
# define __STDC_WANT_LIB_EXT2__ 1
#endif
/* Enable extensions specified by ISO/IEC 24747:2009.  */
#ifndef __STDC_WANT_MATH_SPEC_FUNCS__
# define __STDC_WANT_MATH_SPEC_FUNCS__ 1
#endif
/* Enable extensions on HP NonStop.  */
#ifndef _TANDEM_SOURCE
# define _TANDEM_SOURCE 1
#endif
/* Enable X/Open extensions.  Define to 500 only if necessary
   to make mbstate_t available.  */
#ifndef _XOPEN_SOURCE
/* # undef _XOPEN_SOURCE */
#endif


/* Define to 1 if your processor stores words with the most significant byte
   first (like Motorola and SPARC, unlike Intel and VAX). */
/* #undef WORDS_BIGENDIAN */

/* Define to 1 if your processor stores words with the least significant byte
   first (like Intel and VAX, unlike Motorola and SPARC). */
#define WORDS_LITTLEENDIAN 1

/* Number of bits in a file offset, on hosts where this is settable. */
/* #undef _FILE_OFFSET_BITS */

/* Define to 1 if compiled with -flto in -flto=auto */
/* #undef _FLTO_AUTO_FLTO */

/* Define to 1 if necessary to make fseeko visible. */
/* #undef _LARGEFILE_SOURCE */

/* Define to 1 on platforms where this makes off_t a 64-bit type. */
/* #undef _LARGE_FILES */

/* Number of bits in time_t, on hosts where this is settable. */
/* #undef _TIME_BITS */

/* Define to 1 on platforms where this makes time_t a 64-bit type. */
/* #undef __MINGW_USE_VC2005_COMPAT */

/* Define if __func__ is called something else on your compiler. */
/* #undef __func__ */

/* Define to empty if 'const' does not conform to ANSI C. */
/* #undef const */

/* Define to '__inline__' or '__inline' if that's what the C compiler
   calls it, or to nothing if 'inline' is not supported under any name.  */
#ifndef __cplusplus
/* #undef inline */
#endif

/* Define to the equivalent of the C99 'restrict' keyword, or to
   nothing if this is not supported.  Do not define if restrict is
   supported only directly.  */
#define restrict __restrict__
/* Work around a bug in older versions of Sun C++, which did not
   #define __restrict__ or support _Restrict or __restrict__
   even though the corresponding Sun C compiler ended up with
   "#define restrict _Restrict" or "#define restrict __restrict__"
   in the previous line.  This workaround can be removed once
   we assume Oracle Developer Studio 12.5 (2016) or later.  */
#if defined __SUNPRO_CC && !defined __RESTRICT && !defined __restrict__
# define _Restrict
# define __restrict__
#endif

/* Define as 'unsigned int' if <stddef.h> doesn't define. */
/* #undef size_t */

/* Define if your snprintf function is not named snprintf. */
/* #undef snprintf */

/* Define if your typeof operator is not named `typeof'. */
/* #undef typeof */

/* Define to the widest unsigned integer type if <stdint.h> and <inttypes.h>
   do not define. */
/* #undef uintmax_t */

/* Define to the type of an unsigned integer type wide enough to hold a
   pointer, if such a type exists, and if the system does not define it. */
/* #undef uintptr_t */

/* Define if your vsnprintf function is not named vsnprintf. */
/* #undef vsnprintf */


/* Begin unconfig.h */

#ifndef alloc_size_func2
# ifdef HAVE_FUNC_ATTRIBUTE_2_ALLOC_SIZE
#  define alloc_size_func2(x1,x2) ATTRIBUTE(alloc_size(x1,x2))
# else
#  define alloc_size_func2(x1,x2)
# endif
#endif

#ifndef alloc_size_func2_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_2_ALLOC_SIZE
#  define alloc_size_func2_ptr(x1,x2) ATTRIBUTE(alloc_size(x1,x2))
# else
#  define alloc_size_func2_ptr(x1,x2)
# endif
#endif

#ifndef end_with_null
# ifdef HAVE_FUNC_ATTRIBUTE_SENTINEL
#  define end_with_null ATTRIBUTE(sentinel)
# else
#  define end_with_null
# endif
#endif

#ifndef end_with_null_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_SENTINEL
#  define end_with_null_ptr ATTRIBUTE(sentinel)
# else
#  define end_with_null_ptr
# endif
#endif

#ifndef format_func3
# ifdef HAVE_FUNC_ATTRIBUTE_3_FORMAT
#  define format_func3(x1,x2,x3) ATTRIBUTE(format(x1,x2,x3))
# else
#  define format_func3(x1,x2,x3)
# endif
#endif

#ifndef format_func3_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_3_FORMAT
#  define format_func3_ptr(x1,x2,x3) ATTRIBUTE(format(x1,x2,x3))
# else
#  define format_func3_ptr(x1,x2,x3)
# endif
#endif

#ifndef const_func
# ifdef HAVE_FUNC_ATTRIBUTE_CONST
#  define const_func ATTRIBUTE(const)
# else
#  define const_func
# endif
#endif

#ifndef const_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_CONST
#  define const_func_ptr ATTRIBUTE(const)
# else
#  define const_func_ptr
# endif
#endif

#ifndef unsequenced_func
# ifdef HAVE_FUNC_ATTRIBUTE_UNSEQUENCED
#  define unsequenced_func ATTRIBUTE(unsequenced)
# else
#  define unsequenced_func
# endif
#endif

#ifndef unsequenced_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_UNSEQUENCED
#  define unsequenced_func_ptr ATTRIBUTE(unsequenced)
# else
#  define unsequenced_func_ptr
# endif
#endif

#ifndef noreturn_func
# ifdef HAVE_FUNC_ATTRIBUTE_NORETURN
#  define noreturn_func ATTRIBUTE(noreturn)
# else
#  define noreturn_func
# endif
#endif

#ifndef reproducible_func
# ifdef HAVE_FUNC_ATTRIBUTE_REPRODUCIBLE
#  define reproducible_func ATTRIBUTE(reproducible)
# else
#  define reproducible_func
# endif
#endif

#ifndef reproducible_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_REPRODUCIBLE
#  define reproducible_func_ptr ATTRIBUTE(reproducible)
# else
#  define reproducible_func_ptr
# endif
#endif

#ifndef pure_func
# ifdef HAVE_FUNC_ATTRIBUTE_PURE
#  define pure_func ATTRIBUTE(pure)
# else
#  define pure_func
# endif
#endif

#ifndef pure_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_PURE
#  define pure_func_ptr ATTRIBUTE(pure)
# else
#  define pure_func_ptr
# endif
#endif

#ifndef unlikely_func
# ifdef HAVE_FUNC_ATTRIBUTE_COLD
#  define unlikely_func ATTRIBUTE(cold)
# else
#  define unlikely_func
# endif
#endif

#ifndef unlikely_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_COLD
#  define unlikely_func_ptr ATTRIBUTE(cold)
# else
#  define unlikely_func_ptr
# endif
#endif

#ifndef maybe_unused_func
# ifdef HAVE_FUNC_ATTRIBUTE_MAYBE_UNUSED
#  define maybe_unused_func ATTRIBUTE(maybe_unused)
# else
#  define maybe_unused_func
# endif
#endif

#ifndef maybe_unused_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_MAYBE_UNUSED
#  define maybe_unused_func_ptr ATTRIBUTE(maybe_unused)
# else
#  define maybe_unused_func_ptr
# endif
#endif

#ifndef unused_func
# ifdef HAVE_FUNC_ATTRIBUTE_UNUSED
#  define unused_func ATTRIBUTE(unused)
# else
#  define unused_func
# endif
#endif

#ifndef unused_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_UNUSED
#  define unused_func_ptr ATTRIBUTE(unused)
# else
#  define unused_func_ptr
# endif
#endif

#ifndef noreturn_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_NORETURN
#  define noreturn_func_ptr ATTRIBUTE(noreturn)
# else
#  define noreturn_func_ptr
# endif
#endif

#ifndef never_null
# ifdef HAVE_FUNC_ATTRIBUTE_RETURNS_NONNULL
#  define never_null ATTRIBUTE(returns_nonnull)
# else
#  define never_null
# endif
#endif

#ifndef never_null_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_RETURNS_NONNULL
#  define never_null_ptr ATTRIBUTE(returns_nonnull)
# else
#  define never_null_ptr
# endif
#endif

#ifndef malloc_func
# ifdef HAVE_FUNC_ATTRIBUTE_MALLOC
#  define malloc_func ATTRIBUTE(malloc)
# else
#  define malloc_func
# endif
#endif

#ifndef malloc_func_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_MALLOC
#  define malloc_func_ptr ATTRIBUTE(malloc)
# else
#  define malloc_func_ptr
# endif
#endif

#ifndef alloc_size_func1
# ifdef HAVE_FUNC_ATTRIBUTE_1_ALLOC_SIZE
#  define alloc_size_func1(x1) ATTRIBUTE(alloc_size(x1))
# else
#  define alloc_size_func1(x1)
# endif
#endif

#ifndef alloc_size_func1_ptr
# ifdef HAVE_FUNC_PTR_ATTRIBUTE_1_ALLOC_SIZE
#  define alloc_size_func1_ptr(x1) ATTRIBUTE(alloc_size(x1))
# else
#  define alloc_size_func1_ptr(x1)
# endif
#endif
