/* This file auto-generated from insns.dat by insns.pl - don't edit it */

#include "nasm.h"
#include "insns.h"

static const struct itemplate instrux[] = {
    /*   0 : NOP void [          osz norexb nof3 90                ] BESTDIS,8086 */
        /* 330 314 326 001(90) : BESTDIS,8086 */
        {I_NOP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66387, 2, 47},
    /*   1 : NOP rm16 [m:        o16 0f 1f /0                      ] BESTDIS,P6 */
        /* 320 355 001(1f) 200 : BESTDIS,P6 */
        {I_NOP, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66399, 4, 49},
    /*   2 : NOP rm32 [m:        o32 0f 1f /0                      ] BESTDIS,P6 */
        /* 321 355 001(1f) 200 : BESTDIS,P6 */
        {I_NOP, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66405, 4, 50},
    /*   3 : NOP rm64 [m:        o64 0f 1f /0                      ] LONG,BESTDIS,PROT,X86_64 */
        /* 324 355 001(1f) 200 : LONG,BESTDIS,PROT,X86_64 */
        {I_NOP, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66411, 5, 51},
    /*   4 : MOV reg_al,mem_offs [-i:       o8 a0 iwdq                        ] SM0-1,NOAPX,8086 */
        /* 001(a0) 045 : SM0,SM1,NOAPX,8086 */
        {I_MOV, 2, {REG_AL|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+72584, 6, 54},
    /*   5 : MOV reg_ax,mem_offs [-i:       o16 a1 iwdq                       ] SM0-1,NOAPX,8086 */
        /* 320 001(a1) 045 : SM0,SM1,NOAPX,8086 */
        {I_MOV, 2, {REG_AX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71319, 6, 55},
    /*   6 : MOV reg_eax,mem_offs [-i:       o32 a1 iwdq                       ] SM0-1,NOAPX,ZU,386 */
        /* 321 001(a1) 045 : SM0,SM1,NOAPX,ZU,386 */
        {I_MOV, 2, {REG_EAX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71324, 7, 56},
    /*   7 : MOV reg_rax,mem_offs [-i:       o64 a1 iwdq                       ] SM0-1,NOAPX,ZU,LONG,PROT,X86_64 */
        /* 324 001(a1) 045 : SM0,SM1,NOAPX,ZU,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_RAX|RN_L16,MEM_OFFS,0,0,0}, NO_DECORATOR, nasm_bytecodes+71329, 8, 57},
    /*   8 : MOV mem_offs,reg_al [i-:       o8 a2 iwdq                        ] SM0-1,NOAPX,NOHLE,8086 */
        /* 001(a2) 044 : SM0,SM1,NOAPX,NOHLE,8086 */
        {I_MOV, 2, {MEM_OFFS,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72588, 9, 58},
    /*   9 : MOV mem_offs,reg_ax [i-:       o16 a3 iwdq                       ] SM0-1,NOAPX,NOHLE,8086 */
        /* 320 001(a3) 044 : SM0,SM1,NOAPX,NOHLE,8086 */
        {I_MOV, 2, {MEM_OFFS,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71334, 9, 59},
    /*  10 : MOV mem_offs,reg_eax [i-:       o32 a3 iwdq                       ] SM0-1,NOAPX,NOHLE,386 */
        /* 321 001(a3) 044 : SM0,SM1,NOAPX,NOHLE,386 */
        {I_MOV, 2, {MEM_OFFS,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71339, 10, 60},
    /*  11 : MOV mem_offs,reg_rax [i-:       o64 a3 iwdq                       ] SM0-1,NOAPX,LONG,NOHLE,PROT,X86_64 */
        /* 324 001(a3) 044 : SM0,SM1,NOAPX,LONG,NOHLE,PROT,X86_64 */
        {I_MOV, 2, {MEM_OFFS,REG_RAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71344, 11, 61},
    /*  12 : MOV rm8,reg8 [mr:       hlexr o8 88 /r                    ] SM0-1,8086 */
        /* 271 001(88) 101 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71349, 12, 70},
    /*  13 : MOV rm16,reg16 [mr:       hlexr o16 89 /r                   ] SM0-1,8086 */
        /* 271 320 001(89) 101 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66417, 12, 71},
    /*  14 : MOV rm32,reg32 [mr:       hlexr o32 89 /r                   ] SM0-1,386 */
        /* 271 321 001(89) 101 : SM0,SM1,386 */
        {I_MOV, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66423, 13, 72},
    /*  15 : MOV rm64,reg64 [mr:       hlexr o64 89 /r                   ] SM0-1,LONG,PROT,X86_64 */
        /* 271 324 001(89) 101 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66429, 14, 73},
    /*  16 : MOV reg8,rm8 [rm:       o8 8a /r                          ] SM0-1,8086 */
        /* 001(8a) 110 : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66443, 12, 74},
    /*  17 : MOV reg16,rm16 [rm:       o16 8b /r                         ] SM0-1,8086 */
        /* 320 001(8b) 110 : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66988, 12, 75},
    /*  18 : MOV reg32,rm32 [rm:       o32 8b /r                         ] SM0-1,386 */
        /* 321 001(8b) 110 : SM0,SM1,386 */
        {I_MOV, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66994, 13, 76},
    /*  19 : MOV reg64,rm64 [rm:       o64 8b /r                         ] SM0-1,LONG,PROT,X86_64 */
        /* 324 001(8b) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67000, 14, 77},
    /*  20 : MOV reg8,imm8 [ri:       o8 b0+r ib                        ] SM0-1,8086 */
        /* 010 260(021  : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72592, 12, 80},
    /*  21 : MOV reg16,imm16 [ri:       o16 b8+r iw                       ] SM0-1,8086 */
        /* 320 010 270(031  : SM0,SM1,8086 */
        {I_MOV, 2, {REG_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71359, 12, 81},
    /*  22 : MOV reg32,imm32 [ri:       o32 b8+r id                       ] SM0-1,386 */
        /* 321 010 270(041  : SM0,SM1,386 */
        {I_MOV, 2, {REG_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71364, 13, 82},
    /*  23 : MOV reg64,imm64 [ri:       o64 b8+r iq                       ] SM0-1,LONG,PROT,X86_64 */
        /* 324 010 270(055  : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71369, 14, 83},
    /*  24 : MOV rm8,imm8 [mi:       hlexr o8 c6 /0 ib                 ] SM0-1,8086 */
        /* 271 001(c6) 200 021 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66435, 12, 92},
    /*  25 : MOV rm16,imm16 [mi:       hlexr o16 c7 /0 iw                ] SM0-1,8086 */
        /* 271 320 001(c7) 200 031 : SM0,SM1,8086 */
        {I_MOV, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54851, 12, 93},
    /*  26 : MOV rm32,imm32 [mi:       hlexr o32 c7 /0 id                ] SM0-1,386 */
        /* 271 321 001(c7) 200 041 : SM0,SM1,386 */
        {I_MOV, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54858, 13, 94},
    /*  27 : MOV rm64,sdword64 [mi:       hlexr o64 c7 /0 id,s              ] SM0-1,LONG,PROT,X86_64 */
        /* 271 324 001(c7) 200 255 : SM0,SM1,LONG,PROT,X86_64 */
        {I_MOV, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54865, 14, 95},
    /*  28 : MOVRS reg8,mem8 [rm:       o8 norep 0f38 8a /r               ] SM0-1,FUTURE */
        /* 331 356 001(8a) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS8|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66441, 17, 96},
    /*  29 : MOVRS reg16,mem16 [rm:       o16 norep 0f38 8b /r              ] SM0-1,FUTURE */
        /* 320 331 356 001(8b) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54872, 17, 97},
    /*  30 : MOVRS reg32,mem32 [rm:       o32 norep 0f38 8b /r              ] SM0-1,FUTURE */
        /* 321 331 356 001(8b) 110 : SM0,SM1,NOAPX,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54879, 17, 98},
    /*  31 : MOVRS reg64,mem64 [rm:       o64 norep 0f38 8b /r              ] SM0-1,LONG,PROT,X86_64,FUTURE */
        /* 324 331 356 001(8b) 110 : SM0,SM1,NOAPX,LONG,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54886, 18, 99},
    /*  32 : MOVRS reg8,mem8 [rm:       evex.nf0.nd0.l0.m4.o8 8a /r       ] SM0-1,FUTURE */
        /* 250(f4 7c 08) 300 001(8a) 110 : SM0,SM1,WIG,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS8,MEMORY|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23323, 19, 100},
    /*  33 : MOVRS reg16,mem16 [rm:       evex.nf0.nd0.l0.m4.o16 8b /r      ] SM0-1,FUTURE */
        /* 250(f4 7d 08) 300 320 001(8b) 110 : SM0,SM1,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9143, 20, 101},
    /*  34 : MOVRS reg32,mem32 [rm:       evex.nf0.nd0.l0.m4.o32 8b /r      ] SM0-1,FUTURE */
        /* 250(f4 7c 08) 300 321 001(8b) 110 : SM0,SM1,WW,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9153, 21, 102},
    /*  35 : MOVRS reg64,mem64 [rm:       evex.nf0.nd0.l0.m4.o64 8b /r      ] SM0-1,LONG,PROT,X86_64,FUTURE */
        /* 250(f4 fc 08) 300 324 001(8b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,FUTURE */
        {I_MOVRS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9163, 22, 103},
    /*  36 : LEA reg16,mem [rm:       o16 8d /r                         ] 8086 */
        /* 320 001(8d) 110 : 8086 */
        {I_LEA, 2, {REG_GPR|BITS16,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71374, 23, 106},
    /*  37 : LEA reg32,mem [rm:       o32 8d /r                         ] 386 */
        /* 321 001(8d) 110 : 386 */
        {I_LEA, 2, {REG_GPR|BITS32,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71379, 3, 107},
    /*  38 : LEA reg64,mem [rm:       o64 8d /r                         ] LONG,PROT,X86_64 */
        /* 324 001(8d) 110 : LONG,PROT,X86_64 */
        {I_LEA, 2, {REG_GPR|BITS64,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+71384, 24, 108},
    /*  39 : ADD rm8,reg8 [mr:       hle o8 00 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(00) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71389, 25, 114},
    /*  40 : ADD rm16,reg16 [mr:       hle o16 01 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(01) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66447, 25, 115},
    /*  41 : ADD rm32,reg32 [mr:       hle o32 01 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(01) 101 : SM0,SM1,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66453, 26, 116},
    /*  42 : ADD rm64,reg64 [mr:       hle o64 01 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(01) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66459, 27, 117},
    /*  43 : ADD reg8,rm8 [rm:       o8 02 /r                          ] SM0-1,FL,8086 */
        /* 001(02) 110 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69539, 28, 118},
    /*  44 : ADD reg16,rm16 [rm:       o16 03 /r                         ] SM0-1,FL,8086 */
        /* 320 001(03) 110 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9198, 28, 119},
    /*  45 : ADD reg32,rm32 [rm:       o32 03 /r                         ] SM0-1,FL,386 */
        /* 321 001(03) 110 : SM0,SM1,FL,386 */
        {I_ADD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9228, 29, 120},
    /*  46 : ADD reg64,rm64 [rm:       o64 03 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(03) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9258, 30, 121},
    /*  47 : ADD reg_al,imm8 [-i:       o8 04 ib                          ] SM0-1,FL,8086 */
        /* 001(04) 021 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72596, 28, 122},
    /*  48 : ADD rm8,imm8 [mi:       hle 80 /0 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 200 021 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66465, 25, 123},
    /*  49 : ADD rm16,sbyteword16 [mi:       hle o16 83 /0 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 200 275 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54893, 25, 124},
    /*  50 : ADD reg_ax,imm16 [-i:       o16 05 iw                         ] SM0-1,FL,8086 */
        /* 320 001(05) 031 : SM0,SM1,FL,8086 */
        {I_ADD, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71394, 28, 125},
    /*  51 : ADD rm16,imm16 [mi:       hle o16 81 /0 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 200 031 : SM0,SM1,LOCK,FL,8086 */
        {I_ADD, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54900, 25, 126},
    /*  52 : ADD rm32,sbytedword32 [mi:       hle o32 83 /0 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 200 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54907, 31, 127},
    /*  53 : ADD reg_eax,imm32 [-i:       o32 05 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(05) 041 : SM0,SM1,ZU,FL,386 */
        {I_ADD, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71399, 32, 128},
    /*  54 : ADD rm32,imm32 [mi:       hle o32 81 /0 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 200 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_ADD, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54914, 31, 129},
    /*  55 : ADD rm64,sbytedword64 [mi:       hle o64 83 /0 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 200 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54921, 33, 130},
    /*  56 : ADD reg_rax,sdword64 [-i:       o64 05 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(05) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71404, 34, 131},
    /*  57 : ADD rm64,sdword64 [mi:       hle o64 81 /0 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 200 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54928, 33, 132},
    /*  58 : ADD reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 02 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(02) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23332, 35, 133},
    /*  59 : ADD reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 02 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(02) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23341, 36, 133},
    /*  60 : ADD reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9173, 38, 134},
    /*  61 : ADD reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(03) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9183, 39, 134},
    /*  62 : ADD reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9203, 41, 135},
    /*  63 : ADD reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(03) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9213, 42, 135},
    /*  64 : ADD reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 03 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(03) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9233, 41, 136},
    /*  65 : ADD reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 03 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(03) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9243, 42, 136},
    /*  66 : ADD reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 00 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(00) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23359, 35, 137},
    /*  67 : ADD rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 00 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(00) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23368, 36, 137},
    /*  68 : ADD reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9263, 38, 138},
    /*  69 : ADD rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(01) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9273, 39, 138},
    /*  70 : ADD reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9293, 41, 139},
    /*  71 : ADD rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(01) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9303, 42, 139},
    /*  72 : ADD reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 01 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(01) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9323, 41, 140},
    /*  73 : ADD rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 01 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(01) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9333, 42, 140},
    /*  74 : ADD reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+24, 38, 141},
    /*  75 : ADD rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 200 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+35, 39, 141},
    /*  76 : ADD reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57, 41, 142},
    /*  77 : ADD rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 200 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+68, 42, 142},
    /*  78 : ADD reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /0 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 210 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+90, 41, 143},
    /*  79 : ADD rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /0 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 200 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+101, 42, 143},
    /*  80 : ADD reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /0 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 210 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9353, 35, 144},
    /*  81 : ADD rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /0 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 200 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9363, 36, 144},
    /*  82 : ADD reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /0 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 210 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+123, 38, 145},
    /*  83 : ADD rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /0 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 200 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+134, 39, 145},
    /*  84 : ADD reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /0 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 210 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+156, 41, 146},
    /*  85 : ADD rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /0 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 200 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+167, 42, 146},
    /*  86 : ADD reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /0 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 210 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+189, 41, 147},
    /*  87 : ADD rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /0 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 200 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADD, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+200, 42, 147},
    /*  88 : OR rm8,reg8 [mr:       hle o8 08 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(08) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71409, 25, 148},
    /*  89 : OR rm16,reg16 [mr:       hle o16 09 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(09) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66471, 25, 149},
    /*  90 : OR rm32,reg32 [mr:       hle o32 09 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(09) 101 : SM0,SM1,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66477, 26, 150},
    /*  91 : OR rm64,reg64 [mr:       hle o64 09 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(09) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66483, 27, 151},
    /*  92 : OR reg8,rm8 [rm:       o8 0a /r                          ] SM0-1,FL,8086 */
        /* 001(0a) 110 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69659, 28, 152},
    /*  93 : OR reg16,rm16 [rm:       o16 0b /r                         ] SM0-1,FL,8086 */
        /* 320 001(0b) 110 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9408, 28, 153},
    /*  94 : OR reg32,rm32 [rm:       o32 0b /r                         ] SM0-1,FL,386 */
        /* 321 001(0b) 110 : SM0,SM1,FL,386 */
        {I_OR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9438, 29, 154},
    /*  95 : OR reg64,rm64 [rm:       o64 0b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(0b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9468, 30, 155},
    /*  96 : OR reg_al,imm8 [-i:       o8 0c ib                          ] SM0-1,FL,8086 */
        /* 001(0c) 021 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72600, 28, 156},
    /*  97 : OR rm8,imm8 [mi:       hle 80 /1 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 201 021 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66489, 25, 157},
    /*  98 : OR rm16,sbyteword16 [mi:       hle o16 83 /1 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 201 275 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54935, 25, 158},
    /*  99 : OR reg_ax,imm16 [-i:       o16 0d iw                         ] SM0-1,FL,8086 */
        /* 320 001(0d) 031 : SM0,SM1,FL,8086 */
        {I_OR, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71414, 28, 159},
    /* 100 : OR rm16,imm16 [mi:       hle o16 81 /1 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 201 031 : SM0,SM1,LOCK,FL,8086 */
        {I_OR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54942, 25, 160},
    /* 101 : OR rm32,sbytedword32 [mi:       hle o32 83 /1 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 201 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54949, 31, 161},
    /* 102 : OR reg_eax,imm32 [-i:       o32 0d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(0d) 041 : SM0,SM1,ZU,FL,386 */
        {I_OR, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71419, 32, 162},
    /* 103 : OR rm32,imm32 [mi:       hle o32 81 /1 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 201 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_OR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54956, 31, 163},
    /* 104 : OR rm64,sbytedword64 [mi:       hle o64 83 /1 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 201 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54963, 33, 164},
    /* 105 : OR reg_rax,sdword64 [-i:       o64 0d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(0d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71424, 34, 165},
    /* 106 : OR rm64,sdword64 [mi:       hle o64 81 /1 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 201 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54970, 33, 166},
    /* 107 : OR reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 0a /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(0a) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23386, 35, 167},
    /* 108 : OR reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 0a /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(0a) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23395, 36, 167},
    /* 109 : OR reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9383, 38, 168},
    /* 110 : OR reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(0b) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9393, 39, 168},
    /* 111 : OR reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9413, 41, 169},
    /* 112 : OR reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(0b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9423, 42, 169},
    /* 113 : OR reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 0b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(0b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9443, 41, 170},
    /* 114 : OR reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 0b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(0b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9453, 42, 170},
    /* 115 : OR reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 08 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(08) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23413, 35, 171},
    /* 116 : OR rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 08 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(08) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23422, 36, 171},
    /* 117 : OR reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9473, 38, 172},
    /* 118 : OR rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(09) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9483, 39, 172},
    /* 119 : OR reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9503, 41, 173},
    /* 120 : OR rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(09) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9513, 42, 173},
    /* 121 : OR reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 09 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(09) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9533, 41, 174},
    /* 122 : OR rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 09 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(09) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9543, 42, 174},
    /* 123 : OR reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+222, 38, 175},
    /* 124 : OR rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 201 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+233, 39, 175},
    /* 125 : OR reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+255, 41, 176},
    /* 126 : OR rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 201 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+266, 42, 176},
    /* 127 : OR reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /1 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 211 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+288, 41, 177},
    /* 128 : OR rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /1 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 201 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+299, 42, 177},
    /* 129 : OR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /1 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 211 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9563, 35, 178},
    /* 130 : OR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /1 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 201 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9573, 36, 178},
    /* 131 : OR reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /1 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 211 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+321, 38, 179},
    /* 132 : OR rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /1 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 201 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+332, 39, 179},
    /* 133 : OR reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /1 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 211 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+354, 41, 180},
    /* 134 : OR rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /1 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 201 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+365, 42, 180},
    /* 135 : OR reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /1 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 211 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+387, 41, 181},
    /* 136 : OR rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /1 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 201 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_OR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+398, 42, 181},
    /* 137 : ADC rm8,reg8 [mr:       hle o8 10 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(10) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71429, 25, 182},
    /* 138 : ADC rm16,reg16 [mr:       hle o16 11 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(11) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66495, 25, 183},
    /* 139 : ADC rm32,reg32 [mr:       hle o32 11 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(11) 101 : SM0,SM1,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66501, 26, 184},
    /* 140 : ADC rm64,reg64 [mr:       hle o64 11 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(11) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66507, 27, 185},
    /* 141 : ADC reg8,rm8 [rm:       o8 12 /r                          ] SM0-1,FL,8086 */
        /* 001(12) 110 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69437, 28, 186},
    /* 142 : ADC reg16,rm16 [rm:       o16 13 /r                         ] SM0-1,FL,8086 */
        /* 320 001(13) 110 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9618, 28, 187},
    /* 143 : ADC reg32,rm32 [rm:       o32 13 /r                         ] SM0-1,FL,386 */
        /* 321 001(13) 110 : SM0,SM1,FL,386 */
        {I_ADC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9648, 29, 188},
    /* 144 : ADC reg64,rm64 [rm:       o64 13 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(13) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9678, 30, 189},
    /* 145 : ADC reg_al,imm8 [-i:       o8 14 ib                          ] SM0-1,FL,8086 */
        /* 001(14) 021 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72604, 28, 190},
    /* 146 : ADC rm8,imm8 [mi:       hle 80 /2 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 202 021 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66513, 25, 191},
    /* 147 : ADC rm16,sbyteword16 [mi:       hle o16 83 /2 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 202 275 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54977, 25, 192},
    /* 148 : ADC reg_ax,imm16 [-i:       o16 15 iw                         ] SM0-1,FL,8086 */
        /* 320 001(15) 031 : SM0,SM1,FL,8086 */
        {I_ADC, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71434, 28, 193},
    /* 149 : ADC rm16,imm16 [mi:       hle o16 81 /2 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 202 031 : SM0,SM1,LOCK,FL,8086 */
        {I_ADC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+54984, 25, 194},
    /* 150 : ADC rm32,sbytedword32 [mi:       hle o32 83 /2 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 202 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+54991, 31, 195},
    /* 151 : ADC reg_eax,imm32 [-i:       o32 15 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(15) 041 : SM0,SM1,ZU,FL,386 */
        {I_ADC, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71439, 32, 196},
    /* 152 : ADC rm32,imm32 [mi:       hle o32 81 /2 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 202 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_ADC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+54998, 31, 197},
    /* 153 : ADC rm64,sbytedword64 [mi:       hle o64 83 /2 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 202 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55005, 33, 198},
    /* 154 : ADC reg_rax,sdword64 [-i:       o64 15 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(15) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71444, 34, 199},
    /* 155 : ADC rm64,sdword64 [mi:       hle o64 81 /2 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 202 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55012, 33, 200},
    /* 156 : ADC reg8?,reg8,rm8 [vrm:      evex.nd1..l0.m4.o8 12 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(12) 121 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23440, 44, 201},
    /* 157 : ADC reg8,rm8 [rm:      evex.nd0..l0.m4.o8 12 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(12) 110 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23449, 45, 201},
    /* 158 : ADC reg16?,reg16,rm16 [vrm:      evex.nd1..l0.m4.o16 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(13) 121 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9593, 47, 202},
    /* 159 : ADC reg16,rm16 [rm:      evex.nd0..l0.m4.o16 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(13) 110 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9603, 48, 202},
    /* 160 : ADC reg32?,reg32,rm32 [vrm:      evex.nd1..l0.m4.o32 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(13) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9623, 50, 203},
    /* 161 : ADC reg32,rm32 [rm:      evex.nd0..l0.m4.o32 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(13) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9633, 51, 203},
    /* 162 : ADC reg64?,reg64,rm64 [vrm:      evex.nd1..l0.m4.o64 13 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(13) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9653, 50, 204},
    /* 163 : ADC reg64,rm64 [rm:      evex.nd0..l0.m4.o64 13 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(13) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9663, 51, 204},
    /* 164 : ADC reg8?,rm8,reg8 [vmr:      evex.nd1..l0.m4.o8 10 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(10) 112 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23467, 44, 205},
    /* 165 : ADC rm8,reg8 [mr:      evex.nd0..l0.m4.o8 10 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(10) 101 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23476, 45, 205},
    /* 166 : ADC reg16?,rm16,reg16 [vmr:      evex.nd1..l0.m4.o16 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(11) 112 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9683, 47, 206},
    /* 167 : ADC rm16,reg16 [mr:      evex.nd0..l0.m4.o16 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(11) 101 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9693, 48, 206},
    /* 168 : ADC reg32?,rm32,reg32 [vmr:      evex.nd1..l0.m4.o32 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(11) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9713, 50, 207},
    /* 169 : ADC rm32,reg32 [mr:      evex.nd0..l0.m4.o32 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(11) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9723, 51, 207},
    /* 170 : ADC reg64?,rm64,reg64 [vmr:      evex.nd1..l0.m4.o64 11 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(11) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9743, 50, 208},
    /* 171 : ADC rm64,reg64 [mr:      evex.nd0..l0.m4.o64 11 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(11) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9753, 51, 208},
    /* 172 : ADC reg16?,rm16,sbyteword16 [vmi:      evex.nd1..l0.m4.o16 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 212 276 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+420, 47, 209},
    /* 173 : ADC rm16,sbyteword16 [mi:      evex.nd0..l0.m4.o16 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 202 275 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+431, 48, 209},
    /* 174 : ADC reg32?,rm32,sbytedword32 [vmi:      evex.nd1..l0.m4.o32 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 212 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+453, 50, 210},
    /* 175 : ADC rm32,sbytedword32 [mi:      evex.nd0..l0.m4.o32 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 202 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+464, 51, 210},
    /* 176 : ADC reg64?,rm64,sbytedword64 [vmi:      evex.nd1..l0.m4.o64 83 /2 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 212 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+486, 50, 211},
    /* 177 : ADC rm64,sbytedword64 [mi:      evex.nd0..l0.m4.o64 83 /2 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 202 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+497, 51, 211},
    /* 178 : ADC reg8?,rm8,imm8 [vmi:      evex.nd1..l0.m4.o8 80 /2 ib       ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 212 022 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9773, 44, 212},
    /* 179 : ADC rm8,imm8 [mi:      evex.nd0..l0.m4.o8 80 /2 ib       ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 202 021 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9783, 45, 212},
    /* 180 : ADC reg16?,rm16,imm16 [vmi:      evex.nd1..l0.m4.o16 81 /2 iw      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 212 032 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+519, 47, 213},
    /* 181 : ADC rm16,imm16 [mi:      evex.nd0..l0.m4.o16 81 /2 iw      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 202 031 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+530, 48, 213},
    /* 182 : ADC reg32?,rm32,imm32 [vmi:      evex.nd1..l0.m4.o32 81 /2 id      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 212 042 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+552, 50, 214},
    /* 183 : ADC rm32,imm32 [mi:      evex.nd0..l0.m4.o32 81 /2 id      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 202 041 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+563, 51, 214},
    /* 184 : ADC reg64?,rm64,sdword64 [vmi:      evex.nd1..l0.m4.o64 81 /2 id,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 212 256 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+585, 50, 215},
    /* 185 : ADC rm64,sdword64 [mi:      evex.nd0..l0.m4.o64 81 /2 id,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 202 255 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ADC, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+596, 51, 215},
    /* 186 : SBB rm8,reg8 [mr:       hle o8 18 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(18) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71449, 25, 216},
    /* 187 : SBB rm16,reg16 [mr:       hle o16 19 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(19) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66519, 25, 217},
    /* 188 : SBB rm32,reg32 [mr:       hle o32 19 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(19) 101 : SM0,SM1,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66525, 26, 218},
    /* 189 : SBB rm64,reg64 [mr:       hle o64 19 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(19) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66531, 27, 219},
    /* 190 : SBB reg8,rm8 [rm:       o8 1a /r                          ] SM0-1,FL,8086 */
        /* 001(1a) 110 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+70031, 28, 220},
    /* 191 : SBB reg16,rm16 [rm:       o16 1b /r                         ] SM0-1,FL,8086 */
        /* 320 001(1b) 110 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9828, 28, 221},
    /* 192 : SBB reg32,rm32 [rm:       o32 1b /r                         ] SM0-1,FL,386 */
        /* 321 001(1b) 110 : SM0,SM1,FL,386 */
        {I_SBB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9858, 29, 222},
    /* 193 : SBB reg64,rm64 [rm:       o64 1b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(1b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9888, 30, 223},
    /* 194 : SBB reg_al,imm8 [-i:       o8 1c ib                          ] SM0-1,FL,8086 */
        /* 001(1c) 021 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72608, 28, 224},
    /* 195 : SBB rm8,imm8 [mi:       hle 80 /3 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 203 021 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66537, 25, 225},
    /* 196 : SBB rm16,sbyteword16 [mi:       hle o16 83 /3 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 203 275 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55019, 25, 226},
    /* 197 : SBB reg_ax,imm16 [-i:       o16 1d iw                         ] SM0-1,FL,8086 */
        /* 320 001(1d) 031 : SM0,SM1,FL,8086 */
        {I_SBB, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71454, 28, 227},
    /* 198 : SBB rm16,imm16 [mi:       hle o16 81 /3 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 203 031 : SM0,SM1,LOCK,FL,8086 */
        {I_SBB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55026, 25, 228},
    /* 199 : SBB rm32,sbytedword32 [mi:       hle o32 83 /3 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 203 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55033, 31, 229},
    /* 200 : SBB reg_eax,imm32 [-i:       o32 1d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(1d) 041 : SM0,SM1,ZU,FL,386 */
        {I_SBB, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71459, 32, 230},
    /* 201 : SBB rm32,imm32 [mi:       hle o32 81 /3 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 203 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_SBB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55040, 31, 231},
    /* 202 : SBB rm64,sbytedword64 [mi:       hle o64 83 /3 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 203 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55047, 33, 232},
    /* 203 : SBB reg_rax,sdword64 [-i:       o64 1d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(1d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71464, 34, 233},
    /* 204 : SBB rm64,sdword64 [mi:       hle o64 81 /3 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 203 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55054, 33, 234},
    /* 205 : SBB reg8?,reg8,rm8 [vrm:      evex.nd1..l0.m4.o8 1a /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(1a) 121 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23494, 44, 235},
    /* 206 : SBB reg8,rm8 [rm:      evex.nd0..l0.m4.o8 1a /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(1a) 110 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23503, 45, 235},
    /* 207 : SBB reg16?,reg16,rm16 [vrm:      evex.nd1..l0.m4.o16 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(1b) 121 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9803, 47, 236},
    /* 208 : SBB reg16,rm16 [rm:      evex.nd0..l0.m4.o16 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(1b) 110 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9813, 48, 236},
    /* 209 : SBB reg32?,reg32,rm32 [vrm:      evex.nd1..l0.m4.o32 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(1b) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9833, 50, 237},
    /* 210 : SBB reg32,rm32 [rm:      evex.nd0..l0.m4.o32 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(1b) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9843, 51, 237},
    /* 211 : SBB reg64?,reg64,rm64 [vrm:      evex.nd1..l0.m4.o64 1b /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(1b) 121 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9863, 50, 238},
    /* 212 : SBB reg64,rm64 [rm:      evex.nd0..l0.m4.o64 1b /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(1b) 110 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9873, 51, 238},
    /* 213 : SBB reg8?,rm8,reg8 [vmr:      evex.nd1..l0.m4.o8 18 /r          ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(18) 112 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23521, 44, 239},
    /* 214 : SBB rm8,reg8 [mr:      evex.nd0..l0.m4.o8 18 /r          ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(18) 101 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23530, 45, 239},
    /* 215 : SBB reg16?,rm16,reg16 [vmr:      evex.nd1..l0.m4.o16 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(19) 112 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+9893, 47, 240},
    /* 216 : SBB rm16,reg16 [mr:      evex.nd0..l0.m4.o16 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(19) 101 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+9903, 48, 240},
    /* 217 : SBB reg32?,rm32,reg32 [vmr:      evex.nd1..l0.m4.o32 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(19) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+9923, 50, 241},
    /* 218 : SBB rm32,reg32 [mr:      evex.nd0..l0.m4.o32 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(19) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+9933, 51, 241},
    /* 219 : SBB reg64?,rm64,reg64 [vmr:      evex.nd1..l0.m4.o64 19 /r         ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(19) 112 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+9953, 50, 242},
    /* 220 : SBB rm64,reg64 [mr:      evex.nd0..l0.m4.o64 19 /r         ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(19) 101 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+9963, 51, 242},
    /* 221 : SBB reg16?,rm16,sbyteword16 [vmi:      evex.nd1..l0.m4.o16 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 213 276 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+618, 47, 243},
    /* 222 : SBB rm16,sbyteword16 [mi:      evex.nd0..l0.m4.o16 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 203 275 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+629, 48, 243},
    /* 223 : SBB reg32?,rm32,sbytedword32 [vmi:      evex.nd1..l0.m4.o32 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 213 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+651, 50, 244},
    /* 224 : SBB rm32,sbytedword32 [mi:      evex.nd0..l0.m4.o32 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 203 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+662, 51, 244},
    /* 225 : SBB reg64?,rm64,sbytedword64 [vmi:      evex.nd1..l0.m4.o64 83 /3 ib,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 213 276 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+684, 50, 245},
    /* 226 : SBB rm64,sbytedword64 [mi:      evex.nd0..l0.m4.o64 83 /3 ib,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 203 275 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+695, 51, 245},
    /* 227 : SBB reg8?,rm8,imm8 [vmi:      evex.nd1..l0.m4.o8 80 /3 ib       ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 213 022 : SM0,SM1,SM2,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+9983, 44, 246},
    /* 228 : SBB rm8,imm8 [mi:      evex.nd0..l0.m4.o8 80 /3 ib       ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 203 021 : SM0,SM1,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+9993, 45, 246},
    /* 229 : SBB reg16?,rm16,imm16 [vmi:      evex.nd1..l0.m4.o16 81 /3 iw      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 213 032 : SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+717, 47, 247},
    /* 230 : SBB rm16,imm16 [mi:      evex.nd0..l0.m4.o16 81 /3 iw      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 203 031 : SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+728, 48, 247},
    /* 231 : SBB reg32?,rm32,imm32 [vmi:      evex.nd1..l0.m4.o32 81 /3 id      ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 213 042 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+750, 50, 248},
    /* 232 : SBB rm32,imm32 [mi:      evex.nd0..l0.m4.o32 81 /3 id      ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 203 041 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+761, 51, 248},
    /* 233 : SBB reg64?,rm64,sdword64 [vmi:      evex.nd1..l0.m4.o64 81 /3 id,s    ] SM0,SM1,SM2,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 213 256 : SM0,SM1,SM2,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+783, 50, 249},
    /* 234 : SBB rm64,sdword64 [mi:      evex.nd0..l0.m4.o64 81 /3 id,s    ] SM0,SM1,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 203 255 : SM0,SM1,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SBB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+794, 51, 249},
    /* 235 : AND rm8,reg8 [mr:       hle o8 20 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(20) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71469, 25, 250},
    /* 236 : AND rm16,reg16 [mr:       hle o16 21 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(21) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66543, 25, 251},
    /* 237 : AND rm32,reg32 [mr:       hle o32 21 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(21) 101 : SM0,SM1,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66549, 26, 252},
    /* 238 : AND rm64,reg64 [mr:       hle o64 21 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(21) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66555, 27, 253},
    /* 239 : AND reg8,rm8 [rm:       o8 22 /r                          ] SM0-1,FL,8086 */
        /* 001(22) 110 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69791, 28, 254},
    /* 240 : AND reg16,rm16 [rm:       o16 23 /r                         ] SM0-1,FL,8086 */
        /* 320 001(23) 110 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10038, 28, 255},
    /* 241 : AND reg32,rm32 [rm:       o32 23 /r                         ] SM0-1,FL,386 */
        /* 321 001(23) 110 : SM0,SM1,FL,386 */
        {I_AND, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10068, 29, 256},
    /* 242 : AND reg64,rm64 [rm:       o64 23 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(23) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10098, 30, 257},
    /* 243 : AND reg_al,imm8 [-i:       o8 24 ib                          ] SM0-1,FL,8086 */
        /* 001(24) 021 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72612, 28, 258},
    /* 244 : AND rm8,imm8 [mi:       hle 80 /4 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 204 021 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66561, 25, 259},
    /* 245 : AND rm16,sbyteword16 [mi:       hle o16 83 /4 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 204 275 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55061, 25, 260},
    /* 246 : AND reg_ax,imm16 [-i:       o16 25 iw                         ] SM0-1,FL,8086 */
        /* 320 001(25) 031 : SM0,SM1,FL,8086 */
        {I_AND, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71474, 28, 261},
    /* 247 : AND rm16,imm16 [mi:       hle o16 81 /4 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 204 031 : SM0,SM1,LOCK,FL,8086 */
        {I_AND, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55068, 25, 262},
    /* 248 : AND rm32,sbytedword32 [mi:       hle o32 83 /4 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 204 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55075, 31, 263},
    /* 249 : AND reg_eax,imm32 [-i:       o32 25 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(25) 041 : SM0,SM1,ZU,FL,386 */
        {I_AND, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71479, 32, 264},
    /* 250 : AND rm32,imm32 [mi:       hle o32 81 /4 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 204 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_AND, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55082, 31, 265},
    /* 251 : AND rm64,sbytedword64 [mi:       hle o64 83 /4 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 204 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55089, 33, 266},
    /* 252 : AND reg_rax,sdword64 [-i:       o64 25 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(25) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71484, 34, 267},
    /* 253 : AND rm64,sdword64 [mi:       hle o64 81 /4 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 204 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55096, 33, 268},
    /* 254 : AND reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 22 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(22) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23548, 35, 269},
    /* 255 : AND reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 22 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(22) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23557, 36, 269},
    /* 256 : AND reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10013, 38, 270},
    /* 257 : AND reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(23) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10023, 39, 270},
    /* 258 : AND reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10043, 41, 271},
    /* 259 : AND reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(23) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10053, 42, 271},
    /* 260 : AND reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 23 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(23) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10073, 41, 272},
    /* 261 : AND reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 23 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(23) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10083, 42, 272},
    /* 262 : AND reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 20 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(20) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23575, 35, 273},
    /* 263 : AND rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 20 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(20) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23584, 36, 273},
    /* 264 : AND reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10103, 38, 274},
    /* 265 : AND rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(21) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10113, 39, 274},
    /* 266 : AND reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10133, 41, 275},
    /* 267 : AND rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(21) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10143, 42, 275},
    /* 268 : AND reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 21 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(21) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10163, 41, 276},
    /* 269 : AND rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 21 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(21) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10173, 42, 276},
    /* 270 : AND reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+816, 38, 277},
    /* 271 : AND rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 204 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+827, 39, 277},
    /* 272 : AND reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+849, 41, 278},
    /* 273 : AND rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 204 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+860, 42, 278},
    /* 274 : AND reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /4 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 214 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+882, 41, 279},
    /* 275 : AND rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /4 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 204 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+893, 42, 279},
    /* 276 : AND reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /4 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 214 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10193, 35, 280},
    /* 277 : AND rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /4 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 204 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10203, 36, 280},
    /* 278 : AND reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /4 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 214 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+915, 38, 281},
    /* 279 : AND rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /4 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 204 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+926, 39, 281},
    /* 280 : AND reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /4 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 214 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+948, 41, 282},
    /* 281 : AND rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /4 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 204 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+959, 42, 282},
    /* 282 : AND reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /4 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 214 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+981, 41, 283},
    /* 283 : AND rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /4 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 204 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_AND, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+992, 42, 283},
    /* 284 : SUB rm8,reg8 [mr:       hle o8 28 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(28) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71489, 25, 284},
    /* 285 : SUB rm16,reg16 [mr:       hle o16 29 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(29) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66567, 25, 285},
    /* 286 : SUB rm32,reg32 [mr:       hle o32 29 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(29) 101 : SM0,SM1,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66573, 26, 286},
    /* 287 : SUB rm64,reg64 [mr:       hle o64 29 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(29) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66579, 27, 287},
    /* 288 : SUB reg8,rm8 [rm:       o8 2a /r                          ] SM0-1,FL,8086 */
        /* 001(2a) 110 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69701, 28, 288},
    /* 289 : SUB reg16,rm16 [rm:       o16 2b /r                         ] SM0-1,FL,8086 */
        /* 320 001(2b) 110 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10248, 28, 289},
    /* 290 : SUB reg32,rm32 [rm:       o32 2b /r                         ] SM0-1,FL,386 */
        /* 321 001(2b) 110 : SM0,SM1,FL,386 */
        {I_SUB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10278, 29, 290},
    /* 291 : SUB reg64,rm64 [rm:       o64 2b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(2b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10308, 30, 291},
    /* 292 : SUB reg_al,imm8 [-i:       o8 2c ib                          ] SM0-1,FL,8086 */
        /* 001(2c) 021 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72616, 28, 292},
    /* 293 : SUB rm8,imm8 [mi:       hle 80 /5 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 205 021 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66585, 25, 293},
    /* 294 : SUB rm16,sbyteword16 [mi:       hle o16 83 /5 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 205 275 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55103, 25, 294},
    /* 295 : SUB reg_ax,imm16 [-i:       o16 2d iw                         ] SM0-1,FL,8086 */
        /* 320 001(2d) 031 : SM0,SM1,FL,8086 */
        {I_SUB, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71494, 28, 295},
    /* 296 : SUB rm16,imm16 [mi:       hle o16 81 /5 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 205 031 : SM0,SM1,LOCK,FL,8086 */
        {I_SUB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55110, 25, 296},
    /* 297 : SUB rm32,sbytedword32 [mi:       hle o32 83 /5 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 205 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55117, 31, 297},
    /* 298 : SUB reg_eax,imm32 [-i:       o32 2d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(2d) 041 : SM0,SM1,ZU,FL,386 */
        {I_SUB, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71499, 32, 298},
    /* 299 : SUB rm32,imm32 [mi:       hle o32 81 /5 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 205 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_SUB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55124, 31, 299},
    /* 300 : SUB rm64,sbytedword64 [mi:       hle o64 83 /5 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 205 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55131, 33, 300},
    /* 301 : SUB reg_rax,sdword64 [-i:       o64 2d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(2d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71504, 34, 301},
    /* 302 : SUB rm64,sdword64 [mi:       hle o64 81 /5 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 205 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55138, 33, 302},
    /* 303 : SUB reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 2a /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(2a) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23602, 35, 303},
    /* 304 : SUB reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 2a /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(2a) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23611, 36, 303},
    /* 305 : SUB reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10223, 38, 304},
    /* 306 : SUB reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(2b) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10233, 39, 304},
    /* 307 : SUB reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10253, 41, 305},
    /* 308 : SUB reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(2b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10263, 42, 305},
    /* 309 : SUB reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 2b /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(2b) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10283, 41, 306},
    /* 310 : SUB reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 2b /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(2b) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10293, 42, 306},
    /* 311 : SUB reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 28 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(28) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23629, 35, 307},
    /* 312 : SUB rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 28 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(28) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23638, 36, 307},
    /* 313 : SUB reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10313, 38, 308},
    /* 314 : SUB rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(29) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10323, 39, 308},
    /* 315 : SUB reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10343, 41, 309},
    /* 316 : SUB rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(29) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10353, 42, 309},
    /* 317 : SUB reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 29 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(29) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10373, 41, 310},
    /* 318 : SUB rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 29 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(29) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10383, 42, 310},
    /* 319 : SUB reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1014, 38, 311},
    /* 320 : SUB rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 205 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1025, 39, 311},
    /* 321 : SUB reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1047, 41, 312},
    /* 322 : SUB rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 205 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1058, 42, 312},
    /* 323 : SUB reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /5 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 215 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1080, 41, 313},
    /* 324 : SUB rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /5 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 205 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1091, 42, 313},
    /* 325 : SUB reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /5 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 215 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10403, 35, 314},
    /* 326 : SUB rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /5 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 205 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10413, 36, 314},
    /* 327 : SUB reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /5 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 215 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+1113, 38, 315},
    /* 328 : SUB rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /5 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 205 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+1124, 39, 315},
    /* 329 : SUB reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /5 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 215 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+1146, 41, 316},
    /* 330 : SUB rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /5 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 205 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+1157, 42, 316},
    /* 331 : SUB reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /5 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 215 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1179, 41, 317},
    /* 332 : SUB rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /5 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 205 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SUB, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1190, 42, 317},
    /* 333 : XOR rm8,reg8 [mr:       hle o8 30 /r                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(30) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71509, 25, 318},
    /* 334 : XOR rm16,reg16 [mr:       hle o16 31 /r                     ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(31) 101 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66591, 25, 319},
    /* 335 : XOR rm32,reg32 [mr:       hle o32 31 /r                     ] SM0-1,LOCK,FL,386 */
        /* 273 321 001(31) 101 : SM0,SM1,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66597, 26, 320},
    /* 336 : XOR rm64,reg64 [mr:       hle o64 31 /r                     ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(31) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66603, 27, 321},
    /* 337 : XOR reg8,rm8 [rm:       o8 32 /r                          ] SM0-1,FL,8086 */
        /* 001(32) 110 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69827, 28, 322},
    /* 338 : XOR reg16,rm16 [rm:       o16 33 /r                         ] SM0-1,FL,8086 */
        /* 320 001(33) 110 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10458, 28, 323},
    /* 339 : XOR reg32,rm32 [rm:       o32 33 /r                         ] SM0-1,FL,386 */
        /* 321 001(33) 110 : SM0,SM1,FL,386 */
        {I_XOR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10488, 29, 324},
    /* 340 : XOR reg64,rm64 [rm:       o64 33 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(33) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10518, 30, 325},
    /* 341 : XOR reg_al,imm8 [-i:       o8 34 ib                          ] SM0-1,FL,8086 */
        /* 001(34) 021 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72620, 28, 326},
    /* 342 : XOR rm8,imm8 [mi:       hle 80 /6 ib                      ] SM0-1,LOCK,FL,8086 */
        /* 273 001(80) 206 021 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66609, 25, 327},
    /* 343 : XOR rm16,sbyteword16 [mi:       hle o16 83 /6 ib,s                ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(83) 206 275 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55145, 25, 328},
    /* 344 : XOR reg_ax,imm16 [-i:       o16 35 iw                         ] SM0-1,FL,8086 */
        /* 320 001(35) 031 : SM0,SM1,FL,8086 */
        {I_XOR, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71514, 28, 329},
    /* 345 : XOR rm16,imm16 [mi:       hle o16 81 /6 iw                  ] SM0-1,LOCK,FL,8086 */
        /* 273 320 001(81) 206 031 : SM0,SM1,LOCK,FL,8086 */
        {I_XOR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55152, 25, 330},
    /* 346 : XOR rm32,sbytedword32 [mi:       hle o32 83 /6 ib,s                ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(83) 206 275 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55159, 31, 331},
    /* 347 : XOR reg_eax,imm32 [-i:       o32 35 id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(35) 041 : SM0,SM1,ZU,FL,386 */
        {I_XOR, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71519, 32, 332},
    /* 348 : XOR rm32,imm32 [mi:       hle o32 81 /6 id                  ] SM0-1,ZU,LOCK,FL,386 */
        /* 273 321 001(81) 206 041 : SM0,SM1,ZU,LOCK,FL,386 */
        {I_XOR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55166, 31, 333},
    /* 349 : XOR rm64,sbytedword64 [mi:       hle o64 83 /6 ib,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 206 275 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55173, 33, 334},
    /* 350 : XOR reg_rax,sdword64 [-i:       o64 35 id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(35) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71524, 34, 335},
    /* 351 : XOR rm64,sdword64 [mi:       hle o64 81 /6 id,s                ] SM0-1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 206 255 : SM0,SM1,ZU,LOCK,LONG,FL,PROT,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55180, 33, 336},
    /* 352 : XOR reg8?,reg8,rm8 [vrm:      evex.nd1.nf.l0.m4.o8 32 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(32) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23656, 35, 337},
    /* 353 : XOR reg8,rm8 [rm:      evex.nd0.nf.l0.m4.o8 32 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(32) 110 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23665, 36, 337},
    /* 354 : XOR reg16?,reg16,rm16 [vrm:      evex.nd1.nf.l0.m4.o16 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10433, 38, 338},
    /* 355 : XOR reg16,rm16 [rm:      evex.nd0.nf.l0.m4.o16 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(33) 110 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10443, 39, 338},
    /* 356 : XOR reg32?,reg32,rm32 [vrm:      evex.nd1.nf.l0.m4.o32 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10463, 41, 339},
    /* 357 : XOR reg32,rm32 [rm:      evex.nd0.nf.l0.m4.o32 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(33) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10473, 42, 339},
    /* 358 : XOR reg64?,reg64,rm64 [vrm:      evex.nd1.nf.l0.m4.o64 33 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(33) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10493, 41, 340},
    /* 359 : XOR reg64,rm64 [rm:      evex.nd0.nf.l0.m4.o64 33 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(33) 110 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10503, 42, 340},
    /* 360 : XOR reg8?,rm8,reg8 [vmr:      evex.nd1.nf.l0.m4.o8 30 /r        ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(30) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+23683, 35, 341},
    /* 361 : XOR rm8,reg8 [mr:      evex.nd0.nf.l0.m4.o8 30 /r        ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(30) 101 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+23692, 36, 341},
    /* 362 : XOR reg16?,rm16,reg16 [vmr:      evex.nd1.nf.l0.m4.o16 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+10523, 38, 342},
    /* 363 : XOR rm16,reg16 [mr:      evex.nd0.nf.l0.m4.o16 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(31) 101 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+10533, 39, 342},
    /* 364 : XOR reg32?,rm32,reg32 [vmr:      evex.nd1.nf.l0.m4.o32 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+10553, 41, 343},
    /* 365 : XOR rm32,reg32 [mr:      evex.nd0.nf.l0.m4.o32 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(31) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+10563, 42, 343},
    /* 366 : XOR reg64?,rm64,reg64 [vmr:      evex.nd1.nf.l0.m4.o64 31 /r       ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(31) 112 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+10583, 41, 344},
    /* 367 : XOR rm64,reg64 [mr:      evex.nd0.nf.l0.m4.o64 31 /r       ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(31) 101 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+10593, 42, 344},
    /* 368 : XOR reg16?,rm16,sbyteword16 [vmi:      evex.nd1.nf.l0.m4.o16 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1212, 38, 345},
    /* 369 : XOR rm16,sbyteword16 [mi:      evex.nd0.nf.l0.m4.o16 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(83) 206 275 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1223, 39, 345},
    /* 370 : XOR reg32?,rm32,sbytedword32 [vmi:      evex.nd1.nf.l0.m4.o32 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1245, 41, 346},
    /* 371 : XOR rm32,sbytedword32 [mi:      evex.nd0.nf.l0.m4.o32 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(83) 206 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1256, 42, 346},
    /* 372 : XOR reg64?,rm64,sbytedword64 [vmi:      evex.nd1.nf.l0.m4.o64 83 /6 ib,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(83) 216 276 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1278, 41, 347},
    /* 373 : XOR rm64,sbytedword64 [mi:      evex.nd0.nf.l0.m4.o64 83 /6 ib,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(83) 206 275 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1289, 42, 347},
    /* 374 : XOR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 80 /6 ib     ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(80) 216 022 : SM0,SM1,SM2,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10613, 35, 348},
    /* 375 : XOR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 80 /6 ib     ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(80) 206 021 : SM0,SM1,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10623, 36, 348},
    /* 376 : XOR reg16?,rm16,imm16 [vmi:      evex.nd1.nf.l0.m4.o16 81 /6 iw    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(81) 216 032 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+1311, 38, 349},
    /* 377 : XOR rm16,imm16 [mi:      evex.nd0.nf.l0.m4.o16 81 /6 iw    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(81) 206 031 : SM0,SM1,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+1322, 39, 349},
    /* 378 : XOR reg32?,rm32,imm32 [vmi:      evex.nd1.nf.l0.m4.o32 81 /6 id    ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(81) 216 042 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+1344, 41, 350},
    /* 379 : XOR rm32,imm32 [mi:      evex.nd0.nf.l0.m4.o32 81 /6 id    ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(81) 206 041 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+1355, 42, 350},
    /* 380 : XOR reg64?,rm64,sdword64 [vmi:      evex.nd1.nf.l0.m4.o64 81 /6 id,s  ] SM0,SM1,SM2,NF,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(81) 216 256 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+1377, 41, 351},
    /* 381 : XOR rm64,sdword64 [mi:      evex.nd0.nf.l0.m4.o64 81 /6 id,s  ] SM0,SM1,NF,LONG,FL,EVEX,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(81) 206 255 : SM0,SM1,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_XOR, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+1388, 42, 351},
    /* 382 : CMP rm8,reg8 [mr:       hle o8 38 /r                      ] SM0-1,FL,8086 */
        /* 273 001(38) 101 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71529, 28, 352},
    /* 383 : CMP rm16,reg16 [mr:       hle o16 39 /r                     ] SM0-1,FL,8086 */
        /* 273 320 001(39) 101 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66615, 28, 353},
    /* 384 : CMP rm32,reg32 [mr:       hle o32 39 /r                     ] SM0-1,FL,386 */
        /* 273 321 001(39) 101 : SM0,SM1,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66621, 29, 354},
    /* 385 : CMP rm64,reg64 [mr:       hle o64 39 /r                     ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 273 324 001(39) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66627, 30, 355},
    /* 386 : CMP reg8,rm8 [rm:       o8 3a /r                          ] SM0-1,FL,8086 */
        /* 001(3a) 110 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+69773, 28, 356},
    /* 387 : CMP reg16,rm16 [rm:       o16 3b /r                         ] SM0-1,FL,8086 */
        /* 320 001(3b) 110 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71534, 28, 357},
    /* 388 : CMP reg32,rm32 [rm:       o32 3b /r                         ] SM0-1,FL,386 */
        /* 321 001(3b) 110 : SM0,SM1,FL,386 */
        {I_CMP, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71539, 29, 358},
    /* 389 : CMP reg64,rm64 [rm:       o64 3b /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(3b) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71544, 30, 359},
    /* 390 : CMP reg_al,imm8 [-i:       o8 3c ib                          ] SM0-1,FL,8086 */
        /* 001(3c) 021 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_AL,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72624, 28, 360},
    /* 391 : CMP rm8,imm8 [mi:       hle 80 /7 ib                      ] SM0-1,FL,8086 */
        /* 273 001(80) 207 021 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66633, 28, 361},
    /* 392 : CMP rm16,sbyteword16 [mi:       hle o16 83 /7 ib,s                ] SM0-1,FL,8086 */
        /* 273 320 001(83) 207 275 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55187, 28, 362},
    /* 393 : CMP reg_ax,imm16 [-i:       o16 3d iw                         ] SM0-1,FL,8086 */
        /* 320 001(3d) 031 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {REG_AX,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71549, 28, 363},
    /* 394 : CMP rm16,imm16 [mi:       hle o16 81 /7 iw                  ] SM0-1,FL,8086 */
        /* 273 320 001(81) 207 031 : SM0,SM1,FL,8086 */
        {I_CMP, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55194, 28, 364},
    /* 395 : CMP rm32,sbytedword32 [mi:       hle o32 83 /7 ib,s                ] SM0-1,ZU,FL,386 */
        /* 273 321 001(83) 207 275 : SM0,SM1,ZU,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55201, 32, 365},
    /* 396 : CMP reg_eax,imm32 [-i:       o32 3d id                         ] SM0-1,ZU,FL,386 */
        /* 321 001(3d) 041 : SM0,SM1,ZU,FL,386 */
        {I_CMP, 2, {REG_EAX,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71554, 32, 366},
    /* 397 : CMP rm32,imm32 [mi:       hle o32 81 /7 id                  ] SM0-1,ZU,FL,386 */
        /* 273 321 001(81) 207 041 : SM0,SM1,ZU,FL,386 */
        {I_CMP, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55208, 32, 367},
    /* 398 : CMP rm64,sbytedword64 [mi:       hle o64 83 /7 ib,s                ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 273 324 001(83) 207 275 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55215, 34, 368},
    /* 399 : CMP reg_rax,sdword64 [-i:       o64 3d id,s                       ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 324 001(3d) 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {REG_RAX,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71559, 34, 369},
    /* 400 : CMP rm64,sdword64 [mi:       hle o64 81 /7 id,s                ] SM0-1,ZU,LONG,FL,PROT,X86_64 */
        /* 273 324 001(81) 207 255 : SM0,SM1,ZU,LONG,FL,PROT,X86_64 */
        {I_CMP, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+55222, 34, 370},
    /* 401 : TEST rm8,reg8 [mr:       o8 84 /r                          ] SM0-1,FL,8086 */
        /* 001(84) 101 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72628, 28, 373},
    /* 402 : TEST rm16,reg16 [mr:       o16 85 /r                         ] SM0-1,FL,8086 */
        /* 320 001(85) 101 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71564, 28, 374},
    /* 403 : TEST rm32,reg32 [mr:       o32 85 /r                         ] SM0-1,FL,386 */
        /* 321 001(85) 101 : SM0,SM1,FL,386 */
        {I_TEST, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71569, 29, 375},
    /* 404 : TEST rm64,reg64 [mr:       o64 85 /r                         ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(85) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+71574, 30, 376},
    /* 405 : TEST reg_al,imm8 [-i:       o8 a8 ib                          ] SM0-1,NOAPX,FL,8086 */
        /* 001(a8) 021 : SM0,SM1,NOAPX,FL,8086 */
        {I_TEST, 2, {REG_AL|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72636, 53, 381},
    /* 406 : TEST reg_ax,imm16 [-i:       o16 a9 iw                         ] SM0-1,NOAPX,FL,8086 */
        /* 320 001(a9) 031 : SM0,SM1,NOAPX,FL,8086 */
        {I_TEST, 2, {REG_AX|RN_L16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71594, 53, 382},
    /* 407 : TEST reg_eax,imm32 [-i:       o32 a9 id                         ] SM0-1,NOAPX,FL,386 */
        /* 321 001(a9) 041 : SM0,SM1,NOAPX,FL,386 */
        {I_TEST, 2, {REG_EAX|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71599, 54, 383},
    /* 408 : TEST reg_rax,sdword64 [-i:       o64 a9 id,s                       ] SM0-1,NOAPX,LONG,FL,PROT,X86_64 */
        /* 324 001(a9) 255 : SM0,SM1,NOAPX,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {REG_RAX|RN_L16,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+71604, 55, 384},
    /* 409 : TEST rm8,imm8 [mi:       o8 f6 /0 ib                       ] SM0-1,FL,8086 */
        /* 001(f6) 200 021 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71609, 28, 385},
    /* 410 : TEST rm16,imm16 [mi:       o16 f7 /0 iw                      ] SM0-1,FL,8086 */
        /* 320 001(f7) 200 031 : SM0,SM1,FL,8086 */
        {I_TEST, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66639, 28, 386},
    /* 411 : TEST rm32,imm32 [mi:       o32 f7 /0 id                      ] SM0-1,FL,386 */
        /* 321 001(f7) 200 041 : SM0,SM1,FL,386 */
        {I_TEST, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66645, 29, 387},
    /* 412 : TEST rm64,sdword64 [mi:       o64 f7 /0 id,s                    ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 200 255 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_TEST, 2, {RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+66651, 30, 388},
    /* 413 : ROL rm8,unity [m-:       o8 d0 /0                          ] FL,8086 */
        /* 001(d0) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23733, 56, 391},
    /* 414 : ROL rm16,unity [m-:       o16 d1 /0                         ] FL,8086 */
        /* 320 001(d1) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10688, 56, 392},
    /* 415 : ROL rm32,unity [m-:       o32 d1 /0                         ] FL,386 */
        /* 321 001(d1) 200 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10718, 57, 393},
    /* 416 : ROL rm64,unity [m-:       o64 d1 /0                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 200 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10748, 58, 394},
    /* 417 : ROL rm8,reg_cl [m-:       o8 d2 /0                          ] FL,8086 */
        /* 001(d2) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23760, 56, 395},
    /* 418 : ROL rm16,reg_cl [m-:       o16 d3 /0                         ] FL,8086 */
        /* 320 001(d3) 200 : FL,8086 */
        {I_ROL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10778, 56, 396},
    /* 419 : ROL rm32,reg_cl [m-:       o32 d3 /0                         ] FL,386 */
        /* 321 001(d3) 200 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10808, 57, 397},
    /* 420 : ROL rm64,reg_cl [m-:       o64 d3 /0                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 200 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10838, 58, 398},
    /* 421 : ROL rm8,imm8 [mi:       o8 c0 /0 ib,u                     ] FL,186 */
        /* 001(c0) 200 025 : FL,186 */
        {I_ROL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10868, 59, 411},
    /* 422 : ROL rm16,imm8 [mi:       o16 c1 /0 ib,u                    ] FL,186 */
        /* 320 001(c1) 200 025 : FL,186 */
        {I_ROL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1437, 59, 412},
    /* 423 : ROL rm32,imm8 [mi:       o32 c1 /0 ib,u                    ] FL,386 */
        /* 321 001(c1) 200 025 : FL,386 */
        {I_ROL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1470, 57, 413},
    /* 424 : ROL rm64,imm8 [mi:       o64 c1 /0 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 200 025 : LONG,FL,PROT,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1503, 58, 414},
    /* 425 : ROR rm8,unity [m-:       o8 d0 /1                          ] FL,8086 */
        /* 001(d0) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23787, 56, 415},
    /* 426 : ROR rm16,unity [m-:       o16 d1 /1                         ] FL,8086 */
        /* 320 001(d1) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10898, 56, 416},
    /* 427 : ROR rm32,unity [m-:       o32 d1 /1                         ] FL,386 */
        /* 321 001(d1) 201 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10928, 57, 417},
    /* 428 : ROR rm64,unity [m-:       o64 d1 /1                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 201 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10958, 58, 418},
    /* 429 : ROR rm8,reg_cl [m-:       o8 d2 /1                          ] FL,8086 */
        /* 001(d2) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23814, 56, 419},
    /* 430 : ROR rm16,reg_cl [m-:       o16 d3 /1                         ] FL,8086 */
        /* 320 001(d3) 201 : FL,8086 */
        {I_ROR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10988, 56, 420},
    /* 431 : ROR rm32,reg_cl [m-:       o32 d3 /1                         ] FL,386 */
        /* 321 001(d3) 201 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11018, 57, 421},
    /* 432 : ROR rm64,reg_cl [m-:       o64 d3 /1                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 201 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11048, 58, 422},
    /* 433 : ROR rm8,imm8 [mi:       o8 c0 /1 ib,u                     ] FL,186 */
        /* 001(c0) 201 025 : FL,186 */
        {I_ROR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11078, 59, 435},
    /* 434 : ROR rm16,imm8 [mi:       o16 c1 /1 ib,u                    ] FL,186 */
        /* 320 001(c1) 201 025 : FL,186 */
        {I_ROR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1536, 59, 436},
    /* 435 : ROR rm32,imm8 [mi:       o32 c1 /1 ib,u                    ] FL,386 */
        /* 321 001(c1) 201 025 : FL,386 */
        {I_ROR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1569, 57, 437},
    /* 436 : ROR rm64,imm8 [mi:       o64 c1 /1 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 201 025 : LONG,FL,PROT,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1602, 58, 438},
    /* 437 : RCL rm8,unity [m-:       o8 d0 /2                          ] FL,8086 */
        /* 001(d0) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23841, 56, 439},
    /* 438 : RCL rm16,unity [m-:       o16 d1 /2                         ] FL,8086 */
        /* 320 001(d1) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11108, 56, 440},
    /* 439 : RCL rm32,unity [m-:       o32 d1 /2                         ] FL,386 */
        /* 321 001(d1) 202 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11138, 57, 441},
    /* 440 : RCL rm64,unity [m-:       o64 d1 /2                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 202 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11168, 58, 442},
    /* 441 : RCL rm8,reg_cl [m-:       o8 d2 /2                          ] FL,8086 */
        /* 001(d2) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23868, 56, 443},
    /* 442 : RCL rm16,reg_cl [m-:       o16 d3 /2                         ] FL,8086 */
        /* 320 001(d3) 202 : FL,8086 */
        {I_RCL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11198, 56, 444},
    /* 443 : RCL rm32,reg_cl [m-:       o32 d3 /2                         ] FL,386 */
        /* 321 001(d3) 202 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11228, 57, 445},
    /* 444 : RCL rm64,reg_cl [m-:       o64 d3 /2                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 202 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11258, 58, 446},
    /* 445 : RCL rm8,imm8 [mi:       o8 c0 /2 ib,u                     ] FL,186 */
        /* 001(c0) 202 025 : FL,186 */
        {I_RCL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11288, 59, 459},
    /* 446 : RCL rm16,imm8 [mi:       o16 c1 /2 ib,u                    ] FL,186 */
        /* 320 001(c1) 202 025 : FL,186 */
        {I_RCL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1635, 59, 460},
    /* 447 : RCL rm32,imm8 [mi:       o32 c1 /2 ib,u                    ] FL,386 */
        /* 321 001(c1) 202 025 : FL,386 */
        {I_RCL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1668, 57, 461},
    /* 448 : RCL rm64,imm8 [mi:       o64 c1 /2 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 202 025 : LONG,FL,PROT,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1701, 58, 462},
    /* 449 : RCR rm8,unity [m-:       o8 d0 /3                          ] FL,8086 */
        /* 001(d0) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23895, 56, 463},
    /* 450 : RCR rm16,unity [m-:       o16 d1 /3                         ] FL,8086 */
        /* 320 001(d1) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11318, 56, 464},
    /* 451 : RCR rm32,unity [m-:       o32 d1 /3                         ] FL,386 */
        /* 321 001(d1) 203 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11348, 57, 465},
    /* 452 : RCR rm64,unity [m-:       o64 d1 /3                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 203 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11378, 58, 466},
    /* 453 : RCR rm8,reg_cl [m-:       o8 d2 /3                          ] FL,8086 */
        /* 001(d2) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23922, 56, 467},
    /* 454 : RCR rm16,reg_cl [m-:       o16 d3 /3                         ] FL,8086 */
        /* 320 001(d3) 203 : FL,8086 */
        {I_RCR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11408, 56, 468},
    /* 455 : RCR rm32,reg_cl [m-:       o32 d3 /3                         ] FL,386 */
        /* 321 001(d3) 203 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11438, 57, 469},
    /* 456 : RCR rm64,reg_cl [m-:       o64 d3 /3                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 203 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11468, 58, 470},
    /* 457 : RCR rm8,imm8 [mi:       o8 c0 /3 ib,u                     ] FL,186 */
        /* 001(c0) 203 025 : FL,186 */
        {I_RCR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11498, 59, 483},
    /* 458 : RCR rm16,imm8 [mi:       o16 c1 /3 ib,u                    ] FL,186 */
        /* 320 001(c1) 203 025 : FL,186 */
        {I_RCR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1734, 59, 484},
    /* 459 : RCR rm32,imm8 [mi:       o32 c1 /3 ib,u                    ] FL,386 */
        /* 321 001(c1) 203 025 : FL,386 */
        {I_RCR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1767, 57, 485},
    /* 460 : RCR rm64,imm8 [mi:       o64 c1 /3 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 203 025 : LONG,FL,PROT,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1800, 58, 486},
    /* 461 : SHL rm8,unity [m-:       o8 d0 /4                          ] FL,8086 */
        /* 001(d0) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23949, 56, 487},
    /* 462 : SHL rm16,unity [m-:       o16 d1 /4                         ] FL,8086 */
        /* 320 001(d1) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11528, 56, 488},
    /* 463 : SHL rm32,unity [m-:       o32 d1 /4                         ] FL,386 */
        /* 321 001(d1) 204 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11558, 57, 489},
    /* 464 : SHL rm64,unity [m-:       o64 d1 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 204 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11588, 58, 490},
    /* 465 : SHL rm8,reg_cl [m-:       o8 d2 /4                          ] FL,8086 */
        /* 001(d2) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23976, 56, 491},
    /* 466 : SHL rm16,reg_cl [m-:       o16 d3 /4                         ] FL,8086 */
        /* 320 001(d3) 204 : FL,8086 */
        {I_SHL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11618, 56, 492},
    /* 467 : SHL rm32,reg_cl [m-:       o32 d3 /4                         ] FL,386 */
        /* 321 001(d3) 204 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11648, 57, 493},
    /* 468 : SHL rm64,reg_cl [m-:       o64 d3 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 204 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11678, 58, 494},
    /* 469 : SHL rm8,imm8 [mi:       o8 c0 /4 ib,u                     ] FL,186 */
        /* 001(c0) 204 025 : FL,186 */
        {I_SHL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11708, 59, 507},
    /* 470 : SHL rm16,imm8 [mi:       o16 c1 /4 ib,u                    ] FL,186 */
        /* 320 001(c1) 204 025 : FL,186 */
        {I_SHL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1833, 59, 508},
    /* 471 : SHL rm32,imm8 [mi:       o32 c1 /4 ib,u                    ] FL,386 */
        /* 321 001(c1) 204 025 : FL,386 */
        {I_SHL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1866, 57, 509},
    /* 472 : SHL rm64,imm8 [mi:       o64 c1 /4 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 204 025 : LONG,FL,PROT,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1899, 58, 510},
    /* 473 : SAL rm8,unity [m-:       o8 d0 /4                          ] FL,8086 */
        /* 001(d0) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23949, 56, 511},
    /* 474 : SAL rm16,unity [m-:       o16 d1 /4                         ] FL,8086 */
        /* 320 001(d1) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11528, 56, 512},
    /* 475 : SAL rm32,unity [m-:       o32 d1 /4                         ] FL,386 */
        /* 321 001(d1) 204 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11558, 57, 513},
    /* 476 : SAL rm64,unity [m-:       o64 d1 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 204 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11588, 58, 514},
    /* 477 : SAL rm8,reg_cl [m-:       o8 d2 /4                          ] FL,8086 */
        /* 001(d2) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23976, 56, 515},
    /* 478 : SAL rm16,reg_cl [m-:       o16 d3 /4                         ] FL,8086 */
        /* 320 001(d3) 204 : FL,8086 */
        {I_SAL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11618, 56, 516},
    /* 479 : SAL rm32,reg_cl [m-:       o32 d3 /4                         ] FL,386 */
        /* 321 001(d3) 204 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11648, 57, 517},
    /* 480 : SAL rm64,reg_cl [m-:       o64 d3 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 204 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11678, 58, 518},
    /* 481 : SAL rm8,imm8 [mi:       o8 c0 /4 ib,u                     ] FL,186 */
        /* 001(c0) 204 025 : FL,186 */
        {I_SAL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11708, 59, 531},
    /* 482 : SAL rm16,imm8 [mi:       o16 c1 /4 ib,u                    ] FL,186 */
        /* 320 001(c1) 204 025 : FL,186 */
        {I_SAL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1833, 59, 532},
    /* 483 : SAL rm32,imm8 [mi:       o32 c1 /4 ib,u                    ] FL,386 */
        /* 321 001(c1) 204 025 : FL,386 */
        {I_SAL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1866, 57, 533},
    /* 484 : SAL rm64,imm8 [mi:       o64 c1 /4 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 204 025 : LONG,FL,PROT,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1899, 58, 534},
    /* 485 : SHR rm8,unity [m-:       o8 d0 /5                          ] FL,8086 */
        /* 001(d0) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24003, 56, 535},
    /* 486 : SHR rm16,unity [m-:       o16 d1 /5                         ] FL,8086 */
        /* 320 001(d1) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11738, 56, 536},
    /* 487 : SHR rm32,unity [m-:       o32 d1 /5                         ] FL,386 */
        /* 321 001(d1) 205 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11768, 57, 537},
    /* 488 : SHR rm64,unity [m-:       o64 d1 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 205 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11798, 58, 538},
    /* 489 : SHR rm8,reg_cl [m-:       o8 d2 /5                          ] FL,8086 */
        /* 001(d2) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24030, 56, 539},
    /* 490 : SHR rm16,reg_cl [m-:       o16 d3 /5                         ] FL,8086 */
        /* 320 001(d3) 205 : FL,8086 */
        {I_SHR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11828, 56, 540},
    /* 491 : SHR rm32,reg_cl [m-:       o32 d3 /5                         ] FL,386 */
        /* 321 001(d3) 205 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11858, 57, 541},
    /* 492 : SHR rm64,reg_cl [m-:       o64 d3 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 205 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11888, 58, 542},
    /* 493 : SHR rm8,imm8 [mi:       o8 c0 /5 ib,u                     ] FL,186 */
        /* 001(c0) 205 025 : FL,186 */
        {I_SHR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11918, 59, 555},
    /* 494 : SHR rm16,imm8 [mi:       o16 c1 /5 ib,u                    ] FL,186 */
        /* 320 001(c1) 205 025 : FL,186 */
        {I_SHR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1932, 59, 556},
    /* 495 : SHR rm32,imm8 [mi:       o32 c1 /5 ib,u                    ] FL,386 */
        /* 321 001(c1) 205 025 : FL,386 */
        {I_SHR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1965, 57, 557},
    /* 496 : SHR rm64,imm8 [mi:       o64 c1 /5 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 205 025 : LONG,FL,PROT,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1998, 58, 558},
    /* 497 : SAR rm8,unity [m-:       o8 d0 /7                          ] FL,8086 */
        /* 001(d0) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24057, 56, 559},
    /* 498 : SAR rm16,unity [m-:       o16 d1 /7                         ] FL,8086 */
        /* 320 001(d1) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11948, 56, 560},
    /* 499 : SAR rm32,unity [m-:       o32 d1 /7                         ] FL,386 */
        /* 321 001(d1) 207 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11978, 57, 561},
    /* 500 : SAR rm64,unity [m-:       o64 d1 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d1) 207 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12008, 58, 562},
    /* 501 : SAR rm8,reg_cl [m-:       o8 d2 /7                          ] FL,8086 */
        /* 001(d2) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24084, 56, 563},
    /* 502 : SAR rm16,reg_cl [m-:       o16 d3 /7                         ] FL,8086 */
        /* 320 001(d3) 207 : FL,8086 */
        {I_SAR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12038, 56, 564},
    /* 503 : SAR rm32,reg_cl [m-:       o32 d3 /7                         ] FL,386 */
        /* 321 001(d3) 207 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12068, 57, 565},
    /* 504 : SAR rm64,reg_cl [m-:       o64 d3 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(d3) 207 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12098, 58, 566},
    /* 505 : SAR rm8,imm8 [mi:       o8 c0 /7 ib,u                     ] FL,186 */
        /* 001(c0) 207 025 : FL,186 */
        {I_SAR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+12128, 59, 579},
    /* 506 : SAR rm16,imm8 [mi:       o16 c1 /7 ib,u                    ] FL,186 */
        /* 320 001(c1) 207 025 : FL,186 */
        {I_SAR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2031, 59, 580},
    /* 507 : SAR rm32,imm8 [mi:       o32 c1 /7 ib,u                    ] FL,386 */
        /* 321 001(c1) 207 025 : FL,386 */
        {I_SAR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2064, 57, 581},
    /* 508 : SAR rm64,imm8 [mi:       o64 c1 /7 ib,u                    ] LONG,FL,PROT,X86_64 */
        /* 324 001(c1) 207 025 : LONG,FL,PROT,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2097, 58, 582},
    /* 509 : RORX reg32,rm32,imm8 [rmi:      vex.lz.f2.0f3a.w0 f0 /r ib        ] SM0-1,BMI2,FUTURE */
        /* 270(003 003) 001(f0) 110 022 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_RORX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51259, 60, 585},
    /* 510 : SHLX reg32,rm32*,reg32 [rmv:      vex.lz.66.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 001) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHLX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55229, 60, 595},
    /* 511 : SARX reg32,rm32*,reg32 [rmv:      vex.lz.f3.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 002) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SARX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55243, 60, 597},
    /* 512 : SHRX reg32,rm32*,reg32 [rmv:      vex.lz.f2.0f38.w0 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 003) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHRX, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55257, 60, 598},
    /* 513 : RORX reg64,rm64,imm8 [rmi:      vex.lz.f2.0f3a.w1 f0 /r ib        ] SM0-1,BMI2,FUTURE */
        /* 270(003 203) 001(f0) 110 022 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_RORX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51267, 60, 603},
    /* 514 : SHLX reg64,rm64*,reg64 [rmv:      vex.lz.66.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 201) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHLX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55271, 60, 617},
    /* 515 : SARX reg64,rm64*,reg64 [rmv:      vex.lz.f3.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 202) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SARX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55285, 60, 619},
    /* 516 : SHRX reg64,rm64*,reg64 [rmv:      vex.lz.f2.0f38.w1 f7 /r           ] SM0,SM1,BMI2,FUTURE */
        /* 262(002 203) 001(f7) 110 : SM0,SM1,NOAPX,VEX,BMI2,FUTURE */
        {I_SHRX, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55299, 60, 620},
    /* 517 : ROL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /0        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 210 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23710, 37, 659},
    /* 518 : ROL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /0        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 200 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23719, 64, 659},
    /* 519 : ROL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10663, 40, 660},
    /* 520 : ROL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 200 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10673, 66, 660},
    /* 521 : ROL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10693, 43, 661},
    /* 522 : ROL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10703, 68, 661},
    /* 523 : ROL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10723, 43, 662},
    /* 524 : ROL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10733, 68, 662},
    /* 525 : ROL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /0        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 210 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23737, 37, 663},
    /* 526 : ROL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /0        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 200 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23746, 64, 663},
    /* 527 : ROL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10753, 40, 664},
    /* 528 : ROL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 200 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10763, 66, 664},
    /* 529 : ROL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10783, 43, 665},
    /* 530 : ROL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10793, 68, 665},
    /* 531 : ROL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /0       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 210 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10813, 43, 666},
    /* 532 : ROL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /0       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 200 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10823, 68, 666},
    /* 533 : ROL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /0 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 210 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+10843, 37, 679},
    /* 534 : ROL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /0 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 200 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+10853, 64, 679},
    /* 535 : ROL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1410, 40, 680},
    /* 536 : ROL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 200 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1421, 66, 680},
    /* 537 : ROL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1443, 43, 681},
    /* 538 : ROL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 200 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1454, 68, 681},
    /* 539 : ROL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /0 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 210 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1476, 43, 682},
    /* 540 : ROL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /0 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 200 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1487, 68, 682},
    /* 541 : ROR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /1        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 211 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23764, 37, 683},
    /* 542 : ROR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /1        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 201 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23773, 64, 683},
    /* 543 : ROR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10873, 40, 684},
    /* 544 : ROR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 201 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10883, 66, 684},
    /* 545 : ROR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10903, 43, 685},
    /* 546 : ROR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10913, 68, 685},
    /* 547 : ROR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+10933, 43, 686},
    /* 548 : ROR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10943, 68, 686},
    /* 549 : ROR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /1        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 211 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23791, 37, 687},
    /* 550 : ROR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /1        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 201 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23800, 64, 687},
    /* 551 : ROR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10963, 40, 688},
    /* 552 : ROR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 201 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+10973, 66, 688},
    /* 553 : ROR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+10993, 43, 689},
    /* 554 : ROR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11003, 68, 689},
    /* 555 : ROR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /1       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 211 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11023, 43, 690},
    /* 556 : ROR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /1       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 201 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11033, 68, 690},
    /* 557 : ROR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /1 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 211 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11053, 37, 703},
    /* 558 : ROR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /1 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 201 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11063, 64, 703},
    /* 559 : ROR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1509, 40, 704},
    /* 560 : ROR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 201 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1520, 66, 704},
    /* 561 : ROR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1542, 43, 705},
    /* 562 : ROR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 201 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1553, 68, 705},
    /* 563 : ROR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /1 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 211 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1575, 43, 706},
    /* 564 : ROR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /1 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 201 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_ROR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1586, 68, 706},
    /* 565 : RCL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /2        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 212 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23818, 37, 707},
    /* 566 : RCL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /2        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 202 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23827, 64, 707},
    /* 567 : RCL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11083, 40, 708},
    /* 568 : RCL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 202 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11093, 66, 708},
    /* 569 : RCL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11113, 43, 709},
    /* 570 : RCL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11123, 68, 709},
    /* 571 : RCL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11143, 43, 710},
    /* 572 : RCL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11153, 68, 710},
    /* 573 : RCL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /2        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 212 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23845, 37, 711},
    /* 574 : RCL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /2        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 202 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23854, 64, 711},
    /* 575 : RCL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11173, 40, 712},
    /* 576 : RCL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 202 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11183, 66, 712},
    /* 577 : RCL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11203, 43, 713},
    /* 578 : RCL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11213, 68, 713},
    /* 579 : RCL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /2       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 212 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11233, 43, 714},
    /* 580 : RCL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /2       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 202 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11243, 68, 714},
    /* 581 : RCL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /2 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 212 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11263, 37, 727},
    /* 582 : RCL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /2 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 202 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11273, 64, 727},
    /* 583 : RCL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1608, 40, 728},
    /* 584 : RCL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 202 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1619, 66, 728},
    /* 585 : RCL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1641, 43, 729},
    /* 586 : RCL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 202 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1652, 68, 729},
    /* 587 : RCL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /2 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 212 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1674, 43, 730},
    /* 588 : RCL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /2 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 202 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1685, 68, 730},
    /* 589 : RCR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /3        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 213 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23872, 37, 731},
    /* 590 : RCR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /3        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 203 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23881, 64, 731},
    /* 591 : RCR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11293, 40, 732},
    /* 592 : RCR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 203 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11303, 66, 732},
    /* 593 : RCR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11323, 43, 733},
    /* 594 : RCR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11333, 68, 733},
    /* 595 : RCR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11353, 43, 734},
    /* 596 : RCR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11363, 68, 734},
    /* 597 : RCR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /3        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 213 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23899, 37, 735},
    /* 598 : RCR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /3        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 203 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23908, 64, 735},
    /* 599 : RCR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11383, 40, 736},
    /* 600 : RCR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 203 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11393, 66, 736},
    /* 601 : RCR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11413, 43, 737},
    /* 602 : RCR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11423, 68, 737},
    /* 603 : RCR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /3       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 213 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11443, 43, 738},
    /* 604 : RCR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /3       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 203 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11453, 68, 738},
    /* 605 : RCR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /3 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 213 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11473, 37, 751},
    /* 606 : RCR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /3 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 203 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11483, 64, 751},
    /* 607 : RCR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1707, 40, 752},
    /* 608 : RCR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 203 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1718, 66, 752},
    /* 609 : RCR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1740, 43, 753},
    /* 610 : RCR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 203 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1751, 68, 753},
    /* 611 : RCR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /3 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 213 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1773, 43, 754},
    /* 612 : RCR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /3 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 203 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_RCR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1784, 68, 754},
    /* 613 : SHL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23926, 37, 755},
    /* 614 : SHL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23935, 64, 755},
    /* 615 : SHL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11503, 40, 756},
    /* 616 : SHL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11513, 66, 756},
    /* 617 : SHL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11533, 43, 757},
    /* 618 : SHL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11543, 68, 757},
    /* 619 : SHL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11563, 43, 758},
    /* 620 : SHL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11573, 68, 758},
    /* 621 : SHL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23953, 37, 759},
    /* 622 : SHL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23962, 64, 759},
    /* 623 : SHL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11593, 40, 760},
    /* 624 : SHL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11603, 66, 760},
    /* 625 : SHL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11623, 43, 761},
    /* 626 : SHL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11633, 68, 761},
    /* 627 : SHL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11653, 43, 762},
    /* 628 : SHL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11663, 68, 762},
    /* 629 : SHL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 214 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11683, 37, 775},
    /* 630 : SHL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 204 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11693, 64, 775},
    /* 631 : SHL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1806, 40, 776},
    /* 632 : SHL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 204 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1817, 66, 776},
    /* 633 : SHL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1839, 43, 777},
    /* 634 : SHL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1850, 68, 777},
    /* 635 : SHL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1872, 43, 778},
    /* 636 : SHL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1883, 68, 778},
    /* 637 : SAL reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23926, 37, 779},
    /* 638 : SAL rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23935, 64, 779},
    /* 639 : SAL reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11503, 40, 780},
    /* 640 : SAL rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11513, 66, 780},
    /* 641 : SAL reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11533, 43, 781},
    /* 642 : SAL rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11543, 68, 781},
    /* 643 : SAL reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11563, 43, 782},
    /* 644 : SAL rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11573, 68, 782},
    /* 645 : SAL reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /4        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 214 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+23953, 37, 783},
    /* 646 : SAL rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /4        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 204 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23962, 64, 783},
    /* 647 : SAL reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11593, 40, 784},
    /* 648 : SAL rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 204 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11603, 66, 784},
    /* 649 : SAL reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11623, 43, 785},
    /* 650 : SAL rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11633, 68, 785},
    /* 651 : SAL reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /4       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 214 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11653, 43, 786},
    /* 652 : SAL rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /4       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 204 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11663, 68, 786},
    /* 653 : SAL reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 214 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11683, 37, 799},
    /* 654 : SAL rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /4 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 204 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11693, 64, 799},
    /* 655 : SAL reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1806, 40, 800},
    /* 656 : SAL rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 204 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1817, 66, 800},
    /* 657 : SAL reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1839, 43, 801},
    /* 658 : SAL rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1850, 68, 801},
    /* 659 : SAL reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 214 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1872, 43, 802},
    /* 660 : SAL rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /4 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 204 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAL, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1883, 68, 802},
    /* 661 : SHR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /5        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 215 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+23980, 37, 803},
    /* 662 : SHR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /5        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 205 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+23989, 64, 803},
    /* 663 : SHR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11713, 40, 804},
    /* 664 : SHR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 205 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11723, 66, 804},
    /* 665 : SHR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11743, 43, 805},
    /* 666 : SHR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11753, 68, 805},
    /* 667 : SHR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11773, 43, 806},
    /* 668 : SHR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11783, 68, 806},
    /* 669 : SHR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /5        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 215 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24007, 37, 807},
    /* 670 : SHR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /5        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 205 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24016, 64, 807},
    /* 671 : SHR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11803, 40, 808},
    /* 672 : SHR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 205 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11813, 66, 808},
    /* 673 : SHR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11833, 43, 809},
    /* 674 : SHR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11843, 68, 809},
    /* 675 : SHR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /5       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 215 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+11863, 43, 810},
    /* 676 : SHR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /5       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 205 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11873, 68, 810},
    /* 677 : SHR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /5 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 215 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+11893, 37, 823},
    /* 678 : SHR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /5 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 205 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+11903, 64, 823},
    /* 679 : SHR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1905, 40, 824},
    /* 680 : SHR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 205 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1916, 66, 824},
    /* 681 : SHR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1938, 43, 825},
    /* 682 : SHR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 205 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1949, 68, 825},
    /* 683 : SHR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /5 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 215 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+1971, 43, 826},
    /* 684 : SHR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /5 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 205 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SHR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+1982, 68, 826},
    /* 685 : SAR reg8?,rm8,unity [vm-:      evex.nd1.nf.l0.m4.o8 d0 /7        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d0) 217 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+24034, 37, 827},
    /* 686 : SAR rm8,unity [m-:      evex.nd0.nf.l0.m4.o8 d0 /7        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d0) 207 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24043, 64, 827},
    /* 687 : SAR reg16?,rm16,unity [vm-:      evex.nd1.nf.l0.m4.o16 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11923, 40, 828},
    /* 688 : SAR rm16,unity [m-:      evex.nd0.nf.l0.m4.o16 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d1) 207 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11933, 66, 828},
    /* 689 : SAR reg32?,rm32,unity [vm-:      evex.nd1.nf.l0.m4.o32 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11953, 43, 829},
    /* 690 : SAR rm32,unity [m-:      evex.nd0.nf.l0.m4.o32 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d1) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11963, 68, 829},
    /* 691 : SAR reg64?,rm64,unity [vm-:      evex.nd1.nf.l0.m4.o64 d1 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d1) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+11983, 43, 830},
    /* 692 : SAR rm64,unity [m-:      evex.nd0.nf.l0.m4.o64 d1 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d1) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,UNITY|IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+11993, 68, 830},
    /* 693 : SAR reg8?,rm8,reg_cl [vm-:      evex.nd1.nf.l0.m4.o8 d2 /7        ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(d2) 217 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+24061, 37, 831},
    /* 694 : SAR rm8,reg_cl [m-:      evex.nd0.nf.l0.m4.o8 d2 /7        ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(d2) 207 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+24070, 64, 831},
    /* 695 : SAR reg16?,rm16,reg_cl [vm-:      evex.nd1.nf.l0.m4.o16 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12013, 40, 832},
    /* 696 : SAR rm16,reg_cl [m-:      evex.nd0.nf.l0.m4.o16 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(d3) 207 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12023, 66, 832},
    /* 697 : SAR reg32?,rm32,reg_cl [vm-:      evex.nd1.nf.l0.m4.o32 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12043, 43, 833},
    /* 698 : SAR rm32,reg_cl [m-:      evex.nd0.nf.l0.m4.o32 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(d3) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12053, 68, 833},
    /* 699 : SAR reg64?,rm64,reg_cl [vm-:      evex.nd1.nf.l0.m4.o64 d3 /7       ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(d3) 217 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12073, 43, 834},
    /* 700 : SAR rm64,reg_cl [m-:      evex.nd0.nf.l0.m4.o64 d3 /7       ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(d3) 207 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,REG_CL,0,0,0}, NO_DECORATOR, nasm_bytecodes+12083, 68, 834},
    /* 701 : SAR reg8?,rm8,imm8 [vmi:      evex.nd1.nf.l0.m4.o8 c0 /7 ib,u   ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(c0) 217 026 : SM0,SM1,NF,NF_E,ZU,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS8,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12103, 37, 847},
    /* 702 : SAR rm8,imm8 [mi:      evex.nd0.nf.l0.m4.o8 c0 /7 ib,u   ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(c0) 207 025 : SM0,NF,NF_E,WIG,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+12113, 64, 847},
    /* 703 : SAR reg16?,rm16,imm8 [vmi:      evex.nd1.nf.l0.m4.o16 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2004, 40, 848},
    /* 704 : SAR rm16,imm8 [mi:      evex.nd0.nf.l0.m4.o16 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(c1) 207 025 : SM0,NF,NF_E,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2015, 66, 848},
    /* 705 : SAR reg32?,rm32,imm8 [vmi:      evex.nd1.nf.l0.m4.o32 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2037, 43, 849},
    /* 706 : SAR rm32,imm8 [mi:      evex.nd0.nf.l0.m4.o32 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(c1) 207 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2048, 68, 849},
    /* 707 : SAR reg64?,rm64,imm8 [vmi:      evex.nd1.nf.l0.m4.o64 c1 /7 ib,u  ] SM0,SM1,NF,ZU,LONG,FL,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(c1) 217 026 : SM0,SM1,NF,NF_E,ZU,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2070, 43, 850},
    /* 708 : SAR rm64,imm8 [mi:      evex.nd0.nf.l0.m4.o64 c1 /7 ib,u  ] SM0,NF,LONG,FL,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(c1) 207 025 : SM0,NF,NF_E,WW,LONG,FL,EVEX,PROT,APX,X86_64 */
        {I_SAR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+2081, 68, 850},
    /* 709 : RORX reg32,rm32,imm8 [rmi:      evex.lz.f2.0f3a.w0 f0 /r ib       ] SM0-1,LONG,PROT,BMI2,APX,X86_64 */
        /* 250(f3 7f 08) 300 001(f0) 110 022 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_RORX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12133, 70, 851},
    /* 710 : SHLX reg32,rm32*,reg32 [rmv:      evex.lz.66.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHLX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24088, 70, 861},
    /* 711 : SARX reg32,rm32*,reg32 [rmv:      evex.lz.f3.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7e 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SARX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24106, 70, 863},
    /* 712 : SHRX reg32,rm32*,reg32 [rmv:      evex.lz.f2.0f38.w0 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 7f 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHRX, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24124, 70, 864},
    /* 713 : RORX reg64,rm64,imm8 [rmi:      evex.lz.f2.0f3a.w1 f0 /r ib       ] SM0-1,LONG,PROT,BMI2,APX,X86_64 */
        /* 250(f3 ff 08) 300 001(f0) 110 022 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_RORX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+12143, 70, 869},
    /* 714 : SHLX reg64,rm64*,reg64 [rmv:      evex.lz.66.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHLX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24142, 70, 883},
    /* 715 : SARX reg64,rm64*,reg64 [rmv:      evex.lz.f3.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 fe 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SARX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24160, 70, 885},
    /* 716 : SHRX reg64,rm64*,reg64 [rmv:      evex.lz.f2.0f38.w1 f7 /r          ] SM0,SM1,LONG,PROT,BMI2,APX,X86_64 */
        /* 242(f2 ff 08) 300 001(f7) 110 : SM0,SM1,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_SHRX, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24178, 70, 886},
    /* 717 : INC reg16 [r:        o16 40+r                          ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 320 010 100 : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_INC, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72640, 74, 925},
    /* 718 : INC reg32 [r:        o32 40+r                          ] NOREX,NOAPX,NOLONG,FL,386 */
        /* 321 010 100 : NOREX,NOAPX,NOLONG,FL,386 */
        {I_INC, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72644, 75, 926},
    /* 719 : INC rm8 [m:        hle o8 fe /0                      ] LOCK,FL,8086 */
        /* 273 001(fe) 200 : LOCK,FL,8086 */
        {I_INC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71614, 76, 927},
    /* 720 : INC rm16 [m:        hle o16 ff /0                     ] LOCK,FL,8086 */
        /* 273 320 001(ff) 200 : LOCK,FL,8086 */
        {I_INC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66657, 76, 928},
    /* 721 : INC rm32 [m:        hle o32 ff /0                     ] LOCK,FL,386 */
        /* 273 321 001(ff) 200 : LOCK,FL,386 */
        {I_INC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66663, 77, 929},
    /* 722 : INC rm64 [m:        hle o64 ff /0                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(ff) 200 : LOCK,LONG,FL,PROT,X86_64 */
        {I_INC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66669, 78, 930},
    /* 723 : INC reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 fe /0        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(fe) 210 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24196, 79, 931},
    /* 724 : INC rm8 [m:       evex.nf.nd0.l0.m4.o8 fe /0        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(fe) 200 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24205, 80, 931},
    /* 725 : INC reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ff) 210 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12153, 81, 932},
    /* 726 : INC rm16 [m:       evex.nf.nd0.l0.m4.o16 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ff) 200 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12163, 82, 932},
    /* 727 : INC reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ff) 210 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12183, 83, 933},
    /* 728 : INC rm32 [m:       evex.nf.nd0.l0.m4.o32 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ff) 200 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12193, 84, 933},
    /* 729 : INC reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 ff /0       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ff) 210 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12213, 83, 934},
    /* 730 : INC rm64 [m:       evex.nf.nd0.l0.m4.o64 ff /0       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ff) 200 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_INC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12223, 84, 934},
    /* 731 : DEC reg16 [r:        o16 48+r                          ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 320 010 110 : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DEC, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72648, 74, 935},
    /* 732 : DEC reg32 [r:        o32 48+r                          ] NOREX,NOAPX,NOLONG,FL,386 */
        /* 321 010 110 : NOREX,NOAPX,NOLONG,FL,386 */
        {I_DEC, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72652, 75, 936},
    /* 733 : DEC rm8 [m:        hle o8 fe /1                      ] LOCK,FL,8086 */
        /* 273 001(fe) 201 : LOCK,FL,8086 */
        {I_DEC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71619, 76, 937},
    /* 734 : DEC rm16 [m:        hle o16 ff /1                     ] LOCK,FL,8086 */
        /* 273 320 001(ff) 201 : LOCK,FL,8086 */
        {I_DEC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66675, 76, 938},
    /* 735 : DEC rm32 [m:        hle o32 ff /1                     ] LOCK,FL,386 */
        /* 273 321 001(ff) 201 : LOCK,FL,386 */
        {I_DEC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66681, 77, 939},
    /* 736 : DEC rm64 [m:        hle o64 ff /1                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(ff) 201 : LOCK,LONG,FL,PROT,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66687, 78, 940},
    /* 737 : DEC reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 fe /1        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(fe) 211 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24223, 79, 941},
    /* 738 : DEC rm8 [m:       evex.nf.nd0.l0.m4.o8 fe /1        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(fe) 201 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24232, 80, 941},
    /* 739 : DEC reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ff) 211 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12243, 81, 942},
    /* 740 : DEC rm16 [m:       evex.nf.nd0.l0.m4.o16 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ff) 201 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12253, 82, 942},
    /* 741 : DEC reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ff) 211 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12273, 83, 943},
    /* 742 : DEC rm32 [m:       evex.nf.nd0.l0.m4.o32 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ff) 201 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12283, 84, 943},
    /* 743 : DEC reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 ff /1       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ff) 211 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12303, 83, 944},
    /* 744 : DEC rm64 [m:       evex.nf.nd0.l0.m4.o64 ff /1       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ff) 201 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_DEC, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12313, 84, 944},
    /* 745 : IMUL rm8 [m:        o8 f6 /5                          ] FL,8086 */
        /* 001(f6) 205 : FL,8086 */
        {I_IMUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24255, 56, 946},
    /* 746 : IMUL rm16 [m:        o16 f7 /5                         ] FL,8086 */
        /* 320 001(f7) 205 : FL,8086 */
        {I_IMUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12338, 56, 947},
    /* 747 : IMUL rm32 [m:        o32 f7 /5                         ] FL,386 */
        /* 321 001(f7) 205 : FL,386 */
        {I_IMUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12348, 57, 948},
    /* 748 : IMUL rm64 [m:        o64 f7 /5                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 205 : LONG,FL,PROT,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12358, 58, 949},
    /* 749 : IMUL rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /5        ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 205 : SM0,NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24250, 85, 950},
    /* 750 : IMUL rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 205 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12333, 86, 951},
    /* 751 : IMUL rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 205 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12343, 87, 952},
    /* 752 : IMUL rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /5       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 205 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12353, 87, 953},
    /* 753 : IMUL reg16,rm16 [rm:       o16 0f af /r                      ] SM0-1,FL,386 */
        /* 320 355 001(af) 110 : SM0,SM1,FL,386 */
        {I_IMUL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66693, 29, 954},
    /* 754 : IMUL reg32,rm32 [rm:       o32 0f af /r                      ] SM0-1,FL,386 */
        /* 321 355 001(af) 110 : SM0,SM1,FL,386 */
        {I_IMUL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66699, 29, 955},
    /* 755 : IMUL reg64,rm64 [rm:       o64 0f af /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(af) 110 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66705, 30, 956},
    /* 756 : IMUL reg16,rm16*,sbyteword16 [rmi:      o16 6b /r ib,s                    ] SM0,SM1,SM2,FL,186 */
        /* 320 001(6b) 110 276 : SM0,SM1,SM2,FL,186 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2108, 88, 957},
    /* 757 : IMUL reg32,rm32*,sbytedword32 [rmi:      o32 6b /r ib,s                    ] SM0,SM1,SM2,FL,386 */
        /* 321 001(6b) 110 276 : SM0,SM1,SM2,FL,386 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2119, 90, 958},
    /* 758 : IMUL reg64,rm64*,sbytedword64 [rmi:      o64 6b /r ib,s                    ] SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        /* 324 001(6b) 110 276 : SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2130, 91, 959},
    /* 759 : IMUL reg16,rm16*,imm16 [rmi:      o16 69 /r iw                      ] SM0,SM1,SM2,FL,186 */
        /* 320 001(69) 110 032 : SM0,SM1,SM2,FL,186 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2141, 88, 960},
    /* 760 : IMUL reg32,rm32*,imm32 [rmi:      o32 69 /r id                      ] SM0,SM1,SM2,FL,386 */
        /* 321 001(69) 110 042 : SM0,SM1,SM2,FL,386 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+2152, 90, 961},
    /* 761 : IMUL reg64,rm64*,sdword64 [rmi:      o64 69 /r id,s                    ] SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        /* 324 001(69) 110 256 : SM0,SM1,SM2,LONG,FL,PROT,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2163, 91, 962},
    /* 762 : IMUL reg16?,reg16,rm16 [vrm:      evex.nf.nd1.l0.m4.o16 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+12363, 92, 963},
    /* 763 : IMUL reg16,rm16 [rm:      evex.nf.nd0.l0.m4.o16 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(af) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12373, 93, 963},
    /* 764 : IMUL reg32?,reg32,rm32 [vrm:      evex.nf.nd1.l0.m4.o32 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+12393, 95, 964},
    /* 765 : IMUL reg32,rm32 [rm:      evex.nf.nd0.l0.m4.o32 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(af) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12403, 96, 964},
    /* 766 : IMUL reg64?,reg64,rm64 [vrm:      evex.nf.nd1.l0.m4.o64 af /r       ] SM0,SM1,SM2,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(af) 121 : SM0,SM1,SM2,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+12423, 95, 965},
    /* 767 : IMUL reg64,rm64 [rm:      evex.nf.nd0.l0.m4.o64 af /r       ] SM0,SM1,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(af) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12433, 96, 965},
    /* 768 : IMUL reg16,rm16,sbyteword16 [rmi:      evex.nf.zu.l0.m4.o16 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2103, 98, 966},
    /* 769 : IMUL reg32,rm32,sbytedword32 [rmi:      evex.nf.zu.l0.m4.o32 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2114, 99, 967},
    /* 770 : IMUL reg64,rm64,sbytedword64 [rmi:      evex.nf.zu.l0.m4.o64 6b /r ib,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(6b) 110 276 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2125, 99, 968},
    /* 771 : IMUL reg16,rm16,imm16 [rmi:      evex.nf.zu.l0.m4.o16 69 /r iw     ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(69) 110 032 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS16,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2136, 98, 969},
    /* 772 : IMUL reg32,rm32,imm32 [rmi:      evex.nf.zu.l0.m4.o32 69 /r id     ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(69) 110 042 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS32,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+2147, 99, 970},
    /* 773 : IMUL reg64,rm64,sdword64 [rmi:      evex.nf.zu.l0.m4.o64 69 /r id,s   ] SM0-2,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(69) 110 256 : SM0,SM1,SM2,NF,NF_E,ZU,ZU_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IMUL, 3, {REG_GPR|BITS64,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2158, 99, 971},
    /* 774 : MUL rm8 [m:        o8 f6 /4                          ] FL,8086 */
        /* 001(f6) 204 : FL,8086 */
        {I_MUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24300, 56, 973},
    /* 775 : MUL rm16 [m:        o16 f7 /4                         ] FL,8086 */
        /* 320 001(f7) 204 : FL,8086 */
        {I_MUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12458, 56, 974},
    /* 776 : MUL rm32 [m:        o32 f7 /4                         ] FL,386 */
        /* 321 001(f7) 204 : FL,386 */
        {I_MUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12468, 57, 975},
    /* 777 : MUL rm64 [m:        o64 f7 /4                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 204 : LONG,FL,PROT,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12478, 58, 976},
    /* 778 : MULX reg32,reg32*,rm32 [rvm:      vex.ndd.lz.f2.0f38.w0 f6 /r      ] BMI2,FUTURE */
        /* 261(002 003) 001(f6) 120 : NOAPX,VEX,BMI2,FUTURE */
        {I_MULX, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55313, 100, 977},
    /* 779 : MULX reg32,reg32*,rm32 [rvm:      evex.ndd.lz.f2.0f38.w0 f6 /r      ] BMI2,FUTURE,APX */
        /* 241(f2 7f 08) 300 001(f6) 120 : LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_MULX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24259, 101, 977},
    /* 780 : MULX reg64,reg64*,rm64 [rvm:      vex.ndd.lz.f2.0f38.w1 f6 /r      ] LONG,PROT,BMI2,X86_64 */
        /* 261(002 203) 001(f6) 120 : NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_MULX, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55327, 102, 978},
    /* 781 : MULX reg64,reg64*,rm64 [rvm:      evex.ndd.lz.f2.0f38.w1 f6 /r      ] LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 ff 08) 300 001(f6) 120 : LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_MULX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24277, 103, 978},
    /* 782 : MUL rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /4        ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 204 : SM0,NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24295, 85, 981},
    /* 783 : MUL rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 204 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12453, 86, 982},
    /* 784 : MUL rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 204 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12463, 87, 983},
    /* 785 : MUL rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /4       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 204 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_MUL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12473, 87, 984},
    /* 786 : IDIV rm8 [m:        o8 f6 /7                          ] FL,8086 */
        /* 001(f6) 207 : FL,8086 */
        {I_IDIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24309, 56, 1006},
    /* 787 : IDIV rm16 [m:        o16 f7 /7                         ] FL,8086 */
        /* 320 001(f7) 207 : FL,8086 */
        {I_IDIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12488, 56, 1007},
    /* 788 : IDIV rm32 [m:        o32 f7 /7                         ] FL,386 */
        /* 321 001(f7) 207 : FL,386 */
        {I_IDIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12498, 57, 1008},
    /* 789 : IDIV rm64 [m:        o64 f7 /7                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 207 : LONG,FL,PROT,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12508, 58, 1009},
    /* 790 : IDIV rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /7        ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 207 : NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24304, 106, 1010},
    /* 791 : IDIV rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 207 : NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12483, 107, 1011},
    /* 792 : IDIV rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 207 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12493, 108, 1012},
    /* 793 : IDIV rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /7       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 207 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_IDIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12503, 108, 1013},
    /* 794 : DIV rm8 [m:        o8 f6 /6                          ] FL,8086 */
        /* 001(f6) 206 : FL,8086 */
        {I_DIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24318, 56, 1014},
    /* 795 : DIV rm16 [m:        o16 f7 /6                         ] FL,8086 */
        /* 320 001(f7) 206 : FL,8086 */
        {I_DIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12518, 56, 1015},
    /* 796 : DIV rm32 [m:        o32 f7 /6                         ] FL,386 */
        /* 321 001(f7) 206 : FL,386 */
        {I_DIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12528, 57, 1016},
    /* 797 : DIV rm64 [m:        o64 f7 /6                         ] LONG,FL,PROT,X86_64 */
        /* 324 001(f7) 206 : LONG,FL,PROT,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12538, 58, 1017},
    /* 798 : DIV rm8 [m:        evex.nf.nd0.l0.m4.o8 f6 /6        ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 206 : NF,NF_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24313, 106, 1018},
    /* 799 : DIV rm16 [m:        evex.nf.nd0.l0.m4.o16 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 206 : NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12513, 107, 1019},
    /* 800 : DIV rm32 [m:        evex.nf.nd0.l0.m4.o32 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 206 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12523, 108, 1020},
    /* 801 : DIV rm64 [m:        evex.nf.nd0.l0.m4.o64 f7 /6       ] NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 206 : NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_DIV, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12533, 108, 1021},
    /* 802 : NEG rm8 [m:        hle o8 f6 /3                      ] LOCK,FL,8086 */
        /* 273 001(f6) 203 : LOCK,FL,8086 */
        {I_NEG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71624, 76, 1023},
    /* 803 : NEG rm16 [m:        hle o16 f7 /3                     ] LOCK,FL,8086 */
        /* 273 320 001(f7) 203 : LOCK,FL,8086 */
        {I_NEG, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66747, 76, 1024},
    /* 804 : NEG rm32 [m:        hle o32 f7 /3                     ] LOCK,FL,386 */
        /* 273 321 001(f7) 203 : LOCK,FL,386 */
        {I_NEG, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66753, 77, 1025},
    /* 805 : NEG rm64 [m:        hle o64 f7 /3                     ] LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 001(f7) 203 : LOCK,LONG,FL,PROT,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66759, 78, 1026},
    /* 806 : NEG reg8?,rm8 [vm:       evex.nf.nd1.l0.m4.o8 f6 /3        ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(f6) 213 : NF,NF_E,ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24322, 79, 1027},
    /* 807 : NEG rm8 [m:       evex.nf.nd0.l0.m4.o8 f6 /3        ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 203 : NF,NF_E,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24331, 80, 1027},
    /* 808 : NEG reg16?,rm16 [vm:       evex.nf.nd1.l0.m4.o16 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(f7) 213 : NF,NF_E,ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12543, 81, 1028},
    /* 809 : NEG rm16 [m:       evex.nf.nd0.l0.m4.o16 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 203 : NF,NF_E,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12553, 82, 1028},
    /* 810 : NEG reg32?,rm32 [vm:       evex.nf.nd1.l0.m4.o32 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(f7) 213 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12573, 83, 1029},
    /* 811 : NEG rm32 [m:       evex.nf.nd0.l0.m4.o32 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 203 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12583, 84, 1029},
    /* 812 : NEG reg64?,rm64 [vm:       evex.nf.nd1.l0.m4.o64 f7 /3       ] NF,ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(f7) 213 : NF,NF_E,ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12603, 83, 1030},
    /* 813 : NEG rm64 [m:       evex.nf.nd0.l0.m4.o64 f7 /3       ] NF,LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 203 : NF,NF_E,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NEG, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12613, 84, 1030},
    /* 814 : NOT rm8 [m:        hle o8 f6 /2                      ] LOCK,8086 */
        /* 273 001(f6) 202 : LOCK,8086 */
        {I_NOT, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71629, 109, 1031},
    /* 815 : NOT rm16 [m:        hle o16 f7 /2                     ] LOCK,8086 */
        /* 273 320 001(f7) 202 : LOCK,8086 */
        {I_NOT, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66765, 109, 1032},
    /* 816 : NOT rm32 [m:        hle o32 f7 /2                     ] LOCK,386 */
        /* 273 321 001(f7) 202 : LOCK,386 */
        {I_NOT, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66771, 110, 1033},
    /* 817 : NOT rm64 [m:        hle o64 f7 /2                     ] LOCK,LONG,PROT,X86_64 */
        /* 273 324 001(f7) 202 : LOCK,LONG,PROT,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66777, 111, 1034},
    /* 818 : NOT reg8?,rm8 [vm:       evex.nf0.nd1.l0.m4.o8 f6 /2       ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 001(f6) 212 : ZU,WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+24349, 112, 1035},
    /* 819 : NOT rm8 [m:       evex.nf0.nd0.l0.m4.o8 f6 /2       ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f6) 202 : WIG,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+24358, 113, 1035},
    /* 820 : NOT reg16?,rm16 [vm:       evex.nf0.nd1.l0.m4.o16 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(f7) 212 : ZU,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12633, 114, 1036},
    /* 821 : NOT rm16 [m:       evex.nf0.nd0.l0.m4.o16 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f7) 202 : LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12643, 115, 1036},
    /* 822 : NOT reg32?,rm32 [vm:       evex.nf0.nd1.l0.m4.o32 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(f7) 212 : ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12663, 116, 1037},
    /* 823 : NOT rm32 [m:       evex.nf0.nd0.l0.m4.o32 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f7) 202 : WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12673, 117, 1037},
    /* 824 : NOT reg64?,rm64 [vm:       evex.nf0.nd1.l0.m4.o64 f7 /2      ] ZU,LOCK,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(f7) 212 : ZU,WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12693, 116, 1038},
    /* 825 : NOT rm64 [m:       evex.nf0.nd0.l0.m4.o64 f7 /2      ] LOCK,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f7) 202 : WW,LOCK,LONG,EVEX,PROT,APX,X86_64 */
        {I_NOT, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+12703, 117, 1038},
    /* 826 : ADCX reg32,rm32 [rm:       66 w0 0f38 f6 /r                  ] ZU,FL,ADX,FUTURE */
        /* 361 317 356 001(f6) 110 : NOAPX,ZU,FL,ADX,FUTURE */
        {I_ADCX, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55341, 118, 1041},
    /* 827 : ADCX reg64,rm64 [rm:       66 w1 0f38 f6 /r                  ] ZU,LONG,FL,PROT,ADX,X86_64 */
        /* 361 324 356 001(f6) 110 : NOAPX,ZU,LONG,FL,PROT,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55348, 119, 1042},
    /* 828 : ADCX reg32?,reg32,rm32 [vrm:      evex.nd1.l0.66.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 7d 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24376, 120, 1043},
    /* 829 : ADCX reg32,rm32 [rm:      evex.nd0.l0.66.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 7d 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24385, 120, 1043},
    /* 830 : ADCX reg64?,reg64,rm64 [vrm:      evex.nd1.l0.66.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 fd 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24394, 120, 1044},
    /* 831 : ADCX reg64,rm64 [rm:      evex.nd0.l0.66.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 fd 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADCX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24403, 120, 1044},
    /* 832 : ADOX reg32,rm32 [rm:       f3 !osp w0 0f38 f6 /r             ] ZU,FL,ADX,FUTURE */
        /* 333 364 317 356 001(f6) 110 : NOAPX,ZU,FL,ADX,FUTURE */
        {I_ADOX, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51275, 118, 1045},
    /* 833 : ADOX reg64,rm64 [rm:       f3 !osp w1 0f38 f6 /r             ] ZU,LONG,FL,PROT,ADX,X86_64 */
        /* 333 364 324 356 001(f6) 110 : NOAPX,ZU,LONG,FL,PROT,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51283, 119, 1046},
    /* 834 : ADOX reg32?,reg32,rm32 [vrm:      evex.nd1.l0.f3.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 7e 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24412, 120, 1047},
    /* 835 : ADOX reg32,rm32 [rm:      evex.nd0.l0.f3.m4.w0 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 7e 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24421, 120, 1047},
    /* 836 : ADOX reg64?,reg64,rm64 [vrm:      evex.nd1.l0.f3.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 240(f4 fe 18) 300 001(66) 121 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24430, 120, 1048},
    /* 837 : ADOX reg64,rm64 [rm:      evex.nd0.l0.f3.m4.w1 66 /r        ] ZU,LONG,FL,PROT,APX,ADX,X86_64 */
        /* 250(f4 fe 08) 300 001(66) 110 : ZU,LONG,FL,EVEX,PROT,APX,ADX,X86_64 */
        {I_ADOX, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24439, 120, 1048},
    /* 838 : SHLD rm16,reg16,imm8 [mri:      o16 0f a4 /r ib,u                 ] SM0-1,FL,386 */
        /* 320 355 001(a4) 101 026 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55355, 29, 1051},
    /* 839 : SHLD rm32,reg32,imm8 [mri:      o32 0f a4 /r ib,u                 ] SM0-1,FL,386 */
        /* 321 355 001(a4) 101 026 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55362, 29, 1052},
    /* 840 : SHLD rm64,reg64,imm8 [mri:      o64 0f a4 /r ib,u                 ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a4) 101 026 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55369, 30, 1053},
    /* 841 : SHLD rm16,reg16,reg_cl [mr-:      o16 0f a5 /r                      ] SM0-1,FL,386 */
        /* 320 355 001(a5) 101 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66783, 29, 1054},
    /* 842 : SHLD rm32,reg32,reg_cl [mr-:      o32 0f a5 /r                      ] SM0-1,FL,386 */
        /* 321 355 001(a5) 101 : SM0,SM1,FL,386 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66789, 29, 1055},
    /* 843 : SHLD rm64,reg64,reg_cl [mr-:      o64 0f a5 /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a5) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66795, 30, 1056},
    /* 844 : SHLD reg16?,rm16,reg16,imm8 [vmri:     evex.nd1.nf.l0.m4.o16 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2169, 94, 1057},
    /* 845 : SHLD rm16,reg16,imm8 [mri:     evex.nd0.nf.l0.m4.o16 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(24) 101 026 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2180, 86, 1057},
    /* 846 : SHLD reg32?,rm32,reg32,imm8 [vmri:     evex.nd1.nf.l0.m4.o32 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2202, 97, 1058},
    /* 847 : SHLD rm32,reg32,imm8 [mri:     evex.nd0.nf.l0.m4.o32 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(24) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2213, 87, 1058},
    /* 848 : SHLD reg64?,rm64,reg64,imm8 [vmri:     evex.nd1.nf.l0.m4.o64 24 /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(24) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2235, 97, 1059},
    /* 849 : SHLD rm64,reg64,imm8 [mri:     evex.nd0.nf.l0.m4.o64 24 /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(24) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2246, 87, 1059},
    /* 850 : SHLD reg16?,rm16,reg16,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o16 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12723, 94, 1060},
    /* 851 : SHLD rm16,reg16,reg_cl [mr-:     evex.nd0.nf.l0.m4.o16 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(a5) 101 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12733, 86, 1060},
    /* 852 : SHLD reg32?,rm32,reg32,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o32 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12753, 97, 1061},
    /* 853 : SHLD rm32,reg32,reg_cl [mr-:     evex.nd0.nf.l0.m4.o32 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(a5) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12763, 87, 1061},
    /* 854 : SHLD reg64?,rm64,reg64,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o64 a5 /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(a5) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12783, 97, 1062},
    /* 855 : SHLD rm64,reg64,reg_cl [mr-:     evex.nd0.nf.l0.m4.o64 a5 /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(a5) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHLD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12793, 87, 1062},
    /* 856 : SHRD rm16,reg16,imm8 [mri:      o16 0f ac /r ib,u                 ] SM0-1,FL,386 */
        /* 320 355 001(ac) 101 026 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55376, 29, 1063},
    /* 857 : SHRD rm32,reg32,imm8 [mri:      o32 0f ac /r ib,u                 ] SM0-1,FL,386 */
        /* 321 355 001(ac) 101 026 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55383, 29, 1064},
    /* 858 : SHRD rm64,reg64,imm8 [mri:      o64 0f ac /r ib,u                 ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ac) 101 026 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+55390, 30, 1065},
    /* 859 : SHRD rm16,reg16,reg_cl [mr-:      o16 0f ad /r                      ] SM0-1,FL,386 */
        /* 320 355 001(ad) 101 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66801, 29, 1066},
    /* 860 : SHRD rm32,reg32,reg_cl [mr-:      o32 0f ad /r                      ] SM0-1,FL,386 */
        /* 321 355 001(ad) 101 : SM0,SM1,FL,386 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66807, 29, 1067},
    /* 861 : SHRD rm64,reg64,reg_cl [mr-:      o64 0f ad /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ad) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+66813, 30, 1068},
    /* 862 : SHRD reg16?,rm16,reg16,imm8 [vmri:     evex.nd1.nf.l0.m4.o16 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2268, 94, 1069},
    /* 863 : SHRD rm16,reg16,imm8 [mri:     evex.nd0.nf.l0.m4.o16 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(2c) 101 026 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2279, 86, 1069},
    /* 864 : SHRD reg32?,rm32,reg32,imm8 [vmri:     evex.nd1.nf.l0.m4.o32 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2301, 97, 1070},
    /* 865 : SHRD rm32,reg32,imm8 [mri:     evex.nd0.nf.l0.m4.o32 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(2c) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2312, 87, 1070},
    /* 866 : SHRD reg64?,rm64,reg64,imm8 [vmri:     evex.nd1.nf.l0.m4.o64 2c /r ib,u  ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(2c) 112 027 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+2334, 97, 1071},
    /* 867 : SHRD rm64,reg64,imm8 [mri:     evex.nd0.nf.l0.m4.o64 2c /r ib,u  ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(2c) 101 026 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+2345, 87, 1071},
    /* 868 : SHRD reg16?,rm16,reg16,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o16 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS16,RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12813, 94, 1072},
    /* 869 : SHRD rm16,reg16,reg_cl [mr-:     evex.nd0.nf.l0.m4.o16 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(ad) 101 : SM0,NF,NF_E,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS16,REG_GPR|BITS16,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12823, 86, 1072},
    /* 870 : SHRD reg32?,rm32,reg32,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o32 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12843, 97, 1073},
    /* 871 : SHRD rm32,reg32,reg_cl [mr-:     evex.nd0.nf.l0.m4.o32 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(ad) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS32,REG_GPR|BITS32,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12853, 87, 1073},
    /* 872 : SHRD reg64?,rm64,reg64,reg_cl [vmr-:     evex.nd1.nf.l0.m4.o64 ad /r       ] SM0,SM1,NF,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(ad) 112 : SM0,SM1,NF,NF_E,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 4, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0}, NO_DECORATOR, nasm_bytecodes+12873, 97, 1074},
    /* 873 : SHRD rm64,reg64,reg_cl [mr-:     evex.nd0.nf.l0.m4.o64 ad /r       ] SM0,NF,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(ad) 101 : SM0,NF,NF_E,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_SHRD, 3, {RM_GPR|BITS64,REG_GPR|BITS64,REG_CL,0,0}, NO_DECORATOR, nasm_bytecodes+12883, 87, 1074},
    /* 874 : BT rm16,reg16 [mr:       o16 0f a3 /r                      ] SM0-1,FL,386 */
        /* 320 355 001(a3) 101 : SM0,SM1,FL,386 */
        {I_BT, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66819, 29, 1077},
    /* 875 : BT rm32,reg32 [mr:       o32 0f a3 /r                      ] SM0-1,FL,386 */
        /* 321 355 001(a3) 101 : SM0,SM1,FL,386 */
        {I_BT, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66825, 29, 1078},
    /* 876 : BT rm64,reg64 [mr:       o64 0f a3 /r                      ] SM0-1,LONG,FL,PROT,X86_64 */
        /* 324 355 001(a3) 101 : SM0,SM1,LONG,FL,PROT,X86_64 */
        {I_BT, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66831, 30, 1079},
    /* 877 : BT rm16,imm8 [mi:       o16 0f ba /4 ib,u                 ] FL,386 */
        /* 320 355 001(ba) 204 025 : FL,386 */
        {I_BT, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55397, 57, 1080},
    /* 878 : BT rm32,imm8 [mi:       o32 0f ba /4 ib,u                 ] FL,386 */
        /* 321 355 001(ba) 204 025 : FL,386 */
        {I_BT, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55404, 57, 1081},
    /* 879 : BT rm64,imm8 [mi:       o64 0f ba /4 ib,u                 ] LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 204 025 : LONG,FL,PROT,X86_64 */
        {I_BT, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55411, 58, 1082},
    /* 880 : BTC rm16,reg16 [mr:       o16 0f bb /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(bb) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66837, 26, 1083},
    /* 881 : BTC rm32,reg32 [mr:       o32 0f bb /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(bb) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66843, 26, 1084},
    /* 882 : BTC rm64,reg64 [mr:       o64 0f bb /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(bb) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66849, 27, 1085},
    /* 883 : BTC rm16,imm8 [mi:       o16 0f ba /7 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 207 025 : LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55418, 77, 1086},
    /* 884 : BTC rm32,imm8 [mi:       o32 0f ba /7 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 207 025 : LOCK,FL,386 */
        {I_BTC, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55425, 77, 1087},
    /* 885 : BTC rm64,imm8 [mi:       o64 0f ba /7 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 207 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTC, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55432, 78, 1088},
    /* 886 : BTR rm16,reg16 [mr:       o16 0f b3 /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(b3) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66855, 26, 1089},
    /* 887 : BTR rm32,reg32 [mr:       o32 0f b3 /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(b3) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66861, 26, 1090},
    /* 888 : BTR rm64,reg64 [mr:       o64 0f b3 /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(b3) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTR, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66867, 27, 1091},
    /* 889 : BTR rm16,imm8 [mi:       o16 0f ba /6 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 206 025 : LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55439, 77, 1092},
    /* 890 : BTR rm32,imm8 [mi:       o32 0f ba /6 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 206 025 : LOCK,FL,386 */
        {I_BTR, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55446, 77, 1093},
    /* 891 : BTR rm64,imm8 [mi:       o64 0f ba /6 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 206 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTR, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55453, 78, 1094},
    /* 892 : BTS rm16,reg16 [mr:       o16 0f ab /r                      ] SM0-1,LOCK,FL,386 */
        /* 320 355 001(ab) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66873, 26, 1095},
    /* 893 : BTS rm32,reg32 [mr:       o32 0f ab /r                      ] SM0-1,LOCK,FL,386 */
        /* 321 355 001(ab) 101 : SM0,SM1,LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+66879, 26, 1096},
    /* 894 : BTS rm64,reg64 [mr:       o64 0f ab /r                      ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ab) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_BTS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+66885, 27, 1097},
    /* 895 : BTS rm16,imm8 [mi:       o16 0f ba /5 ib,u                 ] LOCK,FL,386 */
        /* 320 355 001(ba) 205 025 : LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55460, 77, 1098},
    /* 896 : BTS rm32,imm8 [mi:       o32 0f ba /5 ib,u                 ] LOCK,FL,386 */
        /* 321 355 001(ba) 205 025 : LOCK,FL,386 */
        {I_BTS, 2, {RM_GPR|BITS32,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55467, 77, 1099},
    /* 897 : BTS rm64,imm8 [mi:       o64 0f ba /5 ib,u                 ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(ba) 205 025 : LOCK,LONG,FL,PROT,X86_64 */
        {I_BTS, 2, {RM_GPR|BITS64,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+55474, 78, 1100},
    /* 898 : BSF reg16,rm16 [rm:       o16 nof3 0f bc /r                 ] SM0-1,386 */
        /* 320 326 355 001(bc) 110 : SM0,SM1,386 */
        {I_BSF, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55481, 13, 1102},
    /* 899 : BSF reg32,rm32 [rm:       o32 nof3 0f bc /r                 ] SM0-1,386 */
        /* 321 326 355 001(bc) 110 : SM0,SM1,386 */
        {I_BSF, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55488, 13, 1103},
    /* 900 : BSF reg64,rm64 [rm:       o64 nof3 0f bc /r                 ] SM0-1,LONG,PROT,X86_64 */
        /* 324 326 355 001(bc) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_BSF, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55495, 14, 1104},
    /* 901 : BSR reg16,rm16 [rm:       o16 nof3 0f bd /r                 ] SM0-1,386 */
        /* 320 326 355 001(bd) 110 : SM0,SM1,386 */
        {I_BSR, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55502, 13, 1105},
    /* 902 : BSR reg32,rm32 [rm:       o32 nof3 0f bd /r                 ] SM0-1,386 */
        /* 321 326 355 001(bd) 110 : SM0,SM1,386 */
        {I_BSR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55509, 13, 1106},
    /* 903 : BSR reg64,rm64 [rm:       o64 nof3 0f bd /r                 ] SM0-1,LONG,PROT,X86_64 */
        /* 324 326 355 001(bd) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_BSR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55516, 14, 1107},
    /* 904 : LZCNT reg16,rm16 [rm:       o16 f3i 0f bd /r                  ] SM0-1,FL,LZCNT,FUTURE */
        /* 320 333 355 001(bd) 110 : SM0,SM1,FL,LZCNT,FUTURE */
        {I_LZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55523, 124, 1116},
    /* 905 : LZCNT reg32,rm32 [rm:       o32 f3i 0f bd /r                  ] SM0-1,FL,LZCNT,FUTURE */
        /* 321 333 355 001(bd) 110 : SM0,SM1,FL,LZCNT,FUTURE */
        {I_LZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55530, 124, 1117},
    /* 906 : LZCNT reg64,rm64 [rm:       o64 f3i 0f bd /r                  ] SM0-1,LONG,FL,PROT,LZCNT,X86_64 */
        /* 324 333 355 001(bd) 110 : SM0,SM1,LONG,FL,PROT,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55537, 125, 1118},
    /* 907 : LZCNT reg16,rm16 [rm:       evex.nd0.nf.l0.m4.o16 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f5) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12903, 126, 1119},
    /* 908 : LZCNT reg32,rm32 [rm:       evex.nd0.nf.l0.m4.o32 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f5) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12913, 127, 1120},
    /* 909 : LZCNT reg64,rm64 [rm:       evex.nd0.nf.l0.m4.o64 f5 /r       ] SM0-1,NF,LONG,PROT,APX,LZCNT,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f5) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,LZCNT,X86_64 */
        {I_LZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12923, 127, 1121},
    /* 910 : TZCNT reg16,rm16 [rm:       o16 f3i 0f bc /r                  ] SM0-1,FL,BMI1,FUTURE */
        /* 320 333 355 001(bc) 110 : SM0,SM1,FL,BMI1,FUTURE */
        {I_TZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55544, 128, 1122},
    /* 911 : TZCNT reg32,rm32 [rm:       o32 f3i 0f bc /r                  ] SM0-1,FL,BMI1,FUTURE */
        /* 321 333 355 001(bc) 110 : SM0,SM1,FL,BMI1,FUTURE */
        {I_TZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55551, 128, 1123},
    /* 912 : TZCNT reg64,rm64 [rm:       o64 f3i 0f bc /r                  ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 324 333 355 001(bc) 110 : SM0,SM1,LONG,FL,PROT,BMI1,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55558, 129, 1124},
    /* 913 : TZCNT reg16,rm16 [rm:       evex.nd0.nf.l0.m4.o16 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f4) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12933, 130, 1125},
    /* 914 : TZCNT reg32,rm32 [rm:       evex.nd0.nf.l0.m4.o32 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f4) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12943, 131, 1126},
    /* 915 : TZCNT reg64,rm64 [rm:       evex.nd0.nf.l0.m4.o64 f4 /r       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f4) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_TZCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12953, 131, 1127},
    /* 916 : ANDN reg32,reg32*,rm32 [rvm:      vex.nds.lz.0f38.w0 f2 /r          ] SM0,SM1,SM2,FL,BMI1,FUTURE */
        /* 261(002 000) 001(f2) 120 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_ANDN, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55565, 132, 1129},
    /* 917 : ANDN reg64,reg64*,rm64 [rvm:      vex.nds.lz.0f38.w1 f2 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI1,X86_64 */
        /* 261(002 200) 001(f2) 120 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55579, 134, 1130},
    /* 918 : ANDN reg32,reg32*,rm32 [rvm:      evex.np.nf.lz.0f38.w0 f2 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 241(f2 7c 08) 300 001(f2) 120 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24448, 136, 1131},
    /* 919 : ANDN reg64,reg64*,rm64 [rvm:      evex.np.nf.lz.0f38.w1 f2 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 241(f2 fc 08) 300 001(f2) 120 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_ANDN, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24466, 136, 1132},
    /* 920 : BEXTR reg32,rm32*,reg32 [rmv:      vex.nds.lz.0f38.w0 f7 /r          ] SM0,SM1,SM2,FL,BMI1,FUTURE */
        /* 262(002 000) 001(f7) 110 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BEXTR, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55593, 132, 1133},
    /* 921 : BEXTR reg64,rm64*,reg64 [rmv:      vex.nds.lz.0f38.w1 f7 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI1,X86_64 */
        /* 262(002 200) 001(f7) 110 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55607, 134, 1134},
    /* 922 : BEXTR reg32,rm32*,reg32 [rmv:      evex.np.nf.lz.0f38.w0 f7 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 7c 08) 300 001(f7) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24484, 136, 1135},
    /* 923 : BEXTR reg64,rm64*,reg64 [rmv:      evex.np.nf.lz.0f38.w1 f7 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 fc 08) 300 001(f7) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BEXTR, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24502, 136, 1136},
    /* 924 : BLSMSK reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /2          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 212 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55621, 133, 1137},
    /* 925 : BLSMSK reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /2          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 212 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55628, 135, 1138},
    /* 926 : BLSMSK reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /2       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 212 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24520, 130, 1139},
    /* 927 : BLSMSK reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /2       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 212 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSMSK, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24529, 130, 1140},
    /* 928 : BLSR reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /1          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 211 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSR, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55635, 133, 1141},
    /* 929 : BLSR reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /1          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 211 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55642, 135, 1142},
    /* 930 : BLSR reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /1       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 211 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24538, 130, 1143},
    /* 931 : BLSR reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /1       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 211 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSR, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24547, 130, 1144},
    /* 932 : BLSI reg32,rm32 [vm:       vex.ndd.lz.0f38.w0 f3 /3          ] SM0-1,FL,BMI1,FUTURE */
        /* 260(002 000) 001(f3) 213 : SM0,SM1,NOAPX,FL,VEX,BMI1,FUTURE */
        {I_BLSI, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55649, 133, 1145},
    /* 933 : BLSI reg64,rm64 [vm:       vex.ndd.lz.0f38.w1 f3 /3          ] SM0-1,LONG,FL,PROT,BMI1,X86_64 */
        /* 260(002 200) 001(f3) 213 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,BMI1,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55656, 135, 1146},
    /* 934 : BLSI reg32,rm32 [vm:       evex.np.nf.lz.0f38.w0 f3 /3       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 7c 08) 300 001(f3) 213 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+24556, 130, 1147},
    /* 935 : BLSI reg64,rm64 [vm:       evex.np.nf.lz.0f38.w1 f3 /3       ] SM0-1,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 240(f2 fc 08) 300 001(f3) 213 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BLSI, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24565, 130, 1148},
    /* 936 : BZHI reg32,rm32*,reg32 [rmv:      vex.nds.lz.0f38.w0 f5 /r          ] SM0,SM1,SM2,FL,BMI2,FUTURE */
        /* 262(002 000) 001(f5) 110 : SM0,SM1,SM2,NOAPX,FL,VEX,BMI2,FUTURE */
        {I_BZHI, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55663, 137, 1149},
    /* 937 : BZHI reg64,rm64*,reg64 [rmv:      vex.nds.lz.0f38.w1 f5 /r          ] SM0,SM1,SM2,LONG,FL,PROT,BMI2,X86_64 */
        /* 262(002 200) 001(f5) 110 : SM0,SM1,SM2,NOAPX,LONG,FL,VEX,PROT,BMI2,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55677, 139, 1150},
    /* 938 : BZHI reg32,rm32*,reg32 [rmv:      evex.np.nf.lz.0f38.w0 f5 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 7c 08) 300 001(f5) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS32,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24574, 136, 1151},
    /* 939 : BZHI reg64,rm64*,reg64 [rmv:      evex.np.nf.lz.0f38.w1 f5 /r       ] SM0,SM1,SM2,NF,LONG,PROT,BMI1,APX,X86_64 */
        /* 242(f2 fc 08) 300 001(f5) 110 : SM0,SM1,SM2,NF,NF_E,LONG,EVEX,PROT,BMI1,APX,X86_64 */
        {I_BZHI, 3, {REG_GPR|BITS64,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24592, 136, 1152},
    /* 940 : PDEP reg32,reg32*,rm32 [rvm:      vex.nds.lz.f2.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE */
        /* 261(002 003) 001(f5) 120 : SM0,SM1,SM2,NOAPX,VEX,BMI2,FUTURE */
        {I_PDEP, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55691, 141, 1153},
    /* 941 : PDEP reg32,reg32*,rm32 [rvm:      evex.nds.lz.f2.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE,APX */
        /* 241(f2 7f 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_PDEP, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24610, 142, 1153},
    /* 942 : PDEP reg64,reg64*,rm64 [rvm:      vex.nds.lz.f2.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64 */
        /* 261(002 203) 001(f5) 120 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_PDEP, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55705, 144, 1154},
    /* 943 : PDEP reg64,reg64*,rm64 [rvm:      evex.nds.lz.f2.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 ff 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_PDEP, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24628, 145, 1154},
    /* 944 : PEXT reg32,reg32*,rm32 [rvm:      vex.nds.lz.f3.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE */
        /* 261(002 002) 001(f5) 120 : SM0,SM1,SM2,NOAPX,VEX,BMI2,FUTURE */
        {I_PEXT, 3, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55719, 141, 1155},
    /* 945 : PEXT reg32,reg32*,rm32 [rvm:      evex.nds.lz.f3.0f38.w0 f5 /r      ] SM0,SM1,SM2,BMI2,FUTURE,APX */
        /* 241(f2 7e 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,FUTURE */
        {I_PEXT, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+24646, 142, 1155},
    /* 946 : PEXT reg64,reg64*,rm64 [rvm:      vex.nds.lz.f3.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64 */
        /* 261(002 202) 001(f5) 120 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,BMI2,X86_64 */
        {I_PEXT, 3, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+55733, 144, 1156},
    /* 947 : PEXT reg64,reg64*,rm64 [rvm:      evex.nds.lz.f3.0f38.w1 f5 /r      ] SM0,SM1,SM2,LONG,PROT,BMI2,X86_64,APX */
        /* 241(f2 fe 08) 300 001(f5) 120 : SM0,SM1,SM2,LONG,EVEX,PROT,BMI2,APX,X86_64 */
        {I_PEXT, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+24664, 145, 1156},
    /* 948 : BEXTR reg32,rm32*,imm32 [rmi:      xop.m10.lz.w0 10 /r id            ] SM0,SM1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 270(112 000) 001(10) 110 042 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BEXTR, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+51291, 147, 1159},
    /* 949 : BEXTR reg64,rm64*,imm32 [rmi:      xop.m10.lz.w1 10 /r id            ] SM0,SM1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 270(112 200) 001(10) 110 042 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BEXTR, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+51307, 149, 1160},
    /* 950 : BLCI reg32,rm32 [vm:       xop.ndd.lz.m9.w0 02 /6            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(02) 216 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCI, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55747, 147, 1161},
    /* 951 : BLCI reg64,rm64 [vm:       xop.ndd.lz.m9.w1 02 /6            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(02) 216 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCI, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55754, 149, 1162},
    /* 952 : BLCIC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /5            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 215 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCIC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55761, 147, 1163},
    /* 953 : BLCIC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /5            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 215 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCIC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55768, 149, 1164},
    /* 954 : BLSIC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /6            ] SM0-1,FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 216 : SM0,SM1,NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLSIC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55775, 147, 1165},
    /* 955 : BLSIC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /6            ] SM0-1,LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 216 : SM0,SM1,NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLSIC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55782, 149, 1166},
    /* 956 : BLCFILL reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /1            ] SM0-1,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 211 : SM0,SM1,NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCFILL, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55789, 151, 1167},
    /* 957 : BLCFILL reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /1            ] SM0-1,LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 211 : SM0,SM1,NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCFILL, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55796, 152, 1168},
    /* 958 : BLSFILL reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /2            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 212 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLSFILL, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55803, 153, 1169},
    /* 959 : BLSFILL reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /2            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 212 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLSFILL, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55810, 154, 1170},
    /* 960 : BLCMSK reg32,rm32 [vm:       xop.ndd.lz.m9.w0 02 /1            ] FL,TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(02) 211 : NOAPX,FL,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55817, 155, 1171},
    /* 961 : BLCMSK reg64,rm64 [vm:       xop.ndd.lz.m9.w1 02 /1            ] LONG,FL,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(02) 211 : NOAPX,LONG,FL,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55824, 156, 1172},
    /* 962 : BLCS reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /3            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 213 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_BLCS, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55831, 153, 1173},
    /* 963 : BLCS reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /3            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 213 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_BLCS, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55838, 154, 1174},
    /* 964 : TZMSK reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /4            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 214 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_TZMSK, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55845, 153, 1175},
    /* 965 : TZMSK reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /4            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 214 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_TZMSK, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55852, 154, 1176},
    /* 966 : T1MSKC reg32,rm32 [vm:       xop.ndd.lz.m9.w0 01 /7            ] TBM,OBSOLETE,FUTURE,AMD */
        /* 260(111 000) 001(01) 217 : NOAPX,VEX,TBM,OBSOLETE,FUTURE,AMD */
        {I_T1MSKC, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55859, 153, 1177},
    /* 967 : T1MSKC reg64,rm64 [vm:       xop.ndd.lz.m9.w1 01 /7            ] LONG,PROT,TBM,OBSOLETE,X86_64,AMD */
        /* 260(111 200) 001(01) 217 : NOAPX,LONG,VEX,PROT,TBM,OBSOLETE,X86_64,AMD */
        {I_T1MSKC, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55866, 154, 1178},
    /* 968 : AAA void [          37                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(37) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73445, 74, 1181},
    /* 969 : AAD void [          d5 0a                             ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 002(d5 0a) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72656, 74, 1182},
    /* 970 : AAD imm8 [i:        d5 ib,u                           ] AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(d5) 024 : AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAD, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72660, 157, 1183},
    /* 971 : AAM void [          d4 0a                             ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 002(d4 0a) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72664, 74, 1184},
    /* 972 : AAM imm8 [i:        d4 ib,u                           ] AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(d4) 024 : AR0,SB,NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAM, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72668, 157, 1185},
    /* 973 : AAS void [          3f                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(3f) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_AAS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73448, 74, 1186},
    /* 974 : DAA void [          27                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(27) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DAA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73451, 74, 1187},
    /* 975 : DAS void [          2f                                ] NOREX,NOAPX,NOLONG,FL,8086 */
        /* 001(2f) : NOREX,NOAPX,NOLONG,FL,8086 */
        {I_DAS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73454, 74, 1188},
    /* 976 : BSWAP reg32 [r:        o32 0f c8+r                       ] 486 */
        /* 321 355 010 310 : 486 */
        {I_BSWAP, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71634, 158, 1191},
    /* 977 : BSWAP reg64 [r:        o64 0f c8+r                       ] LONG,PROT,X86_64 */
        /* 324 355 010 310 : LONG,PROT,X86_64 */
        {I_BSWAP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71639, 24, 1192},
    /* 978 : MOVBE reg16,mem16 [rm:       o16 norep 0f38 f0 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 320 331 356 001(f0) 110 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55873, 160, 1197},
    /* 979 : MOVBE reg32,mem32 [rm:       o32 norep 0f38 f0 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 321 331 356 001(f0) 110 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55880, 160, 1198},
    /* 980 : MOVBE reg64,mem64 [rm:       o64 norep 0f38 f0 /r              ] SM0-1,LONG,PROT,MOVBE,X86_64,NEHALEM */
        /* 324 331 356 001(f0) 110 : SM0,SM1,NOAPX,LONG,PROT,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55887, 161, 1199},
    /* 981 : MOVBE mem16,reg16 [mr:       o16 norep 0f38 f1 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 320 331 356 001(f1) 101 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS16|RN_L16,REG_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55894, 160, 1200},
    /* 982 : MOVBE mem32,reg32 [mr:       o32 norep 0f38 f1 /r              ] SM0-1,MOVBE,NEHALEM */
        /* 321 331 356 001(f1) 101 : SM0,SM1,NOAPX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55901, 160, 1201},
    /* 983 : MOVBE mem64,reg64 [mr:       o64 norep 0f38 f1 /r              ] SM0-1,LONG,PROT,MOVBE,X86_64,NEHALEM */
        /* 324 331 356 001(f1) 101 : SM0,SM1,NOAPX,LONG,PROT,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55908, 161, 1202},
    /* 984 : MOVBE reg16,mem16 [rm:       evex.nd0.nf0.l0.m4.o16 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(60) 110 : SM0,SM1,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12963, 162, 1203},
    /* 985 : MOVBE reg32,mem32 [rm:       evex.nd0.nf0.l0.m4.o32 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(60) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+12973, 163, 1204},
    /* 986 : MOVBE reg64,mem64 [rm:       evex.nd0.nf0.l0.m4.o64 60 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(60) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+12983, 163, 1205},
    /* 987 : MOVBE mem16,reg16 [mr:       evex.nd0.nf0.l0.m4.o16 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(61) 101 : SM0,SM1,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+12993, 162, 1206},
    /* 988 : MOVBE mem32,reg32 [mr:       evex.nd0.nf0.l0.m4.o32 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(61) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13003, 163, 1207},
    /* 989 : MOVBE mem64,reg64 [mr:       evex.nd0.nf0.l0.m4.o64 61 /r      ] SM0-1,LONG,PROT,APX,MOVBE,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(61) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,MOVBE,NEHALEM */
        {I_MOVBE, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13013, 163, 1208},
    /* 990 : CBW void [          o16 98                            ] 8086 */
        /* 320 001(98) : 8086 */
        {I_CBW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72688, 23, 1211},
    /* 991 : CDQ void [          o32 99                            ] 386 */
        /* 321 001(99) : 386 */
        {I_CDQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72692, 3, 1212},
    /* 992 : CDQE void [          o64 98                            ] LONG,PROT,X86_64 */
        /* 324 001(98) : LONG,PROT,X86_64 */
        {I_CDQE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72696, 24, 1213},
    /* 993 : CQO void [          o64 99                            ] LONG,PROT,X86_64 */
        /* 324 001(99) : LONG,PROT,X86_64 */
        {I_CQO, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72700, 24, 1214},
    /* 994 : CWD void [          o16 99                            ] 8086 */
        /* 320 001(99) : 8086 */
        {I_CWD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72704, 23, 1215},
    /* 995 : CWDE void [          o32 98                            ] 386 */
        /* 321 001(98) : 386 */
        {I_CWDE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72708, 3, 1216},
    /* 996 : MOVSX reg16,rm8 [rm:       o16 0f be /r                      ] AR0-1,SX,386 */
        /* 320 355 001(be) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS16,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66915, 167, 1223},
    /* 997 : MOVSX reg32,rm8 [rm:       o32 0f be /r                      ] AR0-1,SX,386 */
        /* 321 355 001(be) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66921, 167, 1224},
    /* 998 : MOVSX reg64,rm8 [rm:       o64 0f be /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(be) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66927, 168, 1225},
    /* 999 : MOVSX reg16,rm16 [rm:       o16 0f bf /r                      ] AR0-1,SX,386 */
        /* 320 355 001(bf) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66933, 167, 1229},
    /* 1000 : MOVSX reg32,rm16 [rm:       o32 0f bf /r                      ] AR0-1,SX,386 */
        /* 321 355 001(bf) 110 : AR0,AR1,SX,386 */
        {I_MOVSX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66939, 167, 1230},
    /* 1001 : MOVSX reg64,rm16 [rm:       o64 0f bf /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(bf) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVSX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66945, 168, 1231},
    /* 1002 : MOVSXD reg16,rm32 [rm:       o16 63 /r                         ] LONG,PROT,X86_64 */
        /* 320 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS16,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71644, 24, 1238},
    /* 1003 : MOVSXD reg32,rm32 [rm:       o32 63 /r                         ] LONG,PROT,X86_64 */
        /* 321 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71649, 24, 1239},
    /* 1004 : MOVSXD reg64,rm32 [rm:       o64 63 /r                         ] LONG,PROT,X86_64 */
        /* 324 001(63) 110 : LONG,PROT,X86_64 */
        {I_MOVSXD, 2, {REG_GPR|BITS64,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+71654, 24, 1240},
    /* 1005 : MOVZX reg16,rm8 [rm:       o16 0f b6 /r                      ] AR0-1,SX,386 */
        /* 320 355 001(b6) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS16,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66951, 167, 1242},
    /* 1006 : MOVZX reg32,rm8 [rm:       o32 0f b6 /r                      ] AR0-1,SX,386 */
        /* 321 355 001(b6) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66957, 167, 1243},
    /* 1007 : MOVZX reg64,rm8 [rm:       o64 0f b6 /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(b6) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+66963, 168, 1244},
    /* 1008 : MOVZX reg16,rm16 [rm:       o16 0f b7 /r                      ] AR0-1,SX,386 */
        /* 320 355 001(b7) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66969, 167, 1248},
    /* 1009 : MOVZX reg32,rm16 [rm:       o32 0f b7 /r                      ] AR0-1,SX,386 */
        /* 321 355 001(b7) 110 : AR0,AR1,SX,386 */
        {I_MOVZX, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66975, 167, 1249},
    /* 1010 : MOVZX reg64,rm16 [rm:       o64 0f b7 /r                      ] AR0-1,SX,LONG,PROT,X86_64 */
        /* 324 355 001(b7) 110 : AR0,AR1,SX,LONG,PROT,X86_64 */
        {I_MOVZX, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66981, 168, 1250},
    /* 1011 : CMPXCHG rm8,reg8 [mr:       hle o8 0f b0 /r                   ] SM0-1,LOCK,FL,PENT */
        /* 273 355 001(b0) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+67005, 175, 1262},
    /* 1012 : CMPXCHG rm16,reg16 [mr:       hle o16 0f b1 /r                  ] SM0-1,LOCK,FL,PENT */
        /* 273 320 355 001(b1) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55915, 175, 1263},
    /* 1013 : CMPXCHG rm32,reg32 [mr:       hle o32 0f b1 /r                  ] SM0-1,LOCK,FL,PENT */
        /* 273 321 355 001(b1) 101 : SM0,SM1,LOCK,FL,PENT */
        {I_CMPXCHG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55922, 175, 1264},
    /* 1014 : CMPXCHG rm64,reg64 [mr:       hle o64 0f b1 /r                  ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 355 001(b1) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_CMPXCHG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55929, 27, 1265},
    /* 1015 : CMPXCHG8B mem64 [m:        hle norexw 0f c7 /1               ] LOCK,FL,PENT */
        /* 273 317 355 001(c7) 201 : LOCK,FL,PENT */
        {I_CMPXCHG8B, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55936, 176, 1266},
    /* 1016 : CMPXCHG16B mem128 [m:        o64 0f c7 /1                      ] LOCK,LONG,FL,PROT,X86_64 */
        /* 324 355 001(c7) 201 : LOCK,LONG,FL,PROT,X86_64 */
        {I_CMPXCHG16B, 1, {MEMORY|BITS128,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67011, 78, 1267},
    /* 1017 : XADD rm8,reg8 [mr:       hle o8 0f c0 /r                   ] SM0-1,LOCK,FL,486 */
        /* 273 355 001(c0) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS8,REG_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+67017, 177, 1269},
    /* 1018 : XADD rm16,reg16 [mr:       hle o16 0f c1 /r                  ] SM0-1,LOCK,FL,486 */
        /* 273 320 355 001(c1) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+55943, 177, 1270},
    /* 1019 : XADD rm32,reg32 [mr:       hle o32 0f c1 /r                  ] SM0-1,LOCK,FL,486 */
        /* 273 321 355 001(c1) 101 : SM0,SM1,LOCK,FL,486 */
        {I_XADD, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+55950, 177, 1271},
    /* 1020 : XADD rm64,reg64 [mr:       hle o64 0f c1 /r                  ] SM0-1,LOCK,LONG,FL,PROT,X86_64 */
        /* 273 324 355 001(c1) 101 : SM0,SM1,LOCK,LONG,FL,PROT,X86_64 */
        {I_XADD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+55957, 27, 1272},
    /* 1021 : XCHG reg_ax,reg16 [-r:       o16 90+r                          ] 8086 */
        /* 320 011 220 : 8086 */
        {I_XCHG, 2, {REG_AX,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72712, 23, 1274},
    /* 1022 : XCHG reg_rax,reg64 [-r:       o64 90+r                          ] ZU,LONG,PROT,X86_64 */
        /* 324 011 220 : ZU,LONG,PROT,X86_64 */
        {I_XCHG, 2, {REG_RAX,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+72716, 178, 1275},
    /* 1023 : XCHG reg_eax,reg32na [-r:       o32 90+r                          ] ZU,386 */
        /* 321 011 220 : ZU,386 */
        {I_XCHG, 2, {REG_EAX,REG_NA|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+72728, 179, 1279},
    /* 1024 : XCHG reg8,rm8 [rm:       hlenl o8 86 /r                    ] SM0-1,LOCK1,8086 */
        /* 272 001(86) 110 : SM0,SM1,LOCK1,8086 */
        {I_XCHG, 2, {REG_GPR|BITS8,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71659, 181, 1282},
    /* 1025 : XCHG reg16,rm16 [rm:       hlenl o16 87 /r                   ] SM0-1,LOCK1,8086 */
        /* 272 320 001(87) 110 : SM0,SM1,LOCK1,8086 */
        {I_XCHG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67023, 181, 1283},
    /* 1026 : XCHG reg32,rm32 [rm:       hlenl o32 87 /r                   ] SM0-1,LOCK1,386 */
        /* 272 321 001(87) 110 : SM0,SM1,LOCK1,386 */
        {I_XCHG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67029, 182, 1284},
    /* 1027 : XCHG reg64,rm64 [rm:       hlenl o64 87 /r                   ] SM0-1,LOCK1,LONG,PROT,X86_64 */
        /* 272 324 001(87) 110 : SM0,SM1,LOCK1,LONG,PROT,X86_64 */
        {I_XCHG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67035, 183, 1285},
    /* 1028 : JMPABS imm64|near [i:        a64 np rex2 a1 iq                 ] LONG,PROT,APX,X86_64 */
        /* 313 360 350 001(a1) 054 : LONG,REX2,PROT,APX,X86_64 */
        {I_JMPABS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55964, 188, 1298},
    /* 1029 : JMP imm16|near [i:        jmp8 nw o16 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        /* 371 327 320 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55971, 191, 1306},
    /* 1030 : JMP imm32|near [i:        jmp8 nw o32 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        /* 371 327 321 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55978, 191, 1307},
    /* 1031 : JMP imm64|near [i:        jmp8 nw o64 eb rel8               ] AR0,OSIZE,JMP_RELAX,NOAPX,LONG,PROT,X86_64 */
        /* 371 327 324 001(eb) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,NOAPX,LONG,PROT,X86_64 */
        {I_JMP, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55985, 192, 1308},
    /* 1032 : JMP imm16|near [i:        nw o16 e9 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(e9) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67059, 194, 1310},
    /* 1033 : JMP imm32|near [i:        nw o32 e9 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 321 001(e9) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67065, 194, 1311},
    /* 1034 : JMP imm64|near [i:        nw o64 e9 rel                     ] AR0,OSIZE,LONG,BND,PROT,X86_64 */
        /* 327 324 001(e9) 064 : AR0,NWSIZE,OSIZE,LONG,BND,PROT,X86_64 */
        {I_JMP, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67071, 195, 1312},
    /* 1035 : JMP rm16|near [m:        nw o16 ff /4                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(ff) 204 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {RM_GPR|BITS16|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67077, 194, 1314},
    /* 1036 : JMP rm32|near [m:        nw o32 ff /4                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(ff) 204 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JMP, 1, {RM_GPR|BITS32|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67083, 197, 1315},
    /* 1037 : JMP rm64|near [m:        nw o64 ff /4                      ] AR0,OSIZE,LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 204 : AR0,NWSIZE,OSIZE,LONG,BND,PROT,X86_64 */
        {I_JMP, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67089, 195, 1316},
    /* 1038 : JMP imm16:imm16 [ji:       o16 ea iw iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ea) 031 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67107, 202, 1326},
    /* 1039 : JMP imm16:imm32 [ji:       o32 ea id iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 321 001(ea) 041 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JMP, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67113, 203, 1327},
    /* 1040 : JMP mem16|far [m:        o16 ff /5                         ] AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ff) 205 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JMP, 1, {MEMORY|BITS16|RN_L16|FAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71669, 189, 1332},
    /* 1041 : JMP mem32|far [m:        o32 ff /5                         ] AR0,NWSIZE,OSIZE,386 */
        /* 321 001(ff) 205 : AR0,NWSIZE,OSIZE,386 */
        {I_JMP, 1, {MEMORY|BITS32|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71674, 204, 1333},
    /* 1042 : JMP mem64|far [m:        o64 ff /5                         ] AR0,NWSIZE,OSIZE,LONG,386,X86_64 */
        /* 324 001(ff) 205 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_JMP, 1, {MEMORY|BITS64|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71679, 205, 1334},
    /* 1043 : JMP rm16 [m:        nw o16 ff /4                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(ff) 204 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JMP, 1, {RM_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67077, 194, 1337},
    /* 1044 : JMP rm32 [m:        nw o32 ff /4                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(ff) 204 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JMP, 1, {RM_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67083, 197, 1338},
    /* 1045 : JMP rm64 [m:        nw o64 ff /4                      ] AR0,OSIZE,LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 204 : AR0,NWSIZE,OSIZE,LONG,BND,PROT,X86_64 */
        {I_JMP, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67089, 195, 1339},
    /* 1046 : JA imm16|near [i:        jcc8 nw o16 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55992, 208, 1344},
    /* 1047 : JC imm16|near [i:        jcc8 nw o16 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56006, 208, 1344},
    /* 1048 : JG imm16|near [i:        jcc8 nw o16 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56027, 208, 1344},
    /* 1049 : JL imm16|near [i:        jcc8 nw o16 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56041, 208, 1344},
    /* 1050 : JNA imm16|near [i:        jcc8 nw o16 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56013, 208, 1344},
    /* 1051 : JNC imm16|near [i:        jcc8 nw o16 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+55999, 208, 1344},
    /* 1052 : JNG imm16|near [i:        jcc8 nw o16 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56048, 208, 1344},
    /* 1053 : JNL imm16|near [i:        jcc8 nw o16 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56034, 208, 1344},
    /* 1054 : JNO imm16|near [i:        jcc8 nw o16 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56062, 208, 1344},
    /* 1055 : JNS imm16|near [i:        jcc8 nw o16 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56076, 208, 1344},
    /* 1056 : JNZ imm16|near [i:        jcc8 nw o16 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56055, 208, 1344},
    /* 1057 : JO imm16|near [i:        jcc8 nw o16 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56083, 208, 1344},
    /* 1058 : JPE imm16|near [i:        jcc8 nw o16 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56090, 208, 1344},
    /* 1059 : JPO imm16|near [i:        jcc8 nw o16 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56069, 208, 1344},
    /* 1060 : JS imm16|near [i:        jcc8 nw o16 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56097, 208, 1344},
    /* 1061 : JZ imm16|near [i:        jcc8 nw o16 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 320 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56020, 208, 1344},
    /* 1062 : JA imm32|near [i:        jcc8 nw o32 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56104, 208, 1345},
    /* 1063 : JC imm32|near [i:        jcc8 nw o32 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56118, 208, 1345},
    /* 1064 : JG imm32|near [i:        jcc8 nw o32 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56139, 208, 1345},
    /* 1065 : JL imm32|near [i:        jcc8 nw o32 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56153, 208, 1345},
    /* 1066 : JNA imm32|near [i:        jcc8 nw o32 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56125, 208, 1345},
    /* 1067 : JNC imm32|near [i:        jcc8 nw o32 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56111, 208, 1345},
    /* 1068 : JNG imm32|near [i:        jcc8 nw o32 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56160, 208, 1345},
    /* 1069 : JNL imm32|near [i:        jcc8 nw o32 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56146, 208, 1345},
    /* 1070 : JNO imm32|near [i:        jcc8 nw o32 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56174, 208, 1345},
    /* 1071 : JNS imm32|near [i:        jcc8 nw o32 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56188, 208, 1345},
    /* 1072 : JNZ imm32|near [i:        jcc8 nw o32 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JNZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56167, 208, 1345},
    /* 1073 : JO imm32|near [i:        jcc8 nw o32 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56195, 208, 1345},
    /* 1074 : JPE imm32|near [i:        jcc8 nw o32 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56202, 208, 1345},
    /* 1075 : JPO imm32|near [i:        jcc8 nw o32 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JPO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56181, 208, 1345},
    /* 1076 : JS imm32|near [i:        jcc8 nw o32 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56209, 208, 1345},
    /* 1077 : JZ imm32|near [i:        jcc8 nw o32 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 370 327 321 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_JZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56132, 208, 1345},
    /* 1078 : JA imm64|near [i:        jcc8 nw o64 77 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(77) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56216, 209, 1346},
    /* 1079 : JC imm64|near [i:        jcc8 nw o64 72 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(72) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56230, 209, 1346},
    /* 1080 : JG imm64|near [i:        jcc8 nw o64 7f rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7f) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56251, 209, 1346},
    /* 1081 : JL imm64|near [i:        jcc8 nw o64 7c rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7c) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56265, 209, 1346},
    /* 1082 : JNA imm64|near [i:        jcc8 nw o64 76 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(76) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56237, 209, 1346},
    /* 1083 : JNC imm64|near [i:        jcc8 nw o64 73 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(73) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56223, 209, 1346},
    /* 1084 : JNG imm64|near [i:        jcc8 nw o64 7e rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7e) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56272, 209, 1346},
    /* 1085 : JNL imm64|near [i:        jcc8 nw o64 7d rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7d) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56258, 209, 1346},
    /* 1086 : JNO imm64|near [i:        jcc8 nw o64 71 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(71) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56286, 209, 1346},
    /* 1087 : JNS imm64|near [i:        jcc8 nw o64 79 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(79) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56300, 209, 1346},
    /* 1088 : JNZ imm64|near [i:        jcc8 nw o64 75 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(75) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56279, 209, 1346},
    /* 1089 : JO imm64|near [i:        jcc8 nw o64 70 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(70) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56307, 209, 1346},
    /* 1090 : JPE imm64|near [i:        jcc8 nw o64 7a rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7a) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPE, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56314, 209, 1346},
    /* 1091 : JPO imm64|near [i:        jcc8 nw o64 7b rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(7b) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56293, 209, 1346},
    /* 1092 : JS imm64|near [i:        jcc8 nw o64 78 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(78) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56321, 209, 1346},
    /* 1093 : JZ imm64|near [i:        jcc8 nw o64 74 rel8             ] AR0,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 370 327 324 001(74) 050 : AR0,NWSIZE,OSIZE,JMP_RELAX,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56244, 209, 1346},
    /* 1094 : JA imm16|near [i:        nw o16 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56328, 211, 1348},
    /* 1095 : JC imm16|near [i:        nw o16 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56342, 211, 1348},
    /* 1096 : JG imm16|near [i:        nw o16 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56363, 211, 1348},
    /* 1097 : JL imm16|near [i:        nw o16 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56377, 211, 1348},
    /* 1098 : JNA imm16|near [i:        nw o16 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNA, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56349, 211, 1348},
    /* 1099 : JNC imm16|near [i:        nw o16 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNC, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56335, 211, 1348},
    /* 1100 : JNG imm16|near [i:        nw o16 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNG, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56384, 211, 1348},
    /* 1101 : JNL imm16|near [i:        nw o16 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56370, 211, 1348},
    /* 1102 : JNO imm16|near [i:        nw o16 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56398, 211, 1348},
    /* 1103 : JNS imm16|near [i:        nw o16 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56412, 211, 1348},
    /* 1104 : JNZ imm16|near [i:        nw o16 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56391, 211, 1348},
    /* 1105 : JO imm16|near [i:        nw o16 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56419, 211, 1348},
    /* 1106 : JPE imm16|near [i:        nw o16 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56426, 211, 1348},
    /* 1107 : JPO imm16|near [i:        nw o16 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPO, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56405, 211, 1348},
    /* 1108 : JS imm16|near [i:        nw o16 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JS, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56433, 211, 1348},
    /* 1109 : JZ imm16|near [i:        nw o16 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 320 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JZ, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56356, 211, 1348},
    /* 1110 : JA imm32|near [i:        nw o32 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56440, 211, 1349},
    /* 1111 : JC imm32|near [i:        nw o32 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56454, 211, 1349},
    /* 1112 : JG imm32|near [i:        nw o32 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56475, 211, 1349},
    /* 1113 : JL imm32|near [i:        nw o32 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56489, 211, 1349},
    /* 1114 : JNA imm32|near [i:        nw o32 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNA, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56461, 211, 1349},
    /* 1115 : JNC imm32|near [i:        nw o32 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNC, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56447, 211, 1349},
    /* 1116 : JNG imm32|near [i:        nw o32 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNG, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56496, 211, 1349},
    /* 1117 : JNL imm32|near [i:        nw o32 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56482, 211, 1349},
    /* 1118 : JNO imm32|near [i:        nw o32 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56510, 211, 1349},
    /* 1119 : JNS imm32|near [i:        nw o32 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56524, 211, 1349},
    /* 1120 : JNZ imm32|near [i:        nw o32 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JNZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56503, 211, 1349},
    /* 1121 : JO imm32|near [i:        nw o32 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56531, 211, 1349},
    /* 1122 : JPE imm32|near [i:        nw o32 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56538, 211, 1349},
    /* 1123 : JPO imm32|near [i:        nw o32 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JPO, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56517, 211, 1349},
    /* 1124 : JS imm32|near [i:        nw o32 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JS, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56545, 211, 1349},
    /* 1125 : JZ imm32|near [i:        nw o32 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOREX,NOAPX,NOLONG,BND,386 */
        {I_JZ, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56468, 211, 1349},
    /* 1126 : JA imm64|near [i:        nw o64 0f 87 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(87) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56552, 207, 1350},
    /* 1127 : JC imm64|near [i:        nw o64 0f 82 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(82) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56566, 207, 1350},
    /* 1128 : JG imm64|near [i:        nw o64 0f 8f rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8f) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56587, 207, 1350},
    /* 1129 : JL imm64|near [i:        nw o64 0f 8c rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8c) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56601, 207, 1350},
    /* 1130 : JNA imm64|near [i:        nw o64 0f 86 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(86) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNA, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56573, 207, 1350},
    /* 1131 : JNC imm64|near [i:        nw o64 0f 83 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(83) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNC, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56559, 207, 1350},
    /* 1132 : JNG imm64|near [i:        nw o64 0f 8e rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8e) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNG, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56608, 207, 1350},
    /* 1133 : JNL imm64|near [i:        nw o64 0f 8d rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8d) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56594, 207, 1350},
    /* 1134 : JNO imm64|near [i:        nw o64 0f 81 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(81) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56622, 207, 1350},
    /* 1135 : JNS imm64|near [i:        nw o64 0f 89 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(89) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56636, 207, 1350},
    /* 1136 : JNZ imm64|near [i:        nw o64 0f 85 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(85) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JNZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56615, 207, 1350},
    /* 1137 : JO imm64|near [i:        nw o64 0f 80 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(80) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56643, 207, 1350},
    /* 1138 : JPE imm64|near [i:        nw o64 0f 8a rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8a) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPE, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56650, 207, 1350},
    /* 1139 : JPO imm64|near [i:        nw o64 0f 8b rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(8b) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JPO, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56629, 207, 1350},
    /* 1140 : JS imm64|near [i:        nw o64 0f 88 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(88) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JS, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56657, 207, 1350},
    /* 1141 : JZ imm64|near [i:        nw o64 0f 84 rel                ] AR0,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 355 001(84) 064 : AR0,NWSIZE,OSIZE,JCC_HINT,NOAPX,LONG,BND,PROT,X86_64 */
        {I_JZ, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56580, 207, 1350},
    /* 1142 : JCXZ imm16|near|short [i:        a16 nw o16 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JCXZ, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56776, 189, 1358},
    /* 1143 : JECXZ imm16|near|short [i:        a32 nw o16 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56783, 214, 1359},
    /* 1144 : JCXZ imm32|near|short [i:        a16 nw o32 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_JCXZ, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56790, 189, 1360},
    /* 1145 : JECXZ imm32|near|short [i:        a32 nw o32 e3 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e3) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56797, 214, 1361},
    /* 1146 : JECXZ imm64|near|short [i:        a32 nw o64 e3 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e3) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_JECXZ, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56804, 190, 1362},
    /* 1147 : JRCXZ imm64|near|short [i:        a64 nw o64 e3 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e3) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_JRCXZ, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56811, 190, 1363},
    /* 1148 : LOOP imm16|near|short [i:        asm nw o16 e2 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e2) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56818, 189, 1371},
    /* 1149 : LOOP imm32|near|short [i:        asm nw o32 e2 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e2) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56839, 189, 1374},
    /* 1150 : LOOP imm64|near|short [i:        asm nw o64 e2 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e2) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56860, 190, 1377},
    /* 1151 : LOOPE imm16|near|short [i:        asm nw o16 e1 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e1) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56881, 189, 1380},
    /* 1152 : LOOPE imm32|near|short [i:        asm nw o32 e1 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e1) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56902, 189, 1383},
    /* 1153 : LOOPE imm64|near|short [i:        asm nw o64 e1 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e1) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56923, 190, 1386},
    /* 1154 : LOOPNE imm16|near|short [i:        asm nw o16 e0 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 320 001(e0) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS16|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56944, 189, 1389},
    /* 1155 : LOOPNE imm32|near|short [i:        asm nw o32 e0 rel8                ] AR0,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 312 327 321 001(e0) 050 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS32|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56965, 189, 1392},
    /* 1156 : LOOPNE imm64|near|short [i:        asm nw o64 e0 rel8                ] AR0,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 312 327 324 001(e0) 050 : AR0,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 1, {IMM_NORMAL|BITS64|NEAR|SHORT,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+56986, 190, 1395},
    /* 1157 : LOOP imm16|near|short,reg_cx [i-:       a16 nw o16 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56825, 215, 1416},
    /* 1158 : LOOP imm16|near|short,reg_ecx [i-:       a32 nw o16 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOP, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56832, 216, 1417},
    /* 1159 : LOOP imm32|near|short,reg_cx [i-:       a16 nw o32 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOP, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56846, 215, 1418},
    /* 1160 : LOOP imm32|near|short,reg_ecx [i-:       a32 nw o32 e2 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOP, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56853, 216, 1419},
    /* 1161 : LOOP imm64|near|short,reg_ecx [i-:       a32 nw o64 e2 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56867, 217, 1420},
    /* 1162 : LOOP imm64|near|short,reg_rcx [i-:       a64 nw o64 e2 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e2) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOP, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56874, 217, 1421},
    /* 1163 : LOOPE imm16|near|short,reg_cx [i-:       a16 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56888, 215, 1422},
    /* 1164 : LOOPE imm16|near|short,reg_ecx [i-:       a32 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56895, 216, 1423},
    /* 1165 : LOOPE imm32|near|short,reg_cx [i-:       a16 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56909, 215, 1424},
    /* 1166 : LOOPE imm32|near|short,reg_ecx [i-:       a32 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56916, 216, 1425},
    /* 1167 : LOOPE imm64|near|short,reg_ecx [i-:       a32 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56930, 217, 1426},
    /* 1168 : LOOPE imm64|near|short,reg_rcx [i-:       a64 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56937, 217, 1427},
    /* 1169 : LOOPNE imm16|near|short,reg_cx [i-:       a16 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56951, 215, 1428},
    /* 1170 : LOOPNE imm16|near|short,reg_ecx [i-:       a32 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56958, 216, 1429},
    /* 1171 : LOOPNE imm32|near|short,reg_cx [i-:       a16 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56972, 215, 1430},
    /* 1172 : LOOPNE imm32|near|short,reg_ecx [i-:       a32 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56979, 216, 1431},
    /* 1173 : LOOPNE imm64|near|short,reg_ecx [i-:       a32 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56993, 217, 1432},
    /* 1174 : LOOPNE imm64|near|short,reg_rcx [i-:       a64 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNE, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+57000, 217, 1433},
    /* 1175 : LOOPZ imm16|near|short,reg_cx [i-:       a16 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56888, 215, 1434},
    /* 1176 : LOOPZ imm16|near|short,reg_ecx [i-:       a32 nw o16 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56895, 216, 1435},
    /* 1177 : LOOPZ imm32|near|short,reg_cx [i-:       a16 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56909, 215, 1436},
    /* 1178 : LOOPZ imm32|near|short,reg_ecx [i-:       a32 nw o32 e1 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56916, 216, 1437},
    /* 1179 : LOOPZ imm64|near|short,reg_ecx [i-:       a32 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56930, 217, 1438},
    /* 1180 : LOOPZ imm64|near|short,reg_rcx [i-:       a64 nw o64 e1 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e1) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56937, 217, 1439},
    /* 1181 : LOOPNZ imm16|near|short,reg_cx [i-:       a16 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56951, 215, 1440},
    /* 1182 : LOOPNZ imm16|near|short,reg_ecx [i-:       a32 nw o16 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 320 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS16|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56958, 216, 1441},
    /* 1183 : LOOPNZ imm32|near|short,reg_cx [i-:       a16 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 310 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_CX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56972, 215, 1442},
    /* 1184 : LOOPNZ imm32|near|short,reg_ecx [i-:       a32 nw o32 e0 rel8                ] AR0-1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 311 327 321 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS32|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56979, 216, 1443},
    /* 1185 : LOOPNZ imm64|near|short,reg_ecx [i-:       a32 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 311 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+56993, 217, 1444},
    /* 1186 : LOOPNZ imm64|near|short,reg_rcx [i-:       a64 nw o64 e0 rel8                ] AR0-1,OSIZE,NOAPX,LONG,PROT,X86_64 */
        /* 313 327 324 001(e0) 050 : AR0,AR1,NWSIZE,OSIZE,NOAPX,LONG,PROT,X86_64 */
        {I_LOOPNZ, 2, {IMM_NORMAL|BITS64|NEAR|SHORT,REG_RCX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+57000, 217, 1445},
    /* 1187 : JMPE imm16|near [i:        norep o16 0f b8 iw                ] AR0,OSIZE,IA64 */
        /* 331 320 355 001(b8) 030 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57007, 218, 1456},
    /* 1188 : JMPE imm32|near [i:        norep o32 0f b8 id                ] AR0,OSIZE,IA64 */
        /* 331 321 355 001(b8) 040 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57014, 218, 1457},
    /* 1189 : JMPE sdword64|near [i:        norep o64 0f b8 iq                ] AR0,OSIZE,LONG,IA64,X86_64 */
        /* 331 324 355 001(b8) 054 : AR0,OSIZE,LONG,PROT,X86_64 */
        {I_JMPE, 1, {SDWORD|BITS64|IMM_NORMAL|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57021, 219, 1458},
    /* 1190 : JMPE rm16|near [m:        norep o16 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 331 320 355 001(00) 206 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57028, 218, 1459},
    /* 1191 : JMPE rm32|near [m:        norep o32 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 331 321 355 001(00) 206 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57035, 218, 1460},
    /* 1192 : JMPE rm64|near [m:        norep o64 0f 00 /6                ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 331 324 355 001(00) 206 : AR0,OSIZE,LONG,PROT,X86_64 */
        {I_JMPE, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57042, 219, 1461},
    /* 1193 : JMPE rm16 [m:        norep o16 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 331 320 355 001(00) 206 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57028, 218, 1462},
    /* 1194 : JMPE rm32 [m:        norep o32 0f 00 /6                ] AR0,OSIZE,IA64 */
        /* 331 321 355 001(00) 206 : AR0,OSIZE,IA64 */
        {I_JMPE, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57035, 218, 1463},
    /* 1195 : JMPE rm64 [m:        norep o64 0f 00 /6                ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 331 324 355 001(00) 206 : AR0,OSIZE,LONG,PROT,X86_64 */
        {I_JMPE, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57042, 219, 1464},
    /* 1196 : CALL imm16|near [i:        nw o16 e8 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(e8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67119, 194, 1467},
    /* 1197 : CALL imm32|near [i:        nw o32 e8 rel                     ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 321 001(e8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67125, 194, 1468},
    /* 1198 : CALL imm64|near [i:        nw o64 e8 rel                     ] AR0,OSIZE,NOAPX,LONG,BND,PROT,X86_64 */
        /* 327 324 001(e8) 064 : AR0,NWSIZE,OSIZE,NOAPX,LONG,BND,PROT,X86_64 */
        {I_CALL, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67131, 220, 1469},
    /* 1199 : CALL rm16|near [m:        nw o16 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {RM_GPR|BITS16|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67137, 194, 1471},
    /* 1200 : CALL rm32|near [m:        nw o32 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_CALL, 1, {RM_GPR|BITS32|RN_L16|NEAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67143, 197, 1472},
    /* 1201 : CALL rm64|near [m:        nw o64 ff /2                      ] LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 202 : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_CALL, 1, {RM_GPR|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67149, 222, 1473},
    /* 1202 : CALL imm16:imm16 [ji:       o16 9a iw iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(9a) 031 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_CALL, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67167, 202, 1483},
    /* 1203 : CALL imm16:imm32 [ji:       o32 9a id iw                      ] AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 321 001(9a) 041 030 : AR1,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_CALL, 2, {IMM_NORMAL|BITS16|COLON,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67173, 203, 1484},
    /* 1204 : CALL mem16|far [m:        o16 ff /3                         ] AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(ff) 203 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,8086 */
        {I_CALL, 1, {MEMORY|BITS16|RN_L16|FAR|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71684, 189, 1489},
    /* 1205 : CALL mem32|far [m:        o32 ff /3                         ] AR0,NWSIZE,OSIZE,386 */
        /* 321 001(ff) 203 : AR0,NWSIZE,OSIZE,386 */
        {I_CALL, 1, {MEMORY|BITS32|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71689, 204, 1490},
    /* 1206 : CALL mem64|far [m:        o64 ff /3                         ] AR0,NWSIZE,OSIZE,LONG,386,X86_64 */
        /* 324 001(ff) 203 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_CALL, 1, {MEMORY|BITS64|FAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71694, 205, 1491},
    /* 1207 : CALL rm16 [m:        nw o16 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        /* 327 320 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,8086 */
        {I_CALL, 1, {RM_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67137, 194, 1494},
    /* 1208 : CALL rm32 [m:        nw o32 ff /2                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(ff) 202 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_CALL, 1, {RM_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67143, 197, 1495},
    /* 1209 : CALL rm64 [m:        nw o64 ff /2                      ] LONG,BND,PROT,X86_64 */
        /* 327 324 001(ff) 202 : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_CALL, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67149, 222, 1496},
    /* 1210 : RET void [          nw odf c3                         ] BND,8086 */
        /* 327 322 001(c3) : NWSIZE,BND,8086 */
        {I_RET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71699, 223, 1498},
    /* 1211 : RETW void [          nw o16 c3                         ] BND,8086 */
        /* 327 320 001(c3) : NWSIZE,BND,8086 */
        {I_RETW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71704, 223, 1499},
    /* 1212 : RETD void [          nw o32 c3                         ] NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(c3) : NWSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_RETD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71709, 224, 1500},
    /* 1213 : RETQ void [          nw o64 c3                         ] LONG,BND,386,X86_64 */
        /* 327 324 001(c3) : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_RETQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71714, 222, 1501},
    /* 1214 : RET imm16 [i:        nw odf c2 iw                      ] BND,8086 */
        /* 327 322 001(c2) 030 : NWSIZE,BND,8086 */
        {I_RET, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67179, 223, 1502},
    /* 1215 : RETW imm16 [i:        nw o16 c2 iw                      ] BND,8086 */
        /* 327 320 001(c2) 030 : NWSIZE,BND,8086 */
        {I_RETW, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67185, 223, 1503},
    /* 1216 : RETD imm16 [i:        nw o32 c2 iw                      ] NOREX,NOAPX,NOLONG,BND,386 */
        /* 327 321 001(c2) 030 : NWSIZE,NOREX,NOAPX,NOLONG,BND,386 */
        {I_RETD, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67191, 224, 1504},
    /* 1217 : RETQ imm16 [i:        nw o64 c2 iw                      ] LONG,BND,386,X86_64 */
        /* 327 324 001(c2) 030 : NWSIZE,LONG,BND,PROT,X86_64 */
        {I_RETQ, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67197, 222, 1505},
    /* 1218 : RETF void [          odf cb                            ] 8086 */
        /* 322 001(cb) : 8086 */
        {I_RETF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72740, 23, 1506},
    /* 1219 : RETFW void [          o16 cb                            ] 8086 */
        /* 320 001(cb) : 8086 */
        {I_RETFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72744, 23, 1507},
    /* 1220 : RETFD void [          o32 cb                            ] 386 */
        /* 321 001(cb) : 386 */
        {I_RETFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72748, 3, 1508},
    /* 1221 : RETFQ void [          o64 cb                            ] LONG,386,X86_64 */
        /* 324 001(cb) : LONG,PROT,X86_64 */
        {I_RETFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72752, 24, 1509},
    /* 1222 : RETF imm16 [i:        odf ca iw                         ] 8086 */
        /* 322 001(ca) 030 : 8086 */
        {I_RETF, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71719, 23, 1510},
    /* 1223 : RETFW imm16 [i:        o16 ca iw                         ] 8086 */
        /* 320 001(ca) 030 : 8086 */
        {I_RETFW, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71724, 23, 1511},
    /* 1224 : RETFD imm16 [i:        o32 ca iw                         ] 386 */
        /* 321 001(ca) 030 : 386 */
        {I_RETFD, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71729, 3, 1512},
    /* 1225 : RETFQ imm16 [i:        o64 ca iw                         ] LONG,386,X86_64 */
        /* 324 001(ca) 030 : LONG,PROT,X86_64 */
        {I_RETFQ, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71734, 24, 1513},
    /* 1226 : INT imm8 [i:        cd ib,u                           ] 8086 */
        /* 001(cd) 024 : 8086 */
        {I_INT, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72756, 23, 1524},
    /* 1227 : INT1 void [          f1                                ] 386 */
        /* 001(f1) : 386 */
        {I_INT1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73457, 3, 1525},
    /* 1228 : INT3 void [          cc                                ] 8086 */
        /* 001(cc) : 8086 */
        {I_INT3, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73460, 23, 1528},
    /* 1229 : INTO void [          ce                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(ce) : NOREX,NOAPX,NOLONG,8086 */
        {I_INTO, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73463, 225, 1531},
    /* 1230 : SYSCALL void [          0f 05                             ] P6,AMD */
        /* 355 001(05) : P6,AMD */
        {I_SYSCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72760, 226, 1533},
    /* 1231 : SYSENTER void [          0f 34                             ] NOAPX,P6 */
        /* 355 001(34) : NOAPX,P6 */
        {I_SYSENTER, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72764, 227, 1534},
    /* 1232 : SYSEXIT void [          0f 35                             ] NOAPX,PRIV,P6 */
        /* 355 001(35) : NOAPX,PRIV,P6 */
        {I_SYSEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72768, 228, 1536},
    /* 1233 : SYSRET void [          0f 07                             ] PRIV,P6,AMD */
        /* 355 001(07) : PRIV,P6,AMD */
        {I_SYSRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72772, 229, 1537},
    /* 1234 : IRET void [          odf cf                            ] FL,8086 */
        /* 322 001(cf) : FL,8086 */
        {I_IRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72776, 56, 1539},
    /* 1235 : IRETW void [          o16 cf                            ] FL,8086 */
        /* 320 001(cf) : FL,8086 */
        {I_IRETW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72780, 56, 1540},
    /* 1236 : IRETD void [          o32 cf                            ] FL,386 */
        /* 321 001(cf) : FL,386 */
        {I_IRETD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72784, 57, 1541},
    /* 1237 : IRETQ void [          o64 cf                            ] LONG,FL,386,X86_64 */
        /* 324 001(cf) : LONG,FL,PROT,X86_64 */
        {I_IRETQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72788, 58, 1542},
    /* 1238 : ERETS void [          f2 0f 01 ca                       ] LONG,PRIV,PROT,FRED,X86_64 */
        /* 332 355 002(01 ca) : LONG,PRIV,PROT,FRED,X86_64 */
        {I_ERETS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67203, 230, 1544},
    /* 1239 : ERETU void [          f3 0f 01 ca                       ] LONG,PRIV,PROT,FRED,X86_64 */
        /* 333 355 002(01 ca) : LONG,PRIV,PROT,FRED,X86_64 */
        {I_ERETU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67209, 230, 1545},
    /* 1240 : CLC void [          f8                                ] FL,8086 */
        /* 001(f8) : FL,8086 */
        {I_CLC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71936, 56, 1548},
    /* 1241 : CLD void [          fc                                ] FL,8086 */
        /* 001(fc) : FL,8086 */
        {I_CLD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67290, 56, 1549},
    /* 1242 : CLI void [          fa                                ] FL,8086 */
        /* 001(fa) : FL,8086 */
        {I_CLI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67500, 56, 1550},
    /* 1243 : CLAC void [          np 0f 01 ca                       ] FL,PRIV,SMAP,FUTURE */
        /* 360 355 002(01 ca) : FL,PRIV,SMAP,FUTURE */
        {I_CLAC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67215, 231, 1551},
    /* 1244 : STC void [          f9                                ] FL,8086 */
        /* 001(f9) : FL,8086 */
        {I_STC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71826, 56, 1553},
    /* 1245 : STD void [          fd                                ] FL,8086 */
        /* 001(fd) : FL,8086 */
        {I_STD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73466, 56, 1554},
    /* 1246 : STI void [          fb                                ] FL,8086 */
        /* 001(fb) : FL,8086 */
        {I_STI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71876, 56, 1555},
    /* 1247 : STAC void [          np 0f 01 cb                       ] FL,PRIV,SMAP,FUTURE */
        /* 360 355 002(01 cb) : FL,PRIV,SMAP,FUTURE */
        {I_STAC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67221, 231, 1556},
    /* 1248 : CMC void [          f5                                ] FL,8086 */
        /* 001(f5) : FL,8086 */
        {I_CMC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73469, 56, 1558},
    /* 1249 : LAHF void [          9f                                ] 8086 */
        /* 001(9f) : 8086 */
        {I_LAHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73472, 23, 1560},
    /* 1250 : SAHF void [          9e                                ] FL,8086 */
        /* 001(9e) : FL,8086 */
        {I_SAHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57389, 56, 1561},
    /* 1251 : SALC void [          d6                                ] NOREX,NOAPX,NOLONG,UNDOC,8086 */
        /* 001(d6) : NOREX,NOAPX,NOLONG,UNDOC,8086 */
        {I_SALC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72531, 232, 1562},
    /* 1252 : PUSHF void [          nw odf 9c                         ] 8086 */
        /* 327 322 001(9c) : NWSIZE,8086 */
        {I_PUSHF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71739, 233, 1564},
    /* 1253 : PUSHFW void [          nw o16 9c                         ] 8086 */
        /* 327 320 001(9c) : NWSIZE,8086 */
        {I_PUSHFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71744, 233, 1565},
    /* 1254 : PUSHFD void [          nw o32 9c                         ] 386 */
        /* 327 321 001(9c) : NWSIZE,386 */
        {I_PUSHFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71749, 234, 1566},
    /* 1255 : PUSHFQ void [          nw o64 9c                         ] LONG,386,X86_64 */
        /* 327 324 001(9c) : NWSIZE,LONG,PROT,X86_64 */
        {I_PUSHFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71754, 235, 1567},
    /* 1256 : POPF void [          nw odf 9d                         ] FL,8086 */
        /* 327 322 001(9d) : NWSIZE,FL,8086 */
        {I_POPF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71759, 236, 1568},
    /* 1257 : POPFW void [          nw o16 9d                         ] FL,8086 */
        /* 327 320 001(9d) : NWSIZE,FL,8086 */
        {I_POPFW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71764, 236, 1569},
    /* 1258 : POPFD void [          nw o32 9d                         ] FL,386 */
        /* 327 321 001(9d) : NWSIZE,FL,386 */
        {I_POPFD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71769, 237, 1570},
    /* 1259 : POPFQ void [          nw o64 9d                         ] LONG,FL,386,X86_64 */
        /* 327 324 001(9d) : NWSIZE,LONG,FL,PROT,X86_64 */
        {I_POPFQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71774, 238, 1571},
    /* 1260 : CMPSB void [          repe o8 a6                        ] NOAPX,FL,8086 */
        /* 335 001(a6) : NOAPX,FL,8086 */
        {I_CMPSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72792, 239, 1574},
    /* 1261 : CMPSW void [          repe o16 a7                       ] NOAPX,FL,8086 */
        /* 335 320 001(a7) : NOAPX,FL,8086 */
        {I_CMPSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71779, 239, 1575},
    /* 1262 : CMPSD void [          repe o32 a7                       ] NOAPX,FL,386 */
        /* 335 321 001(a7) : NOAPX,FL,386 */
        {I_CMPSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71784, 240, 1576},
    /* 1263 : CMPSQ void [          repe o64 a7                       ] NOAPX,LONG,FL,386,X86_64 */
        /* 335 324 001(a7) : NOAPX,LONG,FL,PROT,X86_64 */
        {I_CMPSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71789, 241, 1577},
    /* 1264 : LODSB void [          o8 ac                             ] NOAPX,8086 */
        /* 001(ac) : NOAPX,8086 */
        {I_LODSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73475, 242, 1578},
    /* 1265 : LODSW void [          o16 ad                            ] NOAPX,8086 */
        /* 320 001(ad) : NOAPX,8086 */
        {I_LODSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72796, 242, 1579},
    /* 1266 : LODSD void [          o32 ad                            ] NOAPX,386 */
        /* 321 001(ad) : NOAPX,386 */
        {I_LODSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72800, 243, 1580},
    /* 1267 : LODSQ void [          o64 ad                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(ad) : NOAPX,LONG,PROT,X86_64 */
        {I_LODSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72804, 244, 1581},
    /* 1268 : MOVSB void [          o8 a4                             ] NOAPX,8086 */
        /* 001(a4) : NOAPX,8086 */
        {I_MOVSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57417, 242, 1582},
    /* 1269 : MOVSW void [          o16 a5                            ] NOAPX,8086 */
        /* 320 001(a5) : NOAPX,8086 */
        {I_MOVSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72808, 242, 1583},
    /* 1270 : MOVSD void [          o32 a5                            ] NOAPX,386 */
        /* 321 001(a5) : NOAPX,386 */
        {I_MOVSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72812, 243, 1584},
    /* 1271 : MOVSQ void [          o64 a5                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(a5) : NOAPX,LONG,PROT,X86_64 */
        {I_MOVSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72816, 244, 1585},
    /* 1272 : STOSB void [          o8 aa                             ] NOAPX,8086 */
        /* 001(aa) : NOAPX,8086 */
        {I_STOSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72909, 242, 1586},
    /* 1273 : STOSW void [          o16 ab                            ] NOAPX,8086 */
        /* 320 001(ab) : NOAPX,8086 */
        {I_STOSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72820, 242, 1587},
    /* 1274 : STOSD void [          o32 ab                            ] NOAPX,386 */
        /* 321 001(ab) : NOAPX,386 */
        {I_STOSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72824, 243, 1588},
    /* 1275 : STOSQ void [          o64 ab                            ] NOAPX,LONG,386,X86_64 */
        /* 324 001(ab) : NOAPX,LONG,PROT,X86_64 */
        {I_STOSQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72828, 244, 1589},
    /* 1276 : SCASB void [          repe o8 ae                        ] NOAPX,FL,8086 */
        /* 335 001(ae) : NOAPX,FL,8086 */
        {I_SCASB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72832, 239, 1590},
    /* 1277 : SCASW void [          repe o16 af                       ] NOAPX,FL,8086 */
        /* 335 320 001(af) : NOAPX,FL,8086 */
        {I_SCASW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71794, 239, 1591},
    /* 1278 : SCASD void [          repe o32 af                       ] NOAPX,FL,386 */
        /* 335 321 001(af) : NOAPX,FL,386 */
        {I_SCASD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71799, 240, 1592},
    /* 1279 : SCASQ void [          repe o64 af                       ] NOAPX,LONG,FL,386,X86_64 */
        /* 335 324 001(af) : NOAPX,LONG,FL,PROT,X86_64 */
        {I_SCASQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71804, 241, 1593},
    /* 1280 : INSB void [          o8 6c                             ] 186 */
        /* 001(6c) : 186 */
        {I_INSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73478, 245, 1594},
    /* 1281 : INSW void [          o16 6d                            ] 186 */
        /* 320 001(6d) : 186 */
        {I_INSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72836, 245, 1595},
    /* 1282 : INSD void [          o32 6d                            ] 386 */
        /* 321 001(6d) : 386 */
        {I_INSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72840, 3, 1596},
    /* 1283 : OUTSB void [          o8 6e                             ] 186 */
        /* 001(6e) : 186 */
        {I_OUTSB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73481, 245, 1597},
    /* 1284 : OUTSW void [          o16 6f                            ] 186 */
        /* 320 001(6f) : 186 */
        {I_OUTSW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72844, 245, 1598},
    /* 1285 : OUTSD void [          o32 6f                            ] 386 */
        /* 321 001(6f) : 386 */
        {I_OUTSD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72848, 3, 1599},
    /* 1286 : LFENCE void [          np 0f ae e8                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae e8) : LONG,PROT,X86_64,AMD */
        {I_LFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67227, 246, 1602},
    /* 1287 : MFENCE void [          np 0f ae f0                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae f0) : LONG,PROT,X86_64,AMD */
        {I_MFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67233, 246, 1603},
    /* 1288 : SFENCE void [          np 0f ae f8                       ] LONG,PROT,X86_64,AMD */
        /* 360 355 002(ae f8) : LONG,PROT,X86_64,AMD */
        {I_SFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67239, 246, 1604},
    /* 1289 : SERIALIZE void [          np 0f 01 e8                       ] SERIALIZE,FUTURE */
        /* 360 355 002(01 e8) : SERIALIZE,FUTURE */
        {I_SERIALIZE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67245, 247, 1605},
    /* 1290 : CLFLUSH mem [m:        np 0f ae /7                       ] SSE2,WILLAMETTE */
        /* 360 355 001(ae) 207 : SSE2,WILLAMETTE */
        {I_CLFLUSH, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67251, 248, 1608},
    /* 1291 : CLFLUSHOPT mem [m:        66 0f ae /7                       ] CLFLUSHOPT,FUTURE */
        /* 361 355 001(ae) 207 : CLFLUSHOPT,FUTURE */
        {I_CLFLUSHOPT, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67257, 249, 1609},
    /* 1292 : CLWB mem [m:        66 0f ae /6                       ] CLWB,FUTURE */
        /* 361 355 001(ae) 206 : CLWB,FUTURE */
        {I_CLWB, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67263, 250, 1610},
    /* 1293 : PCOMMIT void [          66 0f ae f8                       ] OBSOLETE,NEVER,NOP,FUTURE */
        /* 361 355 002(ae f8) : OBSOLETE,NEVER,NOP,FUTURE */
        {I_PCOMMIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67269, 251, 1613},
    /* 1294 : CLZERO reg_ax [-:        a16 0f 01 fc                      ] NOLONG,CLZERO,FUTURE,AMD */
        /* 310 355 002(01 fc) : NOREX,NOAPX,NOLONG,CLZERO,FUTURE,AMD */
        {I_CLZERO, 1, {REG_AX|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67275, 252, 1616},
    /* 1295 : CLZERO reg_eax [-:        a32 0f 01 fc                      ] CLZERO,FUTURE,AMD */
        /* 311 355 002(01 fc) : CLZERO,FUTURE,AMD */
        {I_CLZERO, 1, {REG_EAX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67281, 253, 1617},
    /* 1296 : CLZERO reg_rax [-:        a64 0f 01 fc                      ] LONG,PROT,CLZERO,X86_64,AMD */
        /* 313 355 002(01 fc) : LONG,PROT,CLZERO,X86_64,AMD */
        {I_CLZERO, 1, {REG_RAX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67287, 254, 1618},
    /* 1297 : INVD void [          0f 08                             ] PRIV,486 */
        /* 355 001(08) : PRIV,486 */
        {I_INVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72852, 255, 1621},
    /* 1298 : WBINVD void [          np 0f 09                          ] PRIV,486 */
        /* 360 355 001(09) : PRIV,486 */
        {I_WBINVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71809, 255, 1622},
    /* 1299 : WBNOINVD void [          f3 0f 09                          ] PRIV,WBNOINVD,FUTURE */
        /* 333 355 001(09) : PRIV,WBNOINVD,FUTURE */
        {I_WBNOINVD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71814, 256, 1623},
    /* 1300 : INVPCID reg32,mem128 [rm:       66 0f38 82 /r                     ] NOREX,NOAPX,NOLONG,PRIV,INVPCID,FUTURE */
        /* 361 356 001(82) 110 : NOREX,NOAPX,NOLONG,PRIV,INVPCID,FUTURE */
        {I_INVPCID, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67293, 257, 1625},
    /* 1301 : INVPCID reg64,mem128 [rm:       66 0f38 82 /r                     ] LONG,PRIV,PROT,INVPCID,X86_64 */
        /* 361 356 001(82) 110 : NOAPX,LONG,PRIV,PROT,INVPCID,X86_64 */
        {I_INVPCID, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67293, 258, 1626},
    /* 1302 : INVPCID reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f2 /r    ] LONG,PRIV,PROT,APX,INVPCID,X86_64 */
        /* 250(f4 fe 08) 300 001(f2) 110 : LONG,EVEX,PRIV,PROT,APX,INVPCID,X86_64 */
        {I_INVPCID, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+24682, 259, 1627},
    /* 1303 : INVLPG mem [m:        0f 01 /7                          ] PRIV,486 */
        /* 355 001(01) 207 : PRIV,486 */
        {I_INVLPG, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71819, 255, 1628},
    /* 1304 : INVLPGA reg_ax,reg_ecx [--:       a16 0f 01 df                      ] NOLONG,X86_64,AMD */
        /* 310 355 002(01 df) : NOREX,NOAPX,NOLONG,X86_64,AMD */
        {I_INVLPGA, 2, {REG_AX|RN_L16,REG_ECX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67299, 260, 1629},
    /* 1305 : INVLPGA reg_eax,reg_ecx [--:       a32 0f 01 df                      ] X86_64,AMD */
        /* 311 355 002(01 df) : X86_64,AMD */
        {I_INVLPGA, 2, {REG_EAX,REG_ECX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67305, 261, 1630},
    /* 1306 : INVLPGA reg_rax,reg_ecx [--:       a64 0f 01 df                      ] LONG,PROT,X86_64,AMD */
        /* 313 355 002(01 df) : LONG,PROT,X86_64,AMD */
        {I_INVLPGA, 2, {REG_RAX,REG_ECX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67311, 246, 1631},
    /* 1307 : RDPMC void [          0f 33                             ] NOAPX,P6 */
        /* 355 001(33) : NOAPX,P6 */
        {I_RDPMC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72856, 227, 1635},
    /* 1308 : RDTSC void [          0f 31                             ] NOAPX,PENT */
        /* 355 001(31) : NOAPX,PENT */
        {I_RDTSC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72860, 262, 1636},
    /* 1309 : RDTSCP void [          0f 01 f9                          ] X86_64 */
        /* 355 002(01 f9) : X86_64 */
        {I_RDTSCP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71824, 263, 1637},
    /* 1310 : RDRAND reg16 [m:        o16 0f c7 /6                      ] FL,RDRAND,FUTURE */
        /* 320 355 001(c7) 206 : FL,RDRAND,FUTURE */
        {I_RDRAND, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67323, 264, 1638},
    /* 1311 : RDRAND reg32 [m:        o32 0f c7 /6                      ] FL,RDRAND,FUTURE */
        /* 321 355 001(c7) 206 : FL,RDRAND,FUTURE */
        {I_RDRAND, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67329, 264, 1639},
    /* 1312 : RDRAND reg64 [m:        o64 0f c7 /6                      ] LONG,FL,PROT,RDRAND,X86_64 */
        /* 324 355 001(c7) 206 : LONG,FL,PROT,RDRAND,X86_64 */
        {I_RDRAND, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67335, 265, 1640},
    /* 1313 : RDSEED reg16 [m:        o16 0f c7 /7                      ] FL,RDSEED,FUTURE */
        /* 320 355 001(c7) 207 : FL,RDSEED,FUTURE */
        {I_RDSEED, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67341, 266, 1641},
    /* 1314 : RDSEED reg32 [m:        o32 0f c7 /7                      ] FL,RDSEED,FUTURE */
        /* 321 355 001(c7) 207 : FL,RDSEED,FUTURE */
        {I_RDSEED, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51325, 266, 1642},
    /* 1315 : RDSEED reg64 [m:        o64 0f c7 /7                      ] LONG,FL,PROT,RDSEED,X86_64 */
        /* 324 355 001(c7) 207 : LONG,FL,PROT,RDSEED,X86_64 */
        {I_RDSEED, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51333, 267, 1643},
    /* 1316 : RDPID reg32 [m:        f3 !osp o32 0f c7 /7              ] RDPID,FUTURE */
        /* 333 364 321 355 001(c7) 207 : RDPID,FUTURE */
        {I_RDPID, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51323, 269, 1645},
    /* 1317 : RDPID reg64 [m:        f3 !osp o64 0f c7 /7              ] LONG,PROT,RDPID,X86_64 */
        /* 333 364 324 355 001(c7) 207 : LONG,PROT,RDPID,X86_64 */
        {I_RDPID, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+51331, 270, 1646},
    /* 1318 : CLTS void [          0f 06                             ] PRIV,286 */
        /* 355 001(06) : PRIV,286 */
        {I_CLTS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72864, 271, 1649},
    /* 1319 : CPUID void [          0f a2                             ] PENT */
        /* 355 001(a2) : PENT */
        {I_CPUID, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72868, 272, 1650},
    /* 1320 : LMSW rm16 [m:        0f 01 /6                          ] PRIV,286 */
        /* 355 001(01) 206 : PRIV,286 */
        {I_LMSW, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71829, 271, 1651},
    /* 1321 : SMSW rm16 [m:        0f 01 /4                          ] 286 */
        /* 355 001(01) 204 : 286 */
        {I_SMSW, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67372, 273, 1652},
    /* 1322 : SMSW reg16 [m:        o16 0f 01 /4                      ] 286 */
        /* 320 355 001(01) 204 : 286 */
        {I_SMSW, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67359, 273, 1654},
    /* 1323 : SMSW reg32 [m:        o32 0f 01 /4                      ] 386 */
        /* 321 355 001(01) 204 : 386 */
        {I_SMSW, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67365, 3, 1655},
    /* 1324 : SMSW reg64 [m:        o64 0f 01 /4                      ] LONG,PROT,X86_64 */
        /* 324 355 001(01) 204 : LONG,PROT,X86_64 */
        {I_SMSW, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67371, 24, 1656},
    /* 1325 : MOV reg32,reg_creg [mr:       rex.l 0f 20 /r                    ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 334 355 001(20) 101 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_GPR|BITS32|RN_L16,REG_CREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67377, 274, 1658},
    /* 1326 : MOV reg_creg,reg32 [rm:       rex.l 0f 22 /r                    ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 334 355 001(22) 110 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_CREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67383, 274, 1659},
    /* 1327 : MOV reg64,reg_creg [mr:       o64nw 0f 20 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(20) 101 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_CREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67389, 275, 1660},
    /* 1328 : MOV reg_creg,reg64 [rm:       o64nw 0f 22 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(22) 110 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_CREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67395, 275, 1661},
    /* 1329 : MOV reg32,reg_dreg [mr:       0f 21 /r                          ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 355 001(21) 101 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_GPR|BITS32|RN_L16,REG_DREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67402, 274, 1662},
    /* 1330 : MOV reg_dreg,reg32 [rm:       0f 23 /r                          ] NOREX,NOAPX,NOLONG,PRIV,386 */
        /* 355 001(23) 110 : NOREX,NOAPX,NOLONG,PRIV,386 */
        {I_MOV, 2, {REG_DREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67408, 274, 1663},
    /* 1331 : MOV reg64,reg_dreg [mr:       o64nw 0f 21 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(21) 101 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_DREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67401, 275, 1664},
    /* 1332 : MOV reg_dreg,reg64 [rm:       o64nw 0f 23 /r                    ] LONG,PRIV,PROT,X86_64 */
        /* 323 355 001(23) 110 : NWSIZE,LONG,PRIV,PROT,X86_64 */
        {I_MOV, 2, {REG_DREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67407, 275, 1665},
    /* 1333 : WRMSR void [          0f 30                             ] NOAPX,PRIV,PENT */
        /* 355 001(30) : NOAPX,PRIV,PENT */
        {I_WRMSR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72872, 277, 1669},
    /* 1334 : RDMSR void [          0f 32                             ] NOAPX,PRIV,PENT */
        /* 355 001(32) : NOAPX,PRIV,PENT */
        {I_RDMSR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72876, 277, 1670},
    /* 1335 : RDMSR reg64,imm32 [mi:       vex.128.f2.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,MSR_IMM,X86_64,FUTURE */
        /* 270(007 003) 001(f6) 200 041 : NOAPX,LONG,VEX,PRIV,PROT,MSR_IMM,FUTURE */
        {I_RDMSR, 2, {REG_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51339, 278, 1671},
    /* 1336 : RDMSR reg64,imm32 [mi:       evex.128.f2.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,MSR_IMM,X86_64,FUTURE,APX */
        /* 250(f7 7f 08) 300 001(f6) 200 041 : LONG,EVEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_RDMSR, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13023, 279, 1671},
    /* 1337 : WRMSRNS void [          np 0f 01 c6                       ] PRIV,WRMSRNS,FUTURE */
        /* 360 355 002(01 c6) : PRIV,WRMSRNS,FUTURE */
        {I_WRMSRNS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67413, 280, 1672},
    /* 1338 : WRMSRNS imm32,reg64 [im:       vex.128.f3.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,MSR_IMM,X86_64,FUTURE */
        /* 270(007 002) 001(f6) 210 040 : NOAPX,LONG,VEX,PRIV,PROT,MSR_IMM,FUTURE */
        {I_WRMSRNS, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51347, 278, 1673},
    /* 1339 : WRMSRNS imm32,reg64 [im:       evex.128.f3.m7.w0 f6 /0 id        ] LONG,PRIV,PROT,MSR_IMM,X86_64,FUTURE,APX */
        /* 250(f7 7e 08) 300 001(f6) 210 040 : LONG,EVEX,PRIV,PROT,APX,MSR_IMM,FUTURE */
        {I_WRMSRNS, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13033, 279, 1673},
    /* 1340 : RDMSRLIST void [          f2 0f 01 c6                       ] LONG,PRIV,PROT,MSRLIST,X86_64 */
        /* 332 355 002(01 c6) : LONG,PRIV,PROT,MSRLIST,X86_64 */
        {I_RDMSRLIST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67419, 281, 1674},
    /* 1341 : WRMSRLIST void [          f3 0f 01 c6                       ] LONG,PRIV,PROT,MSRLIST,X86_64 */
        /* 333 355 002(01 c6) : LONG,PRIV,PROT,MSRLIST,X86_64 */
        {I_WRMSRLIST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67425, 281, 1675},
    /* 1342 : URDMSR reg64,reg64 [mr:       f2 0f 38 f8 /r                    ] NOAPX,FUTURE */
        /* 332 355 002(38 f8) 101 : NOAPX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+57049, 282, 1677},
    /* 1343 : URDMSR reg64,reg64 [mr:       evex.nf0.nd0.l0.f2.m4.w0 f8 /r    ] LONG,PROT,APX,X86_64,FUTURE */
        /* 250(f4 7f 08) 300 001(f8) 101 : LONG,EVEX,PROT,APX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24691, 283, 1678},
    /* 1344 : URDMSR reg64,imm32 [mi:       vex.128.f2.m7.w0 f8 /0 id        ] FUTURE */
        /* 270(007 003) 001(f8) 200 041 : NOAPX,VEX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64|RN_L16,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51355, 284, 1679},
    /* 1345 : URDMSR reg64,imm32 [mi:       evex.128.f2.m7.w0 f8 /0 id        ] FUTURE,APX */
        /* 250(f7 7f 08) 300 001(f8) 200 041 : LONG,EVEX,PROT,APX,FUTURE */
        {I_URDMSR, 2, {REG_GPR|BITS64,IMM_NORMAL|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+13043, 283, 1679},
    /* 1346 : UWRMSR reg64,reg64 [rm:       f3 0f 38 f8 /r                    ] NOAPX,FUTURE */
        /* 333 355 002(38 f8) 110 : NOAPX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+57056, 282, 1680},
    /* 1347 : UWRMSR reg64,reg64 [rm:       evex.nf0.nd0.l0.f3.m4.w0 f8 /r    ] LONG,PROT,APX,X86_64,FUTURE */
        /* 250(f4 7e 08) 300 001(f8) 110 : LONG,EVEX,PROT,APX,FUTURE */
        {I_UWRMSR, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+24700, 283, 1681},
    /* 1348 : UWRMSR imm32,reg64 [im:       vex.128.f3.m7.w0 f8 /0 id        ] FUTURE */
        /* 270(007 002) 001(f8) 210 040 : NOAPX,VEX,FUTURE */
        {I_UWRMSR, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51363, 284, 1682},
    /* 1349 : UWRMSR imm32,reg64 [im:       evex.128.f3.m7.w0 f8 /0 id        ] FUTURE,APX */
        /* 250(f7 7e 08) 300 001(f8) 210 040 : LONG,EVEX,PROT,APX,FUTURE */
        {I_UWRMSR, 2, {IMM_NORMAL|BITS32,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13053, 283, 1682},
    /* 1350 : RSM void [          0f aa                             ] FL,SMM,PENT */
        /* 355 001(aa) : FL,SMM,PENT */
        {I_RSM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72908, 289, 1702},
    /* 1351 : HLT void [          f4                                ] PRIV,8086 */
        /* 001(f4) : PRIV,8086 */
        {I_HLT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73484, 292, 1715},
    /* 1352 : PAUSE void [          osz f3i 90                        ] 8086 */
        /* 330 333 001(90) : 8086 */
        {I_PAUSE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71864, 23, 1717},
    /* 1353 : MONITOR void [          asm 0f 01 c8                      ] MONITOR,PRESCOTT */
        /* 312 355 002(01 c8) : MONITOR,PRESCOTT */
        {I_MONITOR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67455, 293, 1719},
    /* 1354 : MONITORW void [          a16 0f 01 c8                      ] NOLONG,MONITOR,PRESCOTT */
        /* 310 355 002(01 c8) : NOREX,NOAPX,NOLONG,MONITOR,PRESCOTT */
        {I_MONITORW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67461, 294, 1720},
    /* 1355 : MONITORD void [          a32 0f 01 c8                      ] MONITOR,PRESCOTT */
        /* 311 355 002(01 c8) : MONITOR,PRESCOTT */
        {I_MONITORD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67467, 293, 1721},
    /* 1356 : MONITORQ void [          a64 0f 01 c8                      ] LONG,MONITOR,PRESCOTT,X86_64 */
        /* 313 355 002(01 c8) : LONG,PROT,MONITOR,X86_64 */
        {I_MONITORQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67473, 295, 1722},
    /* 1357 : MWAIT void [          0f 01 c9                          ] MONITOR,PRESCOTT */
        /* 355 002(01 c9) : MONITOR,PRESCOTT */
        {I_MWAIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71869, 293, 1726},
    /* 1358 : MONITORX void [          asm 0f 01 fa                      ] MONITORX,FUTURE,AMD */
        /* 312 355 002(01 fa) : MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67479, 296, 1729},
    /* 1359 : MONITORX void [          a16 0f 01 fa                      ] NOLONG,MONITORX,FUTURE,AMD */
        /* 310 355 002(01 fa) : NOREX,NOAPX,NOLONG,MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67485, 297, 1730},
    /* 1360 : MONITORX void [          a32 0f 01 fa                      ] MONITORX,FUTURE,AMD */
        /* 311 355 002(01 fa) : MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67491, 296, 1731},
    /* 1361 : MONITORX void [          a64 0f 01 fa                      ] LONG,MONITORX,X86_64,FUTURE,AMD */
        /* 313 355 002(01 fa) : LONG,PROT,MONITORX,FUTURE,AMD */
        {I_MONITORX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67497, 298, 1732},
    /* 1362 : MWAITX void [          0f 01 fb                          ] MONITORX,FUTURE,AMD */
        /* 355 002(01 fb) : MONITORX,FUTURE,AMD */
        {I_MWAITX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71874, 296, 1736},
    /* 1363 : TPAUSE reg32 [m:        66 0f ae /6                       ] FL,WAITPKG,FUTURE */
        /* 361 355 001(ae) 206 : FL,WAITPKG,FUTURE */
        {I_TPAUSE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67263, 300, 1739},
    /* 1364 : UMONITOR reg16 [m:        a16 f3 0f ae /6                   ] NOLONG,WAITPKG,FUTURE */
        /* 310 333 355 001(ae) 206 : NOREX,NOAPX,NOLONG,WAITPKG,FUTURE */
        {I_UMONITOR, 1, {REG_GPR|BITS16|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57091, 301, 1741},
    /* 1365 : UMONITOR reg32 [m:        a32 f3 0f ae /6                   ] WAITPKG,FUTURE */
        /* 311 333 355 001(ae) 206 : WAITPKG,FUTURE */
        {I_UMONITOR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57098, 302, 1742},
    /* 1366 : UMONITOR reg64 [m:        a64 f3 0f ae /6                   ] LONG,PROT,WAITPKG,X86_64 */
        /* 313 333 355 001(ae) 206 : LONG,PROT,WAITPKG,X86_64 */
        {I_UMONITOR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57105, 303, 1743},
    /* 1367 : UMWAIT reg32 [m:        f2 0f ae /6                       ] WAITPKG,FUTURE */
        /* 332 355 001(ae) 206 : WAITPKG,FUTURE */
        {I_UMWAIT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67503, 302, 1744},
    /* 1368 : IN reg_al,imm8 [-i:       o8 e4 ib,u                        ] NOAPX,8086 */
        /* 001(e4) 025 : NOAPX,8086 */
        {I_IN, 2, {REG_AL|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+72912, 242, 1748},
    /* 1369 : IN reg_ax,imm8 [-i:       o16 e5 ib,u                       ] NOAPX,8086 */
        /* 320 001(e5) 025 : NOAPX,8086 */
        {I_IN, 2, {REG_AX|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71879, 242, 1749},
    /* 1370 : IN reg_eax,imm8 [-i:       o32 e5 ib,u                       ] NOAPX,ZU,386 */
        /* 321 001(e5) 025 : NOAPX,ZU,386 */
        {I_IN, 2, {REG_EAX|RN_L16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+71884, 304, 1750},
    /* 1371 : IN reg_al,reg_dx [--:       o8 ec                             ] NOAPX,8086 */
        /* 001(ec) : NOAPX,8086 */
        {I_IN, 2, {REG_AL|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70182, 242, 1751},
    /* 1372 : IN reg_ax,reg_dx [--:       o16 ed                            ] NOAPX,8086 */
        /* 320 001(ed) : NOAPX,8086 */
        {I_IN, 2, {REG_AX|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72916, 242, 1752},
    /* 1373 : IN reg_eax,reg_dx [--:       o32 ed                            ] NOAPX,ZU,386 */
        /* 321 001(ed) : NOAPX,ZU,386 */
        {I_IN, 2, {REG_EAX|RN_L16,REG_DX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72920, 304, 1753},
    /* 1374 : OUT imm8,reg_al [i-:       o8 e6 ib,u                        ] NOAPX,8086 */
        /* 001(e6) 024 : NOAPX,8086 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72924, 242, 1754},
    /* 1375 : OUT imm8,reg_ax [i-:       o16 e7 ib,u                       ] NOAPX,8086 */
        /* 320 001(e7) 024 : NOAPX,8086 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71889, 242, 1755},
    /* 1376 : OUT imm8,reg_eax [i-:       o32 e7 ib,u                       ] NOAPX,386 */
        /* 321 001(e7) 024 : NOAPX,386 */
        {I_OUT, 2, {IMM_NORMAL|BITS8,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71894, 243, 1756},
    /* 1377 : OUT reg_dx,reg_al [--:       o8 ee                             ] NOAPX,8086 */
        /* 001(ee) : NOAPX,8086 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_AL|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70164, 242, 1757},
    /* 1378 : OUT reg_dx,reg_ax [--:       o16 ef                            ] NOAPX,8086 */
        /* 320 001(ef) : NOAPX,8086 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_AX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72928, 242, 1758},
    /* 1379 : OUT reg_dx,reg_eax [--:       o32 ef                            ] NOAPX,386 */
        /* 321 001(ef) : NOAPX,386 */
        {I_OUT, 2, {REG_DX|RN_L16,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72932, 243, 1759},
    /* 1380 : MOV mem16,reg_sreg [mr:       8c /r                             ] 8086 */
        /* 001(8c) 101 : 8086 */
        {I_MOV, 2, {MEMORY|BITS16,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71900, 23, 1762},
    /* 1381 : MOV reg16,reg_sreg [mr:       o16 8c /r                         ] 8086 */
        /* 320 001(8c) 101 : 8086 */
        {I_MOV, 2, {REG_GPR|BITS16,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71899, 23, 1763},
    /* 1382 : MOV reg32,reg_sreg [mr:       optd o32 8c /r                    ] 386 */
        /* 337 321 001(8c) 101 : 386 */
        {I_MOV, 2, {REG_GPR|BITS32,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67509, 3, 1764},
    /* 1383 : MOV reg64,reg_sreg [mr:       optd o64 8c /r                    ] LONG,386,X86_64 */
        /* 337 324 001(8c) 101 : LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_GPR|BITS64,REG_SREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67515, 24, 1765},
    /* 1384 : MOV reg_sreg,mem16 [rm:       8e /r                             ] 8086 */
        /* 001(8e) 110 : 8086 */
        {I_MOV, 2, {REG_SREG,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67535, 23, 1766},
    /* 1385 : MOV reg_sreg,reg16 [rm:       optw o16 8e /r                    ] 8086 */
        /* 336 320 001(8e) 110 : 8086 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67521, 23, 1767},
    /* 1386 : MOV reg_sreg,reg32 [rm:       optw o32 8e /r                    ] 386 */
        /* 336 321 001(8e) 110 : 386 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67527, 3, 1768},
    /* 1387 : MOV reg_sreg,reg64 [rm:       optw o64 8e /r                    ] LONG,386,X86_64 */
        /* 336 324 001(8e) 110 : LONG,PROT,X86_64 */
        {I_MOV, 2, {REG_SREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67533, 24, 1769},
    /* 1388 : LDS reg16,mem16 [rm:       o16 c5 /r                         ] NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(c5) 110 : NOREX,NOAPX,NOLONG,8086 */
        {I_LDS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71904, 225, 1771},
    /* 1389 : LDS reg32,mem32 [rm:       o32 c5 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(c5) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_LDS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71909, 305, 1772},
    /* 1390 : LES reg16,mem16 [rm:       o16 c4 /r                         ] NOREX,NOAPX,NOLONG,8086 */
        /* 320 001(c4) 110 : NOREX,NOAPX,NOLONG,8086 */
        {I_LES, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71914, 225, 1773},
    /* 1391 : LES reg32,mem32 [rm:       o32 c4 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(c4) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_LES, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+71919, 305, 1774},
    /* 1392 : LFS reg16,mem16 [rm:       o16 0f b4 /r                      ] SM0-1,386 */
        /* 320 355 001(b4) 110 : SM0,SM1,386 */
        {I_LFS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67539, 13, 1775},
    /* 1393 : LFS reg32,mem32 [rm:       o32 0f b4 /r                      ] SM0-1,386 */
        /* 321 355 001(b4) 110 : SM0,SM1,386 */
        {I_LFS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67545, 13, 1776},
    /* 1394 : LFS reg64,mem64 [rm:       o64 0f b4 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b4) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LFS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67551, 14, 1777},
    /* 1395 : LGS reg16,mem16 [rm:       o16 0f b5 /r                      ] SM0-1,386 */
        /* 320 355 001(b5) 110 : SM0,SM1,386 */
        {I_LGS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67557, 13, 1778},
    /* 1396 : LGS reg32,mem32 [rm:       o32 0f b5 /r                      ] SM0-1,386 */
        /* 321 355 001(b5) 110 : SM0,SM1,386 */
        {I_LGS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67563, 13, 1779},
    /* 1397 : LGS reg64,mem64 [rm:       o64 0f b5 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b5) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LGS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67569, 14, 1780},
    /* 1398 : LSS reg16,mem16 [rm:       o16 0f b2 /r                      ] SM0-1,386 */
        /* 320 355 001(b2) 110 : SM0,SM1,386 */
        {I_LSS, 2, {REG_GPR|BITS16,MEMORY|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67575, 13, 1781},
    /* 1399 : LSS reg32,mem32 [rm:       o32 0f b2 /r                      ] SM0-1,386 */
        /* 321 355 001(b2) 110 : SM0,SM1,386 */
        {I_LSS, 2, {REG_GPR|BITS32,MEMORY|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67581, 13, 1782},
    /* 1400 : LSS reg64,mem64 [rm:       o64 0f b2 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b2) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_LSS, 2, {REG_GPR|BITS64,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67587, 14, 1783},
    /* 1401 : PUSH reg_es [-:        06                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(06) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_ES|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72865, 225, 1785},
    /* 1402 : PUSH reg_cs [-:        0e                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(0e) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_CS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73073, 225, 1786},
    /* 1403 : PUSH reg_ss [-:        16                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(16) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_SS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29251, 225, 1787},
    /* 1404 : PUSH reg_ds [-:        1e                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(1e) : NOREX,NOAPX,NOLONG,8086 */
        {I_PUSH, 1, {REG_DS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29395, 225, 1788},
    /* 1405 : PUSH reg_fs [-:        0f a0                             ] 386 */
        /* 355 001(a0) : 386 */
        {I_PUSH, 1, {REG_FS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72936, 3, 1789},
    /* 1406 : PUSH reg_gs [-:        0f a8                             ] 386 */
        /* 355 001(a8) : 386 */
        {I_PUSH, 1, {REG_GS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72940, 3, 1790},
    /* 1407 : POP reg_es [-:        07                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(07) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_ES|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72773, 225, 1792},
    /* 1408 : POP reg_ss [-:        17                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(17) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_SS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29269, 225, 1794},
    /* 1409 : POP reg_ds [-:        1f                                ] NOREX,NOAPX,NOLONG,8086 */
        /* 001(1f) : NOREX,NOAPX,NOLONG,8086 */
        {I_POP, 1, {REG_DS|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+29413, 225, 1795},
    /* 1410 : POP reg_fs [-:        0f a1                             ] 386 */
        /* 355 001(a1) : 386 */
        {I_POP, 1, {REG_FS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72944, 3, 1796},
    /* 1411 : POP reg_gs [-:        0f a9                             ] 386 */
        /* 355 001(a9) : 386 */
        {I_POP, 1, {REG_GS,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72948, 3, 1797},
    /* 1412 : RDFSBASE reg32 [m:        w0 f3 0f ae /0                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 200 : LONG,PROT,X86_64 */
        {I_RDFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57112, 24, 1799},
    /* 1413 : RDFSBASE reg64 [m:        w1 f3 0f ae /0                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 200 : LONG,PROT,X86_64 */
        {I_RDFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57119, 24, 1800},
    /* 1414 : RDGSBASE reg32 [m:        w0 f3 0f ae /1                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 201 : LONG,PROT,X86_64 */
        {I_RDGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57126, 24, 1801},
    /* 1415 : RDGSBASE reg64 [m:        w1 f3 0f ae /1                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 201 : LONG,PROT,X86_64 */
        {I_RDGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57133, 24, 1802},
    /* 1416 : WRFSBASE reg32 [m:        w0 f3 0f ae /2                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 202 : LONG,PROT,X86_64 */
        {I_WRFSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57140, 24, 1803},
    /* 1417 : WRFSBASE reg64 [m:        w1 f3 0f ae /2                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 202 : LONG,PROT,X86_64 */
        {I_WRFSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57147, 24, 1804},
    /* 1418 : WRGSBASE reg32 [m:        w0 f3 0f ae /3                    ] LONG,PROT,X86_64 */
        /* 317 333 355 001(ae) 203 : LONG,PROT,X86_64 */
        {I_WRGSBASE, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57154, 24, 1805},
    /* 1419 : WRGSBASE reg64 [m:        w1 f3 0f ae /3                    ] LONG,PROT,X86_64 */
        /* 324 333 355 001(ae) 203 : LONG,PROT,X86_64 */
        {I_WRGSBASE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57161, 24, 1806},
    /* 1420 : ARPL rm16,reg16 [mr:       optw o16 63 /r                    ] SM0-1,NOREX,NOAPX,NOLONG,FL,PROT,286 */
        /* 336 320 001(63) 101 : SM0,SM1,NOREX,NOAPX,NOLONG,FL,PROT,286 */
        {I_ARPL, 2, {RM_GPR|BITS16|RN_L16,REG_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67593, 307, 1808},
    /* 1421 : ARPL rm16,reg32 [mr:       optw o32 63 /r                    ] SM0-1,NOREX,NOAPX,NOLONG,FL,PROT,386 */
        /* 336 321 001(63) 101 : SM0,SM1,NOREX,NOAPX,NOLONG,FL,PROT,386 */
        {I_ARPL, 2, {RM_GPR|BITS16|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+67599, 308, 1809},
    /* 1422 : LAR reg16,rm_sel [rm:       o16 0f 02 /r                      ] FL,PROT,286 */
        /* 320 355 001(02) 110 : FL,PROT,286 */
        {I_LAR, 2, {REG_GPR|BITS16,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+67605, 309, 1810},
    /* 1423 : LAR reg32,rm_sel [rm:       optd o32 0f 02 /r                 ] FL,PROT,386 */
        /* 337 321 355 001(02) 110 : FL,PROT,386 */
        {I_LAR, 2, {REG_GPR|BITS32,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57168, 310, 1811},
    /* 1424 : LAR reg64,rm_sel [rm:       optd o64 0f 02 /r                 ] LONG,FL,PROT,X86_64 */
        /* 337 324 355 001(02) 110 : LONG,FL,PROT,X86_64 */
        {I_LAR, 2, {REG_GPR|BITS64,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57175, 58, 1812},
    /* 1425 : LSL reg16,rm_sel [rm:       o16 0f 03 /r                      ] PROT,286 */
        /* 320 355 001(03) 110 : PROT,286 */
        {I_LSL, 2, {REG_GPR|BITS16,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+67611, 311, 1813},
    /* 1426 : LSL reg32,rm_sel [rm:       optd o32 0f 03 /r                 ] PROT,386 */
        /* 337 321 355 001(03) 110 : PROT,386 */
        {I_LSL, 2, {REG_GPR|BITS32,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57182, 312, 1814},
    /* 1427 : LSL reg64,rm_sel [rm:       optd o64 0f 03 /r                 ] LONG,PROT,X86_64 */
        /* 337 324 355 001(03) 110 : LONG,PROT,X86_64 */
        {I_LSL, 2, {REG_GPR|BITS64,RM_SEL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57189, 24, 1815},
    /* 1428 : VERR rm_sel [m:        0f 00 /4                          ] FL,PROT,286 */
        /* 355 001(00) 204 : FL,PROT,286 */
        {I_VERR, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71924, 309, 1817},
    /* 1429 : VERW rm_sel [m:        0f 00 /5                          ] FL,PROT,286 */
        /* 355 001(00) 205 : FL,PROT,286 */
        {I_VERW, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71929, 309, 1818},
    /* 1430 : SWAPGS void [          0f 01 f8                          ] LONG,PROT,X86_64 */
        /* 355 002(01 f8) : LONG,PROT,X86_64 */
        {I_SWAPGS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71934, 24, 1821},
    /* 1431 : LKGS rm_sel [m:        f2 0f 00 /6                       ] LONG,PRIV,PROT,LKGS,X86_64 */
        /* 332 355 001(00) 206 : LONG,PRIV,PROT,LKGS,X86_64 */
        {I_LKGS, 1, {RM_SEL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67617, 313, 1822},
    /* 1432 : LGDT mem16 [m:        nw o16 0f 01 /2                   ] AR0,OSIZE,PRIV,286 */
        /* 327 320 355 001(01) 202 : AR0,NWSIZE,OSIZE,PRIV,286 */
        {I_LGDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57196, 314, 1824},
    /* 1433 : LGDT mem32 [m:        nw o32 0f 01 /2                   ] AR0,OSIZE,PRIV,386 */
        /* 327 321 355 001(01) 202 : AR0,NWSIZE,OSIZE,PRIV,386 */
        {I_LGDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57203, 315, 1825},
    /* 1434 : LGDT mem64 [m:        nw o64 0f 01 /2                   ] AR0,OSIZE,LONG,PRIV,386,X86_64 */
        /* 327 324 355 001(01) 202 : AR0,NWSIZE,OSIZE,LONG,PRIV,PROT,X86_64 */
        {I_LGDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57210, 316, 1826},
    /* 1435 : LIDT mem16 [m:        nw o16 0f 01 /3                   ] AR0,OSIZE,PRIV,286 */
        /* 327 320 355 001(01) 203 : AR0,NWSIZE,OSIZE,PRIV,286 */
        {I_LIDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57217, 314, 1827},
    /* 1436 : LIDT mem32 [m:        nw o32 0f 01 /3                   ] AR0,OSIZE,PRIV,386 */
        /* 327 321 355 001(01) 203 : AR0,NWSIZE,OSIZE,PRIV,386 */
        {I_LIDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57224, 315, 1828},
    /* 1437 : LIDT mem64 [m:        nw o64 0f 01 /3                   ] AR0,OSIZE,LONG,PRIV,386,X86_64 */
        /* 327 324 355 001(01) 203 : AR0,NWSIZE,OSIZE,LONG,PRIV,PROT,X86_64 */
        {I_LIDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57231, 316, 1829},
    /* 1438 : LLDT mem16 [m:        0f 00 /2                          ] PRIV,PROT,286 */
        /* 355 001(00) 202 : PRIV,PROT,286 */
        {I_LLDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57254, 317, 1830},
    /* 1439 : LLDT reg16 [m:        optw o16 0f 00 /2                 ] PRIV,PROT,286 */
        /* 336 320 355 001(00) 202 : PRIV,PROT,286 */
        {I_LLDT, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57238, 317, 1831},
    /* 1440 : LLDT reg32 [m:        optw o32 0f 00 /2                 ] PRIV,PROT,386 */
        /* 336 321 355 001(00) 202 : PRIV,PROT,386 */
        {I_LLDT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57245, 318, 1832},
    /* 1441 : LLDT reg64 [m:        optw o64 0f 00 /2                 ] LONG,PRIV,PROT,386,X86_64 */
        /* 336 324 355 001(00) 202 : LONG,PRIV,PROT,X86_64 */
        {I_LLDT, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57252, 319, 1833},
    /* 1442 : LTR mem16 [m:        0f 00 /3                          ] PRIV,PROT,286 */
        /* 355 001(00) 203 : PRIV,PROT,286 */
        {I_LTR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57275, 317, 1834},
    /* 1443 : LTR reg16 [m:        optw o16 0f 00 /3                 ] PRIV,PROT,286 */
        /* 336 320 355 001(00) 203 : PRIV,PROT,286 */
        {I_LTR, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57259, 317, 1835},
    /* 1444 : LTR reg32 [m:        optw o32 0f 00 /3                 ] PRIV,PROT,386 */
        /* 336 321 355 001(00) 203 : PRIV,PROT,386 */
        {I_LTR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57266, 318, 1836},
    /* 1445 : LTR reg64 [m:        optw o64 0f 00 /3                 ] LONG,PRIV,PROT,386,X86_64 */
        /* 336 324 355 001(00) 203 : LONG,PRIV,PROT,X86_64 */
        {I_LTR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57273, 319, 1837},
    /* 1446 : SGDT mem16 [m:        nw o16 0f 01 /0                   ] AR0,OSIZE,286 */
        /* 327 320 355 001(01) 200 : AR0,NWSIZE,OSIZE,286 */
        {I_SGDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57280, 320, 1839},
    /* 1447 : SGDT mem32 [m:        nw o32 0f 01 /0                   ] AR0,OSIZE,386 */
        /* 327 321 355 001(01) 200 : AR0,NWSIZE,OSIZE,386 */
        {I_SGDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57287, 204, 1840},
    /* 1448 : SGDT mem64 [m:        nw o64 0f 01 /0                   ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 355 001(01) 200 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_SGDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57294, 205, 1841},
    /* 1449 : SIDT mem16 [m:        nw o16 0f 01 /1                   ] AR0,OSIZE,286 */
        /* 327 320 355 001(01) 201 : AR0,NWSIZE,OSIZE,286 */
        {I_SIDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57301, 320, 1842},
    /* 1450 : SIDT mem32 [m:        nw o32 0f 01 /1                   ] AR0,OSIZE,386 */
        /* 327 321 355 001(01) 201 : AR0,NWSIZE,OSIZE,386 */
        {I_SIDT, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57308, 204, 1843},
    /* 1451 : SIDT mem64 [m:        nw o64 0f 01 /1                   ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 355 001(01) 201 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_SIDT, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57315, 205, 1844},
    /* 1452 : SLDT mem16 [m:        0f 00 /0                          ] PROT,286 */
        /* 355 001(00) 200 : PROT,286 */
        {I_SLDT, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67624, 311, 1845},
    /* 1453 : SLDT reg16 [m:        o16 0f 00 /0                      ] PROT,286 */
        /* 320 355 001(00) 200 : PROT,286 */
        {I_SLDT, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67623, 311, 1846},
    /* 1454 : SLDT reg32 [m:        optd o32 0f 00 /0                 ] PROT,386 */
        /* 337 321 355 001(00) 200 : PROT,386 */
        {I_SLDT, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57322, 312, 1847},
    /* 1455 : SLDT reg64 [m:        optd o64 0f 00 /0                 ] LONG,PROT,386,X86_64 */
        /* 337 324 355 001(00) 200 : LONG,PROT,X86_64 */
        {I_SLDT, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57329, 24, 1848},
    /* 1456 : STR mem16 [m:        0f 00 /1                          ] PROT,286 */
        /* 355 001(00) 201 : PROT,286 */
        {I_STR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67630, 311, 1849},
    /* 1457 : STR reg16 [m:        o16 0f 00 /1                      ] PROT,286 */
        /* 320 355 001(00) 201 : PROT,286 */
        {I_STR, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67629, 311, 1850},
    /* 1458 : STR reg32 [m:        optd o32 0f 00 /1                 ] PROT,386 */
        /* 337 321 355 001(00) 201 : PROT,386 */
        {I_STR, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57336, 312, 1851},
    /* 1459 : STR reg64 [m:        optd o64 0f 00 /1                 ] LONG,PROT,386,X86_64 */
        /* 337 324 355 001(00) 201 : LONG,PROT,X86_64 */
        {I_STR, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57343, 24, 1852},
    /* 1460 : F2XM1 void [          d9 f0                             ] FPU,8086 */
        /* 002(d9 f0) : FPU,8086 */
        {I_F2XM1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72952, 323, 1858},
    /* 1461 : FABS void [          d9 e1                             ] FPU,8086 */
        /* 002(d9 e1) : FPU,8086 */
        {I_FABS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72956, 323, 1859},
    /* 1462 : FADD mem32 [m:        d8 /0                             ] FPU,8086 */
        /* 001(d8) 200 : FPU,8086 */
        {I_FADD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69953, 323, 1860},
    /* 1463 : FADD mem64 [m:        dc /0                             ] FPU,8086 */
        /* 001(dc) 200 : FPU,8086 */
        {I_FADD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72960, 323, 1861},
    /* 1464 : FADD fpureg|to [r:        dc c0+r                           ] FPU,8086 */
        /* 001(dc) 010 300 : FPU,8086 */
        {I_FADD, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71939, 323, 1862},
    /* 1465 : FADD fpureg [r:        d8 c0+r                           ] FPU,8086 */
        /* 001(d8) 010 300 : FPU,8086 */
        {I_FADD, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71944, 323, 1863},
    /* 1466 : FADD fpureg,fpu0 [r-:       dc c0+r                           ] FPU,8086 */
        /* 001(dc) 010 300 : FPU,8086 */
        {I_FADD, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71939, 323, 1864},
    /* 1467 : FADD fpu0,fpureg [-r:       d8 c0+r                           ] FPU,8086 */
        /* 001(d8) 011 300 : FPU,8086 */
        {I_FADD, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71949, 323, 1865},
    /* 1468 : FADDP fpureg [r:        de c0+r                           ] FPU,8086 */
        /* 001(de) 010 300 : FPU,8086 */
        {I_FADDP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71954, 323, 1867},
    /* 1469 : FADDP fpureg,fpu0 [r-:       de c0+r                           ] FPU,8086 */
        /* 001(de) 010 300 : FPU,8086 */
        {I_FADDP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+71954, 323, 1868},
    /* 1470 : FBLD mem80 [m:        df /4                             ] FPU,8086 */
        /* 001(df) 204 : FPU,8086 */
        {I_FBLD, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72968, 323, 1870},
    /* 1471 : FBLD mem [m:        df /4                             ] FPU,8086 */
        /* 001(df) 204 : FPU,8086 */
        {I_FBLD, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72968, 323, 1871},
    /* 1472 : FBSTP mem80 [m:        df /6                             ] FPU,8086 */
        /* 001(df) 206 : FPU,8086 */
        {I_FBSTP, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72972, 323, 1872},
    /* 1473 : FBSTP mem [m:        df /6                             ] FPU,8086 */
        /* 001(df) 206 : FPU,8086 */
        {I_FBSTP, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72972, 323, 1873},
    /* 1474 : FCHS void [          d9 e0                             ] FPU,8086 */
        /* 002(d9 e0) : FPU,8086 */
        {I_FCHS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72976, 323, 1874},
    /* 1475 : FCLEX void [          wait db e2                        ] FPU,8086 */
        /* 341 002(db e2) : FPU,8086 */
        {I_FCLEX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71959, 323, 1875},
    /* 1476 : FCMOVB fpureg [r:        da c0+r                           ] FPU,P6 */
        /* 001(da) 010 300 : FPU,P6 */
        {I_FCMOVB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71964, 324, 1876},
    /* 1477 : FCMOVB fpu0,fpureg [-r:       da c0+r                           ] FPU,P6 */
        /* 001(da) 011 300 : FPU,P6 */
        {I_FCMOVB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71969, 324, 1877},
    /* 1478 : FCMOVBE fpureg [r:        da d0+r                           ] FPU,P6 */
        /* 001(da) 010 320 : FPU,P6 */
        {I_FCMOVBE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71974, 324, 1879},
    /* 1479 : FCMOVBE fpu0,fpureg [-r:       da d0+r                           ] FPU,P6 */
        /* 001(da) 011 320 : FPU,P6 */
        {I_FCMOVBE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71979, 324, 1880},
    /* 1480 : FCMOVE fpureg [r:        da c8+r                           ] FPU,P6 */
        /* 001(da) 010 310 : FPU,P6 */
        {I_FCMOVE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71984, 324, 1882},
    /* 1481 : FCMOVE fpu0,fpureg [-r:       da c8+r                           ] FPU,P6 */
        /* 001(da) 011 310 : FPU,P6 */
        {I_FCMOVE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71989, 324, 1883},
    /* 1482 : FCMOVNB fpureg [r:        db c0+r                           ] FPU,P6 */
        /* 001(db) 010 300 : FPU,P6 */
        {I_FCMOVNB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71994, 324, 1885},
    /* 1483 : FCMOVNB fpu0,fpureg [-r:       db c0+r                           ] FPU,P6 */
        /* 001(db) 011 300 : FPU,P6 */
        {I_FCMOVNB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+71999, 324, 1886},
    /* 1484 : FCMOVNBE fpureg [r:        db d0+r                           ] FPU,P6 */
        /* 001(db) 010 320 : FPU,P6 */
        {I_FCMOVNBE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72004, 324, 1888},
    /* 1485 : FCMOVNBE fpu0,fpureg [-r:       db d0+r                           ] FPU,P6 */
        /* 001(db) 011 320 : FPU,P6 */
        {I_FCMOVNBE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72009, 324, 1889},
    /* 1486 : FCMOVNE fpureg [r:        db c8+r                           ] FPU,P6 */
        /* 001(db) 010 310 : FPU,P6 */
        {I_FCMOVNE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72014, 324, 1891},
    /* 1487 : FCMOVNE fpu0,fpureg [-r:       db c8+r                           ] FPU,P6 */
        /* 001(db) 011 310 : FPU,P6 */
        {I_FCMOVNE, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72019, 324, 1892},
    /* 1488 : FCMOVNU fpureg [r:        db d8+r                           ] FPU,P6 */
        /* 001(db) 010 330 : FPU,P6 */
        {I_FCMOVNU, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72024, 324, 1894},
    /* 1489 : FCMOVNU fpu0,fpureg [-r:       db d8+r                           ] FPU,P6 */
        /* 001(db) 011 330 : FPU,P6 */
        {I_FCMOVNU, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72029, 324, 1895},
    /* 1490 : FCMOVU fpureg [r:        da d8+r                           ] FPU,P6 */
        /* 001(da) 010 330 : FPU,P6 */
        {I_FCMOVU, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72034, 324, 1897},
    /* 1491 : FCMOVU fpu0,fpureg [-r:       da d8+r                           ] FPU,P6 */
        /* 001(da) 011 330 : FPU,P6 */
        {I_FCMOVU, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72039, 324, 1898},
    /* 1492 : FCOM mem32 [m:        d8 /2                             ] FPU,8086 */
        /* 001(d8) 202 : FPU,8086 */
        {I_FCOM, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69959, 323, 1900},
    /* 1493 : FCOM mem64 [m:        dc /2                             ] FPU,8086 */
        /* 001(dc) 202 : FPU,8086 */
        {I_FCOM, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73012, 323, 1901},
    /* 1494 : FCOM fpureg [r:        d8 d0+r                           ] FPU,8086 */
        /* 001(d8) 010 320 : FPU,8086 */
        {I_FCOM, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72044, 323, 1902},
    /* 1495 : FCOM fpu0,fpureg [-r:       d8 d0+r                           ] FPU,8086 */
        /* 001(d8) 011 320 : FPU,8086 */
        {I_FCOM, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72049, 323, 1903},
    /* 1496 : FCOMI fpureg [r:        db f0+r                           ] FL,FPU,P6 */
        /* 001(db) 010 360 : FL,FPU,P6 */
        {I_FCOMI, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72054, 325, 1905},
    /* 1497 : FCOMI fpu0,fpureg [-r:       db f0+r                           ] FL,FPU,P6 */
        /* 001(db) 011 360 : FL,FPU,P6 */
        {I_FCOMI, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72059, 325, 1906},
    /* 1498 : FCOMIP fpureg [r:        df f0+r                           ] FL,FPU,P6 */
        /* 001(df) 010 360 : FL,FPU,P6 */
        {I_FCOMIP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72064, 325, 1908},
    /* 1499 : FCOMIP fpu0,fpureg [-r:       df f0+r                           ] FL,FPU,P6 */
        /* 001(df) 011 360 : FL,FPU,P6 */
        {I_FCOMIP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72069, 325, 1909},
    /* 1500 : FCOMP mem32 [m:        d8 /3                             ] FPU,8086 */
        /* 001(d8) 203 : FPU,8086 */
        {I_FCOMP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69947, 323, 1911},
    /* 1501 : FCOMP mem64 [m:        dc /3                             ] FPU,8086 */
        /* 001(dc) 203 : FPU,8086 */
        {I_FCOMP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73028, 323, 1912},
    /* 1502 : FCOMP fpureg [r:        d8 d8+r                           ] FPU,8086 */
        /* 001(d8) 010 330 : FPU,8086 */
        {I_FCOMP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72074, 323, 1913},
    /* 1503 : FCOMP fpu0,fpureg [-r:       d8 d8+r                           ] FPU,8086 */
        /* 001(d8) 011 330 : FPU,8086 */
        {I_FCOMP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72079, 323, 1914},
    /* 1504 : FCOMPP void [          de d9                             ] FPU,8086 */
        /* 002(de d9) : FPU,8086 */
        {I_FCOMPP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73036, 323, 1916},
    /* 1505 : FCOS void [          d9 ff                             ] FPU,386 */
        /* 002(d9 ff) : FPU,386 */
        {I_FCOS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73040, 326, 1917},
    /* 1506 : FDECSTP void [          d9 f6                             ] FPU,8086 */
        /* 002(d9 f6) : FPU,8086 */
        {I_FDECSTP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73044, 323, 1918},
    /* 1507 : FDISI void [          wait db e1                        ] FPU,8086 */
        /* 341 002(db e1) : FPU,8086 */
        {I_FDISI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72084, 323, 1919},
    /* 1508 : FDIV mem32 [m:        d8 /6                             ] FPU,8086 */
        /* 001(d8) 206 : FPU,8086 */
        {I_FDIV, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73048, 323, 1920},
    /* 1509 : FDIV mem64 [m:        dc /6                             ] FPU,8086 */
        /* 001(dc) 206 : FPU,8086 */
        {I_FDIV, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73052, 323, 1921},
    /* 1510 : FDIV fpureg|to [r:        dc f8+r                           ] FPU,8086 */
        /* 001(dc) 010 370 : FPU,8086 */
        {I_FDIV, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72089, 323, 1922},
    /* 1511 : FDIV fpureg [r:        d8 f0+r                           ] FPU,8086 */
        /* 001(d8) 010 360 : FPU,8086 */
        {I_FDIV, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72094, 323, 1923},
    /* 1512 : FDIV fpureg,fpu0 [r-:       dc f8+r                           ] FPU,8086 */
        /* 001(dc) 010 370 : FPU,8086 */
        {I_FDIV, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72089, 323, 1924},
    /* 1513 : FDIV fpu0,fpureg [-r:       d8 f0+r                           ] FPU,8086 */
        /* 001(d8) 011 360 : FPU,8086 */
        {I_FDIV, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72099, 323, 1925},
    /* 1514 : FDIVP fpureg [r:        de f8+r                           ] FPU,8086 */
        /* 001(de) 010 370 : FPU,8086 */
        {I_FDIVP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72104, 323, 1927},
    /* 1515 : FDIVP fpureg,fpu0 [r-:       de f8+r                           ] FPU,8086 */
        /* 001(de) 010 370 : FPU,8086 */
        {I_FDIVP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72104, 323, 1928},
    /* 1516 : FDIVR mem32 [m:        d8 /7                             ] FPU,8086 */
        /* 001(d8) 207 : FPU,8086 */
        {I_FDIVR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73060, 323, 1930},
    /* 1517 : FDIVR mem64 [m:        dc /7                             ] FPU,8086 */
        /* 001(dc) 207 : FPU,8086 */
        {I_FDIVR, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73064, 323, 1931},
    /* 1518 : FDIVR fpureg|to [r:        dc f0+r                           ] FPU,8086 */
        /* 001(dc) 010 360 : FPU,8086 */
        {I_FDIVR, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72109, 323, 1932},
    /* 1519 : FDIVR fpureg,fpu0 [r-:       dc f0+r                           ] FPU,8086 */
        /* 001(dc) 010 360 : FPU,8086 */
        {I_FDIVR, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72109, 323, 1933},
    /* 1520 : FDIVR fpureg [r:        d8 f8+r                           ] FPU,8086 */
        /* 001(d8) 010 370 : FPU,8086 */
        {I_FDIVR, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72114, 323, 1934},
    /* 1521 : FDIVR fpu0,fpureg [-r:       d8 f8+r                           ] FPU,8086 */
        /* 001(d8) 011 370 : FPU,8086 */
        {I_FDIVR, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72119, 323, 1935},
    /* 1522 : FDIVRP fpureg [r:        de f0+r                           ] FPU,8086 */
        /* 001(de) 010 360 : FPU,8086 */
        {I_FDIVRP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72124, 323, 1937},
    /* 1523 : FDIVRP fpureg,fpu0 [r-:       de f0+r                           ] FPU,8086 */
        /* 001(de) 010 360 : FPU,8086 */
        {I_FDIVRP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72124, 323, 1938},
    /* 1524 : FEMMS void [          0f 0e                             ] 3DNOW,PENT */
        /* 355 001(0e) : 3DNOW,PENT */
        {I_FEMMS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73072, 327, 1940},
    /* 1525 : FENI void [          wait db e0                        ] FPU,8086 */
        /* 341 002(db e0) : FPU,8086 */
        {I_FENI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72129, 323, 1941},
    /* 1526 : FFREE fpureg [r:        dd c0+r                           ] FPU,8086 */
        /* 001(dd) 010 300 : FPU,8086 */
        {I_FFREE, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72134, 323, 1942},
    /* 1527 : FFREE void [          dd c1                             ] FPU,8086 */
        /* 002(dd c1) : FPU,8086 */
        {I_FFREE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73076, 323, 1943},
    /* 1528 : FFREEP fpureg [r:        df c0+r                           ] UNDOC,FPU,286 */
        /* 001(df) 010 300 : UNDOC,FPU,286 */
        {I_FFREEP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72139, 328, 1944},
    /* 1529 : FFREEP void [          df c1                             ] UNDOC,FPU,286 */
        /* 002(df c1) : UNDOC,FPU,286 */
        {I_FFREEP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73080, 328, 1945},
    /* 1530 : FIADD mem32 [m:        da /0                             ] FPU,8086 */
        /* 001(da) 200 : FPU,8086 */
        {I_FIADD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73084, 323, 1946},
    /* 1531 : FIADD mem16 [m:        de /0                             ] FPU,8086 */
        /* 001(de) 200 : FPU,8086 */
        {I_FIADD, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73088, 323, 1947},
    /* 1532 : FICOM mem32 [m:        da /2                             ] FPU,8086 */
        /* 001(da) 202 : FPU,8086 */
        {I_FICOM, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73092, 323, 1948},
    /* 1533 : FICOM mem16 [m:        de /2                             ] FPU,8086 */
        /* 001(de) 202 : FPU,8086 */
        {I_FICOM, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73096, 323, 1949},
    /* 1534 : FICOMP mem32 [m:        da /3                             ] FPU,8086 */
        /* 001(da) 203 : FPU,8086 */
        {I_FICOMP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73100, 323, 1950},
    /* 1535 : FICOMP mem16 [m:        de /3                             ] FPU,8086 */
        /* 001(de) 203 : FPU,8086 */
        {I_FICOMP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73104, 323, 1951},
    /* 1536 : FIDIV mem32 [m:        da /6                             ] FPU,8086 */
        /* 001(da) 206 : FPU,8086 */
        {I_FIDIV, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73108, 323, 1952},
    /* 1537 : FIDIV mem16 [m:        de /6                             ] FPU,8086 */
        /* 001(de) 206 : FPU,8086 */
        {I_FIDIV, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73112, 323, 1953},
    /* 1538 : FIDIVR mem32 [m:        da /7                             ] FPU,8086 */
        /* 001(da) 207 : FPU,8086 */
        {I_FIDIVR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73116, 323, 1954},
    /* 1539 : FIDIVR mem16 [m:        de /7                             ] FPU,8086 */
        /* 001(de) 207 : FPU,8086 */
        {I_FIDIVR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73120, 323, 1955},
    /* 1540 : FILD mem32 [m:        db /0                             ] FPU,8086 */
        /* 001(db) 200 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73124, 323, 1956},
    /* 1541 : FILD mem16 [m:        df /0                             ] FPU,8086 */
        /* 001(df) 200 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73128, 323, 1957},
    /* 1542 : FILD mem64 [m:        df /5                             ] FPU,8086 */
        /* 001(df) 205 : FPU,8086 */
        {I_FILD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73132, 323, 1958},
    /* 1543 : FIMUL mem32 [m:        da /1                             ] FPU,8086 */
        /* 001(da) 201 : FPU,8086 */
        {I_FIMUL, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73136, 323, 1959},
    /* 1544 : FIMUL mem16 [m:        de /1                             ] FPU,8086 */
        /* 001(de) 201 : FPU,8086 */
        {I_FIMUL, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73140, 323, 1960},
    /* 1545 : FINCSTP void [          d9 f7                             ] FPU,8086 */
        /* 002(d9 f7) : FPU,8086 */
        {I_FINCSTP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73144, 323, 1961},
    /* 1546 : FINIT void [          wait db e3                        ] FPU,8086 */
        /* 341 002(db e3) : FPU,8086 */
        {I_FINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72144, 323, 1962},
    /* 1547 : FIST mem32 [m:        db /2                             ] FPU,8086 */
        /* 001(db) 202 : FPU,8086 */
        {I_FIST, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73148, 323, 1963},
    /* 1548 : FIST mem16 [m:        df /2                             ] FPU,8086 */
        /* 001(df) 202 : FPU,8086 */
        {I_FIST, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73152, 323, 1964},
    /* 1549 : FISTP mem32 [m:        db /3                             ] FPU,8086 */
        /* 001(db) 203 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73156, 323, 1965},
    /* 1550 : FISTP mem16 [m:        df /3                             ] FPU,8086 */
        /* 001(df) 203 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73160, 323, 1966},
    /* 1551 : FISTP mem64 [m:        df /7                             ] FPU,8086 */
        /* 001(df) 207 : FPU,8086 */
        {I_FISTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73164, 323, 1967},
    /* 1552 : FISTTP mem16 [m:        df /1                             ] FPU,PRESCOTT */
        /* 001(df) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73168, 329, 1968},
    /* 1553 : FISTTP mem32 [m:        db /1                             ] FPU,PRESCOTT */
        /* 001(db) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73172, 329, 1969},
    /* 1554 : FISTTP mem64 [m:        dd /1                             ] FPU,PRESCOTT */
        /* 001(dd) 201 : FPU,PRESCOTT */
        {I_FISTTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73176, 329, 1970},
    /* 1555 : FISUB mem32 [m:        da /4                             ] FPU,8086 */
        /* 001(da) 204 : FPU,8086 */
        {I_FISUB, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73180, 323, 1971},
    /* 1556 : FISUB mem16 [m:        de /4                             ] FPU,8086 */
        /* 001(de) 204 : FPU,8086 */
        {I_FISUB, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73184, 323, 1972},
    /* 1557 : FISUBR mem32 [m:        da /5                             ] FPU,8086 */
        /* 001(da) 205 : FPU,8086 */
        {I_FISUBR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73188, 323, 1973},
    /* 1558 : FISUBR mem16 [m:        de /5                             ] FPU,8086 */
        /* 001(de) 205 : FPU,8086 */
        {I_FISUBR, 1, {MEMORY|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73192, 323, 1974},
    /* 1559 : FLD mem32 [m:        d9 /0                             ] FPU,8086 */
        /* 001(d9) 200 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73196, 323, 1975},
    /* 1560 : FLD mem64 [m:        dd /0                             ] FPU,8086 */
        /* 001(dd) 200 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73200, 323, 1976},
    /* 1561 : FLD mem80 [m:        db /5                             ] FPU,8086 */
        /* 001(db) 205 : FPU,8086 */
        {I_FLD, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73204, 323, 1977},
    /* 1562 : FLD fpureg [r:        d9 c0+r                           ] FPU,8086 */
        /* 001(d9) 010 300 : FPU,8086 */
        {I_FLD, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72149, 323, 1978},
    /* 1563 : FLD1 void [          d9 e8                             ] FPU,8086 */
        /* 002(d9 e8) : FPU,8086 */
        {I_FLD1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73212, 323, 1980},
    /* 1564 : FLDCW mem [m:        d9 /5                             ] AR0,SW,FPU,8086 */
        /* 001(d9) 205 : AR0,SW,FPU,8086 */
        {I_FLDCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73216, 330, 1981},
    /* 1565 : FLDENV mem [m:        d9 /4                             ] FPU,8086 */
        /* 001(d9) 204 : FPU,8086 */
        {I_FLDENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73220, 323, 1982},
    /* 1566 : FLDL2E void [          d9 ea                             ] FPU,8086 */
        /* 002(d9 ea) : FPU,8086 */
        {I_FLDL2E, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73224, 323, 1983},
    /* 1567 : FLDL2T void [          d9 e9                             ] FPU,8086 */
        /* 002(d9 e9) : FPU,8086 */
        {I_FLDL2T, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73228, 323, 1984},
    /* 1568 : FLDLG2 void [          d9 ec                             ] FPU,8086 */
        /* 002(d9 ec) : FPU,8086 */
        {I_FLDLG2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73232, 323, 1985},
    /* 1569 : FLDLN2 void [          d9 ed                             ] FPU,8086 */
        /* 002(d9 ed) : FPU,8086 */
        {I_FLDLN2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73236, 323, 1986},
    /* 1570 : FLDPI void [          d9 eb                             ] FPU,8086 */
        /* 002(d9 eb) : FPU,8086 */
        {I_FLDPI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73240, 323, 1987},
    /* 1571 : FLDZ void [          d9 ee                             ] FPU,8086 */
        /* 002(d9 ee) : FPU,8086 */
        {I_FLDZ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73244, 323, 1988},
    /* 1572 : FMUL mem32 [m:        d8 /1                             ] FPU,8086 */
        /* 001(d8) 201 : FPU,8086 */
        {I_FMUL, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69941, 323, 1989},
    /* 1573 : FMUL mem64 [m:        dc /1                             ] FPU,8086 */
        /* 001(dc) 201 : FPU,8086 */
        {I_FMUL, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73248, 323, 1990},
    /* 1574 : FMUL fpureg|to [r:        dc c8+r                           ] FPU,8086 */
        /* 001(dc) 010 310 : FPU,8086 */
        {I_FMUL, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72154, 323, 1991},
    /* 1575 : FMUL fpureg,fpu0 [r-:       dc c8+r                           ] FPU,8086 */
        /* 001(dc) 010 310 : FPU,8086 */
        {I_FMUL, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72154, 323, 1992},
    /* 1576 : FMUL fpureg [r:        d8 c8+r                           ] FPU,8086 */
        /* 001(d8) 010 310 : FPU,8086 */
        {I_FMUL, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72159, 323, 1993},
    /* 1577 : FMUL fpu0,fpureg [-r:       d8 c8+r                           ] FPU,8086 */
        /* 001(d8) 011 310 : FPU,8086 */
        {I_FMUL, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72164, 323, 1994},
    /* 1578 : FMULP fpureg [r:        de c8+r                           ] FPU,8086 */
        /* 001(de) 010 310 : FPU,8086 */
        {I_FMULP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72169, 323, 1996},
    /* 1579 : FMULP fpureg,fpu0 [r-:       de c8+r                           ] FPU,8086 */
        /* 001(de) 010 310 : FPU,8086 */
        {I_FMULP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72169, 323, 1997},
    /* 1580 : FNCLEX void [          db e2                             ] FPU,8086 */
        /* 002(db e2) : FPU,8086 */
        {I_FNCLEX, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+71960, 323, 1999},
    /* 1581 : FNDISI void [          db e1                             ] FPU,8086 */
        /* 002(db e1) : FPU,8086 */
        {I_FNDISI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72085, 323, 2000},
    /* 1582 : FNENI void [          db e0                             ] FPU,8086 */
        /* 002(db e0) : FPU,8086 */
        {I_FNENI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72130, 323, 2001},
    /* 1583 : FNINIT void [          db e3                             ] FPU,8086 */
        /* 002(db e3) : FPU,8086 */
        {I_FNINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72145, 323, 2002},
    /* 1584 : FNOP void [          d9 d0                             ] FPU,8086 */
        /* 002(d9 d0) : FPU,8086 */
        {I_FNOP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73256, 323, 2003},
    /* 1585 : FNSAVE mem [m:        dd /6                             ] FPU,8086 */
        /* 001(dd) 206 : FPU,8086 */
        {I_FNSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72175, 323, 2004},
    /* 1586 : FNSTCW mem [m:        d9 /7                             ] AR0,SW,FPU,8086 */
        /* 001(d9) 207 : AR0,SW,FPU,8086 */
        {I_FNSTCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72185, 330, 2005},
    /* 1587 : FNSTENV mem [m:        d9 /6                             ] FPU,8086 */
        /* 001(d9) 206 : FPU,8086 */
        {I_FNSTENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72190, 323, 2006},
    /* 1588 : FNSTSW mem [m:        dd /7                             ] AR0,SW,FPU,8086 */
        /* 001(dd) 207 : AR0,SW,FPU,8086 */
        {I_FNSTSW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72200, 330, 2007},
    /* 1589 : FNSTSW reg_ax [-:        df e0                             ] FPU,286 */
        /* 002(df e0) : FPU,286 */
        {I_FNSTSW, 1, {REG_AX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72205, 331, 2008},
    /* 1590 : FPATAN void [          d9 f3                             ] FPU,8086 */
        /* 002(d9 f3) : FPU,8086 */
        {I_FPATAN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73260, 323, 2009},
    /* 1591 : FPREM void [          d9 f8                             ] FPU,8086 */
        /* 002(d9 f8) : FPU,8086 */
        {I_FPREM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73264, 323, 2010},
    /* 1592 : FPREM1 void [          d9 f5                             ] FPU,386 */
        /* 002(d9 f5) : FPU,386 */
        {I_FPREM1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73268, 326, 2011},
    /* 1593 : FPTAN void [          d9 f2                             ] FPU,8086 */
        /* 002(d9 f2) : FPU,8086 */
        {I_FPTAN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73272, 323, 2012},
    /* 1594 : FRNDINT void [          d9 fc                             ] FPU,8086 */
        /* 002(d9 fc) : FPU,8086 */
        {I_FRNDINT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73276, 323, 2013},
    /* 1595 : FRSTOR mem [m:        dd /4                             ] FPU,8086 */
        /* 001(dd) 204 : FPU,8086 */
        {I_FRSTOR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73280, 323, 2014},
    /* 1596 : FSAVE mem [m:        wait dd /6                        ] FPU,8086 */
        /* 341 001(dd) 206 : FPU,8086 */
        {I_FSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72174, 323, 2015},
    /* 1597 : FSCALE void [          d9 fd                             ] FPU,8086 */
        /* 002(d9 fd) : FPU,8086 */
        {I_FSCALE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73284, 323, 2016},
    /* 1598 : FSETPM void [          db e4                             ] FPU,286 */
        /* 002(db e4) : FPU,286 */
        {I_FSETPM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73288, 331, 2017},
    /* 1599 : FSIN void [          d9 fe                             ] FPU,386 */
        /* 002(d9 fe) : FPU,386 */
        {I_FSIN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73292, 326, 2018},
    /* 1600 : FSINCOS void [          d9 fb                             ] FPU,386 */
        /* 002(d9 fb) : FPU,386 */
        {I_FSINCOS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73296, 326, 2019},
    /* 1601 : FSQRT void [          d9 fa                             ] FPU,8086 */
        /* 002(d9 fa) : FPU,8086 */
        {I_FSQRT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73300, 323, 2020},
    /* 1602 : FST mem32 [m:        d9 /2                             ] FPU,8086 */
        /* 001(d9) 202 : FPU,8086 */
        {I_FST, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73304, 323, 2021},
    /* 1603 : FST mem64 [m:        dd /2                             ] FPU,8086 */
        /* 001(dd) 202 : FPU,8086 */
        {I_FST, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73308, 323, 2022},
    /* 1604 : FST fpureg [r:        dd d0+r                           ] FPU,8086 */
        /* 001(dd) 010 320 : FPU,8086 */
        {I_FST, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72179, 323, 2023},
    /* 1605 : FSTCW mem [m:        wait d9 /7                        ] AR0,SW,FPU,8086 */
        /* 341 001(d9) 207 : AR0,SW,FPU,8086 */
        {I_FSTCW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72184, 330, 2025},
    /* 1606 : FSTENV mem [m:        wait d9 /6                        ] FPU,8086 */
        /* 341 001(d9) 206 : FPU,8086 */
        {I_FSTENV, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72189, 323, 2026},
    /* 1607 : FSTP mem32 [m:        d9 /3                             ] FPU,8086 */
        /* 001(d9) 203 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73316, 323, 2027},
    /* 1608 : FSTP mem64 [m:        dd /3                             ] FPU,8086 */
        /* 001(dd) 203 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73320, 323, 2028},
    /* 1609 : FSTP mem80 [m:        db /7                             ] FPU,8086 */
        /* 001(db) 207 : FPU,8086 */
        {I_FSTP, 1, {MEMORY|BITS80,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73324, 323, 2029},
    /* 1610 : FSTP fpureg [r:        dd d8+r                           ] FPU,8086 */
        /* 001(dd) 010 330 : FPU,8086 */
        {I_FSTP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72194, 323, 2030},
    /* 1611 : FSTSW mem [m:        wait dd /7                        ] AR0,SW,FPU,8086 */
        /* 341 001(dd) 207 : AR0,SW,FPU,8086 */
        {I_FSTSW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72199, 330, 2032},
    /* 1612 : FSTSW reg_ax [-:        wait df e0                        ] FPU,286 */
        /* 341 002(df e0) : FPU,286 */
        {I_FSTSW, 1, {REG_AX,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72204, 331, 2033},
    /* 1613 : FSUB mem32 [m:        d8 /4                             ] FPU,8086 */
        /* 001(d8) 204 : FPU,8086 */
        {I_FSUB, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73332, 323, 2034},
    /* 1614 : FSUB mem64 [m:        dc /4                             ] FPU,8086 */
        /* 001(dc) 204 : FPU,8086 */
        {I_FSUB, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73336, 323, 2035},
    /* 1615 : FSUB fpureg|to [r:        dc e8+r                           ] FPU,8086 */
        /* 001(dc) 010 350 : FPU,8086 */
        {I_FSUB, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72209, 323, 2036},
    /* 1616 : FSUB fpureg,fpu0 [r-:       dc e8+r                           ] FPU,8086 */
        /* 001(dc) 010 350 : FPU,8086 */
        {I_FSUB, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72209, 323, 2037},
    /* 1617 : FSUB fpureg [r:        d8 e0+r                           ] FPU,8086 */
        /* 001(d8) 010 340 : FPU,8086 */
        {I_FSUB, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72214, 323, 2038},
    /* 1618 : FSUB fpu0,fpureg [-r:       d8 e0+r                           ] FPU,8086 */
        /* 001(d8) 011 340 : FPU,8086 */
        {I_FSUB, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72219, 323, 2039},
    /* 1619 : FSUBP fpureg [r:        de e8+r                           ] FPU,8086 */
        /* 001(de) 010 350 : FPU,8086 */
        {I_FSUBP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72224, 323, 2041},
    /* 1620 : FSUBP fpureg,fpu0 [r-:       de e8+r                           ] FPU,8086 */
        /* 001(de) 010 350 : FPU,8086 */
        {I_FSUBP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72224, 323, 2042},
    /* 1621 : FSUBR mem32 [m:        d8 /5                             ] FPU,8086 */
        /* 001(d8) 205 : FPU,8086 */
        {I_FSUBR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73344, 323, 2044},
    /* 1622 : FSUBR mem64 [m:        dc /5                             ] FPU,8086 */
        /* 001(dc) 205 : FPU,8086 */
        {I_FSUBR, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73348, 323, 2045},
    /* 1623 : FSUBR fpureg|to [r:        dc e0+r                           ] FPU,8086 */
        /* 001(dc) 010 340 : FPU,8086 */
        {I_FSUBR, 1, {FPUREG|TO,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72229, 323, 2046},
    /* 1624 : FSUBR fpureg,fpu0 [r-:       dc e0+r                           ] FPU,8086 */
        /* 001(dc) 010 340 : FPU,8086 */
        {I_FSUBR, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72229, 323, 2047},
    /* 1625 : FSUBR fpureg [r:        d8 e8+r                           ] FPU,8086 */
        /* 001(d8) 010 350 : FPU,8086 */
        {I_FSUBR, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72234, 323, 2048},
    /* 1626 : FSUBR fpu0,fpureg [-r:       d8 e8+r                           ] FPU,8086 */
        /* 001(d8) 011 350 : FPU,8086 */
        {I_FSUBR, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72239, 323, 2049},
    /* 1627 : FSUBRP fpureg [r:        de e0+r                           ] FPU,8086 */
        /* 001(de) 010 340 : FPU,8086 */
        {I_FSUBRP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72244, 323, 2051},
    /* 1628 : FSUBRP fpureg,fpu0 [r-:       de e0+r                           ] FPU,8086 */
        /* 001(de) 010 340 : FPU,8086 */
        {I_FSUBRP, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72244, 323, 2052},
    /* 1629 : FTST void [          d9 e4                             ] FPU,8086 */
        /* 002(d9 e4) : FPU,8086 */
        {I_FTST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73356, 323, 2054},
    /* 1630 : FUCOM fpureg [r:        dd e0+r                           ] FPU,386 */
        /* 001(dd) 010 340 : FPU,386 */
        {I_FUCOM, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72249, 326, 2055},
    /* 1631 : FUCOM fpu0,fpureg [-r:       dd e0+r                           ] FPU,386 */
        /* 001(dd) 011 340 : FPU,386 */
        {I_FUCOM, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72254, 326, 2056},
    /* 1632 : FUCOMI fpureg [r:        db e8+r                           ] FL,FPU,P6 */
        /* 001(db) 010 350 : FL,FPU,P6 */
        {I_FUCOMI, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72259, 325, 2058},
    /* 1633 : FUCOMI fpu0,fpureg [-r:       db e8+r                           ] FL,FPU,P6 */
        /* 001(db) 011 350 : FL,FPU,P6 */
        {I_FUCOMI, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72264, 325, 2059},
    /* 1634 : FUCOMIP fpureg [r:        df e8+r                           ] FL,FPU,P6 */
        /* 001(df) 010 350 : FL,FPU,P6 */
        {I_FUCOMIP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72269, 325, 2061},
    /* 1635 : FUCOMIP fpu0,fpureg [-r:       df e8+r                           ] FL,FPU,P6 */
        /* 001(df) 011 350 : FL,FPU,P6 */
        {I_FUCOMIP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72274, 325, 2062},
    /* 1636 : FUCOMP fpureg [r:        dd e8+r                           ] FPU,386 */
        /* 001(dd) 010 350 : FPU,386 */
        {I_FUCOMP, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72279, 326, 2064},
    /* 1637 : FUCOMP fpu0,fpureg [-r:       dd e8+r                           ] FPU,386 */
        /* 001(dd) 011 350 : FPU,386 */
        {I_FUCOMP, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72284, 326, 2065},
    /* 1638 : FUCOMPP void [          da e9                             ] FPU,386 */
        /* 002(da e9) : FPU,386 */
        {I_FUCOMPP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73376, 326, 2067},
    /* 1639 : FXAM void [          d9 e5                             ] FPU,8086 */
        /* 002(d9 e5) : FPU,8086 */
        {I_FXAM, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73380, 323, 2068},
    /* 1640 : FXCH fpureg [r:        d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 010 310 : FPU,8086 */
        {I_FXCH, 1, {FPUREG,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72289, 323, 2069},
    /* 1641 : FXCH fpureg,fpu0 [r-:       d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 010 310 : FPU,8086 */
        {I_FXCH, 2, {FPUREG,FPUREG|RN_ZERO,0,0,0}, NO_DECORATOR, nasm_bytecodes+72289, 323, 2070},
    /* 1642 : FXCH fpu0,fpureg [-r:       d9 c8+r                           ] FPU,8086 */
        /* 001(d9) 011 310 : FPU,8086 */
        {I_FXCH, 2, {FPUREG|RN_ZERO,FPUREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+72294, 323, 2071},
    /* 1643 : FXTRACT void [          d9 f4                             ] FPU,8086 */
        /* 002(d9 f4) : FPU,8086 */
        {I_FXTRACT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73388, 323, 2073},
    /* 1644 : FYL2X void [          d9 f1                             ] FPU,8086 */
        /* 002(d9 f1) : FPU,8086 */
        {I_FYL2X, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73392, 323, 2074},
    /* 1645 : FYL2XP1 void [          d9 f9                             ] FPU,8086 */
        /* 002(d9 f9) : FPU,8086 */
        {I_FYL2XP1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73396, 323, 2075},
    /* 1646 : EMMS void [          0f 77                             ] MMX,PENT */
        /* 355 001(77) : MMX,PENT */
        {I_EMMS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73400, 332, 2078},
    /* 1647 : MOVD mmxreg,rm32 [rm:       np 0f 6e /r                       ] MMX,PENT */
        /* 360 355 001(6e) 110 : MMX,PENT */
        {I_MOVD, 2, {MMXREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+67635, 332, 2079},
    /* 1648 : MOVD rm32,mmxreg [mr:       np 0f 7e /r                       ] MMX,PENT */
        /* 360 355 001(7e) 101 : MMX,PENT */
        {I_MOVD, 2, {RM_GPR|BITS32,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67641, 332, 2080},
    /* 1649 : MOVQ mmxreg,mmxrm64 [rm:       np 0f 6f /r                       ] MMX,PENT */
        /* 360 355 001(6f) 110 : MMX,PENT */
        {I_MOVQ, 2, {MMXREG,RM_MMX|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67647, 332, 2083},
    /* 1650 : MOVQ mmxrm64,mmxreg [mr:       np 0f 7f /r                       ] MMX,PENT */
        /* 360 355 001(7f) 101 : MMX,PENT */
        {I_MOVQ, 2, {RM_MMX|BITS64,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67653, 332, 2084},
    /* 1651 : MOVQ mmxreg,rm64 [rm:       np o64 0f 6e /r                   ] LONG,PROT,MMX,X86_64 */
        /* 360 324 355 001(6e) 110 : LONG,PROT,MMX,X86_64 */
        {I_MOVQ, 2, {MMXREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57350, 333, 2085},
    /* 1652 : MOVQ rm64,mmxreg [mr:       np o64 0f 7e /r                   ] LONG,PROT,MMX,X86_64 */
        /* 360 324 355 001(7e) 101 : LONG,PROT,MMX,X86_64 */
        {I_MOVQ, 2, {RM_GPR|BITS64,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57357, 333, 2086},
    /* 1653 : PACKSSDW mmxreg,mmxrm [rm:       np 0f 6b /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(6b) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKSSDW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67659, 334, 2087},
    /* 1654 : PACKSSWB mmxreg,mmxrm [rm:       np 0f 63 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(63) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKSSWB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67665, 334, 2088},
    /* 1655 : PACKUSWB mmxreg,mmxrm [rm:       np 0f 67 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(67) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PACKUSWB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67671, 334, 2089},
    /* 1656 : PADDB mmxreg,mmxrm [rm:       np 0f fc /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fc) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67677, 334, 2090},
    /* 1657 : PADDD mmxreg,mmxrm [rm:       np 0f fe /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fe) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67683, 334, 2091},
    /* 1658 : PADDSB mmxreg,mmxrm [rm:       np 0f ec /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ec) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67689, 334, 2092},
    /* 1659 : PADDSIW mmxreg,mmxrm [rm:       0f 51 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(51) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PADDSIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69340, 335, 2093},
    /* 1660 : PADDSW mmxreg,mmxrm [rm:       np 0f ed /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ed) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67695, 334, 2094},
    /* 1661 : PADDUSB mmxreg,mmxrm [rm:       np 0f dc /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(dc) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67701, 334, 2095},
    /* 1662 : PADDUSW mmxreg,mmxrm [rm:       np 0f dd /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(dd) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDUSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67707, 334, 2096},
    /* 1663 : PADDW mmxreg,mmxrm [rm:       np 0f fd /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fd) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PADDW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67713, 334, 2097},
    /* 1664 : PAND mmxreg,mmxrm [rm:       np 0f db /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(db) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PAND, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67719, 334, 2098},
    /* 1665 : PANDN mmxreg,mmxrm [rm:       np 0f df /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(df) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PANDN, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67725, 334, 2099},
    /* 1666 : PAVEB mmxreg,mmxrm [rm:       0f 50 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(50) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PAVEB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69286, 335, 2100},
    /* 1667 : PAVGUSB mmxreg,mmxrm [rm:       0f 0f /r bf                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(bf) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PAVGUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57364, 336, 2101},
    /* 1668 : PCMPEQB mmxreg,mmxrm [rm:       np 0f 74 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(74) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67731, 334, 2102},
    /* 1669 : PCMPEQD mmxreg,mmxrm [rm:       np 0f 76 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(76) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67737, 334, 2103},
    /* 1670 : PCMPEQW mmxreg,mmxrm [rm:       np 0f 75 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(75) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPEQW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67743, 334, 2104},
    /* 1671 : PCMPGTB mmxreg,mmxrm [rm:       np 0f 64 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(64) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67749, 334, 2105},
    /* 1672 : PCMPGTD mmxreg,mmxrm [rm:       np 0f 66 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(66) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67755, 334, 2106},
    /* 1673 : PCMPGTW mmxreg,mmxrm [rm:       np 0f 65 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(65) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PCMPGTW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67761, 334, 2107},
    /* 1674 : PDISTIB mmxreg,mem [rm:       0f 54 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(54) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PDISTIB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69124, 335, 2108},
    /* 1675 : PF2ID mmxreg,mmxrm [rm:       0f 0f /r 1d                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(1d) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PF2ID, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57371, 336, 2109},
    /* 1676 : PFACC mmxreg,mmxrm [rm:       0f 0f /r ae                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(ae) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57378, 336, 2110},
    /* 1677 : PFADD mmxreg,mmxrm [rm:       0f 0f /r 9e                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(9e) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFADD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57385, 336, 2111},
    /* 1678 : PFCMPEQ mmxreg,mmxrm [rm:       0f 0f /r b0                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b0) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPEQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57392, 336, 2112},
    /* 1679 : PFCMPGE mmxreg,mmxrm [rm:       0f 0f /r 90                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(90) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPGE, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57399, 336, 2113},
    /* 1680 : PFCMPGT mmxreg,mmxrm [rm:       0f 0f /r a0                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a0) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFCMPGT, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57406, 336, 2114},
    /* 1681 : PFMAX mmxreg,mmxrm [rm:       0f 0f /r a4                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a4) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMAX, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57413, 336, 2115},
    /* 1682 : PFMIN mmxreg,mmxrm [rm:       0f 0f /r 94                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(94) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMIN, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57420, 336, 2116},
    /* 1683 : PFMUL mmxreg,mmxrm [rm:       0f 0f /r b4                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b4) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFMUL, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57427, 336, 2117},
    /* 1684 : PFRCP mmxreg,mmxrm [rm:       0f 0f /r 96                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(96) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCP, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57434, 336, 2118},
    /* 1685 : PFRCPIT1 mmxreg,mmxrm [rm:       0f 0f /r a6                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a6) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCPIT1, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57441, 336, 2119},
    /* 1686 : PFRCPIT2 mmxreg,mmxrm [rm:       0f 0f /r b6                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b6) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRCPIT2, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57448, 336, 2120},
    /* 1687 : PFRSQIT1 mmxreg,mmxrm [rm:       0f 0f /r a7                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(a7) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRSQIT1, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57455, 336, 2121},
    /* 1688 : PFRSQRT mmxreg,mmxrm [rm:       0f 0f /r 97                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(97) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFRSQRT, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57462, 336, 2122},
    /* 1689 : PFSUB mmxreg,mmxrm [rm:       0f 0f /r 9a                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(9a) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFSUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57469, 336, 2123},
    /* 1690 : PFSUBR mmxreg,mmxrm [rm:       0f 0f /r aa                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(aa) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFSUBR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57476, 336, 2124},
    /* 1691 : PI2FD mmxreg,mmxrm [rm:       0f 0f /r 0d                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(0d) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PI2FD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57483, 336, 2125},
    /* 1692 : PMACHRIW mmxreg,mem [rm:       0f 5e /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5e) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMACHRIW, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69220, 335, 2126},
    /* 1693 : PMADDWD mmxreg,mmxrm [rm:       np 0f f5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMADDWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67767, 334, 2127},
    /* 1694 : PMAGW mmxreg,mmxrm [rm:       0f 52 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(52) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMAGW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68548, 335, 2128},
    /* 1695 : PMULHRIW mmxreg,mmxrm [rm:       0f 5d /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5d) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMULHRIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69244, 335, 2129},
    /* 1696 : PMULHRWA mmxreg,mmxrm [rm:       0f 0f /r b7                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(b7) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PMULHRWA, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57490, 336, 2130},
    /* 1697 : PMULHRWC mmxreg,mmxrm [rm:       0f 59 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(59) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMULHRWC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69322, 335, 2131},
    /* 1698 : PMULHW mmxreg,mmxrm [rm:       np 0f e5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMULHW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67773, 334, 2132},
    /* 1699 : PMULLW mmxreg,mmxrm [rm:       np 0f d5 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d5) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PMULLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67779, 334, 2133},
    /* 1700 : PMVGEZB mmxreg,mem [rm:       0f 5c /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5c) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVGEZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69352, 335, 2134},
    /* 1701 : PMVLZB mmxreg,mem [rm:       0f 5b /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5b) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVLZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69208, 335, 2135},
    /* 1702 : PMVNZB mmxreg,mem [rm:       0f 5a /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(5a) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVNZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69190, 335, 2136},
    /* 1703 : PMVZB mmxreg,mem [rm:       0f 58 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(58) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PMVZB, 2, {MMXREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+69112, 335, 2137},
    /* 1704 : POR mmxreg,mmxrm [rm:       np 0f eb /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(eb) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_POR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67785, 334, 2138},
    /* 1705 : PREFETCH mem [m:        0f 0d /0                          ] AR0,SQ,3DNOW,PENT */
        /* 355 001(0d) 200 : AR0,SQ,3DNOW,PENT */
        {I_PREFETCH, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72299, 337, 2139},
    /* 1706 : PREFETCHW mem [m:        0f 0d /1                          ] AR0,SQ,3DNOW,PENT */
        /* 355 001(0d) 201 : AR0,SQ,3DNOW,PENT */
        {I_PREFETCHW, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72304, 337, 2140},
    /* 1707 : PSLLD mmxreg,mmxrm [rm:       np 0f f2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67791, 334, 2141},
    /* 1708 : PSLLD mmxreg,imm [mi:       np 0f 72 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 206 025 : MMX,PENT */
        {I_PSLLD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57497, 332, 2142},
    /* 1709 : PSLLQ mmxreg,mmxrm [rm:       np 0f f3 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f3) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67797, 334, 2143},
    /* 1710 : PSLLQ mmxreg,imm [mi:       np 0f 73 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(73) 206 025 : MMX,PENT */
        {I_PSLLQ, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57504, 332, 2144},
    /* 1711 : PSLLW mmxreg,mmxrm [rm:       np 0f f1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSLLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67803, 334, 2145},
    /* 1712 : PSLLW mmxreg,imm [mi:       np 0f 71 /6 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 206 025 : MMX,PENT */
        {I_PSLLW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57511, 332, 2146},
    /* 1713 : PSRAD mmxreg,mmxrm [rm:       np 0f e2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRAD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67809, 334, 2147},
    /* 1714 : PSRAD mmxreg,imm [mi:       np 0f 72 /4 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 204 025 : MMX,PENT */
        {I_PSRAD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57518, 332, 2148},
    /* 1715 : PSRAW mmxreg,mmxrm [rm:       np 0f e1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRAW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67815, 334, 2149},
    /* 1716 : PSRAW mmxreg,imm [mi:       np 0f 71 /4 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 204 025 : MMX,PENT */
        {I_PSRAW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57525, 332, 2150},
    /* 1717 : PSRLD mmxreg,mmxrm [rm:       np 0f d2 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d2) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67821, 334, 2151},
    /* 1718 : PSRLD mmxreg,imm [mi:       np 0f 72 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(72) 202 025 : MMX,PENT */
        {I_PSRLD, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57532, 332, 2152},
    /* 1719 : PSRLQ mmxreg,mmxrm [rm:       np 0f d3 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d3) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67827, 334, 2153},
    /* 1720 : PSRLQ mmxreg,imm [mi:       np 0f 73 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(73) 202 025 : MMX,PENT */
        {I_PSRLQ, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57539, 332, 2154},
    /* 1721 : PSRLW mmxreg,mmxrm [rm:       np 0f d1 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d1) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSRLW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67833, 334, 2155},
    /* 1722 : PSRLW mmxreg,imm [mi:       np 0f 71 /2 ib,u                  ] MMX,PENT */
        /* 360 355 001(71) 202 025 : MMX,PENT */
        {I_PSRLW, 2, {MMXREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+57546, 332, 2156},
    /* 1723 : PSUBB mmxreg,mmxrm [rm:       np 0f f8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67839, 334, 2157},
    /* 1724 : PSUBD mmxreg,mmxrm [rm:       np 0f fa /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(fa) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67845, 334, 2158},
    /* 1725 : PSUBSB mmxreg,mmxrm [rm:       np 0f e8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67851, 334, 2159},
    /* 1726 : PSUBSIW mmxreg,mmxrm [rm:       0f 55 /r                          ] AR0-1,SQ,MMX,PENT,CYRIX */
        /* 355 001(55) 110 : AR0,AR1,SQ,MMX,PENT,CYRIX */
        {I_PSUBSIW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69118, 335, 2160},
    /* 1727 : PSUBSW mmxreg,mmxrm [rm:       np 0f e9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(e9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67857, 334, 2161},
    /* 1728 : PSUBUSB mmxreg,mmxrm [rm:       np 0f d8 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d8) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBUSB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67863, 334, 2162},
    /* 1729 : PSUBUSW mmxreg,mmxrm [rm:       np 0f d9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(d9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBUSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67869, 334, 2163},
    /* 1730 : PSUBW mmxreg,mmxrm [rm:       np 0f f9 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(f9) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PSUBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67875, 334, 2164},
    /* 1731 : PUNPCKHBW mmxreg,mmxrm [rm:       np 0f 68 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(68) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67881, 334, 2165},
    /* 1732 : PUNPCKHDQ mmxreg,mmxrm [rm:       np 0f 6a /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(6a) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67887, 334, 2166},
    /* 1733 : PUNPCKHWD mmxreg,mmxrm [rm:       np 0f 69 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(69) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKHWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67893, 334, 2167},
    /* 1734 : PUNPCKLBW mmxreg,mmxrm [rm:       np 0f 60 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(60) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67899, 334, 2168},
    /* 1735 : PUNPCKLDQ mmxreg,mmxrm [rm:       np 0f 62 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(62) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67905, 334, 2169},
    /* 1736 : PUNPCKLWD mmxreg,mmxrm [rm:       np 0f 61 /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(61) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PUNPCKLWD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+67911, 334, 2170},
    /* 1737 : PUSH reg16 [r:        nw o16 50+r                       ] 8086 */
        /* 327 320 010 120 : NWSIZE,8086 */
        {I_PUSH, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72309, 233, 2173},
    /* 1738 : PUSH reg32 [r:        nw o32 50+r                       ] 386 */
        /* 327 321 010 120 : NWSIZE,386 */
        {I_PUSH, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72314, 234, 2174},
    /* 1739 : PUSH reg64 [r:        nw o64 50+r                       ] LONG,PROT,X86_64 */
        /* 327 324 010 120 : NWSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72319, 235, 2175},
    /* 1740 : PUSH rm16 [m:        nw o16 ff /6                      ] AR0,OSIZE,8086 */
        /* 327 320 001(ff) 206 : AR0,NWSIZE,OSIZE,8086 */
        {I_PUSH, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67917, 338, 2176},
    /* 1741 : PUSH rm32 [m:        nw o32 ff /6                      ] AR0,OSIZE,386 */
        /* 327 321 001(ff) 206 : AR0,NWSIZE,OSIZE,386 */
        {I_PUSH, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67923, 204, 2177},
    /* 1742 : PUSH rm64 [m:        nw o64 ff /6                      ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 327 324 001(ff) 206 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67929, 205, 2178},
    /* 1743 : PUSH sbyteword16 [i:        nw o16 6a ib,s                    ] AR0,OSIZE,186 */
        /* 327 320 001(6a) 274 : AR0,NWSIZE,OSIZE,186 */
        {I_PUSH, 1, {SBYTEWORD|BITS16|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67941, 340, 2180},
    /* 1744 : PUSH sbytedword32 [i:        nw o32 6a ib,s                    ] AR0,OSIZE,386 */
        /* 327 321 001(6a) 274 : AR0,NWSIZE,OSIZE,386 */
        {I_PUSH, 1, {SBYTEDWORD|BITS32|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67947, 204, 2181},
    /* 1745 : PUSH sbytedword64 [i:        nw o64 6a ib,s                    ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 001(6a) 274 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {SBYTEDWORD|BITS64|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67953, 205, 2182},
    /* 1746 : PUSH imm16 [i:        nw o16 68 iw                      ] AR0,OSIZE,186 */
        /* 327 320 001(68) 030 : AR0,NWSIZE,OSIZE,186 */
        {I_PUSH, 1, {IMM_NORMAL|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67959, 340, 2183},
    /* 1747 : PUSH imm32 [i:        nw o32 68 id                      ] AR0,OSIZE,NOREX,NOAPX,NOLONG,386 */
        /* 327 321 001(68) 040 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,386 */
        {I_PUSH, 1, {IMM_NORMAL|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67965, 214, 2184},
    /* 1748 : PUSH sdword64 [i:        nw o64 68 id,s                    ] AR0,OSIZE,LONG,386,X86_64 */
        /* 327 324 001(68) 254 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_PUSH, 1, {SDWORD|BITS64|IMM_NORMAL,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67971, 205, 2185},
    /* 1749 : POP reg16 [r:        nw o16 58+r                       ] 8086 */
        /* 327 320 010 130 : NWSIZE,8086 */
        {I_POP, 1, {REG_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72324, 233, 2187},
    /* 1750 : POP reg32 [r:        nw o32 58+r                       ] 386 */
        /* 327 321 010 130 : NWSIZE,386 */
        {I_POP, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72329, 234, 2188},
    /* 1751 : POP reg64 [r:        nw o64 58+r                       ] LONG,PROT,X86_64 */
        /* 327 324 010 130 : NWSIZE,LONG,PROT,X86_64 */
        {I_POP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72334, 235, 2189},
    /* 1752 : POP rm16 [m:        nw o16 8f /0                      ] AR0,OSIZE,8086 */
        /* 327 320 001(8f) 200 : AR0,NWSIZE,OSIZE,8086 */
        {I_POP, 1, {RM_GPR|BITS16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67977, 338, 2190},
    /* 1753 : POP rm32 [m:        nw o32 8f /0                      ] AR0,OSIZE,386 */
        /* 327 321 001(8f) 200 : AR0,NWSIZE,OSIZE,386 */
        {I_POP, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67983, 204, 2191},
    /* 1754 : POP rm64 [m:        nw o64 8f /0                      ] AR0,OSIZE,LONG,PROT,X86_64 */
        /* 327 324 001(8f) 200 : AR0,NWSIZE,OSIZE,LONG,PROT,X86_64 */
        {I_POP, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67989, 205, 2192},
    /* 1755 : PUSHA void [          osm 60                            ] NOREX,NOAPX,NOLONG,186 */
        /* 342 001(60) : NOREX,NOAPX,NOLONG,186 */
        {I_PUSHA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73404, 341, 2194},
    /* 1756 : PUSHAW void [          o16 60                            ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(60) : NOREX,NOAPX,NOLONG,186 */
        {I_PUSHAW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73408, 341, 2195},
    /* 1757 : PUSHAD void [          o32 60                            ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(60) : NOREX,NOAPX,NOLONG,386 */
        {I_PUSHAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73412, 305, 2196},
    /* 1758 : POPA void [          osm 61                            ] NOREX,NOAPX,NOLONG,186 */
        /* 342 001(61) : NOREX,NOAPX,NOLONG,186 */
        {I_POPA, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73416, 341, 2197},
    /* 1759 : POPAW void [          o16 61                            ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(61) : NOREX,NOAPX,NOLONG,186 */
        {I_POPAW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73420, 341, 2198},
    /* 1760 : POPAD void [          o32 61                            ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(61) : NOREX,NOAPX,NOLONG,386 */
        {I_POPAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73424, 305, 2199},
    /* 1761 : ENTER imm16,imm8 [ij:       nw osm c8 iw ib,u                 ] 186 */
        /* 327 342 001(c8) 030 025 : NWSIZE,186 */
        {I_ENTER, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57553, 342, 2201},
    /* 1762 : ENTERW imm16,imm8 [ij:       nw o16 c8 iw ib,u                 ] 186 */
        /* 327 320 001(c8) 030 025 : NWSIZE,186 */
        {I_ENTERW, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57560, 342, 2202},
    /* 1763 : ENTERD imm16,imm8 [ij:       nw o32 c8 iw ib,u                 ] 386 */
        /* 327 321 001(c8) 030 025 : NWSIZE,386 */
        {I_ENTERD, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57567, 234, 2203},
    /* 1764 : ENTERQ imm16,imm8 [ij:       nw o64 c8 iw ib,u                 ] LONG,386,X86_64 */
        /* 327 324 001(c8) 030 025 : NWSIZE,LONG,PROT,X86_64 */
        {I_ENTERQ, 2, {IMM_NORMAL|BITS16,IMM_NORMAL|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+57574, 235, 2204},
    /* 1765 : LEAVE void [          nw osm c9                         ] 186 */
        /* 327 342 001(c9) : NWSIZE,186 */
        {I_LEAVE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72339, 342, 2209},
    /* 1766 : LEAVEW void [          nw o16 c9                         ] 186 */
        /* 327 320 001(c9) : NWSIZE,186 */
        {I_LEAVEW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72344, 342, 2210},
    /* 1767 : LEAVED void [          nw o32 c9                         ] 386 */
        /* 327 321 001(c9) : NWSIZE,386 */
        {I_LEAVED, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72349, 234, 2211},
    /* 1768 : LEAVEQ void [          nw o64 c9                         ] LONG,386,X86_64 */
        /* 327 324 001(c9) : NWSIZE,LONG,PROT,X86_64 */
        {I_LEAVEQ, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72354, 235, 2212},
    /* 1769 : BOUND reg16,mem [rm:       o16 62 /r                         ] NOREX,NOAPX,NOLONG,186 */
        /* 320 001(62) 110 : NOREX,NOAPX,NOLONG,186 */
        {I_BOUND, 2, {REG_GPR|BITS16|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72359, 341, 2214},
    /* 1770 : BOUND reg32,mem [rm:       o32 62 /r                         ] NOREX,NOAPX,NOLONG,386 */
        /* 321 001(62) 110 : NOREX,NOAPX,NOLONG,386 */
        {I_BOUND, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72364, 305, 2215},
    /* 1771 : PUSHP reg64 [r:        o64nw rex.w rex2 50+r             ] LONG,PROT,APX,X86_64 */
        /* 323 347 350 010 120 : NWSIZE,LONG,REX2,PROT,APX,X86_64 */
        {I_PUSHP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67995, 343, 2218},
    /* 1772 : POPP reg64 [r:        o64nw rex.w rex2 58+r             ] LONG,PROT,APX,X86_64 */
        /* 323 347 350 010 130 : NWSIZE,LONG,REX2,PROT,APX,X86_64 */
        {I_POPP, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+68001, 343, 2219},
    /* 1773 : PUSH2 reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w0 ff /6  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 7c 18) 300 001(ff) 216 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_PUSH2, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13063, 344, 2225},
    /* 1774 : PUSH2P reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w1 ff /6  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 fc 18) 300 001(ff) 216 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_PUSH2P, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13073, 344, 2227},
    /* 1775 : POP2 reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w0 8f /0  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 7c 18) 300 001(8f) 210 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_POP2, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13083, 344, 2234},
    /* 1776 : POP2P reg64,reg64 [vm:       o64nw evex.nd1.l0.np.m4.w1 8f /0  ] LONG,PROT,APX,X86_64 */
        /* 323 240(f4 fc 18) 300 001(8f) 210 : NWSIZE,LONG,EVEX,PROT,APX,X86_64 */
        {I_POP2P, 2, {REG_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+13103, 344, 2236},
    /* 1777 : PXOR mmxreg,mmxrm [rm:       np 0f ef /r                       ] AR0-1,SQ,MMX,PENT */
        /* 360 355 001(ef) 110 : AR0,AR1,SQ,MMX,PENT */
        {I_PXOR, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68007, 334, 2241},
    /* 1778 : SKINIT void [          0f 01 de                          ] LONG,PROT,X86_64 */
        /* 355 002(01 de) : LONG,PROT,X86_64 */
        {I_SKINIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72369, 24, 2242},
    /* 1779 : UD0 reg16,rm16 [rm:       o16 0f ff /r                      ] SM0-1,186 */
        /* 320 355 001(ff) 110 : SM0,SM1,186 */
        {I_UD0, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68013, 345, 2246},
    /* 1780 : UD0 reg32,rm32 [rm:       o32 0f ff /r                      ] SM0-1,386 */
        /* 321 355 001(ff) 110 : SM0,SM1,386 */
        {I_UD0, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68019, 13, 2247},
    /* 1781 : UD0 reg64,rm64 [rm:       o64 0f ff /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(ff) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_UD0, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68025, 14, 2248},
    /* 1782 : UD0 void [          0f ff                             ] 186 */
        /* 355 001(ff) : 186 */
        {I_UD0, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73428, 245, 2249},
    /* 1783 : UD1 reg16,rm16 [rm:       o16 0f b9 /r                      ] SM0-1,186 */
        /* 320 355 001(b9) 110 : SM0,SM1,186 */
        {I_UD1, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68031, 345, 2250},
    /* 1784 : UD1 reg32,rm32 [rm:       o32 0f b9 /r                      ] SM0-1,386 */
        /* 321 355 001(b9) 110 : SM0,SM1,386 */
        {I_UD1, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68037, 13, 2251},
    /* 1785 : UD1 reg64,rm64 [rm:       o64 0f b9 /r                      ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(b9) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_UD1, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68043, 14, 2252},
    /* 1786 : UD1 void [          0f b9                             ] 186 */
        /* 355 001(b9) : 186 */
        {I_UD1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73432, 245, 2253},
    /* 1787 : UD2 void [          0f 0b                             ] 186 */
        /* 355 001(0b) : 186 */
        {I_UD2, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73436, 245, 2259},
    /* 1788 : UDB void [          d6                                ] LONG,PROT,X86_64 */
        /* 001(d6) : LONG,PROT,X86_64 */
        {I_UDB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72531, 24, 2268},
    /* 1789 : UDW void [          ff ff                             ] UNDOC,186 */
        /* 002(ff ff) : UNDOC,186 */
        {I_UDW, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73440, 346, 2271},
    /* 1790 : FWAIT void [          wait                              ] 8086 */
        /* 341 : 8086 */
        {I_FWAIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73362, 23, 2273},
    /* 1791 : XLATB void [          d7                                ] 8086 */
        /* 001(d7) : 8086 */
        {I_XLATB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70098, 23, 2275},
    /* 1792 : CCMPA spec4,rm8,reg8 [wmr:      evex.scc7.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24718, 347, 2278},
    /* 1793 : CCMPC spec4,rm8,reg8 [wmr:      evex.scc2.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24736, 347, 2278},
    /* 1794 : CCMPF spec4,rm8,reg8 [wmr:      evex.scc11.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24709, 347, 2278},
    /* 1795 : CCMPG spec4,rm8,reg8 [wmr:      evex.scc15.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24763, 347, 2278},
    /* 1796 : CCMPL spec4,rm8,reg8 [wmr:      evex.scc12.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24781, 347, 2278},
    /* 1797 : CCMPNA spec4,rm8,reg8 [wmr:      evex.scc6.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24745, 347, 2278},
    /* 1798 : CCMPNC spec4,rm8,reg8 [wmr:      evex.scc3.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24727, 347, 2278},
    /* 1799 : CCMPNG spec4,rm8,reg8 [wmr:      evex.scc14.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24790, 347, 2278},
    /* 1800 : CCMPNL spec4,rm8,reg8 [wmr:      evex.scc13.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24772, 347, 2278},
    /* 1801 : CCMPNO spec4,rm8,reg8 [wmr:      evex.scc1.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24808, 347, 2278},
    /* 1802 : CCMPNS spec4,rm8,reg8 [wmr:      evex.scc9.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24817, 347, 2278},
    /* 1803 : CCMPNZ spec4,rm8,reg8 [wmr:      evex.scc5.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24799, 347, 2278},
    /* 1804 : CCMPO spec4,rm8,reg8 [wmr:      evex.scc0.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24826, 347, 2278},
    /* 1805 : CCMPS spec4,rm8,reg8 [wmr:      evex.scc8.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24835, 347, 2278},
    /* 1806 : CCMPT spec4,rm8,reg8 [wmr:      evex.scc10.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24844, 347, 2278},
    /* 1807 : CCMPZ spec4,rm8,reg8 [wmr:      evex.scc4.dfv.l0.m4.o8 38 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(38) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24754, 347, 2278},
    /* 1808 : CCMPA spec4,rm16,reg16 [wmr:      evex.scc7.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13133, 348, 2279},
    /* 1809 : CCMPC spec4,rm16,reg16 [wmr:      evex.scc2.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13153, 348, 2279},
    /* 1810 : CCMPF spec4,rm16,reg16 [wmr:      evex.scc11.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13123, 348, 2279},
    /* 1811 : CCMPG spec4,rm16,reg16 [wmr:      evex.scc15.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13183, 348, 2279},
    /* 1812 : CCMPL spec4,rm16,reg16 [wmr:      evex.scc12.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13203, 348, 2279},
    /* 1813 : CCMPNA spec4,rm16,reg16 [wmr:      evex.scc6.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13163, 348, 2279},
    /* 1814 : CCMPNC spec4,rm16,reg16 [wmr:      evex.scc3.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13143, 348, 2279},
    /* 1815 : CCMPNG spec4,rm16,reg16 [wmr:      evex.scc14.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13213, 348, 2279},
    /* 1816 : CCMPNL spec4,rm16,reg16 [wmr:      evex.scc13.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13193, 348, 2279},
    /* 1817 : CCMPNO spec4,rm16,reg16 [wmr:      evex.scc1.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13233, 348, 2279},
    /* 1818 : CCMPNS spec4,rm16,reg16 [wmr:      evex.scc9.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13243, 348, 2279},
    /* 1819 : CCMPNZ spec4,rm16,reg16 [wmr:      evex.scc5.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13223, 348, 2279},
    /* 1820 : CCMPO spec4,rm16,reg16 [wmr:      evex.scc0.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13253, 348, 2279},
    /* 1821 : CCMPS spec4,rm16,reg16 [wmr:      evex.scc8.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13263, 348, 2279},
    /* 1822 : CCMPT spec4,rm16,reg16 [wmr:      evex.scc10.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13273, 348, 2279},
    /* 1823 : CCMPZ spec4,rm16,reg16 [wmr:      evex.scc4.dfv.l0.m4.o16 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(39) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13173, 348, 2279},
    /* 1824 : CCMPA spec4,rm32,reg32 [wmr:      evex.scc7.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13293, 349, 2280},
    /* 1825 : CCMPC spec4,rm32,reg32 [wmr:      evex.scc2.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13313, 349, 2280},
    /* 1826 : CCMPF spec4,rm32,reg32 [wmr:      evex.scc11.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13283, 349, 2280},
    /* 1827 : CCMPG spec4,rm32,reg32 [wmr:      evex.scc15.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13343, 349, 2280},
    /* 1828 : CCMPL spec4,rm32,reg32 [wmr:      evex.scc12.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13363, 349, 2280},
    /* 1829 : CCMPNA spec4,rm32,reg32 [wmr:      evex.scc6.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13323, 349, 2280},
    /* 1830 : CCMPNC spec4,rm32,reg32 [wmr:      evex.scc3.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13303, 349, 2280},
    /* 1831 : CCMPNG spec4,rm32,reg32 [wmr:      evex.scc14.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13373, 349, 2280},
    /* 1832 : CCMPNL spec4,rm32,reg32 [wmr:      evex.scc13.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13353, 349, 2280},
    /* 1833 : CCMPNO spec4,rm32,reg32 [wmr:      evex.scc1.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13393, 349, 2280},
    /* 1834 : CCMPNS spec4,rm32,reg32 [wmr:      evex.scc9.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13403, 349, 2280},
    /* 1835 : CCMPNZ spec4,rm32,reg32 [wmr:      evex.scc5.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13383, 349, 2280},
    /* 1836 : CCMPO spec4,rm32,reg32 [wmr:      evex.scc0.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13413, 349, 2280},
    /* 1837 : CCMPS spec4,rm32,reg32 [wmr:      evex.scc8.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13423, 349, 2280},
    /* 1838 : CCMPT spec4,rm32,reg32 [wmr:      evex.scc10.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13433, 349, 2280},
    /* 1839 : CCMPZ spec4,rm32,reg32 [wmr:      evex.scc4.dfv.l0.m4.o32 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13333, 349, 2280},
    /* 1840 : CCMPA spec4,rm64,reg64 [wmr:      evex.scc7.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13453, 349, 2281},
    /* 1841 : CCMPC spec4,rm64,reg64 [wmr:      evex.scc2.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13473, 349, 2281},
    /* 1842 : CCMPF spec4,rm64,reg64 [wmr:      evex.scc11.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13443, 349, 2281},
    /* 1843 : CCMPG spec4,rm64,reg64 [wmr:      evex.scc15.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13503, 349, 2281},
    /* 1844 : CCMPL spec4,rm64,reg64 [wmr:      evex.scc12.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13523, 349, 2281},
    /* 1845 : CCMPNA spec4,rm64,reg64 [wmr:      evex.scc6.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13483, 349, 2281},
    /* 1846 : CCMPNC spec4,rm64,reg64 [wmr:      evex.scc3.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13463, 349, 2281},
    /* 1847 : CCMPNG spec4,rm64,reg64 [wmr:      evex.scc14.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13533, 349, 2281},
    /* 1848 : CCMPNL spec4,rm64,reg64 [wmr:      evex.scc13.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13513, 349, 2281},
    /* 1849 : CCMPNO spec4,rm64,reg64 [wmr:      evex.scc1.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13553, 349, 2281},
    /* 1850 : CCMPNS spec4,rm64,reg64 [wmr:      evex.scc9.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13563, 349, 2281},
    /* 1851 : CCMPNZ spec4,rm64,reg64 [wmr:      evex.scc5.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13543, 349, 2281},
    /* 1852 : CCMPO spec4,rm64,reg64 [wmr:      evex.scc0.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13573, 349, 2281},
    /* 1853 : CCMPS spec4,rm64,reg64 [wmr:      evex.scc8.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13583, 349, 2281},
    /* 1854 : CCMPT spec4,rm64,reg64 [wmr:      evex.scc10.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13593, 349, 2281},
    /* 1855 : CCMPZ spec4,rm64,reg64 [wmr:      evex.scc4.dfv.l0.m4.o64 39 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(39) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13493, 349, 2281},
    /* 1856 : CCMPA spec4,reg8,rm8 [wrm:      evex.scc7.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24862, 347, 2282},
    /* 1857 : CCMPC spec4,reg8,rm8 [wrm:      evex.scc2.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24880, 347, 2282},
    /* 1858 : CCMPF spec4,reg8,rm8 [wrm:      evex.scc11.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24853, 347, 2282},
    /* 1859 : CCMPG spec4,reg8,rm8 [wrm:      evex.scc15.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24907, 347, 2282},
    /* 1860 : CCMPL spec4,reg8,rm8 [wrm:      evex.scc12.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24925, 347, 2282},
    /* 1861 : CCMPNA spec4,reg8,rm8 [wrm:      evex.scc6.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24889, 347, 2282},
    /* 1862 : CCMPNC spec4,reg8,rm8 [wrm:      evex.scc3.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24871, 347, 2282},
    /* 1863 : CCMPNG spec4,reg8,rm8 [wrm:      evex.scc14.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24934, 347, 2282},
    /* 1864 : CCMPNL spec4,reg8,rm8 [wrm:      evex.scc13.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24916, 347, 2282},
    /* 1865 : CCMPNO spec4,reg8,rm8 [wrm:      evex.scc1.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24952, 347, 2282},
    /* 1866 : CCMPNS spec4,reg8,rm8 [wrm:      evex.scc9.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24961, 347, 2282},
    /* 1867 : CCMPNZ spec4,reg8,rm8 [wrm:      evex.scc5.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24943, 347, 2282},
    /* 1868 : CCMPO spec4,reg8,rm8 [wrm:      evex.scc0.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24970, 347, 2282},
    /* 1869 : CCMPS spec4,reg8,rm8 [wrm:      evex.scc8.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24979, 347, 2282},
    /* 1870 : CCMPT spec4,reg8,rm8 [wrm:      evex.scc10.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24988, 347, 2282},
    /* 1871 : CCMPZ spec4,reg8,rm8 [wrm:      evex.scc4.dfv.l0.m4.o8 3a /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(3a) 121 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS8,RM_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24898, 347, 2282},
    /* 1872 : CCMPA spec4,reg16,rm16 [wrm:      evex.scc7.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13613, 348, 2283},
    /* 1873 : CCMPC spec4,reg16,rm16 [wrm:      evex.scc2.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13633, 348, 2283},
    /* 1874 : CCMPF spec4,reg16,rm16 [wrm:      evex.scc11.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13603, 348, 2283},
    /* 1875 : CCMPG spec4,reg16,rm16 [wrm:      evex.scc15.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13663, 348, 2283},
    /* 1876 : CCMPL spec4,reg16,rm16 [wrm:      evex.scc12.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13683, 348, 2283},
    /* 1877 : CCMPNA spec4,reg16,rm16 [wrm:      evex.scc6.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13643, 348, 2283},
    /* 1878 : CCMPNC spec4,reg16,rm16 [wrm:      evex.scc3.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13623, 348, 2283},
    /* 1879 : CCMPNG spec4,reg16,rm16 [wrm:      evex.scc14.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13693, 348, 2283},
    /* 1880 : CCMPNL spec4,reg16,rm16 [wrm:      evex.scc13.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13673, 348, 2283},
    /* 1881 : CCMPNO spec4,reg16,rm16 [wrm:      evex.scc1.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13713, 348, 2283},
    /* 1882 : CCMPNS spec4,reg16,rm16 [wrm:      evex.scc9.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13723, 348, 2283},
    /* 1883 : CCMPNZ spec4,reg16,rm16 [wrm:      evex.scc5.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13703, 348, 2283},
    /* 1884 : CCMPO spec4,reg16,rm16 [wrm:      evex.scc0.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13733, 348, 2283},
    /* 1885 : CCMPS spec4,reg16,rm16 [wrm:      evex.scc8.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13743, 348, 2283},
    /* 1886 : CCMPT spec4,reg16,rm16 [wrm:      evex.scc10.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13753, 348, 2283},
    /* 1887 : CCMPZ spec4,reg16,rm16 [wrm:      evex.scc4.dfv.l0.m4.o16 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(3b) 121 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+13653, 348, 2283},
    /* 1888 : CCMPA spec4,reg32,rm32 [wrm:      evex.scc7.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13773, 349, 2284},
    /* 1889 : CCMPC spec4,reg32,rm32 [wrm:      evex.scc2.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13793, 349, 2284},
    /* 1890 : CCMPF spec4,reg32,rm32 [wrm:      evex.scc11.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13763, 349, 2284},
    /* 1891 : CCMPG spec4,reg32,rm32 [wrm:      evex.scc15.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13823, 349, 2284},
    /* 1892 : CCMPL spec4,reg32,rm32 [wrm:      evex.scc12.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13843, 349, 2284},
    /* 1893 : CCMPNA spec4,reg32,rm32 [wrm:      evex.scc6.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13803, 349, 2284},
    /* 1894 : CCMPNC spec4,reg32,rm32 [wrm:      evex.scc3.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13783, 349, 2284},
    /* 1895 : CCMPNG spec4,reg32,rm32 [wrm:      evex.scc14.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13853, 349, 2284},
    /* 1896 : CCMPNL spec4,reg32,rm32 [wrm:      evex.scc13.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13833, 349, 2284},
    /* 1897 : CCMPNO spec4,reg32,rm32 [wrm:      evex.scc1.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13873, 349, 2284},
    /* 1898 : CCMPNS spec4,reg32,rm32 [wrm:      evex.scc9.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13883, 349, 2284},
    /* 1899 : CCMPNZ spec4,reg32,rm32 [wrm:      evex.scc5.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13863, 349, 2284},
    /* 1900 : CCMPO spec4,reg32,rm32 [wrm:      evex.scc0.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13893, 349, 2284},
    /* 1901 : CCMPS spec4,reg32,rm32 [wrm:      evex.scc8.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13903, 349, 2284},
    /* 1902 : CCMPT spec4,reg32,rm32 [wrm:      evex.scc10.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13913, 349, 2284},
    /* 1903 : CCMPZ spec4,reg32,rm32 [wrm:      evex.scc4.dfv.l0.m4.o32 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+13813, 349, 2284},
    /* 1904 : CCMPA spec4,reg64,rm64 [wrm:      evex.scc7.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13933, 349, 2285},
    /* 1905 : CCMPC spec4,reg64,rm64 [wrm:      evex.scc2.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13953, 349, 2285},
    /* 1906 : CCMPF spec4,reg64,rm64 [wrm:      evex.scc11.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13923, 349, 2285},
    /* 1907 : CCMPG spec4,reg64,rm64 [wrm:      evex.scc15.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13983, 349, 2285},
    /* 1908 : CCMPL spec4,reg64,rm64 [wrm:      evex.scc12.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14003, 349, 2285},
    /* 1909 : CCMPNA spec4,reg64,rm64 [wrm:      evex.scc6.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13963, 349, 2285},
    /* 1910 : CCMPNC spec4,reg64,rm64 [wrm:      evex.scc3.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13943, 349, 2285},
    /* 1911 : CCMPNG spec4,reg64,rm64 [wrm:      evex.scc14.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14013, 349, 2285},
    /* 1912 : CCMPNL spec4,reg64,rm64 [wrm:      evex.scc13.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13993, 349, 2285},
    /* 1913 : CCMPNO spec4,reg64,rm64 [wrm:      evex.scc1.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14033, 349, 2285},
    /* 1914 : CCMPNS spec4,reg64,rm64 [wrm:      evex.scc9.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14043, 349, 2285},
    /* 1915 : CCMPNZ spec4,reg64,rm64 [wrm:      evex.scc5.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14023, 349, 2285},
    /* 1916 : CCMPO spec4,reg64,rm64 [wrm:      evex.scc0.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14053, 349, 2285},
    /* 1917 : CCMPS spec4,reg64,rm64 [wrm:      evex.scc8.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14063, 349, 2285},
    /* 1918 : CCMPT spec4,reg64,rm64 [wrm:      evex.scc10.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14073, 349, 2285},
    /* 1919 : CCMPZ spec4,reg64,rm64 [wrm:      evex.scc4.dfv.l0.m4.o64 3b /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(3b) 121 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+13973, 349, 2285},
    /* 1920 : CCMPA spec4,rm16,sbyteword16 [wmi:      evex.scc7.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2378, 348, 2286},
    /* 1921 : CCMPC spec4,rm16,sbyteword16 [wmi:      evex.scc2.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2400, 348, 2286},
    /* 1922 : CCMPF spec4,rm16,sbyteword16 [wmi:      evex.scc11.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2367, 348, 2286},
    /* 1923 : CCMPG spec4,rm16,sbyteword16 [wmi:      evex.scc15.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2433, 348, 2286},
    /* 1924 : CCMPL spec4,rm16,sbyteword16 [wmi:      evex.scc12.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2455, 348, 2286},
    /* 1925 : CCMPNA spec4,rm16,sbyteword16 [wmi:      evex.scc6.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2411, 348, 2286},
    /* 1926 : CCMPNC spec4,rm16,sbyteword16 [wmi:      evex.scc3.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2389, 348, 2286},
    /* 1927 : CCMPNG spec4,rm16,sbyteword16 [wmi:      evex.scc14.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2466, 348, 2286},
    /* 1928 : CCMPNL spec4,rm16,sbyteword16 [wmi:      evex.scc13.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2444, 348, 2286},
    /* 1929 : CCMPNO spec4,rm16,sbyteword16 [wmi:      evex.scc1.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2488, 348, 2286},
    /* 1930 : CCMPNS spec4,rm16,sbyteword16 [wmi:      evex.scc9.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2499, 348, 2286},
    /* 1931 : CCMPNZ spec4,rm16,sbyteword16 [wmi:      evex.scc5.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2477, 348, 2286},
    /* 1932 : CCMPO spec4,rm16,sbyteword16 [wmi:      evex.scc0.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2510, 348, 2286},
    /* 1933 : CCMPS spec4,rm16,sbyteword16 [wmi:      evex.scc8.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2521, 348, 2286},
    /* 1934 : CCMPT spec4,rm16,sbyteword16 [wmi:      evex.scc10.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2532, 348, 2286},
    /* 1935 : CCMPZ spec4,rm16,sbyteword16 [wmi:      evex.scc4.dfv.l0.m4.o16 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(83) 217 276 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,SBYTEWORD|BITS16|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2422, 348, 2286},
    /* 1936 : CCMPA spec4,rm32,sbytedword32 [wmi:      evex.scc7.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2554, 349, 2287},
    /* 1937 : CCMPC spec4,rm32,sbytedword32 [wmi:      evex.scc2.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2576, 349, 2287},
    /* 1938 : CCMPF spec4,rm32,sbytedword32 [wmi:      evex.scc11.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2543, 349, 2287},
    /* 1939 : CCMPG spec4,rm32,sbytedword32 [wmi:      evex.scc15.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2609, 349, 2287},
    /* 1940 : CCMPL spec4,rm32,sbytedword32 [wmi:      evex.scc12.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2631, 349, 2287},
    /* 1941 : CCMPNA spec4,rm32,sbytedword32 [wmi:      evex.scc6.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2587, 349, 2287},
    /* 1942 : CCMPNC spec4,rm32,sbytedword32 [wmi:      evex.scc3.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2565, 349, 2287},
    /* 1943 : CCMPNG spec4,rm32,sbytedword32 [wmi:      evex.scc14.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2642, 349, 2287},
    /* 1944 : CCMPNL spec4,rm32,sbytedword32 [wmi:      evex.scc13.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2620, 349, 2287},
    /* 1945 : CCMPNO spec4,rm32,sbytedword32 [wmi:      evex.scc1.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2664, 349, 2287},
    /* 1946 : CCMPNS spec4,rm32,sbytedword32 [wmi:      evex.scc9.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2675, 349, 2287},
    /* 1947 : CCMPNZ spec4,rm32,sbytedword32 [wmi:      evex.scc5.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2653, 349, 2287},
    /* 1948 : CCMPO spec4,rm32,sbytedword32 [wmi:      evex.scc0.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2686, 349, 2287},
    /* 1949 : CCMPS spec4,rm32,sbytedword32 [wmi:      evex.scc8.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2697, 349, 2287},
    /* 1950 : CCMPT spec4,rm32,sbytedword32 [wmi:      evex.scc10.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2708, 349, 2287},
    /* 1951 : CCMPZ spec4,rm32,sbytedword32 [wmi:      evex.scc4.dfv.l0.m4.o32 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,SBYTEDWORD|BITS32|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2598, 349, 2287},
    /* 1952 : CCMPA spec4,rm64,sbytedword64 [wmi:      evex.scc7.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2730, 349, 2288},
    /* 1953 : CCMPC spec4,rm64,sbytedword64 [wmi:      evex.scc2.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2752, 349, 2288},
    /* 1954 : CCMPF spec4,rm64,sbytedword64 [wmi:      evex.scc11.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2719, 349, 2288},
    /* 1955 : CCMPG spec4,rm64,sbytedword64 [wmi:      evex.scc15.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2785, 349, 2288},
    /* 1956 : CCMPL spec4,rm64,sbytedword64 [wmi:      evex.scc12.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2807, 349, 2288},
    /* 1957 : CCMPNA spec4,rm64,sbytedword64 [wmi:      evex.scc6.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2763, 349, 2288},
    /* 1958 : CCMPNC spec4,rm64,sbytedword64 [wmi:      evex.scc3.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2741, 349, 2288},
    /* 1959 : CCMPNG spec4,rm64,sbytedword64 [wmi:      evex.scc14.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2818, 349, 2288},
    /* 1960 : CCMPNL spec4,rm64,sbytedword64 [wmi:      evex.scc13.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2796, 349, 2288},
    /* 1961 : CCMPNO spec4,rm64,sbytedword64 [wmi:      evex.scc1.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2840, 349, 2288},
    /* 1962 : CCMPNS spec4,rm64,sbytedword64 [wmi:      evex.scc9.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2851, 349, 2288},
    /* 1963 : CCMPNZ spec4,rm64,sbytedword64 [wmi:      evex.scc5.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2829, 349, 2288},
    /* 1964 : CCMPO spec4,rm64,sbytedword64 [wmi:      evex.scc0.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2862, 349, 2288},
    /* 1965 : CCMPS spec4,rm64,sbytedword64 [wmi:      evex.scc8.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2873, 349, 2288},
    /* 1966 : CCMPT spec4,rm64,sbytedword64 [wmi:      evex.scc10.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2884, 349, 2288},
    /* 1967 : CCMPZ spec4,rm64,sbytedword64 [wmi:      evex.scc4.dfv.l0.m4.o64 83 /7 ib,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(83) 217 276 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SBYTEDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+2774, 349, 2288},
    /* 1968 : CCMPA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14093, 347, 2289},
    /* 1969 : CCMPC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14113, 347, 2289},
    /* 1970 : CCMPF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14083, 347, 2289},
    /* 1971 : CCMPG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14143, 347, 2289},
    /* 1972 : CCMPL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14163, 347, 2289},
    /* 1973 : CCMPNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14123, 347, 2289},
    /* 1974 : CCMPNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14103, 347, 2289},
    /* 1975 : CCMPNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14173, 347, 2289},
    /* 1976 : CCMPNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14153, 347, 2289},
    /* 1977 : CCMPNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14193, 347, 2289},
    /* 1978 : CCMPNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14203, 347, 2289},
    /* 1979 : CCMPNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14183, 347, 2289},
    /* 1980 : CCMPO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14213, 347, 2289},
    /* 1981 : CCMPS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14223, 347, 2289},
    /* 1982 : CCMPT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14233, 347, 2289},
    /* 1983 : CCMPZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 80 /7 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(80) 217 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14133, 347, 2289},
    /* 1984 : CCMPA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2906, 348, 2290},
    /* 1985 : CCMPC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2928, 348, 2290},
    /* 1986 : CCMPF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2895, 348, 2290},
    /* 1987 : CCMPG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2961, 348, 2290},
    /* 1988 : CCMPL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2983, 348, 2290},
    /* 1989 : CCMPNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2939, 348, 2290},
    /* 1990 : CCMPNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2917, 348, 2290},
    /* 1991 : CCMPNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2994, 348, 2290},
    /* 1992 : CCMPNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2972, 348, 2290},
    /* 1993 : CCMPNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3016, 348, 2290},
    /* 1994 : CCMPNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3027, 348, 2290},
    /* 1995 : CCMPNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3005, 348, 2290},
    /* 1996 : CCMPO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3038, 348, 2290},
    /* 1997 : CCMPS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3049, 348, 2290},
    /* 1998 : CCMPT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3060, 348, 2290},
    /* 1999 : CCMPZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 81 /7 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(81) 217 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+2950, 348, 2290},
    /* 2000 : CCMPA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3082, 349, 2291},
    /* 2001 : CCMPC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3104, 349, 2291},
    /* 2002 : CCMPF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3071, 349, 2291},
    /* 2003 : CCMPG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3137, 349, 2291},
    /* 2004 : CCMPL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3159, 349, 2291},
    /* 2005 : CCMPNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3115, 349, 2291},
    /* 2006 : CCMPNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3093, 349, 2291},
    /* 2007 : CCMPNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3170, 349, 2291},
    /* 2008 : CCMPNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3148, 349, 2291},
    /* 2009 : CCMPNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3192, 349, 2291},
    /* 2010 : CCMPNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3203, 349, 2291},
    /* 2011 : CCMPNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3181, 349, 2291},
    /* 2012 : CCMPO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3214, 349, 2291},
    /* 2013 : CCMPS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3225, 349, 2291},
    /* 2014 : CCMPT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3236, 349, 2291},
    /* 2015 : CCMPZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 81 /7 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(81) 217 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3126, 349, 2291},
    /* 2016 : CCMPA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3258, 349, 2292},
    /* 2017 : CCMPC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3280, 349, 2292},
    /* 2018 : CCMPF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3247, 349, 2292},
    /* 2019 : CCMPG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3313, 349, 2292},
    /* 2020 : CCMPL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3335, 349, 2292},
    /* 2021 : CCMPNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3291, 349, 2292},
    /* 2022 : CCMPNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3269, 349, 2292},
    /* 2023 : CCMPNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3346, 349, 2292},
    /* 2024 : CCMPNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3324, 349, 2292},
    /* 2025 : CCMPNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3368, 349, 2292},
    /* 2026 : CCMPNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3379, 349, 2292},
    /* 2027 : CCMPNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3357, 349, 2292},
    /* 2028 : CCMPO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3390, 349, 2292},
    /* 2029 : CCMPS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3401, 349, 2292},
    /* 2030 : CCMPT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3412, 349, 2292},
    /* 2031 : CCMPZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 81 /7 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(81) 217 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CCMPZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3302, 349, 2292},
    /* 2032 : CTESTA spec4,rm8,reg8 [wmr:      evex.scc7.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25006, 347, 2294},
    /* 2033 : CTESTC spec4,rm8,reg8 [wmr:      evex.scc2.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25024, 347, 2294},
    /* 2034 : CTESTF spec4,rm8,reg8 [wmr:      evex.scc11.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+24997, 347, 2294},
    /* 2035 : CTESTG spec4,rm8,reg8 [wmr:      evex.scc15.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25051, 347, 2294},
    /* 2036 : CTESTL spec4,rm8,reg8 [wmr:      evex.scc12.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25069, 347, 2294},
    /* 2037 : CTESTNA spec4,rm8,reg8 [wmr:      evex.scc6.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25033, 347, 2294},
    /* 2038 : CTESTNC spec4,rm8,reg8 [wmr:      evex.scc3.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25015, 347, 2294},
    /* 2039 : CTESTNG spec4,rm8,reg8 [wmr:      evex.scc14.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25078, 347, 2294},
    /* 2040 : CTESTNL spec4,rm8,reg8 [wmr:      evex.scc13.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25060, 347, 2294},
    /* 2041 : CTESTNO spec4,rm8,reg8 [wmr:      evex.scc1.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25096, 347, 2294},
    /* 2042 : CTESTNS spec4,rm8,reg8 [wmr:      evex.scc9.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25105, 347, 2294},
    /* 2043 : CTESTNZ spec4,rm8,reg8 [wmr:      evex.scc5.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25087, 347, 2294},
    /* 2044 : CTESTO spec4,rm8,reg8 [wmr:      evex.scc0.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25114, 347, 2294},
    /* 2045 : CTESTS spec4,rm8,reg8 [wmr:      evex.scc8.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25123, 347, 2294},
    /* 2046 : CTESTT spec4,rm8,reg8 [wmr:      evex.scc10.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25132, 347, 2294},
    /* 2047 : CTESTZ spec4,rm8,reg8 [wmr:      evex.scc4.dfv.l0.m4.o8 84 /r       ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(84) 112 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,REG_GPR|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+25042, 347, 2294},
    /* 2048 : CTESTA spec4,rm16,reg16 [wmr:      evex.scc7.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14253, 348, 2295},
    /* 2049 : CTESTC spec4,rm16,reg16 [wmr:      evex.scc2.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14273, 348, 2295},
    /* 2050 : CTESTF spec4,rm16,reg16 [wmr:      evex.scc11.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14243, 348, 2295},
    /* 2051 : CTESTG spec4,rm16,reg16 [wmr:      evex.scc15.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14303, 348, 2295},
    /* 2052 : CTESTL spec4,rm16,reg16 [wmr:      evex.scc12.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14323, 348, 2295},
    /* 2053 : CTESTNA spec4,rm16,reg16 [wmr:      evex.scc6.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14283, 348, 2295},
    /* 2054 : CTESTNC spec4,rm16,reg16 [wmr:      evex.scc3.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14263, 348, 2295},
    /* 2055 : CTESTNG spec4,rm16,reg16 [wmr:      evex.scc14.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14333, 348, 2295},
    /* 2056 : CTESTNL spec4,rm16,reg16 [wmr:      evex.scc13.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14313, 348, 2295},
    /* 2057 : CTESTNO spec4,rm16,reg16 [wmr:      evex.scc1.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14353, 348, 2295},
    /* 2058 : CTESTNS spec4,rm16,reg16 [wmr:      evex.scc9.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14363, 348, 2295},
    /* 2059 : CTESTNZ spec4,rm16,reg16 [wmr:      evex.scc5.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14343, 348, 2295},
    /* 2060 : CTESTO spec4,rm16,reg16 [wmr:      evex.scc0.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14373, 348, 2295},
    /* 2061 : CTESTS spec4,rm16,reg16 [wmr:      evex.scc8.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14383, 348, 2295},
    /* 2062 : CTESTT spec4,rm16,reg16 [wmr:      evex.scc10.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14393, 348, 2295},
    /* 2063 : CTESTZ spec4,rm16,reg16 [wmr:      evex.scc4.dfv.l0.m4.o16 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(85) 112 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,REG_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+14293, 348, 2295},
    /* 2064 : CTESTA spec4,rm32,reg32 [wmr:      evex.scc7.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14413, 349, 2296},
    /* 2065 : CTESTC spec4,rm32,reg32 [wmr:      evex.scc2.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14433, 349, 2296},
    /* 2066 : CTESTF spec4,rm32,reg32 [wmr:      evex.scc11.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14403, 349, 2296},
    /* 2067 : CTESTG spec4,rm32,reg32 [wmr:      evex.scc15.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14463, 349, 2296},
    /* 2068 : CTESTL spec4,rm32,reg32 [wmr:      evex.scc12.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14483, 349, 2296},
    /* 2069 : CTESTNA spec4,rm32,reg32 [wmr:      evex.scc6.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14443, 349, 2296},
    /* 2070 : CTESTNC spec4,rm32,reg32 [wmr:      evex.scc3.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14423, 349, 2296},
    /* 2071 : CTESTNG spec4,rm32,reg32 [wmr:      evex.scc14.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14493, 349, 2296},
    /* 2072 : CTESTNL spec4,rm32,reg32 [wmr:      evex.scc13.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14473, 349, 2296},
    /* 2073 : CTESTNO spec4,rm32,reg32 [wmr:      evex.scc1.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14513, 349, 2296},
    /* 2074 : CTESTNS spec4,rm32,reg32 [wmr:      evex.scc9.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14523, 349, 2296},
    /* 2075 : CTESTNZ spec4,rm32,reg32 [wmr:      evex.scc5.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14503, 349, 2296},
    /* 2076 : CTESTO spec4,rm32,reg32 [wmr:      evex.scc0.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14533, 349, 2296},
    /* 2077 : CTESTS spec4,rm32,reg32 [wmr:      evex.scc8.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14543, 349, 2296},
    /* 2078 : CTESTT spec4,rm32,reg32 [wmr:      evex.scc10.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14553, 349, 2296},
    /* 2079 : CTESTZ spec4,rm32,reg32 [wmr:      evex.scc4.dfv.l0.m4.o32 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+14453, 349, 2296},
    /* 2080 : CTESTA spec4,rm64,reg64 [wmr:      evex.scc7.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14573, 349, 2297},
    /* 2081 : CTESTC spec4,rm64,reg64 [wmr:      evex.scc2.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14593, 349, 2297},
    /* 2082 : CTESTF spec4,rm64,reg64 [wmr:      evex.scc11.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14563, 349, 2297},
    /* 2083 : CTESTG spec4,rm64,reg64 [wmr:      evex.scc15.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14623, 349, 2297},
    /* 2084 : CTESTL spec4,rm64,reg64 [wmr:      evex.scc12.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14643, 349, 2297},
    /* 2085 : CTESTNA spec4,rm64,reg64 [wmr:      evex.scc6.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14603, 349, 2297},
    /* 2086 : CTESTNC spec4,rm64,reg64 [wmr:      evex.scc3.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14583, 349, 2297},
    /* 2087 : CTESTNG spec4,rm64,reg64 [wmr:      evex.scc14.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14653, 349, 2297},
    /* 2088 : CTESTNL spec4,rm64,reg64 [wmr:      evex.scc13.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14633, 349, 2297},
    /* 2089 : CTESTNO spec4,rm64,reg64 [wmr:      evex.scc1.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14673, 349, 2297},
    /* 2090 : CTESTNS spec4,rm64,reg64 [wmr:      evex.scc9.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14683, 349, 2297},
    /* 2091 : CTESTNZ spec4,rm64,reg64 [wmr:      evex.scc5.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14663, 349, 2297},
    /* 2092 : CTESTO spec4,rm64,reg64 [wmr:      evex.scc0.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14693, 349, 2297},
    /* 2093 : CTESTS spec4,rm64,reg64 [wmr:      evex.scc8.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14703, 349, 2297},
    /* 2094 : CTESTT spec4,rm64,reg64 [wmr:      evex.scc10.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14713, 349, 2297},
    /* 2095 : CTESTZ spec4,rm64,reg64 [wmr:      evex.scc4.dfv.l0.m4.o64 85 /r      ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(85) 112 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+14613, 349, 2297},
    /* 2096 : CTESTA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14733, 347, 2298},
    /* 2097 : CTESTC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14753, 347, 2298},
    /* 2098 : CTESTF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14723, 347, 2298},
    /* 2099 : CTESTG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14783, 347, 2298},
    /* 2100 : CTESTL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14803, 347, 2298},
    /* 2101 : CTESTNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14763, 347, 2298},
    /* 2102 : CTESTNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14743, 347, 2298},
    /* 2103 : CTESTNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14813, 347, 2298},
    /* 2104 : CTESTNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14793, 347, 2298},
    /* 2105 : CTESTNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14833, 347, 2298},
    /* 2106 : CTESTNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14843, 347, 2298},
    /* 2107 : CTESTNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14823, 347, 2298},
    /* 2108 : CTESTO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14853, 347, 2298},
    /* 2109 : CTESTS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14863, 347, 2298},
    /* 2110 : CTESTT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14873, 347, 2298},
    /* 2111 : CTESTZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 f6 /0 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(f6) 210 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14773, 347, 2298},
    /* 2112 : CTESTA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3434, 348, 2299},
    /* 2113 : CTESTC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3456, 348, 2299},
    /* 2114 : CTESTF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3423, 348, 2299},
    /* 2115 : CTESTG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3489, 348, 2299},
    /* 2116 : CTESTL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3511, 348, 2299},
    /* 2117 : CTESTNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3467, 348, 2299},
    /* 2118 : CTESTNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3445, 348, 2299},
    /* 2119 : CTESTNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3522, 348, 2299},
    /* 2120 : CTESTNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3500, 348, 2299},
    /* 2121 : CTESTNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3544, 348, 2299},
    /* 2122 : CTESTNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3555, 348, 2299},
    /* 2123 : CTESTNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3533, 348, 2299},
    /* 2124 : CTESTO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3566, 348, 2299},
    /* 2125 : CTESTS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3577, 348, 2299},
    /* 2126 : CTESTT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3588, 348, 2299},
    /* 2127 : CTESTZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 f7 /0 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(f7) 210 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3478, 348, 2299},
    /* 2128 : CTESTA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3610, 349, 2300},
    /* 2129 : CTESTC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3632, 349, 2300},
    /* 2130 : CTESTF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3599, 349, 2300},
    /* 2131 : CTESTG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3665, 349, 2300},
    /* 2132 : CTESTL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3687, 349, 2300},
    /* 2133 : CTESTNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3643, 349, 2300},
    /* 2134 : CTESTNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3621, 349, 2300},
    /* 2135 : CTESTNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3698, 349, 2300},
    /* 2136 : CTESTNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3676, 349, 2300},
    /* 2137 : CTESTNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3720, 349, 2300},
    /* 2138 : CTESTNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3731, 349, 2300},
    /* 2139 : CTESTNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3709, 349, 2300},
    /* 2140 : CTESTO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3742, 349, 2300},
    /* 2141 : CTESTS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3753, 349, 2300},
    /* 2142 : CTESTT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3764, 349, 2300},
    /* 2143 : CTESTZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 f7 /0 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(f7) 210 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+3654, 349, 2300},
    /* 2144 : CTESTA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3786, 349, 2301},
    /* 2145 : CTESTC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3808, 349, 2301},
    /* 2146 : CTESTF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3775, 349, 2301},
    /* 2147 : CTESTG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3841, 349, 2301},
    /* 2148 : CTESTL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3863, 349, 2301},
    /* 2149 : CTESTNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3819, 349, 2301},
    /* 2150 : CTESTNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3797, 349, 2301},
    /* 2151 : CTESTNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3874, 349, 2301},
    /* 2152 : CTESTNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3852, 349, 2301},
    /* 2153 : CTESTNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3896, 349, 2301},
    /* 2154 : CTESTNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3907, 349, 2301},
    /* 2155 : CTESTNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3885, 349, 2301},
    /* 2156 : CTESTO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3918, 349, 2301},
    /* 2157 : CTESTS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3929, 349, 2301},
    /* 2158 : CTESTT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3940, 349, 2301},
    /* 2159 : CTESTZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 f7 /0 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(f7) 210 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+3830, 349, 2301},
    /* 2160 : CTESTA spec4,rm8,imm8 [wmi:      evex.scc7.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14893, 347, 2302},
    /* 2161 : CTESTC spec4,rm8,imm8 [wmi:      evex.scc2.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14913, 347, 2302},
    /* 2162 : CTESTF spec4,rm8,imm8 [wmi:      evex.scc11.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14883, 347, 2302},
    /* 2163 : CTESTG spec4,rm8,imm8 [wmi:      evex.scc15.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14943, 347, 2302},
    /* 2164 : CTESTL spec4,rm8,imm8 [wmi:      evex.scc12.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14963, 347, 2302},
    /* 2165 : CTESTNA spec4,rm8,imm8 [wmi:      evex.scc6.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14923, 347, 2302},
    /* 2166 : CTESTNC spec4,rm8,imm8 [wmi:      evex.scc3.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14903, 347, 2302},
    /* 2167 : CTESTNG spec4,rm8,imm8 [wmi:      evex.scc14.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14973, 347, 2302},
    /* 2168 : CTESTNL spec4,rm8,imm8 [wmi:      evex.scc13.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14953, 347, 2302},
    /* 2169 : CTESTNO spec4,rm8,imm8 [wmi:      evex.scc1.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14993, 347, 2302},
    /* 2170 : CTESTNS spec4,rm8,imm8 [wmi:      evex.scc9.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15003, 347, 2302},
    /* 2171 : CTESTNZ spec4,rm8,imm8 [wmi:      evex.scc5.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14983, 347, 2302},
    /* 2172 : CTESTO spec4,rm8,imm8 [wmi:      evex.scc0.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15013, 347, 2302},
    /* 2173 : CTESTS spec4,rm8,imm8 [wmi:      evex.scc8.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15023, 347, 2302},
    /* 2174 : CTESTT spec4,rm8,imm8 [wmi:      evex.scc10.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+15033, 347, 2302},
    /* 2175 : CTESTZ spec4,rm8,imm8 [wmi:      evex.scc4.dfv.l0.m4.o8 f6 /1 ib    ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 001(f6) 211 022 : SM1,SM2,WIG,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+14933, 347, 2302},
    /* 2176 : CTESTA spec4,rm16,imm16 [wmi:      evex.scc7.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 07) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3962, 348, 2303},
    /* 2177 : CTESTC spec4,rm16,imm16 [wmi:      evex.scc2.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 02) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3984, 348, 2303},
    /* 2178 : CTESTF spec4,rm16,imm16 [wmi:      evex.scc11.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0b) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3951, 348, 2303},
    /* 2179 : CTESTG spec4,rm16,imm16 [wmi:      evex.scc15.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0f) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4017, 348, 2303},
    /* 2180 : CTESTL spec4,rm16,imm16 [wmi:      evex.scc12.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0c) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4039, 348, 2303},
    /* 2181 : CTESTNA spec4,rm16,imm16 [wmi:      evex.scc6.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 06) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3995, 348, 2303},
    /* 2182 : CTESTNC spec4,rm16,imm16 [wmi:      evex.scc3.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 03) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+3973, 348, 2303},
    /* 2183 : CTESTNG spec4,rm16,imm16 [wmi:      evex.scc14.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0e) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4050, 348, 2303},
    /* 2184 : CTESTNL spec4,rm16,imm16 [wmi:      evex.scc13.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0d) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4028, 348, 2303},
    /* 2185 : CTESTNO spec4,rm16,imm16 [wmi:      evex.scc1.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 01) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4072, 348, 2303},
    /* 2186 : CTESTNS spec4,rm16,imm16 [wmi:      evex.scc9.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 09) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4083, 348, 2303},
    /* 2187 : CTESTNZ spec4,rm16,imm16 [wmi:      evex.scc5.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 05) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4061, 348, 2303},
    /* 2188 : CTESTO spec4,rm16,imm16 [wmi:      evex.scc0.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 00) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4094, 348, 2303},
    /* 2189 : CTESTS spec4,rm16,imm16 [wmi:      evex.scc8.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 08) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4105, 348, 2303},
    /* 2190 : CTESTT spec4,rm16,imm16 [wmi:      evex.scc10.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 0a) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4116, 348, 2303},
    /* 2191 : CTESTZ spec4,rm16,imm16 [wmi:      evex.scc4.dfv.l0.m4.o16 f7 /1 iw   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 05 04) 300 320 001(f7) 211 032 : SM1,SM2,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS16,IMM_NORMAL|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+4006, 348, 2303},
    /* 2192 : CTESTA spec4,rm32,imm32 [wmi:      evex.scc7.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 07) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4138, 349, 2304},
    /* 2193 : CTESTC spec4,rm32,imm32 [wmi:      evex.scc2.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 02) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4160, 349, 2304},
    /* 2194 : CTESTF spec4,rm32,imm32 [wmi:      evex.scc11.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0b) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4127, 349, 2304},
    /* 2195 : CTESTG spec4,rm32,imm32 [wmi:      evex.scc15.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0f) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4193, 349, 2304},
    /* 2196 : CTESTL spec4,rm32,imm32 [wmi:      evex.scc12.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0c) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4215, 349, 2304},
    /* 2197 : CTESTNA spec4,rm32,imm32 [wmi:      evex.scc6.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 06) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4171, 349, 2304},
    /* 2198 : CTESTNC spec4,rm32,imm32 [wmi:      evex.scc3.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 03) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4149, 349, 2304},
    /* 2199 : CTESTNG spec4,rm32,imm32 [wmi:      evex.scc14.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0e) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4226, 349, 2304},
    /* 2200 : CTESTNL spec4,rm32,imm32 [wmi:      evex.scc13.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0d) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4204, 349, 2304},
    /* 2201 : CTESTNO spec4,rm32,imm32 [wmi:      evex.scc1.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 01) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4248, 349, 2304},
    /* 2202 : CTESTNS spec4,rm32,imm32 [wmi:      evex.scc9.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 09) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4259, 349, 2304},
    /* 2203 : CTESTNZ spec4,rm32,imm32 [wmi:      evex.scc5.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 05) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4237, 349, 2304},
    /* 2204 : CTESTO spec4,rm32,imm32 [wmi:      evex.scc0.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 00) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4270, 349, 2304},
    /* 2205 : CTESTS spec4,rm32,imm32 [wmi:      evex.scc8.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 08) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4281, 349, 2304},
    /* 2206 : CTESTT spec4,rm32,imm32 [wmi:      evex.scc10.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 0a) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4292, 349, 2304},
    /* 2207 : CTESTZ spec4,rm32,imm32 [wmi:      evex.scc4.dfv.l0.m4.o32 f7 /1 id   ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 04 04) 300 321 001(f7) 211 042 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS32,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+4182, 349, 2304},
    /* 2208 : CTESTA spec4,rm64,sdword64 [wmi:      evex.scc7.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 07) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4314, 349, 2305},
    /* 2209 : CTESTC spec4,rm64,sdword64 [wmi:      evex.scc2.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 02) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4336, 349, 2305},
    /* 2210 : CTESTF spec4,rm64,sdword64 [wmi:      evex.scc11.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0b) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTF, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4303, 349, 2305},
    /* 2211 : CTESTG spec4,rm64,sdword64 [wmi:      evex.scc15.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0f) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4369, 349, 2305},
    /* 2212 : CTESTL spec4,rm64,sdword64 [wmi:      evex.scc12.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0c) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4391, 349, 2305},
    /* 2213 : CTESTNA spec4,rm64,sdword64 [wmi:      evex.scc6.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 06) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNA, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4347, 349, 2305},
    /* 2214 : CTESTNC spec4,rm64,sdword64 [wmi:      evex.scc3.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 03) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNC, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4325, 349, 2305},
    /* 2215 : CTESTNG spec4,rm64,sdword64 [wmi:      evex.scc14.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0e) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNG, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4402, 349, 2305},
    /* 2216 : CTESTNL spec4,rm64,sdword64 [wmi:      evex.scc13.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0d) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNL, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4380, 349, 2305},
    /* 2217 : CTESTNO spec4,rm64,sdword64 [wmi:      evex.scc1.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 01) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4424, 349, 2305},
    /* 2218 : CTESTNS spec4,rm64,sdword64 [wmi:      evex.scc9.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 09) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4435, 349, 2305},
    /* 2219 : CTESTNZ spec4,rm64,sdword64 [wmi:      evex.scc5.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 05) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTNZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4413, 349, 2305},
    /* 2220 : CTESTO spec4,rm64,sdword64 [wmi:      evex.scc0.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 00) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTO, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4446, 349, 2305},
    /* 2221 : CTESTS spec4,rm64,sdword64 [wmi:      evex.scc8.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 08) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTS, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4457, 349, 2305},
    /* 2222 : CTESTT spec4,rm64,sdword64 [wmi:      evex.scc10.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 0a) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTT, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4468, 349, 2305},
    /* 2223 : CTESTZ spec4,rm64,sdword64 [wmi:      evex.scc4.dfv.l0.m4.o64 f7 /1 id,s ] SM1-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 84 04) 300 324 001(f7) 211 256 : SM1,SM2,WW,LONG,SCC,DFV,EVEX,PROT,APX,X86_64 */
        {I_CTESTZ, 3, {IMMEDIATE|FOURBITS,RM_GPR|BITS64,SDWORD|BITS64|IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+4358, 349, 2305},
    /* 2224 : CMOVA reg16,rm16 [rm:       o16 0f 47 /r                    ] SM0-1,P6 */
        /* 320 355 001(47) 110 : SM0,SM1,P6 */
        {I_CMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68067, 350, 2308},
    /* 2225 : CMOVC reg16,rm16 [rm:       o16 0f 42 /r                    ] SM0-1,P6 */
        /* 320 355 001(42) 110 : SM0,SM1,P6 */
        {I_CMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68079, 350, 2308},
    /* 2226 : CMOVG reg16,rm16 [rm:       o16 0f 4f /r                    ] SM0-1,P6 */
        /* 320 355 001(4f) 110 : SM0,SM1,P6 */
        {I_CMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68097, 350, 2308},
    /* 2227 : CMOVL reg16,rm16 [rm:       o16 0f 4c /r                    ] SM0-1,P6 */
        /* 320 355 001(4c) 110 : SM0,SM1,P6 */
        {I_CMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68109, 350, 2308},
    /* 2228 : CMOVNA reg16,rm16 [rm:       o16 0f 46 /r                    ] SM0-1,P6 */
        /* 320 355 001(46) 110 : SM0,SM1,P6 */
        {I_CMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68085, 350, 2308},
    /* 2229 : CMOVNC reg16,rm16 [rm:       o16 0f 43 /r                    ] SM0-1,P6 */
        /* 320 355 001(43) 110 : SM0,SM1,P6 */
        {I_CMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68073, 350, 2308},
    /* 2230 : CMOVNG reg16,rm16 [rm:       o16 0f 4e /r                    ] SM0-1,P6 */
        /* 320 355 001(4e) 110 : SM0,SM1,P6 */
        {I_CMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68115, 350, 2308},
    /* 2231 : CMOVNL reg16,rm16 [rm:       o16 0f 4d /r                    ] SM0-1,P6 */
        /* 320 355 001(4d) 110 : SM0,SM1,P6 */
        {I_CMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68103, 350, 2308},
    /* 2232 : CMOVNO reg16,rm16 [rm:       o16 0f 41 /r                    ] SM0-1,P6 */
        /* 320 355 001(41) 110 : SM0,SM1,P6 */
        {I_CMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68127, 350, 2308},
    /* 2233 : CMOVNS reg16,rm16 [rm:       o16 0f 49 /r                    ] SM0-1,P6 */
        /* 320 355 001(49) 110 : SM0,SM1,P6 */
        {I_CMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68139, 350, 2308},
    /* 2234 : CMOVNZ reg16,rm16 [rm:       o16 0f 45 /r                    ] SM0-1,P6 */
        /* 320 355 001(45) 110 : SM0,SM1,P6 */
        {I_CMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68121, 350, 2308},
    /* 2235 : CMOVO reg16,rm16 [rm:       o16 0f 40 /r                    ] SM0-1,P6 */
        /* 320 355 001(40) 110 : SM0,SM1,P6 */
        {I_CMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68145, 350, 2308},
    /* 2236 : CMOVPE reg16,rm16 [rm:       o16 0f 4a /r                    ] SM0-1,P6 */
        /* 320 355 001(4a) 110 : SM0,SM1,P6 */
        {I_CMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68151, 350, 2308},
    /* 2237 : CMOVPO reg16,rm16 [rm:       o16 0f 4b /r                    ] SM0-1,P6 */
        /* 320 355 001(4b) 110 : SM0,SM1,P6 */
        {I_CMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68133, 350, 2308},
    /* 2238 : CMOVS reg16,rm16 [rm:       o16 0f 48 /r                    ] SM0-1,P6 */
        /* 320 355 001(48) 110 : SM0,SM1,P6 */
        {I_CMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68157, 350, 2308},
    /* 2239 : CMOVZ reg16,rm16 [rm:       o16 0f 44 /r                    ] SM0-1,P6 */
        /* 320 355 001(44) 110 : SM0,SM1,P6 */
        {I_CMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+68091, 350, 2308},
    /* 2240 : CMOVA reg32,rm32 [rm:       o32 0f 47 /r                    ] SM0-1,P6 */
        /* 321 355 001(47) 110 : SM0,SM1,P6 */
        {I_CMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68163, 350, 2309},
    /* 2241 : CMOVC reg32,rm32 [rm:       o32 0f 42 /r                    ] SM0-1,P6 */
        /* 321 355 001(42) 110 : SM0,SM1,P6 */
        {I_CMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68175, 350, 2309},
    /* 2242 : CMOVG reg32,rm32 [rm:       o32 0f 4f /r                    ] SM0-1,P6 */
        /* 321 355 001(4f) 110 : SM0,SM1,P6 */
        {I_CMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68193, 350, 2309},
    /* 2243 : CMOVL reg32,rm32 [rm:       o32 0f 4c /r                    ] SM0-1,P6 */
        /* 321 355 001(4c) 110 : SM0,SM1,P6 */
        {I_CMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68205, 350, 2309},
    /* 2244 : CMOVNA reg32,rm32 [rm:       o32 0f 46 /r                    ] SM0-1,P6 */
        /* 321 355 001(46) 110 : SM0,SM1,P6 */
        {I_CMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68181, 350, 2309},
    /* 2245 : CMOVNC reg32,rm32 [rm:       o32 0f 43 /r                    ] SM0-1,P6 */
        /* 321 355 001(43) 110 : SM0,SM1,P6 */
        {I_CMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68169, 350, 2309},
    /* 2246 : CMOVNG reg32,rm32 [rm:       o32 0f 4e /r                    ] SM0-1,P6 */
        /* 321 355 001(4e) 110 : SM0,SM1,P6 */
        {I_CMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68211, 350, 2309},
    /* 2247 : CMOVNL reg32,rm32 [rm:       o32 0f 4d /r                    ] SM0-1,P6 */
        /* 321 355 001(4d) 110 : SM0,SM1,P6 */
        {I_CMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68199, 350, 2309},
    /* 2248 : CMOVNO reg32,rm32 [rm:       o32 0f 41 /r                    ] SM0-1,P6 */
        /* 321 355 001(41) 110 : SM0,SM1,P6 */
        {I_CMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68223, 350, 2309},
    /* 2249 : CMOVNS reg32,rm32 [rm:       o32 0f 49 /r                    ] SM0-1,P6 */
        /* 321 355 001(49) 110 : SM0,SM1,P6 */
        {I_CMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68235, 350, 2309},
    /* 2250 : CMOVNZ reg32,rm32 [rm:       o32 0f 45 /r                    ] SM0-1,P6 */
        /* 321 355 001(45) 110 : SM0,SM1,P6 */
        {I_CMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68217, 350, 2309},
    /* 2251 : CMOVO reg32,rm32 [rm:       o32 0f 40 /r                    ] SM0-1,P6 */
        /* 321 355 001(40) 110 : SM0,SM1,P6 */
        {I_CMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68241, 350, 2309},
    /* 2252 : CMOVPE reg32,rm32 [rm:       o32 0f 4a /r                    ] SM0-1,P6 */
        /* 321 355 001(4a) 110 : SM0,SM1,P6 */
        {I_CMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68247, 350, 2309},
    /* 2253 : CMOVPO reg32,rm32 [rm:       o32 0f 4b /r                    ] SM0-1,P6 */
        /* 321 355 001(4b) 110 : SM0,SM1,P6 */
        {I_CMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68229, 350, 2309},
    /* 2254 : CMOVS reg32,rm32 [rm:       o32 0f 48 /r                    ] SM0-1,P6 */
        /* 321 355 001(48) 110 : SM0,SM1,P6 */
        {I_CMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68253, 350, 2309},
    /* 2255 : CMOVZ reg32,rm32 [rm:       o32 0f 44 /r                    ] SM0-1,P6 */
        /* 321 355 001(44) 110 : SM0,SM1,P6 */
        {I_CMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68187, 350, 2309},
    /* 2256 : CMOVA reg64,rm64 [rm:       o64 0f 47 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(47) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68259, 14, 2310},
    /* 2257 : CMOVC reg64,rm64 [rm:       o64 0f 42 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(42) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68271, 14, 2310},
    /* 2258 : CMOVG reg64,rm64 [rm:       o64 0f 4f /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4f) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68289, 14, 2310},
    /* 2259 : CMOVL reg64,rm64 [rm:       o64 0f 4c /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4c) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68301, 14, 2310},
    /* 2260 : CMOVNA reg64,rm64 [rm:       o64 0f 46 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(46) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68277, 14, 2310},
    /* 2261 : CMOVNC reg64,rm64 [rm:       o64 0f 43 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(43) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68265, 14, 2310},
    /* 2262 : CMOVNG reg64,rm64 [rm:       o64 0f 4e /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4e) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68307, 14, 2310},
    /* 2263 : CMOVNL reg64,rm64 [rm:       o64 0f 4d /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4d) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68295, 14, 2310},
    /* 2264 : CMOVNO reg64,rm64 [rm:       o64 0f 41 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(41) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68319, 14, 2310},
    /* 2265 : CMOVNS reg64,rm64 [rm:       o64 0f 49 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(49) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68331, 14, 2310},
    /* 2266 : CMOVNZ reg64,rm64 [rm:       o64 0f 45 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(45) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68313, 14, 2310},
    /* 2267 : CMOVO reg64,rm64 [rm:       o64 0f 40 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(40) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68337, 14, 2310},
    /* 2268 : CMOVPE reg64,rm64 [rm:       o64 0f 4a /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4a) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68343, 14, 2310},
    /* 2269 : CMOVPO reg64,rm64 [rm:       o64 0f 4b /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(4b) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68325, 14, 2310},
    /* 2270 : CMOVS reg64,rm64 [rm:       o64 0f 48 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(48) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68349, 14, 2310},
    /* 2271 : CMOVZ reg64,rm64 [rm:       o64 0f 44 /r                    ] SM0-1,LONG,PROT,X86_64 */
        /* 324 355 001(44) 110 : SM0,SM1,LONG,PROT,X86_64 */
        {I_CMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68283, 14, 2310},
    /* 2272 : CMOVA reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(47) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15043, 351, 2311},
    /* 2273 : CMOVC reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(42) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15063, 351, 2311},
    /* 2274 : CMOVG reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4f) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15093, 351, 2311},
    /* 2275 : CMOVL reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4c) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15113, 351, 2311},
    /* 2276 : CMOVNA reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(46) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15073, 351, 2311},
    /* 2277 : CMOVNC reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(43) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15053, 351, 2311},
    /* 2278 : CMOVNG reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4e) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15123, 351, 2311},
    /* 2279 : CMOVNL reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4d) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15103, 351, 2311},
    /* 2280 : CMOVNO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(41) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15143, 351, 2311},
    /* 2281 : CMOVNS reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(49) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15163, 351, 2311},
    /* 2282 : CMOVNZ reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(45) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15133, 351, 2311},
    /* 2283 : CMOVO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(40) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15173, 351, 2311},
    /* 2284 : CMOVPE reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4a) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15183, 351, 2311},
    /* 2285 : CMOVPO reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(4b) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15153, 351, 2311},
    /* 2286 : CMOVS reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(48) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15193, 351, 2311},
    /* 2287 : CMOVZ reg16,reg16,rm16 [vrm:      evex.nf0.nd1.l0.m4.o16 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 18) 300 320 001(44) 121 : SM0,SM1,SM2,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+15083, 351, 2311},
    /* 2288 : CMOVA reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(47) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15203, 352, 2312},
    /* 2289 : CMOVC reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(42) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15223, 352, 2312},
    /* 2290 : CMOVG reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4f) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15253, 352, 2312},
    /* 2291 : CMOVL reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4c) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15273, 352, 2312},
    /* 2292 : CMOVNA reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(46) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15233, 352, 2312},
    /* 2293 : CMOVNC reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(43) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15213, 352, 2312},
    /* 2294 : CMOVNG reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4e) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15283, 352, 2312},
    /* 2295 : CMOVNL reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4d) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15263, 352, 2312},
    /* 2296 : CMOVNO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(41) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15303, 352, 2312},
    /* 2297 : CMOVNS reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(49) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15323, 352, 2312},
    /* 2298 : CMOVNZ reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(45) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15293, 352, 2312},
    /* 2299 : CMOVO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(40) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15333, 352, 2312},
    /* 2300 : CMOVPE reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4a) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15343, 352, 2312},
    /* 2301 : CMOVPO reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(4b) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15313, 352, 2312},
    /* 2302 : CMOVS reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(48) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15353, 352, 2312},
    /* 2303 : CMOVZ reg32,reg32,rm32 [vrm:      evex.nf0.nd1.l0.m4.o32 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 18) 300 321 001(44) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+15243, 352, 2312},
    /* 2304 : CMOVA reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 47 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(47) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15363, 352, 2313},
    /* 2305 : CMOVC reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 42 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(42) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15383, 352, 2313},
    /* 2306 : CMOVG reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4f /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4f) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15413, 352, 2313},
    /* 2307 : CMOVL reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4c /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4c) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15433, 352, 2313},
    /* 2308 : CMOVNA reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 46 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(46) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15393, 352, 2313},
    /* 2309 : CMOVNC reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 43 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(43) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15373, 352, 2313},
    /* 2310 : CMOVNG reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4e /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4e) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15443, 352, 2313},
    /* 2311 : CMOVNL reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4d /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4d) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15423, 352, 2313},
    /* 2312 : CMOVNO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 41 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(41) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15463, 352, 2313},
    /* 2313 : CMOVNS reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 49 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(49) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15483, 352, 2313},
    /* 2314 : CMOVNZ reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 45 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(45) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVNZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15453, 352, 2313},
    /* 2315 : CMOVO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 40 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(40) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15493, 352, 2313},
    /* 2316 : CMOVPE reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4a /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4a) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPE, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15503, 352, 2313},
    /* 2317 : CMOVPO reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 4b /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(4b) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVPO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15473, 352, 2313},
    /* 2318 : CMOVS reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 48 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(48) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15513, 352, 2313},
    /* 2319 : CMOVZ reg64,reg64,rm64 [vrm:      evex.nf0.nd1.l0.m4.o64 44 /r    ] SM0-2,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 18) 300 324 001(44) 121 : SM0,SM1,SM2,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CMOVZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+15403, 352, 2313},
    /* 2320 : CFCMOVA rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(47) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15523, 353, 2314},
    /* 2321 : CFCMOVC rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(42) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15543, 353, 2314},
    /* 2322 : CFCMOVG rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4f) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15573, 353, 2314},
    /* 2323 : CFCMOVL rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4c) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15593, 353, 2314},
    /* 2324 : CFCMOVNA rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(46) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15553, 353, 2314},
    /* 2325 : CFCMOVNC rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(43) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15533, 353, 2314},
    /* 2326 : CFCMOVNG rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4e) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15603, 353, 2314},
    /* 2327 : CFCMOVNL rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4d) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15583, 353, 2314},
    /* 2328 : CFCMOVNO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(41) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15623, 353, 2314},
    /* 2329 : CFCMOVNS rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(49) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15643, 353, 2314},
    /* 2330 : CFCMOVNZ rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(45) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15613, 353, 2314},
    /* 2331 : CFCMOVO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(40) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15653, 353, 2314},
    /* 2332 : CFCMOVPE rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4a) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15663, 353, 2314},
    /* 2333 : CFCMOVPO rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4b) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15633, 353, 2314},
    /* 2334 : CFCMOVS rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(48) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15673, 353, 2314},
    /* 2335 : CFCMOVZ rm16,reg16 [mr:       evex.nf1.nd0.l0.m4.o16 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(44) 101 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS16,REG_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+15563, 353, 2314},
    /* 2336 : CFCMOVA rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(47) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15683, 354, 2315},
    /* 2337 : CFCMOVC rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(42) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15703, 354, 2315},
    /* 2338 : CFCMOVG rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4f) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15733, 354, 2315},
    /* 2339 : CFCMOVL rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4c) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15753, 354, 2315},
    /* 2340 : CFCMOVNA rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(46) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15713, 354, 2315},
    /* 2341 : CFCMOVNC rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(43) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15693, 354, 2315},
    /* 2342 : CFCMOVNG rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4e) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15763, 354, 2315},
    /* 2343 : CFCMOVNL rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4d) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15743, 354, 2315},
    /* 2344 : CFCMOVNO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(41) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15783, 354, 2315},
    /* 2345 : CFCMOVNS rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(49) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15803, 354, 2315},
    /* 2346 : CFCMOVNZ rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(45) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15773, 354, 2315},
    /* 2347 : CFCMOVO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(40) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15813, 354, 2315},
    /* 2348 : CFCMOVPE rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4a) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15823, 354, 2315},
    /* 2349 : CFCMOVPO rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4b) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15793, 354, 2315},
    /* 2350 : CFCMOVS rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(48) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15833, 354, 2315},
    /* 2351 : CFCMOVZ rm32,reg32 [mr:       evex.nf1.nd0.l0.m4.o32 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(44) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+15723, 354, 2315},
    /* 2352 : CFCMOVA rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(47) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15843, 354, 2316},
    /* 2353 : CFCMOVC rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(42) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15863, 354, 2316},
    /* 2354 : CFCMOVG rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4f) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15893, 354, 2316},
    /* 2355 : CFCMOVL rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4c) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15913, 354, 2316},
    /* 2356 : CFCMOVNA rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(46) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15873, 354, 2316},
    /* 2357 : CFCMOVNC rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(43) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15853, 354, 2316},
    /* 2358 : CFCMOVNG rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4e) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15923, 354, 2316},
    /* 2359 : CFCMOVNL rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4d) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15903, 354, 2316},
    /* 2360 : CFCMOVNO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(41) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15943, 354, 2316},
    /* 2361 : CFCMOVNS rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(49) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15963, 354, 2316},
    /* 2362 : CFCMOVNZ rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(45) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15933, 354, 2316},
    /* 2363 : CFCMOVO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(40) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15973, 354, 2316},
    /* 2364 : CFCMOVPE rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4a) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15983, 354, 2316},
    /* 2365 : CFCMOVPO rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4b) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15953, 354, 2316},
    /* 2366 : CFCMOVS rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(48) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15993, 354, 2316},
    /* 2367 : CFCMOVZ rm64,reg64 [mr:       evex.nf1.nd0.l0.m4.o64 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(44) 101 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+15883, 354, 2316},
    /* 2368 : CFCMOVA reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(47) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16003, 353, 2317},
    /* 2369 : CFCMOVC reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(42) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16023, 353, 2317},
    /* 2370 : CFCMOVG reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4f) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16053, 353, 2317},
    /* 2371 : CFCMOVL reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4c) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16073, 353, 2317},
    /* 2372 : CFCMOVNA reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(46) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16033, 353, 2317},
    /* 2373 : CFCMOVNC reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(43) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16013, 353, 2317},
    /* 2374 : CFCMOVNG reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4e) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16083, 353, 2317},
    /* 2375 : CFCMOVNL reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4d) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16063, 353, 2317},
    /* 2376 : CFCMOVNO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(41) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16103, 353, 2317},
    /* 2377 : CFCMOVNS reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(49) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16123, 353, 2317},
    /* 2378 : CFCMOVNZ reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(45) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16093, 353, 2317},
    /* 2379 : CFCMOVO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(40) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16133, 353, 2317},
    /* 2380 : CFCMOVPE reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4a) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16143, 353, 2317},
    /* 2381 : CFCMOVPO reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(4b) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16113, 353, 2317},
    /* 2382 : CFCMOVS reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(48) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16153, 353, 2317},
    /* 2383 : CFCMOVZ reg16,rm16 [rm:       evex.nf0.nd0.l0.m4.o16 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(44) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16043, 353, 2317},
    /* 2384 : CFCMOVA reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16163, 354, 2318},
    /* 2385 : CFCMOVC reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16183, 354, 2318},
    /* 2386 : CFCMOVG reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16213, 354, 2318},
    /* 2387 : CFCMOVL reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16233, 354, 2318},
    /* 2388 : CFCMOVNA reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16193, 354, 2318},
    /* 2389 : CFCMOVNC reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16173, 354, 2318},
    /* 2390 : CFCMOVNG reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16243, 354, 2318},
    /* 2391 : CFCMOVNL reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16223, 354, 2318},
    /* 2392 : CFCMOVNO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16263, 354, 2318},
    /* 2393 : CFCMOVNS reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16283, 354, 2318},
    /* 2394 : CFCMOVNZ reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16253, 354, 2318},
    /* 2395 : CFCMOVO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16293, 354, 2318},
    /* 2396 : CFCMOVPE reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16303, 354, 2318},
    /* 2397 : CFCMOVPO reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16273, 354, 2318},
    /* 2398 : CFCMOVS reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16313, 354, 2318},
    /* 2399 : CFCMOVZ reg32,rm32 [rm:       evex.nf0.nd0.l0.m4.o32 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+16203, 354, 2318},
    /* 2400 : CFCMOVA reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 47 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16323, 354, 2319},
    /* 2401 : CFCMOVC reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 42 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16343, 354, 2319},
    /* 2402 : CFCMOVG reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4f /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16373, 354, 2319},
    /* 2403 : CFCMOVL reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4c /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16393, 354, 2319},
    /* 2404 : CFCMOVNA reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 46 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16353, 354, 2319},
    /* 2405 : CFCMOVNC reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 43 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16333, 354, 2319},
    /* 2406 : CFCMOVNG reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4e /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16403, 354, 2319},
    /* 2407 : CFCMOVNL reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4d /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16383, 354, 2319},
    /* 2408 : CFCMOVNO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 41 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16423, 354, 2319},
    /* 2409 : CFCMOVNS reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 49 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16443, 354, 2319},
    /* 2410 : CFCMOVNZ reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 45 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16413, 354, 2319},
    /* 2411 : CFCMOVO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 40 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16453, 354, 2319},
    /* 2412 : CFCMOVPE reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4a /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16463, 354, 2319},
    /* 2413 : CFCMOVPO reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 4b /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16433, 354, 2319},
    /* 2414 : CFCMOVS reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 48 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16473, 354, 2319},
    /* 2415 : CFCMOVZ reg64,rm64 [rm:       evex.nf0.nd0.l0.m4.o64 44 /r    ] SM0-1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+16363, 354, 2319},
    /* 2416 : CFCMOVA reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(47) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16483, 355, 2320},
    /* 2417 : CFCMOVC reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(42) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16503, 355, 2320},
    /* 2418 : CFCMOVG reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4f) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16533, 355, 2320},
    /* 2419 : CFCMOVL reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4c) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16553, 355, 2320},
    /* 2420 : CFCMOVNA reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(46) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16513, 355, 2320},
    /* 2421 : CFCMOVNC reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(43) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16493, 355, 2320},
    /* 2422 : CFCMOVNG reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4e) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16563, 355, 2320},
    /* 2423 : CFCMOVNL reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4d) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16543, 355, 2320},
    /* 2424 : CFCMOVNO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(41) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16583, 355, 2320},
    /* 2425 : CFCMOVNS reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(49) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16603, 355, 2320},
    /* 2426 : CFCMOVNZ reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(45) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16573, 355, 2320},
    /* 2427 : CFCMOVO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(40) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16613, 355, 2320},
    /* 2428 : CFCMOVPE reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4a) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16623, 355, 2320},
    /* 2429 : CFCMOVPO reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(4b) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16593, 355, 2320},
    /* 2430 : CFCMOVS reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(48) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16633, 355, 2320},
    /* 2431 : CFCMOVZ reg16?,reg16,rm16 [vrm:      evex.nf1.nd1.l0.m4.o16 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7d 1c) 300 320 001(44) 121 : SM0,SM1,SM2,ZU,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS16,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+16523, 355, 2320},
    /* 2432 : CFCMOVA reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(47) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16643, 353, 2320},
    /* 2433 : CFCMOVC reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(42) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16663, 353, 2320},
    /* 2434 : CFCMOVG reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4f) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16693, 353, 2320},
    /* 2435 : CFCMOVL reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4c) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16713, 353, 2320},
    /* 2436 : CFCMOVNA reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(46) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16673, 353, 2320},
    /* 2437 : CFCMOVNC reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(43) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16653, 353, 2320},
    /* 2438 : CFCMOVNG reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4e) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16723, 353, 2320},
    /* 2439 : CFCMOVNL reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4d) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16703, 353, 2320},
    /* 2440 : CFCMOVNO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(41) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16743, 353, 2320},
    /* 2441 : CFCMOVNS reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(49) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16763, 353, 2320},
    /* 2442 : CFCMOVNZ reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(45) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16733, 353, 2320},
    /* 2443 : CFCMOVO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(40) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16773, 353, 2320},
    /* 2444 : CFCMOVPE reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4a) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16783, 353, 2320},
    /* 2445 : CFCMOVPO reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(4b) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16753, 353, 2320},
    /* 2446 : CFCMOVS reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(48) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16793, 353, 2320},
    /* 2447 : CFCMOVZ reg16,rm16 [rm:      evex.nf1.nd0.l0.m4.o16 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7d 0c) 300 320 001(44) 110 : SM0,SM1,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+16683, 353, 2320},
    /* 2448 : CFCMOVA reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(47) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16963, 357, 2321},
    /* 2449 : CFCMOVC reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(42) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16983, 357, 2321},
    /* 2450 : CFCMOVG reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4f) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17013, 357, 2321},
    /* 2451 : CFCMOVL reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4c) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17033, 357, 2321},
    /* 2452 : CFCMOVNA reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(46) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16993, 357, 2321},
    /* 2453 : CFCMOVNC reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(43) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+16973, 357, 2321},
    /* 2454 : CFCMOVNG reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4e) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17043, 357, 2321},
    /* 2455 : CFCMOVNL reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4d) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17023, 357, 2321},
    /* 2456 : CFCMOVNO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(41) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17063, 357, 2321},
    /* 2457 : CFCMOVNS reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(49) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17083, 357, 2321},
    /* 2458 : CFCMOVNZ reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(45) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17053, 357, 2321},
    /* 2459 : CFCMOVO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(40) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17093, 357, 2321},
    /* 2460 : CFCMOVPE reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4a) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17103, 357, 2321},
    /* 2461 : CFCMOVPO reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(4b) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17073, 357, 2321},
    /* 2462 : CFCMOVS reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(48) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17113, 357, 2321},
    /* 2463 : CFCMOVZ reg32?,reg32,rm32 [vrm:      evex.nf1.nd1.l0.m4.o32 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 7c 1c) 300 321 001(44) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS32,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+17003, 357, 2321},
    /* 2464 : CFCMOVA reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17123, 354, 2321},
    /* 2465 : CFCMOVC reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17143, 354, 2321},
    /* 2466 : CFCMOVG reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17173, 354, 2321},
    /* 2467 : CFCMOVL reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17193, 354, 2321},
    /* 2468 : CFCMOVNA reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17153, 354, 2321},
    /* 2469 : CFCMOVNC reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17133, 354, 2321},
    /* 2470 : CFCMOVNG reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17203, 354, 2321},
    /* 2471 : CFCMOVNL reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17183, 354, 2321},
    /* 2472 : CFCMOVNO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17223, 354, 2321},
    /* 2473 : CFCMOVNS reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17243, 354, 2321},
    /* 2474 : CFCMOVNZ reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17213, 354, 2321},
    /* 2475 : CFCMOVO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17253, 354, 2321},
    /* 2476 : CFCMOVPE reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17263, 354, 2321},
    /* 2477 : CFCMOVPO reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17233, 354, 2321},
    /* 2478 : CFCMOVS reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17273, 354, 2321},
    /* 2479 : CFCMOVZ reg32,rm32 [rm:      evex.nf1.nd0.l0.m4.o32 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 7c 0c) 300 321 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17163, 354, 2321},
    /* 2480 : CFCMOVA reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 47 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(47) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17443, 357, 2322},
    /* 2481 : CFCMOVC reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 42 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(42) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17463, 357, 2322},
    /* 2482 : CFCMOVG reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4f /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4f) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17493, 357, 2322},
    /* 2483 : CFCMOVL reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4c /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4c) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17513, 357, 2322},
    /* 2484 : CFCMOVNA reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 46 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(46) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17473, 357, 2322},
    /* 2485 : CFCMOVNC reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 43 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(43) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17453, 357, 2322},
    /* 2486 : CFCMOVNG reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4e /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4e) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17523, 357, 2322},
    /* 2487 : CFCMOVNL reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4d /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4d) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17503, 357, 2322},
    /* 2488 : CFCMOVNO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 41 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(41) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17543, 357, 2322},
    /* 2489 : CFCMOVNS reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 49 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(49) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17563, 357, 2322},
    /* 2490 : CFCMOVNZ reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 45 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(45) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17533, 357, 2322},
    /* 2491 : CFCMOVO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 40 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(40) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17573, 357, 2322},
    /* 2492 : CFCMOVPE reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4a /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4a) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17583, 357, 2322},
    /* 2493 : CFCMOVPO reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 4b /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(4b) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17553, 357, 2322},
    /* 2494 : CFCMOVS reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 48 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(48) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17593, 357, 2322},
    /* 2495 : CFCMOVZ reg64?,reg64,rm64 [vrm:      evex.nf1.nd1.l0.m4.o64 44 /r    ] SM0,SM1,SM2,ZU,LONG,PROT,APX,X86_64 */
        /* 240(f4 fc 1c) 300 324 001(44) 121 : SM0,SM1,SM2,ZU,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 3, {REG_GPR|BITS64,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+17483, 357, 2322},
    /* 2496 : CFCMOVA reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 47 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(47) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17603, 354, 2322},
    /* 2497 : CFCMOVC reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 42 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(42) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17623, 354, 2322},
    /* 2498 : CFCMOVG reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4f /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4f) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17653, 354, 2322},
    /* 2499 : CFCMOVL reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4c /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4c) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17673, 354, 2322},
    /* 2500 : CFCMOVNA reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 46 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(46) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNA, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17633, 354, 2322},
    /* 2501 : CFCMOVNC reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 43 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(43) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNC, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17613, 354, 2322},
    /* 2502 : CFCMOVNG reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4e /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4e) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNG, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17683, 354, 2322},
    /* 2503 : CFCMOVNL reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4d /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4d) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNL, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17663, 354, 2322},
    /* 2504 : CFCMOVNO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 41 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(41) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17703, 354, 2322},
    /* 2505 : CFCMOVNS reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 49 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(49) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17723, 354, 2322},
    /* 2506 : CFCMOVNZ reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 45 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(45) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVNZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17693, 354, 2322},
    /* 2507 : CFCMOVO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 40 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(40) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17733, 354, 2322},
    /* 2508 : CFCMOVPE reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4a /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4a) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17743, 354, 2322},
    /* 2509 : CFCMOVPO reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 4b /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(4b) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVPO, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17713, 354, 2322},
    /* 2510 : CFCMOVS reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 48 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(48) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVS, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17753, 354, 2322},
    /* 2511 : CFCMOVZ reg64,rm64 [rm:      evex.nf1.nd0.l0.m4.o64 44 /r    ] SM0,SM1,LONG,PROT,APX,X86_64 */
        /* 250(f4 fc 0c) 300 324 001(44) 110 : SM0,SM1,WW,LONG,EVEX,PROT,APX,X86_64 */
        {I_CFCMOVZ, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17643, 354, 2322},
    /* 2512 : SETA rm8 [m:        0f 97 /0                        ] AR0,SB,386 */
        /* 355 001(97) 200 : AR0,SB,386 */
        {I_SETA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72374, 359, 2324},
    /* 2513 : SETC rm8 [m:        0f 92 /0                        ] AR0,SB,386 */
        /* 355 001(92) 200 : AR0,SB,386 */
        {I_SETC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72384, 359, 2324},
    /* 2514 : SETG rm8 [m:        0f 9f /0                        ] AR0,SB,386 */
        /* 355 001(9f) 200 : AR0,SB,386 */
        {I_SETG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72399, 359, 2324},
    /* 2515 : SETL rm8 [m:        0f 9c /0                        ] AR0,SB,386 */
        /* 355 001(9c) 200 : AR0,SB,386 */
        {I_SETL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72409, 359, 2324},
    /* 2516 : SETNA rm8 [m:        0f 96 /0                        ] AR0,SB,386 */
        /* 355 001(96) 200 : AR0,SB,386 */
        {I_SETNA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72389, 359, 2324},
    /* 2517 : SETNC rm8 [m:        0f 93 /0                        ] AR0,SB,386 */
        /* 355 001(93) 200 : AR0,SB,386 */
        {I_SETNC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72379, 359, 2324},
    /* 2518 : SETNG rm8 [m:        0f 9e /0                        ] AR0,SB,386 */
        /* 355 001(9e) 200 : AR0,SB,386 */
        {I_SETNG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72414, 359, 2324},
    /* 2519 : SETNL rm8 [m:        0f 9d /0                        ] AR0,SB,386 */
        /* 355 001(9d) 200 : AR0,SB,386 */
        {I_SETNL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72404, 359, 2324},
    /* 2520 : SETNO rm8 [m:        0f 91 /0                        ] AR0,SB,386 */
        /* 355 001(91) 200 : AR0,SB,386 */
        {I_SETNO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72424, 359, 2324},
    /* 2521 : SETNS rm8 [m:        0f 99 /0                        ] AR0,SB,386 */
        /* 355 001(99) 200 : AR0,SB,386 */
        {I_SETNS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72434, 359, 2324},
    /* 2522 : SETNZ rm8 [m:        0f 95 /0                        ] AR0,SB,386 */
        /* 355 001(95) 200 : AR0,SB,386 */
        {I_SETNZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72419, 359, 2324},
    /* 2523 : SETO rm8 [m:        0f 90 /0                        ] AR0,SB,386 */
        /* 355 001(90) 200 : AR0,SB,386 */
        {I_SETO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72439, 359, 2324},
    /* 2524 : SETPE rm8 [m:        0f 9a /0                        ] AR0,SB,386 */
        /* 355 001(9a) 200 : AR0,SB,386 */
        {I_SETPE, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72444, 359, 2324},
    /* 2525 : SETPO rm8 [m:        0f 9b /0                        ] AR0,SB,386 */
        /* 355 001(9b) 200 : AR0,SB,386 */
        {I_SETPO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72429, 359, 2324},
    /* 2526 : SETS rm8 [m:        0f 98 /0                        ] AR0,SB,386 */
        /* 355 001(98) 200 : AR0,SB,386 */
        {I_SETS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72449, 359, 2324},
    /* 2527 : SETZ rm8 [m:        0f 94 /0                        ] AR0,SB,386 */
        /* 355 001(94) 200 : AR0,SB,386 */
        {I_SETZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72394, 359, 2324},
    /* 2528 : SETA reg64 [m:        evex.nd1.l0.f2.m4.wig 47 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(47) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETA, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25141, 360, 2325},
    /* 2529 : SETC reg64 [m:        evex.nd1.l0.f2.m4.wig 42 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(42) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETC, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25159, 360, 2325},
    /* 2530 : SETG reg64 [m:        evex.nd1.l0.f2.m4.wig 4f /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4f) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETG, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25186, 360, 2325},
    /* 2531 : SETL reg64 [m:        evex.nd1.l0.f2.m4.wig 4c /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4c) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETL, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25204, 360, 2325},
    /* 2532 : SETNA reg64 [m:        evex.nd1.l0.f2.m4.wig 46 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(46) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNA, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25168, 360, 2325},
    /* 2533 : SETNC reg64 [m:        evex.nd1.l0.f2.m4.wig 43 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(43) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNC, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25150, 360, 2325},
    /* 2534 : SETNG reg64 [m:        evex.nd1.l0.f2.m4.wig 4e /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4e) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNG, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25213, 360, 2325},
    /* 2535 : SETNL reg64 [m:        evex.nd1.l0.f2.m4.wig 4d /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4d) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNL, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25195, 360, 2325},
    /* 2536 : SETNO reg64 [m:        evex.nd1.l0.f2.m4.wig 41 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(41) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25231, 360, 2325},
    /* 2537 : SETNS reg64 [m:        evex.nd1.l0.f2.m4.wig 49 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(49) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNS, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25249, 360, 2325},
    /* 2538 : SETNZ reg64 [m:        evex.nd1.l0.f2.m4.wig 45 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(45) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNZ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25222, 360, 2325},
    /* 2539 : SETO reg64 [m:        evex.nd1.l0.f2.m4.wig 40 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(40) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25258, 360, 2325},
    /* 2540 : SETPE reg64 [m:        evex.nd1.l0.f2.m4.wig 4a /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4a) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPE, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25267, 360, 2325},
    /* 2541 : SETPO reg64 [m:        evex.nd1.l0.f2.m4.wig 4b /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(4b) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPO, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25240, 360, 2325},
    /* 2542 : SETS reg64 [m:        evex.nd1.l0.f2.m4.wig 48 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(48) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETS, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25276, 360, 2325},
    /* 2543 : SETZ reg64 [m:        evex.nd1.l0.f2.m4.wig 44 /0     ] ZU,LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 18) 300 001(44) 200 : ZU,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETZ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25177, 360, 2325},
    /* 2544 : SETA rm8 [m:        evex.zu.l0.f2.m4.wig 47 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(47) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25285, 361, 2329},
    /* 2545 : SETC rm8 [m:        evex.zu.l0.f2.m4.wig 42 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(42) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25303, 361, 2329},
    /* 2546 : SETG rm8 [m:        evex.zu.l0.f2.m4.wig 4f /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4f) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25330, 361, 2329},
    /* 2547 : SETL rm8 [m:        evex.zu.l0.f2.m4.wig 4c /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4c) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25348, 361, 2329},
    /* 2548 : SETNA rm8 [m:        evex.zu.l0.f2.m4.wig 46 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(46) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNA, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25312, 361, 2329},
    /* 2549 : SETNC rm8 [m:        evex.zu.l0.f2.m4.wig 43 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(43) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNC, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25294, 361, 2329},
    /* 2550 : SETNG rm8 [m:        evex.zu.l0.f2.m4.wig 4e /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4e) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNG, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25357, 361, 2329},
    /* 2551 : SETNL rm8 [m:        evex.zu.l0.f2.m4.wig 4d /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4d) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNL, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25339, 361, 2329},
    /* 2552 : SETNO rm8 [m:        evex.zu.l0.f2.m4.wig 41 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(41) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25375, 361, 2329},
    /* 2553 : SETNS rm8 [m:        evex.zu.l0.f2.m4.wig 49 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(49) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25393, 361, 2329},
    /* 2554 : SETNZ rm8 [m:        evex.zu.l0.f2.m4.wig 45 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(45) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETNZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25366, 361, 2329},
    /* 2555 : SETO rm8 [m:        evex.zu.l0.f2.m4.wig 40 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(40) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25402, 361, 2329},
    /* 2556 : SETPE rm8 [m:        evex.zu.l0.f2.m4.wig 4a /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4a) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPE, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25411, 361, 2329},
    /* 2557 : SETPO rm8 [m:        evex.zu.l0.f2.m4.wig 4b /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(4b) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETPO, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25384, 361, 2329},
    /* 2558 : SETS rm8 [m:        evex.zu.l0.f2.m4.wig 48 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(48) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETS, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25420, 361, 2329},
    /* 2559 : SETZ rm8 [m:        evex.zu.l0.f2.m4.wig 44 /0      ] LONG,PROT,APX,X86_64 */
        /* 250(f4 7f 08) 300 001(44) 200 : ZU,ZU_E,WIG,LONG,EVEX,PROT,APX,X86_64 */
        {I_SETZ, 1, {RM_GPR|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+25321, 361, 2329},
    /* 2560 : CMPAXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e7 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e7) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57581, 362, 2332},
    /* 2561 : CMPCXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e2 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e2) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57595, 362, 2332},
    /* 2562 : CMPGXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ef /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ef) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57616, 362, 2332},
    /* 2563 : CMPLXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ec /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ec) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57630, 362, 2332},
    /* 2564 : CMPNAXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e6 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e6) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57602, 362, 2332},
    /* 2565 : CMPNCXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e3 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e3) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57588, 362, 2332},
    /* 2566 : CMPNGXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ee /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ee) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57637, 362, 2332},
    /* 2567 : CMPNLXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ed /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ed) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57623, 362, 2332},
    /* 2568 : CMPNOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e1 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e1) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57651, 362, 2332},
    /* 2569 : CMPNSXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e9 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e9) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57665, 362, 2332},
    /* 2570 : CMPNZXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e5 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e5) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57644, 362, 2332},
    /* 2571 : CMPOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e0 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e0) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57672, 362, 2332},
    /* 2572 : CMPPEXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 ea /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(ea) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57679, 362, 2332},
    /* 2573 : CMPPOXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 eb /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(eb) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57658, 362, 2332},
    /* 2574 : CMPSXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e8 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e8) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57686, 362, 2332},
    /* 2575 : CMPZXADD mem32,reg32,reg32 [mrv:      vex.128.66.0f38.w0 e4 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 001) 001(e4) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57609, 362, 2332},
    /* 2576 : CMPAXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e7 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e7) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57693, 362, 2333},
    /* 2577 : CMPCXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e2 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e2) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57707, 362, 2333},
    /* 2578 : CMPGXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ef /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ef) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57728, 362, 2333},
    /* 2579 : CMPLXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ec /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ec) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57742, 362, 2333},
    /* 2580 : CMPNAXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e6 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e6) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57714, 362, 2333},
    /* 2581 : CMPNCXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e3 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e3) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57700, 362, 2333},
    /* 2582 : CMPNGXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ee /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ee) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57749, 362, 2333},
    /* 2583 : CMPNLXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ed /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ed) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57735, 362, 2333},
    /* 2584 : CMPNOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e1 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e1) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57763, 362, 2333},
    /* 2585 : CMPNSXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e9 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e9) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57777, 362, 2333},
    /* 2586 : CMPNZXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e5 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e5) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57756, 362, 2333},
    /* 2587 : CMPOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e0 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e0) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57784, 362, 2333},
    /* 2588 : CMPPEXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 ea /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(ea) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57791, 362, 2333},
    /* 2589 : CMPPOXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 eb /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(eb) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57770, 362, 2333},
    /* 2590 : CMPSXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e8 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e8) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57798, 362, 2333},
    /* 2591 : CMPZXADD mem64,reg64,reg64 [mrv:      vex.128.66.0f38.w1 e4 /r        ] SM0-2,LONG,PROT,CMPCCXADD,X86_64 */
        /* 262(002 201) 001(e4) 101 : SM0,SM1,SM2,NOAPX,LONG,VEX,PROT,CMPCCXADD,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+57721, 362, 2333},
    /* 2592 : CMPAXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e7 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e7) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25429, 363, 2334},
    /* 2593 : CMPCXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e2 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e2) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25447, 363, 2334},
    /* 2594 : CMPGXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ef /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ef) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25474, 363, 2334},
    /* 2595 : CMPLXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ec /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ec) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25492, 363, 2334},
    /* 2596 : CMPNAXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e6 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e6) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25456, 363, 2334},
    /* 2597 : CMPNCXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e3 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e3) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25438, 363, 2334},
    /* 2598 : CMPNGXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ee /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ee) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25501, 363, 2334},
    /* 2599 : CMPNLXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ed /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ed) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25483, 363, 2334},
    /* 2600 : CMPNOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e1 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e1) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25519, 363, 2334},
    /* 2601 : CMPNSXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e9 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e9) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25537, 363, 2334},
    /* 2602 : CMPNZXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e5 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e5) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25510, 363, 2334},
    /* 2603 : CMPOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e0 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e0) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25546, 363, 2334},
    /* 2604 : CMPPEXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 ea /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(ea) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25555, 363, 2334},
    /* 2605 : CMPPOXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 eb /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(eb) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25528, 363, 2334},
    /* 2606 : CMPSXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e8 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e8) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25564, 363, 2334},
    /* 2607 : CMPZXADD mem32,reg32,reg32 [mrv:      evex.nf0.128.66.0f38.w0 e4 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 7d 08) 300 001(e4) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS32,REG_GPR|BITS32,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+25465, 363, 2334},
    /* 2608 : CMPAXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e7 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e7) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPAXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25573, 363, 2335},
    /* 2609 : CMPCXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e2 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e2) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPCXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25591, 363, 2335},
    /* 2610 : CMPGXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ef /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ef) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPGXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25618, 363, 2335},
    /* 2611 : CMPLXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ec /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ec) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPLXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25636, 363, 2335},
    /* 2612 : CMPNAXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e6 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e6) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNAXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25600, 363, 2335},
    /* 2613 : CMPNCXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e3 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e3) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNCXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25582, 363, 2335},
    /* 2614 : CMPNGXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ee /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ee) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNGXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25645, 363, 2335},
    /* 2615 : CMPNLXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ed /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ed) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNLXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25627, 363, 2335},
    /* 2616 : CMPNOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e1 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e1) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25663, 363, 2335},
    /* 2617 : CMPNSXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e9 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e9) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNSXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25681, 363, 2335},
    /* 2618 : CMPNZXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e5 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e5) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPNZXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25654, 363, 2335},
    /* 2619 : CMPOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e0 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e0) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25690, 363, 2335},
    /* 2620 : CMPPEXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 ea /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(ea) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPEXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25699, 363, 2335},
    /* 2621 : CMPPOXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 eb /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(eb) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPPOXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25672, 363, 2335},
    /* 2622 : CMPSXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e8 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e8) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPSXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25708, 363, 2335},
    /* 2623 : CMPZXADD mem64,reg64,reg64 [mrv:      evex.nf0.128.66.0f38.w1 e4 /r   ] SM0-2,LONG,PROT,CMPCCXADD,APX,X86_64 */
        /* 242(f2 fd 08) 300 001(e4) 101 : SM0,SM1,SM2,LONG,EVEX,PROT,CMPCCXADD,APX,X86_64 */
        {I_CMPZXADD, 3, {MEMORY|BITS64,REG_GPR|BITS64,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+25609, 363, 2335},
    /* 2624 : ADDPS xmmreg,xmmrm128 [rm:       np 0f 58 /r                       ] SSE,KATMAI */
        /* 360 355 001(58) 110 : SSE,KATMAI */
        {I_ADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68355, 364, 2338},
    /* 2625 : ADDSS xmmreg,xmmrm32 [rm:       f3 0f 58 /r                       ] SSE,KATMAI */
        /* 333 355 001(58) 110 : SSE,KATMAI */
        {I_ADDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68361, 364, 2339},
    /* 2626 : ANDNPS xmmreg,xmmrm128 [rm:       np 0f 55 /r                       ] SSE,KATMAI */
        /* 360 355 001(55) 110 : SSE,KATMAI */
        {I_ANDNPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68367, 364, 2340},
    /* 2627 : ANDPS xmmreg,xmmrm128 [rm:       np 0f 54 /r                       ] SSE,KATMAI */
        /* 360 355 001(54) 110 : SSE,KATMAI */
        {I_ANDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68373, 364, 2341},
    /* 2628 : CMPEQPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 00                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(00) : SSE,KATMAI */
        {I_CMPEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51403, 364, 2342},
    /* 2629 : CMPEQSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 00                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(00) : SSE,KATMAI */
        {I_CMPEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51411, 364, 2343},
    /* 2630 : CMPLEPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 02                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(02) : SSE,KATMAI */
        {I_CMPLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51419, 364, 2344},
    /* 2631 : CMPLESS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 02                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(02) : SSE,KATMAI */
        {I_CMPLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51427, 364, 2345},
    /* 2632 : CMPLTPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 01                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(01) : SSE,KATMAI */
        {I_CMPLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51435, 364, 2346},
    /* 2633 : CMPLTSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 01                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(01) : SSE,KATMAI */
        {I_CMPLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51443, 364, 2347},
    /* 2634 : CMPNEQPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 04                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(04) : SSE,KATMAI */
        {I_CMPNEQPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51451, 364, 2348},
    /* 2635 : CMPNEQSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 04                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(04) : SSE,KATMAI */
        {I_CMPNEQSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51459, 364, 2349},
    /* 2636 : CMPNLEPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 06                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(06) : SSE,KATMAI */
        {I_CMPNLEPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51467, 364, 2350},
    /* 2637 : CMPNLESS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 06                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(06) : SSE,KATMAI */
        {I_CMPNLESS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51475, 364, 2351},
    /* 2638 : CMPNLTPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 05                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(05) : SSE,KATMAI */
        {I_CMPNLTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51483, 364, 2352},
    /* 2639 : CMPNLTSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 05                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(05) : SSE,KATMAI */
        {I_CMPNLTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51491, 364, 2353},
    /* 2640 : CMPORDPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 07                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(07) : SSE,KATMAI */
        {I_CMPORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51499, 364, 2354},
    /* 2641 : CMPORDSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 07                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(07) : SSE,KATMAI */
        {I_CMPORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51507, 364, 2355},
    /* 2642 : CMPUNORDPS xmmreg,xmmrm128 [rm:       np 0f c2 /r 03                    ] SSE,KATMAI */
        /* 360 355 001(c2) 110 001(03) : SSE,KATMAI */
        {I_CMPUNORDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+51515, 364, 2356},
    /* 2643 : CMPUNORDSS xmmreg,xmmrm32 [rm:       f3 0f c2 /r 03                    ] SSE,KATMAI */
        /* 333 355 001(c2) 110 001(03) : SSE,KATMAI */
        {I_CMPUNORDSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51523, 364, 2357},
    /* 2644 : CMPPS xmmreg,xmmrm128,imm8 [rmi:      np 0f c2 /r ib,u                  ] SSE,KATMAI */
        /* 360 355 001(c2) 110 026 : SSE,KATMAI */
        {I_CMPPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57805, 364, 2360},
    /* 2645 : CMPSS xmmreg,xmmrm32,imm8 [rmi:      f3 0f c2 /r ib,u                  ] SSE,KATMAI */
        /* 333 355 001(c2) 110 026 : SSE,KATMAI */
        {I_CMPSS, 3, {XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57812, 364, 2361},
    /* 2646 : COMISS xmmreg,xmmrm32 [rm:       np 0f 2f /r                       ] FL,SSE,KATMAI */
        /* 360 355 001(2f) 110 : FL,SSE,KATMAI */
        {I_COMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68379, 365, 2362},
    /* 2647 : CVTPI2PS xmmreg,mmxrm64 [rm:       np 0f 2a /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2a) 110 : MMX,SSE,KATMAI */
        {I_CVTPI2PS, 2, {XMMREG,RM_MMX|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68385, 366, 2363},
    /* 2648 : CVTPS2PI mmxreg,xmmrm64 [rm:       np 0f 2d /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2d) 110 : MMX,SSE,KATMAI */
        {I_CVTPS2PI, 2, {MMXREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68391, 366, 2364},
    /* 2649 : CVTSI2SS xmmreg,rm32 [rm:       f3 0f 2a /r                       ] SSE,KATMAI */
        /* 333 355 001(2a) 110 : SSE,KATMAI */
        {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57820, 364, 2365},
    /* 2650 : CVTSI2SS xmmreg,rm64 [rm:       o64 f3 0f 2a /r                   ] AR1,SX,LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2a) 110 : AR1,SX,LONG,PROT,SSE,X86_64 */
        {I_CVTSI2SS, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57819, 367, 2366},
    /* 2651 : CVTSS2SI reg32,xmmrm32 [rm:       f3 0f 2d /r                       ] SSE,KATMAI */
        /* 333 355 001(2d) 110 : SSE,KATMAI */
        {I_CVTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57827, 364, 2367},
    /* 2652 : CVTSS2SI reg64,xmmrm32 [rm:       o64 f3 0f 2d /r                   ] LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2d) 110 : LONG,PROT,SSE,X86_64 */
        {I_CVTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57826, 368, 2368},
    /* 2653 : CVTTPS2PI mmxreg,xmmrm64 [rm:       np 0f 2c /r                       ] MMX,SSE,KATMAI */
        /* 360 355 001(2c) 110 : MMX,SSE,KATMAI */
        {I_CVTTPS2PI, 2, {MMXREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68397, 366, 2369},
    /* 2654 : CVTTSS2SI reg32,xmmrm32 [rm:       f3 0f 2c /r                       ] SSE,KATMAI */
        /* 333 355 001(2c) 110 : SSE,KATMAI */
        {I_CVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57834, 364, 2370},
    /* 2655 : CVTTSS2SI reg64,xmmrm32 [rm:       o64 f3 0f 2c /r                   ] LONG,PROT,SSE,X86_64 */
        /* 324 333 355 001(2c) 110 : LONG,PROT,SSE,X86_64 */
        {I_CVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57833, 368, 2371},
    /* 2656 : DIVPS xmmreg,xmmrm128 [rm:       np 0f 5e /r                       ] SSE,KATMAI */
        /* 360 355 001(5e) 110 : SSE,KATMAI */
        {I_DIVPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68403, 364, 2372},
    /* 2657 : DIVSS xmmreg,xmmrm32 [rm:       f3 0f 5e /r                       ] SSE,KATMAI */
        /* 333 355 001(5e) 110 : SSE,KATMAI */
        {I_DIVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68409, 364, 2373},
    /* 2658 : LDMXCSR mem32 [m:        np 0f ae /2                       ] SSE,KATMAI */
        /* 360 355 001(ae) 202 : SSE,KATMAI */
        {I_LDMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+68415, 364, 2374},
    /* 2659 : MAXPS xmmreg,xmmrm128 [rm:       np 0f 5f /r                       ] SSE,KATMAI */
        /* 360 355 001(5f) 110 : SSE,KATMAI */
        {I_MAXPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68421, 364, 2375},
    /* 2660 : MAXSS xmmreg,xmmrm32 [rm:       f3 0f 5f /r                       ] SSE,KATMAI */
        /* 333 355 001(5f) 110 : SSE,KATMAI */
        {I_MAXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68427, 364, 2376},
    /* 2661 : MINPS xmmreg,xmmrm128 [rm:       np 0f 5d /r                       ] SSE,KATMAI */
        /* 360 355 001(5d) 110 : SSE,KATMAI */
        {I_MINPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68433, 364, 2377},
    /* 2662 : MINSS xmmreg,xmmrm32 [rm:       f3 0f 5d /r                       ] SSE,KATMAI */
        /* 333 355 001(5d) 110 : SSE,KATMAI */
        {I_MINSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68439, 364, 2378},
    /* 2663 : MOVAPS xmmreg,xmmrm128 [rm:       np 0f 28 /r                       ] SSE,KATMAI */
        /* 360 355 001(28) 110 : SSE,KATMAI */
        {I_MOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68445, 364, 2379},
    /* 2664 : MOVAPS xmmrm128,xmmreg [mr:       np 0f 29 /r                       ] SSE,KATMAI */
        /* 360 355 001(29) 101 : SSE,KATMAI */
        {I_MOVAPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68451, 364, 2380},
    /* 2665 : MOVHPS xmmreg,mem64 [rm:       np 0f 16 /r                       ] SSE,KATMAI */
        /* 360 355 001(16) 110 : SSE,KATMAI */
        {I_MOVHPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+68457, 364, 2381},
    /* 2666 : MOVHPS mem64,xmmreg [mr:       np 0f 17 /r                       ] SSE,KATMAI */
        /* 360 355 001(17) 101 : SSE,KATMAI */
        {I_MOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68463, 364, 2382},
    /* 2667 : MOVLHPS xmmreg,xmmreg [rm:       np 0f 16 /r                       ] SSE,KATMAI */
        /* 360 355 001(16) 110 : SSE,KATMAI */
        {I_MOVLHPS, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68457, 364, 2383},
    /* 2668 : MOVLPS xmmreg,mem64 [rm:       np 0f 12 /r                       ] SSE,KATMAI */
        /* 360 355 001(12) 110 : SSE,KATMAI */
        {I_MOVLPS, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+67437, 364, 2384},
    /* 2669 : MOVLPS mem64,xmmreg [mr:       np 0f 13 /r                       ] SSE,KATMAI */
        /* 360 355 001(13) 101 : SSE,KATMAI */
        {I_MOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68469, 364, 2385},
    /* 2670 : MOVHLPS xmmreg,xmmreg [rm:       np 0f 12 /r                       ] SSE,KATMAI */
        /* 360 355 001(12) 110 : SSE,KATMAI */
        {I_MOVHLPS, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+67437, 364, 2386},
    /* 2671 : MOVMSKPS reg32,xmmreg [rm:       np 0f 50 /r                       ] SSE,KATMAI */
        /* 360 355 001(50) 110 : SSE,KATMAI */
        {I_MOVMSKPS, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68475, 364, 2387},
    /* 2672 : MOVMSKPS reg64,xmmreg [rm:       np o64 0f 50 /r                   ] LONG,PROT,SSE,X86_64 */
        /* 360 324 355 001(50) 110 : LONG,PROT,SSE,X86_64 */
        {I_MOVMSKPS, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57840, 368, 2388},
    /* 2673 : MOVNTPS mem128,xmmreg [mr:       np 0f 2b /r                       ] SSE,KATMAI */
        /* 360 355 001(2b) 101 : SSE,KATMAI */
        {I_MOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68481, 364, 2389},
    /* 2674 : MOVSS xmmreg,xmmrm32 [rm:       f3 0f 10 /r                       ] SSE,KATMAI */
        /* 333 355 001(10) 110 : SSE,KATMAI */
        {I_MOVSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68487, 364, 2390},
    /* 2675 : MOVSS xmmrm32,xmmreg [mr:       f3 0f 11 /r                       ] SSE,KATMAI */
        /* 333 355 001(11) 101 : SSE,KATMAI */
        {I_MOVSS, 2, {RM_XMM|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68493, 364, 2391},
    /* 2676 : MOVUPS xmmreg,xmmrm128 [rm:       np 0f 10 /r                       ] SSE,KATMAI */
        /* 360 355 001(10) 110 : SSE,KATMAI */
        {I_MOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68499, 364, 2392},
    /* 2677 : MOVUPS xmmrm128,xmmreg [mr:       np 0f 11 /r                       ] SSE,KATMAI */
        /* 360 355 001(11) 101 : SSE,KATMAI */
        {I_MOVUPS, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68505, 364, 2393},
    /* 2678 : MULPS xmmreg,xmmrm128 [rm:       np 0f 59 /r                       ] SSE,KATMAI */
        /* 360 355 001(59) 110 : SSE,KATMAI */
        {I_MULPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68511, 364, 2394},
    /* 2679 : MULSS xmmreg,xmmrm32 [rm:       f3 0f 59 /r                       ] SSE,KATMAI */
        /* 333 355 001(59) 110 : SSE,KATMAI */
        {I_MULSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68517, 364, 2395},
    /* 2680 : ORPS xmmreg,xmmrm128 [rm:       np 0f 56 /r                       ] SSE,KATMAI */
        /* 360 355 001(56) 110 : SSE,KATMAI */
        {I_ORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68523, 364, 2396},
    /* 2681 : RCPPS xmmreg,xmmrm128 [rm:       np 0f 53 /r                       ] SSE,KATMAI */
        /* 360 355 001(53) 110 : SSE,KATMAI */
        {I_RCPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68529, 364, 2397},
    /* 2682 : RCPSS xmmreg,xmmrm32 [rm:       f3 0f 53 /r                       ] SSE,KATMAI */
        /* 333 355 001(53) 110 : SSE,KATMAI */
        {I_RCPSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68535, 364, 2398},
    /* 2683 : RSQRTPS xmmreg,xmmrm128 [rm:       np 0f 52 /r                       ] SSE,KATMAI */
        /* 360 355 001(52) 110 : SSE,KATMAI */
        {I_RSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68541, 364, 2399},
    /* 2684 : RSQRTSS xmmreg,xmmrm32 [rm:       f3 0f 52 /r                       ] SSE,KATMAI */
        /* 333 355 001(52) 110 : SSE,KATMAI */
        {I_RSQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68547, 364, 2400},
    /* 2685 : SHUFPS xmmreg,xmmrm128,imm8 [rmi:      np 0f c6 /r ib,u                  ] SSE,KATMAI */
        /* 360 355 001(c6) 110 026 : SSE,KATMAI */
        {I_SHUFPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+57847, 364, 2401},
    /* 2686 : SQRTPS xmmreg,xmmrm128 [rm:       np 0f 51 /r                       ] SSE,KATMAI */
        /* 360 355 001(51) 110 : SSE,KATMAI */
        {I_SQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68553, 364, 2402},
    /* 2687 : SQRTSS xmmreg,xmmrm32 [rm:       f3 0f 51 /r                       ] SSE,KATMAI */
        /* 333 355 001(51) 110 : SSE,KATMAI */
        {I_SQRTSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68559, 364, 2403},
    /* 2688 : STMXCSR mem32 [m:        np 0f ae /3                       ] SSE,KATMAI */
        /* 360 355 001(ae) 203 : SSE,KATMAI */
        {I_STMXCSR, 1, {MEMORY|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+68565, 364, 2404},
    /* 2689 : SUBPS xmmreg,xmmrm128 [rm:       np 0f 5c /r                       ] SSE,KATMAI */
        /* 360 355 001(5c) 110 : SSE,KATMAI */
        {I_SUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68571, 364, 2405},
    /* 2690 : SUBSS xmmreg,xmmrm32 [rm:       f3 0f 5c /r                       ] SSE,KATMAI */
        /* 333 355 001(5c) 110 : SSE,KATMAI */
        {I_SUBSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68577, 364, 2406},
    /* 2691 : UCOMISS xmmreg,xmmrm32 [rm:       np 0f 2e /r                       ] FL,SSE,KATMAI */
        /* 360 355 001(2e) 110 : FL,SSE,KATMAI */
        {I_UCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+68583, 365, 2407},
    /* 2692 : UNPCKHPS xmmreg,xmmrm128 [rm:       np 0f 15 /r                       ] SSE,KATMAI */
        /* 360 355 001(15) 110 : SSE,KATMAI */
        {I_UNPCKHPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68589, 364, 2408},
    /* 2693 : UNPCKLPS xmmreg,xmmrm128 [rm:       np 0f 14 /r                       ] SSE,KATMAI */
        /* 360 355 001(14) 110 : SSE,KATMAI */
        {I_UNPCKLPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68595, 364, 2409},
    /* 2694 : XORPS xmmreg,xmmrm128 [rm:       np 0f 57 /r                       ] SSE,KATMAI */
        /* 360 355 001(57) 110 : SSE,KATMAI */
        {I_XORPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68601, 364, 2410},
    /* 2695 : FXRSTOR mem [m:        np 0f ae /1                       ] FPU,SSE,P6 */
        /* 360 355 001(ae) 201 : FPU,SSE,P6 */
        {I_FXRSTOR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57855, 369, 2413},
    /* 2696 : FXRSTOR64 mem [m:        o64 np 0f ae /1                   ] LONG,PROT,FPU,SSE,X86_64 */
        /* 324 360 355 001(ae) 201 : LONG,PROT,FPU,SSE,X86_64 */
        {I_FXRSTOR64, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57854, 370, 2414},
    /* 2697 : FXSAVE mem [m:        np 0f ae /0                       ] FPU,SSE,P6 */
        /* 360 355 001(ae) 200 : FPU,SSE,P6 */
        {I_FXSAVE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57862, 369, 2415},
    /* 2698 : FXSAVE64 mem [m:        o64 np 0f ae /0                   ] LONG,PROT,FPU,SSE,X86_64 */
        /* 324 360 355 001(ae) 200 : LONG,PROT,FPU,SSE,X86_64 */
        {I_FXSAVE64, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57861, 370, 2416},
    /* 2699 : XGETBV void [          0f 01 d0                          ] NEHALEM */
        /* 355 002(01 d0) : NEHALEM */
        {I_XGETBV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72454, 371, 2421},
    /* 2700 : XSETBV void [          0f 01 d1                          ] PRIV,NEHALEM */
        /* 355 002(01 d1) : PRIV,NEHALEM */
        {I_XSETBV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72459, 372, 2422},
    /* 2701 : XSAVE mem [m:        np 0f ae /4                       ] NOAPX,NEHALEM */
        /* 360 355 001(ae) 204 : NOAPX,NEHALEM */
        {I_XSAVE, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57869, 373, 2423},
    /* 2702 : XSAVE64 mem [m:        o64 np 0f ae /4                   ] NOAPX,LONG,PROT,X86_64,NEHALEM */
        /* 324 360 355 001(ae) 204 : NOAPX,LONG,PROT,NEHALEM */
        {I_XSAVE64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57868, 374, 2424},
    /* 2703 : XSAVEC mem [m:        np 0f c7 /4                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 204 : NOAPX,FUTURE */
        {I_XSAVEC, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57876, 282, 2425},
    /* 2704 : XSAVEC64 mem [m:        o64 np 0f c7 /4                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 204 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVEC64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57875, 244, 2426},
    /* 2705 : XSAVEOPT mem [m:        np 0f ae /6                       ] NOAPX,FUTURE */
        /* 360 355 001(ae) 206 : NOAPX,FUTURE */
        {I_XSAVEOPT, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57883, 282, 2427},
    /* 2706 : XSAVEOPT64 mem [m:        o64 np 0f ae /6                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(ae) 206 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVEOPT64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57882, 244, 2428},
    /* 2707 : XSAVES mem [m:        np 0f c7 /5                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 205 : NOAPX,FUTURE */
        {I_XSAVES, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57890, 282, 2429},
    /* 2708 : XSAVES64 mem [m:        o64 np 0f c7 /5                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 205 : NOAPX,LONG,PROT,X86_64 */
        {I_XSAVES64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57889, 244, 2430},
    /* 2709 : XRSTOR mem [m:        np 0f ae /5                       ] NOAPX,NEHALEM */
        /* 360 355 001(ae) 205 : NOAPX,NEHALEM */
        {I_XRSTOR, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57897, 373, 2431},
    /* 2710 : XRSTOR64 mem [m:        o64 np 0f ae /5                   ] NOAPX,LONG,PROT,X86_64,NEHALEM */
        /* 324 360 355 001(ae) 205 : NOAPX,LONG,PROT,NEHALEM */
        {I_XRSTOR64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57896, 374, 2432},
    /* 2711 : XRSTORS mem [m:        np 0f c7 /3                       ] NOAPX,FUTURE */
        /* 360 355 001(c7) 203 : NOAPX,FUTURE */
        {I_XRSTORS, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57904, 282, 2433},
    /* 2712 : XRSTORS64 mem [m:        o64 np 0f c7 /3                   ] NOAPX,LONG,PROT,X86_64 */
        /* 324 360 355 001(c7) 203 : NOAPX,LONG,PROT,X86_64 */
        {I_XRSTORS64, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57903, 244, 2434},
    /* 2713 : PREFETCHNTA mem8 [m:        0f 18 /0                          ] AR0,SB,KATMAI */
        /* 355 001(18) 200 : AR0,SB,KATMAI */
        {I_PREFETCHNTA, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70198, 375, 2439},
    /* 2714 : PREFETCHT0 mem8 [m:        0f 18 /1                          ] AR0,SB,KATMAI */
        /* 355 001(18) 201 : AR0,SB,KATMAI */
        {I_PREFETCHT0, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70216, 375, 2440},
    /* 2715 : PREFETCHT1 mem8 [m:        0f 18 /2                          ] AR0,SB,KATMAI */
        /* 355 001(18) 202 : AR0,SB,KATMAI */
        {I_PREFETCHT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70234, 375, 2441},
    /* 2716 : PREFETCHT2 mem8 [m:        0f 18 /3                          ] AR0,SB,KATMAI */
        /* 355 001(18) 203 : AR0,SB,KATMAI */
        {I_PREFETCHT2, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70252, 375, 2442},
    /* 2717 : PREFETCHIT0 mem8 [m:        0f 18 /7                          ] AR0,SB,PREFETCHI,FUTURE */
        /* 355 001(18) 207 : AR0,SB,PREFETCHI,FUTURE */
        {I_PREFETCHIT0, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70324, 376, 2443},
    /* 2718 : PREFETCHIT1 mem8 [m:        0f 18 /6                          ] AR0,SB,PREFETCHI,FUTURE */
        /* 355 001(18) 206 : AR0,SB,PREFETCHI,FUTURE */
        {I_PREFETCHIT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70306, 376, 2444},
    /* 2719 : SFENCE void [          np 0f ae f8                       ] KATMAI */
        /* 360 355 002(ae f8) : KATMAI */
        {I_SFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67239, 377, 2445},
    /* 2720 : MASKMOVQ mmxreg,mmxreg [rm:       np 0f f7 /r                       ] MMX,KATMAI */
        /* 360 355 001(f7) 110 : MMX,KATMAI */
        {I_MASKMOVQ, 2, {MMXREG,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68607, 378, 2448},
    /* 2721 : MOVNTQ mem,mmxreg [mr:       np 0f e7 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e7) 101 : AR0,AR1,SQ,MMX,KATMAI */
        {I_MOVNTQ, 2, {MEMORY,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68613, 379, 2449},
    /* 2722 : PAVGB mmxreg,mmxrm [rm:       np 0f e0 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e0) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PAVGB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68619, 379, 2450},
    /* 2723 : PAVGW mmxreg,mmxrm [rm:       np 0f e3 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e3) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PAVGW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68625, 379, 2451},
    /* 2724 : PEXTRW reg32,mmxreg,imm [rmi:      np 0f c5 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c5) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PEXTRW, 3, {REG_GPR|BITS32,MMXREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57910, 380, 2452},
    /* 2725 : PINSRW mmxreg,mem,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57917, 380, 2455},
    /* 2726 : PINSRW mmxreg,rm16,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,RM_GPR|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57917, 380, 2456},
    /* 2727 : PINSRW mmxreg,reg32,imm [rmi:      np 0f c4 /r ib,u                  ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(c4) 110 026 : AR2,SB,MMX,KATMAI */
        {I_PINSRW, 3, {MMXREG,REG_GPR|BITS32,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57917, 380, 2457},
    /* 2728 : PMAXSW mmxreg,mmxrm [rm:       np 0f ee /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(ee) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMAXSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68631, 379, 2458},
    /* 2729 : PMAXUB mmxreg,mmxrm [rm:       np 0f de /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(de) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMAXUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68637, 379, 2459},
    /* 2730 : PMINSW mmxreg,mmxrm [rm:       np 0f ea /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(ea) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMINSW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68643, 379, 2460},
    /* 2731 : PMINUB mmxreg,mmxrm [rm:       np 0f da /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(da) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMINUB, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68649, 379, 2461},
    /* 2732 : PMOVMSKB reg32,mmxreg [rm:       np 0f d7 /r                       ] MMX,KATMAI */
        /* 360 355 001(d7) 110 : MMX,KATMAI */
        {I_PMOVMSKB, 2, {REG_GPR|BITS32,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68655, 378, 2462},
    /* 2733 : PMULHUW mmxreg,mmxrm [rm:       np 0f e4 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(e4) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PMULHUW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68661, 379, 2463},
    /* 2734 : PSADBW mmxreg,mmxrm [rm:       np 0f f6 /r                       ] AR0-1,SQ,MMX,KATMAI */
        /* 360 355 001(f6) 110 : AR0,AR1,SQ,MMX,KATMAI */
        {I_PSADBW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68667, 379, 2464},
    /* 2735 : PSHUFW mmxreg,mmxrm,imm [rmi:      np 0f 70 /r ib                    ] AR2,SB,MMX,KATMAI */
        /* 360 355 001(70) 110 022 : AR2,SB,MMX,KATMAI */
        {I_PSHUFW, 3, {MMXREG,RM_MMX,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+57924, 380, 2465},
    /* 2736 : PF2IW mmxreg,mmxrm [rm:       0f 0f /r 1c                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(1c) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PF2IW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57931, 336, 2468},
    /* 2737 : PFNACC mmxreg,mmxrm [rm:       0f 0f /r 8a                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(8a) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFNACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57938, 336, 2469},
    /* 2738 : PFPNACC mmxreg,mmxrm [rm:       0f 0f /r 8e                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(8e) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PFPNACC, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57945, 336, 2470},
    /* 2739 : PI2FW mmxreg,mmxrm [rm:       0f 0f /r 0c                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(0c) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PI2FW, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57952, 336, 2471},
    /* 2740 : PSWAPD mmxreg,mmxrm [rm:       0f 0f /r bb                       ] AR0-1,SQ,3DNOW,PENT */
        /* 355 001(0f) 110 001(bb) : AR0,AR1,SQ,3DNOW,PENT */
        {I_PSWAPD, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+57959, 336, 2472},
    /* 2741 : MASKMOVDQU xmmreg,xmmreg [rm:       66 0f f7 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(f7) 110 : SSE2,WILLAMETTE */
        {I_MASKMOVDQU, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68673, 248, 2475},
    /* 2742 : MOVNTDQ mem,xmmreg [mr:       66 0f e7 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e7) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVNTDQ, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68679, 381, 2476},
    /* 2743 : MOVNTI mem,reg32 [mr:       np 0f c3 /r                       ] AR0-1,SD,WILLAMETTE */
        /* 360 355 001(c3) 101 : AR0,AR1,SD,WILLAMETTE */
        {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57967, 382, 2477},
    /* 2744 : MOVNTI mem,reg64 [mr:       o64 np 0f c3 /r                   ] AR0-1,SQ,LONG,PROT,X86_64 */
        /* 324 360 355 001(c3) 101 : AR0,AR1,SQ,LONG,PROT,X86_64 */
        {I_MOVNTI, 2, {MEMORY,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57966, 383, 2478},
    /* 2745 : MOVNTPD mem,xmmreg [mr:       66 0f 2b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2b) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVNTPD, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68685, 381, 2479},
    /* 2746 : LFENCE void [          np 0f ae e8                       ] SSE2,WILLAMETTE */
        /* 360 355 002(ae e8) : SSE2,WILLAMETTE */
        {I_LFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67227, 248, 2480},
    /* 2747 : MFENCE void [          np 0f ae f0                       ] SSE2,WILLAMETTE */
        /* 360 355 002(ae f0) : SSE2,WILLAMETTE */
        {I_MFENCE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+67233, 248, 2481},
    /* 2748 : MOVD mem,xmmreg [mr:       66 norexw 0f 7e /r                ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 361 317 355 001(7e) 101 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_MOVD, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57973, 384, 2484},
    /* 2749 : MOVD xmmreg,mem [rm:       66 norexw 0f 6e /r                ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 361 317 355 001(6e) 110 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_MOVD, 2, {XMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+57980, 384, 2485},
    /* 2750 : MOVD xmmreg,rm32 [rm:       66 norexw 0f 6e /r                ] SSE2,WILLAMETTE */
        /* 361 317 355 001(6e) 110 : SSE2,WILLAMETTE */
        {I_MOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+57980, 248, 2486},
    /* 2751 : MOVD rm32,xmmreg [mr:       66 norexw 0f 7e /r                ] SSE2,WILLAMETTE */
        /* 361 317 355 001(7e) 101 : SSE2,WILLAMETTE */
        {I_MOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57973, 248, 2487},
    /* 2752 : MOVDQA xmmreg,xmmrm128 [rm:       66 0f 6f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQA, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68691, 381, 2488},
    /* 2753 : MOVDQA xmmrm128,xmmreg [mr:       66 0f 7f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(7f) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQA, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68697, 381, 2489},
    /* 2754 : MOVDQU xmmreg,xmmrm128 [rm:       f3 0f 6f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(6f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQU, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+68703, 381, 2490},
    /* 2755 : MOVDQU xmmrm128,xmmreg [mr:       f3 0f 7f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(7f) 101 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MOVDQU, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68709, 381, 2491},
    /* 2756 : MOVDQ2Q mmxreg,xmmreg [rm:       f2 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(d6) 110 : SSE2,WILLAMETTE */
        {I_MOVDQ2Q, 2, {MMXREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68715, 248, 2492},
    /* 2757 : MOVQ xmmreg,xmmreg [rm:       f3 0f 7e /r                       ] SSE2,WILLAMETTE */
        /* 333 355 001(7e) 110 : SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68721, 248, 2493},
    /* 2758 : MOVQ xmmreg,xmmreg [mr:       66 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(d6) 101 : SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68727, 248, 2494},
    /* 2759 : MOVQ mem,xmmreg [mr:       66 0f d6 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 361 355 001(d6) 101 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MOVQ, 2, {MEMORY,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68727, 385, 2495},
    /* 2760 : MOVQ xmmreg,mem [rm:       f3 0f 7e /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 333 355 001(7e) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MOVQ, 2, {XMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+68721, 385, 2496},
    /* 2761 : MOVQ xmmreg,rm64 [rm:       66 o64 0f 6e /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(6e) 110 : LONG,PROT,SSE2,X86_64 */
        {I_MOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+57987, 386, 2497},
    /* 2762 : MOVQ rm64,xmmreg [mr:       66 o64 0f 7e /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(7e) 101 : LONG,PROT,SSE2,X86_64 */
        {I_MOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+57994, 386, 2498},
    /* 2763 : MOVQ2DQ xmmreg,mmxreg [rm:       f3 0f d6 /r                       ] SSE2,WILLAMETTE */
        /* 333 355 001(d6) 110 : SSE2,WILLAMETTE */
        {I_MOVQ2DQ, 2, {XMMREG,MMXREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68733, 248, 2499},
    /* 2764 : PACKSSWB xmmreg,xmmrm [rm:       66 0f 63 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(63) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKSSWB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68739, 381, 2500},
    /* 2765 : PACKSSDW xmmreg,xmmrm [rm:       66 0f 6b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKSSDW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68745, 381, 2501},
    /* 2766 : PACKUSWB xmmreg,xmmrm [rm:       66 0f 67 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(67) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PACKUSWB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68751, 381, 2502},
    /* 2767 : PADDB xmmreg,xmmrm [rm:       66 0f fc /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fc) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68757, 381, 2503},
    /* 2768 : PADDW xmmreg,xmmrm [rm:       66 0f fd /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fd) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68763, 381, 2504},
    /* 2769 : PADDD xmmreg,xmmrm [rm:       66 0f fe /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fe) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68769, 381, 2505},
    /* 2770 : PADDQ mmxreg,mmxrm [rm:       np 0f d4 /r                       ] AR0-1,SQ,MMX,WILLAMETTE */
        /* 360 355 001(d4) 110 : AR0,AR1,SQ,MMX,WILLAMETTE */
        {I_PADDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68775, 387, 2506},
    /* 2771 : PADDQ xmmreg,xmmrm [rm:       66 0f d4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68781, 381, 2507},
    /* 2772 : PADDSB xmmreg,xmmrm [rm:       66 0f ec /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ec) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68787, 381, 2508},
    /* 2773 : PADDSW xmmreg,xmmrm [rm:       66 0f ed /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ed) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68793, 381, 2509},
    /* 2774 : PADDUSB xmmreg,xmmrm [rm:       66 0f dc /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(dc) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDUSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68799, 381, 2510},
    /* 2775 : PADDUSW xmmreg,xmmrm [rm:       66 0f dd /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(dd) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PADDUSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68805, 381, 2511},
    /* 2776 : PAND xmmreg,xmmrm [rm:       66 0f db /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(db) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAND, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68811, 381, 2512},
    /* 2777 : PANDN xmmreg,xmmrm [rm:       66 0f df /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(df) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PANDN, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68817, 381, 2513},
    /* 2778 : PAVGB xmmreg,xmmrm [rm:       66 0f e0 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e0) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAVGB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68823, 381, 2514},
    /* 2779 : PAVGW xmmreg,xmmrm [rm:       66 0f e3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PAVGW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68829, 381, 2515},
    /* 2780 : PCMPEQB xmmreg,xmmrm [rm:       66 0f 74 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(74) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68835, 381, 2516},
    /* 2781 : PCMPEQW xmmreg,xmmrm [rm:       66 0f 75 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(75) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68841, 381, 2517},
    /* 2782 : PCMPEQD xmmreg,xmmrm [rm:       66 0f 76 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(76) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPEQD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68847, 381, 2518},
    /* 2783 : PCMPGTB xmmreg,xmmrm [rm:       66 0f 64 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(64) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68853, 381, 2519},
    /* 2784 : PCMPGTW xmmreg,xmmrm [rm:       66 0f 65 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(65) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68859, 381, 2520},
    /* 2785 : PCMPGTD xmmreg,xmmrm [rm:       66 0f 66 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(66) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PCMPGTD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68865, 381, 2521},
    /* 2786 : PEXTRW reg32,xmmreg,imm [rmi:      66 0f c5 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c5) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58001, 388, 2522},
    /* 2787 : PINSRW xmmreg,reg16,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,REG_GPR|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58008, 388, 2524},
    /* 2788 : PINSRW xmmreg,mem,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58008, 388, 2527},
    /* 2789 : PINSRW xmmreg,mem16,imm [rmi:      66 0f c4 /r ib,u                  ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(c4) 110 026 : AR2,SB,SSE2,WILLAMETTE */
        {I_PINSRW, 3, {XMMREG,MEMORY|BITS16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58008, 388, 2528},
    /* 2790 : PMADDWD xmmreg,xmmrm [rm:       66 0f f5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMADDWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68871, 381, 2529},
    /* 2791 : PMAXSW xmmreg,xmmrm [rm:       66 0f ee /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ee) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMAXSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68877, 381, 2530},
    /* 2792 : PMAXUB xmmreg,xmmrm [rm:       66 0f de /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(de) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMAXUB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68883, 381, 2531},
    /* 2793 : PMINSW xmmreg,xmmrm [rm:       66 0f ea /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ea) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMINSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68889, 381, 2532},
    /* 2794 : PMINUB xmmreg,xmmrm [rm:       66 0f da /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(da) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMINUB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68895, 381, 2533},
    /* 2795 : PMOVMSKB reg32,xmmreg [rm:       66 0f d7 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(d7) 110 : SSE2,WILLAMETTE */
        {I_PMOVMSKB, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+68901, 248, 2534},
    /* 2796 : PMULHUW xmmreg,xmmrm [rm:       66 0f e4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULHUW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68907, 381, 2535},
    /* 2797 : PMULHW xmmreg,xmmrm [rm:       66 0f e5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULHW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68913, 381, 2536},
    /* 2798 : PMULLW xmmreg,xmmrm [rm:       66 0f d5 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d5) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68919, 381, 2537},
    /* 2799 : PMULUDQ mmxreg,mmxrm [rm:       np 0f f4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(f4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULUDQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+68925, 381, 2538},
    /* 2800 : PMULUDQ xmmreg,xmmrm [rm:       66 0f f4 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f4) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PMULUDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68931, 381, 2539},
    /* 2801 : POR xmmreg,xmmrm [rm:       66 0f eb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(eb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_POR, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68937, 381, 2540},
    /* 2802 : PSADBW xmmreg,xmmrm [rm:       66 0f f6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSADBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68943, 381, 2541},
    /* 2803 : PSHUFD xmmreg,xmmreg,imm [rmi:      66 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFD, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58015, 388, 2542},
    /* 2804 : PSHUFD xmmreg,mem,imm [rmi:      66 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 361 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFD, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58015, 388, 2543},
    /* 2805 : PSHUFHW xmmreg,xmmreg,imm [rmi:      f3 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 333 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFHW, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58022, 388, 2544},
    /* 2806 : PSHUFHW xmmreg,mem,imm [rmi:      f3 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 333 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFHW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58022, 388, 2545},
    /* 2807 : PSHUFLW xmmreg,xmmreg,imm [rmi:      f2 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 332 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFLW, 3, {XMMREG,XMMREG,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58029, 388, 2546},
    /* 2808 : PSHUFLW xmmreg,mem,imm [rmi:      f2 0f 70 /r ib                    ] AR2,SB,SSE2,WILLAMETTE */
        /* 332 355 001(70) 110 022 : AR2,SB,SSE2,WILLAMETTE */
        {I_PSHUFLW, 3, {XMMREG,MEMORY,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58029, 388, 2547},
    /* 2809 : PSLLDQ xmmreg,imm [mi:       66 0f 73 /7 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 207 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLDQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58036, 390, 2548},
    /* 2810 : PSLLW xmmreg,xmmrm [rm:       66 0f f1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68949, 381, 2549},
    /* 2811 : PSLLW xmmreg,imm [mi:       66 0f 71 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58043, 390, 2550},
    /* 2812 : PSLLD xmmreg,xmmrm [rm:       66 0f f2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68955, 381, 2551},
    /* 2813 : PSLLD xmmreg,imm [mi:       66 0f 72 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58050, 390, 2552},
    /* 2814 : PSLLQ xmmreg,xmmrm [rm:       66 0f f3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSLLQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68961, 381, 2553},
    /* 2815 : PSLLQ xmmreg,imm [mi:       66 0f 73 /6 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 206 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSLLQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58057, 390, 2554},
    /* 2816 : PSRAW xmmreg,xmmrm [rm:       66 0f e1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRAW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68967, 381, 2555},
    /* 2817 : PSRAW xmmreg,imm [mi:       66 0f 71 /4 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 204 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRAW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58064, 390, 2556},
    /* 2818 : PSRAD xmmreg,xmmrm [rm:       66 0f e2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRAD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68973, 381, 2557},
    /* 2819 : PSRAD xmmreg,imm [mi:       66 0f 72 /4 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 204 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRAD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58071, 390, 2558},
    /* 2820 : PSRLDQ xmmreg,imm [mi:       66 0f 73 /3 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 203 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLDQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58078, 390, 2559},
    /* 2821 : PSRLW xmmreg,xmmrm [rm:       66 0f d1 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d1) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68979, 381, 2560},
    /* 2822 : PSRLW xmmreg,imm [mi:       66 0f 71 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(71) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLW, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58085, 390, 2561},
    /* 2823 : PSRLD xmmreg,xmmrm [rm:       66 0f d2 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d2) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68985, 381, 2562},
    /* 2824 : PSRLD xmmreg,imm [mi:       66 0f 72 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(72) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLD, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58092, 390, 2563},
    /* 2825 : PSRLQ xmmreg,xmmrm [rm:       66 0f d3 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d3) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSRLQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68991, 381, 2564},
    /* 2826 : PSRLQ xmmreg,imm [mi:       66 0f 73 /2 ib,u                  ] AR1,SB,SSE2,WILLAMETTE */
        /* 361 355 001(73) 202 025 : AR1,SB,SSE2,WILLAMETTE */
        {I_PSRLQ, 2, {XMMREG,IMM_NORMAL,0,0,0}, NO_DECORATOR, nasm_bytecodes+58099, 390, 2565},
    /* 2827 : PSUBB xmmreg,xmmrm [rm:       66 0f f8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+68997, 381, 2566},
    /* 2828 : PSUBW xmmreg,xmmrm [rm:       66 0f f9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(f9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69003, 381, 2567},
    /* 2829 : PSUBD xmmreg,xmmrm [rm:       66 0f fa /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fa) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69009, 381, 2568},
    /* 2830 : PSUBQ mmxreg,mmxrm [rm:       np 0f fb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(fb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBQ, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69015, 381, 2569},
    /* 2831 : PSUBQ xmmreg,xmmrm [rm:       66 0f fb /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(fb) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69021, 381, 2570},
    /* 2832 : PSUBSB xmmreg,xmmrm [rm:       66 0f e8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69027, 381, 2571},
    /* 2833 : PSUBSW xmmreg,xmmrm [rm:       66 0f e9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69033, 381, 2572},
    /* 2834 : PSUBUSB xmmreg,xmmrm [rm:       66 0f d8 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d8) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBUSB, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69039, 381, 2573},
    /* 2835 : PSUBUSW xmmreg,xmmrm [rm:       66 0f d9 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(d9) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PSUBUSW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69045, 381, 2574},
    /* 2836 : PUNPCKHBW xmmreg,xmmrm [rm:       66 0f 68 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(68) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69051, 381, 2575},
    /* 2837 : PUNPCKHWD xmmreg,xmmrm [rm:       66 0f 69 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(69) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69057, 381, 2576},
    /* 2838 : PUNPCKHDQ xmmreg,xmmrm [rm:       66 0f 6a /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6a) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69063, 381, 2577},
    /* 2839 : PUNPCKHQDQ xmmreg,xmmrm [rm:       66 0f 6d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKHQDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69069, 381, 2578},
    /* 2840 : PUNPCKLBW xmmreg,xmmrm [rm:       66 0f 60 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(60) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLBW, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69075, 381, 2579},
    /* 2841 : PUNPCKLWD xmmreg,xmmrm [rm:       66 0f 61 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(61) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLWD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69081, 381, 2580},
    /* 2842 : PUNPCKLDQ xmmreg,xmmrm [rm:       66 0f 62 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(62) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69087, 381, 2581},
    /* 2843 : PUNPCKLQDQ xmmreg,xmmrm [rm:       66 0f 6c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(6c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PUNPCKLQDQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69093, 381, 2582},
    /* 2844 : PXOR xmmreg,xmmrm [rm:       66 0f ef /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(ef) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_PXOR, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69099, 381, 2583},
    /* 2845 : ADDPD xmmreg,xmmrm [rm:       66 0f 58 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(58) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ADDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69105, 381, 2586},
    /* 2846 : ADDSD xmmreg,xmmrm [rm:       f2 0f 58 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(58) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_ADDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69111, 385, 2587},
    /* 2847 : ANDNPD xmmreg,xmmrm [rm:       66 0f 55 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(55) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ANDNPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69117, 381, 2588},
    /* 2848 : ANDPD xmmreg,xmmrm [rm:       66 0f 54 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(54) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ANDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69123, 381, 2589},
    /* 2849 : CMPEQPD xmmreg,xmmrm [rm:       66 0f c2 /r 00                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(00) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPEQPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51531, 381, 2590},
    /* 2850 : CMPEQSD xmmreg,xmmrm [rm:       f2 0f c2 /r 00                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(00) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPEQSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51539, 385, 2591},
    /* 2851 : CMPLEPD xmmreg,xmmrm [rm:       66 0f c2 /r 02                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(02) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPLEPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51547, 381, 2592},
    /* 2852 : CMPLESD xmmreg,xmmrm [rm:       f2 0f c2 /r 02                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(02) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPLESD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51555, 385, 2593},
    /* 2853 : CMPLTPD xmmreg,xmmrm [rm:       66 0f c2 /r 01                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(01) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPLTPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51563, 381, 2594},
    /* 2854 : CMPLTSD xmmreg,xmmrm [rm:       f2 0f c2 /r 01                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(01) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPLTSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51571, 385, 2595},
    /* 2855 : CMPNEQPD xmmreg,xmmrm [rm:       66 0f c2 /r 04                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(04) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNEQPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51579, 381, 2596},
    /* 2856 : CMPNEQSD xmmreg,xmmrm [rm:       f2 0f c2 /r 04                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(04) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNEQSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51587, 385, 2597},
    /* 2857 : CMPNLEPD xmmreg,xmmrm [rm:       66 0f c2 /r 06                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(06) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNLEPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51595, 381, 2598},
    /* 2858 : CMPNLESD xmmreg,xmmrm [rm:       f2 0f c2 /r 06                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(06) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNLESD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51603, 385, 2599},
    /* 2859 : CMPNLTPD xmmreg,xmmrm [rm:       66 0f c2 /r 05                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(05) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPNLTPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51611, 381, 2600},
    /* 2860 : CMPNLTSD xmmreg,xmmrm [rm:       f2 0f c2 /r 05                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(05) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPNLTSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51619, 385, 2601},
    /* 2861 : CMPORDPD xmmreg,xmmrm [rm:       66 0f c2 /r 07                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(07) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPORDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51627, 381, 2602},
    /* 2862 : CMPORDSD xmmreg,xmmrm [rm:       f2 0f c2 /r 07                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(07) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPORDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51635, 385, 2603},
    /* 2863 : CMPUNORDPD xmmreg,xmmrm [rm:       66 0f c2 /r 03                    ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 001(03) : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CMPUNORDPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51643, 381, 2604},
    /* 2864 : CMPUNORDSD xmmreg,xmmrm [rm:       f2 0f c2 /r 03                    ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 001(03) : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CMPUNORDSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+51651, 385, 2605},
    /* 2865 : CMPPD xmmreg,xmmrm128,imm8 [rmi:      66 0f c2 /r ib,u                  ] SSE2,WILLAMETTE */
        /* 361 355 001(c2) 110 026 : SSE2,WILLAMETTE */
        {I_CMPPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58106, 248, 2608},
    /* 2866 : CMPSD xmmreg,xmmrm64,imm8 [rmi:      f2 0f c2 /r ib,u                  ] FL,SSE2,WILLAMETTE */
        /* 332 355 001(c2) 110 026 : FL,SSE2,WILLAMETTE */
        {I_CMPSD, 3, {XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58113, 391, 2609},
    /* 2867 : COMISD xmmreg,xmmrm64 [rm:       66 0f 2f /r                       ] FL,SSE2,WILLAMETTE */
        /* 361 355 001(2f) 110 : FL,SSE2,WILLAMETTE */
        {I_COMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69129, 391, 2610},
    /* 2868 : CVTDQ2PD xmmreg,xmmrm [rm:       f3 0f e6 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 333 355 001(e6) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTDQ2PD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69135, 385, 2611},
    /* 2869 : CVTDQ2PS xmmreg,xmmrm [rm:       np 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 360 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTDQ2PS, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69141, 381, 2612},
    /* 2870 : CVTPD2DQ xmmreg,xmmrm [rm:       f2 0f e6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 332 355 001(e6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69147, 381, 2613},
    /* 2871 : CVTPD2PI mmxreg,xmmrm [rm:       66 0f 2d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69153, 381, 2614},
    /* 2872 : CVTPD2PS xmmreg,xmmrm [rm:       66 0f 5a /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5a) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPD2PS, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69159, 381, 2615},
    /* 2873 : CVTPI2PD xmmreg,mmxrm [rm:       66 0f 2a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 361 355 001(2a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTPI2PD, 2, {XMMREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+69165, 385, 2616},
    /* 2874 : CVTPS2DQ xmmreg,xmmrm [rm:       66 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69171, 381, 2617},
    /* 2875 : CVTPS2PD xmmreg,xmmrm [rm:       np 0f 5a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 360 355 001(5a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTPS2PD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69177, 385, 2618},
    /* 2876 : CVTSD2SI reg32,xmmrm64 [rm:       norexw f2 0f 2d /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2d) 110 : SSE2,WILLAMETTE */
        {I_CVTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58120, 248, 2619},
    /* 2877 : CVTSD2SI reg64,xmmrm64 [rm:       o64 f2 0f 2d /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2d) 110 : LONG,PROT,SSE2,X86_64 */
        {I_CVTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58127, 386, 2620},
    /* 2878 : CVTSD2SS xmmreg,xmmrm64 [rm:       f2 0f 5a /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5a) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_CVTSD2SS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69183, 385, 2621},
    /* 2879 : CVTSI2SD xmmreg,rm32 [rm:       norexw f2 0f 2a /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2a) 110 : SSE2,WILLAMETTE */
        {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+58134, 248, 2622},
    /* 2880 : CVTSI2SD xmmreg,rm64 [rm:       o64 f2 0f 2a /r                   ] AR1,SX,LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2a) 110 : AR1,SX,LONG,PROT,SSE2,X86_64 */
        {I_CVTSI2SD, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58141, 392, 2623},
    /* 2881 : CVTSS2SD xmmreg,xmmrm [rm:       f3 0f 5a /r                       ] AR0-1,SD,SSE2,WILLAMETTE */
        /* 333 355 001(5a) 110 : AR0,AR1,SD,SSE2,WILLAMETTE */
        {I_CVTSS2SD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69189, 384, 2624},
    /* 2882 : CVTTPD2PI mmxreg,xmmrm [rm:       66 0f 2c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(2c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPD2PI, 2, {MMXREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69195, 381, 2625},
    /* 2883 : CVTTPD2DQ xmmreg,xmmrm [rm:       66 0f e6 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(e6) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPD2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69201, 381, 2626},
    /* 2884 : CVTTPS2DQ xmmreg,xmmrm [rm:       f3 0f 5b /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 333 355 001(5b) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_CVTTPS2DQ, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69207, 381, 2627},
    /* 2885 : CVTTSD2SI reg32,xmmrm64 [rm:       norexw f2 0f 2c /r                ] SSE2,WILLAMETTE */
        /* 317 332 355 001(2c) 110 : SSE2,WILLAMETTE */
        {I_CVTTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58148, 248, 2628},
    /* 2886 : CVTTSD2SI reg64,xmmrm64 [rm:       o64 f2 0f 2c /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 324 332 355 001(2c) 110 : LONG,PROT,SSE2,X86_64 */
        {I_CVTTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58155, 386, 2629},
    /* 2887 : DIVPD xmmreg,xmmrm [rm:       66 0f 5e /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5e) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_DIVPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69213, 381, 2630},
    /* 2888 : DIVSD xmmreg,xmmrm [rm:       f2 0f 5e /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5e) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_DIVSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69219, 385, 2631},
    /* 2889 : MAXPD xmmreg,xmmrm [rm:       66 0f 5f /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5f) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MAXPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69225, 381, 2632},
    /* 2890 : MAXSD xmmreg,xmmrm [rm:       f2 0f 5f /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5f) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MAXSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69231, 385, 2633},
    /* 2891 : MINPD xmmreg,xmmrm [rm:       66 0f 5d /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5d) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MINPD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69237, 381, 2634},
    /* 2892 : MINSD xmmreg,xmmrm [rm:       f2 0f 5d /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(5d) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MINSD, 2, {XMMREG,RM_XMM,0,0,0}, NO_DECORATOR, nasm_bytecodes+69243, 385, 2635},
    /* 2893 : MOVAPD xmmreg,xmmrm128 [rm:       66 0f 28 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(28) 110 : SSE2,WILLAMETTE */
        {I_MOVAPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69249, 248, 2636},
    /* 2894 : MOVAPD xmmrm128,xmmreg [mr:       66 0f 29 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(29) 101 : SSE2,WILLAMETTE */
        {I_MOVAPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69255, 248, 2637},
    /* 2895 : MOVHPD mem64,xmmreg [mr:       66 0f 17 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(17) 101 : SSE2,WILLAMETTE */
        {I_MOVHPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69261, 248, 2638},
    /* 2896 : MOVHPD xmmreg,mem64 [rm:       66 0f 16 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(16) 110 : SSE2,WILLAMETTE */
        {I_MOVHPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69267, 248, 2639},
    /* 2897 : MOVLPD mem64,xmmreg [mr:       66 0f 13 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(13) 101 : SSE2,WILLAMETTE */
        {I_MOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69273, 248, 2640},
    /* 2898 : MOVLPD xmmreg,mem64 [rm:       66 0f 12 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(12) 110 : SSE2,WILLAMETTE */
        {I_MOVLPD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69279, 248, 2641},
    /* 2899 : MOVMSKPD reg32,xmmreg [rm:       66 0f 50 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(50) 110 : SSE2,WILLAMETTE */
        {I_MOVMSKPD, 2, {REG_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69285, 248, 2642},
    /* 2900 : MOVMSKPD reg64,xmmreg [rm:       66 o64 0f 50 /r                   ] LONG,PROT,SSE2,X86_64 */
        /* 361 324 355 001(50) 110 : LONG,PROT,SSE2,X86_64 */
        {I_MOVMSKPD, 2, {REG_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+58162, 386, 2643},
    /* 2901 : MOVSD xmmreg,xmmrm64 [rm:       f2 0f 10 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(10) 110 : SSE2,WILLAMETTE */
        {I_MOVSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69291, 248, 2644},
    /* 2902 : MOVSD xmmrm64,xmmreg [mr:       f2 0f 11 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(11) 101 : SSE2,WILLAMETTE */
        {I_MOVSD, 2, {RM_XMM|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69297, 248, 2645},
    /* 2903 : MOVUPD xmmreg,xmmrm128 [rm:       66 0f 10 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(10) 110 : SSE2,WILLAMETTE */
        {I_MOVUPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69303, 248, 2646},
    /* 2904 : MOVUPD xmmrm128,xmmreg [mr:       66 0f 11 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(11) 101 : SSE2,WILLAMETTE */
        {I_MOVUPD, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69309, 248, 2647},
    /* 2905 : MULPD xmmreg,xmmrm128 [rm:       66 0f 59 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(59) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_MULPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69315, 381, 2648},
    /* 2906 : MULSD xmmreg,xmmrm64 [rm:       f2 0f 59 /r                       ] AR0-1,SQ,SSE2,WILLAMETTE */
        /* 332 355 001(59) 110 : AR0,AR1,SQ,SSE2,WILLAMETTE */
        {I_MULSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69321, 385, 2649},
    /* 2907 : ORPD xmmreg,xmmrm128 [rm:       66 0f 56 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(56) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_ORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69327, 381, 2650},
    /* 2908 : SHUFPD xmmreg,xmmrm128,imm8 [rmi:      66 0f c6 /r ib,u                  ] SSE2,WILLAMETTE */
        /* 361 355 001(c6) 110 026 : SSE2,WILLAMETTE */
        {I_SHUFPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58169, 248, 2651},
    /* 2909 : SQRTPD xmmreg,xmmrm128 [rm:       66 0f 51 /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(51) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_SQRTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69333, 381, 2652},
    /* 2910 : SQRTSD xmmreg,xmmrm64 [rm:       f2 0f 51 /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(51) 110 : SSE2,WILLAMETTE */
        {I_SQRTSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69339, 248, 2653},
    /* 2911 : SUBPD xmmreg,xmmrm128 [rm:       66 0f 5c /r                       ] AR0-1,SO,SSE2,WILLAMETTE */
        /* 361 355 001(5c) 110 : AR0,AR1,SO,SSE2,WILLAMETTE */
        {I_SUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69345, 381, 2654},
    /* 2912 : SUBSD xmmreg,xmmrm64 [rm:       f2 0f 5c /r                       ] SSE2,WILLAMETTE */
        /* 332 355 001(5c) 110 : SSE2,WILLAMETTE */
        {I_SUBSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69351, 248, 2655},
    /* 2913 : UCOMISD xmmreg,xmmrm64 [rm:       66 0f 2e /r                       ] FL,SSE2,WILLAMETTE */
        /* 361 355 001(2e) 110 : FL,SSE2,WILLAMETTE */
        {I_UCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69357, 391, 2656},
    /* 2914 : UNPCKHPD xmmreg,xmmrm128 [rm:       66 0f 15 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(15) 110 : SSE2,WILLAMETTE */
        {I_UNPCKHPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69363, 248, 2657},
    /* 2915 : UNPCKLPD xmmreg,xmmrm128 [rm:       66 0f 14 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(14) 110 : SSE2,WILLAMETTE */
        {I_UNPCKLPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69369, 248, 2658},
    /* 2916 : XORPD xmmreg,xmmrm128 [rm:       66 0f 57 /r                       ] SSE2,WILLAMETTE */
        /* 361 355 001(57) 110 : SSE2,WILLAMETTE */
        {I_XORPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69375, 248, 2659},
    /* 2917 : ADDSUBPD xmmreg,xmmrm128 [rm:       66 0f d0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(d0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_ADDSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69381, 393, 2662},
    /* 2918 : ADDSUBPS xmmreg,xmmrm128 [rm:       f2 0f d0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(d0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_ADDSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69387, 393, 2663},
    /* 2919 : HADDPD xmmreg,xmmrm128 [rm:       66 0f 7c /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(7c) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HADDPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69393, 393, 2664},
    /* 2920 : HADDPS xmmreg,xmmrm128 [rm:       f2 0f 7c /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(7c) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HADDPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69399, 393, 2665},
    /* 2921 : HSUBPD xmmreg,xmmrm128 [rm:       66 0f 7d /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 361 355 001(7d) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HSUBPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69405, 393, 2666},
    /* 2922 : HSUBPS xmmreg,xmmrm128 [rm:       f2 0f 7d /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(7d) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_HSUBPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69411, 393, 2667},
    /* 2923 : LDDQU xmmreg,mem128 [rm:       f2 0f f0 /r                       ] AR0-1,SO,SSE3,PRESCOTT */
        /* 332 355 001(f0) 110 : AR0,AR1,SO,SSE3,PRESCOTT */
        {I_LDDQU, 2, {XMMREG,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69417, 393, 2668},
    /* 2924 : MOVDDUP xmmreg,xmmrm64 [rm:       f2 0f 12 /r                       ] AR0-1,SQ,SSE3,PRESCOTT */
        /* 332 355 001(12) 110 : AR0,AR1,SQ,SSE3,PRESCOTT */
        {I_MOVDDUP, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+69423, 394, 2669},
    /* 2925 : MOVSHDUP xmmreg,xmmrm128 [rm:       f3 0f 16 /r                       ] SSE3,PRESCOTT */
        /* 333 355 001(16) 110 : SSE3,PRESCOTT */
        {I_MOVSHDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69429, 395, 2670},
    /* 2926 : MOVSLDUP xmmreg,xmmrm128 [rm:       f3 0f 12 /r                       ] SSE3,PRESCOTT */
        /* 333 355 001(12) 110 : SSE3,PRESCOTT */
        {I_MOVSLDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+69435, 395, 2671},
    /* 2927 : CLGI void [          0f 01 dd                          ] VMX,FUTURE,AMD */
        /* 355 002(01 dd) : VMX,FUTURE,AMD */
        {I_CLGI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72464, 396, 2674},
    /* 2928 : STGI void [          0f 01 dc                          ] VMX,FUTURE,AMD */
        /* 355 002(01 dc) : VMX,FUTURE,AMD */
        {I_STGI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72469, 396, 2675},
    /* 2929 : VMCALL void [          0f 01 c1                          ] VMX,FUTURE */
        /* 355 002(01 c1) : VMX,FUTURE */
        {I_VMCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69478, 397, 2676},
    /* 2930 : VMCLEAR mem [m:        66 0f c7 /6                       ] VMX,FUTURE */
        /* 361 355 001(c7) 206 : VMX,FUTURE */
        {I_VMCLEAR, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69441, 397, 2677},
    /* 2931 : VMFUNC void [          0f 01 d4                          ] VMX,FUTURE */
        /* 355 002(01 d4) : VMX,FUTURE */
        {I_VMFUNC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72474, 397, 2678},
    /* 2932 : VMLAUNCH void [          0f 01 c2                          ] VMX,FUTURE */
        /* 355 002(01 c2) : VMX,FUTURE */
        {I_VMLAUNCH, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72479, 397, 2679},
    /* 2933 : VMLOAD void [          0f 01 da                          ] VMX,FUTURE,AMD */
        /* 355 002(01 da) : VMX,FUTURE,AMD */
        {I_VMLOAD, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72484, 396, 2680},
    /* 2934 : VMMCALL void [          0f 01 d9                          ] VMX,FUTURE,AMD */
        /* 355 002(01 d9) : VMX,FUTURE,AMD */
        {I_VMMCALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72489, 396, 2681},
    /* 2935 : VMPTRLD mem [m:        np 0f c7 /6                       ] VMX,FUTURE */
        /* 360 355 001(c7) 206 : VMX,FUTURE */
        {I_VMPTRLD, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69447, 397, 2682},
    /* 2936 : VMPTRST mem [m:        np 0f c7 /7                       ] VMX,FUTURE */
        /* 360 355 001(c7) 207 : VMX,FUTURE */
        {I_VMPTRST, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69453, 397, 2683},
    /* 2937 : VMREAD rm32,reg32 [mr:       np 0f 78 /r                       ] AR0-1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 360 355 001(78) 101 : AR0,AR1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_VMREAD, 2, {RM_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58177, 398, 2684},
    /* 2938 : VMREAD rm64,reg64 [mr:       o64nw np 0f 78 /r                 ] AR0-1,SQ,LONG,PROT,VMX,X86_64 */
        /* 323 360 355 001(78) 101 : AR0,AR1,SQ,NWSIZE,LONG,PROT,VMX,X86_64 */
        {I_VMREAD, 2, {RM_GPR|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58176, 399, 2685},
    /* 2939 : VMRESUME void [          0f 01 c3                          ] VMX,FUTURE */
        /* 355 002(01 c3) : VMX,FUTURE */
        {I_VMRESUME, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72494, 397, 2686},
    /* 2940 : VMRUN void [          0f 01 d8                          ] VMX,FUTURE,AMD */
        /* 355 002(01 d8) : VMX,FUTURE,AMD */
        {I_VMRUN, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72499, 396, 2687},
    /* 2941 : VMSAVE void [          0f 01 db                          ] VMX,FUTURE,AMD */
        /* 355 002(01 db) : VMX,FUTURE,AMD */
        {I_VMSAVE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72504, 396, 2688},
    /* 2942 : VMWRITE reg32,rm32 [rm:       np 0f 79 /r                       ] AR0-1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 360 355 001(79) 110 : AR0,AR1,SD,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_VMWRITE, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58184, 398, 2689},
    /* 2943 : VMWRITE reg64,rm64 [rm:       o64nw np 0f 79 /r                 ] AR0-1,SQ,LONG,PROT,VMX,X86_64 */
        /* 323 360 355 001(79) 110 : AR0,AR1,SQ,NWSIZE,LONG,PROT,VMX,X86_64 */
        {I_VMWRITE, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58183, 399, 2690},
    /* 2944 : VMXOFF void [          0f 01 c4                          ] VMX,FUTURE */
        /* 355 002(01 c4) : VMX,FUTURE */
        {I_VMXOFF, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72509, 397, 2691},
    /* 2945 : VMXON mem [m:        f3 0f c7 /6                       ] VMX,FUTURE */
        /* 333 355 001(c7) 206 : VMX,FUTURE */
        {I_VMXON, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66374, 397, 2692},
    /* 2946 : INVEPT reg32,mem [rm:       66 0f38 80 /r                     ] AR0-1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 361 356 001(80) 110 : AR0,AR1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_INVEPT, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58191, 400, 2694},
    /* 2947 : INVEPT reg64,mem [rm:       o64nw 66 0f38 80 /r               ] AR0-1,SO,LONG,PROT,VMX,X86_64 */
        /* 323 361 356 001(80) 110 : AR0,AR1,SO,NWSIZE,NOAPX,LONG,PROT,VMX,X86_64 */
        {I_INVEPT, 2, {REG_GPR|BITS64|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58190, 401, 2695},
    /* 2948 : INVEPT reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f0 /r    ] AR0-1,SO,LONG,PROT,VMX,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f0) 110 : AR0,AR1,SO,LONG,EVEX,PROT,VMX,APX,X86_64 */
        {I_INVEPT, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+25717, 402, 2696},
    /* 2949 : INVVPID reg32,mem [rm:       66 0f38 81 /r                     ] AR0-1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        /* 361 356 001(81) 110 : AR0,AR1,SO,NOREX,NOAPX,NOLONG,VMX,FUTURE */
        {I_INVVPID, 2, {REG_GPR|BITS32|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58198, 400, 2697},
    /* 2950 : INVVPID reg64,mem [rm:       o64nw 66 0f38 81 /r               ] AR0-1,SO,LONG,PROT,VMX,X86_64 */
        /* 323 361 356 001(81) 110 : AR0,AR1,SO,NWSIZE,NOAPX,LONG,PROT,VMX,X86_64 */
        {I_INVVPID, 2, {REG_GPR|BITS64|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58197, 401, 2698},
    /* 2951 : INVVPID reg64,mem128 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f1 /r    ] AR0-1,SO,LONG,PROT,VMX,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f1) 110 : AR0,AR1,SO,LONG,EVEX,PROT,VMX,APX,X86_64 */
        {I_INVVPID, 2, {REG_GPR|BITS64,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+25726, 402, 2699},
    /* 2952 : PVALIDATE void [          f2 0f 01 ff                       ] VMX,FUTURE,AMD */
        /* 332 355 002(01 ff) : VMX,FUTURE,AMD */
        {I_PVALIDATE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69459, 396, 2701},
    /* 2953 : RMPADJUST void [          f3 0f 01 fe                       ] VMX,FUTURE,AMD */
        /* 333 355 002(01 fe) : VMX,FUTURE,AMD */
        {I_RMPADJUST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69465, 396, 2702},
    /* 2954 : VMGEXIT void [          f2 0f 01 c1                       ] VMX,FUTURE,AMD */
        /* 332 355 002(01 c1) : VMX,FUTURE,AMD */
        {I_VMGEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69471, 396, 2703},
    /* 2955 : VMGEXIT void [          f3 0f 01 c1                       ] VMX,FUTURE,AMD */
        /* 333 355 002(01 c1) : VMX,FUTURE,AMD */
        {I_VMGEXIT, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69477, 396, 2704},
    /* 2956 : PABSB mmxreg,mmxrm [rm:       np 0f38 1c /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1c) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69483, 403, 2707},
    /* 2957 : PABSB xmmreg,xmmrm128 [rm:       66 0f38 1c /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1c) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69489, 404, 2708},
    /* 2958 : PABSW mmxreg,mmxrm [rm:       np 0f38 1d /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1d) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69495, 403, 2709},
    /* 2959 : PABSW xmmreg,xmmrm128 [rm:       66 0f38 1d /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1d) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69501, 404, 2710},
    /* 2960 : PABSD mmxreg,mmxrm [rm:       np 0f38 1e /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(1e) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PABSD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69507, 403, 2711},
    /* 2961 : PABSD xmmreg,xmmrm128 [rm:       66 0f38 1e /r                     ] SSSE3,FUTURE */
        /* 361 356 001(1e) 110 : NOAPX,SSSE3,FUTURE */
        {I_PABSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69513, 404, 2712},
    /* 2962 : PALIGNR mmxreg,mmxrm,imm [rmi:      np 0f3a 0f /r ib,u                ] AR0-2,SQ,MMX,SSSE3,FUTURE */
        /* 360 357 001(0f) 110 026 : AR0,AR1,AR2,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PALIGNR, 3, {MMXREG|RN_L16,RM_MMX|RN_L16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58204, 405, 2713},
    /* 2963 : PALIGNR xmmreg,xmmrm,imm [rmi:      66 0f3a 0f /r ib,u                ] SSSE3,FUTURE */
        /* 361 357 001(0f) 110 026 : NOAPX,SSSE3,FUTURE */
        {I_PALIGNR, 3, {XMMREG|RN_L16,RM_XMM|RN_L16,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+58211, 404, 2714},
    /* 2964 : PHADDW mmxreg,mmxrm [rm:       np 0f38 01 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(01) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69519, 403, 2715},
    /* 2965 : PHADDW xmmreg,xmmrm128 [rm:       66 0f38 01 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(01) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69525, 404, 2716},
    /* 2966 : PHADDD mmxreg,mmxrm [rm:       np 0f38 02 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(02) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69531, 403, 2717},
    /* 2967 : PHADDD xmmreg,xmmrm128 [rm:       66 0f38 02 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(02) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69537, 404, 2718},
    /* 2968 : PHADDSW mmxreg,mmxrm [rm:       np 0f38 03 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(03) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHADDSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69543, 403, 2719},
    /* 2969 : PHADDSW xmmreg,xmmrm128 [rm:       66 0f38 03 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(03) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHADDSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69549, 404, 2720},
    /* 2970 : PHSUBW mmxreg,mmxrm [rm:       np 0f38 05 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(05) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69555, 403, 2721},
    /* 2971 : PHSUBW xmmreg,xmmrm128 [rm:       66 0f38 05 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(05) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69561, 404, 2722},
    /* 2972 : PHSUBD mmxreg,mmxrm [rm:       np 0f38 06 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(06) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBD, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69567, 403, 2723},
    /* 2973 : PHSUBD xmmreg,xmmrm128 [rm:       66 0f38 06 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(06) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69573, 404, 2724},
    /* 2974 : PHSUBSW mmxreg,mmxrm [rm:       np 0f38 07 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(07) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PHSUBSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69579, 403, 2725},
    /* 2975 : PHSUBSW xmmreg,xmmrm128 [rm:       66 0f38 07 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(07) 110 : NOAPX,SSSE3,FUTURE */
        {I_PHSUBSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69585, 404, 2726},
    /* 2976 : PMADDUBSW mmxreg,mmxrm [rm:       np 0f38 04 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(04) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PMADDUBSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69591, 403, 2727},
    /* 2977 : PMADDUBSW xmmreg,xmmrm128 [rm:       66 0f38 04 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(04) 110 : NOAPX,SSSE3,FUTURE */
        {I_PMADDUBSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69597, 404, 2728},
    /* 2978 : PMULHRSW mmxreg,mmxrm [rm:       np 0f38 0b /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(0b) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PMULHRSW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69603, 403, 2729},
    /* 2979 : PMULHRSW xmmreg,xmmrm128 [rm:       66 0f38 0b /r                     ] SSSE3,FUTURE */
        /* 361 356 001(0b) 110 : NOAPX,SSSE3,FUTURE */
        {I_PMULHRSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69609, 404, 2730},
    /* 2980 : PSHUFB mmxreg,mmxrm [rm:       np 0f38 00 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(00) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSHUFB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69615, 403, 2731},
    /* 2981 : PSHUFB xmmreg,xmmrm128 [rm:       66 0f38 00 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(00) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSHUFB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69621, 404, 2732},
    /* 2982 : PSIGNB mmxreg,mmxrm [rm:       np 0f38 08 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(08) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGNB, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69627, 403, 2733},
    /* 2983 : PSIGNB xmmreg,xmmrm128 [rm:       66 0f38 08 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(08) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGNB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69633, 404, 2734},
    /* 2984 : PSIGNW mmxreg,mmxrm [rm:       np 0f38 09 /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(09) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGNW, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69639, 403, 2735},
    /* 2985 : PSIGNW xmmreg,xmmrm128 [rm:       66 0f38 09 /r                     ] SSSE3,FUTURE */
        /* 361 356 001(09) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGNW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69645, 404, 2736},
    /* 2986 : PSIGND mmxreg,mmxrm [rm:       np 0f38 0a /r                     ] AR0-1,SQ,MMX,SSSE3,FUTURE */
        /* 360 356 001(0a) 110 : AR0,AR1,SQ,NOAPX,MMX,SSSE3,FUTURE */
        {I_PSIGND, 2, {MMXREG|RN_L16,RM_MMX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69651, 403, 2737},
    /* 2987 : PSIGND xmmreg,xmmrm128 [rm:       66 0f38 0a /r                     ] SSSE3,FUTURE */
        /* 361 356 001(0a) 110 : NOAPX,SSSE3,FUTURE */
        {I_PSIGND, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69657, 404, 2738},
    /* 2988 : EXTRQ xmmreg,imm,imm [mij:      66 0f 78 /0 ib,u ib,u             ] SSE4A,FUTURE,AMD */
        /* 361 355 001(78) 200 025 026 : SSE4A,FUTURE,AMD */
        {I_EXTRQ, 3, {XMMREG,IMM_NORMAL,IMM_NORMAL,0,0}, NO_DECORATOR, nasm_bytecodes+51659, 406, 2741},
    /* 2989 : EXTRQ xmmreg,xmmreg [rm:       66 0f 79 /r                       ] SSE4A,FUTURE,AMD */
        /* 361 355 001(79) 110 : SSE4A,FUTURE,AMD */
        {I_EXTRQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69663, 406, 2742},
    /* 2990 : INSERTQ xmmreg,xmmreg,imm,imm [rmij:     f2 0f 78 /r ib,u ib,u             ] SSE4A,FUTURE,AMD */
        /* 332 355 001(78) 110 026 027 : SSE4A,FUTURE,AMD */
        {I_INSERTQ, 4, {XMMREG,XMMREG,IMM_NORMAL,IMM_NORMAL,0}, NO_DECORATOR, nasm_bytecodes+51667, 406, 2743},
    /* 2991 : INSERTQ xmmreg,xmmreg [rm:       f2 0f 79 /r                       ] SSE4A,FUTURE,AMD */
        /* 332 355 001(79) 110 : SSE4A,FUTURE,AMD */
        {I_INSERTQ, 2, {XMMREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69669, 406, 2744},
    /* 2992 : MOVNTSD mem64,xmmreg [mr:       f2 0f 2b /r                       ] AR0-1,SQ,SSE4A,FUTURE,AMD */
        /* 332 355 001(2b) 101 : AR0,AR1,SQ,SSE4A,FUTURE,AMD */
        {I_MOVNTSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69675, 407, 2745},
    /* 2993 : MOVNTSS mem32,xmmreg [mr:       f3 0f 2b /r                       ] AR0-1,SD,SSE4A,FUTURE,AMD */
        /* 333 355 001(2b) 101 : AR0,AR1,SD,SSE4A,FUTURE,AMD */
        {I_MOVNTSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+69681, 408, 2746},
    /* 2994 : BLENDPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0d /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0d) 110 026 : NOAPX,SSE41,FUTURE */
        {I_BLENDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58218, 409, 2751},
    /* 2995 : BLENDPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0c /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0c) 110 026 : NOAPX,SSE41,FUTURE */
        {I_BLENDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58225, 409, 2752},
    /* 2996 : BLENDVPD xmmreg,xmmrm128,xmm0 [rm-:      66 0f38 15 /r                     ] SSE41,FUTURE */
        /* 361 356 001(15) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69687, 409, 2753},
    /* 2997 : BLENDVPD xmmreg,xmmrm128 [rm:       66 0f38 15 /r                     ] SSE41,FUTURE */
        /* 361 356 001(15) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69687, 409, 2754},
    /* 2998 : BLENDVPS xmmreg,xmmrm128,xmm0 [rm-:      66 0f38 14 /r                     ] SSE41,FUTURE */
        /* 361 356 001(14) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69693, 409, 2755},
    /* 2999 : BLENDVPS xmmreg,xmmrm128 [rm:       66 0f38 14 /r                     ] SSE41,FUTURE */
        /* 361 356 001(14) 110 : NOAPX,SSE41,FUTURE */
        {I_BLENDVPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69693, 409, 2756},
    /* 3000 : DPPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 41 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(41) 110 026 : NOAPX,SSE41,FUTURE */
        {I_DPPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58232, 409, 2757},
    /* 3001 : DPPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 40 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(40) 110 026 : NOAPX,SSE41,FUTURE */
        {I_DPPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58239, 409, 2758},
    /* 3002 : EXTRACTPS rm32,xmmreg,imm8 [mri:      66 0f3a 17 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(17) 101 026 : NOAPX,SSE41,FUTURE */
        {I_EXTRACTPS, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51676, 409, 2759},
    /* 3003 : EXTRACTPS reg64,xmmreg,imm8 [mri:      o64 66 0f3a 17 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(17) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_EXTRACTPS, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51675, 410, 2760},
    /* 3004 : INSERTPS xmmreg,xmmrm32,imm8 [rmi:      66 0f3a 21 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(21) 110 026 : NOAPX,SSE41,FUTURE */
        {I_INSERTPS, 3, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58246, 409, 2761},
    /* 3005 : MOVNTDQA xmmreg,mem128 [rm:       66 0f38 2a /r                     ] SSE41,FUTURE */
        /* 361 356 001(2a) 110 : NOAPX,SSE41,FUTURE */
        {I_MOVNTDQA, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69699, 409, 2762},
    /* 3006 : MPSADBW xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 42 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(42) 110 026 : NOAPX,SSE41,FUTURE */
        {I_MPSADBW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58253, 409, 2763},
    /* 3007 : PACKUSDW xmmreg,xmmrm128 [rm:       66 0f38 2b /r                     ] SSE41,FUTURE */
        /* 361 356 001(2b) 110 : NOAPX,SSE41,FUTURE */
        {I_PACKUSDW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69705, 409, 2764},
    /* 3008 : PBLENDVB xmmreg,xmmrm,xmm0 [rm-:      66 0f38 10 /r                     ] SSE41,FUTURE */
        /* 361 356 001(10) 110 : NOAPX,SSE41,FUTURE */
        {I_PBLENDVB, 3, {XMMREG|RN_L16,RM_XMM|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+69711, 409, 2765},
    /* 3009 : PBLENDVB xmmreg,xmmrm128 [rm:       66 0f38 10 /r                     ] SSE41,FUTURE */
        /* 361 356 001(10) 110 : NOAPX,SSE41,FUTURE */
        {I_PBLENDVB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69711, 409, 2766},
    /* 3010 : PBLENDW xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 0e /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0e) 110 026 : NOAPX,SSE41,FUTURE */
        {I_PBLENDW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58260, 409, 2767},
    /* 3011 : PCMPEQQ xmmreg,xmmrm128 [rm:       66 0f38 29 /r                     ] SSE41,FUTURE */
        /* 361 356 001(29) 110 : NOAPX,SSE41,FUTURE */
        {I_PCMPEQQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69717, 409, 2768},
    /* 3012 : PEXTRB reg32,xmmreg,imm8 [mri:      66 0f3a 14 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(14) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRB, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51684, 409, 2769},
    /* 3013 : PEXTRB mem8,xmmreg,imm8 [mri:      66 0f3a 14 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(14) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRB, 3, {MEMORY|BITS8|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51684, 409, 2770},
    /* 3014 : PEXTRB reg64,xmmreg,imm8 [mri:      o64nw 66 0f3a 14 /r ib,u          ] LONG,PROT,SSE41,X86_64 */
        /* 323 361 357 001(14) 101 026 : NWSIZE,NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRB, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51683, 411, 2771},
    /* 3015 : PEXTRD rm32,xmmreg,imm8 [mri:      norexw 66 0f3a 16 /r ib,u         ] SSE41,FUTURE */
        /* 317 361 357 001(16) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRD, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51691, 409, 2772},
    /* 3016 : PEXTRQ rm64,xmmreg,imm8 [mri:      o64 66 0f3a 16 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(16) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRQ, 3, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51699, 410, 2773},
    /* 3017 : PEXTRW reg32,xmmreg,imm8 [mri:      66 0f3a 15 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(15) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51708, 409, 2774},
    /* 3018 : PEXTRW mem16,xmmreg,imm8 [mri:      66 0f3a 15 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(15) 101 026 : NOAPX,SSE41,FUTURE */
        {I_PEXTRW, 3, {MEMORY|BITS16|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51708, 409, 2775},
    /* 3019 : PEXTRW reg64,xmmreg,imm8 [mri:      o64 66 0f3a 15 /r ib,u            ] LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(15) 101 026 : NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51707, 410, 2776},
    /* 3020 : PHMINPOSUW xmmreg,xmmrm128 [rm:       66 0f38 41 /r                     ] SSE41,FUTURE */
        /* 361 356 001(41) 110 : NOAPX,SSE41,FUTURE */
        {I_PHMINPOSUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69723, 409, 2777},
    /* 3021 : PINSRB xmmreg,mem,imm8 [rmi:      66 0f3a 20 /r ib,u                ] AR2,SB,SSE41,FUTURE */
        /* 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,MEMORY|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51716, 412, 2778},
    /* 3022 : PINSRB xmmreg,rm8,imm8 [rmi:      nohi 66 0f3a 20 /r ib,u           ] AR2,SB,SSE41,FUTURE */
        /* 325 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,RM_GPR|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51715, 412, 2779},
    /* 3023 : PINSRB xmmreg,reg32,imm8 [rmi:      66 0f3a 20 /r ib,u                ] AR2,SB,SSE41,FUTURE */
        /* 361 357 001(20) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRB, 3, {XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51716, 412, 2780},
    /* 3024 : PINSRD xmmreg,rm32,imm8 [rmi:      norexw 66 0f3a 22 /r ib,u         ] AR2,SB,SSE41,FUTURE */
        /* 317 361 357 001(22) 110 026 : AR2,SB,NOAPX,SSE41,FUTURE */
        {I_PINSRD, 3, {XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51723, 412, 2781},
    /* 3025 : PINSRQ xmmreg,rm64,imm8 [rmi:      o64 66 0f3a 22 /r ib,u            ] AR2,SB,LONG,PROT,SSE41,X86_64 */
        /* 324 361 357 001(22) 110 026 : AR2,SB,NOAPX,LONG,PROT,SSE41,X86_64 */
        {I_PINSRQ, 3, {XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51731, 413, 2782},
    /* 3026 : PMAXSB xmmreg,xmmrm128 [rm:       66 0f38 3c /r                     ] SSE41,FUTURE */
        /* 361 356 001(3c) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69729, 409, 2783},
    /* 3027 : PMAXSD xmmreg,xmmrm128 [rm:       66 0f38 3d /r                     ] SSE41,FUTURE */
        /* 361 356 001(3d) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69735, 409, 2784},
    /* 3028 : PMAXUD xmmreg,xmmrm128 [rm:       66 0f38 3f /r                     ] SSE41,FUTURE */
        /* 361 356 001(3f) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXUD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69741, 409, 2785},
    /* 3029 : PMAXUW xmmreg,xmmrm128 [rm:       66 0f38 3e /r                     ] SSE41,FUTURE */
        /* 361 356 001(3e) 110 : NOAPX,SSE41,FUTURE */
        {I_PMAXUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69747, 409, 2786},
    /* 3030 : PMINSB xmmreg,xmmrm128 [rm:       66 0f38 38 /r                     ] SSE41,FUTURE */
        /* 361 356 001(38) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69753, 409, 2787},
    /* 3031 : PMINSD xmmreg,xmmrm128 [rm:       66 0f38 39 /r                     ] SSE41,FUTURE */
        /* 361 356 001(39) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69759, 409, 2788},
    /* 3032 : PMINUD xmmreg,xmmrm128 [rm:       66 0f38 3b /r                     ] SSE41,FUTURE */
        /* 361 356 001(3b) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINUD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69765, 409, 2789},
    /* 3033 : PMINUW xmmreg,xmmrm128 [rm:       66 0f38 3a /r                     ] SSE41,FUTURE */
        /* 361 356 001(3a) 110 : NOAPX,SSE41,FUTURE */
        {I_PMINUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69771, 409, 2790},
    /* 3034 : PMOVSXBW xmmreg,xmmrm64 [rm:       66 0f38 20 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(20) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69777, 414, 2791},
    /* 3035 : PMOVSXBD xmmreg,xmmrm32 [rm:       66 0f38 21 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(21) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69783, 415, 2792},
    /* 3036 : PMOVSXBQ xmmreg,xmmrm16 [rm:       66 0f38 22 /r                     ] AR0-1,SW,SSE41,FUTURE */
        /* 361 356 001(22) 110 : AR0,AR1,SW,NOAPX,SSE41,FUTURE */
        {I_PMOVSXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69789, 416, 2793},
    /* 3037 : PMOVSXWD xmmreg,xmmrm64 [rm:       66 0f38 23 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(23) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69795, 414, 2794},
    /* 3038 : PMOVSXWQ xmmreg,xmmrm32 [rm:       66 0f38 24 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(24) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVSXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69801, 415, 2795},
    /* 3039 : PMOVSXDQ xmmreg,xmmrm64 [rm:       66 0f38 25 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(25) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVSXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69807, 414, 2796},
    /* 3040 : PMOVZXBW xmmreg,xmmrm64 [rm:       66 0f38 30 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(30) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69813, 414, 2797},
    /* 3041 : PMOVZXBD xmmreg,xmmrm32 [rm:       66 0f38 31 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(31) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69819, 415, 2798},
    /* 3042 : PMOVZXBQ xmmreg,xmmrm16 [rm:       66 0f38 32 /r                     ] AR0-1,SW,SSE41,FUTURE */
        /* 361 356 001(32) 110 : AR0,AR1,SW,NOAPX,SSE41,FUTURE */
        {I_PMOVZXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69825, 416, 2799},
    /* 3043 : PMOVZXWD xmmreg,xmmrm64 [rm:       66 0f38 33 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(33) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69831, 414, 2800},
    /* 3044 : PMOVZXWQ xmmreg,xmmrm32 [rm:       66 0f38 34 /r                     ] AR0-1,SD,SSE41,FUTURE */
        /* 361 356 001(34) 110 : AR0,AR1,SD,NOAPX,SSE41,FUTURE */
        {I_PMOVZXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69837, 415, 2801},
    /* 3045 : PMOVZXDQ xmmreg,xmmrm64 [rm:       66 0f38 35 /r                     ] AR0-1,SQ,SSE41,FUTURE */
        /* 361 356 001(35) 110 : AR0,AR1,SQ,NOAPX,SSE41,FUTURE */
        {I_PMOVZXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69843, 414, 2802},
    /* 3046 : PMULDQ xmmreg,xmmrm128 [rm:       66 0f38 28 /r                     ] SSE41,FUTURE */
        /* 361 356 001(28) 110 : NOAPX,SSE41,FUTURE */
        {I_PMULDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69849, 409, 2803},
    /* 3047 : PMULLD xmmreg,xmmrm128 [rm:       66 0f38 40 /r                     ] SSE41,FUTURE */
        /* 361 356 001(40) 110 : NOAPX,SSE41,FUTURE */
        {I_PMULLD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69855, 409, 2804},
    /* 3048 : PTEST xmmreg,xmmrm128 [rm:       66 0f38 17 /r                     ] SSE41,FUTURE */
        /* 361 356 001(17) 110 : NOAPX,SSE41,FUTURE */
        {I_PTEST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69861, 409, 2805},
    /* 3049 : ROUNDPD xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 09 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(09) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58267, 409, 2806},
    /* 3050 : ROUNDPS xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 08 /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(08) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58274, 409, 2807},
    /* 3051 : ROUNDSD xmmreg,xmmrm64,imm8 [rmi:      66 0f3a 0b /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0b) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDSD, 3, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58281, 409, 2808},
    /* 3052 : ROUNDSS xmmreg,xmmrm32,imm8 [rmi:      66 0f3a 0a /r ib,u                ] SSE41,FUTURE */
        /* 361 357 001(0a) 110 026 : NOAPX,SSE41,FUTURE */
        {I_ROUNDSS, 3, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58288, 409, 2809},
    /* 3053 : CRC32 reg32,rm8 [rm:       o8 f2 0f38 f0 /r                  ] SSE42,FUTURE */
        /* 332 356 001(f0) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58317, 417, 2812},
    /* 3054 : CRC32 reg32,rm16 [rm:       o16 f2 0f38 f1 /r                 ] SSE42,FUTURE */
        /* 320 332 356 001(f1) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58295, 417, 2813},
    /* 3055 : CRC32 reg32,rm32 [rm:       o32 f2 0f38 f1 /r                 ] SSE42,FUTURE */
        /* 321 332 356 001(f1) 110 : NOAPX,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58302, 417, 2814},
    /* 3056 : CRC32 reg32,rm8 [rm:       evex.l0.m4.o8 f0 /r               ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f0) 110 : WIG,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+25735, 419, 2816},
    /* 3057 : CRC32 reg32,rm16 [rm:       evex.l0.m4.o16 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f1) 110 : LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17923, 420, 2817},
    /* 3058 : CRC32 reg32,rm32 [rm:       evex.l0.m4.o32 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17933, 421, 2818},
    /* 3059 : CRC32 reg64,rm8 [rm:       o8 f2 0f38 f0 /r                  ] OPT,LONG,PROT,SSE42,X86_64 */
        /* 332 356 001(f0) 110 : OPT,NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58317, 422, 2820},
    /* 3060 : CRC32 reg64,rm16 [rm:       o16 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 320 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58295, 418, 2821},
    /* 3061 : CRC32 reg64,rm32 [rm:       o32 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 321 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58302, 418, 2822},
    /* 3062 : CRC32 reg64,rm64 [rm:       o64 f2 0f38 f1 /r                 ] LONG,PROT,SSE42,X86_64 */
        /* 324 332 356 001(f1) 110 : NOAPX,LONG,PROT,SSE42,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58309, 418, 2823},
    /* 3063 : CRC32 reg64,rm8 [rm:       evex.l0.m4.o8 f0 /r               ] OPT,LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(f0) 110 : OPT,WIG,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+25735, 423, 2824},
    /* 3064 : CRC32 reg64,rm16 [rm:       evex.l0.m4.o16 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7d 08) 300 320 001(f1) 110 : LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17923, 420, 2825},
    /* 3065 : CRC32 reg64,rm32 [rm:       evex.l0.m4.o32 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17933, 421, 2826},
    /* 3066 : CRC32 reg64,rm64 [rm:       evex.l0.m4.o64 f1 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f1) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17943, 421, 2827},
    /* 3067 : CRC32 reg64,rm8 [rm:       o64 f2 0f38 f0 /r                 ] LONG,SSE42,X86_64,FUTURE */
        /* 324 332 356 001(f0) 110 : NOAPX,LONG,PROT,SSE42,FUTURE */
        {I_CRC32, 2, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58316, 424, 2829},
    /* 3068 : CRC32 reg64,rm8 [rm:       evex.l0.m4.o64 f0 /r              ] LONG,PROT,SSE42,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(f0) 110 : WW,LONG,EVEX,PROT,SSE42,APX,X86_64 */
        {I_CRC32, 2, {REG_GPR|BITS64,RM_GPR|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+17953, 421, 2830},
    /* 3069 : PCMPESTRI xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 61 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(61) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPESTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58323, 417, 2832},
    /* 3070 : PCMPESTRM xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 60 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(60) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPESTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58330, 417, 2833},
    /* 3071 : PCMPISTRI xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 63 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(63) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPISTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58337, 417, 2834},
    /* 3072 : PCMPISTRM xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 62 /r ib,u                ] SSE42,FUTURE */
        /* 361 357 001(62) 110 026 : NOAPX,SSE42,FUTURE */
        {I_PCMPISTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58344, 417, 2835},
    /* 3073 : PCMPGTQ xmmreg,xmmrm128 [rm:       66 0f38 37 /r                     ] SSE42,FUTURE */
        /* 361 356 001(37) 110 : NOAPX,SSE42,FUTURE */
        {I_PCMPGTQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69867, 417, 2836},
    /* 3074 : POPCNT reg16,rm16 [rm:       o16 f3i 0f b8 /r                  ] AR0-1,SW,FL,NEHALEM */
        /* 320 333 355 001(b8) 110 : AR0,AR1,SW,FL,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58351, 425, 2837},
    /* 3075 : POPCNT reg32,rm32 [rm:       o32 f3i 0f b8 /r                  ] AR0-1,SD,FL,NEHALEM */
        /* 321 333 355 001(b8) 110 : AR0,AR1,SD,FL,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+58358, 426, 2838},
    /* 3076 : POPCNT reg64,rm64 [rm:       o64 f3i 0f b8 /r                  ] AR0-1,SQ,LONG,FL,PROT,X86_64,NEHALEM */
        /* 324 333 355 001(b8) 110 : AR0,AR1,SQ,LONG,FL,PROT,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+58365, 427, 2839},
    /* 3077 : POPCNT reg16,rm16 [rm:       evex.nf.nd0.l0.m4.o16 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 7d 08) 300 320 001(88) 110 : SM0,SM1,NF,NF_E,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS16,RM_GPR|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+17963, 428, 2840},
    /* 3078 : POPCNT reg32,rm32 [rm:       evex.nf.nd0.l0.m4.o32 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 7c 08) 300 321 001(88) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS32,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+17973, 429, 2841},
    /* 3079 : POPCNT reg64,rm64 [rm:       evex.nf.nd0.l0.m4.o64 88 /r       ] SM0-1,NF,LONG,PROT,APX,X86_64,NEHALEM */
        /* 250(f4 fc 08) 300 324 001(88) 110 : SM0,SM1,NF,NF_E,WW,LONG,EVEX,PROT,APX,NEHALEM */
        {I_POPCNT, 2, {REG_GPR|BITS64,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+17983, 429, 2842},
    /* 3080 : GETSEC void [          0f 37                             ] NOAPX,FL,KATMAI */
        /* 355 001(37) : NOAPX,FL,KATMAI */
        {I_GETSEC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+73444, 430, 2845},
    /* 3081 : PFRCPV mmxreg,mmxrm [rm:       0f 0f /r 86                       ] AR0-1,SQ,3DNOW,PENT,CYRIX */
        /* 355 001(0f) 110 001(86) : AR0,AR1,SQ,3DNOW,PENT,CYRIX */
        {I_PFRCPV, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+58372, 431, 2848},
    /* 3082 : PFRSQRTV mmxreg,mmxrm [rm:       0f 0f /r 87                       ] AR0-1,SQ,3DNOW,PENT,CYRIX */
        /* 355 001(0f) 110 001(87) : AR0,AR1,SQ,3DNOW,PENT,CYRIX */
        {I_PFRSQRTV, 2, {MMXREG,RM_MMX,0,0,0}, NO_DECORATOR, nasm_bytecodes+58379, 431, 2849},
    /* 3083 : AESENC xmmreg,xmmrm128 [rm:       66 0f38 dc /r                     ] SSE,WESTMERE */
        /* 361 356 001(dc) 110 : NOAPX,SSE,WESTMERE */
        {I_AESENC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69873, 432, 2855},
    /* 3084 : AESENCLAST xmmreg,xmmrm128 [rm:       66 0f38 dd /r                     ] SSE,WESTMERE */
        /* 361 356 001(dd) 110 : NOAPX,SSE,WESTMERE */
        {I_AESENCLAST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69879, 432, 2856},
    /* 3085 : AESDEC xmmreg,xmmrm128 [rm:       66 0f38 de /r                     ] SSE,WESTMERE */
        /* 361 356 001(de) 110 : NOAPX,SSE,WESTMERE */
        {I_AESDEC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69885, 432, 2857},
    /* 3086 : AESDECLAST xmmreg,xmmrm128 [rm:       66 0f38 df /r                     ] SSE,WESTMERE */
        /* 361 356 001(df) 110 : NOAPX,SSE,WESTMERE */
        {I_AESDECLAST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69891, 432, 2858},
    /* 3087 : AESIMC xmmreg,xmmrm128 [rm:       66 0f38 db /r                     ] SSE,WESTMERE */
        /* 361 356 001(db) 110 : NOAPX,SSE,WESTMERE */
        {I_AESIMC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69897, 432, 2859},
    /* 3088 : AESKEYGENASSIST xmmreg,xmmrm128,imm8 [rmi:      66 0f3a df /r ib                  ] SSE,WESTMERE */
        /* 361 357 001(df) 110 022 : NOAPX,SSE,WESTMERE */
        {I_AESKEYGENASSIST, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+58386, 432, 2860},
    /* 3089 : VAESENC xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 dc /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(dc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESENC, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58393, 433, 2863},
    /* 3090 : VAESENCLAST xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 dd /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(dd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESENCLAST, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58407, 433, 2864},
    /* 3091 : VAESDEC xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 de /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(de) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESDEC, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58421, 433, 2865},
    /* 3092 : VAESDECLAST xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 df /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(df) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESDECLAST, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58435, 433, 2866},
    /* 3093 : VAESIMC xmmreg,xmmrm128 [rm:       vex.128.66.0f38 db /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(db) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESIMC, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58449, 433, 2867},
    /* 3094 : VAESKEYGENASSIST xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a df /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(df) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VAESKEYGENASSIST, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+51739, 433, 2868},
    /* 3095 : AESDEC128KL xmmreg,mem [rm:       f3 0f38 dd /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dd) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESDEC128KL, 2, {XMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69903, 434, 2871},
    /* 3096 : AESDEC256KL xmmreg,mem512 [rm:       f3 0f38 df /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(df) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESDEC256KL, 2, {XMMREG|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69909, 434, 2872},
    /* 3097 : AESENC128KL xmmreg,mem [rm:       f3 0f38 dc /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dc) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESENC128KL, 2, {XMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69915, 434, 2873},
    /* 3098 : AESENC256KL xmmreg,mem512 [rm:       f3 0f38 de /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(de) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_AESENC256KL, 2, {XMMREG|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69921, 434, 2874},
    /* 3099 : ENCODEKEY128 reg32,reg32 [rm:       f3 0f38 fa /r                     ] AESKLE,FUTURE */
        /* 333 356 001(fa) 110 : NOAPX,AESKLE,FUTURE */
        {I_ENCODEKEY128, 2, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69927, 435, 2875},
    /* 3100 : ENCODEKEY256 reg32,reg32 [rm:       f3 0f38 fb /r                     ] AESKLE,FUTURE */
        /* 333 356 001(fb) 110 : NOAPX,AESKLE,FUTURE */
        {I_ENCODEKEY256, 2, {REG_GPR|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69933, 435, 2876},
    /* 3101 : LOADIWKEY xmmreg,xmmreg [rm:       f3 0f38 dc /r                     ] FL,AESKLE,FUTURE */
        /* 333 356 001(dc) 110 : NOAPX,FL,AESKLE,FUTURE */
        {I_LOADIWKEY, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+69915, 434, 2877},
    /* 3102 : AESDECWIDE128KL mem [m:        f3 0f38 d8 /1                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 201 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESDECWIDE128KL, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69939, 436, 2878},
    /* 3103 : AESDECWIDE256KL mem512 [m:        f3 0f38 d8 /3                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 203 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESDECWIDE256KL, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69945, 436, 2879},
    /* 3104 : AESENCWIDE128KL mem [m:        f3 0f38 d8 /0                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 200 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESENCWIDE128KL, 1, {MEMORY|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69951, 436, 2880},
    /* 3105 : AESENCWIDE256KL mem512 [m:        f3 0f38 d8 /2                     ] FL,AESKLEWIDE_KL,FUTURE */
        /* 333 356 001(d8) 202 : NOAPX,FL,AESKLEWIDE_KL,FUTURE */
        {I_AESENCWIDE256KL, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69957, 436, 2881},
    /* 3106 : VAESENC ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig dc /r     ] VAES,FUTURE */
        /* 261(002 005) 001(dc) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESENC, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58456, 437, 2886},
    /* 3107 : VAESENCLAST ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig dd /r     ] VAES,FUTURE */
        /* 261(002 005) 001(dd) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESENCLAST, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58470, 437, 2887},
    /* 3108 : VAESDEC ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig de /r     ] VAES,FUTURE */
        /* 261(002 005) 001(de) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESDEC, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58484, 437, 2888},
    /* 3109 : VAESDECLAST ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.wig df /r     ] VAES,FUTURE */
        /* 261(002 005) 001(df) 120 : NOAPX,WIG,VEX,VAES,FUTURE */
        {I_VAESDECLAST, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58498, 437, 2889},
    /* 3110 : VAESENC xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig dc /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25744, 438, 2892},
    /* 3111 : VAESENC ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig dc /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENC, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25762, 438, 2893},
    /* 3112 : VAESENCLAST xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig dd /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENCLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25780, 438, 2894},
    /* 3113 : VAESENCLAST ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig dd /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESENCLAST, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25798, 438, 2895},
    /* 3114 : VAESDEC xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig de /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDEC, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25816, 438, 2896},
    /* 3115 : VAESDEC ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig de /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDEC, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25834, 438, 2897},
    /* 3116 : VAESDECLAST xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f38.wig df /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 08) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDECLAST, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+25852, 438, 2898},
    /* 3117 : VAESDECLAST ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f38.wig df /r    ] AVX512,AVX512VL,VAES,FUTURE */
        /* 241(f2 7d 28) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,AVX512VL,VAES,FUTURE */
        {I_VAESDECLAST, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+25870, 438, 2899},
    /* 3118 : VAESENC zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig dc /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(dc) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESENC, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25888, 439, 2902},
    /* 3119 : VAESENCLAST zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig dd /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(dd) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESENCLAST, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25906, 439, 2903},
    /* 3120 : VAESDEC zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig de /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(de) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESDEC, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25924, 439, 2904},
    /* 3121 : VAESDECLAST zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f38.wig df /r    ] AVX512,VAES,FUTURE */
        /* 241(f2 7d 48) 301 001(df) 120 : WIG,EVEX,PROT,AVX512,VAES,FUTURE */
        {I_VAESDECLAST, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+25942, 439, 2905},
    /* 3122 : VADDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58512, 433, 2908},
    /* 3123 : VADDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58526, 433, 2909},
    /* 3124 : VADDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 58 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58540, 433, 2910},
    /* 3125 : VADDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 58 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(58) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58554, 433, 2911},
    /* 3126 : VADDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(58) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58568, 440, 2912},
    /* 3127 : VADDSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 58 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(58) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58582, 440, 2913},
    /* 3128 : VADDSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58596, 433, 2914},
    /* 3129 : VADDSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58610, 433, 2915},
    /* 3130 : VADDSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58624, 433, 2916},
    /* 3131 : VADDSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f d0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(d0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VADDSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58638, 433, 2917},
    /* 3132 : VANDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 54 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58652, 433, 2918},
    /* 3133 : VANDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 54 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58666, 433, 2919},
    /* 3134 : VANDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 54 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58680, 433, 2920},
    /* 3135 : VANDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 54 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(54) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58694, 433, 2921},
    /* 3136 : VANDNPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 55 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58708, 433, 2922},
    /* 3137 : VANDNPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 55 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58722, 433, 2923},
    /* 3138 : VANDNPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 55 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58736, 433, 2924},
    /* 3139 : VANDNPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 55 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(55) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VANDNPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58750, 433, 2925},
    /* 3140 : VBLENDPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0d /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0d) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51747, 433, 2926},
    /* 3141 : VBLENDPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0d /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(0d) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51763, 433, 2927},
    /* 3142 : VBLENDPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0c /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0c) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51779, 433, 2928},
    /* 3143 : VBLENDPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0c /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(0c) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51795, 433, 2929},
    /* 3144 : VBLENDVPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4b /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4b) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51811, 441, 2930},
    /* 3145 : VBLENDVPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a.w0 4b /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(4b) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51827, 441, 2931},
    /* 3146 : VBLENDVPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4a /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4a) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51843, 441, 2932},
    /* 3147 : VBLENDVPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a.w0 4a /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(4a) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBLENDVPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+51859, 441, 2933},
    /* 3148 : VBROADCASTSS xmmreg,mem32 [rm:       vex.128.66.0f38.w0 18 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(18) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSS, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58764, 441, 2934},
    /* 3149 : VBROADCASTSS ymmreg,mem32 [rm:       vex.256.66.0f38.w0 18 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(18) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSS, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58771, 441, 2935},
    /* 3150 : VBROADCASTSD ymmreg,mem64 [rm:       vex.256.66.0f38.w0 19 /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(19) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTSD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58778, 441, 2936},
    /* 3151 : VBROADCASTF128 ymmreg,mem128 [rm:       vex.256.66.0f38.w0 1a /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(1a) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VBROADCASTF128, 2, {YMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58785, 441, 2937},
    /* 3152 : VCMPEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+25960, 433, 2939},
    /* 3153 : VCMPEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+25978, 433, 2940},
    /* 3154 : VCMPEQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+25996, 433, 2941},
    /* 3155 : VCMPEQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26014, 433, 2942},
    /* 3156 : VCMPLT_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26032, 433, 2943},
    /* 3157 : VCMPLT_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26050, 433, 2944},
    /* 3158 : VCMPLTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26032, 433, 2945},
    /* 3159 : VCMPLTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26050, 433, 2946},
    /* 3160 : VCMPLE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26068, 433, 2947},
    /* 3161 : VCMPLE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26086, 433, 2948},
    /* 3162 : VCMPLEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26068, 433, 2949},
    /* 3163 : VCMPLEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26086, 433, 2950},
    /* 3164 : VCMPUNORD_QPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26104, 433, 2951},
    /* 3165 : VCMPUNORD_QPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26122, 433, 2952},
    /* 3166 : VCMPUNORDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26104, 433, 2953},
    /* 3167 : VCMPUNORDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26122, 433, 2954},
    /* 3168 : VCMPNEQ_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26140, 433, 2955},
    /* 3169 : VCMPNEQ_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26158, 433, 2956},
    /* 3170 : VCMPNEQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26140, 433, 2957},
    /* 3171 : VCMPNEQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26158, 433, 2958},
    /* 3172 : VCMPNLT_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26176, 433, 2959},
    /* 3173 : VCMPNLT_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26194, 433, 2960},
    /* 3174 : VCMPNLTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26176, 433, 2961},
    /* 3175 : VCMPNLTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26194, 433, 2962},
    /* 3176 : VCMPNLE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26212, 433, 2963},
    /* 3177 : VCMPNLE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26230, 433, 2964},
    /* 3178 : VCMPNLEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26212, 433, 2965},
    /* 3179 : VCMPNLEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26230, 433, 2966},
    /* 3180 : VCMPORD_QPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26248, 433, 2967},
    /* 3181 : VCMPORD_QPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26266, 433, 2968},
    /* 3182 : VCMPORDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26248, 433, 2969},
    /* 3183 : VCMPORDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26266, 433, 2970},
    /* 3184 : VCMPEQ_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26284, 433, 2971},
    /* 3185 : VCMPEQ_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26302, 433, 2972},
    /* 3186 : VCMPNGE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26320, 433, 2973},
    /* 3187 : VCMPNGE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26338, 433, 2974},
    /* 3188 : VCMPNGEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26320, 433, 2975},
    /* 3189 : VCMPNGEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26338, 433, 2976},
    /* 3190 : VCMPNGT_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26356, 433, 2977},
    /* 3191 : VCMPNGT_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26374, 433, 2978},
    /* 3192 : VCMPNGTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26356, 433, 2979},
    /* 3193 : VCMPNGTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26374, 433, 2980},
    /* 3194 : VCMPFALSE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26392, 433, 2981},
    /* 3195 : VCMPFALSE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26410, 433, 2982},
    /* 3196 : VCMPFALSEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26392, 433, 2983},
    /* 3197 : VCMPFALSEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26410, 433, 2984},
    /* 3198 : VCMPNEQ_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26428, 433, 2985},
    /* 3199 : VCMPNEQ_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26446, 433, 2986},
    /* 3200 : VCMPGE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26464, 433, 2987},
    /* 3201 : VCMPGE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26482, 433, 2988},
    /* 3202 : VCMPGEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26464, 433, 2989},
    /* 3203 : VCMPGEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26482, 433, 2990},
    /* 3204 : VCMPGT_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26500, 433, 2991},
    /* 3205 : VCMPGT_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26518, 433, 2992},
    /* 3206 : VCMPGTPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26500, 433, 2993},
    /* 3207 : VCMPGTPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26518, 433, 2994},
    /* 3208 : VCMPTRUE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26536, 433, 2995},
    /* 3209 : VCMPTRUE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26554, 433, 2996},
    /* 3210 : VCMPTRUEPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26536, 433, 2997},
    /* 3211 : VCMPTRUEPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26554, 433, 2998},
    /* 3212 : VCMPEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+25960, 433, 2999},
    /* 3213 : VCMPEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+25978, 433, 3000},
    /* 3214 : VCMPLT_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26572, 433, 3001},
    /* 3215 : VCMPLT_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26590, 433, 3002},
    /* 3216 : VCMPLE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26608, 433, 3003},
    /* 3217 : VCMPLE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26626, 433, 3004},
    /* 3218 : VCMPUNORD_SPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26644, 433, 3005},
    /* 3219 : VCMPUNORD_SPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26662, 433, 3006},
    /* 3220 : VCMPNEQ_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26680, 433, 3007},
    /* 3221 : VCMPNEQ_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26698, 433, 3008},
    /* 3222 : VCMPNLT_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26716, 433, 3009},
    /* 3223 : VCMPNLT_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26734, 433, 3010},
    /* 3224 : VCMPNLE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26752, 433, 3011},
    /* 3225 : VCMPNLE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26770, 433, 3012},
    /* 3226 : VCMPORD_SPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26788, 433, 3013},
    /* 3227 : VCMPORD_SPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26806, 433, 3014},
    /* 3228 : VCMPEQ_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26824, 433, 3015},
    /* 3229 : VCMPEQ_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26842, 433, 3016},
    /* 3230 : VCMPNGE_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26860, 433, 3017},
    /* 3231 : VCMPNGE_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26878, 433, 3018},
    /* 3232 : VCMPNGT_UQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26896, 433, 3019},
    /* 3233 : VCMPNGT_UQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26914, 433, 3020},
    /* 3234 : VCMPFALSE_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26932, 433, 3021},
    /* 3235 : VCMPFALSE_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26950, 433, 3022},
    /* 3236 : VCMPNEQ_OSPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26968, 433, 3023},
    /* 3237 : VCMPNEQ_OSPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+26986, 433, 3024},
    /* 3238 : VCMPGE_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27004, 433, 3025},
    /* 3239 : VCMPGE_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27022, 433, 3026},
    /* 3240 : VCMPGT_OQPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27040, 433, 3027},
    /* 3241 : VCMPGT_OQPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27058, 433, 3028},
    /* 3242 : VCMPTRUE_USPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27076, 433, 3029},
    /* 3243 : VCMPTRUE_USPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27094, 433, 3030},
    /* 3244 : VCMPPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51875, 433, 3031},
    /* 3245 : VCMPPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51891, 433, 3032},
    /* 3246 : VCMPEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27112, 433, 3034},
    /* 3247 : VCMPEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27130, 433, 3035},
    /* 3248 : VCMPEQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 00           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27148, 433, 3036},
    /* 3249 : VCMPEQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 00           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27166, 433, 3037},
    /* 3250 : VCMPLT_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27184, 433, 3038},
    /* 3251 : VCMPLT_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27202, 433, 3039},
    /* 3252 : VCMPLTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27184, 433, 3040},
    /* 3253 : VCMPLTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 01           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27202, 433, 3041},
    /* 3254 : VCMPLE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27220, 433, 3042},
    /* 3255 : VCMPLE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27238, 433, 3043},
    /* 3256 : VCMPLEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27220, 433, 3044},
    /* 3257 : VCMPLEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 02           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(02) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27238, 433, 3045},
    /* 3258 : VCMPUNORD_QPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27256, 433, 3046},
    /* 3259 : VCMPUNORD_QPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27274, 433, 3047},
    /* 3260 : VCMPUNORDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27256, 433, 3048},
    /* 3261 : VCMPUNORDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 03           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(03) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27274, 433, 3049},
    /* 3262 : VCMPNEQ_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27292, 433, 3050},
    /* 3263 : VCMPNEQ_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27310, 433, 3051},
    /* 3264 : VCMPNEQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27292, 433, 3052},
    /* 3265 : VCMPNEQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 04           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(04) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27310, 433, 3053},
    /* 3266 : VCMPNLT_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27328, 433, 3054},
    /* 3267 : VCMPNLT_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27346, 433, 3055},
    /* 3268 : VCMPNLTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27328, 433, 3056},
    /* 3269 : VCMPNLTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 05           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(05) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27346, 433, 3057},
    /* 3270 : VCMPNLE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27364, 433, 3058},
    /* 3271 : VCMPNLE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27382, 433, 3059},
    /* 3272 : VCMPNLEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27364, 433, 3060},
    /* 3273 : VCMPNLEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 06           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(06) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27382, 433, 3061},
    /* 3274 : VCMPORD_QPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27400, 433, 3062},
    /* 3275 : VCMPORD_QPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27418, 433, 3063},
    /* 3276 : VCMPORDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27400, 433, 3064},
    /* 3277 : VCMPORDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 07           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(07) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27418, 433, 3065},
    /* 3278 : VCMPEQ_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 08           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27436, 433, 3066},
    /* 3279 : VCMPEQ_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 08           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(08) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27454, 433, 3067},
    /* 3280 : VCMPNGE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27472, 433, 3068},
    /* 3281 : VCMPNGE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27490, 433, 3069},
    /* 3282 : VCMPNGEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27472, 433, 3070},
    /* 3283 : VCMPNGEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 09           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(09) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27490, 433, 3071},
    /* 3284 : VCMPNGT_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27508, 433, 3072},
    /* 3285 : VCMPNGT_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27526, 433, 3073},
    /* 3286 : VCMPNGTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27508, 433, 3074},
    /* 3287 : VCMPNGTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27526, 433, 3075},
    /* 3288 : VCMPFALSE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27544, 433, 3076},
    /* 3289 : VCMPFALSE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27562, 433, 3077},
    /* 3290 : VCMPFALSEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27544, 433, 3078},
    /* 3291 : VCMPFALSEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27562, 433, 3079},
    /* 3292 : VCMPNEQ_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0c           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27580, 433, 3080},
    /* 3293 : VCMPNEQ_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0c           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27598, 433, 3081},
    /* 3294 : VCMPGE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27616, 433, 3082},
    /* 3295 : VCMPGE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27634, 433, 3083},
    /* 3296 : VCMPGEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27616, 433, 3084},
    /* 3297 : VCMPGEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27634, 433, 3085},
    /* 3298 : VCMPGT_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27652, 433, 3086},
    /* 3299 : VCMPGT_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27670, 433, 3087},
    /* 3300 : VCMPGTPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27652, 433, 3088},
    /* 3301 : VCMPGTPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27670, 433, 3089},
    /* 3302 : VCMPTRUE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27688, 433, 3090},
    /* 3303 : VCMPTRUE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27706, 433, 3091},
    /* 3304 : VCMPTRUEPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27688, 433, 3092},
    /* 3305 : VCMPTRUEPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 0f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(0f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUEPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27706, 433, 3093},
    /* 3306 : VCMPEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27112, 433, 3094},
    /* 3307 : VCMPEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 10           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27130, 433, 3095},
    /* 3308 : VCMPLT_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 11           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27724, 433, 3096},
    /* 3309 : VCMPLT_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 11           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27742, 433, 3097},
    /* 3310 : VCMPLE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 12           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27760, 433, 3098},
    /* 3311 : VCMPLE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 12           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(12) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27778, 433, 3099},
    /* 3312 : VCMPUNORD_SPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 13           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27796, 433, 3100},
    /* 3313 : VCMPUNORD_SPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 13           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(13) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27814, 433, 3101},
    /* 3314 : VCMPNEQ_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 14           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27832, 433, 3102},
    /* 3315 : VCMPNEQ_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 14           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(14) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27850, 433, 3103},
    /* 3316 : VCMPNLT_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 15           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27868, 433, 3104},
    /* 3317 : VCMPNLT_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 15           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(15) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27886, 433, 3105},
    /* 3318 : VCMPNLE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 16           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27904, 433, 3106},
    /* 3319 : VCMPNLE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 16           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(16) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27922, 433, 3107},
    /* 3320 : VCMPORD_SPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 17           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27940, 433, 3108},
    /* 3321 : VCMPORD_SPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 17           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(17) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27958, 433, 3109},
    /* 3322 : VCMPEQ_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 18           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27976, 433, 3110},
    /* 3323 : VCMPEQ_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 18           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(18) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+27994, 433, 3111},
    /* 3324 : VCMPNGE_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 19           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28012, 433, 3112},
    /* 3325 : VCMPNGE_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 19           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(19) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28030, 433, 3113},
    /* 3326 : VCMPNGT_UQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1a           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28048, 433, 3114},
    /* 3327 : VCMPNGT_UQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1a           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1a) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28066, 433, 3115},
    /* 3328 : VCMPFALSE_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1b           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28084, 433, 3116},
    /* 3329 : VCMPFALSE_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1b           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1b) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28102, 433, 3117},
    /* 3330 : VCMPNEQ_OSPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1c           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28120, 433, 3118},
    /* 3331 : VCMPNEQ_OSPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1c           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1c) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28138, 433, 3119},
    /* 3332 : VCMPGE_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1d           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28156, 433, 3120},
    /* 3333 : VCMPGE_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1d           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1d) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28174, 433, 3121},
    /* 3334 : VCMPGT_OQPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1e           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28192, 433, 3122},
    /* 3335 : VCMPGT_OQPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1e           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1e) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28210, 433, 3123},
    /* 3336 : VCMPTRUE_USPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f c2 /r 1f           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28228, 433, 3124},
    /* 3337 : VCMPTRUE_USPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f c2 /r 1f           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 001(1f) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28246, 433, 3125},
    /* 3338 : VCMPPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.0f c2 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51907, 433, 3126},
    /* 3339 : VCMPPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.0f c2 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c2) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51923, 433, 3127},
    /* 3340 : VCMPEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28264, 440, 3129},
    /* 3341 : VCMPEQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(00) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28282, 440, 3130},
    /* 3342 : VCMPLT_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28300, 440, 3131},
    /* 3343 : VCMPLTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28300, 440, 3132},
    /* 3344 : VCMPLE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28318, 440, 3133},
    /* 3345 : VCMPLESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28318, 440, 3134},
    /* 3346 : VCMPUNORD_QSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28336, 440, 3135},
    /* 3347 : VCMPUNORDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28336, 440, 3136},
    /* 3348 : VCMPNEQ_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28354, 440, 3137},
    /* 3349 : VCMPNEQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28354, 440, 3138},
    /* 3350 : VCMPNLT_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28372, 440, 3139},
    /* 3351 : VCMPNLTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28372, 440, 3140},
    /* 3352 : VCMPNLE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28390, 440, 3141},
    /* 3353 : VCMPNLESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28390, 440, 3142},
    /* 3354 : VCMPORD_QSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28408, 440, 3143},
    /* 3355 : VCMPORDSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28408, 440, 3144},
    /* 3356 : VCMPEQ_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(08) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28426, 440, 3145},
    /* 3357 : VCMPNGE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28444, 440, 3146},
    /* 3358 : VCMPNGESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28444, 440, 3147},
    /* 3359 : VCMPNGT_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28462, 440, 3148},
    /* 3360 : VCMPNGTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28462, 440, 3149},
    /* 3361 : VCMPFALSE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28480, 440, 3150},
    /* 3362 : VCMPFALSESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28480, 440, 3151},
    /* 3363 : VCMPNEQ_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28498, 440, 3152},
    /* 3364 : VCMPGE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28516, 440, 3153},
    /* 3365 : VCMPGESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28516, 440, 3154},
    /* 3366 : VCMPGT_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28534, 440, 3155},
    /* 3367 : VCMPGTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28534, 440, 3156},
    /* 3368 : VCMPTRUE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28552, 440, 3157},
    /* 3369 : VCMPTRUESD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUESD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28552, 440, 3158},
    /* 3370 : VCMPEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28264, 440, 3159},
    /* 3371 : VCMPLT_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(11) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28570, 440, 3160},
    /* 3372 : VCMPLE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(12) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28588, 440, 3161},
    /* 3373 : VCMPUNORD_SSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(13) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28606, 440, 3162},
    /* 3374 : VCMPNEQ_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(14) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28624, 440, 3163},
    /* 3375 : VCMPNLT_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(15) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28642, 440, 3164},
    /* 3376 : VCMPNLE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(16) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28660, 440, 3165},
    /* 3377 : VCMPORD_SSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(17) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28678, 440, 3166},
    /* 3378 : VCMPEQ_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(18) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28696, 440, 3167},
    /* 3379 : VCMPNGE_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(19) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28714, 440, 3168},
    /* 3380 : VCMPNGT_UQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28732, 440, 3169},
    /* 3381 : VCMPFALSE_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28750, 440, 3170},
    /* 3382 : VCMPNEQ_OSSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28768, 440, 3171},
    /* 3383 : VCMPGE_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28786, 440, 3172},
    /* 3384 : VCMPGT_OQSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28804, 440, 3173},
    /* 3385 : VCMPTRUE_USSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 001(1f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28822, 440, 3174},
    /* 3386 : VCMPSD xmmreg,xmmreg*,xmmrm64,imm8 [rvmi:     vex.nds.lig.f2.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(c2) 120 023 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51939, 440, 3175},
    /* 3387 : VCMPEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28840, 440, 3177},
    /* 3388 : VCMPEQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 00        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(00) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28858, 440, 3178},
    /* 3389 : VCMPLT_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28876, 440, 3179},
    /* 3390 : VCMPLTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 01        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(01) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28876, 440, 3180},
    /* 3391 : VCMPLE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28894, 440, 3181},
    /* 3392 : VCMPLESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 02        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(02) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28894, 440, 3182},
    /* 3393 : VCMPUNORD_QSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_QSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28912, 440, 3183},
    /* 3394 : VCMPUNORDSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 03        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(03) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28912, 440, 3184},
    /* 3395 : VCMPNEQ_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28930, 440, 3185},
    /* 3396 : VCMPNEQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 04        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(04) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28930, 440, 3186},
    /* 3397 : VCMPNLT_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28948, 440, 3187},
    /* 3398 : VCMPNLTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 05        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(05) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28948, 440, 3188},
    /* 3399 : VCMPNLE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28966, 440, 3189},
    /* 3400 : VCMPNLESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 06        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(06) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28966, 440, 3190},
    /* 3401 : VCMPORD_QSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_QSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28984, 440, 3191},
    /* 3402 : VCMPORDSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 07        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(07) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORDSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28984, 440, 3192},
    /* 3403 : VCMPEQ_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 08        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(08) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29002, 440, 3193},
    /* 3404 : VCMPNGE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29020, 440, 3194},
    /* 3405 : VCMPNGESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 09        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(09) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29020, 440, 3195},
    /* 3406 : VCMPNGT_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29038, 440, 3196},
    /* 3407 : VCMPNGTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29038, 440, 3197},
    /* 3408 : VCMPFALSE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29056, 440, 3198},
    /* 3409 : VCMPFALSESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29056, 440, 3199},
    /* 3410 : VCMPNEQ_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0c        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29074, 440, 3200},
    /* 3411 : VCMPGE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29092, 440, 3201},
    /* 3412 : VCMPGESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29092, 440, 3202},
    /* 3413 : VCMPGT_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29110, 440, 3203},
    /* 3414 : VCMPGTSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29110, 440, 3204},
    /* 3415 : VCMPTRUE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29128, 440, 3205},
    /* 3416 : VCMPTRUESS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 0f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(0f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUESS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29128, 440, 3206},
    /* 3417 : VCMPEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 10        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(10) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+28840, 440, 3207},
    /* 3418 : VCMPLT_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 11        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(11) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLT_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29146, 440, 3208},
    /* 3419 : VCMPLE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 12        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(12) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPLE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29164, 440, 3209},
    /* 3420 : VCMPUNORD_SSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 13        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(13) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPUNORD_SSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29182, 440, 3210},
    /* 3421 : VCMPNEQ_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 14        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(14) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29200, 440, 3211},
    /* 3422 : VCMPNLT_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 15        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(15) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLT_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29218, 440, 3212},
    /* 3423 : VCMPNLE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 16        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(16) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNLE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29236, 440, 3213},
    /* 3424 : VCMPORD_SSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 17        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(17) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPORD_SSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29254, 440, 3214},
    /* 3425 : VCMPEQ_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 18        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(18) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPEQ_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29272, 440, 3215},
    /* 3426 : VCMPNGE_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 19        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(19) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGE_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29290, 440, 3216},
    /* 3427 : VCMPNGT_UQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1a        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1a) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNGT_UQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29308, 440, 3217},
    /* 3428 : VCMPFALSE_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1b        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1b) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPFALSE_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29326, 440, 3218},
    /* 3429 : VCMPNEQ_OSSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1c        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1c) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPNEQ_OSSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29344, 440, 3219},
    /* 3430 : VCMPGE_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1d        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1d) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGE_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29362, 440, 3220},
    /* 3431 : VCMPGT_OQSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1e        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1e) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPGT_OQSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29380, 440, 3221},
    /* 3432 : VCMPTRUE_USSS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f3.0f c2 /r 1f        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 001(1f) : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPTRUE_USSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29398, 440, 3222},
    /* 3433 : VCMPSS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:     vex.nds.lig.f3.0f c2 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(c2) 120 023 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCMPSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51955, 440, 3223},
    /* 3434 : VCOMISD xmmreg,xmmrm64 [rm:       vex.lig.66.0f 2f /r               ] FL,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2f) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VCOMISD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58792, 442, 3224},
    /* 3435 : VCOMISS xmmreg,xmmrm32 [rm:       vex.lig.0f 2f /r                  ] FL,AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2f) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VCOMISS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58799, 442, 3225},
    /* 3436 : VCVTDQ2PD xmmreg,xmmrm64 [rm:       vex.128.f3.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58806, 433, 3226},
    /* 3437 : VCVTDQ2PD ymmreg,xmmrm128 [rm:       vex.256.f3.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58813, 433, 3227},
    /* 3438 : VCVTDQ2PS xmmreg,xmmrm128 [rm:       vex.128.0f 5b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58820, 433, 3228},
    /* 3439 : VCVTDQ2PS ymmreg,ymmrm256 [rm:       vex.256.0f 5b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTDQ2PS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58827, 433, 3229},
    /* 3440 : VCVTPD2DQ xmmreg,xmmreg [rm:       vex.128.f2.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58834, 433, 3230},
    /* 3441 : VCVTPD2DQ xmmreg,mem128 [rm:       vex.128.f2.0f e6 /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 003) 001(e6) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58834, 443, 3231},
    /* 3442 : VCVTPD2DQ xmmreg,ymmreg [rm:       vex.256.f2.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58841, 433, 3232},
    /* 3443 : VCVTPD2DQ xmmreg,mem256 [rm:       vex.256.f2.0f e6 /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 007) 001(e6) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58841, 444, 3233},
    /* 3444 : VCVTPD2PS xmmreg,xmmreg [rm:       vex.128.66.0f 5a /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58848, 433, 3234},
    /* 3445 : VCVTPD2PS xmmreg,mem128 [rm:       vex.128.66.0f 5a /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5a) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58848, 443, 3235},
    /* 3446 : VCVTPD2PS xmmreg,ymmreg [rm:       vex.256.66.0f 5a /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58855, 433, 3236},
    /* 3447 : VCVTPD2PS xmmreg,mem256 [rm:       vex.256.66.0f 5a /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5a) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPD2PS, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58855, 444, 3237},
    /* 3448 : VCVTPS2DQ xmmreg,xmmrm128 [rm:       vex.128.66.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2DQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58862, 433, 3238},
    /* 3449 : VCVTPS2DQ ymmreg,ymmrm256 [rm:       vex.256.66.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2DQ, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58869, 433, 3239},
    /* 3450 : VCVTPS2PD xmmreg,xmmrm64 [rm:       vex.128.0f 5a /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2PD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58876, 433, 3240},
    /* 3451 : VCVTPS2PD ymmreg,xmmrm128 [rm:       vex.256.0f 5a /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(5a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTPS2PD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58883, 433, 3241},
    /* 3452 : VCVTSD2SI reg32,xmmrm64 [rm:       vex.lig.f2.0f.w0 2d /r            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(2d) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58890, 445, 3242},
    /* 3453 : VCVTSD2SI reg64,xmmrm64 [rm:       vex.lig.f2.0f.w1 2d /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 203) 001(2d) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58897, 446, 3243},
    /* 3454 : VCVTSD2SS xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5a) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSD2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58904, 440, 3244},
    /* 3455 : VCVTSI2SD xmmreg,xmmreg*,rm32 [rvm:      vex.nds.lig.f2.0f.w0 2a /r        ] AR0,AR1,AR2,SD,AVX,SANDYBRIDGE */
        /* 261(001 003) 001(2a) 120 : AR0,AR1,AR2,SD,NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSI2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58918, 447, 3245},
    /* 3456 : VCVTSI2SD xmmreg,xmmreg*,rm64 [rvm:      vex.nds.lig.f2.0f.w1 2a /r        ] AR0,AR1,AR2,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(001 203) 001(2a) 120 : AR0,AR1,AR2,SQ,NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSI2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58932, 449, 3247},
    /* 3457 : VCVTSI2SS xmmreg,xmmreg*,rm32 [rvm:      vex.nds.lig.f3.0f.w0 2a /r        ] AR0,AR1,AR2,SD,AVX,SANDYBRIDGE */
        /* 261(001 002) 001(2a) 120 : AR0,AR1,AR2,SD,NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSI2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58946, 447, 3248},
    /* 3458 : VCVTSI2SS xmmreg,xmmreg*,rm64 [rvm:      vex.nds.lig.f3.0f.w1 2a /r        ] AR0,AR1,AR2,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(001 202) 001(2a) 120 : AR0,AR1,AR2,SQ,NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSI2SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58960, 449, 3250},
    /* 3459 : VCVTSS2SD xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5a) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSS2SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+58974, 440, 3251},
    /* 3460 : VCVTSS2SI reg32,xmmrm32 [rm:       vex.lig.f3.0f.w0 2d /r            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(2d) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58988, 445, 3252},
    /* 3461 : VCVTSS2SI reg64,xmmrm32 [rm:       vex.lig.f3.0f.w1 2d /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 202) 001(2d) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58995, 446, 3253},
    /* 3462 : VCVTTPD2DQ xmmreg,xmmreg [rm:       vex.128.66.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59002, 433, 3254},
    /* 3463 : VCVTTPD2DQ xmmreg,mem128 [rm:       vex.128.66.0f e6 /r               ] AR0-1,SO,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e6) 110 : AR0,AR1,SO,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59002, 443, 3255},
    /* 3464 : VCVTTPD2DQ xmmreg,ymmreg [rm:       vex.256.66.0f e6 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e6) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59009, 433, 3256},
    /* 3465 : VCVTTPD2DQ xmmreg,mem256 [rm:       vex.256.66.0f e6 /r               ] AR0-1,SY,AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e6) 110 : AR0,AR1,SY,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPD2DQ, 2, {XMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59009, 444, 3257},
    /* 3466 : VCVTTPS2DQ xmmreg,xmmrm128 [rm:       vex.128.f3.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPS2DQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59016, 433, 3258},
    /* 3467 : VCVTTPS2DQ ymmreg,ymmrm256 [rm:       vex.256.f3.0f 5b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(5b) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTPS2DQ, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59023, 433, 3259},
    /* 3468 : VCVTTSD2SI reg32,xmmrm64 [rm:       vex.lig.f2.0f.w0 2c /r            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(2c) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59030, 445, 3260},
    /* 3469 : VCVTTSD2SI reg64,xmmrm64 [rm:       vex.lig.f2.0f.w1 2c /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 203) 001(2c) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59037, 446, 3261},
    /* 3470 : VCVTTSS2SI reg32,xmmrm32 [rm:       vex.lig.f3.0f.w0 2c /r            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(2c) 110 : NOAPX,LIG,VEX,AVX,SANDYBRIDGE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS32|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59044, 445, 3262},
    /* 3471 : VCVTTSS2SI reg64,xmmrm32 [rm:       vex.lig.f3.0f.w1 2c /r            ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 202) 001(2c) 110 : NOAPX,LIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS64|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59051, 446, 3263},
    /* 3472 : VDIVPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59058, 433, 3264},
    /* 3473 : VDIVPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59072, 433, 3265},
    /* 3474 : VDIVPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5e /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59086, 433, 3266},
    /* 3475 : VDIVPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5e /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59100, 433, 3267},
    /* 3476 : VDIVSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5e) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59114, 440, 3268},
    /* 3477 : VDIVSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5e /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5e) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDIVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59128, 440, 3269},
    /* 3478 : VDPPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 41 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(41) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51971, 433, 3270},
    /* 3479 : VDPPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 40 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(40) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+51987, 433, 3271},
    /* 3480 : VDPPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 40 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(40) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VDPPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52003, 433, 3272},
    /* 3481 : VEXTRACTF128 xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 19 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(19) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VEXTRACTF128, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52019, 441, 3273},
    /* 3482 : VEXTRACTPS rm32,xmmreg,imm8 [mri:      vex.128.66.0f3a 17 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(17) 101 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VEXTRACTPS, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52027, 433, 3274},
    /* 3483 : VHADDPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59142, 433, 3275},
    /* 3484 : VHADDPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59156, 433, 3276},
    /* 3485 : VHADDPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59170, 433, 3277},
    /* 3486 : VHADDPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f 7c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(7c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHADDPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59184, 433, 3278},
    /* 3487 : VHSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59198, 433, 3279},
    /* 3488 : VHSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59212, 433, 3280},
    /* 3489 : VHSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.f2.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59226, 433, 3281},
    /* 3490 : VHSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.f2.0f 7d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 007) 001(7d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VHSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59240, 433, 3282},
    /* 3491 : VINSERTF128 ymmreg,ymmreg*,xmmrm128,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 18 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(18) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VINSERTF128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52035, 441, 3283},
    /* 3492 : VINSERTPS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:     vex.nds.128.66.0f3a 21 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(21) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VINSERTPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52051, 433, 3284},
    /* 3493 : VLDDQU xmmreg,mem128 [rm:       vex.128.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDDQU, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59254, 433, 3285},
    /* 3494 : VLDQQU ymmreg,mem256 [rm:       vex.256.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDQQU, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59261, 433, 3286},
    /* 3495 : VLDDQU ymmreg,mem256 [rm:       vex.256.f2.0f f0 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(f0) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDDQU, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59261, 433, 3287},
    /* 3496 : VLDMXCSR mem32 [m:        vex.lz.0f ae /2                   ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(ae) 202 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VLDMXCSR, 1, {MEMORY|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+59268, 433, 3288},
    /* 3497 : VMASKMOVDQU xmmreg,xmmreg [rm:       vex.128.66.0f f7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(f7) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVDQU, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59275, 433, 3289},
    /* 3498 : VMASKMOVPS xmmreg,xmmreg,mem128 [rvm:      vex.nds.128.66.0f38.w0 2c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59282, 441, 3290},
    /* 3499 : VMASKMOVPS ymmreg,ymmreg,mem256 [rvm:      vex.nds.256.66.0f38.w0 2c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59289, 441, 3291},
    /* 3500 : VMASKMOVPS mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 2e /r      ] AR0-2,SO,AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2e) 102 : AR0,AR1,AR2,SO,NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59296, 451, 3292},
    /* 3501 : VMASKMOVPS mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 2e /r      ] AR0-2,SY,AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2e) 102 : AR0,AR1,AR2,SY,NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPS, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59303, 452, 3293},
    /* 3502 : VMASKMOVPD xmmreg,xmmreg,mem128 [rvm:      vex.nds.128.66.0f38.w0 2d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59310, 441, 3294},
    /* 3503 : VMASKMOVPD ymmreg,ymmreg,mem256 [rvm:      vex.nds.256.66.0f38.w0 2d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59317, 441, 3295},
    /* 3504 : VMASKMOVPD mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 2f /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2f) 102 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59324, 441, 3296},
    /* 3505 : VMASKMOVPD mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 2f /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(2f) 102 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMASKMOVPD, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59331, 441, 3297},
    /* 3506 : VMAXPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59338, 433, 3298},
    /* 3507 : VMAXPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59352, 433, 3299},
    /* 3508 : VMAXPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5f /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59366, 433, 3300},
    /* 3509 : VMAXPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5f /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59380, 433, 3301},
    /* 3510 : VMAXSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5f) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59394, 440, 3302},
    /* 3511 : VMAXSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5f /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5f) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMAXSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59408, 440, 3303},
    /* 3512 : VMINPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59422, 433, 3304},
    /* 3513 : VMINPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59436, 433, 3305},
    /* 3514 : VMINPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5d /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59450, 433, 3306},
    /* 3515 : VMINPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5d /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59464, 433, 3307},
    /* 3516 : VMINSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5d) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59478, 440, 3308},
    /* 3517 : VMINSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5d) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMINSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59492, 440, 3309},
    /* 3518 : VMOVAPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 28 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59506, 433, 3310},
    /* 3519 : VMOVAPD xmmrm128,xmmreg [mr:       vex.128.66.0f 29 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59513, 433, 3311},
    /* 3520 : VMOVAPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 28 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59520, 433, 3312},
    /* 3521 : VMOVAPD ymmrm256,ymmreg [mr:       vex.256.66.0f 29 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPD, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59527, 433, 3313},
    /* 3522 : VMOVAPS xmmreg,xmmrm128 [rm:       vex.128.0f 28 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59534, 433, 3314},
    /* 3523 : VMOVAPS xmmrm128,xmmreg [mr:       vex.128.0f 29 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59541, 433, 3315},
    /* 3524 : VMOVAPS ymmreg,ymmrm256 [rm:       vex.256.0f 28 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(28) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59548, 433, 3316},
    /* 3525 : VMOVAPS ymmrm256,ymmreg [mr:       vex.256.0f 29 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(29) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVAPS, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59555, 433, 3317},
    /* 3526 : VMOVD xmmreg,rm32 [rm:       vex.128.66.0f.w0 6e /r            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(6e) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMOVD, 2, {XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59562, 441, 3318},
    /* 3527 : VMOVD rm32,xmmreg [mr:       vex.128.66.0f.w0 7e /r            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(7e) 101 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VMOVD, 2, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59569, 441, 3319},
    /* 3528 : VMOVQ xmmreg,xmmrm64 [rm:       vex.128.f3.0f 7e /r               ] AR0-1,SQ,AVX,SANDYBRIDGE */
        /* 270(001 002) 001(7e) 110 : AR0,AR1,SQ,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59576, 453, 3320},
    /* 3529 : VMOVQ xmmrm64,xmmreg [mr:       vex.128.66.0f d6 /r               ] AR0-1,SQ,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(d6) 101 : AR0,AR1,SQ,NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59583, 453, 3321},
    /* 3530 : VMOVQ xmmreg,rm64 [rm:       vex.128.66.0f.w1 6e /r            ] AR0-1,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 201) 001(6e) 110 : AR0,AR1,SQ,NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59590, 454, 3322},
    /* 3531 : VMOVQ rm64,xmmreg [mr:       vex.128.66.0f.w1 7e /r            ] AR0-1,SQ,LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 201) 001(7e) 101 : AR0,AR1,SQ,NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVQ, 2, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59597, 454, 3323},
    /* 3532 : VMOVDDUP xmmreg,xmmrm64 [rm:       vex.128.f2.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59604, 433, 3324},
    /* 3533 : VMOVDDUP ymmreg,ymmrm256 [rm:       vex.256.f2.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 007) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59611, 433, 3325},
    /* 3534 : VMOVDQA xmmreg,xmmrm128 [rm:       vex.128.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59618, 433, 3326},
    /* 3535 : VMOVDQA xmmrm128,xmmreg [mr:       vex.128.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59625, 433, 3327},
    /* 3536 : VMOVQQA ymmreg,ymmrm256 [rm:       vex.256.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQA, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59632, 433, 3329},
    /* 3537 : VMOVQQA ymmrm256,ymmreg [mr:       vex.256.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQA, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59639, 433, 3330},
    /* 3538 : VMOVDQA ymmreg,ymmrm256 [rm:       vex.256.66.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59632, 433, 3331},
    /* 3539 : VMOVDQA ymmrm256,ymmreg [mr:       vex.256.66.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQA, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59639, 433, 3332},
    /* 3540 : VMOVDQU xmmreg,xmmrm128 [rm:       vex.128.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59646, 433, 3333},
    /* 3541 : VMOVDQU xmmrm128,xmmreg [mr:       vex.128.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59653, 433, 3334},
    /* 3542 : VMOVQQU ymmreg,ymmrm256 [rm:       vex.256.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQU, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59660, 433, 3336},
    /* 3543 : VMOVQQU ymmrm256,ymmreg [mr:       vex.256.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVQQU, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59667, 433, 3337},
    /* 3544 : VMOVDQU ymmreg,ymmrm256 [rm:       vex.256.f3.0f 6f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(6f) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59660, 433, 3338},
    /* 3545 : VMOVDQU ymmrm256,ymmreg [mr:       vex.256.f3.0f 7f /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(7f) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVDQU, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59667, 433, 3339},
    /* 3546 : VMOVHLPS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.128.0f 12 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59674, 433, 3340},
    /* 3547 : VMOVHPD xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.66.0f 16 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59688, 433, 3341},
    /* 3548 : VMOVHPD mem64,xmmreg [mr:       vex.128.66.0f 17 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(17) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59702, 433, 3342},
    /* 3549 : VMOVHPS xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.0f 16 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59709, 433, 3343},
    /* 3550 : VMOVHPS mem64,xmmreg [mr:       vex.128.0f 17 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(17) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVHPS, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59723, 433, 3344},
    /* 3551 : VMOVLHPS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.128.0f 16 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(16) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59709, 433, 3345},
    /* 3552 : VMOVLPD xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.66.0f 12 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59730, 433, 3346},
    /* 3553 : VMOVLPD mem64,xmmreg [mr:       vex.128.66.0f 13 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(13) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59744, 433, 3347},
    /* 3554 : VMOVLPS xmmreg,xmmreg*,mem64 [rvm:      vex.nds.128.0f 12 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(12) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59674, 433, 3348},
    /* 3555 : VMOVLPS mem64,xmmreg [mr:       vex.128.0f 13 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(13) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVLPS, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59751, 433, 3349},
    /* 3556 : VMOVMSKPD reg64,xmmreg [rm:       vex.128.66.0f 50 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59758, 455, 3350},
    /* 3557 : VMOVMSKPD reg32,xmmreg [rm:       vex.128.66.0f 50 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59758, 433, 3351},
    /* 3558 : VMOVMSKPD reg64,ymmreg [rm:       vex.256.66.0f 50 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 005) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59765, 455, 3352},
    /* 3559 : VMOVMSKPD reg32,ymmreg [rm:       vex.256.66.0f 50 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPD, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59765, 433, 3353},
    /* 3560 : VMOVMSKPS reg64,xmmreg [rm:       vex.128.0f 50 /r                  ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 000) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59772, 455, 3354},
    /* 3561 : VMOVMSKPS reg32,xmmreg [rm:       vex.128.0f 50 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59772, 433, 3355},
    /* 3562 : VMOVMSKPS reg64,ymmreg [rm:       vex.256.0f 50 /r                  ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 004) 001(50) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59779, 455, 3356},
    /* 3563 : VMOVMSKPS reg32,ymmreg [rm:       vex.256.0f 50 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(50) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVMSKPS, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59779, 433, 3357},
    /* 3564 : VMOVNTDQ mem128,xmmreg [mr:       vex.128.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59786, 433, 3358},
    /* 3565 : VMOVNTQQ mem256,ymmreg [mr:       vex.256.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTQQ, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59793, 433, 3360},
    /* 3566 : VMOVNTDQ mem256,ymmreg [mr:       vex.256.66.0f e7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(e7) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59793, 433, 3361},
    /* 3567 : VMOVNTDQA xmmreg,mem128 [rm:       vex.128.66.0f38 2a /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(2a) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTDQA, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59800, 433, 3362},
    /* 3568 : VMOVNTPD mem128,xmmreg [mr:       vex.128.66.0f 2b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPD, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59807, 433, 3363},
    /* 3569 : VMOVNTPD mem256,ymmreg [mr:       vex.256.66.0f 2b /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPD, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59814, 433, 3364},
    /* 3570 : VMOVNTPS mem128,xmmreg [mr:       vex.128.0f 2b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPS, 2, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59821, 433, 3365},
    /* 3571 : VMOVNTPS mem256,ymmreg [mr:       vex.256.0f 2b /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(2b) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVNTPS, 2, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59828, 433, 3366},
    /* 3572 : VMOVSD xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.lig.f2.0f 10 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(10) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59835, 440, 3367},
    /* 3573 : VMOVSD xmmreg,mem64 [rm:       vex.lig.f2.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(10) 110 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 2, {XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59849, 440, 3368},
    /* 3574 : VMOVSD xmmreg,xmmreg*,xmmreg [mvr:      vex.nds.lig.f2.0f 11 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(11) 102 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59856, 440, 3369},
    /* 3575 : VMOVSD mem64,xmmreg [mr:       vex.lig.f2.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(11) 101 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSD, 2, {MEMORY|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59870, 440, 3370},
    /* 3576 : VMOVSHDUP xmmreg,xmmrm128 [rm:       vex.128.f3.0f 16 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(16) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSHDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59877, 433, 3371},
    /* 3577 : VMOVSHDUP ymmreg,ymmrm256 [rm:       vex.256.f3.0f 16 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(16) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSHDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59884, 433, 3372},
    /* 3578 : VMOVSLDUP xmmreg,xmmrm128 [rm:       vex.128.f3.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSLDUP, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59891, 433, 3373},
    /* 3579 : VMOVSLDUP ymmreg,ymmrm256 [rm:       vex.256.f3.0f 12 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 006) 001(12) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSLDUP, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59898, 433, 3374},
    /* 3580 : VMOVSS xmmreg,xmmreg*,xmmreg [rvm:      vex.nds.lig.f3.0f 10 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(10) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59905, 440, 3375},
    /* 3581 : VMOVSS xmmreg,mem32 [rm:       vex.lig.f3.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(10) 110 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59919, 440, 3376},
    /* 3582 : VMOVSS xmmreg,xmmreg*,xmmreg [mvr:      vex.nds.lig.f3.0f 11 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(11) 102 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+59926, 440, 3377},
    /* 3583 : VMOVSS mem32,xmmreg [mr:       vex.lig.f3.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(11) 101 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVSS, 2, {MEMORY|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59940, 440, 3378},
    /* 3584 : VMOVUPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59947, 433, 3379},
    /* 3585 : VMOVUPD xmmrm128,xmmreg [mr:       vex.128.66.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59954, 433, 3380},
    /* 3586 : VMOVUPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 10 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59961, 433, 3381},
    /* 3587 : VMOVUPD ymmrm256,ymmreg [mr:       vex.256.66.0f 11 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPD, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59968, 433, 3382},
    /* 3588 : VMOVUPS xmmreg,xmmrm128 [rm:       vex.128.0f 10 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59975, 433, 3383},
    /* 3589 : VMOVUPS xmmrm128,xmmreg [mr:       vex.128.0f 11 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59982, 433, 3384},
    /* 3590 : VMOVUPS ymmreg,ymmrm256 [rm:       vex.256.0f 10 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(10) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59989, 433, 3385},
    /* 3591 : VMOVUPS ymmrm256,ymmreg [mr:       vex.256.0f 11 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(11) 101 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMOVUPS, 2, {RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+59996, 433, 3386},
    /* 3592 : VMPSADBW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 42 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(42) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMPSADBW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52067, 433, 3387},
    /* 3593 : VMULPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60003, 433, 3388},
    /* 3594 : VMULPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60017, 433, 3389},
    /* 3595 : VMULPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 59 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60031, 433, 3390},
    /* 3596 : VMULPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 59 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(59) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60045, 433, 3391},
    /* 3597 : VMULSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(59) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60059, 440, 3392},
    /* 3598 : VMULSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 59 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(59) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VMULSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60073, 440, 3393},
    /* 3599 : VORPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 56 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60087, 433, 3394},
    /* 3600 : VORPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 56 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60101, 433, 3395},
    /* 3601 : VORPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 56 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60115, 433, 3396},
    /* 3602 : VORPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 56 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(56) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VORPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60129, 433, 3397},
    /* 3603 : VPABSB xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1c /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1c) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60143, 433, 3398},
    /* 3604 : VPABSW xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1d /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1d) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60150, 433, 3399},
    /* 3605 : VPABSD xmmreg,xmmrm128 [rm:       vex.128.66.0f38 1e /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(1e) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPABSD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60157, 433, 3400},
    /* 3606 : VPACKSSWB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 63 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(63) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKSSWB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60164, 433, 3401},
    /* 3607 : VPACKSSDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6b /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKSSDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60178, 433, 3402},
    /* 3608 : VPACKUSWB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 67 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(67) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKUSWB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60192, 433, 3403},
    /* 3609 : VPACKUSDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 2b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(2b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPACKUSDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60206, 433, 3404},
    /* 3610 : VPADDB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fc /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60220, 433, 3405},
    /* 3611 : VPADDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fd /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60234, 433, 3406},
    /* 3612 : VPADDD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fe /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fe) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60248, 433, 3407},
    /* 3613 : VPADDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60262, 433, 3408},
    /* 3614 : VPADDSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ec /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ec) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60276, 433, 3409},
    /* 3615 : VPADDSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ed /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ed) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60290, 433, 3410},
    /* 3616 : VPADDUSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f dc /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(dc) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDUSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60304, 433, 3411},
    /* 3617 : VPADDUSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f dd /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(dd) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPADDUSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60318, 433, 3412},
    /* 3618 : VPALIGNR xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0f /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0f) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPALIGNR, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52083, 433, 3413},
    /* 3619 : VPAND xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f db /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(db) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAND, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60332, 433, 3414},
    /* 3620 : VPANDN xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f df /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(df) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPANDN, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60346, 433, 3415},
    /* 3621 : VPAVGB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e0 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e0) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAVGB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60360, 433, 3416},
    /* 3622 : VPAVGW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPAVGW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60374, 433, 3417},
    /* 3623 : VPBLENDVB xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.nds.128.66.0f3a.w0 4c /r /is4 ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(4c) 120 177 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPBLENDVB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52099, 441, 3418},
    /* 3624 : VPBLENDW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 0e /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0e) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPBLENDW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52115, 433, 3419},
    /* 3625 : VPCMPESTRI xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 61 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(61) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPESTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52131, 433, 3420},
    /* 3626 : VPCMPESTRM xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 60 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(60) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPESTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52139, 433, 3421},
    /* 3627 : VPCMPISTRI xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 63 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(63) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPISTRI, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52147, 433, 3422},
    /* 3628 : VPCMPISTRM xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 62 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(62) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPISTRM, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52155, 433, 3423},
    /* 3629 : VPCMPEQB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 74 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(74) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60388, 433, 3424},
    /* 3630 : VPCMPEQW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 75 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(75) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60402, 433, 3425},
    /* 3631 : VPCMPEQD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 76 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(76) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60416, 433, 3426},
    /* 3632 : VPCMPEQQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 29 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(29) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPEQQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60430, 433, 3427},
    /* 3633 : VPCMPGTB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 64 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(64) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60444, 433, 3428},
    /* 3634 : VPCMPGTW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 65 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(65) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60458, 433, 3429},
    /* 3635 : VPCMPGTD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 66 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(66) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60472, 433, 3430},
    /* 3636 : VPCMPGTQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 37 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(37) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCMPGTQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60486, 433, 3431},
    /* 3637 : VPERMILPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 0d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60500, 441, 3432},
    /* 3638 : VPERMILPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 0d /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(0d) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60514, 441, 3433},
    /* 3639 : VPERMILPD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a.w0 05 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(05) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52163, 441, 3434},
    /* 3640 : VPERMILPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w0 05 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(05) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52171, 441, 3435},
    /* 3641 : VPERMILPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 0c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60528, 441, 3436},
    /* 3642 : VPERMILPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 0c /r      ] AVX,SANDYBRIDGE */
        /* 261(002 005) 001(0c) 120 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60542, 441, 3437},
    /* 3643 : VPERMILPS xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a.w0 04 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(04) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52179, 441, 3438},
    /* 3644 : VPERMILPS ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w0 04 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(04) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERMILPS, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52187, 441, 3439},
    /* 3645 : VPERM2F128 ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 06 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 005) 001(06) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPERM2F128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52195, 441, 3440},
    /* 3646 : VPEXTRB reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52211, 456, 3441},
    /* 3647 : VPEXTRB reg32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52211, 441, 3442},
    /* 3648 : VPEXTRB mem8,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 14 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(14) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRB, 3, {MEMORY|BITS8|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52211, 441, 3443},
    /* 3649 : VPEXTRW reg64,xmmreg,imm8 [rmi:      vex.128.66.0f.w0 c5 /r ib         ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(c5) 110 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52219, 456, 3444},
    /* 3650 : VPEXTRW reg32,xmmreg,imm8 [rmi:      vex.128.66.0f.w0 c5 /r ib         ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(c5) 110 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52219, 441, 3445},
    /* 3651 : VPEXTRW reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52227, 456, 3446},
    /* 3652 : VPEXTRW reg32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52227, 441, 3447},
    /* 3653 : VPEXTRW mem16,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 15 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(15) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRW, 3, {MEMORY|BITS16|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52227, 441, 3448},
    /* 3654 : VPEXTRD reg64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 16 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 001) 001(16) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRD, 3, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52235, 456, 3449},
    /* 3655 : VPEXTRD rm32,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 16 /r ib       ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(16) 101 022 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPEXTRD, 3, {RM_GPR|BITS32|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52235, 441, 3450},
    /* 3656 : VPEXTRQ rm64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w1 16 /r ib       ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(003 201) 001(16) 101 022 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPEXTRQ, 3, {RM_GPR|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52243, 456, 3451},
    /* 3657 : VPHADDW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 01 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(01) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60556, 433, 3452},
    /* 3658 : VPHADDD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 02 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(02) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60570, 433, 3453},
    /* 3659 : VPHADDSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 03 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(03) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHADDSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60584, 433, 3454},
    /* 3660 : VPHMINPOSUW xmmreg,xmmrm128 [rm:       vex.128.66.0f38 41 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(41) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHMINPOSUW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60598, 433, 3455},
    /* 3661 : VPHSUBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 05 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(05) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60605, 433, 3456},
    /* 3662 : VPHSUBD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 06 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(06) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60619, 433, 3457},
    /* 3663 : VPHSUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 07 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(07) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPHSUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60633, 433, 3458},
    /* 3664 : VPINSRB xmmreg,xmmreg*,mem8,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS8|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52251, 433, 3459},
    /* 3665 : VPINSRB xmmreg,xmmreg*,rm8,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS8|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52251, 433, 3460},
    /* 3666 : VPINSRB xmmreg,xmmreg*,reg32,imm8 [rvmi:     vex.nds.128.66.0f3a 20 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(20) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52251, 433, 3461},
    /* 3667 : VPINSRW xmmreg,xmmreg*,mem16,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS16|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52267, 433, 3462},
    /* 3668 : VPINSRW xmmreg,xmmreg*,rm16,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS16|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52267, 433, 3463},
    /* 3669 : VPINSRW xmmreg,xmmreg*,reg32,imm8 [rvmi:     vex.nds.128.66.0f c4 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c4) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,REG_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52267, 433, 3464},
    /* 3670 : VPINSRD xmmreg,xmmreg*,mem32,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 22 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(22) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52283, 441, 3465},
    /* 3671 : VPINSRD xmmreg,xmmreg*,rm32,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 22 /r ib   ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(22) 120 023 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VPINSRD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52283, 441, 3466},
    /* 3672 : VPINSRQ xmmreg,xmmreg*,mem64,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 22 /r ib   ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(003 201) 001(22) 120 023 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPINSRQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52299, 456, 3467},
    /* 3673 : VPINSRQ xmmreg,xmmreg*,rm64,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 22 /r ib   ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 261(003 201) 001(22) 120 023 : NOAPX,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPINSRQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_GPR|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52299, 456, 3468},
    /* 3674 : VPMADDWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMADDWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60647, 433, 3469},
    /* 3675 : VPMADDUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 04 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(04) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMADDUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60661, 433, 3470},
    /* 3676 : VPMAXSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3c /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60675, 433, 3471},
    /* 3677 : VPMAXSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ee /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ee) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60689, 433, 3472},
    /* 3678 : VPMAXSD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3d /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60703, 433, 3473},
    /* 3679 : VPMAXUB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f de /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(de) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60717, 433, 3474},
    /* 3680 : VPMAXUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3e /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3e) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60731, 433, 3475},
    /* 3681 : VPMAXUD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3f /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3f) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMAXUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60745, 433, 3476},
    /* 3682 : VPMINSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 38 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(38) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60759, 433, 3477},
    /* 3683 : VPMINSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ea /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ea) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60773, 433, 3478},
    /* 3684 : VPMINSD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 39 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(39) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60787, 433, 3479},
    /* 3685 : VPMINUB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f da /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(da) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60801, 433, 3480},
    /* 3686 : VPMINUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3a /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60815, 433, 3481},
    /* 3687 : VPMINUD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 3b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(3b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMINUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60829, 433, 3482},
    /* 3688 : VPMOVMSKB reg64,xmmreg [rm:       vex.128.66.0f d7 /r               ] LONG,PROT,AVX,X86_64,SANDYBRIDGE */
        /* 270(001 001) 001(d7) 110 : NOAPX,WIG,LONG,VEX,PROT,AVX,SANDYBRIDGE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS64|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60843, 455, 3483},
    /* 3689 : VPMOVMSKB reg32,xmmreg [rm:       vex.128.66.0f d7 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(d7) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS32|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60843, 433, 3484},
    /* 3690 : VPMOVSXBW xmmreg,xmmrm64 [rm:       vex.128.66.0f38 20 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(20) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60850, 433, 3485},
    /* 3691 : VPMOVSXBD xmmreg,xmmrm32 [rm:       vex.128.66.0f38 21 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(21) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60857, 433, 3486},
    /* 3692 : VPMOVSXBQ xmmreg,xmmrm16 [rm:       vex.128.66.0f38 22 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(22) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60864, 433, 3487},
    /* 3693 : VPMOVSXWD xmmreg,xmmrm64 [rm:       vex.128.66.0f38 23 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(23) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60871, 433, 3488},
    /* 3694 : VPMOVSXWQ xmmreg,xmmrm32 [rm:       vex.128.66.0f38 24 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(24) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60878, 433, 3489},
    /* 3695 : VPMOVSXDQ xmmreg,xmmrm64 [rm:       vex.128.66.0f38 25 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(25) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVSXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60885, 433, 3490},
    /* 3696 : VPMOVZXBW xmmreg,xmmrm64 [rm:       vex.128.66.0f38 30 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(30) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBW, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60892, 433, 3491},
    /* 3697 : VPMOVZXBD xmmreg,xmmrm32 [rm:       vex.128.66.0f38 31 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(31) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBD, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60899, 433, 3492},
    /* 3698 : VPMOVZXBQ xmmreg,xmmrm16 [rm:       vex.128.66.0f38 32 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(32) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60906, 433, 3493},
    /* 3699 : VPMOVZXWD xmmreg,xmmrm64 [rm:       vex.128.66.0f38 33 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(33) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXWD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60913, 433, 3494},
    /* 3700 : VPMOVZXWQ xmmreg,xmmrm32 [rm:       vex.128.66.0f38 34 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(34) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60920, 433, 3495},
    /* 3701 : VPMOVZXDQ xmmreg,xmmrm64 [rm:       vex.128.66.0f38 35 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(35) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMOVZXDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+60927, 433, 3496},
    /* 3702 : VPMULHUW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHUW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60934, 433, 3497},
    /* 3703 : VPMULHRSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 0b /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0b) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHRSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60948, 433, 3498},
    /* 3704 : VPMULHW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULHW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60962, 433, 3499},
    /* 3705 : VPMULLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d5 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d5) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60976, 433, 3500},
    /* 3706 : VPMULLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 40 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(40) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+60990, 433, 3501},
    /* 3707 : VPMULUDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f4 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f4) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULUDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61004, 433, 3502},
    /* 3708 : VPMULDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 28 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(28) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPMULDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61018, 433, 3503},
    /* 3709 : VPOR xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f eb /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(eb) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPOR, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61032, 433, 3504},
    /* 3710 : VPSADBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f6 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f6) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSADBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61046, 433, 3505},
    /* 3711 : VPSHUFB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 00 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(00) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61060, 433, 3506},
    /* 3712 : VPSHUFD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52315, 433, 3507},
    /* 3713 : VPSHUFHW xmmreg,xmmrm128,imm8 [rmi:      vex.128.f3.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 002) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFHW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52323, 433, 3508},
    /* 3714 : VPSHUFLW xmmreg,xmmrm128,imm8 [rmi:      vex.128.f2.0f 70 /r ib            ] AVX,SANDYBRIDGE */
        /* 270(001 003) 001(70) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSHUFLW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52331, 433, 3509},
    /* 3715 : VPSIGNB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 08 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(08) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGNB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61074, 433, 3510},
    /* 3716 : VPSIGNW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 09 /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(09) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGNW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61088, 433, 3511},
    /* 3717 : VPSIGND xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38 0a /r         ] AVX,SANDYBRIDGE */
        /* 261(002 001) 001(0a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSIGND, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61102, 433, 3512},
    /* 3718 : VPSLLDQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /7 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 217 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52339, 433, 3513},
    /* 3719 : VPSRLDQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /3 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 213 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52355, 433, 3514},
    /* 3720 : VPSLLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61116, 433, 3515},
    /* 3721 : VPSLLW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52371, 433, 3516},
    /* 3722 : VPSLLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61130, 433, 3517},
    /* 3723 : VPSLLD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52387, 433, 3518},
    /* 3724 : VPSLLQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61144, 433, 3519},
    /* 3725 : VPSLLQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /6 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 216 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSLLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52403, 433, 3520},
    /* 3726 : VPSRAW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61158, 433, 3521},
    /* 3727 : VPSRAW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /4 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 214 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52419, 433, 3522},
    /* 3728 : VPSRAD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61172, 433, 3523},
    /* 3729 : VPSRAD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /4 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 214 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52435, 433, 3524},
    /* 3730 : VPSRLW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d1 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d1) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61186, 433, 3525},
    /* 3731 : VPSRLW xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 71 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(71) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52451, 433, 3526},
    /* 3732 : VPSRLD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d2 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d2) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61200, 433, 3527},
    /* 3733 : VPSRLD xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 72 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(72) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52467, 433, 3528},
    /* 3734 : VPSRLQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d3 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d3) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61214, 433, 3529},
    /* 3735 : VPSRLQ xmmreg,xmmreg*,imm8 [vmi:      vex.ndd.128.66.0f 73 /2 ib        ] AVX,SANDYBRIDGE */
        /* 260(001 001) 001(73) 212 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSRLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52483, 433, 3530},
    /* 3736 : VPTEST xmmreg,xmmrm128 [rm:       vex.128.66.0f38 17 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(17) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPTEST, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61228, 433, 3531},
    /* 3737 : VPTEST ymmreg,ymmrm256 [rm:       vex.256.66.0f38 17 /r             ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(17) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPTEST, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61235, 433, 3532},
    /* 3738 : VPSUBB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61242, 433, 3533},
    /* 3739 : VPSUBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f f9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(f9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61256, 433, 3534},
    /* 3740 : VPSUBD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fa /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fa) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61270, 433, 3535},
    /* 3741 : VPSUBQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f fb /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(fb) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61284, 433, 3536},
    /* 3742 : VPSUBSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61298, 433, 3537},
    /* 3743 : VPSUBSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f e9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(e9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61312, 433, 3538},
    /* 3744 : VPSUBUSB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d8 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d8) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBUSB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61326, 433, 3539},
    /* 3745 : VPSUBUSW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f d9 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(d9) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPSUBUSW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61340, 433, 3540},
    /* 3746 : VPUNPCKHBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 68 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(68) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61354, 433, 3541},
    /* 3747 : VPUNPCKHWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 69 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(69) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61368, 433, 3542},
    /* 3748 : VPUNPCKHDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6a /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6a) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61382, 433, 3543},
    /* 3749 : VPUNPCKHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6d /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6d) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61396, 433, 3544},
    /* 3750 : VPUNPCKLBW xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 60 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(60) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLBW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61410, 433, 3545},
    /* 3751 : VPUNPCKLWD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 61 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(61) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLWD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61424, 433, 3546},
    /* 3752 : VPUNPCKLDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 62 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(62) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61438, 433, 3547},
    /* 3753 : VPUNPCKLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 6c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(6c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPUNPCKLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61452, 433, 3548},
    /* 3754 : VPXOR xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f ef /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(ef) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPXOR, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61466, 433, 3549},
    /* 3755 : VRCPPS xmmreg,xmmrm128 [rm:       vex.128.0f 53 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(53) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61480, 433, 3550},
    /* 3756 : VRCPPS ymmreg,ymmrm256 [rm:       vex.256.0f 53 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(53) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61487, 433, 3551},
    /* 3757 : VRCPSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 53 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(53) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRCPSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61494, 440, 3552},
    /* 3758 : VRSQRTPS xmmreg,xmmrm128 [rm:       vex.128.0f 52 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(52) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61508, 433, 3553},
    /* 3759 : VRSQRTPS ymmreg,ymmrm256 [rm:       vex.256.0f 52 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(52) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61515, 433, 3554},
    /* 3760 : VRSQRTSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 52 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(52) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VRSQRTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61522, 440, 3555},
    /* 3761 : VROUNDPD xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 09 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(09) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52499, 433, 3556},
    /* 3762 : VROUNDPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a 09 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(09) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52507, 433, 3557},
    /* 3763 : VROUNDPS xmmreg,xmmrm128,imm8 [rmi:      vex.128.66.0f3a 08 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 001) 001(08) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPS, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52515, 433, 3558},
    /* 3764 : VROUNDPS ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a 08 /r ib          ] AVX,SANDYBRIDGE */
        /* 270(003 005) 001(08) 110 022 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDPS, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52523, 433, 3559},
    /* 3765 : VROUNDSD xmmreg,xmmreg*,xmmrm64,imm8 [rvmi:     vex.nds.128.66.0f3a 0b /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0b) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52531, 433, 3560},
    /* 3766 : VROUNDSS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:     vex.nds.128.66.0f3a 0a /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(0a) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VROUNDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52547, 433, 3561},
    /* 3767 : VSHUFPD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f c6 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52563, 433, 3562},
    /* 3768 : VSHUFPD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f c6 /r ib        ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52579, 433, 3563},
    /* 3769 : VSHUFPS xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.0f c6 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52595, 433, 3564},
    /* 3770 : VSHUFPS ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.0f c6 /r ib           ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(c6) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSHUFPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52611, 433, 3565},
    /* 3771 : VSQRTPD xmmreg,xmmrm128 [rm:       vex.128.66.0f 51 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 001) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61536, 433, 3566},
    /* 3772 : VSQRTPD ymmreg,ymmrm256 [rm:       vex.256.66.0f 51 /r               ] AVX,SANDYBRIDGE */
        /* 270(001 005) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61543, 433, 3567},
    /* 3773 : VSQRTPS xmmreg,xmmrm128 [rm:       vex.128.0f 51 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61550, 433, 3568},
    /* 3774 : VSQRTPS ymmreg,ymmrm256 [rm:       vex.256.0f 51 /r                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(51) 110 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61557, 433, 3569},
    /* 3775 : VSQRTSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 51 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(51) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61564, 440, 3570},
    /* 3776 : VSQRTSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 51 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(51) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSQRTSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61578, 440, 3571},
    /* 3777 : VSTMXCSR mem32 [m:        vex.128.0f ae /3                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(ae) 203 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSTMXCSR, 1, {MEMORY|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+61592, 433, 3572},
    /* 3778 : VSUBPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61599, 433, 3573},
    /* 3779 : VSUBPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61613, 433, 3574},
    /* 3780 : VSUBPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 5c /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61627, 433, 3575},
    /* 3781 : VSUBPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 5c /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(5c) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61641, 433, 3576},
    /* 3782 : VSUBSD xmmreg,xmmreg*,xmmrm64 [rvm:      vex.nds.lig.f2.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 003) 001(5c) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61655, 440, 3577},
    /* 3783 : VSUBSS xmmreg,xmmreg*,xmmrm32 [rvm:      vex.nds.lig.f3.0f 5c /r           ] AVX,SANDYBRIDGE */
        /* 261(001 002) 001(5c) 120 : NOAPX,LIG,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VSUBSS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61669, 440, 3578},
    /* 3784 : VTESTPS xmmreg,xmmrm128 [rm:       vex.128.66.0f38.w0 0e /r          ] FL,AVX,SANDYBRIDGE */
        /* 270(002 001) 001(0e) 110 : NOAPX,FL,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61683, 457, 3579},
    /* 3785 : VTESTPS ymmreg,ymmrm256 [rm:       vex.256.66.0f38.w0 0e /r          ] FL,AVX,SANDYBRIDGE */
        /* 270(002 005) 001(0e) 110 : NOAPX,FL,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61690, 457, 3580},
    /* 3786 : VTESTPD xmmreg,xmmrm128 [rm:       vex.128.66.0f38.w0 0f /r          ] AVX,SANDYBRIDGE */
        /* 270(002 001) 001(0f) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61697, 441, 3581},
    /* 3787 : VTESTPD ymmreg,ymmrm256 [rm:       vex.256.66.0f38.w0 0f /r          ] AVX,SANDYBRIDGE */
        /* 270(002 005) 001(0f) 110 : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VTESTPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61704, 441, 3582},
    /* 3788 : VUCOMISD xmmreg,xmmrm64 [rm:       vex.lig.66.0f 2e /r               ] FL,AVX,SANDYBRIDGE */
        /* 270(001 001) 001(2e) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VUCOMISD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61711, 442, 3583},
    /* 3789 : VUCOMISS xmmreg,xmmrm32 [rm:       vex.lig.0f 2e /r                  ] FL,AVX,SANDYBRIDGE */
        /* 270(001 000) 001(2e) 110 : NOAPX,LIG,WIG,FL,VEX,AVX,SANDYBRIDGE */
        {I_VUCOMISS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+61718, 442, 3584},
    /* 3790 : VUNPCKHPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 15 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61725, 433, 3585},
    /* 3791 : VUNPCKHPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 15 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61739, 433, 3586},
    /* 3792 : VUNPCKHPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 15 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61753, 433, 3587},
    /* 3793 : VUNPCKHPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 15 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(15) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKHPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61767, 433, 3588},
    /* 3794 : VUNPCKLPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 14 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61781, 433, 3589},
    /* 3795 : VUNPCKLPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 14 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61795, 433, 3590},
    /* 3796 : VUNPCKLPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 14 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61809, 433, 3591},
    /* 3797 : VUNPCKLPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 14 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(14) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VUNPCKLPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61823, 433, 3592},
    /* 3798 : VXORPD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f 57 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 001) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61837, 433, 3593},
    /* 3799 : VXORPD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 57 /r           ] AVX,SANDYBRIDGE */
        /* 261(001 005) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61851, 433, 3594},
    /* 3800 : VXORPS xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.0f 57 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 000) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61865, 433, 3595},
    /* 3801 : VXORPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.0f 57 /r              ] AVX,SANDYBRIDGE */
        /* 261(001 004) 001(57) 120 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VXORPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61879, 433, 3596},
    /* 3802 : VZEROALL void [          vex.256.0f.w0 77                  ] AVX,SANDYBRIDGE */
        /* 270(001 004) 001(77) : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VZEROALL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69963, 441, 3597},
    /* 3803 : VZEROUPPER void [          vex.128.0f.w0 77                  ] AVX,SANDYBRIDGE */
        /* 270(001 000) 001(77) : NOAPX,VEX,AVX,SANDYBRIDGE */
        {I_VZEROUPPER, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69969, 441, 3598},
    /* 3804 : PCLMULLQLQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 00                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(00) : NOAPX,SSE,WESTMERE */
        {I_PCLMULLQLQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52627, 432, 3601},
    /* 3805 : PCLMULHQLQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 01                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(01) : NOAPX,SSE,WESTMERE */
        {I_PCLMULHQLQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52635, 432, 3602},
    /* 3806 : PCLMULLQHQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 10                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(10) : NOAPX,SSE,WESTMERE */
        {I_PCLMULLQHQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52643, 432, 3603},
    /* 3807 : PCLMULHQHQDQ xmmreg,xmmrm128 [rm:       66 0f3a 44 /r 11                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 001(11) : NOAPX,SSE,WESTMERE */
        {I_PCLMULHQHQDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+52651, 432, 3604},
    /* 3808 : PCLMULQDQ xmmreg,xmmrm128,imm8 [rmi:      66 0f3a 44 /r ib                  ] SSE,WESTMERE */
        /* 361 357 001(44) 110 022 : NOAPX,SSE,WESTMERE */
        {I_PCLMULQDQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+61893, 432, 3605},
    /* 3809 : VPCLMULLQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 00      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(00) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULLQLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29416, 433, 3608},
    /* 3810 : VPCLMULHQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 01      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(01) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULHQLQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29434, 433, 3609},
    /* 3811 : VPCLMULLQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 10      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(10) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULLQHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29452, 433, 3610},
    /* 3812 : VPCLMULHQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f3a 44 /r 11      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 001(11) : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULHQHQDQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29470, 433, 3611},
    /* 3813 : VPCLMULQDQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a 44 /r ib      ] AVX,SANDYBRIDGE */
        /* 261(003 001) 001(44) 120 023 : NOAPX,WIG,VEX,AVX,SANDYBRIDGE */
        {I_VPCLMULQDQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52659, 433, 3612},
    /* 3814 : VPCLMULLQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 00  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(00) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29488, 458, 3615},
    /* 3815 : VPCLMULHQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 01  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(01) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29506, 458, 3616},
    /* 3816 : VPCLMULLQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 10  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(10) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29524, 458, 3617},
    /* 3817 : VPCLMULHQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   vex.nds.256.66.0f3a.wig 44 /r 11  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 001(11) : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+29542, 458, 3618},
    /* 3818 : VPCLMULQDQ ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:fv:  vex.nds.256.66.0f3a.wig 44 /r ib  ] VPCLMULQDQ,FUTURE */
        /* 261(003 005) 001(44) 120 023 : NOAPX,WIG,VEX,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+52675, 458, 3619},
    /* 3819 : VPCLMULLQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 00 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4479, 459, 3622},
    /* 3820 : VPCLMULHQLQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 01 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4501, 459, 3623},
    /* 3821 : VPCLMULLQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 10 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4523, 459, 3624},
    /* 3822 : VPCLMULHQHQDQ xmmreg,xmmreg*,xmmrm128 [rvm:fv:   evex.nds.128.66.0f3a.wig 44 /r 11 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+4545, 459, 3625},
    /* 3823 : VPCLMULQDQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.wig 44 /r ib ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 08) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+17993, 459, 3626},
    /* 3824 : VPCLMULLQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 00 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4567, 459, 3628},
    /* 3825 : VPCLMULHQLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 01 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4589, 459, 3629},
    /* 3826 : VPCLMULLQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 10 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4611, 459, 3630},
    /* 3827 : VPCLMULHQHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:fv:   evex.nds.256.66.0f3a.wig 44 /r 11 ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+4633, 459, 3631},
    /* 3828 : VPCLMULQDQ ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.wig 44 /r ib ] AVX512VL,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 28) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512VL,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+18013, 459, 3632},
    /* 3829 : VPCLMULLQLQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 00 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(00) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4655, 460, 3635},
    /* 3830 : VPCLMULHQLQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 01 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(01) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4677, 460, 3636},
    /* 3831 : VPCLMULLQHQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 10 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(10) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULLQHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4699, 460, 3637},
    /* 3832 : VPCLMULHQHQDQ zmmreg,zmmreg*,zmmrm512 [rvm:fv:   evex.nds.512.66.0f3a.wig 44 /r 11 ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 001(11) : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULHQHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+4721, 460, 3638},
    /* 3833 : VPCLMULQDQ zmmreg,zmmreg*,zmmrm512,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.wig 44 /r ib ] AVX512,VPCLMULQDQ,FUTURE */
        /* 241(f3 7d 48) 301 001(44) 120 023 : WIG,EVEX,PROT,AVX512,VPCLMULQDQ,FUTURE */
        {I_VPCLMULQDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+18033, 460, 3639},
    /* 3834 : VFMADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61900, 461, 3643},
    /* 3835 : VFMADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61907, 461, 3644},
    /* 3836 : VFMADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61914, 461, 3645},
    /* 3837 : VFMADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61921, 461, 3646},
    /* 3838 : VFMADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61900, 461, 3647},
    /* 3839 : VFMADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 98 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61907, 461, 3648},
    /* 3840 : VFMADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61914, 461, 3649},
    /* 3841 : VFMADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 98 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(98) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61921, 461, 3650},
    /* 3842 : VFMADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61928, 461, 3651},
    /* 3843 : VFMADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61935, 461, 3652},
    /* 3844 : VFMADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61942, 461, 3653},
    /* 3845 : VFMADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61949, 461, 3654},
    /* 3846 : VFMADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61928, 461, 3655},
    /* 3847 : VFMADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61935, 461, 3656},
    /* 3848 : VFMADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61942, 461, 3657},
    /* 3849 : VFMADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61949, 461, 3658},
    /* 3850 : VFMADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61956, 461, 3659},
    /* 3851 : VFMADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61963, 461, 3660},
    /* 3852 : VFMADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61970, 461, 3661},
    /* 3853 : VFMADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61977, 461, 3662},
    /* 3854 : VFMADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61956, 461, 3663},
    /* 3855 : VFMADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b8 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61963, 461, 3664},
    /* 3856 : VFMADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61970, 461, 3665},
    /* 3857 : VFMADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b8 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b8) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61977, 461, 3666},
    /* 3858 : VFMADDSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61984, 461, 3667},
    /* 3859 : VFMADDSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61991, 461, 3668},
    /* 3860 : VFMADDSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61998, 461, 3669},
    /* 3861 : VFMADDSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62005, 461, 3670},
    /* 3862 : VFMADDSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61984, 461, 3671},
    /* 3863 : VFMADDSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 96 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61991, 461, 3672},
    /* 3864 : VFMADDSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+61998, 461, 3673},
    /* 3865 : VFMADDSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 96 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(96) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62005, 461, 3674},
    /* 3866 : VFMADDSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62012, 461, 3675},
    /* 3867 : VFMADDSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62019, 461, 3676},
    /* 3868 : VFMADDSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62026, 461, 3677},
    /* 3869 : VFMADDSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62033, 461, 3678},
    /* 3870 : VFMADDSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62012, 461, 3679},
    /* 3871 : VFMADDSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62019, 461, 3680},
    /* 3872 : VFMADDSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62026, 461, 3681},
    /* 3873 : VFMADDSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62033, 461, 3682},
    /* 3874 : VFMADDSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62040, 461, 3683},
    /* 3875 : VFMADDSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62047, 461, 3684},
    /* 3876 : VFMADDSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62054, 461, 3685},
    /* 3877 : VFMADDSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62061, 461, 3686},
    /* 3878 : VFMADDSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62040, 461, 3687},
    /* 3879 : VFMADDSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b6 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62047, 461, 3688},
    /* 3880 : VFMADDSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62054, 461, 3689},
    /* 3881 : VFMADDSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b6 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b6) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADDSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62061, 461, 3690},
    /* 3882 : VFMSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62068, 461, 3691},
    /* 3883 : VFMSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62075, 461, 3692},
    /* 3884 : VFMSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62082, 461, 3693},
    /* 3885 : VFMSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62089, 461, 3694},
    /* 3886 : VFMSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62068, 461, 3695},
    /* 3887 : VFMSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9a /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62075, 461, 3696},
    /* 3888 : VFMSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62082, 461, 3697},
    /* 3889 : VFMSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9a /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9a) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62089, 461, 3698},
    /* 3890 : VFMSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 001) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62096, 461, 3699},
    /* 3891 : VFMSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 005) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62103, 461, 3700},
    /* 3892 : VFMSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 201) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62110, 461, 3701},
    /* 3893 : VFMSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 205) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62117, 461, 3702},
    /* 3894 : VFMSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 001) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62096, 461, 3703},
    /* 3895 : VFMSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 aa /r      ] FMA,FUTURE */
        /* 261(002 005) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62103, 461, 3704},
    /* 3896 : VFMSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 201) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62110, 461, 3705},
    /* 3897 : VFMSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 aa /r      ] FMA,FUTURE */
        /* 261(002 205) 001(aa) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62117, 461, 3706},
    /* 3898 : VFMSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62124, 461, 3707},
    /* 3899 : VFMSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62131, 461, 3708},
    /* 3900 : VFMSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62138, 461, 3709},
    /* 3901 : VFMSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62145, 461, 3710},
    /* 3902 : VFMSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62124, 461, 3711},
    /* 3903 : VFMSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ba /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62131, 461, 3712},
    /* 3904 : VFMSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62138, 461, 3713},
    /* 3905 : VFMSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ba /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ba) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62145, 461, 3714},
    /* 3906 : VFMSUBADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62152, 461, 3715},
    /* 3907 : VFMSUBADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62159, 461, 3716},
    /* 3908 : VFMSUBADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62166, 461, 3717},
    /* 3909 : VFMSUBADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62173, 461, 3718},
    /* 3910 : VFMSUBADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62152, 461, 3719},
    /* 3911 : VFMSUBADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 97 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62159, 461, 3720},
    /* 3912 : VFMSUBADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62166, 461, 3721},
    /* 3913 : VFMSUBADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 97 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(97) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62173, 461, 3722},
    /* 3914 : VFMSUBADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62180, 461, 3723},
    /* 3915 : VFMSUBADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62187, 461, 3724},
    /* 3916 : VFMSUBADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62194, 461, 3725},
    /* 3917 : VFMSUBADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62201, 461, 3726},
    /* 3918 : VFMSUBADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62180, 461, 3727},
    /* 3919 : VFMSUBADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 a7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62187, 461, 3728},
    /* 3920 : VFMSUBADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62194, 461, 3729},
    /* 3921 : VFMSUBADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 a7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(a7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62201, 461, 3730},
    /* 3922 : VFMSUBADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62208, 461, 3731},
    /* 3923 : VFMSUBADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62215, 461, 3732},
    /* 3924 : VFMSUBADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62222, 461, 3733},
    /* 3925 : VFMSUBADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62229, 461, 3734},
    /* 3926 : VFMSUBADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62208, 461, 3735},
    /* 3927 : VFMSUBADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 b7 /r      ] FMA,FUTURE */
        /* 261(002 005) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62215, 461, 3736},
    /* 3928 : VFMSUBADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62222, 461, 3737},
    /* 3929 : VFMSUBADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 b7 /r      ] FMA,FUTURE */
        /* 261(002 205) 001(b7) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUBADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62229, 461, 3738},
    /* 3930 : VFNMADD132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62236, 461, 3739},
    /* 3931 : VFNMADD132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62243, 461, 3740},
    /* 3932 : VFNMADD132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62250, 461, 3741},
    /* 3933 : VFNMADD132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62257, 461, 3742},
    /* 3934 : VFNMADD312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62236, 461, 3743},
    /* 3935 : VFNMADD312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9c /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62243, 461, 3744},
    /* 3936 : VFNMADD312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62250, 461, 3745},
    /* 3937 : VFNMADD312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9c /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9c) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62257, 461, 3746},
    /* 3938 : VFNMADD213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62264, 461, 3747},
    /* 3939 : VFNMADD213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62271, 461, 3748},
    /* 3940 : VFNMADD213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62278, 461, 3749},
    /* 3941 : VFNMADD213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62285, 461, 3750},
    /* 3942 : VFNMADD123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62264, 461, 3751},
    /* 3943 : VFNMADD123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ac /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62271, 461, 3752},
    /* 3944 : VFNMADD123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62278, 461, 3753},
    /* 3945 : VFNMADD123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ac /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ac) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62285, 461, 3754},
    /* 3946 : VFNMADD231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62292, 461, 3755},
    /* 3947 : VFNMADD231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 005) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62299, 461, 3756},
    /* 3948 : VFNMADD231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62306, 461, 3757},
    /* 3949 : VFNMADD231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 205) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62313, 461, 3758},
    /* 3950 : VFNMADD321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62292, 461, 3759},
    /* 3951 : VFNMADD321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 bc /r      ] FMA,FUTURE */
        /* 261(002 005) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62299, 461, 3760},
    /* 3952 : VFNMADD321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62306, 461, 3761},
    /* 3953 : VFNMADD321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 bc /r      ] FMA,FUTURE */
        /* 261(002 205) 001(bc) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62313, 461, 3762},
    /* 3954 : VFNMSUB132PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62320, 461, 3763},
    /* 3955 : VFNMSUB132PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62327, 461, 3764},
    /* 3956 : VFNMSUB132PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62334, 461, 3765},
    /* 3957 : VFNMSUB132PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62341, 461, 3766},
    /* 3958 : VFNMSUB312PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62320, 461, 3767},
    /* 3959 : VFNMSUB312PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 9e /r      ] FMA,FUTURE */
        /* 261(002 005) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62327, 461, 3768},
    /* 3960 : VFNMSUB312PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62334, 461, 3769},
    /* 3961 : VFNMSUB312PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 9e /r      ] FMA,FUTURE */
        /* 261(002 205) 001(9e) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62341, 461, 3770},
    /* 3962 : VFNMSUB213PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62348, 461, 3771},
    /* 3963 : VFNMSUB213PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62355, 461, 3772},
    /* 3964 : VFNMSUB213PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62362, 461, 3773},
    /* 3965 : VFNMSUB213PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62369, 461, 3774},
    /* 3966 : VFNMSUB123PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62348, 461, 3775},
    /* 3967 : VFNMSUB123PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 ae /r      ] FMA,FUTURE */
        /* 261(002 005) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62355, 461, 3776},
    /* 3968 : VFNMSUB123PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62362, 461, 3777},
    /* 3969 : VFNMSUB123PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 ae /r      ] FMA,FUTURE */
        /* 261(002 205) 001(ae) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62369, 461, 3778},
    /* 3970 : VFNMSUB231PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 001) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62376, 461, 3779},
    /* 3971 : VFNMSUB231PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 005) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62383, 461, 3780},
    /* 3972 : VFNMSUB231PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 201) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62390, 461, 3781},
    /* 3973 : VFNMSUB231PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 205) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62397, 461, 3782},
    /* 3974 : VFNMSUB321PS xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 001) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62376, 461, 3783},
    /* 3975 : VFNMSUB321PS ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w0 be /r      ] FMA,FUTURE */
        /* 261(002 005) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62383, 461, 3784},
    /* 3976 : VFNMSUB321PD xmmreg,xmmreg,xmmrm128 [rvm:      vex.dds.128.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 201) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62390, 461, 3785},
    /* 3977 : VFNMSUB321PD ymmreg,ymmreg,ymmrm256 [rvm:      vex.dds.256.66.0f38.w1 be /r      ] FMA,FUTURE */
        /* 261(002 205) 001(be) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321PD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62397, 461, 3786},
    /* 3978 : VFMADD132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 99 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62404, 461, 3787},
    /* 3979 : VFMADD132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 99 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62411, 461, 3788},
    /* 3980 : VFMADD312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 99 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62404, 461, 3789},
    /* 3981 : VFMADD312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 99 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(99) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62411, 461, 3790},
    /* 3982 : VFMADD213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 a9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62418, 461, 3791},
    /* 3983 : VFMADD213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 a9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62425, 461, 3792},
    /* 3984 : VFMADD123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 a9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62418, 461, 3793},
    /* 3985 : VFMADD123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 a9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(a9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62425, 461, 3794},
    /* 3986 : VFMADD231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 b9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62432, 461, 3795},
    /* 3987 : VFMADD231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 b9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62439, 461, 3796},
    /* 3988 : VFMADD321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 b9 /r      ] FMA,FUTURE */
        /* 261(002 001) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62432, 461, 3797},
    /* 3989 : VFMADD321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 b9 /r      ] FMA,FUTURE */
        /* 261(002 201) 001(b9) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMADD321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62439, 461, 3798},
    /* 3990 : VFMSUB132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9b /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62446, 461, 3799},
    /* 3991 : VFMSUB132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9b /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62453, 461, 3800},
    /* 3992 : VFMSUB312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9b /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62446, 461, 3801},
    /* 3993 : VFMSUB312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9b /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9b) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62453, 461, 3802},
    /* 3994 : VFMSUB213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ab /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62460, 461, 3803},
    /* 3995 : VFMSUB213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ab /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62467, 461, 3804},
    /* 3996 : VFMSUB123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ab /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62460, 461, 3805},
    /* 3997 : VFMSUB123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ab /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ab) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62467, 461, 3806},
    /* 3998 : VFMSUB231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bb /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62474, 461, 3807},
    /* 3999 : VFMSUB231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bb /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62481, 461, 3808},
    /* 4000 : VFMSUB321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bb /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62474, 461, 3809},
    /* 4001 : VFMSUB321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bb /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bb) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFMSUB321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62481, 461, 3810},
    /* 4002 : VFNMADD132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9d /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62488, 461, 3811},
    /* 4003 : VFNMADD132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9d /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62495, 461, 3812},
    /* 4004 : VFNMADD312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9d /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62488, 461, 3813},
    /* 4005 : VFNMADD312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9d /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9d) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62495, 461, 3814},
    /* 4006 : VFNMADD213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ad /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62502, 461, 3815},
    /* 4007 : VFNMADD213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ad /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62509, 461, 3816},
    /* 4008 : VFNMADD123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 ad /r      ] FMA,FUTURE */
        /* 261(002 001) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62502, 461, 3817},
    /* 4009 : VFNMADD123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 ad /r      ] FMA,FUTURE */
        /* 261(002 201) 001(ad) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62509, 461, 3818},
    /* 4010 : VFNMADD231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bd /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62516, 461, 3819},
    /* 4011 : VFNMADD231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bd /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62523, 461, 3820},
    /* 4012 : VFNMADD321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bd /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62516, 461, 3821},
    /* 4013 : VFNMADD321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bd /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bd) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMADD321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62523, 461, 3822},
    /* 4014 : VFNMSUB132SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9f /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62530, 461, 3823},
    /* 4015 : VFNMSUB132SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9f /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB132SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62537, 461, 3824},
    /* 4016 : VFNMSUB312SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 9f /r      ] FMA,FUTURE */
        /* 261(002 001) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62530, 461, 3825},
    /* 4017 : VFNMSUB312SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 9f /r      ] FMA,FUTURE */
        /* 261(002 201) 001(9f) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB312SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62537, 461, 3826},
    /* 4018 : VFNMSUB213SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 af /r      ] FMA,FUTURE */
        /* 261(002 001) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62544, 461, 3827},
    /* 4019 : VFNMSUB213SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 af /r      ] FMA,FUTURE */
        /* 261(002 201) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB213SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62551, 461, 3828},
    /* 4020 : VFNMSUB123SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 af /r      ] FMA,FUTURE */
        /* 261(002 001) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62544, 461, 3829},
    /* 4021 : VFNMSUB123SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 af /r      ] FMA,FUTURE */
        /* 261(002 201) 001(af) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB123SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62551, 461, 3830},
    /* 4022 : VFNMSUB231SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bf /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62558, 461, 3831},
    /* 4023 : VFNMSUB231SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bf /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB231SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62565, 461, 3832},
    /* 4024 : VFNMSUB321SS xmmreg,xmmreg,xmmrm32 [rvm:      vex.dds.128.66.0f38.w0 bf /r      ] FMA,FUTURE */
        /* 261(002 001) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321SS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62558, 461, 3833},
    /* 4025 : VFNMSUB321SD xmmreg,xmmreg,xmmrm64 [rvm:      vex.dds.128.66.0f38.w1 bf /r      ] FMA,FUTURE */
        /* 261(002 201) 001(bf) 120 : NOAPX,VEX,FMA,FUTURE */
        {I_VFNMSUB321SD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62565, 461, 3834},
    /* 4026 : VCVTPH2PS ymmreg,xmmrm128 [rm:       vex.256.66.0f38.w0 13 /r          ] F16C,FUTURE */
        /* 270(002 005) 001(13) 110 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62572, 462, 3839},
    /* 4027 : VCVTPH2PS xmmreg,xmmrm64 [rm:       vex.128.66.0f38.w0 13 /r          ] F16C,FUTURE */
        /* 270(002 001) 001(13) 110 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62579, 462, 3840},
    /* 4028 : VCVTPS2PH xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 1d /r ib       ] F16C,FUTURE */
        /* 270(003 005) 001(1d) 101 022 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPS2PH, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52691, 462, 3841},
    /* 4029 : VCVTPS2PH xmmrm64,xmmreg,imm8 [mri:      vex.128.66.0f3a.w0 1d /r ib       ] F16C,FUTURE */
        /* 270(003 001) 001(1d) 101 022 : NOAPX,VEX,F16C,FUTURE */
        {I_VCVTPS2PH, 3, {RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+52699, 462, 3842},
    /* 4030 : XSTORE void [          0f a7 c0                          ] PENT,CYRIX */
        /* 355 002(a7 c0) : PENT,CYRIX */
        {I_XSTORE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72514, 463, 3847},
    /* 4031 : XCRYPTECB void [          f3 0f a7 c8                       ] PENT,CYRIX */
        /* 333 355 002(a7 c8) : PENT,CYRIX */
        {I_XCRYPTECB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69975, 463, 3848},
    /* 4032 : XCRYPTCBC void [          f3 0f a7 d0                       ] PENT,CYRIX */
        /* 333 355 002(a7 d0) : PENT,CYRIX */
        {I_XCRYPTCBC, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69981, 463, 3849},
    /* 4033 : XCRYPTCTR void [          f3 0f a7 d8                       ] PENT,CYRIX */
        /* 333 355 002(a7 d8) : PENT,CYRIX */
        {I_XCRYPTCTR, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69987, 463, 3850},
    /* 4034 : XCRYPTCFB void [          f3 0f a7 e0                       ] PENT,CYRIX */
        /* 333 355 002(a7 e0) : PENT,CYRIX */
        {I_XCRYPTCFB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69993, 463, 3851},
    /* 4035 : XCRYPTOFB void [          f3 0f a7 e8                       ] PENT,CYRIX */
        /* 333 355 002(a7 e8) : PENT,CYRIX */
        {I_XCRYPTOFB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+69999, 463, 3852},
    /* 4036 : MONTMUL void [          f3 0f a6 c0                       ] PENT,CYRIX */
        /* 333 355 002(a6 c0) : PENT,CYRIX */
        {I_MONTMUL, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70005, 463, 3853},
    /* 4037 : XSHA1 void [          f3 0f a6 c8                       ] PENT,CYRIX */
        /* 333 355 002(a6 c8) : PENT,CYRIX */
        {I_XSHA1, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70011, 463, 3854},
    /* 4038 : XSHA256 void [          f3 0f a6 d0                       ] PENT,CYRIX */
        /* 333 355 002(a6 d0) : PENT,CYRIX */
        {I_XSHA256, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70017, 463, 3855},
    /* 4039 : LLWPCB reg32 [m:        xop.m9.w0.l0.p0 12 /0             ] 386,AMD */
        /* 270(111 000) 001(12) 200 : NOAPX,VEX,386,AMD */
        {I_LLWPCB, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62586, 464, 3866},
    /* 4040 : LLWPCB reg64 [m:        xop.m9.w1.l0.p0 12 /0             ] LONG,PROT,X86_64,AMD */
        /* 270(111 200) 001(12) 200 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LLWPCB, 1, {REG_GPR|BITS64|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62593, 465, 3867},
    /* 4041 : SLWPCB reg32 [m:        xop.m9.w0.l0.p0 12 /1             ] 386,AMD */
        /* 270(111 000) 001(12) 201 : NOAPX,VEX,386,AMD */
        {I_SLWPCB, 1, {REG_GPR|BITS32|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62600, 464, 3869},
    /* 4042 : SLWPCB reg64 [m:        xop.m9.w1.l0.p0 12 /1             ] LONG,PROT,X86_64,AMD */
        /* 270(111 200) 001(12) 201 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_SLWPCB, 1, {REG_GPR|BITS64|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+62607, 465, 3870},
    /* 4043 : LWPVAL reg32,rm32,imm32 [vmi:      xop.m10.w0.ndd.l0.p0 12 /1 id     ] 386,AMD */
        /* 260(112 000) 001(12) 211 042 : NOAPX,VEX,386,AMD */
        {I_LWPVAL, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52707, 464, 3872},
    /* 4044 : LWPVAL reg64,rm32,imm32 [vmi:      xop.m10.w1.ndd.l0.p0 12 /1 id     ] LONG,PROT,X86_64,AMD */
        /* 260(112 200) 001(12) 211 042 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LWPVAL, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52715, 465, 3873},
    /* 4045 : LWPINS reg32,rm32,imm32 [vmi:      xop.m10.w0.ndd.l0.p0 12 /0 id     ] 386,AMD */
        /* 260(112 000) 001(12) 210 042 : NOAPX,VEX,386,AMD */
        {I_LWPINS, 3, {REG_GPR|BITS32|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52723, 464, 3875},
    /* 4046 : LWPINS reg64,rm32,imm32 [vmi:      xop.m10.w1.ndd.l0.p0 12 /0 id     ] LONG,PROT,X86_64,AMD */
        /* 260(112 200) 001(12) 210 042 : NOAPX,LONG,VEX,PROT,X86_64,AMD */
        {I_LWPINS, 3, {REG_GPR|BITS64|RN_L16,RM_GPR|BITS32|RN_L16,IMM_NORMAL|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+52731, 465, 3876},
    /* 4047 : VFMADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(69) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52739, 466, 3882},
    /* 4048 : VFMADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(69) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52755, 466, 3883},
    /* 4049 : VFMADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(69) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52771, 466, 3884},
    /* 4050 : VFMADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 69 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(69) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52787, 466, 3885},
    /* 4051 : VFMADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(68) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52803, 466, 3887},
    /* 4052 : VFMADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(68) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52819, 466, 3888},
    /* 4053 : VFMADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(68) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52835, 466, 3889},
    /* 4054 : VFMADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 68 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(68) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52851, 466, 3890},
    /* 4055 : VFMADDSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6b) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52867, 466, 3892},
    /* 4056 : VFMADDSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 6b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6b) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52883, 466, 3893},
    /* 4057 : VFMADDSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6a) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52899, 466, 3895},
    /* 4058 : VFMADDSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 6a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6a) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52915, 466, 3896},
    /* 4059 : VFMADDSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52931, 466, 3898},
    /* 4060 : VFMADDSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52947, 466, 3899},
    /* 4061 : VFMADDSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52963, 466, 3900},
    /* 4062 : VFMADDSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52979, 466, 3901},
    /* 4063 : VFMADDSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+52995, 466, 3903},
    /* 4064 : VFMADDSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53011, 466, 3904},
    /* 4065 : VFMADDSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53027, 466, 3905},
    /* 4066 : VFMADDSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMADDSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53043, 466, 3906},
    /* 4067 : VFMSUBADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53059, 466, 3908},
    /* 4068 : VFMSUBADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53075, 466, 3909},
    /* 4069 : VFMSUBADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53091, 466, 3910},
    /* 4070 : VFMSUBADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53107, 466, 3911},
    /* 4071 : VFMSUBADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(5e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53123, 466, 3913},
    /* 4072 : VFMSUBADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(5e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53139, 466, 3914},
    /* 4073 : VFMSUBADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(5e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53155, 466, 3915},
    /* 4074 : VFMSUBADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 5e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(5e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53171, 466, 3916},
    /* 4075 : VFMSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53187, 466, 3918},
    /* 4076 : VFMSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(6d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53203, 466, 3919},
    /* 4077 : VFMSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53219, 466, 3920},
    /* 4078 : VFMSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 6d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(6d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53235, 466, 3921},
    /* 4079 : VFMSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53251, 466, 3923},
    /* 4080 : VFMSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(6c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53267, 466, 3924},
    /* 4081 : VFMSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53283, 466, 3925},
    /* 4082 : VFMSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 6c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(6c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53299, 466, 3926},
    /* 4083 : VFMSUBSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53315, 466, 3928},
    /* 4084 : VFMSUBSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 6f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53331, 466, 3929},
    /* 4085 : VFMSUBSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 6e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(6e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53347, 466, 3931},
    /* 4086 : VFMSUBSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 6e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(6e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53363, 466, 3932},
    /* 4087 : VFNMADDPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(79) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53379, 466, 3934},
    /* 4088 : VFNMADDPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(79) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53395, 466, 3935},
    /* 4089 : VFNMADDPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(79) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53411, 466, 3936},
    /* 4090 : VFNMADDPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 79 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(79) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53427, 466, 3937},
    /* 4091 : VFNMADDPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(78) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53443, 466, 3939},
    /* 4092 : VFNMADDPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(78) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53459, 466, 3940},
    /* 4093 : VFNMADDPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(78) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53475, 466, 3941},
    /* 4094 : VFNMADDPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 78 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(78) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53491, 466, 3942},
    /* 4095 : VFNMADDSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7b) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53507, 466, 3944},
    /* 4096 : VFNMADDSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 7b /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7b) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53523, 466, 3945},
    /* 4097 : VFNMADDSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7a) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53539, 466, 3947},
    /* 4098 : VFNMADDSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 7a /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7a) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMADDSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53555, 466, 3948},
    /* 4099 : VFNMSUBPD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53571, 466, 3950},
    /* 4100 : VFNMSUBPD ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(7d) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53587, 466, 3951},
    /* 4101 : VFNMSUBPD xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53603, 466, 3952},
    /* 4102 : VFNMSUBPD ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 7d /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(7d) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53619, 466, 3953},
    /* 4103 : VFNMSUBPS xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53635, 466, 3955},
    /* 4104 : VFNMSUBPS ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.m3.w0.nds.l1.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 005) 001(7c) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53651, 466, 3956},
    /* 4105 : VFNMSUBPS xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     vex.m3.w1.nds.l0.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53667, 466, 3957},
    /* 4106 : VFNMSUBPS ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     vex.m3.w1.nds.l1.p1 7c /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 205) 001(7c) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBPS, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53683, 466, 3958},
    /* 4107 : VFNMSUBSD xmmreg,xmmreg*,xmmrm64,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53699, 466, 3960},
    /* 4108 : VFNMSUBSD xmmreg,xmmreg*,xmmreg,xmmrm64 [rvsm:     vex.m3.w1.nds.l0.p1 7f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7f) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53715, 466, 3961},
    /* 4109 : VFNMSUBSS xmmreg,xmmreg*,xmmrm32,xmmreg [rvms:     vex.m3.w0.nds.l0.p1 7e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 001) 001(7e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53731, 466, 3963},
    /* 4110 : VFNMSUBSS xmmreg,xmmreg*,xmmreg,xmmrm32 [rvsm:     vex.m3.w1.nds.l0.p1 7e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(003 201) 001(7e) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFNMSUBSS, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53747, 466, 3964},
    /* 4111 : VFRCZPD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 81 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(81) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62614, 466, 3966},
    /* 4112 : VFRCZPD ymmreg,ymmrm256* [rm:       xop.m9.w0.l1.p0 81 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 004) 001(81) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62628, 466, 3967},
    /* 4113 : VFRCZPS xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 80 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(80) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPS, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62642, 466, 3969},
    /* 4114 : VFRCZPS ymmreg,ymmrm256* [rm:       xop.m9.w0.l1.p0 80 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 004) 001(80) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZPS, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62656, 466, 3970},
    /* 4115 : VFRCZSD xmmreg,xmmrm64* [rm:       xop.m9.w0.l0.p0 83 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(83) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZSD, 2, {XMMREG|RN_L16,RM_XMM|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62670, 466, 3972},
    /* 4116 : VFRCZSS xmmreg,xmmrm32* [rm:       xop.m9.w0.l0.p0 82 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(82) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VFRCZSS, 2, {XMMREG|RN_L16,RM_XMM|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62684, 466, 3974},
    /* 4117 : VPCMOV xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a2) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53763, 466, 3977},
    /* 4118 : VPCMOV ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     xop.m8.w0.nds.l1.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 004) 001(a2) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53779, 466, 3978},
    /* 4119 : VPCMOV xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     xop.m8.w1.nds.l0.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 200) 001(a2) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53795, 466, 3979},
    /* 4120 : VPCMOV ymmreg,ymmreg*,ymmreg,ymmrm256 [rvsm:     xop.m8.w1.nds.l1.p0 a2 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 204) 001(a2) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCMOV, 4, {YMMREG|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53811, 466, 3980},
    /* 4121 : VPCOMB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cc /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cc) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53827, 466, 3982},
    /* 4122 : VPCOMD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ce /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ce) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53843, 466, 3983},
    /* 4123 : VPCOMQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cf /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cf) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53859, 466, 3984},
    /* 4124 : VPCOMUB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ec /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ec) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53875, 466, 3987},
    /* 4125 : VPCOMUD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ee /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ee) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53891, 466, 3988},
    /* 4126 : VPCOMUQ xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ef /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ef) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUQ, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53907, 466, 3989},
    /* 4127 : VPCOMUW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 ed /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(ed) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMUW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53923, 466, 3992},
    /* 4128 : VPCOMW xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     xop.m8.w0.nds.l0.p0 cd /r ib      ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(cd) 120 023 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPCOMW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+53939, 466, 3993},
    /* 4129 : VPHADDBD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62698, 466, 3995},
    /* 4130 : VPHADDBQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62712, 466, 3996},
    /* 4131 : VPHADDBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62726, 466, 3997},
    /* 4132 : VPHADDDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 cb /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(cb) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62740, 466, 3998},
    /* 4133 : VPHADDUBD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62754, 466, 4001},
    /* 4134 : VPHADDUBQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62768, 466, 4002},
    /* 4135 : VPHADDUBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62782, 466, 4005},
    /* 4136 : VPHADDUDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 db /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(db) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62796, 466, 4008},
    /* 4137 : VPHADDUWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d6 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d6) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62810, 466, 4009},
    /* 4138 : VPHADDUWQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 d7 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(d7) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDUWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62824, 466, 4010},
    /* 4139 : VPHADDWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c6 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c6) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62838, 466, 4013},
    /* 4140 : VPHADDWQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 c7 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(c7) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHADDWQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62852, 466, 4016},
    /* 4141 : VPHSUBBW xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e1 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e1) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBBW, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62866, 466, 4018},
    /* 4142 : VPHSUBDQ xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e3 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e3) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBDQ, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62880, 466, 4019},
    /* 4143 : VPHSUBWD xmmreg,xmmrm128* [rm:       xop.m9.w0.l0.p0 e2 /r             ] SSE5,FUTURE,AMD */
        /* 270(111 000) 001(e2) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPHSUBWD, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+62894, 466, 4020},
    /* 4144 : VPMACSDD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 9e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(9e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53955, 466, 4022},
    /* 4145 : VPMACSDQH xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 9f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(9f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDQH, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53971, 466, 4025},
    /* 4146 : VPMACSDQL xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 97 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(97) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSDQL, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+53987, 466, 4026},
    /* 4147 : VPMACSSDD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 8e /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(8e) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54003, 466, 4027},
    /* 4148 : VPMACSSDQH xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 8f /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(8f) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDQH, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54019, 466, 4028},
    /* 4149 : VPMACSSDQL xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 87 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(87) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSDQL, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54035, 466, 4029},
    /* 4150 : VPMACSSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 86 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(86) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54051, 466, 4030},
    /* 4151 : VPMACSSWW xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 85 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(85) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSSWW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54067, 466, 4031},
    /* 4152 : VPMACSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 96 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(96) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54083, 466, 4032},
    /* 4153 : VPMACSWW xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 95 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(95) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMACSWW, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54099, 466, 4033},
    /* 4154 : VPMADCSSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a6 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a6) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMADCSSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54115, 466, 4034},
    /* 4155 : VPMADCSWD xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 b6 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(b6) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPMADCSWD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54131, 466, 4035},
    /* 4156 : VPPERM xmmreg,xmmreg*,xmmreg,xmmrm128 [rvsm:     xop.m8.w1.nds.l0.p0 a3 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 200) 001(a3) 130 176 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPPERM, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54147, 466, 4037},
    /* 4157 : VPPERM xmmreg,xmmreg*,xmmrm128,xmmreg [rvms:     xop.m8.w0.nds.l0.p0 a3 /r /is4    ] SSE5,FUTURE,AMD */
        /* 261(110 000) 001(a3) 120 177 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPPERM, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54163, 466, 4038},
    /* 4158 : VPROTB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 90 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(90) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62908, 466, 4040},
    /* 4159 : VPROTB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 90 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(90) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62922, 466, 4041},
    /* 4160 : VPROTB xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c0 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c0) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54179, 466, 4044},
    /* 4161 : VPROTD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 92 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(92) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62936, 466, 4046},
    /* 4162 : VPROTD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 92 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(92) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62950, 466, 4047},
    /* 4163 : VPROTD xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c2 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c2) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54195, 466, 4050},
    /* 4164 : VPROTQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 93 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(93) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62964, 466, 4051},
    /* 4165 : VPROTQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 93 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(93) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62978, 466, 4052},
    /* 4166 : VPROTQ xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c3 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c3) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54211, 466, 4055},
    /* 4167 : VPROTW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 91 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(91) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+62992, 466, 4056},
    /* 4168 : VPROTW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 91 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(91) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63006, 466, 4057},
    /* 4169 : VPROTW xmmreg,xmmrm128*,imm8 [rmi:      xop.m8.w0.l0.p0 c1 /r ib          ] SSE5,FUTURE,AMD */
        /* 270(110 000) 001(c1) 110 022 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPROTW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54227, 466, 4058},
    /* 4170 : VPSHAB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 98 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(98) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63020, 466, 4060},
    /* 4171 : VPSHAB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 98 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(98) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63034, 466, 4061},
    /* 4172 : VPSHAD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 9a /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(9a) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63048, 466, 4063},
    /* 4173 : VPSHAD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 9a /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(9a) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63062, 466, 4064},
    /* 4174 : VPSHAQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 9b /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(9b) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63076, 466, 4066},
    /* 4175 : VPSHAQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 9b /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(9b) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63090, 466, 4067},
    /* 4176 : VPSHAW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 99 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(99) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63104, 466, 4069},
    /* 4177 : VPSHAW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 99 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(99) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHAW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63118, 466, 4070},
    /* 4178 : VPSHLB xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 94 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(94) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63132, 466, 4072},
    /* 4179 : VPSHLB xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 94 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(94) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63146, 466, 4073},
    /* 4180 : VPSHLD xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 96 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(96) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLD, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63160, 466, 4077},
    /* 4181 : VPSHLD xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 96 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(96) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63174, 466, 4078},
    /* 4182 : VPSHLQ xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 97 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(97) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLQ, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63188, 466, 4080},
    /* 4183 : VPSHLQ xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 97 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(97) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63202, 466, 4081},
    /* 4184 : VPSHLW xmmreg,xmmrm128*,xmmreg [rmv:      xop.m9.w0.nds.l0.p0 95 /r         ] SSE5,FUTURE,AMD */
        /* 262(111 000) 001(95) 110 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLW, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63216, 466, 4083},
    /* 4185 : VPSHLW xmmreg,xmmreg*,xmmrm128 [rvm:      xop.m9.w1.nds.l0.p0 95 /r         ] SSE5,FUTURE,AMD */
        /* 261(111 200) 001(95) 120 : NOAPX,VEX,SSE5,FUTURE,AMD */
        {I_VPSHLW, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63230, 466, 4084},
    /* 4186 : VMPSADBW ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 42 /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(42) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VMPSADBW, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54243, 467, 4090},
    /* 4187 : VPABSB ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1c /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1c) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSB, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63244, 467, 4091},
    /* 4188 : VPABSW ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1d /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1d) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSW, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63251, 467, 4092},
    /* 4189 : VPABSD ymmreg,ymmrm256 [rm:       vex.256.66.0f38 1e /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(1e) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPABSD, 2, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63258, 467, 4093},
    /* 4190 : VPACKSSWB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 63 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(63) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKSSWB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63265, 467, 4094},
    /* 4191 : VPACKSSDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6b /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKSSDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63279, 467, 4095},
    /* 4192 : VPACKUSDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 2b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(2b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKUSDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63293, 467, 4096},
    /* 4193 : VPACKUSWB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 67 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(67) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPACKUSWB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63307, 467, 4097},
    /* 4194 : VPADDB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fc /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fc) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63321, 467, 4098},
    /* 4195 : VPADDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fd /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fd) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63335, 467, 4099},
    /* 4196 : VPADDD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fe /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fe) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63349, 467, 4100},
    /* 4197 : VPADDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63363, 467, 4101},
    /* 4198 : VPADDSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ec /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ec) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63377, 467, 4102},
    /* 4199 : VPADDSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ed /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ed) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63391, 467, 4103},
    /* 4200 : VPADDUSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f dc /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(dc) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDUSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63405, 467, 4104},
    /* 4201 : VPADDUSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f dd /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(dd) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPADDUSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63419, 467, 4105},
    /* 4202 : VPALIGNR ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0f /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(0f) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPALIGNR, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54259, 467, 4106},
    /* 4203 : VPAND ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f db /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(db) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAND, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63433, 467, 4107},
    /* 4204 : VPANDN ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f df /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(df) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPANDN, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63447, 467, 4108},
    /* 4205 : VPAVGB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e0 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e0) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAVGB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63461, 467, 4109},
    /* 4206 : VPAVGW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPAVGW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63475, 467, 4110},
    /* 4207 : VPBLENDVB ymmreg,ymmreg*,ymmrm256,ymmreg [rvms:     vex.nds.256.66.0f3a 4c /r /is4    ] AVX2,FUTURE */
        /* 261(003 005) 001(4c) 120 177 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPBLENDVB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,YMMREG|RN_L16,0}, NO_DECORATOR, nasm_bytecodes+54275, 467, 4111},
    /* 4208 : VPBLENDW ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a 0e /r ib      ] AVX2,FUTURE */
        /* 261(003 005) 001(0e) 120 023 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPBLENDW, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54291, 467, 4112},
    /* 4209 : VPCMPEQB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 74 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(74) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63489, 467, 4113},
    /* 4210 : VPCMPEQW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 75 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(75) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63503, 467, 4114},
    /* 4211 : VPCMPEQD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 76 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(76) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63517, 467, 4115},
    /* 4212 : VPCMPEQQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 29 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(29) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPEQQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63531, 467, 4116},
    /* 4213 : VPCMPGTB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 64 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(64) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63545, 467, 4117},
    /* 4214 : VPCMPGTW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 65 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(65) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63559, 467, 4118},
    /* 4215 : VPCMPGTD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 66 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(66) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63573, 467, 4119},
    /* 4216 : VPCMPGTQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 37 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(37) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPCMPGTQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63587, 467, 4120},
    /* 4217 : VPHADDW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 01 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(01) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63601, 467, 4121},
    /* 4218 : VPHADDD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 02 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(02) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63615, 467, 4122},
    /* 4219 : VPHADDSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 03 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(03) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHADDSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63629, 467, 4123},
    /* 4220 : VPHSUBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 05 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(05) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63643, 467, 4124},
    /* 4221 : VPHSUBD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 06 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(06) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63657, 467, 4125},
    /* 4222 : VPHSUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 07 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(07) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPHSUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63671, 467, 4126},
    /* 4223 : VPMADDUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 04 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(04) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMADDUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63685, 467, 4127},
    /* 4224 : VPMADDWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMADDWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63699, 467, 4128},
    /* 4225 : VPMAXSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3c /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3c) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63713, 467, 4129},
    /* 4226 : VPMAXSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ee /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ee) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63727, 467, 4130},
    /* 4227 : VPMAXSD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3d /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3d) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63741, 467, 4131},
    /* 4228 : VPMAXUB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f de /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(de) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63755, 467, 4132},
    /* 4229 : VPMAXUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3e /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3e) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63769, 467, 4133},
    /* 4230 : VPMAXUD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3f /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3f) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMAXUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63783, 467, 4134},
    /* 4231 : VPMINSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 38 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(38) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63797, 467, 4135},
    /* 4232 : VPMINSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ea /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ea) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63811, 467, 4136},
    /* 4233 : VPMINSD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 39 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(39) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63825, 467, 4137},
    /* 4234 : VPMINUB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f da /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(da) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63839, 467, 4138},
    /* 4235 : VPMINUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3a /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63853, 467, 4139},
    /* 4236 : VPMINUD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 3b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(3b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMINUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63867, 467, 4140},
    /* 4237 : VPMOVMSKB reg32,ymmreg [rm:       vex.256.66.0f d7 /r               ] AVX2,FUTURE */
        /* 270(001 005) 001(d7) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS32|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63881, 467, 4141},
    /* 4238 : VPMOVMSKB reg64,ymmreg [rm:       vex.256.66.0f d7 /r               ] AVX2,FUTURE */
        /* 270(001 005) 001(d7) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVMSKB, 2, {REG_GPR|BITS64|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63881, 467, 4142},
    /* 4239 : VPMOVSXBW ymmreg,xmmrm128 [rm:       vex.256.66.0f38 20 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(20) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBW, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63888, 467, 4143},
    /* 4240 : VPMOVSXBD ymmreg,mem64 [rm:       vex.256.66.0f38 21 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(21) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63895, 467, 4144},
    /* 4241 : VPMOVSXBD ymmreg,xmmreg [rm:       vex.256.66.0f38 21 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(21) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63895, 467, 4145},
    /* 4242 : VPMOVSXBQ ymmreg,mem32 [rm:       vex.256.66.0f38 22 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(22) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63902, 467, 4146},
    /* 4243 : VPMOVSXBQ ymmreg,xmmreg [rm:       vex.256.66.0f38 22 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(22) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63902, 467, 4147},
    /* 4244 : VPMOVSXWD ymmreg,xmmrm128 [rm:       vex.256.66.0f38 23 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(23) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63909, 467, 4148},
    /* 4245 : VPMOVSXWQ ymmreg,mem64 [rm:       vex.256.66.0f38 24 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(24) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63916, 467, 4149},
    /* 4246 : VPMOVSXWQ ymmreg,xmmreg [rm:       vex.256.66.0f38 24 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(24) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63916, 467, 4150},
    /* 4247 : VPMOVSXDQ ymmreg,xmmrm128 [rm:       vex.256.66.0f38 25 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(25) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVSXDQ, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63923, 467, 4151},
    /* 4248 : VPMOVZXBW ymmreg,xmmrm128 [rm:       vex.256.66.0f38 30 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(30) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBW, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63930, 467, 4152},
    /* 4249 : VPMOVZXBD ymmreg,mem64 [rm:       vex.256.66.0f38 31 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(31) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63937, 467, 4153},
    /* 4250 : VPMOVZXBD ymmreg,xmmreg [rm:       vex.256.66.0f38 31 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(31) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63937, 467, 4154},
    /* 4251 : VPMOVZXBQ ymmreg,mem32 [rm:       vex.256.66.0f38 32 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(32) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63944, 467, 4155},
    /* 4252 : VPMOVZXBQ ymmreg,xmmreg [rm:       vex.256.66.0f38 32 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(32) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63944, 467, 4156},
    /* 4253 : VPMOVZXWD ymmreg,xmmrm128 [rm:       vex.256.66.0f38 33 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(33) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWD, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63951, 467, 4157},
    /* 4254 : VPMOVZXWQ ymmreg,mem64 [rm:       vex.256.66.0f38 34 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(34) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63958, 467, 4158},
    /* 4255 : VPMOVZXWQ ymmreg,xmmreg [rm:       vex.256.66.0f38 34 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(34) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63958, 467, 4159},
    /* 4256 : VPMOVZXDQ ymmreg,xmmrm128 [rm:       vex.256.66.0f38 35 /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(35) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMOVZXDQ, 2, {YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+63965, 467, 4160},
    /* 4257 : VPMULDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 28 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(28) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63972, 467, 4161},
    /* 4258 : VPMULHRSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 0b /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(0b) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHRSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+63986, 467, 4162},
    /* 4259 : VPMULHUW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHUW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64000, 467, 4163},
    /* 4260 : VPMULHW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULHW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64014, 467, 4164},
    /* 4261 : VPMULLW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d5 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d5) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64028, 467, 4165},
    /* 4262 : VPMULLD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 40 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(40) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64042, 467, 4166},
    /* 4263 : VPMULUDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f4 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f4) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPMULUDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64056, 467, 4167},
    /* 4264 : VPOR ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f eb /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(eb) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPOR, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64070, 467, 4168},
    /* 4265 : VPSADBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f6 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f6) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSADBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64084, 467, 4169},
    /* 4266 : VPSHUFB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 00 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(00) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64098, 467, 4170},
    /* 4267 : VPSHUFD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 005) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54307, 467, 4171},
    /* 4268 : VPSHUFHW ymmreg,ymmrm256,imm8 [rmi:      vex.256.f3.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 006) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFHW, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54315, 467, 4172},
    /* 4269 : VPSHUFLW ymmreg,ymmrm256,imm8 [rmi:      vex.256.f2.0f 70 /r ib            ] AVX2,FUTURE */
        /* 270(001 007) 001(70) 110 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSHUFLW, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54323, 467, 4173},
    /* 4270 : VPSIGNB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 08 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(08) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGNB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64112, 467, 4174},
    /* 4271 : VPSIGNW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 09 /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(09) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGNW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64126, 467, 4175},
    /* 4272 : VPSIGND ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38 0a /r         ] AVX2,FUTURE */
        /* 261(002 005) 001(0a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSIGND, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64140, 467, 4176},
    /* 4273 : VPSLLDQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /7 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 217 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54331, 467, 4177},
    /* 4274 : VPSLLW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64154, 467, 4178},
    /* 4275 : VPSLLW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54347, 467, 4179},
    /* 4276 : VPSLLD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64168, 467, 4180},
    /* 4277 : VPSLLD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54363, 467, 4181},
    /* 4278 : VPSLLQ ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f f3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64182, 467, 4182},
    /* 4279 : VPSLLQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /6 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 216 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSLLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54379, 467, 4183},
    /* 4280 : VPSRAW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f e1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64196, 467, 4184},
    /* 4281 : VPSRAW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /4 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 214 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54395, 467, 4185},
    /* 4282 : VPSRAD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f e2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64210, 467, 4186},
    /* 4283 : VPSRAD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /4 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 214 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRAD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54411, 467, 4187},
    /* 4284 : VPSRLDQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 73 /3 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 213 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54427, 467, 4188},
    /* 4285 : VPSRLW ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d1 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d1) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64224, 467, 4189},
    /* 4286 : VPSRLW ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 71 /2 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(71) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54443, 467, 4190},
    /* 4287 : VPSRLD ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d2 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d2) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64238, 467, 4191},
    /* 4288 : VPSRLD ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f 72 /2 ib        ] AVX2,FUTURE */
        /* 260(001 005) 001(72) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54459, 467, 4192},
    /* 4289 : VPSRLQ ymmreg,ymmreg*,xmmrm128 [rvm:      vex.nds.256.66.0f d3 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d3) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64252, 467, 4193},
    /* 4290 : VPSRLQ ymmreg,ymmreg*,imm8 [vmi:      vex.ndd.256.66.0f.wig 73 /2 ib    ] AVX2,FUTURE */
        /* 260(001 005) 001(73) 212 022 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSRLQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54475, 467, 4194},
    /* 4291 : VPSUBB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64266, 467, 4195},
    /* 4292 : VPSUBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f f9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(f9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64280, 467, 4196},
    /* 4293 : VPSUBD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fa /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fa) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64294, 467, 4197},
    /* 4294 : VPSUBQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f fb /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(fb) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64308, 467, 4198},
    /* 4295 : VPSUBSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64322, 467, 4199},
    /* 4296 : VPSUBSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f e9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(e9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64336, 467, 4200},
    /* 4297 : VPSUBUSB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d8 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d8) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBUSB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64350, 467, 4201},
    /* 4298 : VPSUBUSW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f d9 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(d9) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPSUBUSW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64364, 467, 4202},
    /* 4299 : VPUNPCKHBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 68 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(68) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64378, 467, 4203},
    /* 4300 : VPUNPCKHWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 69 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(69) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64392, 467, 4204},
    /* 4301 : VPUNPCKHDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6a /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6a) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64406, 467, 4205},
    /* 4302 : VPUNPCKHQDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6d /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6d) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKHQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64420, 467, 4206},
    /* 4303 : VPUNPCKLBW ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 60 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(60) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLBW, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64434, 467, 4207},
    /* 4304 : VPUNPCKLWD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 61 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(61) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLWD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64448, 467, 4208},
    /* 4305 : VPUNPCKLDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 62 /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(62) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64462, 467, 4209},
    /* 4306 : VPUNPCKLQDQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f 6c /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(6c) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPUNPCKLQDQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64476, 467, 4210},
    /* 4307 : VPXOR ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f ef /r           ] AVX2,FUTURE */
        /* 261(001 005) 001(ef) 120 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VPXOR, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64490, 467, 4211},
    /* 4308 : VMOVNTDQA ymmreg,mem256 [rm:       vex.256.66.0f38 2a /r             ] AVX2,FUTURE */
        /* 270(002 005) 001(2a) 110 : NOAPX,WIG,VEX,AVX2,FUTURE */
        {I_VMOVNTDQA, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64504, 467, 4212},
    /* 4309 : VBROADCASTSS xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 18 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(18) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58764, 468, 4213},
    /* 4310 : VBROADCASTSS ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 18 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(18) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58771, 468, 4214},
    /* 4311 : VBROADCASTSD ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 19 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(19) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+58778, 468, 4215},
    /* 4312 : VBROADCASTI128 ymmreg,mem128 [rm:       vex.256.66.0f38.w0 5a /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(5a) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VBROADCASTI128, 2, {YMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64511, 468, 4216},
    /* 4313 : VPBLENDD xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 02 /r ib   ] AVX2,FUTURE */
        /* 261(003 001) 001(02) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBLENDD, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54491, 468, 4217},
    /* 4314 : VPBLENDD ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 02 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(02) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBLENDD, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54507, 468, 4218},
    /* 4315 : VPBROADCASTB xmmreg,mem8 [rm:       vex.128.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64518, 468, 4219},
    /* 4316 : VPBROADCASTB xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64518, 468, 4220},
    /* 4317 : VPBROADCASTB ymmreg,mem8 [rm:       vex.256.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG|RN_L16,MEMORY|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64525, 468, 4221},
    /* 4318 : VPBROADCASTB ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 78 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(78) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64525, 468, 4222},
    /* 4319 : VPBROADCASTW xmmreg,mem16 [rm:       vex.128.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64532, 468, 4223},
    /* 4320 : VPBROADCASTW xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64532, 468, 4224},
    /* 4321 : VPBROADCASTW ymmreg,mem16 [rm:       vex.256.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64539, 468, 4225},
    /* 4322 : VPBROADCASTW ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 79 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(79) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64539, 468, 4226},
    /* 4323 : VPBROADCASTD xmmreg,mem32 [rm:       vex.128.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64546, 468, 4227},
    /* 4324 : VPBROADCASTD xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64546, 468, 4228},
    /* 4325 : VPBROADCASTD ymmreg,mem32 [rm:       vex.256.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG|RN_L16,MEMORY|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64553, 468, 4229},
    /* 4326 : VPBROADCASTD ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 58 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(58) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64553, 468, 4230},
    /* 4327 : VPBROADCASTQ xmmreg,mem64 [rm:       vex.128.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64560, 468, 4231},
    /* 4328 : VPBROADCASTQ xmmreg,xmmreg [rm:       vex.128.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 001) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64560, 468, 4232},
    /* 4329 : VPBROADCASTQ ymmreg,mem64 [rm:       vex.256.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG|RN_L16,MEMORY|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64567, 468, 4233},
    /* 4330 : VPBROADCASTQ ymmreg,xmmreg [rm:       vex.256.66.0f38.w0 59 /r          ] AVX2,FUTURE */
        /* 270(002 005) 001(59) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64567, 468, 4234},
    /* 4331 : VPERMD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 36 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(36) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64574, 468, 4236},
    /* 4332 : VPERMPD ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w1 01 /r ib       ] AVX2,FUTURE */
        /* 270(003 205) 001(01) 110 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMPD, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54523, 468, 4237},
    /* 4333 : VPERMPS ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 16 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(16) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMPS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64588, 468, 4238},
    /* 4334 : VPERMQ ymmreg,ymmrm256,imm8 [rmi:      vex.256.66.0f3a.w1 00 /r ib       ] AVX2,FUTURE */
        /* 270(003 205) 001(00) 110 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERMQ, 3, {YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54531, 468, 4239},
    /* 4335 : VPERM2I128 ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 46 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(46) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPERM2I128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54539, 468, 4240},
    /* 4336 : VEXTRACTI128 xmmrm128,ymmreg,imm8 [mri:      vex.256.66.0f3a.w0 39 /r ib       ] AVX2,FUTURE */
        /* 270(003 005) 001(39) 101 022 : NOAPX,VEX,AVX2,FUTURE */
        {I_VEXTRACTI128, 3, {RM_XMM|BITS128|RN_L16,YMMREG|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54555, 468, 4241},
    /* 4337 : VINSERTI128 ymmreg,ymmreg*,xmmrm128,imm8 [rvmi:     vex.nds.256.66.0f3a.w0 38 /r ib   ] AVX2,FUTURE */
        /* 261(003 005) 001(38) 120 023 : NOAPX,VEX,AVX2,FUTURE */
        {I_VINSERTI128, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54563, 468, 4243},
    /* 4338 : VPMASKMOVD xmmreg,xmmreg*,mem128 [rvm:      vex.nds.128.66.0f38.w0 8c /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64602, 468, 4244},
    /* 4339 : VPMASKMOVD ymmreg,ymmreg*,mem256 [rvm:      vex.nds.256.66.0f38.w0 8c /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64616, 468, 4245},
    /* 4340 : VPMASKMOVQ xmmreg,xmmreg*,mem128 [rvm:      vex.nds.128.66.0f38.w1 8c /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64630, 468, 4246},
    /* 4341 : VPMASKMOVQ ymmreg,ymmreg*,mem256 [rvm:      vex.nds.256.66.0f38.w1 8c /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(8c) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64644, 468, 4247},
    /* 4342 : VPMASKMOVD mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w0 8e /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64658, 468, 4249},
    /* 4343 : VPMASKMOVD mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w0 8e /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVD, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64665, 468, 4250},
    /* 4344 : VPMASKMOVQ mem128,xmmreg,xmmreg [mvr:      vex.nds.128.66.0f38.w1 8e /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {MEMORY|BITS128|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64672, 468, 4251},
    /* 4345 : VPMASKMOVQ mem256,ymmreg,ymmreg [mvr:      vex.nds.256.66.0f38.w1 8e /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(8e) 102 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPMASKMOVQ, 3, {MEMORY|BITS256|RN_L16,YMMREG|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64679, 468, 4252},
    /* 4346 : VPSLLVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 47 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64686, 468, 4254},
    /* 4347 : VPSLLVQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w1 47 /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64700, 468, 4255},
    /* 4348 : VPSLLVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 47 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64714, 468, 4256},
    /* 4349 : VPSLLVQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w1 47 /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(47) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSLLVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64728, 468, 4257},
    /* 4350 : VPSRAVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 46 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(46) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRAVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64742, 468, 4259},
    /* 4351 : VPSRAVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 46 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(46) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRAVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64756, 468, 4260},
    /* 4352 : VPSRLVD xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 45 /r      ] AVX2,FUTURE */
        /* 261(002 001) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64770, 468, 4262},
    /* 4353 : VPSRLVQ xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w1 45 /r      ] AVX2,FUTURE */
        /* 261(002 201) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64784, 468, 4263},
    /* 4354 : VPSRLVD ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 45 /r      ] AVX2,FUTURE */
        /* 261(002 005) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64798, 468, 4264},
    /* 4355 : VPSRLVQ ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w1 45 /r      ] AVX2,FUTURE */
        /* 261(002 205) 001(45) 120 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPSRLVQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64812, 468, 4265},
    /* 4356 : VGATHERDPD xmmreg,xmem64,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w1 92 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPD, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54579, 468, 4267},
    /* 4357 : VGATHERQPD xmmreg,xmem64,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w1 93 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPD, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54587, 468, 4268},
    /* 4358 : VGATHERDPD ymmreg,xmem64,ymmreg [rmv:      vm32x vex.dds.256.66.0f38.w1 92 /r] AVX2,FUTURE */
        /* 374 262(002 205) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPD, 3, {YMMREG|RN_L16,XMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54595, 468, 4269},
    /* 4359 : VGATHERQPD ymmreg,ymem64,ymmreg [rmv:      vm64y vex.dds.256.66.0f38.w1 93 /r] AVX2,FUTURE */
        /* 375 262(002 205) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPD, 3, {YMMREG|RN_L16,YMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54603, 468, 4270},
    /* 4360 : VGATHERDPS xmmreg,xmem32,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w0 92 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPS, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54611, 468, 4272},
    /* 4361 : VGATHERQPS xmmreg,xmem32,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w0 93 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPS, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54619, 468, 4273},
    /* 4362 : VGATHERDPS ymmreg,ymem32,ymmreg [rmv:      vm32y vex.dds.256.66.0f38.w0 92 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(92) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERDPS, 3, {YMMREG|RN_L16,YMEM|BITS32|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54627, 468, 4274},
    /* 4363 : VGATHERQPS xmmreg,ymem32,xmmreg [rmv:      vm64y vex.dds.256.66.0f38.w0 93 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(93) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VGATHERQPS, 3, {XMMREG|RN_L16,YMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54635, 468, 4275},
    /* 4364 : VPGATHERDD xmmreg,xmem32,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w0 90 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDD, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54643, 468, 4277},
    /* 4365 : VPGATHERQD xmmreg,xmem32,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w0 91 /r] AVX2,FUTURE */
        /* 374 262(002 001) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQD, 3, {XMMREG|RN_L16,XMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54651, 468, 4278},
    /* 4366 : VPGATHERDD ymmreg,ymem32,ymmreg [rmv:      vm32y vex.dds.256.66.0f38.w0 90 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDD, 3, {YMMREG|RN_L16,YMEM|BITS32|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54659, 468, 4279},
    /* 4367 : VPGATHERQD xmmreg,ymem32,xmmreg [rmv:      vm64y vex.dds.256.66.0f38.w0 91 /r] AVX2,FUTURE */
        /* 375 262(002 005) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQD, 3, {XMMREG|RN_L16,YMEM|BITS32|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54667, 468, 4280},
    /* 4368 : VPGATHERDQ xmmreg,xmem64,xmmreg [rmv:      vm32x vex.dds.128.66.0f38.w1 90 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDQ, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54675, 468, 4282},
    /* 4369 : VPGATHERQQ xmmreg,xmem64,xmmreg [rmv:      vm64x vex.dds.128.66.0f38.w1 91 /r] AVX2,FUTURE */
        /* 374 262(002 201) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQQ, 3, {XMMREG|RN_L16,XMEM|BITS64|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54683, 468, 4283},
    /* 4370 : VPGATHERDQ ymmreg,xmem64,ymmreg [rmv:      vm32x vex.dds.256.66.0f38.w1 90 /r] AVX2,FUTURE */
        /* 374 262(002 205) 001(90) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERDQ, 3, {YMMREG|RN_L16,XMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54691, 468, 4284},
    /* 4371 : VPGATHERQQ ymmreg,ymem64,ymmreg [rmv:      vm64y vex.dds.256.66.0f38.w1 91 /r] AVX2,FUTURE */
        /* 375 262(002 205) 001(91) 110 : NOAPX,VEX,AVX2,FUTURE */
        {I_VPGATHERQQ, 3, {YMMREG|RN_L16,YMEM|BITS64|RN_L16,YMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+54699, 468, 4285},
    /* 4372 : XABORT imm8 [i:        c6 f8 ib                          ] RTM,FUTURE */
        /* 002(c6 f8) 020 : RTM,FUTURE */
        {I_XABORT, 1, {IMM_NORMAL|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72519, 469, 4288},
    /* 4373 : XBEGIN imm16|near [i:        nw o16 c7 f8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        /* 327 320 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS16|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64826, 470, 4289},
    /* 4374 : XBEGIN imm32|near [i:        nw o32 c7 f8 rel                  ] AR0,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        /* 327 321 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,NOREX,NOAPX,NOLONG,RTM,FUTURE */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS32|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64833, 470, 4290},
    /* 4375 : XBEGIN imm64|near [i:        nw o64 c7 f8 rel                  ] AR0,OSIZE,LONG,PROT,RTM,X86_64 */
        /* 327 324 002(c7 f8) 064 : AR0,NWSIZE,OSIZE,LONG,PROT,RTM,X86_64 */
        {I_XBEGIN, 1, {IMM_NORMAL|BITS64|NEAR,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+64840, 471, 4291},
    /* 4376 : XEND void [          0f 01 d5                          ] RTM,FUTURE */
        /* 355 002(01 d5) : RTM,FUTURE */
        {I_XEND, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72524, 469, 4293},
    /* 4377 : XTEST void [          0f 01 d6                          ] HLE,FL,RTM,FUTURE */
        /* 355 002(01 d6) : HLE,FL,RTM,FUTURE */
        {I_XTEST, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72529, 473, 4294},
    /* 4378 : PREFETCHWT1 mem8 [m:        0f 0d /2                          ] PREFETCHWT1,FUTURE */
        /* 355 001(0d) 202 : PREFETCHWT1,FUTURE */
        {I_PREFETCHWT1, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+72534, 474, 4296},
    /* 4379 : BNDMK bndreg,mem [rm:       f3 0f 1b /r                       ] MIB,MPX,FUTURE */
        /* 333 355 001(1b) 110 : MIB,MPX,FUTURE */
        {I_BNDMK, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+70023, 475, 4299},
    /* 4380 : BNDCL bndreg,mem [rm:       f3 0f 1a /r                       ] MPX,FUTURE */
        /* 333 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDCL, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64848, 476, 4300},
    /* 4381 : BNDCL bndreg,reg32 [rm:       f3 0f 1a /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 333 355 001(1a) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCL, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64848, 477, 4301},
    /* 4382 : BNDCL bndreg,reg64 [rm:       o64nw f3 0f 1a /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 333 355 001(1a) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCL, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64847, 478, 4302},
    /* 4383 : BNDCU bndreg,mem [rm:       f2 0f 1a /r                       ] MPX,FUTURE */
        /* 332 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDCU, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64855, 476, 4303},
    /* 4384 : BNDCU bndreg,reg32 [rm:       f2 0f 1a /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 332 355 001(1a) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCU, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64855, 477, 4304},
    /* 4385 : BNDCU bndreg,reg64 [rm:       o64nw f2 0f 1a /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 332 355 001(1a) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCU, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64854, 478, 4305},
    /* 4386 : BNDCN bndreg,mem [rm:       f2 0f 1b /r                       ] MPX,FUTURE */
        /* 332 355 001(1b) 110 : MPX,FUTURE */
        {I_BNDCN, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+64862, 476, 4306},
    /* 4387 : BNDCN bndreg,reg32 [rm:       f2 0f 1b /r                       ] NOREX,NOAPX,NOLONG,MPX,FUTURE */
        /* 332 355 001(1b) 110 : NOREX,NOAPX,NOLONG,MPX,FUTURE */
        {I_BNDCN, 2, {BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64862, 477, 4307},
    /* 4388 : BNDCN bndreg,reg64 [rm:       o64nw f2 0f 1b /r                 ] LONG,PROT,MPX,X86_64 */
        /* 323 332 355 001(1b) 110 : NWSIZE,LONG,PROT,MPX,X86_64 */
        {I_BNDCN, 2, {BNDREG,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+64861, 478, 4308},
    /* 4389 : BNDMOV bndreg,bndreg [rm:       66 0f 1a /r                       ] MPX,FUTURE */
        /* 361 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+70029, 476, 4309},
    /* 4390 : BNDMOV bndreg,mem [rm:       66 0f 1a /r                       ] MPX,FUTURE */
        /* 361 355 001(1a) 110 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+70029, 476, 4310},
    /* 4391 : BNDMOV bndreg,bndreg [mr:       66 0f 1b /r                       ] MPX,FUTURE */
        /* 361 355 001(1b) 101 : MPX,FUTURE */
        {I_BNDMOV, 2, {BNDREG,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+70035, 476, 4311},
    /* 4392 : BNDMOV mem,bndreg [mr:       66 0f 1b /r                       ] MPX,FUTURE */
        /* 361 355 001(1b) 101 : MPX,FUTURE */
        {I_BNDMOV, 2, {MEMORY,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+70035, 476, 4312},
    /* 4393 : BNDLDX bndreg,mem [rm:       0f 1a /r                          ] MIB,MPX,FUTURE */
        /* 355 001(1a) 110 : MIB,MPX,FUTURE */
        {I_BNDLDX, 2, {BNDREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+70030, 475, 4313},
    /* 4394 : BNDLDX bndreg,mem,reg32 [rmx:      0f 1a /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1a) 016 110 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDLDX, 3, {BNDREG|RN_L16,MEMORY|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+70041, 479, 4314},
    /* 4395 : BNDLDX bndreg,mem,reg64 [rmx:      0f 1a /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1a) 016 110 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDLDX, 3, {BNDREG,MEMORY,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+70041, 480, 4315},
    /* 4396 : BNDSTX mem,bndreg [mr:       0f 1b /r                          ] MIB,MPX,FUTURE */
        /* 355 001(1b) 101 : MIB,MPX,FUTURE */
        {I_BNDSTX, 2, {MEMORY,BNDREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+70036, 475, 4316},
    /* 4397 : BNDSTX mem,reg32,bndreg [mxr:      0f 1b /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1b) 015 102 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDSTX, 3, {MEMORY|RN_L16,REG_GPR|BITS32|RN_L16,BNDREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+70047, 479, 4317},
    /* 4398 : BNDSTX mem,reg64,bndreg [mxr:      0f 1b /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1b) 015 102 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDSTX, 3, {MEMORY,REG_GPR|BITS64,BNDREG,0,0}, NO_DECORATOR, nasm_bytecodes+70047, 480, 4318},
    /* 4399 : BNDSTX mem,bndreg,reg32 [mrx:      0f 1b /r                          ] NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        /* 355 001(1b) 016 101 : NOREX,NOAPX,NOLONG,MIB,MPX,FUTURE */
        {I_BNDSTX, 3, {MEMORY|RN_L16,BNDREG|RN_L16,REG_GPR|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+70053, 479, 4319},
    /* 4400 : BNDSTX mem,bndreg,reg64 [mrx:      0f 1b /r                          ] LONG,MIB,PROT,MPX,X86_64 */
        /* 355 001(1b) 016 101 : LONG,MIB,PROT,MPX,X86_64 */
        {I_BNDSTX, 3, {MEMORY,BNDREG,REG_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+70053, 480, 4320},
    /* 4401 : SHA1MSG1 xmmreg,xmmrm128 [rm:       0f38 c9 /r                        ] SHA,FUTURE */
        /* 356 001(c9) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1MSG1, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72539, 481, 4323},
    /* 4402 : SHA1MSG2 xmmreg,xmmrm128 [rm:       0f38 ca /r                        ] SHA,FUTURE */
        /* 356 001(ca) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1MSG2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72544, 481, 4324},
    /* 4403 : SHA1NEXTE xmmreg,xmmrm128 [rm:       0f38 c8 /r                        ] SHA,FUTURE */
        /* 356 001(c8) 110 : NOAPX,SHA,FUTURE */
        {I_SHA1NEXTE, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72549, 481, 4325},
    /* 4404 : SHA1RNDS4 xmmreg,xmmrm128,imm8 [rmi:      0f3a cc /r ib                     ] SHA,FUTURE */
        /* 357 001(cc) 110 022 : NOAPX,SHA,FUTURE */
        {I_SHA1RNDS4, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+70059, 481, 4326},
    /* 4405 : SHA256MSG1 xmmreg,xmmrm128 [rm:       0f38 cc /r                        ] SHA,FUTURE */
        /* 356 001(cc) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256MSG1, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72554, 481, 4327},
    /* 4406 : SHA256MSG2 xmmreg,xmmrm128 [rm:       0f38 cd /r                        ] SHA,FUTURE */
        /* 356 001(cd) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256MSG2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72559, 481, 4328},
    /* 4407 : SHA256RNDS2 xmmreg,xmmrm128,xmm0 [rm-:      0f38 cb /r                        ] SHA,FUTURE */
        /* 356 001(cb) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256RNDS2, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,XMMREG|RN_ZERO|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+72564, 481, 4329},
    /* 4408 : SHA256RNDS2 xmmreg,xmmrm128 [rm:       0f38 cb /r                        ] SHA,FUTURE */
        /* 356 001(cb) 110 : NOAPX,SHA,FUTURE */
        {I_SHA256RNDS2, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+72564, 481, 4330},
    /* 4409 : VSHA512MSG1 ymmreg,xmmreg [rm:       vex.256.f2.0f38.w0 cc /r          ] AVX,SHA512,FUTURE */
        /* 270(002 007) 001(cc) 110 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512MSG1, 2, {YMMREG|RN_L16,XMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64868, 482, 4331},
    /* 4410 : VSHA512MSG2 ymmreg,ymmreg [rm:       vex.256.f2.0f38.w0 cd /r          ] AVX,SHA512,FUTURE */
        /* 270(002 007) 001(cd) 110 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512MSG2, 2, {YMMREG|RN_L16,YMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64875, 482, 4332},
    /* 4411 : VSHA512RNDS2 ymmreg,ymmreg,xmmreg [rvm:      vex.nds.256.f2.0f38.w0 cb /r      ] AVX,SHA512,FUTURE */
        /* 261(002 007) 001(cb) 120 : NOAPX,VEX,AVX,SHA512,FUTURE */
        {I_VSHA512RNDS2, 3, {YMMREG|RN_L16,YMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64882, 482, 4333},
    /* 4412 : VSM3MSG1 xmmreg,xmmreg,xmmreg [rvm:      vex.nds.128.p0.0f38.w0 da /r      ] AVX,HSM3,FUTURE */
        /* 261(002 000) 001(da) 120 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3MSG1, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64889, 483, 4336},
    /* 4413 : VSM3MSG2 xmmreg,xmmreg,xmmreg [rvm:      vex.nds.128.66.0f38.w0 da /r      ] AVX,HSM3,FUTURE */
        /* 261(002 001) 001(da) 120 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3MSG2, 3, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64896, 483, 4337},
    /* 4414 : VSM3RNDS2 xmmreg,xmmreg,xmmreg,imm8 [rvmi:     vex.nds.128.66.0f3a.w0 de /r ib   ] AVX,HSM3,FUTURE */
        /* 261(003 001) 001(de) 120 023 : NOAPX,VEX,AVX,HSM3,FUTURE */
        {I_VSM3RNDS2, 4, {XMMREG|RN_L16,XMMREG|RN_L16,XMMREG|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54707, 483, 4338},
    /* 4415 : VSM4KEY4 xmmreg,xmmreg,xmmrm128 [rvm:      vex.nds.128.f3.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 002) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64903, 484, 4341},
    /* 4416 : VSM4KEY4 ymmreg,ymmreg,ymmrm256 [rvm:      vex.nds.256.f3.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 006) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64910, 484, 4342},
    /* 4417 : VSM4KEY4 xmmreg,xmmreg,xmmrm128 [rvm:      evex.nds.128.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 08) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+29560, 485, 4343},
    /* 4418 : VSM4KEY4 ymmreg,ymmreg,ymmrm256 [rvm:      evex.nds.256.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 28) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+29569, 485, 4344},
    /* 4419 : VSM4KEY4 zmmreg,zmmreg,zmmrm512 [rvm:      evex.nds.512.f3.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7e 48) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4KEY4, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+29578, 485, 4345},
    /* 4420 : VSM4RNDS4 xmmreg,xmmreg,xmmrm128 [rvm:      vex.nds.128.f2.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 003) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64917, 484, 4346},
    /* 4421 : VSM4RNDS4 ymmreg,ymmreg,ymmrm256 [rvm:      vex.nds.256.f2.0f38.w0 da /r      ] AVX,HSM4,FUTURE */
        /* 261(002 007) 001(da) 120 : NOAPX,VEX,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+64924, 484, 4347},
    /* 4422 : VSM4RNDS4 xmmreg,xmmreg,xmmrm128 [rvm:      evex.nds.128.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 08) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+29587, 485, 4348},
    /* 4423 : VSM4RNDS4 ymmreg,ymmreg,ymmrm256 [rvm:      evex.nds.256.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 28) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+29596, 485, 4349},
    /* 4424 : VSM4RNDS4 zmmreg,zmmreg,zmmrm512 [rvm:      evex.nds.512.f2.0f38.w0 da /r     ] AVX,HSM4,FUTURE */
        /* 241(f2 7f 48) 300 001(da) 120 : EVEX,PROT,AVX,HSM4,FUTURE */
        {I_VSM4RNDS4, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+29605, 485, 4350},
    /* 4425 : VBCSTNEBF16PS xmmreg,mem16 [rm:       vex.128.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF16PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64931, 486, 4354},
    /* 4426 : VBCSTNEBF16PS ymmreg,mem16 [rm:       vex.256.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF16PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64938, 486, 4355},
    /* 4427 : VBCSTNEBF162PS xmmreg,mem16 [rm:       vex.128.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64931, 486, 4356},
    /* 4428 : VBCSTNEBF162PS ymmreg,mem16 [rm:       vex.256.f3.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNEBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64938, 486, 4357},
    /* 4429 : VBCSTNESH2PS xmmreg,mem16 [rm:       vex.128.66.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 001) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNESH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64945, 486, 4358},
    /* 4430 : VBCSTNESH2PS ymmreg,mem16 [rm:       vex.256.66.0f38.w0 b1 /r          ] AR0-1,SW,AVXNECONVERT,FUTURE */
        /* 270(002 005) 001(b1) 110 : AR0,AR1,SW,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VBCSTNESH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64952, 486, 4359},
    /* 4431 : VCVTNEEBF162PS xmmreg,mem128 [rm:       vex.128.f3.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64959, 487, 4360},
    /* 4432 : VCVTNEEBF162PS ymmreg,mem256 [rm:       vex.256.f3.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64966, 488, 4361},
    /* 4433 : VCVTNEEPH2PS xmmreg,mem128 [rm:       vex.128.66.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 001) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEPH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64973, 487, 4362},
    /* 4434 : VCVTNEEPH2PS ymmreg,mem256 [rm:       vex.256.66.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 005) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEEPH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64980, 488, 4363},
    /* 4435 : VCVTNEOBF162PS xmmreg,mem128 [rm:       vex.128.f2.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 003) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOBF162PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64987, 487, 4364},
    /* 4436 : VCVTNEOBF162PS ymmreg,mem256 [rm:       vex.256.f2.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 007) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOBF162PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+64994, 488, 4365},
    /* 4437 : VCVTNEOPH2PS xmmreg,mem128 [rm:       vex.128.np.0f38.w0 b0 /r          ] AR0-1,SO,AVXNECONVERT,FUTURE */
        /* 270(002 000) 001(b0) 110 : AR0,AR1,SO,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOPH2PS, 2, {XMMREG|RN_L16,MEMORY|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65001, 487, 4366},
    /* 4438 : VCVTNEOPH2PS ymmreg,mem256 [rm:       vex.256.np.0f38.w0 b0 /r          ] AR0-1,SY,AVXNECONVERT,FUTURE */
        /* 270(002 004) 001(b0) 110 : AR0,AR1,SY,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEOPH2PS, 2, {YMMREG|RN_L16,MEMORY|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65008, 488, 4367},
    /* 4439 : VCVTNEPS2BF16 xmmreg,xmmrm128 [rm:       vex.128.f3.0f38.w0 72 /r          ] AR0-1,SO,LATEVEX,AVXNECONVERT,FUTURE */
        /* 270(002 002) 001(72) 110 : AR0,AR1,SO,LATEVEX,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65015, 489, 4368},
    /* 4440 : VCVTNEPS2BF16 xmmreg,ymmrm256 [rm:       vex.256.f3.0f38.w0 72 /r          ] AR0-1,SY,LATEVEX,AVXNECONVERT,FUTURE */
        /* 270(002 006) 001(72) 110 : AR0,AR1,SY,LATEVEX,NOAPX,VEX,AVXNECONVERT,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65022, 490, 4369},
    /* 4441 : VPDPBUSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65029, 491, 4373},
    /* 4442 : VPDPBUSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65036, 492, 4374},
    /* 4443 : VPDPBUSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65043, 491, 4375},
    /* 4444 : VPDPBUSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPBUSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65050, 492, 4376},
    /* 4445 : VPDPWSSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 52 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(52) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65057, 491, 4377},
    /* 4446 : VPDPWSSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 52 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(52) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65064, 492, 4378},
    /* 4447 : VPDPWSSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 53 /r          ] AR0-2,SO,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 001) 001(53) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65071, 491, 4379},
    /* 4448 : VPDPWSSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 53 /r          ] AR0-2,SY,LATEVEX,AVXVNNI,FUTURE */
        /* 261(002 005) 001(53) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXVNNI,FUTURE */
        {I_VPDPWSSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65078, 492, 4380},
    /* 4449 : VPDPBSSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f2.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 003) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65085, 493, 4383},
    /* 4450 : VPDPBSSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f2.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 007) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65092, 494, 4384},
    /* 4451 : VPDPBSSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f2.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 003) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65099, 493, 4385},
    /* 4452 : VPDPBSSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f2.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 007) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65106, 494, 4386},
    /* 4453 : VPDPBSUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 002) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65113, 493, 4387},
    /* 4454 : VPDPBSUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 006) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65120, 494, 4388},
    /* 4455 : VPDPBSUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 002) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65127, 493, 4389},
    /* 4456 : VPDPBSUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 006) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBSUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65134, 494, 4390},
    /* 4457 : VPDPBUUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 50 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 000) 001(50) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65141, 493, 4391},
    /* 4458 : VPDPBUUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 50 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 004) 001(50) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65148, 494, 4392},
    /* 4459 : VPDPBUUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 51 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 000) 001(51) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65155, 493, 4393},
    /* 4460 : VPDPBUUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 51 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT8,FUTURE */
        /* 261(002 004) 001(51) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT8,FUTURE */
        {I_VPDPBUUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65162, 494, 4394},
    /* 4461 : VPDPWSUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 002) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65169, 495, 4397},
    /* 4462 : VPDPWSUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 006) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65176, 496, 4398},
    /* 4463 : VPDPWSUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.f3.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 002) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65183, 495, 4399},
    /* 4464 : VPDPWSUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.f3.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 006) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWSUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65190, 496, 4400},
    /* 4465 : VPDPWUSD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 001) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65197, 495, 4401},
    /* 4466 : VPDPWUSD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 005) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65204, 496, 4402},
    /* 4467 : VPDPWUSDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 001) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65211, 495, 4403},
    /* 4468 : VPDPWUSDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 005) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUSDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65218, 496, 4404},
    /* 4469 : VPDPWUUD xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 d2 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 000) 001(d2) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUD, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65225, 495, 4405},
    /* 4470 : VPDPWUUD ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 d2 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 004) 001(d2) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUD, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65232, 496, 4406},
    /* 4471 : VPDPWUUDS xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.np.0f38.w0 d3 /r          ] AR0-2,SO,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 000) 001(d3) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUDS, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65239, 495, 4407},
    /* 4472 : VPDPWUUDS ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.np.0f38.w0 d3 /r          ] AR0-2,SY,LATEVEX,AVX,AVXVNNIINT16,FUTURE */
        /* 261(002 004) 001(d3) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVX,AVXVNNIINT16,FUTURE */
        {I_VPDPWUUDS, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65246, 496, 4408},
    /* 4473 : VPMADD52HUQ xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w1 b5 /r          ] AR0-2,SO,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 201) 001(b5) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65253, 497, 4412},
    /* 4474 : VPMADD52HUQ ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w1 b5 /r          ] AR0-2,SY,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 205) 001(b5) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65260, 498, 4413},
    /* 4475 : VPMADD52LUQ xmmreg,xmmreg,xmmrm128 [rvm:      vex.128.66.0f38.w1 b4 /r          ] AR0-2,SO,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 201) 001(b4) 120 : AR0,AR1,AR2,SO,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65267, 497, 4414},
    /* 4476 : VPMADD52LUQ ymmreg,ymmreg,ymmrm256 [rvm:      vex.256.66.0f38.w1 b4 /r          ] AR0-2,SY,LATEVEX,AVXIFMA,FUTURE */
        /* 261(002 205) 001(b4) 120 : AR0,AR1,AR2,SY,LATEVEX,NOAPX,VEX,AVXIFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65274, 498, 4415},
    /* 4477 : KMOVB kreg8,krm8 [rm:       vex.l0.0f.ko8 90 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(90) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8|RN_L16,RM_OPMASK|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65281, 499, 4418},
    /* 4478 : KMOVB kreg8,krm8 [rm:       evex.l0.0f.ko8 90 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8,RM_OPMASK|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29614, 500, 4418},
    /* 4479 : KMOVW kreg16,krm16 [rm:       vex.l0.0f.ko16 90 /r             ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(90) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16|RN_L16,RM_OPMASK|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65288, 501, 4419},
    /* 4480 : KMOVW kreg16,krm16 [rm:       evex.l0.0f.ko16 90 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16,RM_OPMASK|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29623, 502, 4419},
    /* 4481 : KMOVD kreg32,krm32 [rm:       vex.l0.0f.ko32 90 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(90) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32|RN_L16,RM_OPMASK|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65295, 503, 4420},
    /* 4482 : KMOVD kreg32,krm32 [rm:       evex.l0.0f.ko32 90 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32,RM_OPMASK|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29632, 504, 4420},
    /* 4483 : KMOVQ kreg64,krm64 [rm:       vex.l0.0f.ko64 90 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(90) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVQ, 2, {OPMASKREG|BITS64|RN_L16,RM_OPMASK|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65302, 503, 4421},
    /* 4484 : KMOVQ kreg64,krm64 [rm:       evex.l0.0f.ko64 90 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(90) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVQ, 2, {OPMASKREG|BITS64,RM_OPMASK|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29641, 504, 4421},
    /* 4485 : KMOVB mem8,kreg8 [mr:       vex.l0.0f.ko8 91 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(91) 101 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {MEMORY|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65309, 499, 4434},
    /* 4486 : KMOVB mem8,kreg8 [mr:       evex.l0.0f.ko8 91 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {MEMORY|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29650, 500, 4434},
    /* 4487 : KMOVW mem16,kreg16 [mr:       vex.l0.0f.ko16 91 /r             ] ZU,AVX512DQ,FUTURE */
        /* 270(001 000) 001(91) 101 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVW, 2, {MEMORY|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65316, 499, 4435},
    /* 4488 : KMOVW mem16,kreg16 [mr:       evex.l0.0f.ko16 91 /r             ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVW, 2, {MEMORY|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29659, 500, 4435},
    /* 4489 : KMOVD mem32,kreg32 [mr:       vex.l0.0f.ko32 91 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(91) 101 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {MEMORY|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65323, 503, 4436},
    /* 4490 : KMOVD mem32,kreg32 [mr:       evex.l0.0f.ko32 91 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {MEMORY|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29668, 504, 4436},
    /* 4491 : KMOVQ mem64,kreg64 [mr:       vex.l0.0f.ko64 91 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(91) 101 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVQ, 2, {MEMORY|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65330, 503, 4437},
    /* 4492 : KMOVQ mem64,kreg64 [mr:       evex.l0.0f.ko64 91 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(91) 101 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVQ, 2, {MEMORY|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29677, 504, 4437},
    /* 4493 : KMOVD kreg32,reg32 [rm:       vex.l0.0f.f2.w0 92 /r            ] ZU,AVX512BW,FUTURE */
        /* 270(001 003) 001(92) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65337, 503, 4450},
    /* 4494 : KMOVD kreg32,reg32 [rm:       evex.l0.0f.f2.w0 92 /r            ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 7f 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {OPMASKREG|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29686, 504, 4450},
    /* 4495 : KMOVQ kreg64,reg64 [rm:       vex.l0.0f.f2.w1 92 /r            ] ZU,LONG,PROT,AVX512BW,X86_64 */
        /* 270(001 203) 001(92) 110 : NOAPX,ZU,LONG,VEX,PROT,AVX512BW,X86_64 */
        {I_KMOVQ, 2, {OPMASKREG|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65344, 505, 4451},
    /* 4496 : KMOVQ kreg64,reg64 [rm:       evex.l0.0f.f2.w1 92 /r            ] ZU,LONG,PROT,AVX512BW,X86_64,APX */
        /* 250(f1 ff 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,X86_64 */
        {I_KMOVQ, 2, {OPMASKREG|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29695, 506, 4451},
    /* 4497 : KMOVD reg32,kreg32 [rm:       vex.l0.0f.f2.w0 93 /r            ] ZU,AVX512BW,FUTURE */
        /* 270(001 003) 001(93) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KMOVD, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65351, 503, 4458},
    /* 4498 : KMOVD reg32,kreg32 [rm:       evex.l0.0f.f2.w0 93 /r            ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 7f 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KMOVD, 2, {REG_GPR|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29704, 504, 4458},
    /* 4499 : KMOVQ reg64,kreg64 [rm:       vex.l0.0f.f2.w1 93 /r            ] ZU,LONG,PROT,AVX512BW,X86_64 */
        /* 270(001 203) 001(93) 110 : NOAPX,ZU,LONG,VEX,PROT,AVX512BW,X86_64 */
        {I_KMOVQ, 2, {REG_GPR|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65358, 505, 4459},
    /* 4500 : KMOVQ reg64,kreg64 [rm:       evex.l0.0f.f2.w1 93 /r            ] ZU,LONG,PROT,AVX512BW,X86_64,APX */
        /* 250(f1 ff 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,X86_64 */
        {I_KMOVQ, 2, {REG_GPR|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+29713, 506, 4459},
    /* 4501 : KMOVB kreg8,reg32 [rm:       vex.l0.0f.66.w0 92 /r            ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(92) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65365, 499, 4466},
    /* 4502 : KMOVB kreg8,reg32 [rm:       evex.l0.0f.66.w0 92 /r            ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {OPMASKREG|BITS8,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29722, 500, 4466},
    /* 4503 : KMOVW kreg16,reg32 [rm:       vex.l0.0f.np.w0 92 /r            ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(92) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65372, 501, 4474},
    /* 4504 : KMOVW kreg16,reg32 [rm:       evex.l0.0f.np.w0 92 /r            ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(92) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {OPMASKREG|BITS16,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+29731, 502, 4474},
    /* 4505 : KMOVB reg32,kreg8 [rm:       vex.l0.0f.66.w0 93 /r            ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(93) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KMOVB, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65379, 499, 4482},
    /* 4506 : KMOVB reg32,kreg8 [rm:       evex.l0.0f.66.w0 93 /r            ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KMOVB, 2, {REG_GPR|BITS32,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29740, 500, 4482},
    /* 4507 : KMOVW reg32,kreg16 [rm:       vex.l0.0f.np.w0 93 /r            ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(93) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KMOVW, 2, {REG_GPR|BITS32|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65386, 501, 4486},
    /* 4508 : KMOVW reg32,kreg16 [rm:       evex.l0.0f.np.w0 93 /r            ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(93) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KMOVW, 2, {REG_GPR|BITS32,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29749, 502, 4486},
    /* 4509 : KADDB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 4a /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(4a) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KADDB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65393, 499, 4491},
    /* 4510 : KADDB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 4a /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KADDB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29758, 500, 4491},
    /* 4511 : KADDW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 4a /r             ] ZU,AVX512DQ,FUTURE */
        /* 261(001 004) 001(4a) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KADDW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65407, 499, 4492},
    /* 4512 : KADDW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 4a /r             ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KADDW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29776, 500, 4492},
    /* 4513 : KADDD kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 4a /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(4a) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KADDD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65421, 503, 4493},
    /* 4514 : KADDD kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 4a /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KADDD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29794, 504, 4493},
    /* 4515 : KADDQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 4a /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(4a) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KADDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65435, 503, 4494},
    /* 4516 : KADDQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 4a /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(4a) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KADDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+29812, 504, 4494},
    /* 4517 : KANDB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 41 /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(41) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KANDB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65449, 499, 4507},
    /* 4518 : KANDB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 41 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KANDB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29830, 500, 4507},
    /* 4519 : KANDW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 41 /r             ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(41) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KANDW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65463, 501, 4508},
    /* 4520 : KANDW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 41 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KANDW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29848, 502, 4508},
    /* 4521 : KANDD kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 41 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(41) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65477, 503, 4509},
    /* 4522 : KANDD kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 41 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29866, 504, 4509},
    /* 4523 : KANDQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 41 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(41) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65491, 503, 4510},
    /* 4524 : KANDQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 41 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(41) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+29884, 504, 4510},
    /* 4525 : KANDNB kreg8,kreg8*,kreg8 [rvm:      vex.l1.0f.ko8 42 /r              ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(42) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KANDNB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65505, 499, 4523},
    /* 4526 : KANDNB kreg8,kreg8*,kreg8 [rvm:      evex.l1.0f.ko8 42 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KANDNB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+29902, 500, 4523},
    /* 4527 : KANDNW kreg16,kreg16*,kreg16 [rvm:      vex.l1.0f.ko16 42 /r             ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(42) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KANDNW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65519, 501, 4524},
    /* 4528 : KANDNW kreg16,kreg16*,kreg16 [rvm:      evex.l1.0f.ko16 42 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KANDNW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+29920, 502, 4524},
    /* 4529 : KANDND kreg32,kreg32*,kreg32 [rvm:      vex.l1.0f.ko32 42 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(42) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDND, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65533, 503, 4525},
    /* 4530 : KANDND kreg32,kreg32*,kreg32 [rvm:      evex.l1.0f.ko32 42 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDND, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+29938, 504, 4525},
    /* 4531 : KANDNQ kreg64,kreg64*,kreg64 [rvm:      vex.l1.0f.ko64 42 /r             ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(42) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KANDNQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65547, 503, 4526},
    /* 4532 : KANDNQ kreg64,kreg64*,kreg64 [rvm:      evex.l1.0f.ko64 42 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(42) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KANDNQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+29956, 504, 4526},
    /* 4533 : KNOTB kreg8,kreg8* [rm:       vex.l0.0f.ko8 44 /r              ] ZU,AVX512DQ,FUTURE */
        /* 270(001 001) 001(44) 110 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KNOTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65561, 499, 4539},
    /* 4534 : KNOTB kreg8,kreg8* [rm:       evex.l0.0f.ko8 44 /r              ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KNOTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+29974, 500, 4539},
    /* 4535 : KNOTW kreg16,kreg16* [rm:       vex.l0.0f.ko16 44 /r             ] ZU,AVX512F,FUTURE */
        /* 270(001 000) 001(44) 110 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KNOTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65575, 501, 4540},
    /* 4536 : KNOTW kreg16,kreg16* [rm:       evex.l0.0f.ko16 44 /r             ] ZU,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KNOTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+29992, 502, 4540},
    /* 4537 : KNOTD kreg32,kreg32* [rm:       vex.l0.0f.ko32 44 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 201) 001(44) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KNOTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65589, 503, 4541},
    /* 4538 : KNOTD kreg32,kreg32* [rm:       evex.l0.0f.ko32 44 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KNOTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30010, 504, 4541},
    /* 4539 : KNOTQ kreg64,kreg64* [rm:       vex.l0.0f.ko64 44 /r             ] ZU,AVX512BW,FUTURE */
        /* 270(001 200) 001(44) 110 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KNOTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65603, 503, 4542},
    /* 4540 : KNOTQ kreg64,kreg64* [rm:       evex.l0.0f.ko64 44 /r             ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(44) 110 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KNOTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30028, 504, 4542},
    /* 4541 : KORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 45 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(45) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65617, 499, 4555},
    /* 4542 : KORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 45 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30046, 500, 4555},
    /* 4543 : KORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 45 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(45) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65631, 501, 4556},
    /* 4544 : KORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 45 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30064, 502, 4556},
    /* 4545 : KORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 45 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(45) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65645, 503, 4557},
    /* 4546 : KORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 45 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30082, 504, 4557},
    /* 4547 : KORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 45 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(45) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65659, 503, 4558},
    /* 4548 : KORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 45 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(45) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30100, 504, 4558},
    /* 4549 : KORTESTB kreg8,kreg8 [rm:       vex.l0.0f.ko8 98 /r              ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 001) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KORTESTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65673, 537, 4571},
    /* 4550 : KORTESTB kreg8,kreg8 [rm:       evex.l0.0f.ko8 98 /r              ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KORTESTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+30118, 538, 4571},
    /* 4551 : KORTESTW kreg16,kreg16 [rm:       vex.l0.0f.ko16 98 /r             ] ZU,FL,AVX512F,FUTURE */
        /* 270(001 000) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512F,FUTURE */
        {I_KORTESTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65680, 539, 4572},
    /* 4552 : KORTESTW kreg16,kreg16 [rm:       evex.l0.0f.ko16 98 /r             ] ZU,FL,AVX512F,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KORTESTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+30127, 540, 4572},
    /* 4553 : KORTESTD kreg32,kreg32 [rm:       vex.l0.0f.ko32 98 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 201) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KORTESTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65687, 541, 4573},
    /* 4554 : KORTESTD kreg32,kreg32 [rm:       evex.l0.0f.ko32 98 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORTESTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30136, 542, 4573},
    /* 4555 : KORTESTQ kreg64,kreg64 [rm:       vex.l0.0f.ko64 98 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 200) 001(98) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KORTESTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65694, 541, 4574},
    /* 4556 : KORTESTQ kreg64,kreg64 [rm:       evex.l0.0f.ko64 98 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(98) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KORTESTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30145, 542, 4574},
    /* 4557 : KSHIFTLB kreg8,kreg8,imm8 [rmi:      vex.l0.66.0f3a.w0 32 /r ib       ] ZU,AVX512DQ,FUTURE */
        /* 270(003 001) 001(32) 110 022 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KSHIFTLB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54715, 499, 4587},
    /* 4558 : KSHIFTLB kreg8,kreg8,imm8 [rmi:      evex.l0.66.0f3a.w0 32 /r ib       ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(32) 110 022 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KSHIFTLB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18053, 500, 4587},
    /* 4559 : KSHIFTLW kreg16,kreg16,imm8 [rmi:      vex.l0.66.0f3a.w1 32 /r ib       ] ZU,AVX512F,FUTURE */
        /* 270(003 201) 001(32) 110 022 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KSHIFTLW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54723, 501, 4588},
    /* 4560 : KSHIFTLW kreg16,kreg16,imm8 [rmi:      evex.l0.66.0f3a.w1 32 /r ib       ] ZU,AVX512F,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(32) 110 022 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KSHIFTLW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18063, 502, 4588},
    /* 4561 : KSHIFTLD kreg32,kreg32,imm8 [rmi:      vex.l0.66.0f3a.w0 33 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 001) 001(33) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTLD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54731, 503, 4589},
    /* 4562 : KSHIFTLD kreg32,kreg32,imm8 [rmi:      evex.l0.66.0f3a.w0 33 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(33) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTLD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18073, 504, 4589},
    /* 4563 : KSHIFTLQ kreg64,kreg64,imm8 [rmi:      vex.l0.66.0f3a.w1 33 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 201) 001(33) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTLQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54739, 503, 4590},
    /* 4564 : KSHIFTLQ kreg64,kreg64,imm8 [rmi:      evex.l0.66.0f3a.w1 33 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(33) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTLQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18083, 504, 4590},
    /* 4565 : KSHIFTRB kreg8,kreg8,imm8 [rmi:      vex.l0.66.0f3a.w0 30 /r ib       ] ZU,AVX512DQ,FUTURE */
        /* 270(003 001) 001(30) 110 022 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KSHIFTRB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54747, 499, 4619},
    /* 4566 : KSHIFTRB kreg8,kreg8,imm8 [rmi:      evex.l0.66.0f3a.w0 30 /r ib       ] ZU,AVX512DQ,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(30) 110 022 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KSHIFTRB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18093, 500, 4619},
    /* 4567 : KSHIFTRW kreg16,kreg16,imm8 [rmi:      vex.l0.66.0f3a.w1 30 /r ib       ] ZU,AVX512F,FUTURE */
        /* 270(003 201) 001(30) 110 022 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KSHIFTRW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54755, 501, 4620},
    /* 4568 : KSHIFTRW kreg16,kreg16,imm8 [rmi:      evex.l0.66.0f3a.w1 30 /r ib       ] ZU,AVX512F,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(30) 110 022 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KSHIFTRW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18103, 502, 4620},
    /* 4569 : KSHIFTRD kreg32,kreg32,imm8 [rmi:      vex.l0.66.0f3a.w0 31 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 001) 001(31) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTRD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54763, 503, 4621},
    /* 4570 : KSHIFTRD kreg32,kreg32,imm8 [rmi:      evex.l0.66.0f3a.w0 31 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 7d 08) 300 001(31) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTRD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18113, 504, 4621},
    /* 4571 : KSHIFTRQ kreg64,kreg64,imm8 [rmi:      vex.l0.66.0f3a.w1 31 /r ib       ] ZU,AVX512BW,FUTURE */
        /* 270(003 201) 001(31) 110 022 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KSHIFTRQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+54771, 503, 4622},
    /* 4572 : KSHIFTRQ kreg64,kreg64,imm8 [rmi:      evex.l0.66.0f3a.w1 31 /r ib       ] ZU,AVX512BW,FUTURE,APX */
        /* 250(f3 fd 08) 300 001(31) 110 022 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KSHIFTRQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18123, 504, 4622},
    /* 4573 : KTESTB kreg8,kreg8 [rm:       vex.l0.0f.ko8 99 /r              ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 001) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KTESTB, 2, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65701, 537, 4651},
    /* 4574 : KTESTB kreg8,kreg8 [rm:       evex.l0.0f.ko8 99 /r              ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7d 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KTESTB, 2, {OPMASKREG|BITS8,OPMASKREG|BITS8,0,0,0}, NO_DECORATOR, nasm_bytecodes+30154, 538, 4651},
    /* 4575 : KTESTW kreg16,kreg16 [rm:       vex.l0.0f.ko16 99 /r             ] ZU,FL,AVX512DQ,FUTURE */
        /* 270(001 000) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512DQ,FUTURE */
        {I_KTESTW, 2, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65708, 537, 4652},
    /* 4576 : KTESTW kreg16,kreg16 [rm:       evex.l0.0f.ko16 99 /r             ] ZU,FL,AVX512DQ,FUTURE,APX */
        /* 250(f1 7c 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KTESTW, 2, {OPMASKREG|BITS16,OPMASKREG|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+30163, 538, 4652},
    /* 4577 : KTESTD kreg32,kreg32 [rm:       vex.l0.0f.ko32 99 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 201) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KTESTD, 2, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65715, 541, 4653},
    /* 4578 : KTESTD kreg32,kreg32 [rm:       evex.l0.0f.ko32 99 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fd 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KTESTD, 2, {OPMASKREG|BITS32,OPMASKREG|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+30172, 542, 4653},
    /* 4579 : KTESTQ kreg64,kreg64 [rm:       vex.l0.0f.ko64 99 /r             ] ZU,FL,AVX512BW,FUTURE */
        /* 270(001 200) 001(99) 110 : NOAPX,ZU,FL,VEX,AVX512BW,FUTURE */
        {I_KTESTQ, 2, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65722, 541, 4654},
    /* 4580 : KTESTQ kreg64,kreg64 [rm:       evex.l0.0f.ko64 99 /r             ] ZU,FL,AVX512BW,FUTURE,APX */
        /* 250(f1 fc 08) 300 001(99) 110 : ZU,LONG,FL,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KTESTQ, 2, {OPMASKREG|BITS64,OPMASKREG|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+30181, 542, 4654},
    /* 4581 : KUNPCKBW kreg16,kreg8*,kreg8 [rvm:      vex.nds.l1.66.0f.w0 4b /r        ] ZU,AVX512F,FUTURE */
        /* 261(001 005) 001(4b) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KUNPCKBW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65729, 501, 4667},
    /* 4582 : KUNPCKBW kreg16,kreg8*,kreg8 [rvm:      evex.nds.l1.66.0f.w0 4b /r        ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KUNPCKBW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30190, 502, 4667},
    /* 4583 : KUNPCKWD kreg32,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.w0 4b /r           ] ZU,AVX512BW,FUTURE */
        /* 261(001 004) 001(4b) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KUNPCKWD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65743, 503, 4673},
    /* 4584 : KUNPCKWD kreg32,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.w0 4b /r           ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KUNPCKWD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30208, 504, 4673},
    /* 4585 : KUNPCKDQ kreg64,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.w1 4b /r           ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(4b) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KUNPCKDQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65757, 503, 4679},
    /* 4586 : KUNPCKDQ kreg64,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.w1 4b /r           ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(4b) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KUNPCKDQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30226, 504, 4679},
    /* 4587 : KXNORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 46 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(46) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KXNORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65771, 499, 4685},
    /* 4588 : KXNORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 46 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KXNORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30244, 500, 4685},
    /* 4589 : KXNORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 46 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(46) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KXNORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65785, 501, 4686},
    /* 4590 : KXNORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 46 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KXNORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30262, 502, 4686},
    /* 4591 : KXNORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 46 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(46) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXNORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65799, 503, 4687},
    /* 4592 : KXNORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 46 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXNORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30280, 504, 4687},
    /* 4593 : KXNORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 46 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(46) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXNORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65813, 503, 4688},
    /* 4594 : KXNORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 46 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(46) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXNORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30298, 504, 4688},
    /* 4595 : KXORB kreg8,kreg8*,kreg8 [rvm:      vex.nds.l1.0f.ko8 47 /r          ] ZU,AVX512DQ,FUTURE */
        /* 261(001 005) 001(47) 120 : NOAPX,ZU,VEX,AVX512DQ,FUTURE */
        {I_KXORB, 3, {OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,OPMASKREG|BITS8|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65827, 499, 4701},
    /* 4596 : KXORB kreg8,kreg8*,kreg8 [rvm:      evex.nds.l1.0f.ko8 47 /r          ] ZU,AVX512DQ,FUTURE,APX */
        /* 241(f1 7d 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512DQ,APX,FUTURE */
        {I_KXORB, 3, {OPMASKREG|BITS8,OPMASKREG|BITS8,OPMASKREG|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+30316, 500, 4701},
    /* 4597 : KXORW kreg16,kreg16*,kreg16 [rvm:      vex.nds.l1.0f.ko16 47 /r         ] ZU,AVX512F,FUTURE */
        /* 261(001 004) 001(47) 120 : NOAPX,ZU,VEX,AVX512F,FUTURE */
        {I_KXORW, 3, {OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,OPMASKREG|BITS16|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65841, 501, 4702},
    /* 4598 : KXORW kreg16,kreg16*,kreg16 [rvm:      evex.nds.l1.0f.ko16 47 /r         ] ZU,AVX512F,FUTURE,APX */
        /* 241(f1 7c 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512F,APX,FUTURE */
        {I_KXORW, 3, {OPMASKREG|BITS16,OPMASKREG|BITS16,OPMASKREG|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+30334, 502, 4702},
    /* 4599 : KXORD kreg32,kreg32*,kreg32 [rvm:      vex.nds.l1.0f.ko32 47 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 205) 001(47) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXORD, 3, {OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,OPMASKREG|BITS32|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65855, 503, 4703},
    /* 4600 : KXORD kreg32,kreg32*,kreg32 [rvm:      evex.nds.l1.0f.ko32 47 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fd 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXORD, 3, {OPMASKREG|BITS32,OPMASKREG|BITS32,OPMASKREG|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+30352, 504, 4703},
    /* 4601 : KXORQ kreg64,kreg64*,kreg64 [rvm:      vex.nds.l1.0f.ko64 47 /r         ] ZU,AVX512BW,FUTURE */
        /* 261(001 204) 001(47) 120 : NOAPX,ZU,VEX,AVX512BW,FUTURE */
        {I_KXORQ, 3, {OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,OPMASKREG|BITS64|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65869, 503, 4704},
    /* 4602 : KXORQ kreg64,kreg64*,kreg64 [rvm:      evex.nds.l1.0f.ko64 47 /r         ] ZU,AVX512BW,FUTURE,APX */
        /* 241(f1 fc 28) 300 001(47) 120 : ZU,LONG,EVEX,PROT,AVX512BW,APX,FUTURE */
        {I_KXORQ, 3, {OPMASKREG|BITS64,OPMASKREG|BITS64,OPMASKREG|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+30370, 504, 4704},
    /* 4603 : VADDPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 58 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30388, 543, 4719},
    /* 4604 : VADDPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 58 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30406, 543, 4720},
    /* 4605 : VADDPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 58 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(58) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VADDPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+30424, 544, 4721},
    /* 4606 : VADDPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 58 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30442, 543, 4722},
    /* 4607 : VADDPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 58 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(58) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VADDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30460, 543, 4723},
    /* 4608 : VADDPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 58 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(58) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VADDPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+30478, 544, 4724},
    /* 4609 : VADDSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 58 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(58) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VADDSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+30496, 545, 4725},
    /* 4610 : VADDSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 58 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(58) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VADDSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+30514, 545, 4726},
    /* 4611 : VALIGND xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGND, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18133, 543, 4727},
    /* 4612 : VALIGND ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGND, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18153, 543, 4728},
    /* 4613 : VALIGND zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 03 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(03) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VALIGND, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18173, 544, 4729},
    /* 4614 : VALIGNQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGNQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18193, 543, 4730},
    /* 4615 : VALIGNQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 03 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(03) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VALIGNQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18213, 543, 4731},
    /* 4616 : VALIGNQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 03 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(03) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VALIGNQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18233, 544, 4732},
    /* 4617 : VANDNPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 55 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30532, 546, 4733},
    /* 4618 : VANDNPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 55 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30550, 546, 4734},
    /* 4619 : VANDNPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 55 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(55) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDNPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30568, 547, 4735},
    /* 4620 : VANDNPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 55 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30586, 546, 4736},
    /* 4621 : VANDNPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 55 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(55) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30604, 546, 4737},
    /* 4622 : VANDNPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 55 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(55) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDNPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30622, 547, 4738},
    /* 4623 : VANDPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 54 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30640, 546, 4739},
    /* 4624 : VANDPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 54 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30658, 546, 4740},
    /* 4625 : VANDPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 54 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(54) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30676, 547, 4741},
    /* 4626 : VANDPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 54 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30694, 546, 4742},
    /* 4627 : VANDPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 54 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(54) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30712, 546, 4743},
    /* 4628 : VANDPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 54 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(54) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VANDPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30730, 547, 4744},
    /* 4629 : VBLENDMPD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30748, 543, 4745},
    /* 4630 : VBLENDMPD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30757, 543, 4746},
    /* 4631 : VBLENDMPD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 65 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(65) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VBLENDMPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+30766, 544, 4747},
    /* 4632 : VBLENDMPS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30775, 543, 4748},
    /* 4633 : VBLENDMPS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 65 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(65) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBLENDMPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30784, 543, 4749},
    /* 4634 : VBLENDMPS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 65 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(65) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VBLENDMPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+30793, 544, 4750},
    /* 4635 : VBROADCASTF32X2 ymmreg|mask|z,xmmrm64 [rm:t2:    evex.256.66.0f38.w0 19 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 28) 311 001(19) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X2, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30802, 546, 4751},
    /* 4636 : VBROADCASTF32X2 zmmreg|mask|z,xmmrm64 [rm:t2:    evex.512.66.0f38.w0 19 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 311 001(19) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X2, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30811, 547, 4752},
    /* 4637 : VBROADCASTF32X4 ymmreg|mask|z,mem128 [rm:t4:    evex.256.66.0f38.w0 1a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 312 001(1a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTF32X4, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30820, 543, 4753},
    /* 4638 : VBROADCASTF32X4 zmmreg|mask|z,mem128 [rm:t4:    evex.512.66.0f38.w0 1a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 312 001(1a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTF32X4, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30829, 544, 4754},
    /* 4639 : VBROADCASTF32X8 zmmreg|mask|z,mem256 [rm:t8:    evex.512.66.0f38.w0 1b /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 313 001(1b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF32X8, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30838, 547, 4755},
    /* 4640 : VBROADCASTF64X2 ymmreg|mask|z,mem128 [rm:t2:    evex.256.66.0f38.w1 1a /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fd 28) 311 001(1a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTF64X2, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30847, 546, 4756},
    /* 4641 : VBROADCASTF64X2 zmmreg|mask|z,mem128 [rm:t2:    evex.512.66.0f38.w1 1a /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fd 48) 311 001(1a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTF64X2, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30856, 547, 4757},
    /* 4642 : VBROADCASTF64X4 zmmreg|mask|z,mem256 [rm:t4:    evex.512.66.0f38.w1 1b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 312 001(1b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTF64X4, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30865, 544, 4758},
    /* 4643 : VBROADCASTI32X2 xmmreg|mask|z,xmmrm64 [rm:t2:    evex.128.66.0f38.w0 59 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 08) 311 001(59) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30874, 546, 4759},
    /* 4644 : VBROADCASTI32X2 ymmreg|mask|z,xmmrm64 [rm:t2:    evex.256.66.0f38.w0 59 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7d 28) 311 001(59) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30883, 546, 4760},
    /* 4645 : VBROADCASTI32X2 zmmreg|mask|z,xmmrm64 [rm:t2:    evex.512.66.0f38.w0 59 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 311 001(59) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X2, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30892, 547, 4761},
    /* 4646 : VBROADCASTI32X4 ymmreg|mask|z,mem128 [rm:t4:    evex.256.66.0f38.w0 5a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 312 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTI32X4, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30901, 543, 4762},
    /* 4647 : VBROADCASTI32X4 zmmreg|mask|z,mem128 [rm:t4:    evex.512.66.0f38.w0 5a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 312 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTI32X4, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30910, 544, 4763},
    /* 4648 : VBROADCASTI32X8 zmmreg|mask|z,mem256 [rm:t8:    evex.512.66.0f38.w0 5b /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7d 48) 313 001(5b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI32X8, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30919, 547, 4764},
    /* 4649 : VBROADCASTI64X2 ymmreg|mask|z,mem128 [rm:t2:    evex.256.66.0f38.w1 5a /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fd 28) 311 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VBROADCASTI64X2, 2, {YMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30928, 546, 4765},
    /* 4650 : VBROADCASTI64X2 zmmreg|mask|z,mem128 [rm:t2:    evex.512.66.0f38.w1 5a /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fd 48) 311 001(5a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VBROADCASTI64X2, 2, {ZMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30937, 547, 4766},
    /* 4651 : VBROADCASTI64X4 zmmreg|mask|z,mem256 [rm:t4:    evex.512.66.0f38.w1 5b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 312 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTI64X4, 2, {ZMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30946, 544, 4767},
    /* 4652 : VBROADCASTSD ymmreg|mask|z,mem64 [rm:t1s:   evex.256.66.0f38.w1 19 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(19) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30955, 543, 4768},
    /* 4653 : VBROADCASTSD zmmreg|mask|z,mem64 [rm:t1s:   evex.512.66.0f38.w1 19 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(19) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSD, 2, {ZMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30964, 544, 4769},
    /* 4654 : VBROADCASTSD ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w1 19 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(19) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSD, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30973, 543, 4770},
    /* 4655 : VBROADCASTSD zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w1 19 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(19) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSD, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30982, 544, 4771},
    /* 4656 : VBROADCASTSS xmmreg|mask|z,mem32 [rm:t1s:   evex.128.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+30991, 543, 4772},
    /* 4657 : VBROADCASTSS ymmreg|mask|z,mem32 [rm:t1s:   evex.256.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31000, 543, 4773},
    /* 4658 : VBROADCASTSS zmmreg|mask|z,mem32 [rm:t1s:   evex.512.66.0f38.w0 18 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(18) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSS, 2, {ZMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31009, 544, 4774},
    /* 4659 : VBROADCASTSS xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31018, 543, 4775},
    /* 4660 : VBROADCASTSS ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w0 18 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(18) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VBROADCASTSS, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31027, 543, 4776},
    /* 4661 : VBROADCASTSS zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w0 18 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(18) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VBROADCASTSS, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31036, 544, 4777},
    /* 4662 : VCMPEQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4743, 543, 4778},
    /* 4663 : VCMPEQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4754, 543, 4779},
    /* 4664 : VCMPEQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4765, 544, 4780},
    /* 4665 : VCMPEQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4776, 543, 4781},
    /* 4666 : VCMPEQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4787, 543, 4782},
    /* 4667 : VCMPEQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 00       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4798, 544, 4783},
    /* 4668 : VCMPEQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4809, 545, 4784},
    /* 4669 : VCMPEQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4820, 545, 4785},
    /* 4670 : VCMPEQ_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4743, 543, 4786},
    /* 4671 : VCMPEQ_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 00    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4754, 543, 4787},
    /* 4672 : VCMPEQ_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4765, 544, 4788},
    /* 4673 : VCMPEQ_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4776, 543, 4789},
    /* 4674 : VCMPEQ_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 00       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4787, 543, 4790},
    /* 4675 : VCMPEQ_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 00       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4798, 544, 4791},
    /* 4676 : VCMPEQ_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4809, 545, 4792},
    /* 4677 : VCMPEQ_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 00    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(00) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4820, 545, 4793},
    /* 4678 : VCMPLTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4831, 543, 4794},
    /* 4679 : VCMPLTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4842, 543, 4795},
    /* 4680 : VCMPLTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4853, 544, 4796},
    /* 4681 : VCMPLTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4864, 543, 4797},
    /* 4682 : VCMPLTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4875, 543, 4798},
    /* 4683 : VCMPLTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 01       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4886, 544, 4799},
    /* 4684 : VCMPLTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4897, 545, 4800},
    /* 4685 : VCMPLTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4908, 545, 4801},
    /* 4686 : VCMPLT_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4831, 543, 4802},
    /* 4687 : VCMPLT_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 01    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4842, 543, 4803},
    /* 4688 : VCMPLT_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4853, 544, 4804},
    /* 4689 : VCMPLT_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4864, 543, 4805},
    /* 4690 : VCMPLT_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 01       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4875, 543, 4806},
    /* 4691 : VCMPLT_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 01       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4886, 544, 4807},
    /* 4692 : VCMPLT_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4897, 545, 4808},
    /* 4693 : VCMPLT_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 01    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(01) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4908, 545, 4809},
    /* 4694 : VCMPLEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4919, 543, 4810},
    /* 4695 : VCMPLEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4930, 543, 4811},
    /* 4696 : VCMPLEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4941, 544, 4812},
    /* 4697 : VCMPLEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4952, 543, 4813},
    /* 4698 : VCMPLEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4963, 543, 4814},
    /* 4699 : VCMPLEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 02       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4974, 544, 4815},
    /* 4700 : VCMPLESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4985, 545, 4816},
    /* 4701 : VCMPLESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4996, 545, 4817},
    /* 4702 : VCMPLE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4919, 543, 4818},
    /* 4703 : VCMPLE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 02    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+4930, 543, 4819},
    /* 4704 : VCMPLE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+4941, 544, 4820},
    /* 4705 : VCMPLE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4952, 543, 4821},
    /* 4706 : VCMPLE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 02       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+4963, 543, 4822},
    /* 4707 : VCMPLE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 02       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+4974, 544, 4823},
    /* 4708 : VCMPLE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4985, 545, 4824},
    /* 4709 : VCMPLE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 02    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(02) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+4996, 545, 4825},
    /* 4710 : VCMPUNORDPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5007, 543, 4826},
    /* 4711 : VCMPUNORDPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5018, 543, 4827},
    /* 4712 : VCMPUNORDPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5029, 544, 4828},
    /* 4713 : VCMPUNORDPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5040, 543, 4829},
    /* 4714 : VCMPUNORDPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5051, 543, 4830},
    /* 4715 : VCMPUNORDPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 03       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5062, 544, 4831},
    /* 4716 : VCMPUNORDSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5073, 545, 4832},
    /* 4717 : VCMPUNORDSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORDSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5084, 545, 4833},
    /* 4718 : VCMPUNORD_QPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5007, 543, 4834},
    /* 4719 : VCMPUNORD_QPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 03    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5018, 543, 4835},
    /* 4720 : VCMPUNORD_QPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5029, 544, 4836},
    /* 4721 : VCMPUNORD_QPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5040, 543, 4837},
    /* 4722 : VCMPUNORD_QPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 03       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5051, 543, 4838},
    /* 4723 : VCMPUNORD_QPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 03       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(03) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5062, 544, 4839},
    /* 4724 : VCMPUNORD_QSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5073, 545, 4840},
    /* 4725 : VCMPUNORD_QSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 03    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(03) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_QSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5084, 545, 4841},
    /* 4726 : VCMPNEQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5095, 543, 4842},
    /* 4727 : VCMPNEQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5106, 543, 4843},
    /* 4728 : VCMPNEQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5117, 544, 4844},
    /* 4729 : VCMPNEQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5128, 543, 4845},
    /* 4730 : VCMPNEQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5139, 543, 4846},
    /* 4731 : VCMPNEQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 04       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5150, 544, 4847},
    /* 4732 : VCMPNEQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5161, 545, 4848},
    /* 4733 : VCMPNEQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5172, 545, 4849},
    /* 4734 : VCMPNEQ_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5095, 543, 4850},
    /* 4735 : VCMPNEQ_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 04    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5106, 543, 4851},
    /* 4736 : VCMPNEQ_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5117, 544, 4852},
    /* 4737 : VCMPNEQ_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5128, 543, 4853},
    /* 4738 : VCMPNEQ_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 04       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5139, 543, 4854},
    /* 4739 : VCMPNEQ_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 04       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5150, 544, 4855},
    /* 4740 : VCMPNEQ_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5161, 545, 4856},
    /* 4741 : VCMPNEQ_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 04    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(04) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5172, 545, 4857},
    /* 4742 : VCMPNLTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5183, 543, 4858},
    /* 4743 : VCMPNLTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5194, 543, 4859},
    /* 4744 : VCMPNLTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5205, 544, 4860},
    /* 4745 : VCMPNLTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5216, 543, 4861},
    /* 4746 : VCMPNLTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5227, 543, 4862},
    /* 4747 : VCMPNLTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 05       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5238, 544, 4863},
    /* 4748 : VCMPNLTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5249, 545, 4864},
    /* 4749 : VCMPNLTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5260, 545, 4865},
    /* 4750 : VCMPNLT_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5183, 543, 4866},
    /* 4751 : VCMPNLT_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 05    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5194, 543, 4867},
    /* 4752 : VCMPNLT_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5205, 544, 4868},
    /* 4753 : VCMPNLT_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5216, 543, 4869},
    /* 4754 : VCMPNLT_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 05       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5227, 543, 4870},
    /* 4755 : VCMPNLT_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 05       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5238, 544, 4871},
    /* 4756 : VCMPNLT_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5249, 545, 4872},
    /* 4757 : VCMPNLT_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 05    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(05) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5260, 545, 4873},
    /* 4758 : VCMPNLEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5271, 543, 4874},
    /* 4759 : VCMPNLEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5282, 543, 4875},
    /* 4760 : VCMPNLEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5293, 544, 4876},
    /* 4761 : VCMPNLEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5304, 543, 4877},
    /* 4762 : VCMPNLEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5315, 543, 4878},
    /* 4763 : VCMPNLEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 06       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5326, 544, 4879},
    /* 4764 : VCMPNLESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5337, 545, 4880},
    /* 4765 : VCMPNLESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5348, 545, 4881},
    /* 4766 : VCMPNLE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5271, 543, 4882},
    /* 4767 : VCMPNLE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 06    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5282, 543, 4883},
    /* 4768 : VCMPNLE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5293, 544, 4884},
    /* 4769 : VCMPNLE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5304, 543, 4885},
    /* 4770 : VCMPNLE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 06       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5315, 543, 4886},
    /* 4771 : VCMPNLE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 06       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5326, 544, 4887},
    /* 4772 : VCMPNLE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5337, 545, 4888},
    /* 4773 : VCMPNLE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 06    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(06) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5348, 545, 4889},
    /* 4774 : VCMPORDPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5359, 543, 4890},
    /* 4775 : VCMPORDPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5370, 543, 4891},
    /* 4776 : VCMPORDPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5381, 544, 4892},
    /* 4777 : VCMPORDPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5392, 543, 4893},
    /* 4778 : VCMPORDPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5403, 543, 4894},
    /* 4779 : VCMPORDPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 07       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5414, 544, 4895},
    /* 4780 : VCMPORDSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5425, 545, 4896},
    /* 4781 : VCMPORDSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORDSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5436, 545, 4897},
    /* 4782 : VCMPORD_QPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5359, 543, 4898},
    /* 4783 : VCMPORD_QPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 07    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5370, 543, 4899},
    /* 4784 : VCMPORD_QPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5381, 544, 4900},
    /* 4785 : VCMPORD_QPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5392, 543, 4901},
    /* 4786 : VCMPORD_QPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 07       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5403, 543, 4902},
    /* 4787 : VCMPORD_QPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 07       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(07) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5414, 544, 4903},
    /* 4788 : VCMPORD_QSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5425, 545, 4904},
    /* 4789 : VCMPORD_QSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 07    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(07) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_QSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5436, 545, 4905},
    /* 4790 : VCMPEQ_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 08    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5447, 543, 4906},
    /* 4791 : VCMPEQ_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 08    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5458, 543, 4907},
    /* 4792 : VCMPEQ_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5469, 544, 4908},
    /* 4793 : VCMPEQ_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 08       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5480, 543, 4909},
    /* 4794 : VCMPEQ_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 08       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5491, 543, 4910},
    /* 4795 : VCMPEQ_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 08       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(08) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5502, 544, 4911},
    /* 4796 : VCMPEQ_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(08) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5513, 545, 4912},
    /* 4797 : VCMPEQ_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 08    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(08) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5524, 545, 4913},
    /* 4798 : VCMPNGEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5535, 543, 4914},
    /* 4799 : VCMPNGEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5546, 543, 4915},
    /* 4800 : VCMPNGEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5557, 544, 4916},
    /* 4801 : VCMPNGEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5568, 543, 4917},
    /* 4802 : VCMPNGEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5579, 543, 4918},
    /* 4803 : VCMPNGEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 09       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5590, 544, 4919},
    /* 4804 : VCMPNGESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5601, 545, 4920},
    /* 4805 : VCMPNGESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5612, 545, 4921},
    /* 4806 : VCMPNGE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5535, 543, 4922},
    /* 4807 : VCMPNGE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 09    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5546, 543, 4923},
    /* 4808 : VCMPNGE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5557, 544, 4924},
    /* 4809 : VCMPNGE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5568, 543, 4925},
    /* 4810 : VCMPNGE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 09       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5579, 543, 4926},
    /* 4811 : VCMPNGE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 09       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(09) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5590, 544, 4927},
    /* 4812 : VCMPNGE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5601, 545, 4928},
    /* 4813 : VCMPNGE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 09    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(09) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5612, 545, 4929},
    /* 4814 : VCMPNGTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5623, 543, 4930},
    /* 4815 : VCMPNGTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5634, 543, 4931},
    /* 4816 : VCMPNGTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5645, 544, 4932},
    /* 4817 : VCMPNGTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5656, 543, 4933},
    /* 4818 : VCMPNGTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5667, 543, 4934},
    /* 4819 : VCMPNGTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5678, 544, 4935},
    /* 4820 : VCMPNGTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5689, 545, 4936},
    /* 4821 : VCMPNGTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5700, 545, 4937},
    /* 4822 : VCMPNGT_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5623, 543, 4938},
    /* 4823 : VCMPNGT_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5634, 543, 4939},
    /* 4824 : VCMPNGT_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5645, 544, 4940},
    /* 4825 : VCMPNGT_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5656, 543, 4941},
    /* 4826 : VCMPNGT_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5667, 543, 4942},
    /* 4827 : VCMPNGT_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5678, 544, 4943},
    /* 4828 : VCMPNGT_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5689, 545, 4944},
    /* 4829 : VCMPNGT_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5700, 545, 4945},
    /* 4830 : VCMPFALSEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5711, 543, 4946},
    /* 4831 : VCMPFALSEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5722, 543, 4947},
    /* 4832 : VCMPFALSEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5733, 544, 4948},
    /* 4833 : VCMPFALSEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5744, 543, 4949},
    /* 4834 : VCMPFALSEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5755, 543, 4950},
    /* 4835 : VCMPFALSEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5766, 544, 4951},
    /* 4836 : VCMPFALSESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5777, 545, 4952},
    /* 4837 : VCMPFALSESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5788, 545, 4953},
    /* 4838 : VCMPFALSE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5711, 543, 4954},
    /* 4839 : VCMPFALSE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5722, 543, 4955},
    /* 4840 : VCMPFALSE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5733, 544, 4956},
    /* 4841 : VCMPFALSE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5744, 543, 4957},
    /* 4842 : VCMPFALSE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5755, 543, 4958},
    /* 4843 : VCMPFALSE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5766, 544, 4959},
    /* 4844 : VCMPFALSE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5777, 545, 4960},
    /* 4845 : VCMPFALSE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5788, 545, 4961},
    /* 4846 : VCMPNEQ_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5799, 543, 4962},
    /* 4847 : VCMPNEQ_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5810, 543, 4963},
    /* 4848 : VCMPNEQ_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5821, 544, 4964},
    /* 4849 : VCMPNEQ_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5832, 543, 4965},
    /* 4850 : VCMPNEQ_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5843, 543, 4966},
    /* 4851 : VCMPNEQ_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0c       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5854, 544, 4967},
    /* 4852 : VCMPNEQ_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5865, 545, 4968},
    /* 4853 : VCMPNEQ_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0c    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5876, 545, 4969},
    /* 4854 : VCMPGEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5887, 543, 4970},
    /* 4855 : VCMPGEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5898, 543, 4971},
    /* 4856 : VCMPGEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5909, 544, 4972},
    /* 4857 : VCMPGEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5920, 543, 4973},
    /* 4858 : VCMPGEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5931, 543, 4974},
    /* 4859 : VCMPGEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5942, 544, 4975},
    /* 4860 : VCMPGESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5953, 545, 4976},
    /* 4861 : VCMPGESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5964, 545, 4977},
    /* 4862 : VCMPGE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5887, 543, 4978},
    /* 4863 : VCMPGE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5898, 543, 4979},
    /* 4864 : VCMPGE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5909, 544, 4980},
    /* 4865 : VCMPGE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5920, 543, 4981},
    /* 4866 : VCMPGE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+5931, 543, 4982},
    /* 4867 : VCMPGE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+5942, 544, 4983},
    /* 4868 : VCMPGE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5953, 545, 4984},
    /* 4869 : VCMPGE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+5964, 545, 4985},
    /* 4870 : VCMPGTPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5975, 543, 4986},
    /* 4871 : VCMPGTPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5986, 543, 4987},
    /* 4872 : VCMPGTPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5997, 544, 4988},
    /* 4873 : VCMPGTPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6008, 543, 4989},
    /* 4874 : VCMPGTPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6019, 543, 4990},
    /* 4875 : VCMPGTPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6030, 544, 4991},
    /* 4876 : VCMPGTSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6041, 545, 4992},
    /* 4877 : VCMPGTSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGTSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6052, 545, 4993},
    /* 4878 : VCMPGT_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5975, 543, 4994},
    /* 4879 : VCMPGT_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+5986, 543, 4995},
    /* 4880 : VCMPGT_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+5997, 544, 4996},
    /* 4881 : VCMPGT_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6008, 543, 4997},
    /* 4882 : VCMPGT_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6019, 543, 4998},
    /* 4883 : VCMPGT_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6030, 544, 4999},
    /* 4884 : VCMPGT_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6041, 545, 5000},
    /* 4885 : VCMPGT_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6052, 545, 5001},
    /* 4886 : VCMPTRUEPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6063, 543, 5002},
    /* 4887 : VCMPTRUEPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6074, 543, 5003},
    /* 4888 : VCMPTRUEPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUEPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6085, 544, 5004},
    /* 4889 : VCMPTRUEPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6096, 543, 5005},
    /* 4890 : VCMPTRUEPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6107, 543, 5006},
    /* 4891 : VCMPTRUEPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUEPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6118, 544, 5007},
    /* 4892 : VCMPTRUESD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUESD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6129, 545, 5008},
    /* 4893 : VCMPTRUESS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUESS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6140, 545, 5009},
    /* 4894 : VCMPTRUE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6063, 543, 5010},
    /* 4895 : VCMPTRUE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 0f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6074, 543, 5011},
    /* 4896 : VCMPTRUE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6085, 544, 5012},
    /* 4897 : VCMPTRUE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6096, 543, 5013},
    /* 4898 : VCMPTRUE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 0f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6107, 543, 5014},
    /* 4899 : VCMPTRUE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 0f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(0f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6118, 544, 5015},
    /* 4900 : VCMPTRUE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6129, 545, 5016},
    /* 4901 : VCMPTRUE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 0f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(0f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6140, 545, 5017},
    /* 4902 : VCMPEQ_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 10    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6151, 543, 5018},
    /* 4903 : VCMPEQ_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 10    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6162, 543, 5019},
    /* 4904 : VCMPEQ_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6173, 544, 5020},
    /* 4905 : VCMPEQ_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 10       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6184, 543, 5021},
    /* 4906 : VCMPEQ_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 10       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6195, 543, 5022},
    /* 4907 : VCMPEQ_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 10       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(10) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6206, 544, 5023},
    /* 4908 : VCMPEQ_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(10) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6217, 545, 5024},
    /* 4909 : VCMPEQ_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 10    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(10) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6228, 545, 5025},
    /* 4910 : VCMPLT_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 11    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6239, 543, 5026},
    /* 4911 : VCMPLT_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 11    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6250, 543, 5027},
    /* 4912 : VCMPLT_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6261, 544, 5028},
    /* 4913 : VCMPLT_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 11       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6272, 543, 5029},
    /* 4914 : VCMPLT_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 11       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6283, 543, 5030},
    /* 4915 : VCMPLT_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 11       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(11) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6294, 544, 5031},
    /* 4916 : VCMPLT_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(11) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6305, 545, 5032},
    /* 4917 : VCMPLT_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 11    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(11) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLT_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6316, 545, 5033},
    /* 4918 : VCMPLE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 12    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6327, 543, 5034},
    /* 4919 : VCMPLE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 12    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6338, 543, 5035},
    /* 4920 : VCMPLE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6349, 544, 5036},
    /* 4921 : VCMPLE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 12       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6360, 543, 5037},
    /* 4922 : VCMPLE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 12       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6371, 543, 5038},
    /* 4923 : VCMPLE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 12       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(12) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6382, 544, 5039},
    /* 4924 : VCMPLE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(12) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6393, 545, 5040},
    /* 4925 : VCMPLE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 12    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(12) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPLE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6404, 545, 5041},
    /* 4926 : VCMPUNORD_SPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 13    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6415, 543, 5042},
    /* 4927 : VCMPUNORD_SPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 13    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6426, 543, 5043},
    /* 4928 : VCMPUNORD_SPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6437, 544, 5044},
    /* 4929 : VCMPUNORD_SPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 13       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6448, 543, 5045},
    /* 4930 : VCMPUNORD_SPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 13       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6459, 543, 5046},
    /* 4931 : VCMPUNORD_SPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 13       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(13) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6470, 544, 5047},
    /* 4932 : VCMPUNORD_SSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(13) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6481, 545, 5048},
    /* 4933 : VCMPUNORD_SSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 13    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(13) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPUNORD_SSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6492, 545, 5049},
    /* 4934 : VCMPNEQ_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 14    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6503, 543, 5050},
    /* 4935 : VCMPNEQ_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 14    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6514, 543, 5051},
    /* 4936 : VCMPNEQ_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6525, 544, 5052},
    /* 4937 : VCMPNEQ_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 14       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6536, 543, 5053},
    /* 4938 : VCMPNEQ_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 14       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6547, 543, 5054},
    /* 4939 : VCMPNEQ_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 14       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(14) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6558, 544, 5055},
    /* 4940 : VCMPNEQ_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(14) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6569, 545, 5056},
    /* 4941 : VCMPNEQ_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 14    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(14) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6580, 545, 5057},
    /* 4942 : VCMPNLT_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 15    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6591, 543, 5058},
    /* 4943 : VCMPNLT_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 15    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6602, 543, 5059},
    /* 4944 : VCMPNLT_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6613, 544, 5060},
    /* 4945 : VCMPNLT_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 15       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6624, 543, 5061},
    /* 4946 : VCMPNLT_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 15       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6635, 543, 5062},
    /* 4947 : VCMPNLT_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 15       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(15) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6646, 544, 5063},
    /* 4948 : VCMPNLT_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(15) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6657, 545, 5064},
    /* 4949 : VCMPNLT_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 15    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(15) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLT_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6668, 545, 5065},
    /* 4950 : VCMPNLE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 16    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6679, 543, 5066},
    /* 4951 : VCMPNLE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 16    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6690, 543, 5067},
    /* 4952 : VCMPNLE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6701, 544, 5068},
    /* 4953 : VCMPNLE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 16       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6712, 543, 5069},
    /* 4954 : VCMPNLE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 16       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6723, 543, 5070},
    /* 4955 : VCMPNLE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 16       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(16) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6734, 544, 5071},
    /* 4956 : VCMPNLE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(16) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6745, 545, 5072},
    /* 4957 : VCMPNLE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 16    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(16) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNLE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6756, 545, 5073},
    /* 4958 : VCMPORD_SPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 17    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6767, 543, 5074},
    /* 4959 : VCMPORD_SPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 17    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6778, 543, 5075},
    /* 4960 : VCMPORD_SPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6789, 544, 5076},
    /* 4961 : VCMPORD_SPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 17       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6800, 543, 5077},
    /* 4962 : VCMPORD_SPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 17       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6811, 543, 5078},
    /* 4963 : VCMPORD_SPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 17       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(17) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6822, 544, 5079},
    /* 4964 : VCMPORD_SSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(17) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6833, 545, 5080},
    /* 4965 : VCMPORD_SSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 17    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(17) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPORD_SSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6844, 545, 5081},
    /* 4966 : VCMPEQ_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 18    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6855, 543, 5082},
    /* 4967 : VCMPEQ_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 18    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6866, 543, 5083},
    /* 4968 : VCMPEQ_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6877, 544, 5084},
    /* 4969 : VCMPEQ_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 18       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6888, 543, 5085},
    /* 4970 : VCMPEQ_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 18       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6899, 543, 5086},
    /* 4971 : VCMPEQ_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 18       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(18) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6910, 544, 5087},
    /* 4972 : VCMPEQ_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(18) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6921, 545, 5088},
    /* 4973 : VCMPEQ_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 18    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(18) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPEQ_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+6932, 545, 5089},
    /* 4974 : VCMPNGE_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 19    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6943, 543, 5090},
    /* 4975 : VCMPNGE_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 19    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+6954, 543, 5091},
    /* 4976 : VCMPNGE_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+6965, 544, 5092},
    /* 4977 : VCMPNGE_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 19       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6976, 543, 5093},
    /* 4978 : VCMPNGE_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 19       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+6987, 543, 5094},
    /* 4979 : VCMPNGE_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 19       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(19) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+6998, 544, 5095},
    /* 4980 : VCMPNGE_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(19) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7009, 545, 5096},
    /* 4981 : VCMPNGE_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 19    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(19) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGE_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7020, 545, 5097},
    /* 4982 : VCMPNGT_UQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7031, 543, 5098},
    /* 4983 : VCMPNGT_UQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1a    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7042, 543, 5099},
    /* 4984 : VCMPNGT_UQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7053, 544, 5100},
    /* 4985 : VCMPNGT_UQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7064, 543, 5101},
    /* 4986 : VCMPNGT_UQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1a       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7075, 543, 5102},
    /* 4987 : VCMPNGT_UQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1a       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1a) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7086, 544, 5103},
    /* 4988 : VCMPNGT_UQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7097, 545, 5104},
    /* 4989 : VCMPNGT_UQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1a    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1a) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNGT_UQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7108, 545, 5105},
    /* 4990 : VCMPFALSE_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7119, 543, 5106},
    /* 4991 : VCMPFALSE_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1b    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7130, 543, 5107},
    /* 4992 : VCMPFALSE_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7141, 544, 5108},
    /* 4993 : VCMPFALSE_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7152, 543, 5109},
    /* 4994 : VCMPFALSE_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1b       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7163, 543, 5110},
    /* 4995 : VCMPFALSE_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1b       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1b) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7174, 544, 5111},
    /* 4996 : VCMPFALSE_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7185, 545, 5112},
    /* 4997 : VCMPFALSE_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1b    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1b) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPFALSE_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7196, 545, 5113},
    /* 4998 : VCMPNEQ_OSPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7207, 543, 5114},
    /* 4999 : VCMPNEQ_OSPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1c    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7218, 543, 5115},
    /* 5000 : VCMPNEQ_OSPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7229, 544, 5116},
    /* 5001 : VCMPNEQ_OSPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7240, 543, 5117},
    /* 5002 : VCMPNEQ_OSPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1c       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7251, 543, 5118},
    /* 5003 : VCMPNEQ_OSPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1c       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1c) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7262, 544, 5119},
    /* 5004 : VCMPNEQ_OSSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7273, 545, 5120},
    /* 5005 : VCMPNEQ_OSSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1c    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1c) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPNEQ_OSSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7284, 545, 5121},
    /* 5006 : VCMPGE_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7295, 543, 5122},
    /* 5007 : VCMPGE_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1d    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7306, 543, 5123},
    /* 5008 : VCMPGE_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7317, 544, 5124},
    /* 5009 : VCMPGE_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7328, 543, 5125},
    /* 5010 : VCMPGE_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1d       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7339, 543, 5126},
    /* 5011 : VCMPGE_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1d       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1d) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7350, 544, 5127},
    /* 5012 : VCMPGE_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7361, 545, 5128},
    /* 5013 : VCMPGE_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1d    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1d) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGE_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7372, 545, 5129},
    /* 5014 : VCMPGT_OQPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7383, 543, 5130},
    /* 5015 : VCMPGT_OQPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1e    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7394, 543, 5131},
    /* 5016 : VCMPGT_OQPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7405, 544, 5132},
    /* 5017 : VCMPGT_OQPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7416, 543, 5133},
    /* 5018 : VCMPGT_OQPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1e       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7427, 543, 5134},
    /* 5019 : VCMPGT_OQPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1e       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1e) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7438, 544, 5135},
    /* 5020 : VCMPGT_OQSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7449, 545, 5136},
    /* 5021 : VCMPGT_OQSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1e    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1e) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPGT_OQSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7460, 545, 5137},
    /* 5022 : VCMPTRUE_USPD kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r 1f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7471, 543, 5138},
    /* 5023 : VCMPTRUE_USPD kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r 1f    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7482, 543, 5139},
    /* 5024 : VCMPTRUE_USPD kreg|mask,zmmreg,zmmrm512|b64|sae [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USPD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+7493, 544, 5140},
    /* 5025 : VCMPTRUE_USPS kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r 1f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7504, 543, 5141},
    /* 5026 : VCMPTRUE_USPS kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r 1f       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7515, 543, 5142},
    /* 5027 : VCMPTRUE_USPS kreg|mask,zmmreg,zmmrm512|b32|sae [rvmi:fv:  evex.nds.512.0f.w0 c2 /r 1f       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 001(1f) : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USPS, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+7526, 544, 5143},
    /* 5028 : VCMPTRUE_USSD kreg|mask,xmmreg,xmmrm64|sae [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 001(1f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USSD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7537, 545, 5144},
    /* 5029 : VCMPTRUE_USSS kreg|mask,xmmreg,xmmrm32|sae [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r 1f    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 001(1f) : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPTRUE_USSS, 3, {OPMASKREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+7548, 545, 5145},
    /* 5030 : VCMPPD kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f.w1 c2 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+18253, 543, 5146},
    /* 5031 : VCMPPD kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f.w1 c2 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+18263, 543, 5147},
    /* 5032 : VCMPPD kreg|mask,zmmreg,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f.w1 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPPD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64|SAE,0,0}, nasm_bytecodes+18273, 544, 5148},
    /* 5033 : VCMPPS kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.0f.w0 c2 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+18283, 543, 5149},
    /* 5034 : VCMPPS kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.0f.w0 c2 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+18293, 543, 5150},
    /* 5035 : VCMPPS kreg|mask,zmmreg,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.0f.w0 c2 /r ib       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VCMPPS, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32|SAE,0,0}, nasm_bytecodes+18303, 544, 5151},
    /* 5036 : VCMPSD kreg|mask,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.f2.0f.w1 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(c2) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPSD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+18313, 545, 5152},
    /* 5037 : VCMPSS kreg|mask,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.f3.0f.w0 c2 /r ib    ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(c2) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCMPSS, 4, {OPMASKREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+18323, 545, 5153},
    /* 5038 : VCOMISD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.66.0f.w1 2f /r           ] FL,AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31045, 548, 5154},
    /* 5039 : VCOMISS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.0f.w0 2f /r              ] FL,AVX512,FUTURE */
        /* 250(f1 7c 08) 306 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31054, 548, 5155},
    /* 5040 : VCOMPRESSPD mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31063, 543, 5156},
    /* 5041 : VCOMPRESSPD mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31072, 543, 5157},
    /* 5042 : VCOMPRESSPD mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w1 8a /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31081, 544, 5158},
    /* 5043 : VCOMPRESSPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31090, 543, 5159},
    /* 5044 : VCOMPRESSPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31099, 543, 5160},
    /* 5045 : VCOMPRESSPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 8a /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31108, 544, 5161},
    /* 5046 : VCOMPRESSPS mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31117, 543, 5162},
    /* 5047 : VCOMPRESSPS mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31126, 543, 5163},
    /* 5048 : VCOMPRESSPS mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w0 8a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+31135, 544, 5164},
    /* 5049 : VCOMPRESSPS xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31144, 543, 5165},
    /* 5050 : VCOMPRESSPS ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 8a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(8a) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCOMPRESSPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31153, 543, 5166},
    /* 5051 : VCOMPRESSPS zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 8a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(8a) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VCOMPRESSPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31162, 544, 5167},
    /* 5052 : VCVTDQ2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.f3.0f.w0 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 302 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31171, 543, 5168},
    /* 5053 : VCVTDQ2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.f3.0f.w0 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 302 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31180, 543, 5169},
    /* 5054 : VCVTDQ2PD zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.f3.0f.w0 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 302 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTDQ2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31189, 544, 5170},
    /* 5055 : VCVTDQ2PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 5b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31198, 543, 5171},
    /* 5056 : VCVTDQ2PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 5b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTDQ2PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31207, 543, 5172},
    /* 5057 : VCVTDQ2PS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 5b /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTDQ2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31216, 544, 5173},
    /* 5058 : VCVTPD2DQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f2.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 08) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31225, 543, 5174},
    /* 5059 : VCVTPD2DQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f2.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 28) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2DQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31234, 543, 5175},
    /* 5060 : VCVTPD2DQ ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f2.0f.w1 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 48) 301 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2DQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31243, 544, 5176},
    /* 5061 : VCVTPD2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 5a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31252, 543, 5177},
    /* 5062 : VCVTPD2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 5a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31261, 543, 5178},
    /* 5063 : VCVTPD2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 5a /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31270, 544, 5179},
    /* 5064 : VCVTPD2QQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31279, 546, 5180},
    /* 5065 : VCVTPD2QQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31288, 546, 5181},
    /* 5066 : VCVTPD2QQ zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 7b /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(7b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPD2QQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31297, 547, 5182},
    /* 5067 : VCVTPD2UDQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 08) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31306, 543, 5183},
    /* 5068 : VCVTPD2UDQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 28) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPD2UDQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31315, 543, 5184},
    /* 5069 : VCVTPD2UDQ ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.0f.w1 79 /r              ] AVX512,FUTURE */
        /* 250(f1 fc 48) 301 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPD2UDQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31324, 544, 5185},
    /* 5070 : VCVTPD2UQQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31333, 546, 5186},
    /* 5071 : VCVTPD2UQQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31342, 546, 5187},
    /* 5072 : VCVTPD2UQQ zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 79 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(79) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPD2UQQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31351, 547, 5188},
    /* 5073 : VCVTPH2PS xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31360, 543, 5189},
    /* 5074 : VCVTPH2PS ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31369, 543, 5190},
    /* 5075 : VCVTPH2PS zmmreg|mask|z,ymmrm256|sae [rm:hvm:   evex.512.66.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(13) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPH2PS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+31378, 544, 5191},
    /* 5076 : VCVTPS2DQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31387, 543, 5192},
    /* 5077 : VCVTPS2DQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31396, 543, 5193},
    /* 5078 : VCVTPS2DQ zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.0f.w0 5b /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2DQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31405, 544, 5194},
    /* 5079 : VCVTPS2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.0f.w0 5a /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 302 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31414, 543, 5195},
    /* 5080 : VCVTPS2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.0f.w0 5a /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 302 001(5a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31423, 543, 5196},
    /* 5081 : VCVTPS2PD zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.0f.w0 5a /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 302 001(5a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31432, 544, 5197},
    /* 5082 : VCVTPS2PH xmmreg|mask|z,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18333, 543, 5198},
    /* 5083 : VCVTPS2PH xmmreg|mask|z,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18343, 543, 5199},
    /* 5084 : VCVTPS2PH ymmreg|mask|z,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+18353, 544, 5200},
    /* 5085 : VCVTPS2PH mem64|mask,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18333, 543, 5201},
    /* 5086 : VCVTPS2PH mem128|mask,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18343, 543, 5202},
    /* 5087 : VCVTPS2PH mem256|mask,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,SAE,0,0,0}, nasm_bytecodes+18353, 544, 5203},
    /* 5088 : VCVTPS2QQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31441, 546, 5204},
    /* 5089 : VCVTPS2QQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 7b /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31450, 546, 5205},
    /* 5090 : VCVTPS2QQ zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.66.0f.w0 7b /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(7b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPS2QQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31459, 547, 5206},
    /* 5091 : VCVTPS2UDQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31468, 543, 5207},
    /* 5092 : VCVTPS2UDQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 79 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(79) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2UDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31477, 543, 5208},
    /* 5093 : VCVTPS2UDQ zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 79 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(79) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2UDQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31486, 544, 5209},
    /* 5094 : VCVTPS2UQQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31495, 546, 5210},
    /* 5095 : VCVTPS2UQQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 79 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31504, 546, 5211},
    /* 5096 : VCVTPS2UQQ zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.66.0f.w0 79 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(79) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTPS2UQQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+31513, 547, 5212},
    /* 5097 : VCVTQQ2PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f3.0f.w1 e6 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 08) 301 001(e6) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31522, 546, 5213},
    /* 5098 : VCVTQQ2PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f3.0f.w1 e6 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 28) 301 001(e6) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31531, 546, 5214},
    /* 5099 : VCVTQQ2PD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f3.0f.w1 e6 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fe 48) 301 001(e6) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTQQ2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31540, 547, 5215},
    /* 5100 : VCVTQQ2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 5b /r              ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fc 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31549, 546, 5216},
    /* 5101 : VCVTQQ2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 5b /r              ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fc 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31558, 546, 5217},
    /* 5102 : VCVTQQ2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.0f.w1 5b /r              ] AVX512DQ,FUTURE */
        /* 250(f1 fc 48) 301 001(5b) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTQQ2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+31567, 547, 5218},
    /* 5103 : VCVTSD2SI reg32,xmmrm64|er [rm:t1f64: evex.lig.f2.0f.w0 2d /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(2d) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31576, 545, 5219},
    /* 5104 : VCVTSD2SI reg64,xmmrm64|er [rm:t1f64: evex.lig.f2.0f.w1 2d /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(2d) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31585, 545, 5220},
    /* 5105 : VCVTSD2SS xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5a /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2SS, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+31594, 545, 5221},
    /* 5106 : VCVTSD2USI reg32,xmmrm64|er [rm:t1f64: evex.lig.f2.0f.w0 79 /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(79) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31612, 545, 5222},
    /* 5107 : VCVTSD2USI reg64,xmmrm64|er [rm:t1f64: evex.lig.f2.0f.w1 79 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(79) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSD2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31621, 545, 5223},
    /* 5108 : VCVTSI2SD xmmreg,xmmreg*,rm32 [rvm:t1s:  evex.nds.lig.f2.0f.w0 2a /r       ] AVX512,FUTURE */
        /* 241(f1 7f 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+31630, 545, 5224},
    /* 5109 : VCVTSI2SD xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 2a /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31648, 545, 5225},
    /* 5110 : VCVTSI2SS xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 2a /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31666, 545, 5226},
    /* 5111 : VCVTSI2SS xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.0f.w1 2a /r       ] AVX512,FUTURE */
        /* 241(f1 fe 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+31684, 545, 5227},
    /* 5112 : VCVTSS2SD xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5a /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SD, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+31702, 545, 5228},
    /* 5113 : VCVTSS2SI reg32,xmmrm32|er [rm:t1f32: evex.lig.f3.0f.w0 2d /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(2d) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31720, 545, 5229},
    /* 5114 : VCVTSS2SI reg64,xmmrm32|er [rm:t1f32: evex.lig.f3.0f.w1 2d /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(2d) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31729, 545, 5230},
    /* 5115 : VCVTSS2USI reg32,xmmrm32|er [rm:t1f32: evex.lig.f3.0f.w0 79 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(79) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31738, 545, 5231},
    /* 5116 : VCVTSS2USI reg64,xmmrm32|er [rm:t1f32: evex.lig.f3.0f.w1 79 /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(79) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTSS2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+31747, 545, 5232},
    /* 5117 : VCVTTPD2DQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31756, 543, 5233},
    /* 5118 : VCVTTPD2DQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 e6 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(e6) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2DQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31765, 543, 5234},
    /* 5119 : VCVTTPD2DQ ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 e6 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(e6) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPD2DQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31774, 544, 5235},
    /* 5120 : VCVTTPD2QQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31783, 546, 5236},
    /* 5121 : VCVTTPD2QQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31792, 546, 5237},
    /* 5122 : VCVTTPD2QQ zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPD2QQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31801, 547, 5238},
    /* 5123 : VCVTTPD2UDQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.0f.w1 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 08) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31810, 543, 5239},
    /* 5124 : VCVTTPD2UDQ xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.0f.w1 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fc 28) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPD2UDQ, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31819, 543, 5240},
    /* 5125 : VCVTTPD2UDQ ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.0f.w1 78 /r              ] AVX512,FUTURE */
        /* 250(f1 fc 48) 301 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPD2UDQ, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31828, 544, 5241},
    /* 5126 : VCVTTPD2UQQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 08) 301 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31837, 546, 5242},
    /* 5127 : VCVTTPD2UQQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fd 28) 301 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+31846, 546, 5243},
    /* 5128 : VCVTTPD2UQQ zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f.w1 78 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fd 48) 301 001(78) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPD2UQQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+31855, 547, 5244},
    /* 5129 : VCVTTPS2DQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f3.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2DQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31864, 543, 5245},
    /* 5130 : VCVTTPS2DQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f3.0f.w0 5b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 301 001(5b) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2DQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31873, 543, 5246},
    /* 5131 : VCVTTPS2DQ zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.f3.0f.w0 5b /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 301 001(5b) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPS2DQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31882, 544, 5247},
    /* 5132 : VCVTTPS2QQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31891, 546, 5248},
    /* 5133 : VCVTTPS2QQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31900, 546, 5249},
    /* 5134 : VCVTTPS2QQ zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.0f.w0 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPS2QQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31909, 547, 5250},
    /* 5135 : VCVTTPS2UDQ xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2UDQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31918, 543, 5251},
    /* 5136 : VCVTTPS2UDQ ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 78 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(78) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTTPS2UDQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31927, 543, 5252},
    /* 5137 : VCVTTPS2UDQ zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.0f.w0 78 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(78) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTPS2UDQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31936, 544, 5253},
    /* 5138 : VCVTTPS2UQQ xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.0f.w0 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 08) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31945, 546, 5254},
    /* 5139 : VCVTTPS2UQQ ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.0f.w0 78 /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 7d 28) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+31954, 546, 5255},
    /* 5140 : VCVTTPS2UQQ zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.0f.w0 78 /r           ] AVX512DQ,FUTURE */
        /* 250(f1 7d 48) 302 001(78) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTTPS2UQQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+31963, 547, 5256},
    /* 5141 : VCVTTSD2SI reg32,xmmrm64|sae [rm:t1f64: evex.lig.f2.0f.w0 2c /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(2c) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31972, 545, 5257},
    /* 5142 : VCVTTSD2SI reg64,xmmrm64|sae [rm:t1f64: evex.lig.f2.0f.w1 2c /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(2c) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31981, 545, 5258},
    /* 5143 : VCVTTSD2USI reg32,xmmrm64|sae [rm:t1f64: evex.lig.f2.0f.w0 78 /r           ] AVX512,FUTURE */
        /* 250(f1 7f 08) 310 001(78) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31990, 545, 5259},
    /* 5144 : VCVTTSD2USI reg64,xmmrm64|sae [rm:t1f64: evex.lig.f2.0f.w1 78 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 310 001(78) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSD2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+31999, 545, 5260},
    /* 5145 : VCVTTSS2SI reg32,xmmrm32|sae [rm:t1f32: evex.lig.f3.0f.w0 2c /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(2c) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32008, 545, 5261},
    /* 5146 : VCVTTSS2SI reg64,xmmrm32|sae [rm:t1f32: evex.lig.f3.0f.w1 2c /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(2c) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32017, 545, 5262},
    /* 5147 : VCVTTSS2USI reg32,xmmrm32|sae [rm:t1f32: evex.lig.f3.0f.w0 78 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 307 001(78) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32026, 545, 5263},
    /* 5148 : VCVTTSS2USI reg64,xmmrm32|sae [rm:t1f32: evex.lig.f3.0f.w1 78 /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 307 001(78) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTTSS2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+32035, 545, 5264},
    /* 5149 : VCVTUDQ2PD xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.f3.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 302 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32044, 543, 5265},
    /* 5150 : VCVTUDQ2PD ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.f3.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 302 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32053, 543, 5266},
    /* 5151 : VCVTUDQ2PD zmmreg|mask|z,ymmrm256|b32|er [rm:hv:    evex.512.f3.0f.w0 7a /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 302 001(7a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUDQ2PD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+32062, 544, 5267},
    /* 5152 : VCVTUDQ2PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7f 08) 301 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32071, 543, 5268},
    /* 5153 : VCVTUDQ2PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.0f.w0 7a /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7f 28) 301 001(7a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTUDQ2PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+32080, 543, 5269},
    /* 5154 : VCVTUDQ2PS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.f2.0f.w0 7a /r           ] AVX512,FUTURE */
        /* 250(f1 7f 48) 301 001(7a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUDQ2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+32089, 544, 5270},
    /* 5155 : VCVTUQQ2PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f3.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32098, 546, 5271},
    /* 5156 : VCVTUQQ2PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f3.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 fe 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32107, 546, 5272},
    /* 5157 : VCVTUQQ2PD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f3.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 fe 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+32116, 547, 5273},
    /* 5158 : VCVTUQQ2PS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.f2.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 ff 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32125, 546, 5274},
    /* 5159 : VCVTUQQ2PS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.f2.0f.w1 7a /r           ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f1 ff 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+32134, 546, 5275},
    /* 5160 : VCVTUQQ2PS ymmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.f2.0f.w1 7a /r           ] AVX512DQ,FUTURE */
        /* 250(f1 ff 48) 301 001(7a) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VCVTUQQ2PS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+32143, 547, 5276},
    /* 5161 : VCVTUSI2SD xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f2.0f.w0 7b /r       ] AVX512,FUTURE */
        /* 241(f1 7f 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+32152, 545, 5277},
    /* 5162 : VCVTUSI2SD xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 7b /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SD, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+32170, 545, 5278},
    /* 5163 : VCVTUSI2SS xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 7b /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+32188, 545, 5279},
    /* 5164 : VCVTUSI2SS xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.0f.w1 7b /r       ] AVX512,FUTURE */
        /* 241(f1 fe 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VCVTUSI2SS, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+32206, 545, 5280},
    /* 5165 : VDBPSADBW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 42 /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(42) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18363, 549, 5281},
    /* 5166 : VDBPSADBW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 42 /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(42) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18383, 549, 5282},
    /* 5167 : VDBPSADBW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 42 /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(42) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VDBPSADBW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18403, 550, 5283},
    /* 5168 : VDIVPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5e /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32224, 543, 5284},
    /* 5169 : VDIVPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5e /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32242, 543, 5285},
    /* 5170 : VDIVPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 5e /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VDIVPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32260, 544, 5286},
    /* 5171 : VDIVPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5e /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32278, 543, 5287},
    /* 5172 : VDIVPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5e /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VDIVPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32296, 543, 5288},
    /* 5173 : VDIVPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 5e /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VDIVPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32314, 544, 5289},
    /* 5174 : VDIVSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5e /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5e) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VDIVSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32332, 545, 5290},
    /* 5175 : VDIVSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 5e /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5e) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VDIVSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32350, 545, 5291},
    /* 5176 : VEXP2PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 c8 /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(c8) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VEXP2PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+32368, 551, 5292},
    /* 5177 : VEXP2PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 c8 /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(c8) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VEXP2PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+32377, 551, 5293},
    /* 5178 : VEXPANDPD xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32386, 543, 5294},
    /* 5179 : VEXPANDPD ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32395, 543, 5295},
    /* 5180 : VEXPANDPD zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w1 88 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPD, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32404, 544, 5296},
    /* 5181 : VEXPANDPD xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32386, 543, 5297},
    /* 5182 : VEXPANDPD ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w1 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32395, 543, 5298},
    /* 5183 : VEXPANDPD zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w1 88 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32404, 544, 5299},
    /* 5184 : VEXPANDPS xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32413, 543, 5300},
    /* 5185 : VEXPANDPS ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32422, 543, 5301},
    /* 5186 : VEXPANDPS zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w0 88 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPS, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32431, 544, 5302},
    /* 5187 : VEXPANDPS xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32413, 543, 5303},
    /* 5188 : VEXPANDPS ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w0 88 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(88) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXPANDPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32422, 543, 5304},
    /* 5189 : VEXPANDPS zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w0 88 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(88) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXPANDPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+32431, 544, 5305},
    /* 5190 : VEXTRACTF32X4 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w0 19 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 300 001(19) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTF32X4, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18423, 543, 5306},
    /* 5191 : VEXTRACTF32X4 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 19 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(19) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF32X4, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18433, 544, 5307},
    /* 5192 : VEXTRACTF32X4 mem128|mask,ymmreg,imm8 [mri:t4:   evex.256.66.0f3a.w0 19 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 312 001(19) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTF32X4, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18443, 543, 5308},
    /* 5193 : VEXTRACTF32X4 mem128|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w0 19 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 312 001(19) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF32X4, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18453, 544, 5309},
    /* 5194 : VEXTRACTF32X8 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 1b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 300 001(1b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF32X8, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18463, 547, 5310},
    /* 5195 : VEXTRACTF32X8 mem256|mask,zmmreg,imm8 [mri:t8:   evex.512.66.0f3a.w0 1b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 313 001(1b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF32X8, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18473, 547, 5311},
    /* 5196 : VEXTRACTF64X2 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w1 19 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 300 001(19) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18483, 546, 5312},
    /* 5197 : VEXTRACTF64X2 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 19 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 300 001(19) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18493, 547, 5313},
    /* 5198 : VEXTRACTF64X2 mem128|mask,ymmreg,imm8 [mri:t2:   evex.256.66.0f3a.w1 19 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 311 001(19) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18503, 546, 5314},
    /* 5199 : VEXTRACTF64X2 mem128|mask,zmmreg,imm8 [mri:t2:   evex.512.66.0f3a.w1 19 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 311 001(19) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTF64X2, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18513, 547, 5315},
    /* 5200 : VEXTRACTF64X4 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 1b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 300 001(1b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF64X4, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18523, 544, 5316},
    /* 5201 : VEXTRACTF64X4 mem256|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w1 1b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 312 001(1b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTF64X4, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18533, 544, 5317},
    /* 5202 : VEXTRACTI32X4 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w0 39 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 300 001(39) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTI32X4, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18543, 543, 5318},
    /* 5203 : VEXTRACTI32X4 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 39 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(39) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI32X4, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18553, 544, 5319},
    /* 5204 : VEXTRACTI32X4 mem128|mask,ymmreg,imm8 [mri:t4:   evex.256.66.0f3a.w0 39 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 312 001(39) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VEXTRACTI32X4, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18563, 543, 5320},
    /* 5205 : VEXTRACTI32X4 mem128|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w0 39 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 312 001(39) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI32X4, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18573, 544, 5321},
    /* 5206 : VEXTRACTI32X8 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w0 3b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 300 001(3b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI32X8, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18583, 547, 5322},
    /* 5207 : VEXTRACTI32X8 mem256|mask,zmmreg,imm8 [mri:t8:   evex.512.66.0f3a.w0 3b /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 313 001(3b) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI32X8, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18593, 547, 5323},
    /* 5208 : VEXTRACTI64X2 xmmreg|mask|z,ymmreg,imm8 [mri:      evex.256.66.0f3a.w1 39 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 300 001(39) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18603, 546, 5324},
    /* 5209 : VEXTRACTI64X2 xmmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 39 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 300 001(39) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {XMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18613, 547, 5325},
    /* 5210 : VEXTRACTI64X2 mem128|mask,ymmreg,imm8 [mri:t2:   evex.256.66.0f3a.w1 39 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 311 001(39) 101 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18623, 546, 5326},
    /* 5211 : VEXTRACTI64X2 mem128|mask,zmmreg,imm8 [mri:t2:   evex.512.66.0f3a.w1 39 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 311 001(39) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VEXTRACTI64X2, 3, {MEMORY|BITS128,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18633, 547, 5327},
    /* 5212 : VEXTRACTI64X4 ymmreg|mask|z,zmmreg,imm8 [mri:      evex.512.66.0f3a.w1 3b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 300 001(3b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI64X4, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18643, 544, 5328},
    /* 5213 : VEXTRACTI64X4 mem256|mask,zmmreg,imm8 [mri:t4:   evex.512.66.0f3a.w1 3b /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 312 001(3b) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTI64X4, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18653, 544, 5329},
    /* 5214 : VEXTRACTPS reg32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 552, 5330},
    /* 5215 : VEXTRACTPS reg64,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 552, 5331},
    /* 5216 : VEXTRACTPS mem32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.wig 17 /r ib     ] AVX512,FUTURE */
        /* 250(f3 7d 08) 306 001(17) 101 022 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VEXTRACTPS, 3, {MEMORY|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+18663, 552, 5332},
    /* 5217 : VFIXUPIMMPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18673, 543, 5333},
    /* 5218 : VFIXUPIMMPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+18693, 543, 5334},
    /* 5219 : VFIXUPIMMPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 54 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(54) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+18713, 544, 5335},
    /* 5220 : VFIXUPIMMPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18733, 543, 5336},
    /* 5221 : VFIXUPIMMPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 54 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(54) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFIXUPIMMPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+18753, 543, 5337},
    /* 5222 : VFIXUPIMMPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 54 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(54) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+18773, 544, 5338},
    /* 5223 : VFIXUPIMMSD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 55 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(55) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+18793, 545, 5339},
    /* 5224 : VFIXUPIMMSS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 55 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(55) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFIXUPIMMSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+18813, 545, 5340},
    /* 5225 : VFMADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32440, 543, 5341},
    /* 5226 : VFMADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32449, 543, 5342},
    /* 5227 : VFMADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 98 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(98) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32458, 544, 5343},
    /* 5228 : VFMADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32467, 543, 5344},
    /* 5229 : VFMADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 98 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(98) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32476, 543, 5345},
    /* 5230 : VFMADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 98 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(98) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32485, 544, 5346},
    /* 5231 : VFMADD132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 99 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32494, 545, 5347},
    /* 5232 : VFMADD132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 99 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32503, 545, 5348},
    /* 5233 : VFMADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32512, 543, 5349},
    /* 5234 : VFMADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32521, 543, 5350},
    /* 5235 : VFMADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a8 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32530, 544, 5351},
    /* 5236 : VFMADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32539, 543, 5352},
    /* 5237 : VFMADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32548, 543, 5353},
    /* 5238 : VFMADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a8 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32557, 544, 5354},
    /* 5239 : VFMADD213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 a9 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32566, 545, 5355},
    /* 5240 : VFMADD213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 a9 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32575, 545, 5356},
    /* 5241 : VFMADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32584, 543, 5357},
    /* 5242 : VFMADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32593, 543, 5358},
    /* 5243 : VFMADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b8 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32602, 544, 5359},
    /* 5244 : VFMADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32611, 543, 5360},
    /* 5245 : VFMADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b8 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b8) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32620, 543, 5361},
    /* 5246 : VFMADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b8 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b8) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32629, 544, 5362},
    /* 5247 : VFMADD231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 b9 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32638, 545, 5363},
    /* 5248 : VFMADD231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 b9 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32647, 545, 5364},
    /* 5249 : VFMADDSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32656, 543, 5365},
    /* 5250 : VFMADDSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32665, 543, 5366},
    /* 5251 : VFMADDSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 96 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(96) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32674, 544, 5367},
    /* 5252 : VFMADDSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32683, 543, 5368},
    /* 5253 : VFMADDSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 96 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(96) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32692, 543, 5369},
    /* 5254 : VFMADDSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 96 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(96) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32701, 544, 5370},
    /* 5255 : VFMADDSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32710, 543, 5371},
    /* 5256 : VFMADDSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32719, 543, 5372},
    /* 5257 : VFMADDSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a6 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32728, 544, 5373},
    /* 5258 : VFMADDSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32737, 543, 5374},
    /* 5259 : VFMADDSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32746, 543, 5375},
    /* 5260 : VFMADDSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a6 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32755, 544, 5376},
    /* 5261 : VFMADDSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32764, 543, 5377},
    /* 5262 : VFMADDSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32773, 543, 5378},
    /* 5263 : VFMADDSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b6 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32782, 544, 5379},
    /* 5264 : VFMADDSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32791, 543, 5380},
    /* 5265 : VFMADDSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b6 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b6) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMADDSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32800, 543, 5381},
    /* 5266 : VFMADDSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b6 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b6) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMADDSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32809, 544, 5382},
    /* 5267 : VFMSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32818, 543, 5383},
    /* 5268 : VFMSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32827, 543, 5384},
    /* 5269 : VFMSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9a /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32836, 544, 5385},
    /* 5270 : VFMSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32845, 543, 5386},
    /* 5271 : VFMSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9a /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32854, 543, 5387},
    /* 5272 : VFMSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9a /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32863, 544, 5388},
    /* 5273 : VFMSUB132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9b /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32872, 545, 5389},
    /* 5274 : VFMSUB132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9b /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32881, 545, 5390},
    /* 5275 : VFMSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32890, 543, 5391},
    /* 5276 : VFMSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32899, 543, 5392},
    /* 5277 : VFMSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 aa /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(aa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32908, 544, 5393},
    /* 5278 : VFMSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32917, 543, 5394},
    /* 5279 : VFMSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 aa /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(aa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32926, 543, 5395},
    /* 5280 : VFMSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 aa /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(aa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+32935, 544, 5396},
    /* 5281 : VFMSUB213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 ab /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32944, 545, 5397},
    /* 5282 : VFMSUB213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 ab /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+32953, 545, 5398},
    /* 5283 : VFMSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32962, 543, 5399},
    /* 5284 : VFMSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+32971, 543, 5400},
    /* 5285 : VFMSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ba /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ba) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+32980, 544, 5401},
    /* 5286 : VFMSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32989, 543, 5402},
    /* 5287 : VFMSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ba /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ba) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+32998, 543, 5403},
    /* 5288 : VFMSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ba /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ba) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33007, 544, 5404},
    /* 5289 : VFMSUB231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bb /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33016, 545, 5405},
    /* 5290 : VFMSUB231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bb /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33025, 545, 5406},
    /* 5291 : VFMSUBADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33034, 543, 5407},
    /* 5292 : VFMSUBADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33043, 543, 5408},
    /* 5293 : VFMSUBADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 97 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(97) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33052, 544, 5409},
    /* 5294 : VFMSUBADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33061, 543, 5410},
    /* 5295 : VFMSUBADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 97 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(97) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33070, 543, 5411},
    /* 5296 : VFMSUBADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 97 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(97) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33079, 544, 5412},
    /* 5297 : VFMSUBADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33088, 543, 5413},
    /* 5298 : VFMSUBADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33097, 543, 5414},
    /* 5299 : VFMSUBADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 a7 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(a7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33106, 544, 5415},
    /* 5300 : VFMSUBADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33115, 543, 5416},
    /* 5301 : VFMSUBADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 a7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(a7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33124, 543, 5417},
    /* 5302 : VFMSUBADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 a7 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(a7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33133, 544, 5418},
    /* 5303 : VFMSUBADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33142, 543, 5419},
    /* 5304 : VFMSUBADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33151, 543, 5420},
    /* 5305 : VFMSUBADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 b7 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(b7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33160, 544, 5421},
    /* 5306 : VFMSUBADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33169, 543, 5422},
    /* 5307 : VFMSUBADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 b7 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(b7) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFMSUBADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33178, 543, 5423},
    /* 5308 : VFMSUBADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 b7 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(b7) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFMSUBADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33187, 544, 5424},
    /* 5309 : VFNMADD132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33196, 543, 5425},
    /* 5310 : VFNMADD132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33205, 543, 5426},
    /* 5311 : VFNMADD132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9c /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33214, 544, 5427},
    /* 5312 : VFNMADD132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33223, 543, 5428},
    /* 5313 : VFNMADD132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33232, 543, 5429},
    /* 5314 : VFNMADD132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33241, 544, 5430},
    /* 5315 : VFNMADD132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33250, 545, 5431},
    /* 5316 : VFNMADD132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33259, 545, 5432},
    /* 5317 : VFNMADD213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33268, 543, 5433},
    /* 5318 : VFNMADD213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33277, 543, 5434},
    /* 5319 : VFNMADD213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ac /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ac) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33286, 544, 5435},
    /* 5320 : VFNMADD213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33295, 543, 5436},
    /* 5321 : VFNMADD213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ac /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ac) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33304, 543, 5437},
    /* 5322 : VFNMADD213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ac /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ac) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33313, 544, 5438},
    /* 5323 : VFNMADD213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 ad /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33322, 545, 5439},
    /* 5324 : VFNMADD213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 ad /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33331, 545, 5440},
    /* 5325 : VFNMADD231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33340, 543, 5441},
    /* 5326 : VFNMADD231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33349, 543, 5442},
    /* 5327 : VFNMADD231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 bc /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(bc) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33358, 544, 5443},
    /* 5328 : VFNMADD231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33367, 543, 5444},
    /* 5329 : VFNMADD231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 bc /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(bc) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMADD231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33376, 543, 5445},
    /* 5330 : VFNMADD231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 bc /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(bc) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33385, 544, 5446},
    /* 5331 : VFNMADD231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bd /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33394, 545, 5447},
    /* 5332 : VFNMADD231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bd /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMADD231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33403, 545, 5448},
    /* 5333 : VFNMSUB132PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33412, 543, 5449},
    /* 5334 : VFNMSUB132PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33421, 543, 5450},
    /* 5335 : VFNMSUB132PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 9e /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(9e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33430, 544, 5451},
    /* 5336 : VFNMSUB132PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33439, 543, 5452},
    /* 5337 : VFNMSUB132PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 9e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(9e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB132PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33448, 543, 5453},
    /* 5338 : VFNMSUB132PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 9e /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(9e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33457, 544, 5454},
    /* 5339 : VFNMSUB132SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 9f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33466, 545, 5455},
    /* 5340 : VFNMSUB132SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 9f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB132SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33475, 545, 5456},
    /* 5341 : VFNMSUB213PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33484, 543, 5457},
    /* 5342 : VFNMSUB213PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33493, 543, 5458},
    /* 5343 : VFNMSUB213PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 ae /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(ae) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33502, 544, 5459},
    /* 5344 : VFNMSUB213PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33511, 543, 5460},
    /* 5345 : VFNMSUB213PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 ae /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(ae) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB213PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33520, 543, 5461},
    /* 5346 : VFNMSUB213PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 ae /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(ae) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33529, 544, 5462},
    /* 5347 : VFNMSUB213SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 af /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33538, 545, 5463},
    /* 5348 : VFNMSUB213SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 af /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB213SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33547, 545, 5464},
    /* 5349 : VFNMSUB231PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33556, 543, 5465},
    /* 5350 : VFNMSUB231PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33565, 543, 5466},
    /* 5351 : VFNMSUB231PD zmmreg|mask|z,zmmreg,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 be /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(be) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+33574, 544, 5467},
    /* 5352 : VFNMSUB231PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33583, 543, 5468},
    /* 5353 : VFNMSUB231PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 be /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(be) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VFNMSUB231PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33592, 543, 5469},
    /* 5354 : VFNMSUB231PS zmmreg|mask|z,zmmreg,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 be /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(be) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+33601, 544, 5470},
    /* 5355 : VFNMSUB231SD xmmreg|mask|z,xmmreg,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 bf /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33610, 545, 5471},
    /* 5356 : VFNMSUB231SS xmmreg|mask|z,xmmreg,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 bf /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VFNMSUB231SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+33619, 545, 5472},
    /* 5357 : VFPCLASSPD kreg|mask,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18833, 546, 5473},
    /* 5358 : VFPCLASSPD kreg|mask,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18843, 546, 5474},
    /* 5359 : VFPCLASSPD kreg|mask,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 66 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 301 001(66) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSPD, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B64,0,0,0}, nasm_bytecodes+18853, 547, 5475},
    /* 5360 : VFPCLASSPS kreg|mask,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18863, 546, 5476},
    /* 5361 : VFPCLASSPS kreg|mask,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 66 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18873, 546, 5477},
    /* 5362 : VFPCLASSPS kreg|mask,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f3a.w0 66 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 301 001(66) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSPS, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B32,0,0,0}, nasm_bytecodes+18883, 547, 5478},
    /* 5363 : VFPCLASSSD kreg|mask,xmmrm64,imm8 [rmi:t1s:  evex.lig.66.0f3a.w1 67 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSSD, 3, {OPMASKREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18893, 553, 5479},
    /* 5364 : VFPCLASSSS kreg|mask,xmmrm32,imm8 [rmi:t1s:  evex.lig.66.0f3a.w0 67 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VFPCLASSSS, 3, {OPMASKREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18903, 553, 5480},
    /* 5365 : VGATHERDPD xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPD, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18913, 543, 5481},
    /* 5366 : VGATHERDPD ymmreg|mask,xmem64 [rm:t1s:   vsibx evex.256.66.0f38.w1 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPD, 2, {YMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18923, 543, 5482},
    /* 5367 : VGATHERDPD zmmreg|mask,ymem64 [rm:t1s:   vsiby evex.512.66.0f38.w1 92 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(92) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERDPD, 2, {ZMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18933, 544, 5483},
    /* 5368 : VGATHERDPS xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPS, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18943, 543, 5484},
    /* 5369 : VGATHERDPS ymmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 92 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(92) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERDPS, 2, {YMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18953, 543, 5485},
    /* 5370 : VGATHERDPS zmmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 92 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(92) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERDPS, 2, {ZMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18963, 544, 5486},
    /* 5371 : VGATHERPF0DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /1   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18973, 554, 5487},
    /* 5372 : VGATHERPF0DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18983, 554, 5488},
    /* 5373 : VGATHERPF0QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18993, 554, 5489},
    /* 5374 : VGATHERPF0QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /1   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 201 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF0QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19003, 554, 5490},
    /* 5375 : VGATHERPF1DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /2   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19013, 554, 5491},
    /* 5376 : VGATHERPF1DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19023, 554, 5492},
    /* 5377 : VGATHERPF1QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19033, 554, 5493},
    /* 5378 : VGATHERPF1QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /2   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 202 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VGATHERPF1QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19043, 554, 5494},
    /* 5379 : VGATHERQPD xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPD, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19053, 543, 5495},
    /* 5380 : VGATHERQPD ymmreg|mask,ymem64 [rm:t1s:   vsiby evex.256.66.0f38.w1 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPD, 2, {YMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19063, 543, 5496},
    /* 5381 : VGATHERQPD zmmreg|mask,zmem64 [rm:t1s:   vsibz evex.512.66.0f38.w1 93 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(93) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERQPD, 2, {ZMMREG,ZMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19073, 544, 5497},
    /* 5382 : VGATHERQPS xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPS, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19083, 543, 5498},
    /* 5383 : VGATHERQPS xmmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 93 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(93) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGATHERQPS, 2, {XMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19093, 543, 5499},
    /* 5384 : VGATHERQPS ymmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 93 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(93) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGATHERQPS, 2, {YMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19103, 544, 5500},
    /* 5385 : VGETEXPPD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+33628, 543, 5501},
    /* 5386 : VGETEXPPD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+33637, 543, 5502},
    /* 5387 : VGETEXPPD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 42 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(42) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+33646, 544, 5503},
    /* 5388 : VGETEXPPS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+33655, 543, 5504},
    /* 5389 : VGETEXPPS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 42 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(42) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETEXPPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+33664, 543, 5505},
    /* 5390 : VGETEXPPS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 42 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(42) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+33673, 544, 5506},
    /* 5391 : VGETEXPSD xmmreg|mask|z,xmmreg,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 43 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(43) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33682, 545, 5507},
    /* 5392 : VGETEXPSS xmmreg|mask|z,xmmreg,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 43 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(43) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETEXPSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33691, 545, 5508},
    /* 5393 : VGETMANTPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19113, 543, 5509},
    /* 5394 : VGETMANTPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19123, 543, 5510},
    /* 5395 : VGETMANTPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 26 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(26) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+19133, 544, 5511},
    /* 5396 : VGETMANTPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19143, 543, 5512},
    /* 5397 : VGETMANTPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 26 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(26) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VGETMANTPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19153, 543, 5513},
    /* 5398 : VGETMANTPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 26 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(26) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+19163, 544, 5514},
    /* 5399 : VGETMANTSD xmmreg|mask|z,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 27 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(27) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+19173, 545, 5515},
    /* 5400 : VGETMANTSS xmmreg|mask|z,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 27 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(27) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VGETMANTSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+19183, 545, 5516},
    /* 5401 : VINSERTF32X4 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.256.66.0f3a.w0 18 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 312 001(18) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VINSERTF32X4, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19193, 543, 5517},
    /* 5402 : VINSERTF32X4 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w0 18 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 312 001(18) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTF32X4, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19213, 544, 5518},
    /* 5403 : VINSERTF32X8 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t8:  evex.nds.512.66.0f3a.w0 1a /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 313 001(1a) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTF32X8, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19233, 547, 5519},
    /* 5404 : VINSERTF64X2 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.256.66.0f3a.w1 18 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 311 001(18) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VINSERTF64X2, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19253, 546, 5520},
    /* 5405 : VINSERTF64X2 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.512.66.0f3a.w1 18 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 311 001(18) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTF64X2, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19273, 547, 5521},
    /* 5406 : VINSERTF64X4 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w1 1a /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 312 001(1a) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTF64X4, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19293, 544, 5522},
    /* 5407 : VINSERTI32X4 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.256.66.0f3a.w0 38 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 312 001(38) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VINSERTI32X4, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19313, 543, 5523},
    /* 5408 : VINSERTI32X4 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w0 38 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 312 001(38) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTI32X4, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19333, 544, 5524},
    /* 5409 : VINSERTI32X8 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t8:  evex.nds.512.66.0f3a.w0 3a /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 313 001(3a) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTI32X8, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19353, 547, 5525},
    /* 5410 : VINSERTI64X2 ymmreg|mask|z,ymmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.256.66.0f3a.w1 38 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 311 001(38) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VINSERTI64X2, 4, {YMMREG,YMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19373, 546, 5526},
    /* 5411 : VINSERTI64X2 zmmreg|mask|z,zmmreg*,xmmrm128,imm8 [rvmi:t2:  evex.nds.512.66.0f3a.w1 38 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 311 001(38) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VINSERTI64X2, 4, {ZMMREG,ZMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19393, 547, 5527},
    /* 5412 : VINSERTI64X4 zmmreg|mask|z,zmmreg*,ymmrm256,imm8 [rvmi:t4:  evex.nds.512.66.0f3a.w1 3a /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 312 001(3a) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTI64X4, 4, {ZMMREG,ZMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19413, 544, 5528},
    /* 5413 : VINSERTPS xmmreg,xmmreg*,xmmrm32,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w0 21 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(21) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VINSERTPS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+19433, 544, 5529},
    /* 5414 : VMAXPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5f /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33700, 543, 5530},
    /* 5415 : VMAXPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5f /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33718, 543, 5531},
    /* 5416 : VMAXPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae [rvm:fv:   evex.nds.512.66.0f.w1 5f /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMAXPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+33736, 544, 5532},
    /* 5417 : VMAXPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5f /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33754, 543, 5533},
    /* 5418 : VMAXPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5f /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMAXPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33772, 543, 5534},
    /* 5419 : VMAXPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae [rvm:fv:   evex.nds.512.0f.w0 5f /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMAXPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+33790, 544, 5535},
    /* 5420 : VMAXSD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.f2.0f.w1 5f /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33808, 545, 5536},
    /* 5421 : VMAXSS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5f /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMAXSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33826, 545, 5537},
    /* 5422 : VMINPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33844, 543, 5538},
    /* 5423 : VMINPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+33862, 543, 5539},
    /* 5424 : VMINPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae [rvm:fv:   evex.nds.512.66.0f.w1 5d /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMINPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+33880, 544, 5540},
    /* 5425 : VMINPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5d /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33898, 543, 5541},
    /* 5426 : VMINPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5d /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMINPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+33916, 543, 5542},
    /* 5427 : VMINPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae [rvm:fv:   evex.nds.512.0f.w0 5d /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMINPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+33934, 544, 5543},
    /* 5428 : VMINSD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.f2.0f.w1 5d /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33952, 545, 5544},
    /* 5429 : VMINSS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.f3.0f.w0 5d /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMINSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+33970, 545, 5545},
    /* 5430 : VMOVAPD xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 28 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+33988, 543, 5546},
    /* 5431 : VMOVAPD ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 28 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+33997, 543, 5547},
    /* 5432 : VMOVAPD zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 28 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(28) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34006, 544, 5548},
    /* 5433 : VMOVAPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34015, 543, 5549},
    /* 5434 : VMOVAPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34024, 543, 5550},
    /* 5435 : VMOVAPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f.w1 29 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 300 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34033, 544, 5551},
    /* 5436 : VMOVAPD mem128|mask,xmmreg [mr:fvm:   evex.128.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34042, 543, 5552},
    /* 5437 : VMOVAPD mem256|mask,ymmreg [mr:fvm:   evex.256.66.0f.w1 29 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34051, 543, 5553},
    /* 5438 : VMOVAPD mem512|mask,zmmreg [mr:fvm:   evex.512.66.0f.w1 29 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34060, 544, 5554},
    /* 5439 : VMOVAPS xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.0f.w0 28 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34069, 543, 5555},
    /* 5440 : VMOVAPS ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.0f.w0 28 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(28) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34078, 543, 5556},
    /* 5441 : VMOVAPS zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.0f.w0 28 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(28) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34087, 544, 5557},
    /* 5442 : VMOVAPS xmmreg|mask|z,xmmreg [mr:       evex.128.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34096, 543, 5558},
    /* 5443 : VMOVAPS ymmreg|mask|z,ymmreg [mr:       evex.256.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 300 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34105, 543, 5559},
    /* 5444 : VMOVAPS zmmreg|mask|z,zmmreg [mr:       evex.512.0f.w0 29 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 300 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34114, 544, 5560},
    /* 5445 : VMOVAPS mem128|mask,xmmreg [mr:fvm:   evex.128.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34123, 543, 5561},
    /* 5446 : VMOVAPS mem256|mask,ymmreg [mr:fvm:   evex.256.0f.w0 29 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(29) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34132, 543, 5562},
    /* 5447 : VMOVAPS mem512|mask,zmmreg [mr:fvm:   evex.512.0f.w0 29 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(29) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVAPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34141, 544, 5563},
    /* 5448 : VMOVD xmmreg,rm32 [rm:t1s:   evex.128.66.0f.w0 6e /r           ] AVX512,FUTURE */
        /* 250(f1 7d 08) 306 001(6e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVD, 2, {XMMREG,RM_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+34150, 544, 5564},
    /* 5449 : VMOVD rm32,xmmreg [mr:t1s:   evex.128.66.0f.w0 7e /r           ] AVX512,FUTURE */
        /* 250(f1 7d 08) 306 001(7e) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVD, 2, {RM_GPR|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34159, 544, 5565},
    /* 5450 : VMOVDDUP xmmreg|mask|z,xmmrm64 [rm:dup:   evex.128.f2.0f.w1 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 08) 320 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDDUP, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34168, 543, 5566},
    /* 5451 : VMOVDDUP ymmreg|mask|z,ymmrm256 [rm:dup:   evex.256.f2.0f.w1 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 ff 28) 320 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34177, 543, 5567},
    /* 5452 : VMOVDDUP zmmreg|mask|z,zmmrm512 [rm:dup:   evex.512.f2.0f.w1 12 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 48) 320 001(12) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34186, 544, 5568},
    /* 5453 : VMOVDQA32 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34195, 543, 5569},
    /* 5454 : VMOVDQA32 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34204, 543, 5570},
    /* 5455 : VMOVDQA32 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w0 6f /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA32, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34213, 544, 5571},
    /* 5456 : VMOVDQA32 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.66.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34222, 543, 5572},
    /* 5457 : VMOVDQA32 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.66.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA32, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34231, 543, 5573},
    /* 5458 : VMOVDQA32 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.66.0f.w0 7f /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA32, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34240, 544, 5574},
    /* 5459 : VMOVDQA64 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34249, 543, 5575},
    /* 5460 : VMOVDQA64 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34258, 543, 5576},
    /* 5461 : VMOVDQA64 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 6f /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA64, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34267, 544, 5577},
    /* 5462 : VMOVDQA64 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.66.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34276, 543, 5578},
    /* 5463 : VMOVDQA64 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.66.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQA64, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34285, 543, 5579},
    /* 5464 : VMOVDQA64 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.66.0f.w1 7f /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQA64, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34294, 544, 5580},
    /* 5465 : VMOVDQU16 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f2.0f.w1 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 08) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34303, 549, 5581},
    /* 5466 : VMOVDQU16 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f2.0f.w1 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 28) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34312, 549, 5582},
    /* 5467 : VMOVDQU16 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f2.0f.w1 6f /r           ] AVX512BW,FUTURE */
        /* 250(f1 ff 48) 303 001(6f) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34321, 550, 5583},
    /* 5468 : VMOVDQU16 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f2.0f.w1 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 08) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34330, 549, 5584},
    /* 5469 : VMOVDQU16 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f2.0f.w1 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 ff 28) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34339, 549, 5585},
    /* 5470 : VMOVDQU16 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f2.0f.w1 7f /r           ] AVX512BW,FUTURE */
        /* 250(f1 ff 48) 303 001(7f) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU16, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34348, 550, 5586},
    /* 5471 : VMOVDQU32 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34357, 543, 5587},
    /* 5472 : VMOVDQU32 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34366, 543, 5588},
    /* 5473 : VMOVDQU32 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 6f /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU32, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34375, 544, 5589},
    /* 5474 : VMOVDQU32 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f3.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34384, 543, 5590},
    /* 5475 : VMOVDQU32 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f3.0f.w0 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU32, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34393, 543, 5591},
    /* 5476 : VMOVDQU32 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f3.0f.w0 7f /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU32, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34402, 544, 5592},
    /* 5477 : VMOVDQU64 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 08) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34411, 543, 5593},
    /* 5478 : VMOVDQU64 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w1 6f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 28) 303 001(6f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34420, 543, 5594},
    /* 5479 : VMOVDQU64 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w1 6f /r           ] AVX512,FUTURE */
        /* 250(f1 fe 48) 303 001(6f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU64, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34429, 544, 5595},
    /* 5480 : VMOVDQU64 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f3.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 08) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34438, 543, 5596},
    /* 5481 : VMOVDQU64 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f3.0f.w1 7f /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fe 28) 303 001(7f) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVDQU64, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34447, 543, 5597},
    /* 5482 : VMOVDQU64 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f3.0f.w1 7f /r           ] AVX512,FUTURE */
        /* 250(f1 fe 48) 303 001(7f) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVDQU64, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34456, 544, 5598},
    /* 5483 : VMOVDQU8 xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f2.0f.w0 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34465, 549, 5599},
    /* 5484 : VMOVDQU8 ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f2.0f.w0 6f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(6f) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34474, 549, 5600},
    /* 5485 : VMOVDQU8 zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f2.0f.w0 6f /r           ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(6f) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34483, 550, 5601},
    /* 5486 : VMOVDQU8 xmmrm128|mask|z,xmmreg [mr:fvm:   evex.128.f2.0f.w0 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_XMM|BITS128,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34492, 549, 5602},
    /* 5487 : VMOVDQU8 ymmrm256|mask|z,ymmreg [mr:fvm:   evex.256.f2.0f.w0 7f /r           ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(7f) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_YMM|BITS256,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34501, 549, 5603},
    /* 5488 : VMOVDQU8 zmmrm512|mask|z,zmmreg [mr:fvm:   evex.512.f2.0f.w0 7f /r           ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(7f) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VMOVDQU8, 2, {RM_ZMM|BITS512,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34510, 550, 5604},
    /* 5489 : VMOVHLPS xmmreg,xmmreg*,xmmreg [rvm:      evex.nds.128.0f.w0 12 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 300 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHLPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, NO_DECORATOR, nasm_bytecodes+34519, 544, 5605},
    /* 5490 : VMOVHPD xmmreg,xmmreg*,mem64 [rvm:t1s:  evex.nds.128.66.0f.w1 16 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 08) 306 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34537, 544, 5606},
    /* 5491 : VMOVHPD mem64,xmmreg [mr:t1s:   evex.128.66.0f.w1 17 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(17) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34555, 544, 5607},
    /* 5492 : VMOVHPS xmmreg,xmmreg*,mem64 [rvm:t2:   evex.nds.128.0f.w0 16 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 311 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34564, 544, 5608},
    /* 5493 : VMOVHPS mem64,xmmreg [mr:t2:    evex.128.0f.w0 17 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 08) 311 001(17) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVHPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34582, 544, 5609},
    /* 5494 : VMOVLHPS xmmreg,xmmreg*,xmmreg [rvm:      evex.nds.128.0f.w0 16 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 300 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLHPS, 3, {XMMREG,XMMREG,XMMREG,0,0}, NO_DECORATOR, nasm_bytecodes+34591, 544, 5610},
    /* 5495 : VMOVLPD xmmreg,xmmreg*,mem64 [rvm:t1s:  evex.nds.128.66.0f.w1 12 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 08) 306 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPD, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34609, 544, 5611},
    /* 5496 : VMOVLPD mem64,xmmreg [mr:t1s:   evex.128.66.0f.w1 13 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34627, 544, 5612},
    /* 5497 : VMOVLPS xmmreg,xmmreg*,mem64 [rvm:t2:   evex.nds.128.0f.w0 12 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 08) 311 001(12) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPS, 3, {XMMREG,XMMREG,MEMORY|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+34636, 544, 5613},
    /* 5498 : VMOVLPS mem64,xmmreg [mr:t2:    evex.128.0f.w0 13 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 08) 311 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVLPS, 2, {MEMORY|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34654, 544, 5614},
    /* 5499 : VMOVNTDQ mem128,xmmreg [mr:fvm:   evex.128.66.0f.w0 e7 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 303 001(e7) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34663, 543, 5615},
    /* 5500 : VMOVNTDQ mem256,ymmreg [mr:fvm:   evex.256.66.0f.w0 e7 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 303 001(e7) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34672, 543, 5616},
    /* 5501 : VMOVNTDQ mem512,zmmreg [mr:fvm:   evex.512.66.0f.w0 e7 /r           ] AVX512,FUTURE */
        /* 250(f1 7d 48) 303 001(e7) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTDQ, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34681, 544, 5617},
    /* 5502 : VMOVNTDQA xmmreg,mem128 [rm:fvm:   evex.128.66.0f38.w0 2a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 303 001(2a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQA, 2, {XMMREG,MEMORY|BITS128,0,0,0}, NO_DECORATOR, nasm_bytecodes+34690, 543, 5618},
    /* 5503 : VMOVNTDQA ymmreg,mem256 [rm:fvm:   evex.256.66.0f38.w0 2a /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 303 001(2a) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTDQA, 2, {YMMREG,MEMORY|BITS256,0,0,0}, NO_DECORATOR, nasm_bytecodes+34699, 543, 5619},
    /* 5504 : VMOVNTDQA zmmreg,mem512 [rm:fvm:   evex.512.66.0f38.w0 2a /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 303 001(2a) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTDQA, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+34708, 544, 5620},
    /* 5505 : VMOVNTPD mem128,xmmreg [mr:fvm:   evex.128.66.0f.w1 2b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34717, 543, 5621},
    /* 5506 : VMOVNTPD mem256,ymmreg [mr:fvm:   evex.256.66.0f.w1 2b /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34726, 543, 5622},
    /* 5507 : VMOVNTPD mem512,zmmreg [mr:fvm:   evex.512.66.0f.w1 2b /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(2b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34735, 544, 5623},
    /* 5508 : VMOVNTPS mem128,xmmreg [mr:fvm:   evex.128.0f.w0 2b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34744, 543, 5624},
    /* 5509 : VMOVNTPS mem256,ymmreg [mr:fvm:   evex.256.0f.w0 2b /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(2b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34753, 543, 5625},
    /* 5510 : VMOVNTPS mem512,zmmreg [mr:fvm:   evex.512.0f.w0 2b /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(2b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVNTPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34762, 544, 5626},
    /* 5511 : VMOVQ xmmreg,rm64 [rm:t1s:   evex.128.66.0f.w1 6e /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(6e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {XMMREG,RM_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+34771, 544, 5627},
    /* 5512 : VMOVQ rm64,xmmreg [mr:t1s:   evex.128.66.0f.w1 7e /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(7e) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {RM_GPR|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34780, 544, 5628},
    /* 5513 : VMOVQ xmmreg,xmmrm64 [rm:t1s:   evex.128.f3.0f.w1 7e /r           ] AVX512,FUTURE */
        /* 250(f1 fe 08) 306 001(7e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+34789, 544, 5629},
    /* 5514 : VMOVQ xmmrm64,xmmreg [mr:t1s:   evex.128.66.0f.w1 d6 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(d6) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVQ, 2, {RM_XMM|BITS64,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+34798, 544, 5630},
    /* 5515 : VMOVSD xmmreg|mask|z,mem64 [rm:t1s:   evex.lig.f2.0f.w1 10 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 306 001(10) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 2, {XMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34807, 545, 5631},
    /* 5516 : VMOVSD mem64|mask,xmmreg [mr:t1s:   evex.lig.f2.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 ff 08) 306 001(11) 101 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34816, 545, 5632},
    /* 5517 : VMOVSD xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f2.0f.w1 10 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34825, 545, 5633},
    /* 5518 : VMOVSD xmmreg|mask|z,xmmreg*,xmmreg [mvr:      evex.nds.lig.f2.0f.w1 11 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 300 001(11) 102 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSD, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34843, 545, 5634},
    /* 5519 : VMOVSHDUP xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 16 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(16) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSHDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34861, 543, 5635},
    /* 5520 : VMOVSHDUP ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 16 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(16) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSHDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34870, 543, 5636},
    /* 5521 : VMOVSHDUP zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 16 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(16) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSHDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34879, 544, 5637},
    /* 5522 : VMOVSLDUP xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.f3.0f.w0 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 08) 303 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSLDUP, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34888, 543, 5638},
    /* 5523 : VMOVSLDUP ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.f3.0f.w0 12 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7e 28) 303 001(12) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVSLDUP, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34897, 543, 5639},
    /* 5524 : VMOVSLDUP zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.f3.0f.w0 12 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 48) 303 001(12) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSLDUP, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34906, 544, 5640},
    /* 5525 : VMOVSS xmmreg|mask|z,mem32 [rm:t1s:   evex.lig.f3.0f.w0 10 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 306 001(10) 110 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34915, 545, 5641},
    /* 5526 : VMOVSS mem32|mask,xmmreg [mr:t1s:   evex.lig.f3.0f.w0 11 /r           ] AVX512,FUTURE */
        /* 250(f1 7e 08) 306 001(11) 101 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+34924, 545, 5642},
    /* 5527 : VMOVSS xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.0f.w0 10 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34933, 545, 5643},
    /* 5528 : VMOVSS xmmreg|mask|z,xmmreg*,xmmreg [mvr:      evex.nds.lig.f3.0f.w0 11 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 300 001(11) 102 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMOVSS, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34951, 545, 5644},
    /* 5529 : VMOVUPD xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f.w1 10 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34969, 543, 5645},
    /* 5530 : VMOVUPD ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f.w1 10 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34978, 543, 5646},
    /* 5531 : VMOVUPD zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f.w1 10 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(10) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34987, 544, 5647},
    /* 5532 : VMOVUPD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+34996, 543, 5648},
    /* 5533 : VMOVUPD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35005, 543, 5649},
    /* 5534 : VMOVUPD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35014, 544, 5650},
    /* 5535 : VMOVUPD mem128|mask,xmmreg [mr:fvm:   evex.128.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35023, 543, 5651},
    /* 5536 : VMOVUPD mem256|mask,ymmreg [mr:fvm:   evex.256.66.0f.w1 11 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35032, 543, 5652},
    /* 5537 : VMOVUPD mem512|mask,zmmreg [mr:fvm:   evex.512.66.0f.w1 11 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 303 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35041, 544, 5653},
    /* 5538 : VMOVUPS xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.0f.w0 10 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35050, 543, 5654},
    /* 5539 : VMOVUPS ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.0f.w0 10 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(10) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35059, 543, 5655},
    /* 5540 : VMOVUPS zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.0f.w0 10 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(10) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35068, 544, 5656},
    /* 5541 : VMOVUPS xmmreg|mask|z,xmmreg [mr:       evex.128.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35077, 543, 5657},
    /* 5542 : VMOVUPS ymmreg|mask|z,ymmreg [mr:       evex.256.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35086, 543, 5658},
    /* 5543 : VMOVUPS zmmreg|mask|z,zmmreg [mr:       evex.512.0f.w0 11 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35095, 544, 5659},
    /* 5544 : VMOVUPS mem128|mask,xmmreg [mr:fvm:   evex.128.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35104, 543, 5660},
    /* 5545 : VMOVUPS mem256|mask,ymmreg [mr:fvm:   evex.256.0f.w0 11 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 303 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35113, 543, 5661},
    /* 5546 : VMOVUPS mem512|mask,zmmreg [mr:fvm:   evex.512.0f.w0 11 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 303 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VMOVUPS, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+35122, 544, 5662},
    /* 5547 : VMULPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 59 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35131, 543, 5663},
    /* 5548 : VMULPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 59 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35149, 543, 5664},
    /* 5549 : VMULPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 59 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(59) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMULPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+35167, 544, 5665},
    /* 5550 : VMULPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 59 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35185, 543, 5666},
    /* 5551 : VMULPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 59 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(59) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VMULPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35203, 543, 5667},
    /* 5552 : VMULPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 59 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(59) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VMULPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+35221, 544, 5668},
    /* 5553 : VMULSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 59 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(59) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMULSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+35239, 545, 5669},
    /* 5554 : VMULSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 59 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(59) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VMULSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+35257, 545, 5670},
    /* 5555 : VORPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 56 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35275, 546, 5671},
    /* 5556 : VORPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 56 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35293, 546, 5672},
    /* 5557 : VORPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 56 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(56) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VORPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35311, 547, 5673},
    /* 5558 : VORPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 56 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35329, 546, 5674},
    /* 5559 : VORPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 56 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35347, 546, 5675},
    /* 5560 : VORPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 56 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(56) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VORPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35365, 547, 5676},
    /* 5561 : VPABSB xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.wig 1c /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35383, 555, 5677},
    /* 5562 : VPABSB ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.wig 1c /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35392, 555, 5678},
    /* 5563 : VPABSB zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.wig 1c /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 303 001(1c) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPABSB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35401, 556, 5679},
    /* 5564 : VPABSD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 1e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(1e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35410, 543, 5680},
    /* 5565 : VPABSD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 1e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(1e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35419, 543, 5681},
    /* 5566 : VPABSD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 1e /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(1e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPABSD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+35428, 544, 5682},
    /* 5567 : VPABSQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 1f /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(1f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35437, 543, 5683},
    /* 5568 : VPABSQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 1f /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(1f) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPABSQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35446, 543, 5684},
    /* 5569 : VPABSQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 1f /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(1f) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPABSQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+35455, 544, 5685},
    /* 5570 : VPABSW xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.wig 1d /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35464, 555, 5686},
    /* 5571 : VPABSW ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.wig 1d /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPABSW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35473, 555, 5687},
    /* 5572 : VPABSW zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.wig 1d /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 303 001(1d) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPABSW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35482, 556, 5688},
    /* 5573 : VPACKSSDW xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 6b /r       ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 301 001(6b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35491, 549, 5689},
    /* 5574 : VPACKSSDW ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 6b /r       ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 301 001(6b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35509, 549, 5690},
    /* 5575 : VPACKSSDW zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 6b /r       ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 301 001(6b) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKSSDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35527, 550, 5691},
    /* 5576 : VPACKSSWB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 63 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(63) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35545, 555, 5692},
    /* 5577 : VPACKSSWB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 63 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(63) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35563, 555, 5693},
    /* 5578 : VPACKSSWB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 63 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(63) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKSSWB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35581, 556, 5694},
    /* 5579 : VPACKUSDW xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 2b /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 301 001(2b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35599, 549, 5695},
    /* 5580 : VPACKUSDW ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 2b /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 301 001(2b) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35617, 549, 5696},
    /* 5581 : VPACKUSDW zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 2b /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 301 001(2b) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKUSDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35635, 550, 5697},
    /* 5582 : VPACKUSWB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 67 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(67) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35653, 555, 5698},
    /* 5583 : VPACKUSWB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 67 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(67) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35671, 555, 5699},
    /* 5584 : VPACKUSWB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 67 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(67) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPACKUSWB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35689, 556, 5700},
    /* 5585 : VPADDB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig fc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35707, 555, 5701},
    /* 5586 : VPADDB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig fc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35725, 555, 5702},
    /* 5587 : VPADDB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig fc /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(fc) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35743, 556, 5703},
    /* 5588 : VPADDD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 fe /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(fe) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35761, 543, 5704},
    /* 5589 : VPADDD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 fe /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(fe) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35779, 543, 5705},
    /* 5590 : VPADDD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 fe /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(fe) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPADDD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+35797, 544, 5706},
    /* 5591 : VPADDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 d4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(d4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35815, 543, 5707},
    /* 5592 : VPADDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 d4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(d4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPADDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35833, 543, 5708},
    /* 5593 : VPADDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 d4 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(d4) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPADDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+35851, 544, 5709},
    /* 5594 : VPADDSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ec /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35869, 555, 5710},
    /* 5595 : VPADDSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ec /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35887, 555, 5711},
    /* 5596 : VPADDSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ec /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ec) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35905, 556, 5712},
    /* 5597 : VPADDSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ed /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35923, 555, 5713},
    /* 5598 : VPADDSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ed /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35941, 555, 5714},
    /* 5599 : VPADDSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ed /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ed) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35959, 556, 5715},
    /* 5600 : VPADDUSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig dc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35977, 555, 5716},
    /* 5601 : VPADDUSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig dc /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+35995, 555, 5717},
    /* 5602 : VPADDUSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig dc /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(dc) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDUSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36013, 556, 5718},
    /* 5603 : VPADDUSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig dd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36031, 555, 5719},
    /* 5604 : VPADDUSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig dd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36049, 555, 5720},
    /* 5605 : VPADDUSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig dd /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(dd) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDUSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36067, 556, 5721},
    /* 5606 : VPADDW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig fd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36085, 555, 5722},
    /* 5607 : VPADDW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig fd /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPADDW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36103, 555, 5723},
    /* 5608 : VPADDW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig fd /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(fd) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPADDW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36121, 556, 5724},
    /* 5609 : VPALIGNR xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.wig 0f /r ib ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19453, 555, 5725},
    /* 5610 : VPALIGNR ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.wig 0f /r ib ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19473, 555, 5726},
    /* 5611 : VPALIGNR zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.wig 0f /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(0f) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPALIGNR, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+19493, 556, 5727},
    /* 5612 : VPANDD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36139, 543, 5728},
    /* 5613 : VPANDD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36157, 543, 5729},
    /* 5614 : VPANDD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 db /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(db) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36175, 544, 5730},
    /* 5615 : VPANDND xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDND, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36193, 543, 5731},
    /* 5616 : VPANDND ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDND, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36211, 543, 5732},
    /* 5617 : VPANDND zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 df /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(df) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDND, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36229, 544, 5733},
    /* 5618 : VPANDNQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDNQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36247, 543, 5734},
    /* 5619 : VPANDNQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 df /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(df) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDNQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36265, 543, 5735},
    /* 5620 : VPANDNQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 df /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(df) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDNQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36283, 544, 5736},
    /* 5621 : VPANDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36301, 543, 5737},
    /* 5622 : VPANDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 db /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(db) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPANDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36319, 543, 5738},
    /* 5623 : VPANDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 db /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(db) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPANDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36337, 544, 5739},
    /* 5624 : VPAVGB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e0 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36355, 555, 5740},
    /* 5625 : VPAVGB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e0 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36373, 555, 5741},
    /* 5626 : VPAVGB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e0 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e0) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPAVGB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36391, 556, 5742},
    /* 5627 : VPAVGW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e3 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36409, 555, 5743},
    /* 5628 : VPAVGW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e3 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36427, 555, 5744},
    /* 5629 : VPAVGW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e3 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e3) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPAVGW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36445, 556, 5745},
    /* 5630 : VPBLENDMB xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36463, 549, 5746},
    /* 5631 : VPBLENDMB ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36472, 549, 5747},
    /* 5632 : VPBLENDMB zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 66 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(66) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBLENDMB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36481, 550, 5748},
    /* 5633 : VPBLENDMD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36490, 543, 5749},
    /* 5634 : VPBLENDMD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36499, 543, 5750},
    /* 5635 : VPBLENDMD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 64 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(64) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBLENDMD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+36508, 544, 5751},
    /* 5636 : VPBLENDMQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36517, 543, 5752},
    /* 5637 : VPBLENDMQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 64 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(64) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBLENDMQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36526, 543, 5753},
    /* 5638 : VPBLENDMQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 64 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(64) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBLENDMQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+36535, 544, 5754},
    /* 5639 : VPBLENDMW xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36544, 549, 5755},
    /* 5640 : VPBLENDMW ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 66 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(66) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36553, 549, 5756},
    /* 5641 : VPBLENDMW zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 66 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(66) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBLENDMW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36562, 550, 5757},
    /* 5642 : VPBROADCASTB xmmreg|mask|z,xmmrm8 [rm:t1s8:  evex.128.66.0f38.w0 78 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 304 001(78) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36571, 549, 5758},
    /* 5643 : VPBROADCASTB ymmreg|mask|z,xmmrm8 [rm:t1s8:  evex.256.66.0f38.w0 78 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 304 001(78) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36580, 549, 5759},
    /* 5644 : VPBROADCASTB zmmreg|mask|z,xmmrm8 [rm:t1s8:  evex.512.66.0f38.w0 78 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 304 001(78) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,RM_XMM|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36589, 550, 5760},
    /* 5645 : VPBROADCASTB xmmreg|mask|z,reg8 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36598, 549, 5761},
    /* 5646 : VPBROADCASTB xmmreg|mask|z,reg16 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36598, 549, 5762},
    /* 5647 : VPBROADCASTB xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36598, 549, 5763},
    /* 5648 : VPBROADCASTB xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36598, 549, 5764},
    /* 5649 : VPBROADCASTB ymmreg|mask|z,reg8 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36607, 549, 5765},
    /* 5650 : VPBROADCASTB ymmreg|mask|z,reg16 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36607, 549, 5766},
    /* 5651 : VPBROADCASTB ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36607, 549, 5767},
    /* 5652 : VPBROADCASTB ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w0 7a /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36607, 549, 5768},
    /* 5653 : VPBROADCASTB zmmreg|mask|z,reg8 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS8,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36616, 550, 5769},
    /* 5654 : VPBROADCASTB zmmreg|mask|z,reg16 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36616, 550, 5770},
    /* 5655 : VPBROADCASTB zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36616, 550, 5771},
    /* 5656 : VPBROADCASTB zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w0 7a /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7a) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTB, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36616, 550, 5772},
    /* 5657 : VPBROADCASTD xmmreg|mask|z,mem32 [rm:t1s:   evex.128.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36625, 543, 5773},
    /* 5658 : VPBROADCASTD ymmreg|mask|z,mem32 [rm:t1s:   evex.256.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36634, 543, 5774},
    /* 5659 : VPBROADCASTD zmmreg|mask|z,mem32 [rm:t1s:   evex.512.66.0f38.w0 58 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(58) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,MEMORY|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36643, 544, 5775},
    /* 5660 : VPBROADCASTD xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36652, 543, 5776},
    /* 5661 : VPBROADCASTD ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w0 58 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(58) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36661, 543, 5777},
    /* 5662 : VPBROADCASTD zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w0 58 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(58) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36670, 544, 5778},
    /* 5663 : VPBROADCASTD xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36679, 543, 5779},
    /* 5664 : VPBROADCASTD ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTD, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36688, 543, 5780},
    /* 5665 : VPBROADCASTD zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7c /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(7c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTD, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36697, 544, 5781},
    /* 5666 : VPBROADCASTMB2Q xmmreg,kreg [rm:       evex.128.f3.0f38.w1 2a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fe 08) 300 001(2a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36706, 557, 5782},
    /* 5667 : VPBROADCASTMB2Q ymmreg,kreg [rm:       evex.256.f3.0f38.w1 2a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fe 28) 300 001(2a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36715, 557, 5783},
    /* 5668 : VPBROADCASTMB2Q zmmreg,kreg [rm:       evex.512.f3.0f38.w1 2a /r         ] AVX512CD,FUTURE */
        /* 250(f2 fe 48) 300 001(2a) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPBROADCASTMB2Q, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36724, 558, 5784},
    /* 5669 : VPBROADCASTMW2D xmmreg,kreg [rm:       evex.128.f3.0f38.w0 3a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(3a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMW2D, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36733, 557, 5785},
    /* 5670 : VPBROADCASTMW2D ymmreg,kreg [rm:       evex.256.f3.0f38.w0 3a /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(3a) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPBROADCASTMW2D, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36742, 557, 5786},
    /* 5671 : VPBROADCASTMW2D zmmreg,kreg [rm:       evex.512.f3.0f38.w0 3a /r         ] AVX512CD,FUTURE */
        /* 250(f2 7e 48) 300 001(3a) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPBROADCASTMW2D, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+36751, 558, 5787},
    /* 5672 : VPBROADCASTQ xmmreg|mask|z,mem64 [rm:t1s:   evex.128.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36760, 543, 5788},
    /* 5673 : VPBROADCASTQ ymmreg|mask|z,mem64 [rm:t1s:   evex.256.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36769, 543, 5789},
    /* 5674 : VPBROADCASTQ zmmreg|mask|z,mem64 [rm:t1s:   evex.512.66.0f38.w1 59 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(59) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,MEMORY|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36778, 544, 5790},
    /* 5675 : VPBROADCASTQ xmmreg|mask|z,xmmreg [rm:       evex.128.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36787, 543, 5791},
    /* 5676 : VPBROADCASTQ ymmreg|mask|z,xmmreg [rm:       evex.256.66.0f38.w1 59 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(59) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36796, 543, 5792},
    /* 5677 : VPBROADCASTQ zmmreg|mask|z,xmmreg [rm:       evex.512.66.0f38.w1 59 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(59) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36805, 544, 5793},
    /* 5678 : VPBROADCASTQ xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w1 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36814, 543, 5794},
    /* 5679 : VPBROADCASTQ ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w1 7c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(7c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPBROADCASTQ, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36823, 543, 5795},
    /* 5680 : VPBROADCASTQ zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w1 7c /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(7c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPBROADCASTQ, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36832, 544, 5796},
    /* 5681 : VPBROADCASTW xmmreg|mask|z,xmmrm16 [rm:t1s16: evex.128.66.0f38.w0 79 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 305 001(79) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36841, 549, 5797},
    /* 5682 : VPBROADCASTW ymmreg|mask|z,xmmrm16 [rm:t1s16: evex.256.66.0f38.w0 79 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 305 001(79) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36850, 549, 5798},
    /* 5683 : VPBROADCASTW zmmreg|mask|z,xmmrm16 [rm:t1s16: evex.512.66.0f38.w0 79 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 305 001(79) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36859, 550, 5799},
    /* 5684 : VPBROADCASTW xmmreg|mask|z,reg16 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36868, 549, 5800},
    /* 5685 : VPBROADCASTW xmmreg|mask|z,reg32 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36868, 549, 5801},
    /* 5686 : VPBROADCASTW xmmreg|mask|z,reg64 [rm:       evex.128.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {XMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36868, 549, 5802},
    /* 5687 : VPBROADCASTW ymmreg|mask|z,reg16 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36877, 549, 5803},
    /* 5688 : VPBROADCASTW ymmreg|mask|z,reg32 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36877, 549, 5804},
    /* 5689 : VPBROADCASTW ymmreg|mask|z,reg64 [rm:       evex.256.66.0f38.w0 7b /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 300 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {YMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36877, 549, 5805},
    /* 5690 : VPBROADCASTW zmmreg|mask|z,reg16 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36886, 550, 5806},
    /* 5691 : VPBROADCASTW zmmreg|mask|z,reg32 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36886, 550, 5807},
    /* 5692 : VPBROADCASTW zmmreg|mask|z,reg64 [rm:       evex.512.66.0f38.w0 7b /r         ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 300 001(7b) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPBROADCASTW, 2, {ZMMREG,REG_GPR|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+36886, 550, 5808},
    /* 5693 : VPCMPEQB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 74 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(74) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36895, 555, 5811},
    /* 5694 : VPCMPEQB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 74 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(74) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36904, 555, 5812},
    /* 5695 : VPCMPEQB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 74 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(74) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36913, 556, 5813},
    /* 5696 : VPCMPEQD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 76 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36922, 543, 5814},
    /* 5697 : VPCMPEQD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 76 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36931, 543, 5815},
    /* 5698 : VPCMPEQD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 76 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+36940, 544, 5816},
    /* 5699 : VPCMPEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 29 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(29) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36949, 543, 5817},
    /* 5700 : VPCMPEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 29 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(29) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36958, 543, 5818},
    /* 5701 : VPCMPEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 29 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(29) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+36967, 544, 5819},
    /* 5702 : VPCMPEQW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 75 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(75) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36976, 555, 5820},
    /* 5703 : VPCMPEQW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 75 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(75) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36985, 555, 5821},
    /* 5704 : VPCMPEQW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 75 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(75) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+36994, 556, 5822},
    /* 5705 : VPCMPGTB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 64 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(64) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37003, 555, 5823},
    /* 5706 : VPCMPGTB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 64 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(64) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37012, 555, 5824},
    /* 5707 : VPCMPGTB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 64 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(64) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37021, 556, 5825},
    /* 5708 : VPCMPGTD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 66 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(66) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37030, 543, 5826},
    /* 5709 : VPCMPGTD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 66 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(66) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37039, 543, 5827},
    /* 5710 : VPCMPGTD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 66 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(66) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+37048, 544, 5828},
    /* 5711 : VPCMPGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 37 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(37) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37057, 543, 5829},
    /* 5712 : VPCMPGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 37 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(37) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37066, 543, 5830},
    /* 5713 : VPCMPGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 37 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(37) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+37075, 544, 5831},
    /* 5714 : VPCMPGTW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 65 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(65) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37084, 555, 5832},
    /* 5715 : VPCMPGTW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 65 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(65) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37093, 555, 5833},
    /* 5716 : VPCMPGTW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 65 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(65) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37102, 556, 5834},
    /* 5717 : VPCMPEQB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7559, 549, 5836},
    /* 5718 : VPCMPEQB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7570, 549, 5837},
    /* 5719 : VPCMPEQB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7581, 550, 5838},
    /* 5720 : VPCMPEQD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7592, 543, 5839},
    /* 5721 : VPCMPEQD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7603, 543, 5840},
    /* 5722 : VPCMPEQD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 00  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7614, 544, 5841},
    /* 5723 : VPCMPEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7625, 543, 5842},
    /* 5724 : VPCMPEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7636, 543, 5843},
    /* 5725 : VPCMPEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 00  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7647, 544, 5844},
    /* 5726 : VPCMPEQUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7658, 549, 5845},
    /* 5727 : VPCMPEQUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7669, 549, 5846},
    /* 5728 : VPCMPEQUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7680, 550, 5847},
    /* 5729 : VPCMPEQUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7691, 543, 5848},
    /* 5730 : VPCMPEQUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7702, 543, 5849},
    /* 5731 : VPCMPEQUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 00  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7713, 544, 5850},
    /* 5732 : VPCMPEQUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7724, 543, 5851},
    /* 5733 : VPCMPEQUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 00  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7735, 543, 5852},
    /* 5734 : VPCMPEQUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 00  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(00) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPEQUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7746, 544, 5853},
    /* 5735 : VPCMPEQUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7757, 549, 5854},
    /* 5736 : VPCMPEQUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7768, 549, 5855},
    /* 5737 : VPCMPEQUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7779, 550, 5856},
    /* 5738 : VPCMPEQW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7790, 549, 5857},
    /* 5739 : VPCMPEQW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 00  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7801, 549, 5858},
    /* 5740 : VPCMPEQW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 00  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(00) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7812, 550, 5859},
    /* 5741 : VPCMPGEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7823, 549, 5860},
    /* 5742 : VPCMPGEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7834, 549, 5861},
    /* 5743 : VPCMPGEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7845, 550, 5862},
    /* 5744 : VPCMPGED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7856, 543, 5863},
    /* 5745 : VPCMPGED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7867, 543, 5864},
    /* 5746 : VPCMPGED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7878, 544, 5865},
    /* 5747 : VPCMPGEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7889, 543, 5866},
    /* 5748 : VPCMPGEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7900, 543, 5867},
    /* 5749 : VPCMPGEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7911, 544, 5868},
    /* 5750 : VPCMPGEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7922, 549, 5869},
    /* 5751 : VPCMPGEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7933, 549, 5870},
    /* 5752 : VPCMPGEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7944, 550, 5871},
    /* 5753 : VPCMPGEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7955, 543, 5872},
    /* 5754 : VPCMPGEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7966, 543, 5873},
    /* 5755 : VPCMPGEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7977, 544, 5874},
    /* 5756 : VPCMPGEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7988, 543, 5875},
    /* 5757 : VPCMPGEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7999, 543, 5876},
    /* 5758 : VPCMPGEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8010, 544, 5877},
    /* 5759 : VPCMPGEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8021, 549, 5878},
    /* 5760 : VPCMPGEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8032, 549, 5879},
    /* 5761 : VPCMPGEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8043, 550, 5880},
    /* 5762 : VPCMPGEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8054, 549, 5881},
    /* 5763 : VPCMPGEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8065, 549, 5882},
    /* 5764 : VPCMPGEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8076, 550, 5883},
    /* 5765 : VPCMPGTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8087, 549, 5884},
    /* 5766 : VPCMPGTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8098, 549, 5885},
    /* 5767 : VPCMPGTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8109, 550, 5886},
    /* 5768 : VPCMPGTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8120, 543, 5887},
    /* 5769 : VPCMPGTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8131, 543, 5888},
    /* 5770 : VPCMPGTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8142, 544, 5889},
    /* 5771 : VPCMPGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8153, 543, 5890},
    /* 5772 : VPCMPGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8164, 543, 5891},
    /* 5773 : VPCMPGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8175, 544, 5892},
    /* 5774 : VPCMPGTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8186, 549, 5893},
    /* 5775 : VPCMPGTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8197, 549, 5894},
    /* 5776 : VPCMPGTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8208, 550, 5895},
    /* 5777 : VPCMPGTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8219, 543, 5896},
    /* 5778 : VPCMPGTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8230, 543, 5897},
    /* 5779 : VPCMPGTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8241, 544, 5898},
    /* 5780 : VPCMPGTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8252, 543, 5899},
    /* 5781 : VPCMPGTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8263, 543, 5900},
    /* 5782 : VPCMPGTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPGTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8274, 544, 5901},
    /* 5783 : VPCMPGTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8285, 549, 5902},
    /* 5784 : VPCMPGTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8296, 549, 5903},
    /* 5785 : VPCMPGTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8307, 550, 5904},
    /* 5786 : VPCMPGTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8318, 549, 5905},
    /* 5787 : VPCMPGTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8329, 549, 5906},
    /* 5788 : VPCMPGTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8340, 550, 5907},
    /* 5789 : VPCMPLEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8351, 549, 5908},
    /* 5790 : VPCMPLEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8362, 549, 5909},
    /* 5791 : VPCMPLEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8373, 550, 5910},
    /* 5792 : VPCMPLED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8384, 543, 5911},
    /* 5793 : VPCMPLED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8395, 543, 5912},
    /* 5794 : VPCMPLED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8406, 544, 5913},
    /* 5795 : VPCMPLEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8417, 543, 5914},
    /* 5796 : VPCMPLEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8428, 543, 5915},
    /* 5797 : VPCMPLEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8439, 544, 5916},
    /* 5798 : VPCMPLEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8450, 549, 5917},
    /* 5799 : VPCMPLEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8461, 549, 5918},
    /* 5800 : VPCMPLEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8472, 550, 5919},
    /* 5801 : VPCMPLEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8483, 543, 5920},
    /* 5802 : VPCMPLEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8494, 543, 5921},
    /* 5803 : VPCMPLEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8505, 544, 5922},
    /* 5804 : VPCMPLEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8516, 543, 5923},
    /* 5805 : VPCMPLEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8527, 543, 5924},
    /* 5806 : VPCMPLEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8538, 544, 5925},
    /* 5807 : VPCMPLEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8549, 549, 5926},
    /* 5808 : VPCMPLEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8560, 549, 5927},
    /* 5809 : VPCMPLEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8571, 550, 5928},
    /* 5810 : VPCMPLEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8582, 549, 5929},
    /* 5811 : VPCMPLEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8593, 549, 5930},
    /* 5812 : VPCMPLEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8604, 550, 5931},
    /* 5813 : VPCMPLTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8615, 549, 5932},
    /* 5814 : VPCMPLTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8626, 549, 5933},
    /* 5815 : VPCMPLTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8637, 550, 5934},
    /* 5816 : VPCMPLTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8648, 543, 5935},
    /* 5817 : VPCMPLTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8659, 543, 5936},
    /* 5818 : VPCMPLTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 01  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8670, 544, 5937},
    /* 5819 : VPCMPLTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8681, 543, 5938},
    /* 5820 : VPCMPLTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8692, 543, 5939},
    /* 5821 : VPCMPLTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 01  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8703, 544, 5940},
    /* 5822 : VPCMPLTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8714, 549, 5941},
    /* 5823 : VPCMPLTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8725, 549, 5942},
    /* 5824 : VPCMPLTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8736, 550, 5943},
    /* 5825 : VPCMPLTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8747, 543, 5944},
    /* 5826 : VPCMPLTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8758, 543, 5945},
    /* 5827 : VPCMPLTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 01  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8769, 544, 5946},
    /* 5828 : VPCMPLTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8780, 543, 5947},
    /* 5829 : VPCMPLTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 01  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8791, 543, 5948},
    /* 5830 : VPCMPLTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 01  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(01) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPLTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8802, 544, 5949},
    /* 5831 : VPCMPLTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8813, 549, 5950},
    /* 5832 : VPCMPLTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8824, 549, 5951},
    /* 5833 : VPCMPLTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8835, 550, 5952},
    /* 5834 : VPCMPLTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8846, 549, 5953},
    /* 5835 : VPCMPLTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 01  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8857, 549, 5954},
    /* 5836 : VPCMPLTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 01  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(01) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPLTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8868, 550, 5955},
    /* 5837 : VPCMPNEQB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8879, 549, 5956},
    /* 5838 : VPCMPNEQB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8890, 549, 5957},
    /* 5839 : VPCMPNEQB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8901, 550, 5958},
    /* 5840 : VPCMPNEQD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8912, 543, 5959},
    /* 5841 : VPCMPNEQD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8923, 543, 5960},
    /* 5842 : VPCMPNEQD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 04  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8934, 544, 5961},
    /* 5843 : VPCMPNEQQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8945, 543, 5962},
    /* 5844 : VPCMPNEQQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8956, 543, 5963},
    /* 5845 : VPCMPNEQQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 04  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8967, 544, 5964},
    /* 5846 : VPCMPNEQUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8978, 549, 5965},
    /* 5847 : VPCMPNEQUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8989, 549, 5966},
    /* 5848 : VPCMPNEQUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9000, 550, 5967},
    /* 5849 : VPCMPNEQUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9011, 543, 5968},
    /* 5850 : VPCMPNEQUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9022, 543, 5969},
    /* 5851 : VPCMPNEQUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 04  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+9033, 544, 5970},
    /* 5852 : VPCMPNEQUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9044, 543, 5971},
    /* 5853 : VPCMPNEQUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 04  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9055, 543, 5972},
    /* 5854 : VPCMPNEQUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 04  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(04) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNEQUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+9066, 544, 5973},
    /* 5855 : VPCMPNEQUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9077, 549, 5974},
    /* 5856 : VPCMPNEQUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9088, 549, 5975},
    /* 5857 : VPCMPNEQUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9099, 550, 5976},
    /* 5858 : VPCMPNEQW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9110, 549, 5977},
    /* 5859 : VPCMPNEQW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 04  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9121, 549, 5978},
    /* 5860 : VPCMPNEQW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 04  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(04) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNEQW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+9132, 550, 5979},
    /* 5861 : VPCMPNGTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8351, 549, 5980},
    /* 5862 : VPCMPNGTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8362, 549, 5981},
    /* 5863 : VPCMPNGTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8373, 550, 5982},
    /* 5864 : VPCMPNGTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8384, 543, 5983},
    /* 5865 : VPCMPNGTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8395, 543, 5984},
    /* 5866 : VPCMPNGTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8406, 544, 5985},
    /* 5867 : VPCMPNGTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8417, 543, 5986},
    /* 5868 : VPCMPNGTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8428, 543, 5987},
    /* 5869 : VPCMPNGTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8439, 544, 5988},
    /* 5870 : VPCMPNGTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8450, 549, 5989},
    /* 5871 : VPCMPNGTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8461, 549, 5990},
    /* 5872 : VPCMPNGTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8472, 550, 5991},
    /* 5873 : VPCMPNGTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8483, 543, 5992},
    /* 5874 : VPCMPNGTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8494, 543, 5993},
    /* 5875 : VPCMPNGTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8505, 544, 5994},
    /* 5876 : VPCMPNGTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8516, 543, 5995},
    /* 5877 : VPCMPNGTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 02  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8527, 543, 5996},
    /* 5878 : VPCMPNGTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 02  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(02) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNGTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8538, 544, 5997},
    /* 5879 : VPCMPNGTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8549, 549, 5998},
    /* 5880 : VPCMPNGTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8560, 549, 5999},
    /* 5881 : VPCMPNGTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8571, 550, 6000},
    /* 5882 : VPCMPNGTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8582, 549, 6001},
    /* 5883 : VPCMPNGTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 02  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8593, 549, 6002},
    /* 5884 : VPCMPNGTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 02  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(02) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNGTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8604, 550, 6003},
    /* 5885 : VPCMPNLEB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8087, 549, 6004},
    /* 5886 : VPCMPNLEB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8098, 549, 6005},
    /* 5887 : VPCMPNLEB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8109, 550, 6006},
    /* 5888 : VPCMPNLED kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8120, 543, 6007},
    /* 5889 : VPCMPNLED kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8131, 543, 6008},
    /* 5890 : VPCMPNLED kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLED, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8142, 544, 6009},
    /* 5891 : VPCMPNLEQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8153, 543, 6010},
    /* 5892 : VPCMPNLEQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8164, 543, 6011},
    /* 5893 : VPCMPNLEQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8175, 544, 6012},
    /* 5894 : VPCMPNLEUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8186, 549, 6013},
    /* 5895 : VPCMPNLEUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8197, 549, 6014},
    /* 5896 : VPCMPNLEUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8208, 550, 6015},
    /* 5897 : VPCMPNLEUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8219, 543, 6016},
    /* 5898 : VPCMPNLEUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8230, 543, 6017},
    /* 5899 : VPCMPNLEUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+8241, 544, 6018},
    /* 5900 : VPCMPNLEUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8252, 543, 6019},
    /* 5901 : VPCMPNLEUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 06  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8263, 543, 6020},
    /* 5902 : VPCMPNLEUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 06  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(06) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLEUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8274, 544, 6021},
    /* 5903 : VPCMPNLEUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8285, 549, 6022},
    /* 5904 : VPCMPNLEUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8296, 549, 6023},
    /* 5905 : VPCMPNLEUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8307, 550, 6024},
    /* 5906 : VPCMPNLEW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8318, 549, 6025},
    /* 5907 : VPCMPNLEW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 06  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8329, 549, 6026},
    /* 5908 : VPCMPNLEW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 06  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(06) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLEW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8340, 550, 6027},
    /* 5909 : VPCMPNLTB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7823, 549, 6028},
    /* 5910 : VPCMPNLTB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7834, 549, 6029},
    /* 5911 : VPCMPNLTB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7845, 550, 6030},
    /* 5912 : VPCMPNLTD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7856, 543, 6031},
    /* 5913 : VPCMPNLTD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7867, 543, 6032},
    /* 5914 : VPCMPNLTD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7878, 544, 6033},
    /* 5915 : VPCMPNLTQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7889, 543, 6034},
    /* 5916 : VPCMPNLTQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7900, 543, 6035},
    /* 5917 : VPCMPNLTQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7911, 544, 6036},
    /* 5918 : VPCMPNLTUB kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7922, 549, 6037},
    /* 5919 : VPCMPNLTUB kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7933, 549, 6038},
    /* 5920 : VPCMPNLTUB kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTUB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+7944, 550, 6039},
    /* 5921 : VPCMPNLTUD kreg|mask,xmmreg,xmmrm128|b32 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7955, 543, 6040},
    /* 5922 : VPCMPNLTUD kreg|mask,ymmreg,ymmrm256|b32 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7966, 543, 6041},
    /* 5923 : VPCMPNLTUD kreg|mask,zmmreg,zmmrm512|b32 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTUD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+7977, 544, 6042},
    /* 5924 : VPCMPNLTUQ kreg|mask,xmmreg,xmmrm128|b64 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7988, 543, 6043},
    /* 5925 : VPCMPNLTUQ kreg|mask,ymmreg,ymmrm256|b64 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r 05  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+7999, 543, 6044},
    /* 5926 : VPCMPNLTUQ kreg|mask,zmmreg,zmmrm512|b64 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r 05  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 001(05) : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPNLTUQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+8010, 544, 6045},
    /* 5927 : VPCMPNLTUW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8021, 549, 6046},
    /* 5928 : VPCMPNLTUW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8032, 549, 6047},
    /* 5929 : VPCMPNLTUW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTUW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8043, 550, 6048},
    /* 5930 : VPCMPNLTW kreg|mask,xmmreg,xmmrm128 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8054, 549, 6049},
    /* 5931 : VPCMPNLTW kreg|mask,ymmreg,ymmrm256 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r 05  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8065, 549, 6050},
    /* 5932 : VPCMPNLTW kreg|mask,zmmreg,zmmrm512 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r 05  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 001(05) : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPNLTW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+8076, 550, 6051},
    /* 5933 : VPCMPB kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19513, 549, 6052},
    /* 5934 : VPCMPB kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19523, 549, 6053},
    /* 5935 : VPCMPB kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3f /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3f) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPB, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19533, 550, 6054},
    /* 5936 : VPCMPD kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19543, 543, 6055},
    /* 5937 : VPCMPD kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19553, 543, 6056},
    /* 5938 : VPCMPD kreg|mask,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1f /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1f) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19563, 544, 6057},
    /* 5939 : VPCMPQ kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19573, 543, 6058},
    /* 5940 : VPCMPQ kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1f /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1f) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19583, 543, 6059},
    /* 5941 : VPCMPQ kreg|mask,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1f /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1f) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPQ, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19593, 544, 6060},
    /* 5942 : VPCMPUB kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w0 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 08) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19603, 549, 6061},
    /* 5943 : VPCMPUB kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w0 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 7d 28) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19613, 549, 6062},
    /* 5944 : VPCMPUB kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w0 3e /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 7d 48) 303 001(3e) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPUB, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19623, 550, 6063},
    /* 5945 : VPCMPUD kreg|mask,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19633, 543, 6064},
    /* 5946 : VPCMPUD kreg|mask,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19643, 543, 6065},
    /* 5947 : VPCMPUD kreg|mask,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 1e /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(1e) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPUD, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B32,0,0}, nasm_bytecodes+19653, 544, 6066},
    /* 5948 : VPCMPUQ kreg|mask,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19663, 543, 6067},
    /* 5949 : VPCMPUQ kreg|mask,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 1e /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(1e) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19673, 543, 6068},
    /* 5950 : VPCMPUQ kreg|mask,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 1e /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(1e) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCMPUQ, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B64,0,0}, nasm_bytecodes+19683, 544, 6069},
    /* 5951 : VPCMPUW kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19693, 549, 6070},
    /* 5952 : VPCMPUW kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3e /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3e) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19703, 549, 6071},
    /* 5953 : VPCMPUW kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3e /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3e) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPUW, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19713, 550, 6072},
    /* 5954 : VPCMPW kreg|mask,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 08) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19723, 549, 6073},
    /* 5955 : VPCMPW kreg|mask,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 3f /r ib  ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f3 fd 28) 303 001(3f) 120 023 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19733, 549, 6074},
    /* 5956 : VPCMPW kreg|mask,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 3f /r ib  ] AVX512BW,FUTURE */
        /* 241(f3 fd 48) 303 001(3f) 120 023 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPCMPW, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,0,0,0}, nasm_bytecodes+19743, 550, 6075},
    /* 5957 : VPCOMPRESSD mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37111, 543, 6076},
    /* 5958 : VPCOMPRESSD mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37120, 543, 6077},
    /* 5959 : VPCOMPRESSD mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w0 8b /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSD, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37129, 544, 6078},
    /* 5960 : VPCOMPRESSD xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37138, 543, 6079},
    /* 5961 : VPCOMPRESSD ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37147, 543, 6080},
    /* 5962 : VPCOMPRESSD zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 8b /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 300 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37156, 544, 6081},
    /* 5963 : VPCOMPRESSQ mem128|mask,xmmreg [mr:t1s:   evex.128.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37165, 543, 6082},
    /* 5964 : VPCOMPRESSQ mem256|mask,ymmreg [mr:t1s:   evex.256.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37174, 543, 6083},
    /* 5965 : VPCOMPRESSQ mem512|mask,zmmreg [mr:t1s:   evex.512.66.0f38.w1 8b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSQ, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+37183, 544, 6084},
    /* 5966 : VPCOMPRESSQ xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37192, 543, 6085},
    /* 5967 : VPCOMPRESSQ ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 8b /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 300 001(8b) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPCOMPRESSQ, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37201, 543, 6086},
    /* 5968 : VPCOMPRESSQ zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 8b /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 300 001(8b) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPCOMPRESSQ, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37210, 544, 6087},
    /* 5969 : VPCONFLICTD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37219, 557, 6088},
    /* 5970 : VPCONFLICTD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37228, 557, 6089},
    /* 5971 : VPCONFLICTD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 c4 /r         ] AVX512CD,FUTURE */
        /* 250(f2 7d 48) 301 001(c4) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPCONFLICTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+37237, 558, 6090},
    /* 5972 : VPCONFLICTQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37246, 557, 6091},
    /* 5973 : VPCONFLICTQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 c4 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(c4) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPCONFLICTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37255, 557, 6092},
    /* 5974 : VPCONFLICTQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 c4 /r         ] AVX512CD,FUTURE */
        /* 250(f2 fd 48) 301 001(c4) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPCONFLICTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+37264, 558, 6093},
    /* 5975 : VPERMB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 8d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37273, 559, 6094},
    /* 5976 : VPERMB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 8d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37291, 559, 6095},
    /* 5977 : VPERMB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 8d /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(8d) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37309, 560, 6096},
    /* 5978 : VPERMD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 36 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(36) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37327, 543, 6097},
    /* 5979 : VPERMD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 36 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(36) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37345, 544, 6098},
    /* 5980 : VPERMI2B xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 75 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37363, 559, 6099},
    /* 5981 : VPERMI2B ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 75 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37372, 559, 6100},
    /* 5982 : VPERMI2B zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 75 /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(75) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMI2B, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37381, 560, 6101},
    /* 5983 : VPERMI2D xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2D, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37390, 543, 6102},
    /* 5984 : VPERMI2D ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2D, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37399, 543, 6103},
    /* 5985 : VPERMI2D zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 76 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2D, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37408, 544, 6104},
    /* 5986 : VPERMI2PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37417, 543, 6105},
    /* 5987 : VPERMI2PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37426, 543, 6106},
    /* 5988 : VPERMI2PD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 77 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(77) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37435, 544, 6107},
    /* 5989 : VPERMI2PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37444, 543, 6108},
    /* 5990 : VPERMI2PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 77 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(77) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37453, 543, 6109},
    /* 5991 : VPERMI2PS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 77 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(77) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37462, 544, 6110},
    /* 5992 : VPERMI2Q xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2Q, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37471, 543, 6111},
    /* 5993 : VPERMI2Q ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 76 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(76) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMI2Q, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37480, 543, 6112},
    /* 5994 : VPERMI2Q zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 76 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(76) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMI2Q, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37489, 544, 6113},
    /* 5995 : VPERMI2W xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 75 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37498, 549, 6114},
    /* 5996 : VPERMI2W ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 75 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(75) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37507, 549, 6115},
    /* 5997 : VPERMI2W zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 75 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(75) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMI2W, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37516, 550, 6116},
    /* 5998 : VPERMILPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 05 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(05) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19753, 543, 6117},
    /* 5999 : VPERMILPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 05 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(05) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19763, 543, 6118},
    /* 6000 : VPERMILPD zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 05 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(05) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19773, 544, 6119},
    /* 6001 : VPERMILPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 0d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(0d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37525, 543, 6120},
    /* 6002 : VPERMILPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 0d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(0d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37543, 543, 6121},
    /* 6003 : VPERMILPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 0d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(0d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37561, 544, 6122},
    /* 6004 : VPERMILPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 04 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(04) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19783, 543, 6123},
    /* 6005 : VPERMILPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 04 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(04) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19793, 543, 6124},
    /* 6006 : VPERMILPS zmmreg|mask|z,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f3a.w0 04 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(04) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+19803, 544, 6125},
    /* 6007 : VPERMILPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 0c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(0c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37579, 543, 6126},
    /* 6008 : VPERMILPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 0c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(0c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMILPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37597, 543, 6127},
    /* 6009 : VPERMILPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 0c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(0c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMILPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37615, 544, 6128},
    /* 6010 : VPERMPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 01 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(01) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19813, 543, 6129},
    /* 6011 : VPERMPD zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 01 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(01) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19823, 544, 6130},
    /* 6012 : VPERMPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 16 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(16) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37633, 543, 6131},
    /* 6013 : VPERMPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 16 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37651, 544, 6132},
    /* 6014 : VPERMPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 16 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(16) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37669, 543, 6133},
    /* 6015 : VPERMPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 16 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(16) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37687, 544, 6134},
    /* 6016 : VPERMQ ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 00 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(00) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19833, 543, 6135},
    /* 6017 : VPERMQ zmmreg|mask|z,zmmrm512|b64,imm8 [rmi:fv:   evex.512.66.0f3a.w1 00 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(00) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+19843, 544, 6136},
    /* 6018 : VPERMQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 36 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(36) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37705, 543, 6137},
    /* 6019 : VPERMQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 36 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(36) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37723, 544, 6138},
    /* 6020 : VPERMT2B xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 7d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 08) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37741, 559, 6139},
    /* 6021 : VPERMT2B ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 7d /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 7d 28) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37750, 559, 6140},
    /* 6022 : VPERMT2B zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 7d /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 7d 48) 303 001(7d) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPERMT2B, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37759, 560, 6141},
    /* 6023 : VPERMT2D xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2D, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37768, 543, 6142},
    /* 6024 : VPERMT2D ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2D, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37777, 543, 6143},
    /* 6025 : VPERMT2D zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 7e /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(7e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2D, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37786, 544, 6144},
    /* 6026 : VPERMT2PD xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37795, 543, 6145},
    /* 6027 : VPERMT2PD ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37804, 543, 6146},
    /* 6028 : VPERMT2PD zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 7f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(7f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2PD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37813, 544, 6147},
    /* 6029 : VPERMT2PS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37822, 543, 6148},
    /* 6030 : VPERMT2PS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 7f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(7f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37831, 543, 6149},
    /* 6031 : VPERMT2PS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 7f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(7f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+37840, 544, 6150},
    /* 6032 : VPERMT2Q xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2Q, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37849, 543, 6151},
    /* 6033 : VPERMT2Q ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 7e /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(7e) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPERMT2Q, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37858, 543, 6152},
    /* 6034 : VPERMT2Q zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 7e /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(7e) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPERMT2Q, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+37867, 544, 6153},
    /* 6035 : VPERMT2W xmmreg|mask|z,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 7d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37876, 549, 6154},
    /* 6036 : VPERMT2W ymmreg|mask|z,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 7d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(7d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37885, 549, 6155},
    /* 6037 : VPERMT2W zmmreg|mask|z,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 7d /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(7d) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMT2W, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37894, 550, 6156},
    /* 6038 : VPERMW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 8d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37903, 549, 6157},
    /* 6039 : VPERMW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 8d /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(8d) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPERMW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37921, 549, 6158},
    /* 6040 : VPERMW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 8d /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(8d) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPERMW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37939, 550, 6159},
    /* 6041 : VPEXPANDD xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37957, 543, 6160},
    /* 6042 : VPEXPANDD ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37966, 543, 6161},
    /* 6043 : VPEXPANDD zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w0 89 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDD, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37975, 544, 6162},
    /* 6044 : VPEXPANDD xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37957, 543, 6163},
    /* 6045 : VPEXPANDD ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w0 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDD, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37966, 543, 6164},
    /* 6046 : VPEXPANDD zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w0 89 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDD, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37975, 544, 6165},
    /* 6047 : VPEXPANDQ xmmreg|mask|z,mem128 [rm:t1s:   evex.128.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {XMMREG,MEMORY|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37984, 543, 6166},
    /* 6048 : VPEXPANDQ ymmreg|mask|z,mem256 [rm:t1s:   evex.256.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {YMMREG,MEMORY|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37993, 543, 6167},
    /* 6049 : VPEXPANDQ zmmreg|mask|z,mem512 [rm:t1s:   evex.512.66.0f38.w1 89 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDQ, 2, {ZMMREG,MEMORY|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38002, 544, 6168},
    /* 6050 : VPEXPANDQ xmmreg|mask|z,xmmreg [rm:t1s:   evex.128.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37984, 543, 6169},
    /* 6051 : VPEXPANDQ ymmreg|mask|z,ymmreg [rm:t1s:   evex.256.66.0f38.w1 89 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 306 001(89) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPEXPANDQ, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+37993, 543, 6170},
    /* 6052 : VPEXPANDQ zmmreg|mask|z,zmmreg [rm:t1s:   evex.512.66.0f38.w1 89 /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 306 001(89) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPEXPANDQ, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38002, 544, 6171},
    /* 6053 : VPEXTRB reg8,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS8,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 556, 6172},
    /* 6054 : VPEXTRB reg16,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 556, 6173},
    /* 6055 : VPEXTRB reg32,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 556, 6174},
    /* 6056 : VPEXTRB reg64,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 556, 6175},
    /* 6057 : VPEXTRB mem8,xmmreg,imm8 [mri:t1s8: evex.128.66.0f3a.wig 14 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 304 001(14) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRB, 3, {MEMORY|BITS8,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19853, 556, 6176},
    /* 6058 : VPEXTRD rm32,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.w0 16 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 306 001(16) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPEXTRD, 3, {RM_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19863, 547, 6177},
    /* 6059 : VPEXTRQ rm64,xmmreg,imm8 [mri:t1s:  evex.128.66.0f3a.w1 16 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 306 001(16) 101 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPEXTRQ, 3, {RM_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19873, 547, 6178},
    /* 6060 : VPEXTRW reg16,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 556, 6179},
    /* 6061 : VPEXTRW reg32,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 556, 6180},
    /* 6062 : VPEXTRW reg64,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 556, 6181},
    /* 6063 : VPEXTRW mem16,xmmreg,imm8 [mri:t1s16:evex.128.66.0f3a.wig 15 /r ib     ] AVX512BW,FUTURE */
        /* 250(f3 7d 08) 305 001(15) 101 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {MEMORY|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19883, 556, 6182},
    /* 6064 : VPEXTRW reg16,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS16,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 556, 6183},
    /* 6065 : VPEXTRW reg32,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS32,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 556, 6184},
    /* 6066 : VPEXTRW reg64,xmmreg,imm8 [rmi:      evex.128.66.0f.wig c5 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7d 08) 300 001(c5) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPEXTRW, 3, {REG_GPR|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+19893, 556, 6185},
    /* 6067 : VPGATHERDD xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDD, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19903, 543, 6186},
    /* 6068 : VPGATHERDD ymmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDD, 2, {YMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19913, 543, 6187},
    /* 6069 : VPGATHERDD zmmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 90 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(90) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERDD, 2, {ZMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19923, 544, 6188},
    /* 6070 : VPGATHERDQ xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDQ, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19933, 543, 6189},
    /* 6071 : VPGATHERDQ ymmreg|mask,xmem64 [rm:t1s:   vsibx evex.256.66.0f38.w1 90 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(90) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERDQ, 2, {YMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19943, 543, 6190},
    /* 6072 : VPGATHERDQ zmmreg|mask,ymem64 [rm:t1s:   vsiby evex.512.66.0f38.w1 90 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(90) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERDQ, 2, {ZMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19953, 544, 6191},
    /* 6073 : VPGATHERQD xmmreg|mask,xmem32 [rm:t1s:   vsibx evex.128.66.0f38.w0 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQD, 2, {XMMREG,XMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19963, 543, 6192},
    /* 6074 : VPGATHERQD xmmreg|mask,ymem32 [rm:t1s:   vsiby evex.256.66.0f38.w0 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQD, 2, {XMMREG,YMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19973, 543, 6193},
    /* 6075 : VPGATHERQD ymmreg|mask,zmem32 [rm:t1s:   vsibz evex.512.66.0f38.w0 91 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(91) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERQD, 2, {YMMREG,ZMEM|BITS32,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19983, 544, 6194},
    /* 6076 : VPGATHERQQ xmmreg|mask,xmem64 [rm:t1s:   vsibx evex.128.66.0f38.w1 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQQ, 2, {XMMREG,XMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+19993, 543, 6195},
    /* 6077 : VPGATHERQQ ymmreg|mask,ymem64 [rm:t1s:   vsiby evex.256.66.0f38.w1 91 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(91) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPGATHERQQ, 2, {YMMREG,YMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20003, 543, 6196},
    /* 6078 : VPGATHERQQ zmmreg|mask,zmem64 [rm:t1s:   vsibz evex.512.66.0f38.w1 91 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(91) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPGATHERQQ, 2, {ZMMREG,ZMEM|BITS64,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20013, 544, 6197},
    /* 6079 : VPINSRB xmmreg,xmmreg*,reg32,imm8 [rvmi:t1s8:evex.nds.128.66.0f3a.wig 20 /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 08) 304 001(20) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRB, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20023, 556, 6198},
    /* 6080 : VPINSRB xmmreg,xmmreg*,mem8,imm8 [rvmi:t1s8:evex.nds.128.66.0f3a.wig 20 /r ib ] AVX512BW,FUTURE */
        /* 241(f3 7d 08) 304 001(20) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRB, 4, {XMMREG,XMMREG,MEMORY|BITS8,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20023, 556, 6199},
    /* 6081 : VPINSRD xmmreg,xmmreg*,rm32,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w0 22 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(22) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPINSRD, 4, {XMMREG,XMMREG,RM_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20043, 547, 6200},
    /* 6082 : VPINSRQ xmmreg,xmmreg*,rm64,imm8 [rvmi:t1s: evex.nds.128.66.0f3a.w1 22 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(22) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPINSRQ, 4, {XMMREG,XMMREG,RM_GPR|BITS64,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20063, 547, 6201},
    /* 6083 : VPINSRW xmmreg,xmmreg*,reg32,imm8 [rvmi:t1s16:evex.nds.128.66.0f.wig c4 /r ib   ] AVX512BW,FUTURE */
        /* 241(f1 7d 08) 305 001(c4) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRW, 4, {XMMREG,XMMREG,REG_GPR|BITS32,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20083, 556, 6202},
    /* 6084 : VPINSRW xmmreg,xmmreg*,mem16,imm8 [rvmi:t1s16:evex.nds.128.66.0f.wig c4 /r ib   ] AVX512BW,FUTURE */
        /* 241(f1 7d 08) 305 001(c4) 120 023 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPINSRW, 4, {XMMREG,XMMREG,MEMORY|BITS16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+20083, 556, 6203},
    /* 6085 : VPLZCNTD xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38011, 557, 6204},
    /* 6086 : VPLZCNTD ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38020, 557, 6205},
    /* 6087 : VPLZCNTD zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 44 /r         ] AVX512CD,FUTURE */
        /* 250(f2 7d 48) 301 001(44) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPLZCNTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+38029, 558, 6206},
    /* 6088 : VPLZCNTQ xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38038, 557, 6207},
    /* 6089 : VPLZCNTQ ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 44 /r         ] AVX512CD,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(44) 110 : EVEX,PROT,AVX512CD,AVX512VL,FUTURE */
        {I_VPLZCNTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38047, 557, 6208},
    /* 6090 : VPLZCNTQ zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 44 /r         ] AVX512CD,FUTURE */
        /* 250(f2 fd 48) 301 001(44) 110 : EVEX,PROT,AVX512CD,FUTURE */
        {I_VPLZCNTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+38056, 558, 6209},
    /* 6091 : VPMADD52HUQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b5 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 08) 301 001(b5) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38065, 561, 6210},
    /* 6092 : VPMADD52HUQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b5 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 28) 301 001(b5) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38074, 561, 6211},
    /* 6093 : VPMADD52HUQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 b5 /r     ] AVX512IFMA,FUTURE */
        /* 241(f2 fd 48) 301 001(b5) 120 : EVEX,PROT,AVX512IFMA,FUTURE */
        {I_VPMADD52HUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38083, 562, 6212},
    /* 6094 : VPMADD52LUQ xmmreg|mask|z,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 b4 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 08) 301 001(b4) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38092, 561, 6213},
    /* 6095 : VPMADD52LUQ ymmreg|mask|z,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 b4 /r     ] AVX512VL,AVX512IFMA,FUTURE */
        /* 241(f2 fd 28) 301 001(b4) 120 : EVEX,PROT,AVX512VL,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38101, 561, 6214},
    /* 6096 : VPMADD52LUQ zmmreg|mask|z,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 b4 /r     ] AVX512IFMA,FUTURE */
        /* 241(f2 fd 48) 301 001(b4) 120 : EVEX,PROT,AVX512IFMA,FUTURE */
        {I_VPMADD52LUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38110, 562, 6215},
    /* 6097 : VPMADDUBSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 04 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(04) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38119, 555, 6216},
    /* 6098 : VPMADDUBSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 04 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(04) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38137, 555, 6217},
    /* 6099 : VPMADDUBSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 04 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(04) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMADDUBSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38155, 556, 6218},
    /* 6100 : VPMADDWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38173, 555, 6219},
    /* 6101 : VPMADDWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38191, 555, 6220},
    /* 6102 : VPMADDWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMADDWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38209, 556, 6221},
    /* 6103 : VPMAXSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3c /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38227, 555, 6222},
    /* 6104 : VPMAXSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3c /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38245, 555, 6223},
    /* 6105 : VPMAXSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3c /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3c) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38263, 556, 6224},
    /* 6106 : VPMAXSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38281, 543, 6225},
    /* 6107 : VPMAXSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38299, 543, 6226},
    /* 6108 : VPMAXSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38317, 544, 6227},
    /* 6109 : VPMAXSQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38335, 543, 6228},
    /* 6110 : VPMAXSQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3d /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXSQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38353, 543, 6229},
    /* 6111 : VPMAXSQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXSQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38371, 544, 6230},
    /* 6112 : VPMAXSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ee /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38389, 555, 6231},
    /* 6113 : VPMAXSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ee /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38407, 555, 6232},
    /* 6114 : VPMAXSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ee /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ee) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38425, 556, 6233},
    /* 6115 : VPMAXUB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig de /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(de) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38443, 555, 6234},
    /* 6116 : VPMAXUB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig de /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(de) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38461, 555, 6235},
    /* 6117 : VPMAXUB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig de /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(de) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXUB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38479, 556, 6236},
    /* 6118 : VPMAXUD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38497, 543, 6237},
    /* 6119 : VPMAXUD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38515, 543, 6238},
    /* 6120 : VPMAXUD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38533, 544, 6239},
    /* 6121 : VPMAXUQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38551, 543, 6240},
    /* 6122 : VPMAXUQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3f /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3f) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMAXUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38569, 543, 6241},
    /* 6123 : VPMAXUQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3f) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMAXUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38587, 544, 6242},
    /* 6124 : VPMAXUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3e /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38605, 555, 6243},
    /* 6125 : VPMAXUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3e /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38623, 555, 6244},
    /* 6126 : VPMAXUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3e /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3e) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMAXUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38641, 556, 6245},
    /* 6127 : VPMINSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 38 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(38) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38659, 555, 6246},
    /* 6128 : VPMINSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 38 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(38) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38677, 555, 6247},
    /* 6129 : VPMINSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 38 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(38) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38695, 556, 6248},
    /* 6130 : VPMINSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38713, 543, 6249},
    /* 6131 : VPMINSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38731, 543, 6250},
    /* 6132 : VPMINSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 39 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(39) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38749, 544, 6251},
    /* 6133 : VPMINSQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38767, 543, 6252},
    /* 6134 : VPMINSQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 39 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(39) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINSQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38785, 543, 6253},
    /* 6135 : VPMINSQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 39 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(39) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINSQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38803, 544, 6254},
    /* 6136 : VPMINSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig ea /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38821, 555, 6255},
    /* 6137 : VPMINSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig ea /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38839, 555, 6256},
    /* 6138 : VPMINSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig ea /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(ea) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38857, 556, 6257},
    /* 6139 : VPMINUB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig da /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(da) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38875, 555, 6258},
    /* 6140 : VPMINUB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig da /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(da) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38893, 555, 6259},
    /* 6141 : VPMINUB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig da /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(da) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINUB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+38911, 556, 6260},
    /* 6142 : VPMINUD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38929, 543, 6261},
    /* 6143 : VPMINUD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38947, 543, 6262},
    /* 6144 : VPMINUD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 3b /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(3b) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+38965, 544, 6263},
    /* 6145 : VPMINUQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+38983, 543, 6264},
    /* 6146 : VPMINUQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 3b /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(3b) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMINUQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+39001, 543, 6265},
    /* 6147 : VPMINUQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 3b /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(3b) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMINUQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+39019, 544, 6266},
    /* 6148 : VPMINUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 3a /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39037, 555, 6267},
    /* 6149 : VPMINUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 3a /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39055, 555, 6268},
    /* 6150 : VPMINUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 3a /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(3a) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMINUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39073, 556, 6269},
    /* 6151 : VPMOVB2M kreg,xmmreg [rm:       evex.128.f3.0f38.w0 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39091, 549, 6270},
    /* 6152 : VPMOVB2M kreg,ymmreg [rm:       evex.256.f3.0f38.w0 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39100, 549, 6271},
    /* 6153 : VPMOVB2M kreg,zmmreg [rm:       evex.512.f3.0f38.w0 29 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(29) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVB2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39109, 550, 6272},
    /* 6154 : VPMOVD2M kreg,xmmreg [rm:       evex.128.f3.0f38.w0 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 08) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39118, 546, 6273},
    /* 6155 : VPMOVD2M kreg,ymmreg [rm:       evex.256.f3.0f38.w0 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 28) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39127, 546, 6274},
    /* 6156 : VPMOVD2M kreg,zmmreg [rm:       evex.512.f3.0f38.w0 39 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7e 48) 300 001(39) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVD2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39136, 547, 6275},
    /* 6157 : VPMOVDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39145, 543, 6276},
    /* 6158 : VPMOVDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39154, 543, 6277},
    /* 6159 : VPMOVDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 31 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(31) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39163, 544, 6278},
    /* 6160 : VPMOVDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39172, 543, 6279},
    /* 6161 : VPMOVDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 31 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(31) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39181, 543, 6280},
    /* 6162 : VPMOVDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 31 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(31) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39190, 544, 6281},
    /* 6163 : VPMOVDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39199, 543, 6282},
    /* 6164 : VPMOVDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39208, 543, 6283},
    /* 6165 : VPMOVDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 33 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(33) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39217, 544, 6284},
    /* 6166 : VPMOVDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39226, 543, 6285},
    /* 6167 : VPMOVDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 33 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(33) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39235, 543, 6286},
    /* 6168 : VPMOVDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 33 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(33) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39244, 544, 6287},
    /* 6169 : VPMOVM2B xmmreg,kreg [rm:       evex.128.f3.0f38.w0 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39253, 549, 6288},
    /* 6170 : VPMOVM2B ymmreg,kreg [rm:       evex.256.f3.0f38.w0 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39262, 549, 6289},
    /* 6171 : VPMOVM2B zmmreg,kreg [rm:       evex.512.f3.0f38.w0 28 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(28) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVM2B, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39271, 550, 6290},
    /* 6172 : VPMOVM2D xmmreg,kreg [rm:       evex.128.f3.0f38.w0 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 08) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39280, 546, 6291},
    /* 6173 : VPMOVM2D ymmreg,kreg [rm:       evex.256.f3.0f38.w0 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 7e 28) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39289, 546, 6292},
    /* 6174 : VPMOVM2D zmmreg,kreg [rm:       evex.512.f3.0f38.w0 38 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 7e 48) 300 001(38) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVM2D, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39298, 547, 6293},
    /* 6175 : VPMOVM2Q xmmreg,kreg [rm:       evex.128.f3.0f38.w1 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 08) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39307, 546, 6294},
    /* 6176 : VPMOVM2Q ymmreg,kreg [rm:       evex.256.f3.0f38.w1 38 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 28) 300 001(38) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39316, 546, 6295},
    /* 6177 : VPMOVM2Q zmmreg,kreg [rm:       evex.512.f3.0f38.w1 38 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fe 48) 300 001(38) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVM2Q, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39325, 547, 6296},
    /* 6178 : VPMOVM2W xmmreg,kreg [rm:       evex.128.f3.0f38.w1 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 08) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {XMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39334, 549, 6297},
    /* 6179 : VPMOVM2W ymmreg,kreg [rm:       evex.256.f3.0f38.w1 28 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 28) 300 001(28) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {YMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39343, 549, 6298},
    /* 6180 : VPMOVM2W zmmreg,kreg [rm:       evex.512.f3.0f38.w1 28 /r         ] AVX512BW,FUTURE */
        /* 250(f2 fe 48) 300 001(28) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVM2W, 2, {ZMMREG,OPMASKREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39352, 550, 6299},
    /* 6181 : VPMOVQ2M kreg,xmmreg [rm:       evex.128.f3.0f38.w1 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 08) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39361, 546, 6300},
    /* 6182 : VPMOVQ2M kreg,ymmreg [rm:       evex.256.f3.0f38.w1 39 /r         ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f2 fe 28) 300 001(39) 110 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39370, 546, 6301},
    /* 6183 : VPMOVQ2M kreg,zmmreg [rm:       evex.512.f3.0f38.w1 39 /r         ] AVX512DQ,FUTURE */
        /* 250(f2 fe 48) 300 001(39) 110 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMOVQ2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+39379, 547, 6302},
    /* 6184 : VPMOVQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39388, 543, 6303},
    /* 6185 : VPMOVQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39397, 543, 6304},
    /* 6186 : VPMOVQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 32 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(32) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39406, 544, 6305},
    /* 6187 : VPMOVQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39415, 543, 6306},
    /* 6188 : VPMOVQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 32 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(32) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39424, 543, 6307},
    /* 6189 : VPMOVQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 32 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(32) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39433, 544, 6308},
    /* 6190 : VPMOVQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39442, 543, 6309},
    /* 6191 : VPMOVQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39451, 543, 6310},
    /* 6192 : VPMOVQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(35) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39460, 544, 6311},
    /* 6193 : VPMOVQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39469, 543, 6312},
    /* 6194 : VPMOVQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(35) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39478, 543, 6313},
    /* 6195 : VPMOVQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(35) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39487, 544, 6314},
    /* 6196 : VPMOVQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39496, 543, 6315},
    /* 6197 : VPMOVQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39505, 543, 6316},
    /* 6198 : VPMOVQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 34 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(34) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39514, 544, 6317},
    /* 6199 : VPMOVQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39523, 543, 6318},
    /* 6200 : VPMOVQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 34 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(34) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39532, 543, 6319},
    /* 6201 : VPMOVQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 34 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(34) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39541, 544, 6320},
    /* 6202 : VPMOVSDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39550, 543, 6321},
    /* 6203 : VPMOVSDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39559, 543, 6322},
    /* 6204 : VPMOVSDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 21 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(21) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39568, 544, 6323},
    /* 6205 : VPMOVSDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39577, 543, 6324},
    /* 6206 : VPMOVSDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 21 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(21) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39586, 543, 6325},
    /* 6207 : VPMOVSDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 21 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(21) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39595, 544, 6326},
    /* 6208 : VPMOVSDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39604, 543, 6327},
    /* 6209 : VPMOVSDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39613, 543, 6328},
    /* 6210 : VPMOVSDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 23 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(23) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39622, 544, 6329},
    /* 6211 : VPMOVSDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39631, 543, 6330},
    /* 6212 : VPMOVSDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 23 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(23) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39640, 543, 6331},
    /* 6213 : VPMOVSDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 23 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(23) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39649, 544, 6332},
    /* 6214 : VPMOVSQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39658, 543, 6333},
    /* 6215 : VPMOVSQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39667, 543, 6334},
    /* 6216 : VPMOVSQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 22 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(22) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39676, 544, 6335},
    /* 6217 : VPMOVSQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39685, 543, 6336},
    /* 6218 : VPMOVSQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 22 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(22) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39694, 543, 6337},
    /* 6219 : VPMOVSQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 22 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(22) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39703, 544, 6338},
    /* 6220 : VPMOVSQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39712, 543, 6339},
    /* 6221 : VPMOVSQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39721, 543, 6340},
    /* 6222 : VPMOVSQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(25) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39730, 544, 6341},
    /* 6223 : VPMOVSQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39739, 543, 6342},
    /* 6224 : VPMOVSQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(25) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39748, 543, 6343},
    /* 6225 : VPMOVSQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(25) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39757, 544, 6344},
    /* 6226 : VPMOVSQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39766, 543, 6345},
    /* 6227 : VPMOVSQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39775, 543, 6346},
    /* 6228 : VPMOVSQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 24 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(24) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39784, 544, 6347},
    /* 6229 : VPMOVSQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39793, 543, 6348},
    /* 6230 : VPMOVSQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 24 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(24) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39802, 543, 6349},
    /* 6231 : VPMOVSQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 24 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(24) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39811, 544, 6350},
    /* 6232 : VPMOVSWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39820, 549, 6351},
    /* 6233 : VPMOVSWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39829, 549, 6352},
    /* 6234 : VPMOVSWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 20 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(20) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39838, 550, 6353},
    /* 6235 : VPMOVSWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39847, 549, 6354},
    /* 6236 : VPMOVSWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 20 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(20) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39856, 549, 6355},
    /* 6237 : VPMOVSWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 20 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(20) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+39865, 550, 6356},
    /* 6238 : VPMOVSXBD xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 21 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39874, 563, 6357},
    /* 6239 : VPMOVSXBD ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 21 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39883, 563, 6358},
    /* 6240 : VPMOVSXBD zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 21 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(21) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXBD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39892, 552, 6359},
    /* 6241 : VPMOVSXBQ xmmreg|mask|z,xmmrm16 [rm:ovm:   evex.128.66.0f38.wig 22 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39901, 563, 6360},
    /* 6242 : VPMOVSXBQ ymmreg|mask|z,xmmrm32 [rm:ovm:   evex.256.66.0f38.wig 22 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXBQ, 2, {YMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39910, 563, 6361},
    /* 6243 : VPMOVSXBQ zmmreg|mask|z,xmmrm64 [rm:ovm:   evex.512.66.0f38.wig 22 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 316 001(22) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXBQ, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39919, 552, 6362},
    /* 6244 : VPMOVSXBW xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 20 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 314 001(20) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39928, 555, 6363},
    /* 6245 : VPMOVSXBW ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 20 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 314 001(20) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39937, 555, 6364},
    /* 6246 : VPMOVSXBW zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 20 /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 314 001(20) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVSXBW, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39946, 556, 6365},
    /* 6247 : VPMOVSXDQ xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(25) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39955, 543, 6366},
    /* 6248 : VPMOVSXDQ ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 25 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(25) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39964, 543, 6367},
    /* 6249 : VPMOVSXDQ zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.w0 25 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(25) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39973, 544, 6368},
    /* 6250 : VPMOVSXWD xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 23 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39982, 563, 6369},
    /* 6251 : VPMOVSXWD ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 23 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+39991, 563, 6370},
    /* 6252 : VPMOVSXWD zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 23 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(23) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXWD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40000, 552, 6371},
    /* 6253 : VPMOVSXWQ xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 24 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40009, 563, 6372},
    /* 6254 : VPMOVSXWQ ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 24 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVSXWQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40018, 563, 6373},
    /* 6255 : VPMOVSXWQ zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 24 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(24) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVSXWQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40027, 552, 6374},
    /* 6256 : VPMOVUSDB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40036, 543, 6375},
    /* 6257 : VPMOVUSDB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40045, 543, 6376},
    /* 6258 : VPMOVUSDB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 11 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40054, 544, 6377},
    /* 6259 : VPMOVUSDB mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40063, 543, 6378},
    /* 6260 : VPMOVUSDB mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 11 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(11) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40072, 543, 6379},
    /* 6261 : VPMOVUSDB mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 11 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(11) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDB, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40081, 544, 6380},
    /* 6262 : VPMOVUSDW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40090, 543, 6381},
    /* 6263 : VPMOVUSDW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40099, 543, 6382},
    /* 6264 : VPMOVUSDW ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDW, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40108, 544, 6383},
    /* 6265 : VPMOVUSDW mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40117, 543, 6384},
    /* 6266 : VPMOVUSDW mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(13) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40126, 543, 6385},
    /* 6267 : VPMOVUSDW mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(13) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSDW, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40135, 544, 6386},
    /* 6268 : VPMOVUSQB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40144, 543, 6387},
    /* 6269 : VPMOVUSQB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40153, 543, 6388},
    /* 6270 : VPMOVUSQB xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 12 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(12) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQB, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40162, 544, 6389},
    /* 6271 : VPMOVUSQB mem16|mask,xmmreg [mr:ovm:   evex.128.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 316 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40171, 543, 6390},
    /* 6272 : VPMOVUSQB mem32|mask,ymmreg [mr:ovm:   evex.256.f3.0f38.w0 12 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 316 001(12) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40180, 543, 6391},
    /* 6273 : VPMOVUSQB mem64|mask,zmmreg [mr:ovm:   evex.512.f3.0f38.w0 12 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 316 001(12) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQB, 2, {MEMORY|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40189, 544, 6392},
    /* 6274 : VPMOVUSQD xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40198, 543, 6393},
    /* 6275 : VPMOVUSQD xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40207, 543, 6394},
    /* 6276 : VPMOVUSQD ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 15 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(15) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQD, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40216, 544, 6395},
    /* 6277 : VPMOVUSQD mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 314 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40225, 543, 6396},
    /* 6278 : VPMOVUSQD mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 15 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 314 001(15) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40234, 543, 6397},
    /* 6279 : VPMOVUSQD mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 15 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 314 001(15) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQD, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40243, 544, 6398},
    /* 6280 : VPMOVUSQW xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 300 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40252, 543, 6399},
    /* 6281 : VPMOVUSQW xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 300 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40261, 543, 6400},
    /* 6282 : VPMOVUSQW xmmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 14 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 300 001(14) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQW, 2, {XMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40270, 544, 6401},
    /* 6283 : VPMOVUSQW mem32|mask,xmmreg [mr:qvm:   evex.128.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 08) 315 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40279, 543, 6402},
    /* 6284 : VPMOVUSQW mem64|mask,ymmreg [mr:qvm:   evex.256.f3.0f38.w0 14 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7e 28) 315 001(14) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40288, 543, 6403},
    /* 6285 : VPMOVUSQW mem128|mask,zmmreg [mr:qvm:   evex.512.f3.0f38.w0 14 /r         ] AVX512,FUTURE */
        /* 250(f2 7e 48) 315 001(14) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVUSQW, 2, {MEMORY|BITS128,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40297, 544, 6404},
    /* 6286 : VPMOVUSWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40306, 549, 6405},
    /* 6287 : VPMOVUSWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40315, 549, 6406},
    /* 6288 : VPMOVUSWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 10 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(10) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40324, 550, 6407},
    /* 6289 : VPMOVUSWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40333, 549, 6408},
    /* 6290 : VPMOVUSWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 10 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(10) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40342, 549, 6409},
    /* 6291 : VPMOVUSWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 10 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(10) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVUSWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40351, 550, 6410},
    /* 6292 : VPMOVW2M kreg,xmmreg [rm:       evex.128.f3.0f38.w1 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 08) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40360, 549, 6411},
    /* 6293 : VPMOVW2M kreg,ymmreg [rm:       evex.256.f3.0f38.w1 29 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 fe 28) 300 001(29) 110 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,YMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40369, 549, 6412},
    /* 6294 : VPMOVW2M kreg,zmmreg [rm:       evex.512.f3.0f38.w1 29 /r         ] AVX512BW,FUTURE */
        /* 250(f2 fe 48) 300 001(29) 110 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVW2M, 2, {OPMASKREG,ZMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+40378, 550, 6413},
    /* 6295 : VPMOVWB xmmreg|mask|z,xmmreg [mr:       evex.128.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 300 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40387, 549, 6414},
    /* 6296 : VPMOVWB xmmreg|mask|z,ymmreg [mr:       evex.256.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 300 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {XMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40396, 549, 6415},
    /* 6297 : VPMOVWB ymmreg|mask|z,zmmreg [mr:       evex.512.f3.0f38.w0 30 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 300 001(30) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {YMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40405, 550, 6416},
    /* 6298 : VPMOVWB mem64|mask,xmmreg [mr:hvm:   evex.128.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 08) 314 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40414, 549, 6417},
    /* 6299 : VPMOVWB mem128|mask,ymmreg [mr:hvm:   evex.256.f3.0f38.w0 30 /r         ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7e 28) 314 001(30) 101 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS128,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40423, 549, 6418},
    /* 6300 : VPMOVWB mem256|mask,zmmreg [mr:hvm:   evex.512.f3.0f38.w0 30 /r         ] AVX512BW,FUTURE */
        /* 250(f2 7e 48) 314 001(30) 101 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVWB, 2, {MEMORY|BITS256,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+40432, 550, 6419},
    /* 6301 : VPMOVZXBD xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 31 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40441, 563, 6420},
    /* 6302 : VPMOVZXBD ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 31 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40450, 563, 6421},
    /* 6303 : VPMOVZXBD zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 31 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(31) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXBD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40459, 552, 6422},
    /* 6304 : VPMOVZXBQ xmmreg|mask|z,xmmrm16 [rm:ovm:   evex.128.66.0f38.wig 32 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBQ, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40468, 563, 6423},
    /* 6305 : VPMOVZXBQ ymmreg|mask|z,xmmrm32 [rm:ovm:   evex.256.66.0f38.wig 32 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXBQ, 2, {YMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40477, 563, 6424},
    /* 6306 : VPMOVZXBQ zmmreg|mask|z,xmmrm64 [rm:ovm:   evex.512.66.0f38.wig 32 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 316 001(32) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXBQ, 2, {ZMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40486, 552, 6425},
    /* 6307 : VPMOVZXBW xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 30 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 08) 314 001(30) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40495, 555, 6426},
    /* 6308 : VPMOVZXBW ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 30 /r        ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f2 7d 28) 314 001(30) 110 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40504, 555, 6427},
    /* 6309 : VPMOVZXBW zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 30 /r        ] AVX512BW,FUTURE */
        /* 250(f2 7d 48) 314 001(30) 110 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMOVZXBW, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40513, 556, 6428},
    /* 6310 : VPMOVZXDQ xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(35) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40522, 543, 6429},
    /* 6311 : VPMOVZXDQ ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 35 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(35) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40531, 543, 6430},
    /* 6312 : VPMOVZXDQ zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.w0 35 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(35) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40540, 544, 6431},
    /* 6313 : VPMOVZXWD xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.wig 33 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40549, 563, 6432},
    /* 6314 : VPMOVZXWD ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.wig 33 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWD, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40558, 563, 6433},
    /* 6315 : VPMOVZXWD zmmreg|mask|z,ymmrm256 [rm:hvm:   evex.512.66.0f38.wig 33 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(33) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXWD, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40567, 552, 6434},
    /* 6316 : VPMOVZXWQ xmmreg|mask|z,xmmrm32 [rm:qvm:   evex.128.66.0f38.wig 34 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40576, 563, 6435},
    /* 6317 : VPMOVZXWQ ymmreg|mask|z,xmmrm64 [rm:qvm:   evex.256.66.0f38.wig 34 /r        ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMOVZXWQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40585, 563, 6436},
    /* 6318 : VPMOVZXWQ zmmreg|mask|z,xmmrm128 [rm:qvm:   evex.512.66.0f38.wig 34 /r        ] AVX512,FUTURE */
        /* 250(f2 7d 48) 315 001(34) 110 : WIG,EVEX,PROT,AVX512,FUTURE */
        {I_VPMOVZXWQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40594, 552, 6437},
    /* 6319 : VPMULDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 28 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(28) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40603, 543, 6438},
    /* 6320 : VPMULDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 28 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(28) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40621, 543, 6439},
    /* 6321 : VPMULDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 28 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(28) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40639, 544, 6440},
    /* 6322 : VPMULHRSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 0b /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40657, 555, 6441},
    /* 6323 : VPMULHRSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 0b /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40675, 555, 6442},
    /* 6324 : VPMULHRSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 0b /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(0b) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHRSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40693, 556, 6443},
    /* 6325 : VPMULHUW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e4 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40711, 555, 6444},
    /* 6326 : VPMULHUW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e4 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40729, 555, 6445},
    /* 6327 : VPMULHUW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e4 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e4) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHUW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40747, 556, 6446},
    /* 6328 : VPMULHW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40765, 555, 6447},
    /* 6329 : VPMULHW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40783, 555, 6448},
    /* 6330 : VPMULHW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULHW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40801, 556, 6449},
    /* 6331 : VPMULLD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 40 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(40) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40819, 543, 6450},
    /* 6332 : VPMULLD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 40 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(40) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULLD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40837, 543, 6451},
    /* 6333 : VPMULLD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 40 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(40) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULLD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+40855, 544, 6452},
    /* 6334 : VPMULLQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 40 /r     ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f2 fd 08) 301 001(40) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40873, 546, 6453},
    /* 6335 : VPMULLQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 40 /r     ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f2 fd 28) 301 001(40) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40891, 546, 6454},
    /* 6336 : VPMULLQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 40 /r     ] AVX512DQ,FUTURE */
        /* 241(f2 fd 48) 301 001(40) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VPMULLQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40909, 547, 6455},
    /* 6337 : VPMULLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40927, 555, 6456},
    /* 6338 : VPMULLW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d5 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40945, 555, 6457},
    /* 6339 : VPMULLW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d5 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d5) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPMULLW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+40963, 556, 6458},
    /* 6340 : VPMULTISHIFTQB xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 83 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 fd 08) 301 001(83) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40981, 559, 6459},
    /* 6341 : VPMULTISHIFTQB ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 83 /r     ] AVX512VL,AVX512VBMI,FUTURE */
        /* 241(f2 fd 28) 301 001(83) 120 : EVEX,PROT,AVX512VL,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+40999, 559, 6460},
    /* 6342 : VPMULTISHIFTQB zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 83 /r     ] AVX512VBMI,FUTURE */
        /* 241(f2 fd 48) 301 001(83) 120 : EVEX,PROT,AVX512VBMI,FUTURE */
        {I_VPMULTISHIFTQB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41017, 560, 6461},
    /* 6343 : VPMULUDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 f4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(f4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULUDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41035, 543, 6462},
    /* 6344 : VPMULUDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 f4 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(f4) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPMULUDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41053, 543, 6463},
    /* 6345 : VPMULUDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 f4 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(f4) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPMULUDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41071, 544, 6464},
    /* 6346 : VPORD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41089, 543, 6465},
    /* 6347 : VPORD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41107, 543, 6466},
    /* 6348 : VPORD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 eb /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(eb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPORD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41125, 544, 6467},
    /* 6349 : VPORQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41143, 543, 6468},
    /* 6350 : VPORQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 eb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(eb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPORQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41161, 543, 6469},
    /* 6351 : VPORQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 eb /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(eb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPORQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41179, 544, 6470},
    /* 6352 : VPROLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20103, 543, 6471},
    /* 6353 : VPROLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20123, 543, 6472},
    /* 6354 : VPROLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /1 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 211 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20143, 544, 6473},
    /* 6355 : VPROLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20163, 543, 6474},
    /* 6356 : VPROLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /1 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 211 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20183, 543, 6475},
    /* 6357 : VPROLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /1 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 211 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20203, 544, 6476},
    /* 6358 : VPROLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41197, 543, 6477},
    /* 6359 : VPROLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41215, 543, 6478},
    /* 6360 : VPROLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 15 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41233, 544, 6479},
    /* 6361 : VPROLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41251, 543, 6480},
    /* 6362 : VPROLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 15 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPROLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41269, 543, 6481},
    /* 6363 : VPROLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 15 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPROLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41287, 544, 6482},
    /* 6364 : VPRORD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20223, 543, 6483},
    /* 6365 : VPRORD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20243, 543, 6484},
    /* 6366 : VPRORD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /0 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 210 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20263, 544, 6485},
    /* 6367 : VPRORQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20283, 543, 6486},
    /* 6368 : VPRORQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /0 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 210 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20303, 543, 6487},
    /* 6369 : VPRORQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /0 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 210 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20323, 544, 6488},
    /* 6370 : VPRORVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41305, 543, 6489},
    /* 6371 : VPRORVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41323, 543, 6490},
    /* 6372 : VPRORVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 14 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41341, 544, 6491},
    /* 6373 : VPRORVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41359, 543, 6492},
    /* 6374 : VPRORVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 14 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPRORVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41377, 543, 6493},
    /* 6375 : VPRORVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 14 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPRORVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41395, 544, 6494},
    /* 6376 : VPSADBW xmmreg,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f6 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, NO_DECORATOR, nasm_bytecodes+41413, 555, 6495},
    /* 6377 : VPSADBW ymmreg,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f6 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+41431, 555, 6496},
    /* 6378 : VPSADBW zmmreg,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f6 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f6) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSADBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+41449, 556, 6497},
    /* 6379 : VPSCATTERDD xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDD, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20343, 543, 6498},
    /* 6380 : VPSCATTERDD ymem32|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDD, 2, {YMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20353, 543, 6499},
    /* 6381 : VPSCATTERDD zmem32|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a0 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a0) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERDD, 2, {ZMEM|BITS32,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20363, 544, 6500},
    /* 6382 : VPSCATTERDQ xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDQ, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20373, 543, 6501},
    /* 6383 : VPSCATTERDQ xmem64|mask,ymmreg [mr:t1s:   vsibx evex.256.66.0f38.w1 a0 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(a0) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERDQ, 2, {XMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20383, 543, 6502},
    /* 6384 : VPSCATTERDQ ymem64|mask,zmmreg [mr:t1s:   vsiby evex.512.66.0f38.w1 a0 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(a0) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERDQ, 2, {YMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20393, 544, 6503},
    /* 6385 : VPSCATTERQD xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQD, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20403, 543, 6504},
    /* 6386 : VPSCATTERQD ymem32|mask,xmmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQD, 2, {YMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20413, 543, 6505},
    /* 6387 : VPSCATTERQD zmem32|mask,ymmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a1 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a1) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERQD, 2, {ZMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20423, 544, 6506},
    /* 6388 : VPSCATTERQQ xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQQ, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20433, 543, 6507},
    /* 6389 : VPSCATTERQQ ymem64|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w1 a1 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(a1) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSCATTERQQ, 2, {YMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20443, 543, 6508},
    /* 6390 : VPSCATTERQQ zmem64|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w1 a1 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(a1) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSCATTERQQ, 2, {ZMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+20453, 544, 6509},
    /* 6391 : VPSHUFB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.wig 00 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(00) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41467, 555, 6510},
    /* 6392 : VPSHUFB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.wig 00 /r    ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(00) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41485, 555, 6511},
    /* 6393 : VPSHUFB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.wig 00 /r    ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(00) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41503, 556, 6512},
    /* 6394 : VPSHUFD xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f.w0 70 /r ib        ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 08) 301 001(70) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSHUFD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20463, 543, 6513},
    /* 6395 : VPSHUFD ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f.w0 70 /r ib        ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7d 28) 301 001(70) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSHUFD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20473, 543, 6514},
    /* 6396 : VPSHUFD zmmreg|mask|z,zmmrm512|b32,imm8 [rmi:fv:   evex.512.66.0f.w0 70 /r ib        ] AVX512,FUTURE */
        /* 250(f1 7d 48) 301 001(70) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSHUFD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20483, 544, 6515},
    /* 6397 : VPSHUFHW xmmreg|mask|z,xmmrm128,imm8 [rmi:fvm:  evex.128.f3.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7e 08) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20493, 555, 6516},
    /* 6398 : VPSHUFHW ymmreg|mask|z,ymmrm256,imm8 [rmi:fvm:  evex.256.f3.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7e 28) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20503, 555, 6517},
    /* 6399 : VPSHUFHW zmmreg|mask|z,zmmrm512,imm8 [rmi:fvm:  evex.512.f3.0f.wig 70 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7e 48) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFHW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20513, 556, 6518},
    /* 6400 : VPSHUFLW xmmreg|mask|z,xmmrm128,imm8 [rmi:fvm:  evex.128.f2.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 08) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20523, 555, 6519},
    /* 6401 : VPSHUFLW ymmreg|mask|z,ymmrm256,imm8 [rmi:fvm:  evex.256.f2.0f.wig 70 /r ib       ] AVX512VL,AVX512BW,FUTURE */
        /* 250(f1 7f 28) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20533, 555, 6520},
    /* 6402 : VPSHUFLW zmmreg|mask|z,zmmrm512,imm8 [rmi:fvm:  evex.512.f2.0f.wig 70 /r ib       ] AVX512BW,FUTURE */
        /* 250(f1 7f 48) 303 001(70) 110 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSHUFLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20543, 556, 6521},
    /* 6403 : VPSLLD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 f2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(f2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41521, 543, 6522},
    /* 6404 : VPSLLD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 f2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(f2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41539, 543, 6523},
    /* 6405 : VPSLLD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 f2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(f2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41557, 544, 6524},
    /* 6406 : VPSLLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20553, 543, 6525},
    /* 6407 : VPSLLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20573, 543, 6526},
    /* 6408 : VPSLLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /6 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 216 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20593, 544, 6527},
    /* 6409 : VPSLLDQ xmmreg,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 73 /7 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20613, 555, 6528},
    /* 6410 : VPSLLDQ ymmreg,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 73 /7 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20633, 555, 6529},
    /* 6411 : VPSLLDQ zmmreg,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 73 /7 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(73) 217 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLDQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+20653, 556, 6530},
    /* 6412 : VPSLLQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 f3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(f3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41575, 543, 6531},
    /* 6413 : VPSLLQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 f3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(f3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41593, 543, 6532},
    /* 6414 : VPSLLQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 f3 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(f3) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41611, 544, 6533},
    /* 6415 : VPSLLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 73 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(73) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20673, 543, 6534},
    /* 6416 : VPSLLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 73 /6 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(73) 216 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20693, 543, 6535},
    /* 6417 : VPSLLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 73 /6 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(73) 216 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20713, 544, 6536},
    /* 6418 : VPSLLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41629, 543, 6537},
    /* 6419 : VPSLLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41647, 543, 6538},
    /* 6420 : VPSLLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 47 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(47) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41665, 544, 6539},
    /* 6421 : VPSLLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41683, 543, 6540},
    /* 6422 : VPSLLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 47 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(47) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSLLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41701, 543, 6541},
    /* 6423 : VPSLLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 47 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(47) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSLLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+41719, 544, 6542},
    /* 6424 : VPSLLVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 12 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(12) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41737, 549, 6543},
    /* 6425 : VPSLLVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 12 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(12) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41755, 549, 6544},
    /* 6426 : VPSLLVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 12 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(12) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41773, 550, 6545},
    /* 6427 : VPSLLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig f1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41791, 555, 6546},
    /* 6428 : VPSLLW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig f1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41809, 555, 6547},
    /* 6429 : VPSLLW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig f1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(f1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41827, 556, 6548},
    /* 6430 : VPSLLW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /6 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20733, 555, 6549},
    /* 6431 : VPSLLW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /6 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20753, 555, 6550},
    /* 6432 : VPSLLW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /6 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 216 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSLLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20773, 556, 6551},
    /* 6433 : VPSRAD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41845, 543, 6552},
    /* 6434 : VPSRAD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41863, 543, 6553},
    /* 6435 : VPSRAD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 e2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(e2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41881, 544, 6554},
    /* 6436 : VPSRAD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20793, 543, 6555},
    /* 6437 : VPSRAD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20813, 543, 6556},
    /* 6438 : VPSRAD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /4 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 214 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20833, 544, 6557},
    /* 6439 : VPSRAQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41899, 543, 6558},
    /* 6440 : VPSRAQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 e2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(e2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41917, 543, 6559},
    /* 6441 : VPSRAQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 e2 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(e2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+41935, 544, 6560},
    /* 6442 : VPSRAQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20853, 543, 6561},
    /* 6443 : VPSRAQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 72 /4 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(72) 214 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20873, 543, 6562},
    /* 6444 : VPSRAQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 72 /4 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(72) 214 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+20893, 544, 6563},
    /* 6445 : VPSRAVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41953, 543, 6564},
    /* 6446 : VPSRAVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41971, 543, 6565},
    /* 6447 : VPSRAVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 46 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(46) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+41989, 544, 6566},
    /* 6448 : VPSRAVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42007, 543, 6567},
    /* 6449 : VPSRAVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 46 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(46) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRAVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42025, 543, 6568},
    /* 6450 : VPSRAVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 46 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(46) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRAVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42043, 544, 6569},
    /* 6451 : VPSRAVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 11 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(11) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42061, 549, 6570},
    /* 6452 : VPSRAVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 11 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(11) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42079, 549, 6571},
    /* 6453 : VPSRAVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 11 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(11) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42097, 550, 6572},
    /* 6454 : VPSRAW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig e1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42115, 555, 6573},
    /* 6455 : VPSRAW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig e1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42133, 555, 6574},
    /* 6456 : VPSRAW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig e1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(e1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42151, 556, 6575},
    /* 6457 : VPSRAW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /4 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20913, 555, 6576},
    /* 6458 : VPSRAW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /4 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20933, 555, 6577},
    /* 6459 : VPSRAW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /4 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 214 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRAW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+20953, 556, 6578},
    /* 6460 : VPSRLD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w0 d2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 317 001(d2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42169, 543, 6579},
    /* 6461 : VPSRLD ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w0 d2 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 317 001(d2) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42187, 543, 6580},
    /* 6462 : VPSRLD zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w0 d2 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 317 001(d2) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLD, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42205, 544, 6581},
    /* 6463 : VPSRLD xmmreg|mask|z,xmmrm128|b32*,imm8 [vmi:fv:   evex.nds.128.66.0f.w0 72 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 08) 301 001(72) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20973, 543, 6582},
    /* 6464 : VPSRLD ymmreg|mask|z,ymmrm256|b32*,imm8 [vmi:fv:   evex.nds.256.66.0f.w0 72 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 7d 28) 301 001(72) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+20993, 543, 6583},
    /* 6465 : VPSRLD zmmreg|mask|z,zmmrm512|b32*,imm8 [vmi:fv:   evex.nds.512.66.0f.w0 72 /2 ib    ] AVX512,FUTURE */
        /* 240(f1 7d 48) 301 001(72) 212 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21013, 544, 6584},
    /* 6466 : VPSRLDQ xmmreg,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 73 /3 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21033, 555, 6585},
    /* 6467 : VPSRLDQ ymmreg,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 73 /3 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21053, 555, 6586},
    /* 6468 : VPSRLDQ zmmreg,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 73 /3 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(73) 213 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLDQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+21073, 556, 6587},
    /* 6469 : VPSRLQ xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.w1 d3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 317 001(d3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42223, 543, 6588},
    /* 6470 : VPSRLQ ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.w1 d3 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 317 001(d3) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42241, 543, 6589},
    /* 6471 : VPSRLQ zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.w1 d3 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 317 001(d3) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLQ, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42259, 544, 6590},
    /* 6472 : VPSRLQ xmmreg|mask|z,xmmrm128|b64*,imm8 [vmi:fv:   evex.nds.128.66.0f.w1 73 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 08) 301 001(73) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21093, 543, 6591},
    /* 6473 : VPSRLQ ymmreg|mask|z,ymmrm256|b64*,imm8 [vmi:fv:   evex.nds.256.66.0f.w1 73 /2 ib    ] AVX512,AVX512VL,FUTURE */
        /* 240(f1 fd 28) 301 001(73) 212 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLQ, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21113, 543, 6592},
    /* 6474 : VPSRLQ zmmreg|mask|z,zmmrm512|b64*,imm8 [vmi:fv:   evex.nds.512.66.0f.w1 73 /2 ib    ] AVX512,FUTURE */
        /* 240(f1 fd 48) 301 001(73) 212 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLQ, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21133, 544, 6593},
    /* 6475 : VPSRLVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42277, 543, 6594},
    /* 6476 : VPSRLVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42295, 543, 6595},
    /* 6477 : VPSRLVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 45 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(45) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42313, 544, 6596},
    /* 6478 : VPSRLVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42331, 543, 6597},
    /* 6479 : VPSRLVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 45 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(45) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSRLVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42349, 543, 6598},
    /* 6480 : VPSRLVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 45 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(45) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSRLVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42367, 544, 6599},
    /* 6481 : VPSRLVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 10 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(10) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42385, 549, 6600},
    /* 6482 : VPSRLVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 10 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(10) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42403, 549, 6601},
    /* 6483 : VPSRLVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 10 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(10) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42421, 550, 6602},
    /* 6484 : VPSRLW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:m128: evex.nds.128.66.0f.wig d1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42439, 555, 6603},
    /* 6485 : VPSRLW ymmreg|mask|z,ymmreg*,xmmrm128 [rvm:m128: evex.nds.256.66.0f.wig d1 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {YMMREG,YMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42457, 555, 6604},
    /* 6486 : VPSRLW zmmreg|mask|z,zmmreg*,xmmrm128 [rvm:m128: evex.nds.512.66.0f.wig d1 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 317 001(d1) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {ZMMREG,ZMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42475, 556, 6605},
    /* 6487 : VPSRLW xmmreg|mask|z,xmmrm128*,imm8 [vmi:fvm:  evex.nds.128.66.0f.wig 71 /2 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 08) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21153, 555, 6606},
    /* 6488 : VPSRLW ymmreg|mask|z,ymmrm256*,imm8 [vmi:fvm:  evex.nds.256.66.0f.wig 71 /2 ib   ] AVX512VL,AVX512BW,FUTURE */
        /* 240(f1 7d 28) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21173, 555, 6607},
    /* 6489 : VPSRLW zmmreg|mask|z,zmmrm512*,imm8 [vmi:fvm:  evex.nds.512.66.0f.wig 71 /2 ib   ] AVX512BW,FUTURE */
        /* 240(f1 7d 48) 303 001(71) 212 022 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSRLW, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+21193, 556, 6608},
    /* 6490 : VPSUBB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42493, 555, 6609},
    /* 6491 : VPSUBB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42511, 555, 6610},
    /* 6492 : VPSUBB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42529, 556, 6611},
    /* 6493 : VPSUBD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 fa /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(fa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42547, 543, 6612},
    /* 6494 : VPSUBD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 fa /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(fa) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42565, 543, 6613},
    /* 6495 : VPSUBD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 fa /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(fa) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSUBD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+42583, 544, 6614},
    /* 6496 : VPSUBQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 fb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(fb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42601, 543, 6615},
    /* 6497 : VPSUBQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 fb /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(fb) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPSUBQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42619, 543, 6616},
    /* 6498 : VPSUBQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 fb /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(fb) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPSUBQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+42637, 544, 6617},
    /* 6499 : VPSUBSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42655, 555, 6618},
    /* 6500 : VPSUBSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42673, 555, 6619},
    /* 6501 : VPSUBSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42691, 556, 6620},
    /* 6502 : VPSUBSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig e9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42709, 555, 6621},
    /* 6503 : VPSUBSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig e9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42727, 555, 6622},
    /* 6504 : VPSUBSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig e9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(e9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42745, 556, 6623},
    /* 6505 : VPSUBUSB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42763, 555, 6624},
    /* 6506 : VPSUBUSB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d8 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42781, 555, 6625},
    /* 6507 : VPSUBUSB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d8 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d8) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBUSB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42799, 556, 6626},
    /* 6508 : VPSUBUSW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig d9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42817, 555, 6627},
    /* 6509 : VPSUBUSW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig d9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42835, 555, 6628},
    /* 6510 : VPSUBUSW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig d9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(d9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBUSW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42853, 556, 6629},
    /* 6511 : VPSUBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig f9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42871, 555, 6630},
    /* 6512 : VPSUBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig f9 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42889, 555, 6631},
    /* 6513 : VPSUBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig f9 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(f9) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPSUBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+42907, 556, 6632},
    /* 6514 : VPTERNLOGD xmmreg|mask|z,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 08) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21213, 543, 6633},
    /* 6515 : VPTERNLOGD ymmreg|mask|z,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21223, 543, 6634},
    /* 6516 : VPTERNLOGD zmmreg|mask|z,zmmreg,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 25 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(25) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTERNLOGD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21233, 544, 6635},
    /* 6517 : VPTERNLOGQ xmmreg|mask|z,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 08) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21243, 543, 6636},
    /* 6518 : VPTERNLOGQ ymmreg|mask|z,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 25 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(25) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTERNLOGQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21253, 543, 6637},
    /* 6519 : VPTERNLOGQ zmmreg|mask|z,zmmreg,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 25 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(25) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTERNLOGQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21263, 544, 6638},
    /* 6520 : VPTESTMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42925, 549, 6639},
    /* 6521 : VPTESTMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7d 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42934, 549, 6640},
    /* 6522 : VPTESTMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7d 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+42943, 550, 6641},
    /* 6523 : VPTESTMD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42952, 543, 6642},
    /* 6524 : VPTESTMD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42961, 543, 6643},
    /* 6525 : VPTESTMD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f38.w0 27 /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTMD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+42970, 544, 6644},
    /* 6526 : VPTESTMQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+42979, 543, 6645},
    /* 6527 : VPTESTMQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+42988, 543, 6646},
    /* 6528 : VPTESTMQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f38.w1 27 /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTMQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+42997, 544, 6647},
    /* 6529 : VPTESTMW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43006, 549, 6648},
    /* 6530 : VPTESTMW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fd 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43015, 549, 6649},
    /* 6531 : VPTESTMW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w1 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fd 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTMW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43024, 550, 6650},
    /* 6532 : VPTESTNMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.f3.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7e 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43033, 549, 6651},
    /* 6533 : VPTESTNMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.f3.0f38.w0 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 7e 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43042, 549, 6652},
    /* 6534 : VPTESTNMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.f3.0f38.w0 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 7e 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTNMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43051, 550, 6653},
    /* 6535 : VPTESTNMD kreg|mask,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7e 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43060, 543, 6654},
    /* 6536 : VPTESTNMD kreg|mask,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.0f38.w0 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7e 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43069, 543, 6655},
    /* 6537 : VPTESTNMD kreg|mask,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.f3.0f38.w0 27 /r     ] AVX512,FUTURE */
        /* 241(f2 7e 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTNMD, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B32,0,0}, nasm_bytecodes+43078, 544, 6656},
    /* 6538 : VPTESTNMQ kreg|mask,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.f3.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fe 08) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43087, 543, 6657},
    /* 6539 : VPTESTNMQ kreg|mask,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.f3.0f38.w1 27 /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fe 28) 301 001(27) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43096, 543, 6658},
    /* 6540 : VPTESTNMQ kreg|mask,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.f3.0f38.w1 27 /r     ] AVX512,FUTURE */
        /* 241(f2 fe 48) 301 001(27) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPTESTNMQ, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,B64,0,0}, nasm_bytecodes+43105, 544, 6659},
    /* 6541 : VPTESTNMW kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.f3.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fe 08) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43114, 549, 6660},
    /* 6542 : VPTESTNMW kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.f3.0f38.w1 26 /r     ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f2 fe 28) 303 001(26) 120 : EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43123, 549, 6661},
    /* 6543 : VPTESTNMW kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.f3.0f38.w1 26 /r     ] AVX512BW,FUTURE */
        /* 241(f2 fe 48) 303 001(26) 120 : EVEX,PROT,AVX512BW,FUTURE */
        {I_VPTESTNMW, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+43132, 550, 6662},
    /* 6544 : VPUNPCKHBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 68 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(68) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43141, 555, 6663},
    /* 6545 : VPUNPCKHBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 68 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(68) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43159, 555, 6664},
    /* 6546 : VPUNPCKHBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 68 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(68) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKHBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43177, 556, 6665},
    /* 6547 : VPUNPCKHDQ xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 6a /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(6a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43195, 543, 6666},
    /* 6548 : VPUNPCKHDQ ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 6a /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(6a) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43213, 543, 6667},
    /* 6549 : VPUNPCKHDQ zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 6a /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(6a) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKHDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43231, 544, 6668},
    /* 6550 : VPUNPCKHQDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 6d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(6d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43249, 543, 6669},
    /* 6551 : VPUNPCKHQDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 6d /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(6d) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKHQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43267, 543, 6670},
    /* 6552 : VPUNPCKHQDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 6d /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(6d) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKHQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43285, 544, 6671},
    /* 6553 : VPUNPCKHWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 69 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(69) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43303, 555, 6672},
    /* 6554 : VPUNPCKHWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 69 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(69) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43321, 555, 6673},
    /* 6555 : VPUNPCKHWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 69 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(69) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKHWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43339, 556, 6674},
    /* 6556 : VPUNPCKLBW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 60 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(60) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43357, 555, 6675},
    /* 6557 : VPUNPCKLBW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 60 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(60) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43375, 555, 6676},
    /* 6558 : VPUNPCKLBW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 60 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(60) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKLBW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43393, 556, 6677},
    /* 6559 : VPUNPCKLDQ xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 62 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(62) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43411, 543, 6678},
    /* 6560 : VPUNPCKLDQ ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 62 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(62) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43429, 543, 6679},
    /* 6561 : VPUNPCKLDQ zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 62 /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(62) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKLDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43447, 544, 6680},
    /* 6562 : VPUNPCKLQDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 6c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(6c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLQDQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43465, 543, 6681},
    /* 6563 : VPUNPCKLQDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 6c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(6c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPUNPCKLQDQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43483, 543, 6682},
    /* 6564 : VPUNPCKLQDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 6c /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(6c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPUNPCKLQDQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43501, 544, 6683},
    /* 6565 : VPUNPCKLWD xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f.wig 61 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 08) 303 001(61) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43519, 555, 6684},
    /* 6566 : VPUNPCKLWD ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f.wig 61 /r      ] AVX512VL,AVX512BW,FUTURE */
        /* 241(f1 7d 28) 303 001(61) 120 : WIG,EVEX,PROT,AVX512VL,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43537, 555, 6685},
    /* 6567 : VPUNPCKLWD zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f.wig 61 /r      ] AVX512BW,FUTURE */
        /* 241(f1 7d 48) 303 001(61) 120 : WIG,EVEX,PROT,AVX512BW,FUTURE */
        {I_VPUNPCKLWD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43555, 556, 6686},
    /* 6568 : VPXORD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f.w0 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 08) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43573, 543, 6687},
    /* 6569 : VPXORD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f.w0 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7d 28) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43591, 543, 6688},
    /* 6570 : VPXORD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.66.0f.w0 ef /r       ] AVX512,FUTURE */
        /* 241(f1 7d 48) 301 001(ef) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPXORD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+43609, 544, 6689},
    /* 6571 : VPXORQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43627, 543, 6690},
    /* 6572 : VPXORQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 ef /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(ef) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VPXORQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43645, 543, 6691},
    /* 6573 : VPXORQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 ef /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(ef) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VPXORQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43663, 544, 6692},
    /* 6574 : VRANGEPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21273, 546, 6693},
    /* 6575 : VRANGEPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 fd 28) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21293, 546, 6694},
    /* 6576 : VRANGEPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 50 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 48) 301 001(50) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGEPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+21313, 547, 6695},
    /* 6577 : VRANGEPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21333, 546, 6696},
    /* 6578 : VRANGEPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 50 /r ib  ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f3 7d 28) 301 001(50) 120 023 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21353, 546, 6697},
    /* 6579 : VRANGEPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 50 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 48) 301 001(50) 120 023 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGEPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+21373, 547, 6698},
    /* 6580 : VRANGESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 51 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(51) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21393, 553, 6699},
    /* 6581 : VRANGESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 51 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(51) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VRANGESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21413, 553, 6700},
    /* 6582 : VRCP14PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43681, 543, 6701},
    /* 6583 : VRCP14PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43690, 543, 6702},
    /* 6584 : VRCP14PD zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 4c /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(4c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43699, 544, 6703},
    /* 6585 : VRCP14PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43708, 543, 6704},
    /* 6586 : VRCP14PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 4c /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(4c) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRCP14PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43717, 543, 6705},
    /* 6587 : VRCP14PS zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 4c /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(4c) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43726, 544, 6706},
    /* 6588 : VRCP14SD xmmreg|mask|z,xmmreg*,xmmrm64 [rvm:t1s:  evex.nds.lig.66.0f38.w1 4d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43735, 545, 6707},
    /* 6589 : VRCP14SS xmmreg|mask|z,xmmreg*,xmmrm32 [rvm:t1s:  evex.nds.lig.66.0f38.w0 4d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRCP14SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43753, 545, 6708},
    /* 6590 : VRCP28PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 ca /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(ca) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+43771, 551, 6709},
    /* 6591 : VRCP28PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 ca /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(ca) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+43780, 551, 6710},
    /* 6592 : VRCP28SD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 cb /r     ] AVX512ER,FUTURE */
        /* 241(f2 fd 08) 306 001(cb) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43789, 564, 6711},
    /* 6593 : VRCP28SS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 cb /r     ] AVX512ER,FUTURE */
        /* 241(f2 7d 08) 306 001(cb) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRCP28SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43807, 564, 6712},
    /* 6594 : VREDUCEPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21433, 546, 6713},
    /* 6595 : VREDUCEPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 fd 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21443, 546, 6714},
    /* 6596 : VREDUCEPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 56 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 fd 48) 301 001(56) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCEPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+21453, 547, 6715},
    /* 6597 : VREDUCEPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21463, 546, 6716},
    /* 6598 : VREDUCEPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 56 /r ib      ] AVX512VL,AVX512DQ,FUTURE */
        /* 250(f3 7d 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21473, 546, 6717},
    /* 6599 : VREDUCEPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 56 /r ib      ] AVX512DQ,FUTURE */
        /* 250(f3 7d 48) 301 001(56) 110 022 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCEPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+21483, 547, 6718},
    /* 6600 : VREDUCESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 57 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 fd 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21493, 553, 6719},
    /* 6601 : VREDUCESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 57 /r ib  ] AVX512DQ,FUTURE */
        /* 241(f3 7d 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512DQ,FUTURE */
        {I_VREDUCESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21513, 553, 6720},
    /* 6602 : VRNDSCALEPD xmmreg|mask|z,xmmrm128|b64,imm8 [rmi:fv:   evex.128.66.0f3a.w1 09 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 08) 301 001(09) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPD, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21533, 543, 6721},
    /* 6603 : VRNDSCALEPD ymmreg|mask|z,ymmrm256|b64,imm8 [rmi:fv:   evex.256.66.0f3a.w1 09 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 fd 28) 301 001(09) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPD, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+21543, 543, 6722},
    /* 6604 : VRNDSCALEPD zmmreg|mask|z,zmmrm512|b64|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w1 09 /r ib      ] AVX512,FUTURE */
        /* 250(f3 fd 48) 301 001(09) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALEPD, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+21553, 544, 6723},
    /* 6605 : VRNDSCALEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21563, 565, 6724},
    /* 6606 : VRNDSCALEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21573, 565, 6725},
    /* 6607 : VRNDSCALEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 08 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(08) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRNDSCALEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+21583, 566, 6726},
    /* 6608 : VRNDSCALEPS xmmreg|mask|z,xmmrm128|b32,imm8 [rmi:fv:   evex.128.66.0f3a.w0 08 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 301 001(08) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPS, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21593, 543, 6727},
    /* 6609 : VRNDSCALEPS ymmreg|mask|z,ymmrm256|b32,imm8 [rmi:fv:   evex.256.66.0f3a.w0 08 /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 301 001(08) 110 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRNDSCALEPS, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+21603, 543, 6728},
    /* 6610 : VRNDSCALEPS zmmreg|mask|z,zmmrm512|b32|sae,imm8 [rmi:fv:   evex.512.66.0f3a.w0 08 /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 301 001(08) 110 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALEPS, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+21613, 544, 6729},
    /* 6611 : VRNDSCALESD xmmreg|mask|z,xmmreg*,xmmrm64|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w1 0b /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 08) 306 001(0b) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALESD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21623, 545, 6730},
    /* 6612 : VRNDSCALESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 0a /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRNDSCALESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21643, 567, 6731},
    /* 6613 : VRNDSCALESS xmmreg|mask|z,xmmreg*,xmmrm32|sae,imm8 [rvmi:t1s: evex.nds.lig.66.0f3a.w0 0a /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRNDSCALESS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21663, 545, 6732},
    /* 6614 : VRSQRT14PD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f38.w1 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 08) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43825, 543, 6733},
    /* 6615 : VRSQRT14PD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f38.w1 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 fd 28) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43834, 543, 6734},
    /* 6616 : VRSQRT14PD zmmreg|mask|z,zmmrm512|b64 [rm:fv:    evex.512.66.0f38.w1 4e /r         ] AVX512,FUTURE */
        /* 250(f2 fd 48) 301 001(4e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+43843, 544, 6735},
    /* 6617 : VRSQRT14PS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.0f38.w0 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43852, 543, 6736},
    /* 6618 : VRSQRT14PS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.0f38.w0 4e /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 301 001(4e) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VRSQRT14PS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43861, 543, 6737},
    /* 6619 : VRSQRT14PS zmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.66.0f38.w0 4e /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 301 001(4e) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+43870, 544, 6738},
    /* 6620 : VRSQRT14SD xmmreg|mask|z,xmmreg*,xmmrm64 [rvm:t1s:  evex.nds.lig.66.0f38.w1 4f /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(4f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43879, 545, 6739},
    /* 6621 : VRSQRT14SS xmmreg|mask|z,xmmreg*,xmmrm32 [rvm:t1s:  evex.nds.lig.66.0f38.w0 4f /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(4f) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VRSQRT14SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+43897, 545, 6740},
    /* 6622 : VRSQRT28PD zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.0f38.w1 cc /r         ] AVX512ER,FUTURE */
        /* 250(f2 fd 48) 301 001(cc) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28PD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+43915, 551, 6741},
    /* 6623 : VRSQRT28PS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.0f38.w0 cc /r         ] AVX512ER,FUTURE */
        /* 250(f2 7d 48) 301 001(cc) 110 : EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28PS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+43924, 551, 6742},
    /* 6624 : VRSQRT28SD xmmreg|mask|z,xmmreg*,xmmrm64|sae [rvm:t1s:  evex.nds.lig.66.0f38.w1 cd /r     ] AVX512ER,FUTURE */
        /* 241(f2 fd 08) 306 001(cd) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28SD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43933, 564, 6743},
    /* 6625 : VRSQRT28SS xmmreg|mask|z,xmmreg*,xmmrm32|sae [rvm:t1s:  evex.nds.lig.66.0f38.w0 cd /r     ] AVX512ER,FUTURE */
        /* 241(f2 7d 08) 306 001(cd) 120 : LIG,EVEX,PROT,AVX512ER,FUTURE */
        {I_VRSQRT28SS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+43951, 564, 6744},
    /* 6626 : VSCALEFPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f38.w1 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 08) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43969, 543, 6745},
    /* 6627 : VSCALEFPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f38.w1 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 fd 28) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+43987, 543, 6746},
    /* 6628 : VSCALEFPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f38.w1 2c /r     ] AVX512,FUTURE */
        /* 241(f2 fd 48) 301 001(2c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+44005, 544, 6747},
    /* 6629 : VSCALEFPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.66.0f38.w0 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 08) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44023, 543, 6748},
    /* 6630 : VSCALEFPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.66.0f38.w0 2c /r     ] AVX512,AVX512VL,FUTURE */
        /* 241(f2 7d 28) 301 001(2c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCALEFPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44041, 543, 6749},
    /* 6631 : VSCALEFPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.66.0f38.w0 2c /r     ] AVX512,FUTURE */
        /* 241(f2 7d 48) 301 001(2c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+44059, 544, 6750},
    /* 6632 : VSCALEFSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.66.0f38.w1 2d /r     ] AVX512,FUTURE */
        /* 241(f2 fd 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44077, 545, 6751},
    /* 6633 : VSCALEFSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.66.0f38.w0 2d /r     ] AVX512,FUTURE */
        /* 241(f2 7d 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSCALEFSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44095, 545, 6752},
    /* 6634 : VSCATTERDPD xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPD, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21683, 543, 6753},
    /* 6635 : VSCATTERDPD xmem64|mask,ymmreg [mr:t1s:   vsibx evex.256.66.0f38.w1 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 28) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPD, 2, {XMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21693, 543, 6754},
    /* 6636 : VSCATTERDPD ymem64|mask,zmmreg [mr:t1s:   vsiby evex.512.66.0f38.w1 a2 /r   ] AVX512,FUTURE */
        /* 375 250(f2 fd 48) 306 001(a2) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERDPD, 2, {YMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21703, 544, 6755},
    /* 6637 : VSCATTERDPS xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPS, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21713, 543, 6756},
    /* 6638 : VSCATTERDPS ymem32|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a2 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a2) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERDPS, 2, {YMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21723, 543, 6757},
    /* 6639 : VSCATTERDPS zmem32|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a2 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a2) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERDPS, 2, {ZMEM|BITS32,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21733, 544, 6758},
    /* 6640 : VSCATTERPF0DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /5   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21743, 554, 6759},
    /* 6641 : VSCATTERPF0DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21753, 554, 6760},
    /* 6642 : VSCATTERPF0QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21763, 554, 6761},
    /* 6643 : VSCATTERPF0QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /5   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 205 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF0QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21773, 554, 6762},
    /* 6644 : VSCATTERPF1DPD ymem64|mask [m:t1s:    vsiby evex.512.66.0f38.w1 c6 /6   ] AVX512PF,FUTURE */
        /* 375 250(f2 fd 48) 306 001(c6) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1DPD, 1, {YMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21783, 554, 6763},
    /* 6645 : VSCATTERPF1DPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c6 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c6) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1DPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21793, 554, 6764},
    /* 6646 : VSCATTERPF1QPD zmem64|mask [m:t1s:    vsibz evex.512.66.0f38.w1 c7 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 fd 48) 306 001(c7) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1QPD, 1, {ZMEM|BITS64,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21803, 554, 6765},
    /* 6647 : VSCATTERPF1QPS zmem32|mask [m:t1s:    vsibz evex.512.66.0f38.w0 c7 /6   ] AVX512PF,FUTURE */
        /* 376 250(f2 7d 48) 306 001(c7) 206 : EVEX,PROT,AVX512PF,FUTURE */
        {I_VSCATTERPF1QPS, 1, {ZMEM|BITS32,0,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21813, 554, 6766},
    /* 6648 : VSCATTERQPD xmem64|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w1 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 fd 08) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPD, 2, {XMEM|BITS64,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21823, 543, 6767},
    /* 6649 : VSCATTERQPD ymem64|mask,ymmreg [mr:t1s:   vsiby evex.256.66.0f38.w1 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 fd 28) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPD, 2, {YMEM|BITS64,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21833, 543, 6768},
    /* 6650 : VSCATTERQPD zmem64|mask,zmmreg [mr:t1s:   vsibz evex.512.66.0f38.w1 a3 /r   ] AVX512,FUTURE */
        /* 376 250(f2 fd 48) 306 001(a3) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERQPD, 2, {ZMEM|BITS64,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21843, 544, 6769},
    /* 6651 : VSCATTERQPS xmem32|mask,xmmreg [mr:t1s:   vsibx evex.128.66.0f38.w0 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 374 250(f2 7d 08) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPS, 2, {XMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21853, 543, 6770},
    /* 6652 : VSCATTERQPS ymem32|mask,xmmreg [mr:t1s:   vsiby evex.256.66.0f38.w0 a3 /r   ] AVX512,AVX512VL,FUTURE */
        /* 375 250(f2 7d 28) 306 001(a3) 101 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSCATTERQPS, 2, {YMEM|BITS32,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21863, 543, 6771},
    /* 6653 : VSCATTERQPS zmem32|mask,ymmreg [mr:t1s:   vsibz evex.512.66.0f38.w0 a3 /r   ] AVX512,FUTURE */
        /* 376 250(f2 7d 48) 306 001(a3) 101 : EVEX,PROT,AVX512,FUTURE */
        {I_VSCATTERQPS, 2, {ZMEM|BITS32,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+21873, 544, 6772},
    /* 6654 : VSHUFF32X4 ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 23 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(23) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFF32X4, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21883, 543, 6773},
    /* 6655 : VSHUFF32X4 zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 23 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(23) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFF32X4, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21903, 544, 6774},
    /* 6656 : VSHUFF64X2 ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 23 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(23) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFF64X2, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21923, 543, 6775},
    /* 6657 : VSHUFF64X2 zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 23 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(23) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFF64X2, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+21943, 544, 6776},
    /* 6658 : VSHUFI32X4 ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 43 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 7d 28) 301 001(43) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFI32X4, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21963, 543, 6777},
    /* 6659 : VSHUFI32X4 zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 43 /r ib  ] AVX512,FUTURE */
        /* 241(f3 7d 48) 301 001(43) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFI32X4, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+21983, 544, 6778},
    /* 6660 : VSHUFI64X2 ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 43 /r ib  ] AVX512,AVX512VL,FUTURE */
        /* 241(f3 fd 28) 301 001(43) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFI64X2, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22003, 543, 6779},
    /* 6661 : VSHUFI64X2 zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 43 /r ib  ] AVX512,FUTURE */
        /* 241(f3 fd 48) 301 001(43) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFI64X2, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22023, 544, 6780},
    /* 6662 : VSHUFPD xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f.w1 c6 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22043, 543, 6781},
    /* 6663 : VSHUFPD ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f.w1 c6 /r ib    ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22063, 543, 6782},
    /* 6664 : VSHUFPD zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f.w1 c6 /r ib    ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(c6) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22083, 544, 6783},
    /* 6665 : VSHUFPS xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.0f.w0 c6 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22103, 543, 6784},
    /* 6666 : VSHUFPS ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.0f.w0 c6 /r ib       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(c6) 120 023 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSHUFPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22123, 543, 6785},
    /* 6667 : VSHUFPS zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.0f.w0 c6 /r ib       ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(c6) 120 023 : EVEX,PROT,AVX512,FUTURE */
        {I_VSHUFPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22143, 544, 6786},
    /* 6668 : VSQRTPD xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.0f.w1 51 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 08) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+44113, 543, 6787},
    /* 6669 : VSQRTPD ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.0f.w1 51 /r           ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 fd 28) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+44122, 543, 6788},
    /* 6670 : VSQRTPD zmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.0f.w1 51 /r           ] AVX512,FUTURE */
        /* 250(f1 fd 48) 301 001(51) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTPD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+44131, 544, 6789},
    /* 6671 : VSQRTPS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.0f.w0 51 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 08) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+44140, 543, 6790},
    /* 6672 : VSQRTPS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.0f.w0 51 /r              ] AVX512,AVX512VL,FUTURE */
        /* 250(f1 7c 28) 301 001(51) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSQRTPS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+44149, 543, 6791},
    /* 6673 : VSQRTPS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.0f.w0 51 /r              ] AVX512,FUTURE */
        /* 250(f1 7c 48) 301 001(51) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTPS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+44158, 544, 6792},
    /* 6674 : VSQRTSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 51 /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44167, 545, 6793},
    /* 6675 : VSQRTSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 51 /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSQRTSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44185, 545, 6794},
    /* 6676 : VSUBPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 5c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44203, 543, 6795},
    /* 6677 : VSUBPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 5c /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44221, 543, 6796},
    /* 6678 : VSUBPD zmmreg|mask|z,zmmreg*,zmmrm512|b64|er [rvm:fv:   evex.nds.512.66.0f.w1 5c /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(5c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSUBPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64|ER,0,0}, nasm_bytecodes+44239, 544, 6797},
    /* 6679 : VSUBPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 5c /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44257, 543, 6798},
    /* 6680 : VSUBPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 5c /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(5c) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VSUBPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44275, 543, 6799},
    /* 6681 : VSUBPS zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.0f.w0 5c /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(5c) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VSUBPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+44293, 544, 6800},
    /* 6682 : VSUBSD xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.0f.w1 5c /r       ] AVX512,FUTURE */
        /* 241(f1 ff 08) 306 001(5c) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSUBSD, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44311, 545, 6801},
    /* 6683 : VSUBSS xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.0f.w0 5c /r       ] AVX512,FUTURE */
        /* 241(f1 7e 08) 306 001(5c) 120 : LIG,EVEX,PROT,AVX512,FUTURE */
        {I_VSUBSS, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+44329, 545, 6802},
    /* 6684 : VUCOMISD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.66.0f.w1 2e /r           ] FL,AVX512,FUTURE */
        /* 250(f1 fd 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VUCOMISD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+44347, 548, 6803},
    /* 6685 : VUCOMISS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.0f.w0 2e /r              ] FL,AVX512,FUTURE */
        /* 250(f1 7c 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512,FUTURE */
        {I_VUCOMISS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+44356, 548, 6804},
    /* 6686 : VUNPCKHPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 15 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44365, 543, 6805},
    /* 6687 : VUNPCKHPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 15 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44383, 543, 6806},
    /* 6688 : VUNPCKHPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 15 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKHPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44401, 544, 6807},
    /* 6689 : VUNPCKHPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 15 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44419, 543, 6808},
    /* 6690 : VUNPCKHPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 15 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(15) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKHPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44437, 543, 6809},
    /* 6691 : VUNPCKHPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 15 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(15) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKHPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44455, 544, 6810},
    /* 6692 : VUNPCKLPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 14 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44473, 543, 6811},
    /* 6693 : VUNPCKLPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 14 /r       ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 fd 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44491, 543, 6812},
    /* 6694 : VUNPCKLPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 14 /r       ] AVX512,FUTURE */
        /* 241(f1 fd 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKLPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44509, 544, 6813},
    /* 6695 : VUNPCKLPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 14 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 08) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44527, 543, 6814},
    /* 6696 : VUNPCKLPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 14 /r          ] AVX512,AVX512VL,FUTURE */
        /* 241(f1 7c 28) 301 001(14) 120 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VUNPCKLPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44545, 543, 6815},
    /* 6697 : VUNPCKLPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 14 /r          ] AVX512,FUTURE */
        /* 241(f1 7c 48) 301 001(14) 120 : EVEX,PROT,AVX512,FUTURE */
        {I_VUNPCKLPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44563, 544, 6816},
    /* 6698 : VXORPD xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:   evex.nds.128.66.0f.w1 57 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 08) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44581, 546, 6817},
    /* 6699 : VXORPD ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:   evex.nds.256.66.0f.w1 57 /r       ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 fd 28) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44599, 546, 6818},
    /* 6700 : VXORPD zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:   evex.nds.512.66.0f.w1 57 /r       ] AVX512DQ,FUTURE */
        /* 241(f1 fd 48) 301 001(57) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VXORPD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+44617, 547, 6819},
    /* 6701 : VXORPS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.0f.w0 57 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 08) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44635, 546, 6820},
    /* 6702 : VXORPS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.0f.w0 57 /r          ] AVX512VL,AVX512DQ,FUTURE */
        /* 241(f1 7c 28) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44653, 546, 6821},
    /* 6703 : VXORPS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.nds.512.0f.w0 57 /r          ] AVX512DQ,FUTURE */
        /* 241(f1 7c 48) 301 001(57) 120 : EVEX,PROT,AVX512DQ,FUTURE */
        {I_VXORPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44671, 547, 6822},
    /* 6704 : RDPKRU void [          0f 01 ee                          ] LONG,PROT,PKU,X86_64 */
        /* 355 002(01 ee) : LONG,PROT,PKU,X86_64 */
        {I_RDPKRU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70162, 568, 6826},
    /* 6705 : WRPKRU void [          0f 01 ef                          ] LONG,PROT,PKU,X86_64 */
        /* 355 002(01 ef) : LONG,PROT,PKU,X86_64 */
        {I_WRPKRU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70168, 568, 6827},
    /* 6706 : PTWRITE rm32 [m:        f3 o32 0f ae /4                   ] PTWRITE,FUTURE */
        /* 333 321 355 001(ae) 204 : PTWRITE,FUTURE */
        {I_PTWRITE, 1, {RM_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65883, 569, 6833},
    /* 6707 : PTWRITE rm64 [m:        f3 o64 0f ae /4                   ] AR0,SX,LONG,PROT,PTWRITE,X86_64 */
        /* 333 324 355 001(ae) 204 : AR0,SX,LONG,PROT,PTWRITE,X86_64 */
        {I_PTWRITE, 1, {RM_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65890, 570, 6834},
    /* 6708 : CLDEMOTE mem [m:        np 0f 1c /0                       ] CLDEMOTE,FUTURE */
        /* 360 355 001(1c) 200 : CLDEMOTE,FUTURE */
        {I_CLDEMOTE, 1, {MEMORY,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70065, 571, 6838},
    /* 6709 : MOVDIRI mem32,reg32 [mr:       np o32 0f38 f9 /r                 ] MOVDIRI,FUTURE */
        /* 360 321 356 001(f9) 101 : NOAPX,MOVDIRI,FUTURE */
        {I_MOVDIRI, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65897, 572, 6839},
    /* 6710 : MOVDIRI mem64,reg64 [mr:       np o64 0f38 f9 /r                 ] LONG,PROT,MOVDIRI,X86_64 */
        /* 360 324 356 001(f9) 101 : NOAPX,LONG,PROT,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65904, 573, 6840},
    /* 6711 : MOVDIRI mem32,reg32 [mr:       evex.nd0.nf0.np.l0.m4.w0 f9 /r    ] LONG,PROT,APX,MOVDIRI,X86_64 */
        /* 250(f4 7c 08) 300 001(f9) 101 : LONG,EVEX,PROT,APX,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+44689, 574, 6841},
    /* 6712 : MOVDIRI mem64,reg64 [mr:       evex.nd0.nf0.np.l0.m4.w1 f9 /r    ] LONG,PROT,APX,MOVDIRI,X86_64 */
        /* 250(f4 fc 08) 300 001(f9) 101 : LONG,EVEX,PROT,APX,MOVDIRI,X86_64 */
        {I_MOVDIRI, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+44698, 574, 6842},
    /* 6713 : MOVDIR64B reg16,mem512 [rm:       a16 66 0f38 f8 /r                 ] NOLONG,MOVDIR64B,FUTURE */
        /* 310 361 356 001(f8) 110 : NOREX,NOAPX,NOLONG,MOVDIR64B,FUTURE */
        {I_MOVDIR64B, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65911, 575, 6843},
    /* 6714 : MOVDIR64B reg32,mem512 [rm:       a32 66 0f38 f8 /r                 ] MOVDIR64B,FUTURE */
        /* 311 361 356 001(f8) 110 : NOAPX,MOVDIR64B,FUTURE */
        {I_MOVDIR64B, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65918, 576, 6844},
    /* 6715 : MOVDIR64B reg64,mem512 [rm:       a64 66 0f38 f8 /r                 ] LONG,PROT,MOVDIR64B,X86_64 */
        /* 313 361 356 001(f8) 110 : NOAPX,LONG,PROT,MOVDIR64B,X86_64 */
        {I_MOVDIR64B, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+65925, 577, 6845},
    /* 6716 : MOVDIR64B reg64,mem512 [rm:       evex.nd0.nf0.66.l0.m4.w1 f8 /r    ] LONG,PROT,APX,MOVDIR64B,X86_64 */
        /* 250(f4 fd 08) 300 001(f8) 110 : LONG,EVEX,PROT,APX,MOVDIR64B,X86_64 */
        {I_MOVDIR64B, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+44707, 578, 6846},
    /* 6717 : PCONFIG void [          np 0f 01 c5                       ] FL,PCONFIG,FUTURE */
        /* 360 355 002(01 c5) : FL,PCONFIG,FUTURE */
        {I_PCONFIG, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70071, 579, 6847},
    /* 6718 : PBNDKB void [          np 0f 01 c7                       ] PBNDKB,FUTURE */
        /* 360 355 002(01 c7) : PBNDKB,FUTURE */
        {I_PBNDKB, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70077, 580, 6850},
    /* 6719 : PREFETCHRST2 mem8 [m:        0f 18 /4                          ] MOVRS,FUTURE */
        /* 355 001(18) 204 : MOVRS,FUTURE */
        {I_PREFETCHRST2, 1, {MEMORY|BITS8,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70270, 581, 6851},
    /* 6720 : GF2P8AFFINEINVQB xmmreg,xmmrm128,imm8 [rmi:      66 0f3a cf /r ib                  ] SSE,GFNI,FUTURE */
        /* 361 357 001(cf) 110 022 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8AFFINEINVQB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+65932, 582, 6855},
    /* 6721 : VGF2P8AFFINEINVQB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 cf /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 201) 001(cf) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54779, 583, 6856},
    /* 6722 : VGF2P8AFFINEINVQB ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w1 cf /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 205) 001(cf) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54795, 583, 6857},
    /* 6723 : VGF2P8AFFINEINVQB xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 cf /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 08) 301 001(cf) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22163, 584, 6858},
    /* 6724 : VGF2P8AFFINEINVQB ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 cf /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 28) 301 001(cf) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22183, 584, 6859},
    /* 6725 : VGF2P8AFFINEINVQB zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 cf /r ib  ] AVX512,GFNI,FUTURE */
        /* 241(f3 fd 48) 301 001(cf) 120 023 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8AFFINEINVQB, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22203, 585, 6860},
    /* 6726 : GF2P8AFFINEQB xmmreg,xmmrm128,imm8 [rmi:      66 0f3a ce /r ib                  ] SSE,GFNI,FUTURE */
        /* 361 357 001(ce) 110 022 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8AFFINEQB, 3, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+65939, 582, 6861},
    /* 6727 : VGF2P8AFFINEQB xmmreg,xmmreg*,xmmrm128,imm8 [rvmi:     vex.nds.128.66.0f3a.w1 ce /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 201) 001(ce) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54811, 583, 6862},
    /* 6728 : VGF2P8AFFINEQB ymmreg,ymmreg*,ymmrm256,imm8 [rvmi:     vex.nds.256.66.0f3a.w1 ce /r ib   ] AVX,GFNI,FUTURE */
        /* 261(003 205) 001(ce) 120 023 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,IMM_NORMAL|BITS8,0}, NO_DECORATOR, nasm_bytecodes+54827, 583, 6863},
    /* 6729 : VGF2P8AFFINEQB xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 ce /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 08) 301 001(ce) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22223, 584, 6864},
    /* 6730 : VGF2P8AFFINEQB ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 ce /r ib  ] AVX512VL,GFNI,FUTURE */
        /* 241(f3 fd 28) 301 001(ce) 120 023 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22243, 584, 6865},
    /* 6731 : VGF2P8AFFINEQB zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 ce /r ib  ] AVX512,GFNI,FUTURE */
        /* 241(f3 fd 48) 301 001(ce) 120 023 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8AFFINEQB, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22263, 585, 6866},
    /* 6732 : GF2P8MULB xmmreg,xmmrm128 [rm:       66 0f38 cf /r                     ] SSE,GFNI,FUTURE */
        /* 361 356 001(cf) 110 : NOAPX,SSE,GFNI,FUTURE */
        {I_GF2P8MULB, 2, {XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70083, 582, 6867},
    /* 6733 : VGF2P8MULB xmmreg,xmmreg*,xmmrm128 [rvm:      vex.nds.128.66.0f38.w0 cf /r      ] AVX,GFNI,FUTURE */
        /* 261(002 001) 001(cf) 120 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {XMMREG|RN_L16,XMMREG|RN_L16,RM_XMM|BITS128|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65946, 583, 6868},
    /* 6734 : VGF2P8MULB ymmreg,ymmreg*,ymmrm256 [rvm:      vex.nds.256.66.0f38.w0 cf /r      ] AVX,GFNI,FUTURE */
        /* 261(002 005) 001(cf) 120 : NOAPX,VEX,AVX,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {YMMREG|RN_L16,YMMREG|RN_L16,RM_YMM|BITS256|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+65960, 583, 6869},
    /* 6735 : VGF2P8MULB xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 cf /r     ] AVX512VL,GFNI,FUTURE */
        /* 241(f2 7d 08) 303 001(cf) 120 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44716, 584, 6870},
    /* 6736 : VGF2P8MULB ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 cf /r     ] AVX512VL,GFNI,FUTURE */
        /* 241(f2 7d 28) 303 001(cf) 120 : EVEX,PROT,AVX512VL,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44734, 584, 6871},
    /* 6737 : VGF2P8MULB zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 cf /r     ] AVX512,GFNI,FUTURE */
        /* 241(f2 7d 48) 303 001(cf) 120 : EVEX,PROT,AVX512,GFNI,FUTURE */
        {I_VGF2P8MULB, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44752, 585, 6872},
    /* 6738 : VPCOMPRESSB mem128|mask,xmmreg [mr:t1s8:  evex.128.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 304 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44770, 586, 6875},
    /* 6739 : VPCOMPRESSB mem256|mask,ymmreg [mr:t1s8:  evex.256.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 304 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44779, 586, 6876},
    /* 6740 : VPCOMPRESSB mem512|mask,zmmreg [mr:t1s8:  evex.512.66.0f38.w0 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 304 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44788, 587, 6877},
    /* 6741 : VPCOMPRESSB xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44797, 586, 6878},
    /* 6742 : VPCOMPRESSB ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w0 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44806, 586, 6879},
    /* 6743 : VPCOMPRESSB zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w0 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 300 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSB, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44815, 587, 6880},
    /* 6744 : VPCOMPRESSW mem128|mask,xmmreg [mr:t1s16: evex.128.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 305 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS128,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44824, 586, 6881},
    /* 6745 : VPCOMPRESSW mem256|mask,ymmreg [mr:t1s16: evex.256.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 305 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS256,YMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44833, 586, 6882},
    /* 6746 : VPCOMPRESSW mem512|mask,zmmreg [mr:t1s16: evex.512.66.0f38.w1 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 305 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {MEMORY|BITS512,ZMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+44842, 587, 6883},
    /* 6747 : VPCOMPRESSW xmmreg|mask|z,xmmreg [mr:       evex.128.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {XMMREG,XMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44851, 586, 6884},
    /* 6748 : VPCOMPRESSW ymmreg|mask|z,ymmreg [mr:       evex.256.66.0f38.w1 63 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 300 001(63) 101 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {YMMREG,YMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44860, 586, 6885},
    /* 6749 : VPCOMPRESSW zmmreg|mask|z,zmmreg [mr:       evex.512.66.0f38.w1 63 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 300 001(63) 101 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPCOMPRESSW, 2, {ZMMREG,ZMMREG,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44869, 587, 6886},
    /* 6750 : VPEXPANDB xmmreg|mask|z,xmmrm128 [rm:t1s8:  evex.128.66.0f38.w0 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 08) 304 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44878, 586, 6887},
    /* 6751 : VPEXPANDB ymmreg|mask|z,ymmrm256 [rm:t1s8:  evex.256.66.0f38.w0 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 7d 28) 304 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44887, 586, 6888},
    /* 6752 : VPEXPANDB zmmreg|mask|z,zmmrm512 [rm:t1s8:  evex.512.66.0f38.w0 62 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 7d 48) 304 001(62) 110 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPEXPANDB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44896, 587, 6889},
    /* 6753 : VPEXPANDW xmmreg|mask|z,xmmrm128 [rm:t1s16: evex.128.66.0f38.w1 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 08) 305 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44905, 586, 6890},
    /* 6754 : VPEXPANDW ymmreg|mask|z,ymmrm256 [rm:t1s16: evex.256.66.0f38.w1 62 /r         ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 250(f2 fd 28) 305 001(62) 110 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44914, 586, 6891},
    /* 6755 : VPEXPANDW zmmreg|mask|z,zmmrm512 [rm:t1s16: evex.512.66.0f38.w1 62 /r         ] AVX512VBMI2,FUTURE */
        /* 250(f2 fd 48) 305 001(62) 110 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPEXPANDW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44923, 587, 6892},
    /* 6756 : VPSHLDW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 70 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 303 001(70) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22283, 586, 6893},
    /* 6757 : VPSHLDW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 70 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 303 001(70) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22303, 586, 6894},
    /* 6758 : VPSHLDW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 70 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 303 001(70) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22323, 587, 6895},
    /* 6759 : VPSHLDD xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 08) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22343, 586, 6896},
    /* 6760 : VPSHLDD ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 28) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22363, 586, 6897},
    /* 6761 : VPSHLDD zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 71 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 7d 48) 301 001(71) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22383, 587, 6898},
    /* 6762 : VPSHLDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22403, 586, 6899},
    /* 6763 : VPSHLDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 71 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 301 001(71) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22423, 586, 6900},
    /* 6764 : VPSHLDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 71 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 301 001(71) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22443, 587, 6901},
    /* 6765 : VPSHLDVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm: evex.dds.128.66.0f38.w1 70 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 303 001(70) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44932, 586, 6902},
    /* 6766 : VPSHLDVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm: evex.dds.256.66.0f38.w1 70 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 303 001(70) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44950, 586, 6903},
    /* 6767 : VPSHLDVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm: evex.dds.512.66.0f38.w1 70 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 303 001(70) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+44968, 587, 6904},
    /* 6768 : VPSHLDVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:  evex.dds.128.66.0f38.w0 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 08) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+44986, 586, 6905},
    /* 6769 : VPSHLDVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:  evex.dds.256.66.0f38.w0 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 28) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45004, 586, 6906},
    /* 6770 : VPSHLDVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:  evex.dds.512.66.0f38.w0 71 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 7d 48) 301 001(71) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45022, 587, 6907},
    /* 6771 : VPSHLDVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:  evex.dds.128.66.0f38.w1 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45040, 586, 6908},
    /* 6772 : VPSHLDVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:  evex.dds.256.66.0f38.w1 71 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 301 001(71) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45058, 586, 6909},
    /* 6773 : VPSHLDVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:  evex.dds.512.66.0f38.w1 71 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 301 001(71) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHLDVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45076, 587, 6910},
    /* 6774 : VPSHRDW xmmreg|mask|z,xmmreg*,xmmrm128,imm8 [rvmi:fvm: evex.nds.128.66.0f3a.w1 72 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 303 001(72) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22463, 586, 6911},
    /* 6775 : VPSHRDW ymmreg|mask|z,ymmreg*,ymmrm256,imm8 [rvmi:fvm: evex.nds.256.66.0f3a.w1 72 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 303 001(72) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22483, 586, 6912},
    /* 6776 : VPSHRDW zmmreg|mask|z,zmmreg*,zmmrm512,imm8 [rvmi:fvm: evex.nds.512.66.0f3a.w1 72 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 303 001(72) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+22503, 587, 6913},
    /* 6777 : VPSHRDD xmmreg|mask|z,xmmreg*,xmmrm128|b32,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w0 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 08) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22523, 586, 6914},
    /* 6778 : VPSHRDD ymmreg|mask|z,ymmreg*,ymmrm256|b32,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w0 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 7d 28) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22543, 586, 6915},
    /* 6779 : VPSHRDD zmmreg|mask|z,zmmreg*,zmmrm512|b32,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w0 73 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 7d 48) 301 001(73) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+22563, 587, 6916},
    /* 6780 : VPSHRDQ xmmreg|mask|z,xmmreg*,xmmrm128|b64,imm8 [rvmi:fv:  evex.nds.128.66.0f3a.w1 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 08) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22583, 586, 6917},
    /* 6781 : VPSHRDQ ymmreg|mask|z,ymmreg*,ymmrm256|b64,imm8 [rvmi:fv:  evex.nds.256.66.0f3a.w1 73 /r ib  ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f3 fd 28) 301 001(73) 120 023 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22603, 586, 6918},
    /* 6782 : VPSHRDQ zmmreg|mask|z,zmmreg*,zmmrm512|b64,imm8 [rvmi:fv:  evex.nds.512.66.0f3a.w1 73 /r ib  ] AVX512VBMI2,FUTURE */
        /* 241(f3 fd 48) 301 001(73) 120 023 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDQ, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+22623, 587, 6919},
    /* 6783 : VPSHRDVW xmmreg|mask|z,xmmreg*,xmmrm128 [rvm:fvm: evex.dds.128.66.0f38.w1 72 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 303 001(72) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45094, 586, 6920},
    /* 6784 : VPSHRDVW ymmreg|mask|z,ymmreg*,ymmrm256 [rvm:fvm: evex.dds.256.66.0f38.w1 72 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 303 001(72) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45112, 586, 6921},
    /* 6785 : VPSHRDVW zmmreg|mask|z,zmmreg*,zmmrm512 [rvm:fvm: evex.dds.512.66.0f38.w1 72 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 303 001(72) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVW, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45130, 587, 6922},
    /* 6786 : VPSHRDVD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:  evex.dds.128.66.0f38.w0 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 08) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45148, 586, 6923},
    /* 6787 : VPSHRDVD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:  evex.dds.256.66.0f38.w0 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 7d 28) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45166, 586, 6924},
    /* 6788 : VPSHRDVD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:  evex.dds.512.66.0f38.w0 73 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 7d 48) 301 001(73) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45184, 587, 6925},
    /* 6789 : VPSHRDVQ xmmreg|mask|z,xmmreg*,xmmrm128|b64 [rvm:fv:  evex.dds.128.66.0f38.w1 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 08) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45202, 586, 6926},
    /* 6790 : VPSHRDVQ ymmreg|mask|z,ymmreg*,ymmrm256|b64 [rvm:fv:  evex.dds.256.66.0f38.w1 73 /r     ] AVX512VL,AVX512VBMI2,FUTURE */
        /* 241(f2 fd 28) 301 001(73) 120 : EVEX,PROT,AVX512VL,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45220, 586, 6927},
    /* 6791 : VPSHRDVQ zmmreg|mask|z,zmmreg*,zmmrm512|b64 [rvm:fv:  evex.dds.512.66.0f38.w1 73 /r     ] AVX512VBMI2,FUTURE */
        /* 241(f2 fd 48) 301 001(73) 120 : EVEX,PROT,AVX512VBMI2,FUTURE */
        {I_VPSHRDVQ, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+45238, 587, 6928},
    /* 6792 : VPDPBUSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 50 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(50) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45256, 588, 6931},
    /* 6793 : VPDPBUSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 50 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(50) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45274, 588, 6932},
    /* 6794 : VPDPBUSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 50 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(50) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPBUSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45292, 589, 6933},
    /* 6795 : VPDPBUSDS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 51 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(51) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45310, 588, 6934},
    /* 6796 : VPDPBUSDS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 51 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(51) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45328, 588, 6935},
    /* 6797 : VPDPBUSDS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 51 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(51) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPBUSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45346, 589, 6936},
    /* 6798 : VPDPWSSD xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 52 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45364, 588, 6937},
    /* 6799 : VPDPWSSD ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 52 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45382, 588, 6938},
    /* 6800 : VPDPWSSD zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 52 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(52) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPWSSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45400, 589, 6939},
    /* 6801 : VPDPWSSDS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.dds.128.66.0f38.w0 53 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 08) 301 001(53) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45418, 588, 6940},
    /* 6802 : VPDPWSSDS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.dds.256.66.0f38.w0 53 /r     ] AVX512VL,AVX512VNNI,FUTURE */
        /* 241(f2 7d 28) 301 001(53) 120 : EVEX,PROT,AVX512VL,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45436, 588, 6941},
    /* 6803 : VPDPWSSDS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.dds.512.66.0f38.w0 53 /r     ] AVX512VNNI,FUTURE */
        /* 241(f2 7d 48) 301 001(53) 120 : EVEX,PROT,AVX512VNNI,FUTURE */
        {I_VPDPWSSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45454, 589, 6942},
    /* 6804 : VPOPCNTB xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.w0 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 7d 08) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45472, 590, 6945},
    /* 6805 : VPOPCNTB ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.w0 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 7d 28) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45481, 590, 6946},
    /* 6806 : VPOPCNTB zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.w0 54 /r         ] AVX512BITALG,FUTURE */
        /* 250(f2 7d 48) 303 001(54) 110 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPOPCNTB, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45490, 591, 6947},
    /* 6807 : VPOPCNTW xmmreg|mask|z,xmmrm128 [rm:fvm:   evex.128.66.0f38.w1 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 fd 08) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45499, 590, 6948},
    /* 6808 : VPOPCNTW ymmreg|mask|z,ymmrm256 [rm:fvm:   evex.256.66.0f38.w1 54 /r         ] AVX512VL,AVX512BITALG,FUTURE */
        /* 250(f2 fd 28) 303 001(54) 110 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45508, 590, 6949},
    /* 6809 : VPOPCNTW zmmreg|mask|z,zmmrm512 [rm:fvm:   evex.512.66.0f38.w1 54 /r         ] AVX512BITALG,FUTURE */
        /* 250(f2 fd 48) 303 001(54) 110 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPOPCNTW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45517, 591, 6950},
    /* 6810 : VPOPCNTD xmmreg|mask|z,xmmrm128 [rm:fv:    evex.128.66.0f38.w0 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 08) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45526, 592, 6951},
    /* 6811 : VPOPCNTD ymmreg|mask|z,ymmrm256 [rm:fv:    evex.256.66.0f38.w0 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 28) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45535, 592, 6952},
    /* 6812 : VPOPCNTD zmmreg|mask|z,zmmrm512 [rm:fv:    evex.512.66.0f38.w0 55 /r         ] AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 7d 48) 301 001(55) 110 : EVEX,PROT,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTD, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45544, 593, 6953},
    /* 6813 : VPOPCNTQ xmmreg|mask|z,xmmrm128 [rm:fv:    evex.128.66.0f38.w1 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 08) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45553, 592, 6954},
    /* 6814 : VPOPCNTQ ymmreg|mask|z,ymmrm256 [rm:fv:    evex.256.66.0f38.w1 55 /r         ] AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 28) 301 001(55) 110 : EVEX,PROT,AVX512VL,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45562, 592, 6955},
    /* 6815 : VPOPCNTQ zmmreg|mask|z,zmmrm512 [rm:fv:    evex.512.66.0f38.w1 55 /r         ] AVX512VPOPCNTDQ,FUTURE */
        /* 250(f2 fd 48) 301 001(55) 110 : EVEX,PROT,AVX512VPOPCNTDQ,FUTURE */
        {I_VPOPCNTQ, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45571, 593, 6956},
    /* 6816 : VPSHUFBITQMB kreg|mask,xmmreg,xmmrm128 [rvm:fvm:  evex.nds.128.66.0f38.w0 8f /r     ] AVX512VL,AVX512BITALG,FUTURE */
        /* 241(f2 7d 08) 303 001(8f) 120 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45580, 590, 6957},
    /* 6817 : VPSHUFBITQMB kreg|mask,ymmreg,ymmrm256 [rvm:fvm:  evex.nds.256.66.0f38.w0 8f /r     ] AVX512VL,AVX512BITALG,FUTURE */
        /* 241(f2 7d 28) 303 001(8f) 120 : EVEX,PROT,AVX512VL,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45589, 590, 6958},
    /* 6818 : VPSHUFBITQMB kreg|mask,zmmreg,zmmrm512 [rvm:fvm:  evex.nds.512.66.0f38.w0 8f /r     ] AVX512BITALG,FUTURE */
        /* 241(f2 7d 48) 303 001(8f) 120 : EVEX,PROT,AVX512BITALG,FUTURE */
        {I_VPSHUFBITQMB, 3, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+45598, 591, 6959},
    /* 6819 : V4FMADDPS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 9a /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 48) 317 001(9a) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FMADDPS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45607, 594, 6962},
    /* 6820 : V4FNMADDPS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 aa /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 48) 317 001(aa) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FNMADDPS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45616, 594, 6963},
    /* 6821 : V4FMADDSS xmmreg|mask|z,xmmreg|rs4,mem [rvm:m128: evex.dds.lig.f2.0f38.w0 9b /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 08) 317 001(9b) 120 : AR0,AR1,AR2,SO,LIG,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FMADDSS, 3, {XMMREG,XMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45625, 595, 6964},
    /* 6822 : V4FNMADDSS xmmreg|mask|z,xmmreg|rs4,mem [rvm:m128: evex.dds.lig.f2.0f38.w0 ab /r     ] AR0-2,SO,AVX5124FMAPS,FUTURE */
        /* 241(f2 7f 08) 317 001(ab) 120 : AR0,AR1,AR2,SO,LIG,EVEX,PROT,AVX5124FMAPS,FUTURE */
        {I_V4FNMADDSS, 3, {XMMREG,XMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45634, 595, 6965},
    /* 6823 : VP4DPWSSDS zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 53 /r     ] AR0-2,SO,AVX5124VNNIW,FUTURE */
        /* 241(f2 7f 48) 317 001(53) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124VNNIW,FUTURE */
        {I_VP4DPWSSDS, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45643, 596, 6968},
    /* 6824 : VP4DPWSSD zmmreg|mask|z,zmmreg|rs4,mem [rvm:m128: evex.dds.512.f2.0f38.w0 52 /r     ] AR0-2,SO,AVX5124VNNIW,FUTURE */
        /* 241(f2 7f 48) 317 001(52) 120 : AR0,AR1,AR2,SO,EVEX,PROT,AVX5124VNNIW,FUTURE */
        {I_VP4DPWSSD, 3, {ZMMREG,ZMMREG|RS4,MEMORY,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+45652, 596, 6969},
    /* 6825 : ENCLS void [          np 0f 01 cf                       ] SGX,FUTURE */
        /* 360 355 002(01 cf) : SGX,FUTURE */
        {I_ENCLS, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70089, 597, 6972},
    /* 6826 : ENCLU void [          np 0f 01 d7                       ] SGX,FUTURE */
        /* 360 355 002(01 d7) : SGX,FUTURE */
        {I_ENCLU, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70095, 597, 6973},
    /* 6827 : ENCLV void [          np 0f 01 c0                       ] SGX,FUTURE */
        /* 360 355 002(01 c0) : SGX,FUTURE */
        {I_ENCLV, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70101, 597, 6974},
    /* 6828 : CLRSSBSY mem64 [m:        f3 0f ae /6                       ] FL,CET,FUTURE */
        /* 333 355 001(ae) 206 : FL,CET,FUTURE */
        {I_CLRSSBSY, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+57106, 598, 6977},
    /* 6829 : ENDBR32 void [          f3 0f 1e fb                       ] CET,FUTURE */
        /* 333 355 002(1e fb) : CET,FUTURE */
        {I_ENDBR32, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70107, 599, 6978},
    /* 6830 : ENDBR64 void [          f3 0f 1e fa                       ] CET,FUTURE */
        /* 333 355 002(1e fa) : CET,FUTURE */
        {I_ENDBR64, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70113, 599, 6979},
    /* 6831 : INCSSPD reg32 [m:        o32 f3 0f ae /5                   ] CET,FUTURE */
        /* 321 333 355 001(ae) 205 : CET,FUTURE */
        {I_INCSSPD, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65974, 599, 6980},
    /* 6832 : INCSSPQ reg64 [m:        o64 f3 0f ae /5                   ] LONG,PROT,CET,X86_64 */
        /* 324 333 355 001(ae) 205 : LONG,PROT,CET,X86_64 */
        {I_INCSSPQ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65981, 600, 6981},
    /* 6833 : RDSSPD reg32 [m:        o32 f3 0f 1e /1                   ] CET,FUTURE */
        /* 321 333 355 001(1e) 201 : CET,FUTURE */
        {I_RDSSPD, 1, {REG_GPR|BITS32,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65988, 599, 6982},
    /* 6834 : RDSSPQ reg64 [m:        o64 f3 0f 1e /1                   ] LONG,PROT,CET,X86_64 */
        /* 324 333 355 001(1e) 201 : LONG,PROT,CET,X86_64 */
        {I_RDSSPQ, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+65995, 600, 6983},
    /* 6835 : RSTORSSP mem64 [m:        f3 0f 01 /5                       ] CET,FUTURE */
        /* 333 355 001(01) 205 : CET,FUTURE */
        {I_RSTORSSP, 1, {MEMORY|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70119, 599, 6984},
    /* 6836 : SAVEPREVSSP void [          f3 0f 01 ea                       ] CET,FUTURE */
        /* 333 355 002(01 ea) : CET,FUTURE */
        {I_SAVEPREVSSP, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70125, 599, 6985},
    /* 6837 : SETSSBSY void [          f3 0f 01 e8                       ] CET,FUTURE */
        /* 333 355 002(01 e8) : CET,FUTURE */
        {I_SETSSBSY, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70131, 599, 6986},
    /* 6838 : WRUSSD mem32,reg32 [mr:       o32 66 0f38 f5 /r                 ] CET,FUTURE */
        /* 321 361 356 001(f5) 101 : NOAPX,CET,FUTURE */
        {I_WRUSSD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66002, 601, 6987},
    /* 6839 : WRUSSD mem32,reg32 [mr:       evex.nf0.nd0.l0.66.m4.w0 65 /r    ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 7d 08) 300 001(65) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRUSSD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+45661, 602, 6988},
    /* 6840 : WRUSSQ mem64,reg64 [mr:       o64 66 0f38 f5 /r                 ] LONG,PROT,CET,X86_64 */
        /* 324 361 356 001(f5) 101 : NOAPX,LONG,PROT,CET,X86_64 */
        {I_WRUSSQ, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66009, 603, 6989},
    /* 6841 : WRUSSQ mem64,reg64 [mr:       evex.nf0.nd0.l0.66.m4.w1 65 /r    ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 fd 08) 300 001(65) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRUSSQ, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+45670, 602, 6990},
    /* 6842 : WRSSD mem32,reg32 [mr:       o32 0f38 f6 /r                    ] CET,FUTURE */
        /* 321 356 001(f6) 101 : NOAPX,CET,FUTURE */
        {I_WRSSD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70137, 601, 6991},
    /* 6843 : WRSSD mem32,reg32 [mr:       evex.nf0.nd0.l0.m4.w0 66 /r       ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 7c 08) 300 001(66) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRSSD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+45679, 602, 6992},
    /* 6844 : WRSSQ mem64,reg64 [mr:       o64 0f38 f6 /r                    ] LONG,PROT,CET,X86_64 */
        /* 324 356 001(f6) 101 : NOAPX,LONG,PROT,CET,X86_64 */
        {I_WRSSQ, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+70143, 603, 6993},
    /* 6845 : WRSSQ mem64,reg64 [mr:       evex.nf0.nd0.l0.m4.w1 66 /r       ] LONG,PROT,CET,APX,X86_64 */
        /* 250(f4 fc 08) 300 001(66) 101 : LONG,EVEX,PROT,CET,APX,X86_64 */
        {I_WRSSQ, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+45688, 602, 6994},
    /* 6846 : ENQCMD reg16,mem512 [rm:       a16 f2 0f38 f8 /r                 ] AR0-1,SZ,NOREX,NOAPX,NOLONG,FL,ENQCMD,FUTURE */
        /* 310 332 356 001(f8) 110 : AR0,AR1,SZ,NOREX,NOAPX,NOLONG,FL,ENQCMD,FUTURE */
        {I_ENQCMD, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66016, 604, 6997},
    /* 6847 : ENQCMD reg32,mem512 [rm:       a32 f2 0f38 f8 /r                 ] AR0-1,SZ,FL,ENQCMD,FUTURE */
        /* 311 332 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,FL,ENQCMD,FUTURE */
        {I_ENQCMD, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66023, 605, 6999},
    /* 6848 : ENQCMD reg64,mem512 [rm:       a64 f2 0f38 f8 /r                 ] AR0-1,SZ,LONG,FL,PROT,ENQCMD,X86_64 */
        /* 313 332 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,LONG,FL,PROT,ENQCMD,X86_64 */
        {I_ENQCMD, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66030, 606, 7000},
    /* 6849 : ENQCMD reg64,mem512 [rm:       evex.nf0.nd0.l0.f2.m4.w1 f8 /r    ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,APX,X86_64 */
        /* 250(f4 ff 08) 300 001(f8) 110 : AR0,AR1,SZ,LONG,FL,EVEX,PRIV,PROT,ENQCMD,APX,X86_64 */
        {I_ENQCMD, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+45697, 607, 7001},
    /* 6850 : ENQCMDS reg16,mem512 [rm:       a16 f3 0f38 f8 /r                 ] AR0-1,SZ,NOREX,NOAPX,NOLONG,FL,PRIV,ENQCMD,FUTURE */
        /* 310 333 356 001(f8) 110 : AR0,AR1,SZ,NOREX,NOAPX,NOLONG,FL,PRIV,ENQCMD,FUTURE */
        {I_ENQCMDS, 2, {REG_GPR|BITS16|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66037, 608, 7002},
    /* 6851 : ENQCMDS reg32,mem512 [rm:       a32 f3 0f38 f8 /r                 ] AR0-1,SZ,FL,PRIV,ENQCMD,FUTURE */
        /* 311 333 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,FL,PRIV,ENQCMD,FUTURE */
        {I_ENQCMDS, 2, {REG_GPR|BITS32|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66044, 609, 7004},
    /* 6852 : ENQCMDS reg64,mem512 [rm:       a64 f3 0f38 f8 /r                 ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,X86_64 */
        /* 313 333 356 001(f8) 110 : AR0,AR1,SZ,NOAPX,LONG,FL,PRIV,PROT,ENQCMD,X86_64 */
        {I_ENQCMDS, 2, {REG_GPR|BITS64|RN_L16,MEMORY|BITS512|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66051, 610, 7005},
    /* 6853 : ENQCMDS reg64,mem512 [rm:       evex.nf0.nd0.l0.f3.m4.w1 f8 /r    ] AR0-1,SZ,LONG,FL,PRIV,PROT,ENQCMD,APX,X86_64 */
        /* 250(f4 fe 08) 300 001(f8) 110 : AR0,AR1,SZ,LONG,FL,EVEX,PRIV,PROT,ENQCMD,APX,X86_64 */
        {I_ENQCMDS, 2, {REG_GPR|BITS64,MEMORY|BITS512,0,0,0}, NO_DECORATOR, nasm_bytecodes+45706, 607, 7006},
    /* 6854 : PCONFIG void [          np 0f 01 c5                       ] FL,PRIV,PCONFIG,FUTURE */
        /* 360 355 002(01 c5) : FL,PRIV,PCONFIG,FUTURE */
        {I_PCONFIG, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70071, 611, 7007},
    /* 6855 : XRESLDTRK void [          f2 0f 01 e9                       ] TSXLDTRK,FUTURE */
        /* 332 355 002(01 e9) : TSXLDTRK,FUTURE */
        {I_XRESLDTRK, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70149, 612, 7008},
    /* 6856 : XSUSLDTRK void [          f2 0f 01 e8                       ] TSXLDTRK,FUTURE */
        /* 332 355 002(01 e8) : TSXLDTRK,FUTURE */
        {I_XSUSLDTRK, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70155, 612, 7009},
    /* 6857 : VCVTNE2PS2BF16 xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7f 08) 301 001(72) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45715, 613, 7012},
    /* 6858 : VCVTNE2PS2BF16 ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7f 28) 301 001(72) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45733, 613, 7013},
    /* 6859 : VCVTNE2PS2BF16 zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 72 /r         ] AVX512BF16,FUTURE */
        /* 241(f2 7f 48) 301 001(72) 120 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VCVTNE2PS2BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45751, 614, 7014},
    /* 6860 : VCVTNEPS2BF16 xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f3.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 250(f2 7e 08) 301 001(72) 110 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45769, 613, 7015},
    /* 6861 : VCVTNEPS2BF16 xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f3.0f38.w0 72 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 250(f2 7e 28) 301 001(72) 110 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45778, 613, 7016},
    /* 6862 : VCVTNEPS2BF16 ymmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f3.0f38.w0 72 /r         ] AVX512BF16,FUTURE */
        /* 250(f2 7e 48) 301 001(72) 110 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VCVTNEPS2BF16, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+45787, 614, 7017},
    /* 6863 : VDPBF16PS xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 52 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7e 08) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45796, 613, 7018},
    /* 6864 : VDPBF16PS ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 52 /r         ] AVX512VL,AVX512BF16,FUTURE */
        /* 241(f2 7e 28) 301 001(52) 120 : EVEX,PROT,AVX512VL,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45814, 613, 7019},
    /* 6865 : VDPBF16PS zmmreg|mask|z,zmmreg*,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 52 /r         ] AVX512BF16,FUTURE */
        /* 241(f2 7e 48) 301 001(52) 120 : EVEX,PROT,AVX512BF16,FUTURE */
        {I_VDPBF16PS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+45832, 614, 7020},
    /* 6866 : VP2INTERSECTD kreg|rs2,xmmreg,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.0f38.w0 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 08) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,XMMREG,RM_XMM|BITS128,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45850, 615, 7023},
    /* 6867 : VP2INTERSECTD kreg|rs2,ymmreg,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.0f38.w0 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 28) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,YMMREG,RM_YMM|BITS256,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45859, 615, 7024},
    /* 6868 : VP2INTERSECTD kreg|rs2,zmmreg,zmmrm512|b32 [rvm:fv:   evex.nds.512.f2.0f38.w0 68 /r     ] AVX512F,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 7f 48) 301 001(68) 120 : EVEX,PROT,AVX512F,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTD, 3, {OPMASKREG|RS2,ZMMREG,RM_ZMM|BITS512,0,0}, {0,0,B32,0,0}, nasm_bytecodes+45868, 616, 7025},
    /* 6869 : VP2INTERSECTQ kreg|rs2,xmmreg,xmmrm128|b64 [rvm:fv:   evex.nds.128.f2.0f38.w1 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 08) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,XMMREG,RM_XMM|BITS128,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45877, 615, 7026},
    /* 6870 : VP2INTERSECTQ kreg|rs2,ymmreg,ymmrm256|b64 [rvm:fv:   evex.nds.256.f2.0f38.w1 68 /r     ] AVX512VL,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 28) 301 001(68) 120 : EVEX,PROT,AVX512VL,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,YMMREG,RM_YMM|BITS256,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45886, 615, 7027},
    /* 6871 : VP2INTERSECTQ kreg|rs2,zmmreg,zmmrm512|b64 [rvm:fv:   evex.nds.512.f2.0f38.w1 68 /r     ] AVX512F,AVX512VP2INTERSECT,FUTURE */
        /* 241(f2 ff 48) 301 001(68) 120 : EVEX,PROT,AVX512F,AVX512VP2INTERSECT,FUTURE */
        {I_VP2INTERSECTQ, 3, {OPMASKREG|RS2,ZMMREG,RM_ZMM|BITS512,0,0}, {0,0,B64,0,0}, nasm_bytecodes+45895, 616, 7028},
    /* 6872 : LDTILECFG mem512 [m:        vex.128.np.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 000) 001(49) 200 : AR0,SZ,NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_LDTILECFG, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66058, 617, 7032},
    /* 6873 : LDTILECFG mem512 [m:        evex.128.np.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64,APX */
        /* 250(f2 7c 08) 300 001(49) 200 : AR0,SZ,LONG,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_LDTILECFG, 1, {MEMORY|BITS512,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+45904, 618, 7032},
    /* 6874 : STTILECFG mem512 [m:        vex.128.66.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 001) 001(49) 200 : AR0,SZ,NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_STTILECFG, 1, {MEMORY|BITS512|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66065, 617, 7033},
    /* 6875 : STTILECFG mem512 [m:        evex.128.66.0f38.w0 49 /0         ] AR0,SZ,LONG,PROT,AMXTILE,X86_64,APX */
        /* 250(f2 7d 08) 300 001(49) 200 : AR0,SZ,LONG,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_STTILECFG, 1, {MEMORY|BITS512,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+45913, 618, 7033},
    /* 6876 : TDPBF16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 5c /r          ] LONG,PROT,AMXBF16,X86_64 */
        /* 262(002 002) 001(5c) 110 : NOAPX,LONG,VEX,PROT,AMXBF16,X86_64 */
        {I_TDPBF16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66072, 619, 7034},
    /* 6877 : TDPFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 5c /r          ] LONG,PROT,AMXFP16,X86_64 */
        /* 262(002 003) 001(5c) 110 : NOAPX,LONG,VEX,PROT,AMXFP16,X86_64 */
        {I_TDPFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66079, 620, 7035},
    /* 6878 : TCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 6c /r          ] LONG,PROT,AMXCOMPLEX,X86_64 */
        /* 262(002 001) 001(6c) 110 : NOAPX,LONG,VEX,PROT,AMXCOMPLEX,X86_64 */
        {I_TCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66086, 621, 7036},
    /* 6879 : TCMMRLFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 6c /r          ] LONG,PROT,AMXCOMPLEX,X86_64 */
        /* 262(002 000) 001(6c) 110 : NOAPX,LONG,VEX,PROT,AMXCOMPLEX,X86_64 */
        {I_TCMMRLFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66093, 621, 7037},
    /* 6880 : TDPBSSD tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 003) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBSSD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66100, 622, 7038},
    /* 6881 : TDPBSUD tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 002) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBSUD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66107, 622, 7039},
    /* 6882 : TDPBUSD tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 001) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBUSD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66114, 622, 7040},
    /* 6883 : TDPBUUD tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 5e /r          ] LONG,PROT,AMXINT8,X86_64 */
        /* 262(002 000) 001(5e) 110 : NOAPX,LONG,VEX,PROT,AMXINT8,X86_64 */
        {I_TDPBUUD, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66121, 622, 7041},
    /* 6884 : TILELOADD tmmreg,mem [rm:       vex.128.f2.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 003) 001(4b) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADD, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66128, 623, 7042},
    /* 6885 : TILELOADD tmmreg,mem [rm:       evex.128.f2.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7f 08) 300 001(4b) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADD, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45922, 624, 7042},
    /* 6886 : TILELOADDT1 tmmreg,mem [rm:       vex.128.66.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 001) 001(4b) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADDT1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66135, 623, 7043},
    /* 6887 : TILELOADDT1 tmmreg,mem [rm:       evex.128.66.0f38.w0 4b /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7d 08) 300 001(4b) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILELOADDT1, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45931, 624, 7043},
    /* 6888 : TILERELEASE void [          vex.128.np.0f38.w0 49 c0          ] LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 000) 002(49 c0) : NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_TILERELEASE, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66142, 625, 7044},
    /* 6889 : TILESTORED mem,tmmreg [mr:       vex.128.f3.0f38.w0 4b /r         ] AR0,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64 */
        /* 270(002 002) 001(4b) 101 : AR0,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILESTORED, 2, {MEMORY|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66149, 626, 7045},
    /* 6890 : TILESTORED mem,tmmreg [mr:       evex.128.f3.0f38.w0 4b /r         ] AR0,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,APX,X86_64,APX */
        /* 250(f2 7e 08) 300 001(4b) 101 : AR0,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,APX,X86_64 */
        {I_TILESTORED, 2, {MEMORY,TMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+45940, 627, 7045},
    /* 6891 : TILEZERO tmmreg [r:        vex.128.f2.0f38.w0 49 /3r0        ] LONG,PROT,AMXTILE,X86_64 */
        /* 270(002 003) 001(49) 171(300) : NOAPX,LONG,VEX,PROT,AMXTILE,X86_64 */
        {I_TILEZERO, 1, {TMMREG|RN_L16,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+54843, 625, 7046},
    /* 6892 : TILELOADDRS tmmreg,mem [rm:       vex.128.f2.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE */
        /* 270(002 003) 001(4a) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66156, 628, 7048},
    /* 6893 : TILELOADDRS tmmreg,mem [rm:       evex.128.f2.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE,APX */
        /* 250(f2 7f 08) 300 001(4a) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRS, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45949, 629, 7048},
    /* 6894 : TILELOADDRST1 tmmreg,mem [rm:       vex.128.66.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE */
        /* 270(002 001) 001(4a) 110 : AR1,ANYSIZE,NOAPX,SIB,LONG,MIB,VEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66163, 628, 7049},
    /* 6895 : TILELOADDRST1 tmmreg,mem [rm:       evex.128.66.0f38.w0 4a /r         ] AR1,ANYSIZE,SIB,LONG,MIB,PROT,AMXTILE,AMXMOVRS,APX,X86_64,FUTURE,APX */
        /* 250(f2 7d 08) 300 001(4a) 110 : AR1,ANYSIZE,SIB,LONG,MIB,EVEX,PROT,AMXTILE,AMXMOVRS,APX,FUTURE */
        {I_TILELOADDRST1, 2, {TMMREG,MEMORY,0,0,0}, NO_DECORATOR, nasm_bytecodes+45958, 629, 7049},
    /* 6896 : T2RPNTLVWZ0 tmmreg,mem [rm:       vex.128.np.0f38.w0 6e /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 000) 001(6e) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66170, 630, 7051},
    /* 6897 : T2RPNTLVWZ0T1 tmmreg,mem [rm:       vex.128.np.0f38.w0 6f /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 000) 001(6f) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0T1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66177, 630, 7052},
    /* 6898 : T2RPNTLVWZ1 tmmreg,mem [rm:       vex.128.66.0f38.w0 6e /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6e) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66184, 630, 7053},
    /* 6899 : T2RPNTLVWZ1T1 tmmreg,mem [rm:       vex.128.66.0f38.w0 6f /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6f) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1T1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66191, 630, 7054},
    /* 6900 : T2RPNTLVWZ0RS tmmreg,mem [rm:       vex.128.np.map5.w0 f8 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 000) 001(f8) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0RS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66198, 630, 7055},
    /* 6901 : T2RPNTLVWZ0RST1 tmmreg,mem [rm:       vex.128.np.map5.w0 f9 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 000) 001(f9) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ0RST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66205, 630, 7056},
    /* 6902 : T2RPNTLVWZ1RS tmmreg,mem [rm:       vex.128.66.map5.w0 f8 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 001) 001(f8) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1RS, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66212, 630, 7057},
    /* 6903 : T2RPNTLVWZ1RST1 tmmreg,mem [rm:       vex.128.66.map5.w0 f9 /r          ] SIB,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(005 001) 001(f9) 110 : NOAPX,SIB,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_T2RPNTLVWZ1RST1, 2, {TMMREG|RN_L16,MEMORY|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66219, 630, 7058},
    /* 6904 : TCONJTCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 000) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TCONJTCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66226, 631, 7059},
    /* 6905 : TCONJTFP16 tmmreg,tmmreg [rm:       vex.128.66.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 001) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TCONJTFP16, 2, {TMMREG|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66233, 631, 7060},
    /* 6906 : TCVTROWD2PS zmmreg,tmmreg,reg32 [rmv:      evex.512.f3.0f38.w0 4a /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7e 48) 300 001(4a) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWD2PS, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45967, 632, 7061},
    /* 6907 : TCVTROWD2PS zmmreg,tmmreg,imm8 [rmi:      evex.512.f3.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7e 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWD2PS, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22643, 632, 7062},
    /* 6908 : TCVTROWPS2BF16H zmmreg,tmmreg,reg32 [rmv:      evex.512.f2.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7f 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16H, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45976, 632, 7063},
    /* 6909 : TCVTROWPS2BF16H zmmreg,tmmreg,imm8 [rmi:      evex.512.f2.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7f 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16H, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22653, 632, 7064},
    /* 6910 : TCVTROWPS2BF16L zmmreg,tmmreg,reg32 [rmv:      evex.512.f3.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7e 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16L, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45985, 632, 7065},
    /* 6911 : TCVTROWPS2BF16L zmmreg,tmmreg,imm8 [rmi:      evex.512.f3.0f3a.w0 77 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7e 48) 300 001(77) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2BF16L, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22663, 632, 7066},
    /* 6912 : TCVTROWPS2PHH zmmreg,tmmreg,reg32 [rmv:      evex.512.np.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7c 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHH, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+45994, 632, 7067},
    /* 6913 : TCVTROWPS2PHH zmmreg,tmmreg,imm8 [rmi:      evex.512.np.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7c 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHH, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22673, 632, 7068},
    /* 6914 : TCVTROWPS2PHL zmmreg,tmmreg,reg32 [rmv:      evex.512.66.0f38.w0 6d /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7d 48) 300 001(6d) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHL, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+46003, 632, 7069},
    /* 6915 : TCVTROWPS2PHL zmmreg,tmmreg,imm8 [rmi:      evex.512.f2.0f3a.w0 77 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7f 48) 300 001(77) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TCVTROWPS2PHL, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22683, 632, 7070},
    /* 6916 : TDPBF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 000) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPBF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66240, 633, 7071},
    /* 6917 : TDPBHF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 003) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPBHF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66247, 633, 7072},
    /* 6918 : TDPHBF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 002) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPHBF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66254, 633, 7073},
    /* 6919 : TDPHF8PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.map5.w0 fd /r          ] AMXFP8,FUTURE */
        /* 262(005 001) 001(fd) 110 : NOAPX,VEX,AMXFP8,FUTURE */
        {I_TDPHF8PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66261, 633, 7074},
    /* 6920 : TILEMOVROW zmmreg,tmmreg,imm8 [rmi:      evex.512.66.0f3a.w0 07 /r ib      ] AMXAVX512,FUTURE */
        /* 250(f3 7d 48) 300 001(07) 110 022 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TILEMOVROW, 3, {ZMMREG,TMMREG,IMM_NORMAL|BITS8,0,0}, NO_DECORATOR, nasm_bytecodes+22693, 632, 7075},
    /* 6921 : TILEMOVROW zmmreg,tmmreg,reg32 [rmv:      evex.512.66.0f38.w0 4a /r         ] AMXAVX512,FUTURE */
        /* 242(f2 7d 48) 300 001(4a) 110 : EVEX,PROT,AMXAVX512,FUTURE */
        {I_TILEMOVROW, 3, {ZMMREG,TMMREG,REG_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+46012, 632, 7076},
    /* 6922 : TMMULTF32PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.66.0f38.w0 48 /r          ] AMXTF32,FUTURE */
        /* 262(002 001) 001(48) 110 : NOAPX,VEX,AMXTF32,FUTURE */
        {I_TMMULTF32PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66268, 634, 7077},
    /* 6923 : TTCMMIMFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 003) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTCMMIMFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66275, 631, 7078},
    /* 6924 : TTCMMRLFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 6b /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 002) 001(6b) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTCMMRLFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66282, 631, 7079},
    /* 6925 : TTDPBF16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f3.0f38.w0 6c /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 002) 001(6c) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTDPBF16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66289, 631, 7080},
    /* 6926 : TTDPFP16PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.f2.0f38.w0 6c /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 003) 001(6c) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTDPFP16PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66296, 631, 7081},
    /* 6927 : TTMMULTF32PS tmmreg,tmmreg,tmmreg [rmv:      vex.128.np.0f38.w0 48 /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 262(002 000) 001(48) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTMMULTF32PS, 3, {TMMREG|RN_L16,TMMREG|RN_L16,TMMREG|RN_L16,0,0}, NO_DECORATOR, nasm_bytecodes+66303, 631, 7082},
    /* 6928 : TTRANSPOSED tmmreg,tmmreg [rm:       vex.128.f3.0f38.w0 5f /r          ] AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        /* 270(002 002) 001(5f) 110 : NOAPX,VEX,AMXTRANSPOSE,OBSOLETE,NEVER,FUTURE */
        {I_TTRANSPOSED, 2, {TMMREG|RN_L16,TMMREG|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66310, 631, 7083},
    /* 6929 : VADDPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 58 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(58) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46021, 565, 7086},
    /* 6930 : VADDPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 58 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(58) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46039, 565, 7087},
    /* 6931 : VADDPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5.w0 58 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(58) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VADDPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+46057, 566, 7088},
    /* 6932 : VADDSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s16:evex.nds.lig.f3.map5.w0 58 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 305 001(58) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VADDSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46075, 567, 7089},
    /* 6933 : VCMPPH kreg|mask,xmmreg*,xmmrm128|b16,imm8 [rvmi:fv:  evex.nds.128.np.0f3a.w0 C2 /r ib  ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 301 001(c2) 120 023 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+22703, 565, 7090},
    /* 6934 : VCMPPH kreg|mask,ymmreg*,ymmrm256|b16,imm8 [rvmi:fv:  evex.nds.256.np.0f3a.w0 C2 /r ib  ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f3 7c 28) 301 001(c2) 120 023 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+22723, 565, 7091},
    /* 6935 : VCMPPH kreg|mask,zmmreg*,zmmrm512|b16|sae,imm8 [rvmi:fv:  evex.nds.512.np.0f3a.w0 C2 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 48) 301 001(c2) 120 023 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCMPPH, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B16|SAE,0,0}, nasm_bytecodes+22743, 566, 7092},
    /* 6936 : VCMPSH kreg|mask,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s16:evex.nds.lig.f3.0f3a.w0 C2 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7e 08) 305 001(c2) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCMPSH, 4, {OPMASKREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK,0,SAE,0,0}, nasm_bytecodes+22763, 567, 7093},
    /* 6937 : VCOMISH xmmreg,xmmrm16|sae [rm:fv:    evex.lig.np.map5.w0 2F /r         ] FL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(2f) 110 : LIG,FL,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCOMISH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46093, 635, 7094},
    /* 6938 : VCVTDQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46102, 565, 7095},
    /* 6939 : VCVTDQ2PH xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46111, 565, 7096},
    /* 6940 : VCVTDQ2PH ymmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.np.map5.w0 5B /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTDQ2PH, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+46120, 566, 7097},
    /* 6941 : VCVTPD2PH xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 5a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fd 08) 301 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46129, 565, 7098},
    /* 6942 : VCVTPD2PH xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 5a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fd 28) 301 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46138, 565, 7099},
    /* 6943 : VCVTPD2PH xmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.66.map5.w1 5a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fd 48) 301 001(5a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPD2PH, 2, {XMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+46147, 566, 7100},
    /* 6944 : VCVTPH2DQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.66.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46156, 565, 7101},
    /* 6945 : VCVTPH2DQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.66.map5.w0 5B /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46165, 565, 7102},
    /* 6946 : VCVTPH2DQ zmmreg|mask|z,ymmrm256|b16|er [rm:hv:    evex.512.66.map5.w0 5B /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 302 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2DQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46174, 566, 7103},
    /* 6947 : VCVTPH2PD xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.np.map5.w0 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 315 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46183, 565, 7104},
    /* 6948 : VCVTPH2PD ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.np.map5.w0 5A /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 315 001(5a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46192, 565, 7105},
    /* 6949 : VCVTPH2PD zmmreg|mask|z,xmmrm128|b16|sae [rm:qvm:   evex.512.np.map5.w0 5A /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 315 001(5a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2PD, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46201, 566, 7106},
    /* 6950 : VCVTPH2PS xmmreg|mask|z,xmmrm64 [rm:hvm:   evex.128.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 08) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31360, 543, 7107},
    /* 6951 : VCVTPH2PS ymmreg|mask|z,xmmrm128 [rm:hvm:   evex.256.66.0f38.w0 13 /r         ] AVX512,AVX512VL,FUTURE */
        /* 250(f2 7d 28) 314 001(13) 110 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPH2PS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+31369, 543, 7108},
    /* 6952 : VCVTPH2PS zmmreg|mask|z,ymmrm256|sae [rm:hvm:   evex.512.66.0f38.w0 13 /r         ] AVX512,FUTURE */
        /* 250(f2 7d 48) 314 001(13) 110 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPH2PS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+31378, 544, 7109},
    /* 6953 : VCVTPH2PSX xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.66.map6.w0 13 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 302 001(13) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46210, 565, 7110},
    /* 6954 : VCVTPH2PSX ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.66.map6.w0 13 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 302 001(13) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46219, 565, 7111},
    /* 6955 : VCVTPH2PSX zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.66.map6.w0 13 /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 302 001(13) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2PSX, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46228, 566, 7112},
    /* 6956 : VCVTPH2QQ xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.66.map5.w0 7b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 315 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46237, 565, 7113},
    /* 6957 : VCVTPH2QQ ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.66.map5.w0 7b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 315 001(7b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46246, 565, 7114},
    /* 6958 : VCVTPH2QQ zmmreg|mask|z,xmmrm128|b16|er [rm:qvm:   evex.512.66.map5.w0 7b /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 315 001(7b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2QQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46255, 566, 7115},
    /* 6959 : VCVTPH2UDQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.map5.w0 79 /r            ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46264, 565, 7116},
    /* 6960 : VCVTPH2UDQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.map5.w0 79 /r            ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 302 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46273, 565, 7117},
    /* 6961 : VCVTPH2UDQ zmmreg|mask|z,ymmrm256|b16|er [rm:hv:    evex.512.map5.w0 79 /r            ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 302 001(79) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46282, 566, 7118},
    /* 6962 : VCVTPH2UQQ xmmreg|mask|z,xmmrm32|b16 [rm:qvm:   evex.128.66.map5.w0 79 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 315 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46291, 565, 7119},
    /* 6963 : VCVTPH2UQQ ymmreg|mask|z,xmmrm64|b16 [rm:qvm:   evex.256.66.map5.w0 79 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 315 001(79) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46300, 565, 7120},
    /* 6964 : VCVTPH2UQQ zmmreg|mask|z,xmmrm128|b16|er [rm:qvm:   evex.512.66.map5.w0 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 315 001(79) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UQQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46309, 566, 7121},
    /* 6965 : VCVTPH2UW xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46318, 565, 7122},
    /* 6966 : VCVTPH2UW ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46327, 565, 7123},
    /* 6967 : VCVTPH2UW zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2UW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46336, 566, 7124},
    /* 6968 : VCVTPH2W xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46345, 565, 7125},
    /* 6969 : VCVTPH2W ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46354, 565, 7126},
    /* 6970 : VCVTPH2W zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.66.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPH2W, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46363, 566, 7127},
    /* 6971 : VCVTPS2PH xmmreg|mask|z,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18333, 543, 7128},
    /* 6972 : VCVTPS2PH mem64|mask,xmmreg,imm8 [mri:hvm:  evex.128.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 08) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS64,XMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18333, 543, 7129},
    /* 6973 : VCVTPS2PH xmmreg|mask|z,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {XMMREG,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+18343, 543, 7130},
    /* 6974 : VCVTPS2PH mem128|mask,ymmreg,imm8 [mri:hvm:  evex.256.66.0f3a.w0 1d /r ib      ] AVX512,AVX512VL,FUTURE */
        /* 250(f3 7d 28) 314 001(1d) 101 022 : EVEX,PROT,AVX512,AVX512VL,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS128,YMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+18343, 543, 7131},
    /* 6975 : VCVTPS2PH ymmreg|mask|z,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {YMMREG,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK|Z,SAE,0,0,0}, nasm_bytecodes+18353, 544, 7132},
    /* 6976 : VCVTPS2PH mem256|mask,zmmreg|sae,imm8 [mri:hvm:  evex.512.66.0f3a.w0 1d /r ib      ] AVX512,FUTURE */
        /* 250(f3 7d 48) 314 001(1d) 101 022 : EVEX,PROT,AVX512,FUTURE */
        {I_VCVTPS2PH, 3, {MEMORY|BITS256,ZMMREG,IMM_NORMAL|BITS8,0,0}, {MASK,SAE,0,0,0}, nasm_bytecodes+18353, 544, 7133},
    /* 6977 : VCVTPS2PHX xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 1d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(1d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46372, 565, 7134},
    /* 6978 : VCVTPS2PHX xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 1d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(1d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46381, 565, 7135},
    /* 6979 : VCVTPS2PHX ymmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 1d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(1d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTPS2PHX, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+46390, 566, 7136},
    /* 6980 : VCVTQQ2PH xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 08) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46399, 565, 7137},
    /* 6981 : VCVTQQ2PH xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 28) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+46408, 565, 7138},
    /* 6982 : VCVTQQ2PH xmmreg|mask|z,zmmrm512|b64|er [rm:fv:    evex.512.np.map5.w1 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 fc 48) 301 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTQQ2PH, 2, {XMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|ER,0,0,0}, nasm_bytecodes+46417, 565, 7139},
    /* 6983 : VCVTSD2SH xmmreg|mask|z,xmmreg*,xmmrm64|er [rvm:t1s:  evex.nds.lig.f2.map5.w1 5a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 ff 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSD2SH, 3, {XMMREG,XMMREG,RM_XMM|BITS64,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46426, 567, 7140},
    /* 6984 : VCVTSH2SD xmmreg,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.f3.map5.w0 5a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SD, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {0,0,SAE,0,0}, nasm_bytecodes+46444, 567, 7141},
    /* 6985 : VCVTSH2SI reg32,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w0 2d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(2d) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46462, 567, 7142},
    /* 6986 : VCVTSH2SI reg64,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w1 2d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(2d) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46471, 567, 7143},
    /* 6987 : VCVTSH2SS xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.map6.w0 13 /r        ] AVX512FP16,FUTURE */
        /* 241(f6 7c 08) 306 001(13) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2SS, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+46480, 567, 7144},
    /* 6988 : VCVTSH2USI reg32,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w0 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(79) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46498, 567, 7145},
    /* 6989 : VCVTSH2USI reg64,xmmrm16|er [rm:t1s:   evex.lig.f3.map5.w1 79 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(79) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSH2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,ER,0,0,0}, nasm_bytecodes+46507, 567, 7146},
    /* 6990 : VCVTSI2SH xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 2a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46516, 567, 7147},
    /* 6991 : VCVTSI2SH xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.map5.w1 2a /r     ] AVX512FP16,FUTURE */
        /* 241(f5 fe 08) 306 001(2a) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46534, 567, 7148},
    /* 6992 : VCVTSS2SH xmmreg,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.np.map5 1d /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 306 001(1d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTSS2SH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46552, 567, 7149},
    /* 6993 : VCVTTPH2DQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.f3.map5.w0 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46570, 565, 7150},
    /* 6994 : VCVTTPH2DQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.f3.map5.w0 5b /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 28) 302 001(5b) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46579, 565, 7151},
    /* 6995 : VCVTTPH2DQ zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.f3.map5.w0 5b /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 48) 302 001(5b) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2DQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46588, 566, 7152},
    /* 6996 : VCVTTPH2QQ xmmreg|mask|z,xmmrm32|b16 [rm:t1s:   evex.128.66.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46597, 565, 7153},
    /* 6997 : VCVTTPH2QQ ymmreg|mask|z,xmmrm64|b16 [rm:t1s:   evex.256.66.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 306 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46606, 565, 7154},
    /* 6998 : VCVTTPH2QQ zmmreg|mask|z,xmmrm128|b16|sae [rm:t1s:   evex.512.66.map5.w0 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 306 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2QQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46615, 566, 7155},
    /* 6999 : VCVTTPH2UDQ xmmreg|mask|z,xmmrm64|b16 [rm:hv:    evex.128.np.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46624, 565, 7156},
    /* 7000 : VCVTTPH2UDQ ymmreg|mask|z,xmmrm128|b16 [rm:hv:    evex.256.np.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 302 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46633, 565, 7157},
    /* 7001 : VCVTTPH2UDQ zmmreg|mask|z,ymmrm256|b16|sae [rm:hv:    evex.512.np.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 302 001(78) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UDQ, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46642, 566, 7158},
    /* 7002 : VCVTTPH2UQQ xmmreg|mask|z,xmmrm32|b16 [rm:t1s:   evex.128.66.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46651, 565, 7159},
    /* 7003 : VCVTTPH2UQQ ymmreg|mask|z,xmmrm64|b16 [rm:t1s:   evex.256.66.map5.w0 78 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 306 001(78) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {YMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46660, 565, 7160},
    /* 7004 : VCVTTPH2UQQ zmmreg|mask|z,xmmrm128|b16|sae [rm:t1s:   evex.512.66.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 306 001(78) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UQQ, 2, {ZMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46669, 566, 7161},
    /* 7005 : VCVTTPH2UW xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46678, 565, 7162},
    /* 7006 : VCVTTPH2UW ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46687, 565, 7163},
    /* 7007 : VCVTTPH2UW zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 7c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(7c) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2UW, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46696, 566, 7164},
    /* 7008 : VCVTTPH2W xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46705, 565, 7165},
    /* 7009 : VCVTTPH2W ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 7c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7d 28) 301 001(7c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46714, 565, 7166},
    /* 7010 : VCVTTPH2W zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.66.map5.w0 7c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7d 48) 301 001(7c) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTPH2W, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+46723, 566, 7167},
    /* 7011 : VCVTTSH2SI reg32,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(2c) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2SI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46732, 567, 7168},
    /* 7012 : VCVTTSH2SI reg64,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w1 2c /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(2c) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2SI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46741, 567, 7169},
    /* 7013 : VCVTTSH2USI reg32,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 306 001(78) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2USI, 2, {REG_GPR|BITS32,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46750, 567, 7170},
    /* 7014 : VCVTTSH2USI reg64,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w1 78 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 fe 08) 306 001(78) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTTSH2USI, 2, {REG_GPR|BITS64,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+46759, 567, 7171},
    /* 7015 : VCVTUDQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46768, 565, 7172},
    /* 7016 : VCVTUDQ2PH xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.map5.w0 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46777, 565, 7173},
    /* 7017 : VCVTUDQ2PH ymmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f2.map5.w0 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7f 48) 301 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUDQ2PH, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46786, 566, 7174},
    /* 7018 : VCVTUQQ2PH xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.f2.map5.w1 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 ff 08) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46795, 565, 7175},
    /* 7019 : VCVTUQQ2PH xmmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.f2.map5.w1 7a /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 ff 28) 301 001(7a) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46804, 565, 7176},
    /* 7020 : VCVTUQQ2PH xmmreg|mask|z,zmmrm512|b32 [rm:fv:    evex.512.f2.map5.w1 7a /r         ] AVX512FP16,FUTURE */
        /* 250(f5 ff 48) 301 001(7a) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUQQ2PH, 2, {XMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+46813, 566, 7177},
    /* 7021 : VCVTUSI2SH xmmreg,xmmreg*,rm32|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 7b /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS32,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46822, 567, 7178},
    /* 7022 : VCVTUSI2SH xmmreg,xmmreg*,rm64|er [rvm:t1s:  evex.nds.lig.f3.map5.w1 7b /r     ] AVX512FP16,FUTURE */
        /* 241(f5 fe 08) 306 001(7b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUSI2SH, 3, {XMMREG,XMMREG,RM_GPR|BITS64,0,0}, {0,0,ER,0,0}, nasm_bytecodes+46840, 567, 7179},
    /* 7023 : VCVTUW2PH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46858, 565, 7180},
    /* 7024 : VCVTUW2PH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7f 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46867, 565, 7181},
    /* 7025 : VCVTUW2PH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.f2.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7f 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTUW2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46876, 566, 7182},
    /* 7026 : VCVTW2PH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46885, 565, 7183},
    /* 7027 : VCVTW2PH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 7d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7e 28) 301 001(7d) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+46894, 565, 7184},
    /* 7028 : VCVTW2PH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.f3.map5.w0 7d /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 48) 301 001(7d) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VCVTW2PH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+46903, 566, 7185},
    /* 7029 : VDIVPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5 5e /r        ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46912, 565, 7186},
    /* 7030 : VDIVPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5 5e /r        ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+46930, 565, 7187},
    /* 7031 : VDIVPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5 5e /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5e) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VDIVPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+46948, 566, 7188},
    /* 7032 : VDIVSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s16:evex.nds.lig.f3.map5 5e /r        ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 305 001(5e) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VDIVSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+46966, 567, 7189},
    /* 7033 : VFCMADDCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+46984, 565, 7190},
    /* 7034 : VFCMADDCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47002, 565, 7191},
    /* 7035 : VFCMADDCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f2.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 48) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMADDCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47020, 565, 7192},
    /* 7036 : VFMADDCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47038, 565, 7193},
    /* 7037 : VFMADDCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 28) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47056, 565, 7194},
    /* 7038 : VFMADDCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f3.map6.w0 56 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 48) 301 001(56) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47074, 565, 7195},
    /* 7039 : VFCMADDCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f2.map6.w0 57 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 306 001(57) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMADDCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47092, 567, 7196},
    /* 7040 : VFMADDCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.map6.w0 57 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 306 001(57) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47110, 567, 7197},
    /* 7041 : VFCMULCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f2.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47128, 565, 7198},
    /* 7042 : VFCMULCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f2.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7f 28) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47146, 565, 7199},
    /* 7043 : VFCMULCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f2.map6.w0 d6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 48) 301 001(d6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMULCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47164, 566, 7200},
    /* 7044 : VFMULCPH xmmreg|mask|z,xmmreg*,xmmrm128|b32 [rvm:fv:   evex.nds.128.f3.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47182, 565, 7201},
    /* 7045 : VFMULCPH ymmreg|mask|z,ymmreg*,ymmrm256|b32 [rvm:fv:   evex.nds.256.f3.map6.w0 d6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7e 28) 301 001(d6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+47200, 565, 7202},
    /* 7046 : VFMULCPH zmmreg|mask|z,zmmreg*,zmmrm512|b32|er [rvm:fv:   evex.nds.512.f3.map6.w0 d6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 48) 301 001(d6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMULCPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32|ER,0,0}, nasm_bytecodes+47218, 566, 7203},
    /* 7047 : VFCMULCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f2.map6.w0 d7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7f 08) 306 001(d7) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFCMULCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47236, 567, 7204},
    /* 7048 : VFMULCSH xmmreg|mask|z,xmmreg*,xmmrm32|er [rvm:t1s:  evex.nds.lig.f3.map6.w0 d7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7e 08) 306 001(d7) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMULCSH, 3, {XMMREG,XMMREG,RM_XMM|BITS32,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47254, 567, 7205},
    /* 7049 : VFMADDSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 96 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(96) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47272, 565, 7206},
    /* 7050 : VFMADDSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 96 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(96) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47290, 565, 7207},
    /* 7051 : VFMADDSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 96 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(96) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47308, 566, 7208},
    /* 7052 : VFMADDSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47326, 565, 7209},
    /* 7053 : VFMADDSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47344, 565, 7210},
    /* 7054 : VFMADDSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47362, 566, 7211},
    /* 7055 : VFMADDSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47380, 565, 7212},
    /* 7056 : VFMADDSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b6 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b6) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47398, 565, 7213},
    /* 7057 : VFMADDSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b6 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b6) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADDSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47416, 566, 7214},
    /* 7058 : VFMSUBADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 97 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(97) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47434, 565, 7215},
    /* 7059 : VFMSUBADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 97 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(97) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47452, 565, 7216},
    /* 7060 : VFMSUBADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 97 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(97) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47470, 566, 7217},
    /* 7061 : VFMSUBADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47488, 565, 7218},
    /* 7062 : VFMSUBADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47506, 565, 7219},
    /* 7063 : VFMSUBADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a7) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47524, 566, 7220},
    /* 7064 : VFMSUBADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47542, 565, 7221},
    /* 7065 : VFMSUBADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b7 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b7) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47560, 565, 7222},
    /* 7066 : VFMSUBADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b7 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b7) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUBADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47578, 566, 7223},
    /* 7067 : VFMADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 98 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(98) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47596, 565, 7224},
    /* 7068 : VFMADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 98 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(98) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47614, 565, 7225},
    /* 7069 : VFMADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 98 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(98) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47632, 566, 7226},
    /* 7070 : VFMADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 a8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(a8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47650, 565, 7227},
    /* 7071 : VFMADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 a8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(a8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47668, 565, 7228},
    /* 7072 : VFMADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 a8 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(a8) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47686, 566, 7229},
    /* 7073 : VFMADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 b8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(b8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47704, 565, 7230},
    /* 7074 : VFMADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 b8 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(b8) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47722, 565, 7231},
    /* 7075 : VFMADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 b8 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(b8) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47740, 566, 7232},
    /* 7076 : VFNMADD132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47758, 565, 7233},
    /* 7077 : VFNMADD132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47776, 565, 7234},
    /* 7078 : VFNMADD132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9c /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47794, 566, 7235},
    /* 7079 : VFNMADD213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ac /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ac) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47812, 565, 7236},
    /* 7080 : VFNMADD213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ac /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ac) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47830, 565, 7237},
    /* 7081 : VFNMADD213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ac /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ac) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47848, 566, 7238},
    /* 7082 : VFNMADD231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 bc /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(bc) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47866, 565, 7239},
    /* 7083 : VFNMADD231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 bc /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(bc) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+47884, 565, 7240},
    /* 7084 : VFNMADD231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 bc /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(bc) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+47902, 566, 7241},
    /* 7085 : VFMADD132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 99 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(99) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47920, 567, 7242},
    /* 7086 : VFMADD213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 a9 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(a9) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47938, 567, 7243},
    /* 7087 : VFMADD231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 b9 /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(b9) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMADD231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47956, 567, 7244},
    /* 7088 : VFNMADD132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47974, 567, 7245},
    /* 7089 : VFNMADD213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 ad /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(ad) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+47992, 567, 7246},
    /* 7090 : VFNMADD231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bd /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bd) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMADD231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48010, 567, 7247},
    /* 7091 : VFMSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9a /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9a) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48028, 565, 7248},
    /* 7092 : VFMSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9a /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9a) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48046, 565, 7249},
    /* 7093 : VFMSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9a /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9a) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48064, 566, 7250},
    /* 7094 : VFMSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 aa /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(aa) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48082, 565, 7251},
    /* 7095 : VFMSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 aa /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(aa) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48100, 565, 7252},
    /* 7096 : VFMSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 aa /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(aa) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48118, 566, 7253},
    /* 7097 : VFMSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ba /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ba) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48136, 565, 7254},
    /* 7098 : VFMSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ba /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ba) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48154, 565, 7255},
    /* 7099 : VFMSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ba /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ba) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48172, 566, 7256},
    /* 7100 : VFNMSUB132PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 9e /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(9e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48190, 565, 7257},
    /* 7101 : VFNMSUB132PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 9e /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(9e) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48208, 565, 7258},
    /* 7102 : VFNMSUB132PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 9e /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(9e) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB132PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48226, 566, 7259},
    /* 7103 : VFNMSUB213PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 ae /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(ae) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48244, 565, 7260},
    /* 7104 : VFNMSUB213PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 ae /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(ae) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48262, 565, 7261},
    /* 7105 : VFNMSUB213PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 ae /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(ae) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB213PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48280, 566, 7262},
    /* 7106 : VFNMSUB231PH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 be /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(be) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48298, 565, 7263},
    /* 7107 : VFNMSUB231PH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 be /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(be) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48316, 565, 7264},
    /* 7108 : VFNMSUB231PH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 be /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(be) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB231PH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48334, 566, 7265},
    /* 7109 : VFMSUB132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9b /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9b) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48352, 567, 7266},
    /* 7110 : VFMSUB213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 ab /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(ab) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48370, 567, 7267},
    /* 7111 : VFMSUB231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bb /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bb) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFMSUB231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48388, 567, 7268},
    /* 7112 : VFNMSUB132SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 9f /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(9f) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB132SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48406, 567, 7269},
    /* 7113 : VFNMSUB213SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 af /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(af) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB213SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48424, 567, 7270},
    /* 7114 : VFNMSUB231SH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 bf /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(bf) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFNMSUB231SH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48442, 567, 7271},
    /* 7115 : VFPCLASSPH kreg|mask,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 66 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22783, 565, 7272},
    /* 7116 : VFPCLASSPH kreg|mask,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 66 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(66) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22793, 565, 7273},
    /* 7117 : VFPCLASSPH kreg|mask,zmmrm512|b16,imm8 [rmi:fv:   evex.512.np.0f3a.w0 66 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(66) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFPCLASSPH, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+22803, 566, 7274},
    /* 7118 : VFPCLASSSH kreg|mask,xmmrm16,imm8 [rmi:t1s:  evex.lig.np.0f3a.w0 67 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 306 001(67) 110 022 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VFPCLASSSH, 3, {OPMASKREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+22813, 567, 7275},
    /* 7119 : VGETEXPPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map6.w0 42 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 301 001(42) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48460, 565, 7276},
    /* 7120 : VGETEXPPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map6.w0 42 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 301 001(42) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48469, 565, 7277},
    /* 7121 : VGETEXPPH zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.66.map6.w0 42 /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 301 001(42) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETEXPPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+48478, 566, 7278},
    /* 7122 : VGETEXPSH xmmreg|mask|z,xmmreg,xmmrm16|sae [rvm:t1s:  evex.lig.66.map6.w0 43 /r         ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(43) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETEXPSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48487, 567, 7279},
    /* 7123 : VGETMANTPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 26 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(26) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22823, 565, 7280},
    /* 7124 : VGETMANTPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 26 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(26) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22833, 565, 7281},
    /* 7125 : VGETMANTPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 26 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(26) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETMANTPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+22843, 566, 7282},
    /* 7126 : VGETMANTSH xmmreg|mask|z,xmmreg,xmmrm16|sae,imm8 [rvmi:t1s: evex.lig.np.0f3a.w0 27 /r ib      ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(27) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VGETMANTSH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+22853, 567, 7283},
    /* 7127 : VMAXPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 5f /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5f) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48496, 565, 7284},
    /* 7128 : VMAXPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 5f /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5f) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48514, 565, 7285},
    /* 7129 : VMAXPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 5f /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5f) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+48532, 566, 7286},
    /* 7130 : VMAXSH xmmreg|mask|z,xmmreg,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 5f /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5f) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48550, 567, 7287},
    /* 7131 : VMINPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48559, 565, 7288},
    /* 7132 : VMINPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48577, 565, 7289},
    /* 7133 : VMINPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5d) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+48595, 566, 7290},
    /* 7134 : VMINSH xmmreg|mask|z,xmmreg,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48613, 567, 7291},
    /* 7135 : VMOVSH xmmreg|mask|z,mem16 [rm:t1s16: evex.lig.f3.map5.w0 10 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 305 001(10) 110 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 2, {XMMREG,MEMORY|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48622, 567, 7292},
    /* 7136 : VMOVSH mem16|mask,xmmreg [mr:t1s16: evex.lig.f3.map5.w0 11 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7e 08) 305 001(11) 101 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 2, {MEMORY|BITS16,XMMREG,0,0,0}, {MASK,0,0,0,0}, nasm_bytecodes+48631, 567, 7293},
    /* 7137 : VMOVSH xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.map5.w0 10 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 300 001(10) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48640, 567, 7294},
    /* 7138 : VMOVSH xmmreg|mask|z,xmmreg*,xmmreg [rvm:      evex.nds.lig.f3.map5.w0 11 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 300 001(11) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVSH, 3, {XMMREG,XMMREG,XMMREG,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48658, 567, 7295},
    /* 7139 : VMOVW xmmreg|mask|z,rm16 [rm:t1s:   evex.128.66.map5.wig 6e /r        ] AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(6e) 110 : WIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVW, 2, {XMMREG,RM_GPR|BITS16,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+48676, 636, 7296},
    /* 7140 : VMOVW rm16,xmmreg [mr:t1s:   evex.128.66.map5.wig 7e /r        ] AVX512FP16,FUTURE */
        /* 250(f5 7d 08) 306 001(7e) 101 : WIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMOVW, 2, {RM_GPR|BITS16,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+48685, 636, 7297},
    /* 7141 : VMULPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 59 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(59) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48694, 565, 7298},
    /* 7142 : VMULPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 59 /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(59) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48712, 565, 7299},
    /* 7143 : VMULPH zmmreg|mask|z,zmmreg*,zmmrm512|b16 [rvm:fv:   evex.nds.512.np.map5.w0 59 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(59) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMULPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48730, 566, 7300},
    /* 7144 : VMULSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s16:evex.nds.lig.f3.map5.w0 59 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 305 001(59) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMULSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48748, 567, 7301},
    /* 7145 : VRCPPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map6.w0 4c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 301 001(4c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRCPPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48766, 565, 7302},
    /* 7146 : VRCPPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map6.w0 4c /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 301 001(4c) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRCPPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48775, 565, 7303},
    /* 7147 : VRCPPH zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.66.map6.w0 4c /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 301 001(4c) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRCPPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48784, 566, 7304},
    /* 7148 : VRCPSH xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.66.map6.w0 4d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(4d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRCPSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48793, 567, 7305},
    /* 7149 : VREDUCEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 56 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22863, 565, 7306},
    /* 7150 : VREDUCEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 56 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(56) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+22873, 565, 7307},
    /* 7151 : VREDUCEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 56 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(56) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VREDUCEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+22883, 566, 7308},
    /* 7152 : VREDUCESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 57 /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(57) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VREDUCESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+22893, 567, 7309},
    /* 7153 : VENDSCALEPH xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 08) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21563, 565, 7310},
    /* 7154 : VENDSCALEPH ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.np.0f3a.w0 08 /r ib      ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f3 7c 28) 301 001(08) 110 022 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+21573, 565, 7311},
    /* 7155 : VENDSCALEPH zmmreg|mask|z,zmmrm512|b16|sae,imm8 [rmi:fv:   evex.512.np.0f3a.w0 08 /r ib      ] AVX512FP16,FUTURE */
        /* 250(f3 7c 48) 301 001(08) 110 022 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VENDSCALEPH, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+21583, 566, 7312},
    /* 7156 : VENDSCALESH xmmreg|mask|z,xmmreg*,xmmrm16|sae,imm8 [rvmi:t1s: evex.nds.lig.np.0f3a.w0 0a /r ib  ] AVX512FP16,FUTURE */
        /* 241(f3 7c 08) 306 001(0a) 120 023 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VENDSCALESH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+21643, 567, 7313},
    /* 7157 : VRSQRTPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map6.w0 4e /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 08) 301 001(4e) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48811, 565, 7314},
    /* 7158 : VRSQRTPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map6.w0 4e /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f6 7d 28) 301 001(4e) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48820, 565, 7315},
    /* 7159 : VRSQRTPH zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.66.map6.w0 4e /r         ] AVX512FP16,FUTURE */
        /* 250(f6 7d 48) 301 001(4e) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRSQRTPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+48829, 566, 7316},
    /* 7160 : VRSQRTSH xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.nds.lig.66.map6.w0 4f /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(4f) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VRSQRTSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48838, 567, 7317},
    /* 7161 : VSCALEFPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.66.map6.w0 2c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 301 001(2c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48856, 565, 7318},
    /* 7162 : VSCALEFPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.66.map6.w0 2c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f6 7d 28) 301 001(2c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48874, 565, 7319},
    /* 7163 : VSCALEFPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.66.map6.w0 2c /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 48) 301 001(2c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSCALEFPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+48892, 566, 7320},
    /* 7164 : VSCALEFSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.66.map6.w0 2d /r     ] AVX512FP16,FUTURE */
        /* 241(f6 7d 08) 306 001(2d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSCALEFSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48910, 567, 7321},
    /* 7165 : VSQRTPH xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 51 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 301 001(51) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48928, 565, 7322},
    /* 7166 : VSQRTPH ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 51 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 250(f5 7c 28) 301 001(51) 110 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+48937, 565, 7323},
    /* 7167 : VSQRTPH zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 51 /r         ] AVX512FP16,FUTURE */
        /* 250(f5 7c 48) 301 001(51) 110 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSQRTPH, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+48946, 566, 7324},
    /* 7168 : VSQRTSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s:  evex.nds.lig.f3.map5.w0 51 /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(51) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSQRTSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+48955, 567, 7325},
    /* 7169 : VSUBPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.nds.128.np.map5.w0 5c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48973, 565, 7326},
    /* 7170 : VSUBPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.nds.256.np.map5.w0 5c /r     ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5c) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48991, 565, 7327},
    /* 7171 : VSUBPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|er [rvm:fv:   evex.nds.512.np.map5.w0 5c /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5c) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSUBPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|ER,0,0}, nasm_bytecodes+49009, 566, 7328},
    /* 7172 : VSUBSH xmmreg|mask|z,xmmreg*,xmmrm16|er [rvm:t1s16:evex.nds.lig.f3.map5.w0 5c /r     ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 305 001(5c) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VSUBSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,ER,0,0}, nasm_bytecodes+49027, 567, 7329},
    /* 7173 : VUCOMISH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.np.map5.w0 2e /r         ] FL,AVX512FP16,FUTURE */
        /* 250(f5 7c 08) 306 001(2e) 110 : LIG,FL,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VUCOMISH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49045, 635, 7330},
    /* 7174 : VMINPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48559, 565, 7331},
    /* 7175 : VMINPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 5d /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(5d) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+48577, 565, 7332},
    /* 7176 : VMINPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(5d) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+48595, 566, 7333},
    /* 7177 : VMINSH xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 5d /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(5d) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMINSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+48613, 567, 7334},
    /* 7178 : VMAXPH xmmreg|mask|z,xmmreg*,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 57 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 08) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49063, 565, 7335},
    /* 7179 : VMAXPH ymmreg|mask|z,ymmreg*,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 57 /r         ] AVX512VL,AVX512FP16,FUTURE */
        /* 241(f5 7c 28) 301 001(57) 120 : EVEX,PROT,AVX512VL,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49081, 565, 7336},
    /* 7180 : VMAXPH zmmreg|mask|z,zmmreg*,zmmrm512|b16|sae [rvm:fv:   evex.512.np.map5.w0 57 /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7c 48) 301 001(57) 120 : EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXPH, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+49099, 566, 7337},
    /* 7181 : VMAXSH xmmreg|mask|z,xmmreg*,xmmrm16|sae [rvm:t1s:  evex.lig.f3.map5.w0 57 /r         ] AVX512FP16,FUTURE */
        /* 241(f5 7e 08) 306 001(57) 120 : LIG,EVEX,PROT,AVX512FP16,FUTURE */
        {I_VMAXSH, 3, {XMMREG,XMMREG,RM_XMM|BITS16,0,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+49117, 567, 7338},
    /* 7182 : AADD mem32,reg32 [mr:       o32 np 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 360 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AADD, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66317, 637, 7341},
    /* 7183 : AADD mem64,reg64 [mr:       o64 np 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 360 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AADD, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66324, 637, 7342},
    /* 7184 : AADD mem32,reg32 [mr:       evex.nf0.nd0.l0.np.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7c 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AADD, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22913, 638, 7343},
    /* 7185 : AADD mem64,reg64 [mr:       evex.nf0.nd0.l0.np.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fc 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AADD, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22923, 638, 7344},
    /* 7186 : AAND mem32,reg32 [mr:       o32 66 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 361 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AAND, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66331, 637, 7345},
    /* 7187 : AAND mem64,reg64 [mr:       o64 66 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 361 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AAND, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66338, 637, 7346},
    /* 7188 : AAND mem32,reg32 [mr:       evex.nf0.nd0.l0.66.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7d 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AAND, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22933, 638, 7347},
    /* 7189 : AAND mem64,reg64 [mr:       evex.nf0.nd0.l0.66.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fd 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AAND, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22943, 638, 7348},
    /* 7190 : AOR mem32,reg32 [mr:       o32 f2 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 332 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AOR, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66345, 637, 7349},
    /* 7191 : AOR mem64,reg64 [mr:       o64 f2 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 332 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AOR, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66352, 637, 7350},
    /* 7192 : AOR mem32,reg32 [mr:       evex.nf0.nd0.l0.f2.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7f 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AOR, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22953, 638, 7351},
    /* 7193 : AOR mem64,reg64 [mr:       evex.nf0.nd0.l0.f2.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 ff 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AOR, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22963, 638, 7352},
    /* 7194 : AXOR mem32,reg32 [mr:       o32 f3 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 321 333 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS32|RN_L16,REG_GPR|BITS32|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66359, 637, 7353},
    /* 7195 : AXOR mem64,reg64 [mr:       o64 f3 0f38 fc /r                 ] AR0-1,SQ,LONG,PROT,RAOINT,X86_64 */
        /* 324 333 356 001(fc) 101 : AR0,AR1,SQ,NOAPX,LONG,PROT,RAOINT,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS64|RN_L16,REG_GPR|BITS64|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66366, 637, 7354},
    /* 7196 : AXOR mem32,reg32 [mr:       evex.nf0.nd0.l0.f3.m4.o32 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 7e 08) 300 321 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS32,REG_GPR|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+22973, 638, 7355},
    /* 7197 : AXOR mem64,reg64 [mr:       evex.nf0.nd0.l0.f3.m4.o64 fc /r   ] AR0-1,SQ,LONG,PROT,RAOINT,APX,X86_64 */
        /* 250(f4 fe 08) 300 324 001(fc) 101 : AR0,AR1,SQ,WW,LONG,EVEX,PROT,RAOINT,APX,X86_64 */
        {I_AXOR, 2, {MEMORY|BITS64,REG_GPR|BITS64,0,0,0}, NO_DECORATOR, nasm_bytecodes+22983, 638, 7356},
    /* 7198 : CLUI void [          f3 0f 01 ee                       ] LONG,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ee) : LONG,PROT,UINTR,X86_64 */
        {I_CLUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70161, 639, 7359},
    /* 7199 : SENDUIPI reg64 [m:        o64nw f3 0f c7 /6                 ] LONG,PROT,UINTR,X86_64 */
        /* 323 333 355 001(c7) 206 : NWSIZE,LONG,PROT,UINTR,X86_64 */
        {I_SENDUIPI, 1, {REG_GPR|BITS64,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+66373, 640, 7360},
    /* 7200 : STUI void [          f3 0f 01 ef                       ] LONG,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ef) : LONG,PROT,UINTR,X86_64 */
        {I_STUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70167, 639, 7361},
    /* 7201 : TESTUI void [          f3 0f 01 ed                       ] LONG,FL,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ed) : LONG,FL,PROT,UINTR,X86_64 */
        {I_TESTUI, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70173, 641, 7362},
    /* 7202 : UIRET void [          f3 0f 01 ec                       ] LONG,FL,PROT,UINTR,X86_64 */
        /* 333 355 002(01 ec) : LONG,FL,PROT,UINTR,X86_64 */
        {I_UIRET, 0, {0,0,0,0,0}, NO_DECORATOR, nasm_bytecodes+70179, 641, 7363},
    /* 7203 : HRESET imm,reg_eax [i-:       f3 0f3a f0 c0 ib                  ] AR0-1,SB,PRIV,HRESET,FUTURE */
        /* 333 357 002(f0 c0) 020 : AR0,AR1,SB,NOAPX,PRIV,HRESET,FUTURE */
        {I_HRESET, 2, {IMM_NORMAL,REG_EAX|RN_L16,0,0,0}, NO_DECORATOR, nasm_bytecodes+66380, 642, 7368},
    /* 7204 : VADDBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49135, 644, 7372},
    /* 7205 : VADDBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49144, 644, 7373},
    /* 7206 : VADDBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 58 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(58) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VADDBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49153, 644, 7374},
    /* 7207 : VCMPBF16 kreg|mask,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 08) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+22993, 644, 7375},
    /* 7208 : VCMPBF16 kreg|mask,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 28) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+23003, 644, 7376},
    /* 7209 : VCMPBF16 kreg|mask,zmmreg,zmmrm512|b16,imm8 [rvmi:fv:  evex.512.f2.0f3a.w0 c2 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 48) 301 001(c2) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCMPBF16, 4, {OPMASKREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK,0,B16,0,0}, nasm_bytecodes+23013, 644, 7377},
    /* 7210 : VCOMISBF16 xmmreg,xmmrm16 [rm:t1s:   evex.lig.66.map5.w0 2f /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMISBF16, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+49162, 645, 7378},
    /* 7211 : VDIVBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49171, 644, 7379},
    /* 7212 : VDIVBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49180, 644, 7380},
    /* 7213 : VDIVBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5e /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDIVBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49189, 644, 7381},
    /* 7214 : VFMADD132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49198, 644, 7382},
    /* 7215 : VFMADD132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49207, 644, 7383},
    /* 7216 : VFMADD132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 98 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(98) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49216, 644, 7384},
    /* 7217 : VFMADD213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49225, 644, 7385},
    /* 7218 : VFMADD213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49234, 644, 7386},
    /* 7219 : VFMADD213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 a8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(a8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49243, 644, 7387},
    /* 7220 : VFMADD231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49252, 644, 7388},
    /* 7221 : VFMADD231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49261, 644, 7389},
    /* 7222 : VFMADD231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 b8 /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(b8) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMADD231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49270, 644, 7390},
    /* 7223 : VFMSUB132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49279, 644, 7391},
    /* 7224 : VFMSUB132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49288, 644, 7392},
    /* 7225 : VFMSUB132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9a /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9a) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49297, 644, 7393},
    /* 7226 : VFMSUB213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49306, 644, 7394},
    /* 7227 : VFMSUB213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49315, 644, 7395},
    /* 7228 : VFMSUB213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 aa /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(aa) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49324, 644, 7396},
    /* 7229 : VFMSUB231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49333, 644, 7397},
    /* 7230 : VFMSUB231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49342, 644, 7398},
    /* 7231 : VFMSUB231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ba /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ba) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFMSUB231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49351, 644, 7399},
    /* 7232 : VFNMADD132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49360, 644, 7400},
    /* 7233 : VFNMADD132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49369, 644, 7401},
    /* 7234 : VFNMADD132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49378, 644, 7402},
    /* 7235 : VFNMADD213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49387, 644, 7403},
    /* 7236 : VFNMADD213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49396, 644, 7404},
    /* 7237 : VFNMADD213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ac /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ac) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49405, 644, 7405},
    /* 7238 : VFNMADD231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49414, 644, 7406},
    /* 7239 : VFNMADD231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49423, 644, 7407},
    /* 7240 : VFNMADD231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 bc /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(bc) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMADD231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49432, 644, 7408},
    /* 7241 : VFNMSUB132BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49441, 644, 7409},
    /* 7242 : VFNMSUB132BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49450, 644, 7410},
    /* 7243 : VFNMSUB132BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 9e /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(9e) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB132BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49459, 644, 7411},
    /* 7244 : VFNMSUB213BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49468, 644, 7412},
    /* 7245 : VFNMSUB213BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49477, 644, 7413},
    /* 7246 : VFNMSUB213BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 ae /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(ae) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB213BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49486, 644, 7414},
    /* 7247 : VFNMSUB231BF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49495, 644, 7415},
    /* 7248 : VFNMSUB231BF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49504, 644, 7416},
    /* 7249 : VFNMSUB231BF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 be /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(be) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFNMSUB231BF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49513, 644, 7417},
    /* 7250 : VFPCLASSBF16 kreg|mask,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23023, 644, 7418},
    /* 7251 : VFPCLASSBF16 kreg|mask,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23033, 644, 7419},
    /* 7252 : VFPCLASSBF16 kreg|mask,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 66 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(66) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VFPCLASSBF16, 3, {OPMASKREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK,B16,0,0,0}, nasm_bytecodes+23043, 644, 7420},
    /* 7253 : VGETEXPBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49522, 644, 7421},
    /* 7254 : VGETEXPBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49531, 644, 7422},
    /* 7255 : VGETEXPBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 42 /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(42) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETEXPBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49540, 644, 7423},
    /* 7256 : VGETMANTBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23053, 644, 7424},
    /* 7257 : VGETMANTBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23063, 644, 7425},
    /* 7258 : VGETMANTBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 26 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(26) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VGETMANTBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23073, 644, 7426},
    /* 7259 : VMAXBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49549, 644, 7427},
    /* 7260 : VMAXBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49558, 644, 7428},
    /* 7261 : VMAXBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5f /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5f) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMAXBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49567, 644, 7429},
    /* 7262 : VMINBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49576, 644, 7430},
    /* 7263 : VMINBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49585, 644, 7431},
    /* 7264 : VMINBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5d /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5d) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49594, 644, 7432},
    /* 7265 : VMULBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49603, 644, 7433},
    /* 7266 : VMULBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49612, 644, 7434},
    /* 7267 : VMULBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 59 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(59) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMULBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49621, 644, 7435},
    /* 7268 : VRCPBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49630, 644, 7436},
    /* 7269 : VRCPBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49639, 644, 7437},
    /* 7270 : VRCPBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 4c /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(4c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRCPBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49648, 644, 7438},
    /* 7271 : VREDUCEBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23083, 644, 7439},
    /* 7272 : VREDUCEBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23093, 644, 7440},
    /* 7273 : VREDUCEBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 56 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(56) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VREDUCEBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23103, 644, 7441},
    /* 7274 : VRNDSCALEBF16 xmmreg|mask|z,xmmrm128|b16,imm8 [rmi:fv:   evex.128.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 08) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23113, 644, 7442},
    /* 7275 : VRNDSCALEBF16 ymmreg|mask|z,ymmrm256|b16,imm8 [rmi:fv:   evex.256.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 28) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23123, 644, 7443},
    /* 7276 : VRNDSCALEBF16 zmmreg|mask|z,zmmrm512|b16,imm8 [rmi:fv:   evex.512.f2.0f3a.w0 08 /r ib      ] AVX10_2,FUTURE */
        /* 250(f3 7f 48) 301 001(08) 110 022 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRNDSCALEBF16, 3, {ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+23133, 644, 7444},
    /* 7277 : VRSQRTBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 08) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49657, 644, 7445},
    /* 7278 : VRSQRTBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 28) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49666, 644, 7446},
    /* 7279 : VRSQRTBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.np.map6.w0 4e /r         ] AVX10_2,FUTURE */
        /* 250(f6 7c 48) 301 001(4e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VRSQRTBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49675, 644, 7447},
    /* 7280 : VSCALEFBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 08) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49684, 644, 7448},
    /* 7281 : VSCALEFBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 28) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49693, 644, 7449},
    /* 7282 : VSCALEFBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map6.w0 2c /r         ] AVX10_2,FUTURE */
        /* 241(f6 7c 48) 301 001(2c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSCALEFBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49702, 644, 7450},
    /* 7283 : VSQRTBF16 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49711, 644, 7451},
    /* 7284 : VSQRTBF16 ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49720, 644, 7452},
    /* 7285 : VSQRTBF16 zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.66.map5.w0 51 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(51) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSQRTBF16, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49729, 644, 7453},
    /* 7286 : VSUBBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 08) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49738, 644, 7454},
    /* 7287 : VSUBBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 28) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49747, 644, 7455},
    /* 7288 : VSUBBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.66.map5.w0 5c /r         ] AVX10_2,FUTURE */
        /* 241(f5 7d 48) 301 001(5c) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VSUBBF16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49756, 644, 7456},
    /* 7289 : VCOMXSD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.f2.0f.w1 2f /r           ] AVX10_2,FUTURE */
        /* 250(f1 ff 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49765, 645, 7459},
    /* 7290 : VCOMXSH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2f /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49774, 645, 7460},
    /* 7291 : VCOMXSS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.f3.0f.w0 2f /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(2f) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCOMXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49783, 645, 7461},
    /* 7292 : VUCOMXSD xmmreg,xmmrm64|sae [rm:t1s:   evex.lig.f2.0f.w1 2e /r           ] AVX10_2,FUTURE */
        /* 250(f1 ff 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSD, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49792, 645, 7462},
    /* 7293 : VUCOMXSH xmmreg,xmmrm16|sae [rm:t1s:   evex.lig.f3.map5.w0 2e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSH, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49801, 645, 7463},
    /* 7294 : VUCOMXSS xmmreg,xmmrm32|sae [rm:t1s:   evex.lig.f3.0f.w0 2e /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(2e) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VUCOMXSS, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+49810, 645, 7464},
    /* 7295 : VCVT2PH2BF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49819, 644, 7467},
    /* 7296 : VCVT2PH2BF8 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49828, 644, 7468},
    /* 7297 : VCVT2PH2BF8 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7f 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49837, 644, 7469},
    /* 7298 : VCVT2PH2BF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49846, 644, 7470},
    /* 7299 : VCVT2PH2BF8S ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49855, 644, 7471},
    /* 7300 : VCVT2PH2BF8S zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2BF8S, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49864, 644, 7472},
    /* 7301 : VCVT2PH2HF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49873, 644, 7473},
    /* 7302 : VCVT2PH2HF8 ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49882, 644, 7474},
    /* 7303 : VCVT2PH2HF8 zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49891, 644, 7475},
    /* 7304 : VCVT2PH2HF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 08) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49900, 644, 7476},
    /* 7305 : VCVT2PH2HF8S ymmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 28) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49909, 644, 7477},
    /* 7306 : VCVT2PH2HF8S zmmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.f2.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7f 48) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PH2HF8S, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+49918, 644, 7478},
    /* 7307 : VCVTPH2BF8 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 08) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49927, 644, 7479},
    /* 7308 : VCVTPH2BF8 xmmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 28) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49936, 644, 7480},
    /* 7309 : VCVTPH2BF8 ymmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f2 7e 48) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49945, 644, 7481},
    /* 7310 : VCVTPH2BF8S xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49954, 644, 7482},
    /* 7311 : VCVTPH2BF8S xmmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49963, 644, 7483},
    /* 7312 : VCVTPH2BF8S ymmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(74) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2BF8S, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49972, 644, 7484},
    /* 7313 : VCVTPH2HF8 xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49981, 644, 7485},
    /* 7314 : VCVTPH2HF8 xmmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49990, 644, 7486},
    /* 7315 : VCVTPH2HF8 ymmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(18) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+49999, 644, 7487},
    /* 7316 : VCVTPH2HF8S xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50008, 644, 7488},
    /* 7317 : VCVTPH2HF8S xmmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 28) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50017, 644, 7489},
    /* 7318 : VCVTPH2HF8S ymmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f3.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 48) 301 001(1b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2HF8S, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50026, 644, 7490},
    /* 7319 : VCVT2PS2PHX xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 08) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50035, 644, 7491},
    /* 7320 : VCVT2PS2PHX ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 28) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50044, 644, 7492},
    /* 7321 : VCVT2PS2PHX zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 67 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7d 48) 301 001(67) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVT2PS2PHX, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50053, 644, 7493},
    /* 7322 : VCVTBIASPH2BF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50062, 644, 7494},
    /* 7323 : VCVTBIASPH2BF8 xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50071, 644, 7495},
    /* 7324 : VCVTBIASPH2BF8 ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.0f38.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50080, 644, 7496},
    /* 7325 : VCVTBIASPH2BF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50089, 644, 7497},
    /* 7326 : VCVTBIASPH2BF8S xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50098, 644, 7498},
    /* 7327 : VCVTBIASPH2BF8S ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 74 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(74) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2BF8S, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50107, 644, 7499},
    /* 7328 : VCVTBIASPH2HF8 xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50116, 644, 7500},
    /* 7329 : VCVTBIASPH2HF8 xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50125, 644, 7501},
    /* 7330 : VCVTBIASPH2HF8 ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 18 /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(18) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50134, 644, 7502},
    /* 7331 : VCVTBIASPH2HF8S xmmreg|mask|z,xmmreg,xmmrm128|b16 [rvm:fv:   evex.128.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 08) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50143, 644, 7503},
    /* 7332 : VCVTBIASPH2HF8S xmmreg|mask|z,ymmreg,ymmrm256|b16 [rvm:fv:   evex.256.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 28) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {XMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50152, 644, 7504},
    /* 7333 : VCVTBIASPH2HF8S ymmreg|mask|z,zmmreg,zmmrm512|b16 [rvm:fv:   evex.512.np.map5.w0 1b /r         ] AVX10_2,FUTURE */
        /* 241(f5 7c 48) 301 001(1b) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBIASPH2HF8S, 3, {YMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+50161, 644, 7505},
    /* 7334 : VCVTHF82PH xmmreg|mask|z,xmmrm64 [rm:hv:    evex.128.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50170, 644, 7506},
    /* 7335 : VCVTHF82PH ymmreg|mask|z,xmmrm128 [rm:hv:    evex.256.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50179, 644, 7507},
    /* 7336 : VCVTHF82PH zmmreg|mask|z,ymmrm256 [rm:hv:    evex.512.f2.map5.w0 1e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 302 001(1e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTHF82PH, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+50188, 644, 7508},
    /* 7337 : VDPPHPS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 08) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50197, 644, 7511},
    /* 7338 : VDPPHPS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 28) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50206, 644, 7512},
    /* 7339 : VDPPHPS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 52 /r         ] AVX10_2,FUTURE */
        /* 241(f2 7c 48) 301 001(52) 120 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VDPPHPS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50215, 644, 7513},
    /* 7340 : VMPSADBW xmmreg|mask|z,xmmreg,xmmrm128,imm8 [rvmi:fvm: evex.128.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 08) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23143, 644, 7514},
    /* 7341 : VMPSADBW ymmreg|mask|z,ymmreg,ymmrm256,imm8 [rvmi:fvm: evex.256.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 28) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23153, 644, 7515},
    /* 7342 : VMPSADBW zmmreg|mask|z,zmmreg,zmmrm512,imm8 [rvmi:fvm: evex.512.f3.0f3a.w0 42 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7e 48) 303 001(42) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMPSADBW, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+23163, 644, 7516},
    /* 7343 : VPDPBSSD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50224, 646, 7517},
    /* 7344 : VPDPBSSD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50233, 646, 7518},
    /* 7345 : VPDPBSSD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50242, 646, 7519},
    /* 7346 : VPDPBSSDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50251, 646, 7520},
    /* 7347 : VPDPBSSDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50260, 646, 7521},
    /* 7348 : VPDPBSSDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f2.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7f 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50269, 646, 7522},
    /* 7349 : VPDPBSUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50278, 646, 7523},
    /* 7350 : VPDPBSUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50287, 646, 7524},
    /* 7351 : VPDPBSUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50296, 646, 7525},
    /* 7352 : VPDPBSUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50305, 646, 7526},
    /* 7353 : VPDPBSUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50314, 646, 7527},
    /* 7354 : VPDPBSUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBSUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50323, 646, 7528},
    /* 7355 : VPDPBUUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50332, 646, 7529},
    /* 7356 : VPDPBUUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50341, 646, 7530},
    /* 7357 : VPDPBUUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 50 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(50) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50350, 646, 7531},
    /* 7358 : VPDPBUUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50359, 646, 7532},
    /* 7359 : VPDPBUUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50368, 646, 7533},
    /* 7360 : VPDPBUUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 51 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(51) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPBUUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50377, 646, 7534},
    /* 7361 : VPDPWSUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50386, 646, 7535},
    /* 7362 : VPDPWSUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50395, 646, 7536},
    /* 7363 : VPDPWSUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50404, 646, 7537},
    /* 7364 : VPDPWSUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50413, 646, 7538},
    /* 7365 : VPDPWSUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50422, 646, 7539},
    /* 7366 : VPDPWSUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.f3.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7e 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWSUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50431, 646, 7540},
    /* 7367 : VPDPWUSD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50440, 646, 7541},
    /* 7368 : VPDPWUSD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50449, 646, 7542},
    /* 7369 : VPDPWUSD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50458, 646, 7543},
    /* 7370 : VPDPWUSDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50467, 646, 7544},
    /* 7371 : VPDPWUSDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50476, 646, 7545},
    /* 7372 : VPDPWUSDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.66.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7d 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUSDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50485, 646, 7546},
    /* 7373 : VPDPWUUD xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50494, 646, 7547},
    /* 7374 : VPDPWUUD ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50503, 646, 7548},
    /* 7375 : VPDPWUUD zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 d2 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(d2) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUD, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50512, 646, 7549},
    /* 7376 : VPDPWUUDS xmmreg|mask|z,xmmreg,xmmrm128|b32 [rvm:fv:   evex.128.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 08) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {XMMREG,XMMREG,RM_XMM|BITS128,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50521, 646, 7550},
    /* 7377 : VPDPWUUDS ymmreg|mask|z,ymmreg,ymmrm256|b32 [rvm:fv:   evex.256.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 28) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50530, 646, 7551},
    /* 7378 : VPDPWUUDS zmmreg|mask|z,zmmreg,zmmrm512|b32 [rvm:fv:   evex.512.np.0f38.w0 d3 /r         ] AVX10_2,AVX10_VNNIINT,FUTURE */
        /* 241(f2 7c 48) 301 001(d3) 120 : EVEX,PROT,AVX10_2,AVX10_VNNIINT,FUTURE */
        {I_VPDPWUUDS, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+50539, 646, 7552},
    /* 7379 : VMINMAXBF16 xmmreg|mask|z,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23173, 644, 7555},
    /* 7380 : VMINMAXBF16 ymmreg|mask|z,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23183, 644, 7556},
    /* 7381 : VMINMAXBF16 zmmreg|mask|z,zmmreg,zmmrm512|b16,imm8 [rvmi:fv:  evex.512.f2.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7f 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXBF16, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23193, 644, 7557},
    /* 7382 : VMINMAXPD xmmreg|mask|z,xmmreg,xmmrm128|b64,imm8 [rvmi:fv:  evex.128.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+23203, 644, 7558},
    /* 7383 : VMINMAXPD ymmreg|mask|z,ymmreg,ymmrm256|b64,imm8 [rvmi:fv:  evex.256.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64,0,0}, nasm_bytecodes+23213, 644, 7559},
    /* 7384 : VMINMAXPD zmmreg|mask|z,zmmreg,zmmrm512|b64|sae,imm8 [rvmi:fv:  evex.512.66.0f3a.w1 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPD, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B64|SAE,0,0}, nasm_bytecodes+23223, 644, 7560},
    /* 7385 : VMINMAXPH xmmreg|mask|z,xmmreg,xmmrm128|b16,imm8 [rvmi:fv:  evex.128.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23233, 644, 7561},
    /* 7386 : VMINMAXPH ymmreg|mask|z,ymmreg,ymmrm256|b16,imm8 [rvmi:fv:  evex.256.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16,0,0}, nasm_bytecodes+23243, 644, 7562},
    /* 7387 : VMINMAXPH zmmreg|mask|z,zmmreg,zmmrm512|b16|sae,imm8 [rvmi:fv:  evex.512.np.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPH, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B16|SAE,0,0}, nasm_bytecodes+23253, 644, 7563},
    /* 7388 : VMINMAXPS xmmreg|mask|z,xmmreg,xmmrm128|b32,imm8 [rvmi:fv:  evex.128.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 08) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {XMMREG,XMMREG,RM_XMM|BITS128,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+23263, 644, 7564},
    /* 7389 : VMINMAXPS ymmreg|mask|z,ymmreg,ymmrm256|b32,imm8 [rvmi:fv:  evex.256.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 28) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {YMMREG,YMMREG,RM_YMM|BITS256,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32,0,0}, nasm_bytecodes+23273, 644, 7565},
    /* 7390 : VMINMAXPS zmmreg|mask|z,zmmreg,zmmrm512|b32|sae,imm8 [rvmi:fv:  evex.512.66.0f3a.w0 52 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 48) 301 001(52) 120 023 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXPS, 4, {ZMMREG,ZMMREG,RM_ZMM|BITS512,IMM_NORMAL|BITS8,0}, {MASK|Z,0,B32|SAE,0,0}, nasm_bytecodes+23283, 644, 7566},
    /* 7391 : VMINMAXSD xmmreg|mask|z,xmmreg,xmmrm64|sae,imm8 [rvmi:t1s: evex.lig.66.0f3a.w1 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 fd 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSD, 4, {XMMREG,XMMREG,RM_XMM|BITS64,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23293, 645, 7567},
    /* 7392 : VMINMAXSH xmmreg|mask|z,xmmreg,xmmrm16|sae,imm8 [rvmi:t1s: evex.lig.np.0f3a.w0 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7c 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSH, 4, {XMMREG,XMMREG,RM_XMM|BITS16,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23303, 645, 7568},
    /* 7393 : VMINMAXSS xmmreg|mask|z,xmmreg,xmmrm32|sae,imm8 [rvmi:t1s: evex.lig.66.0f3a.w0 53 /r ib      ] AVX10_2,FUTURE */
        /* 241(f3 7d 08) 306 001(53) 120 023 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VMINMAXSS, 4, {XMMREG,XMMREG,RM_XMM|BITS32,IMM_NORMAL|BITS8,0}, {MASK|Z,0,SAE,0,0}, nasm_bytecodes+23313, 645, 7569},
    /* 7394 : VCVTBF162IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50548, 644, 7572},
    /* 7395 : VCVTBF162IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50557, 644, 7573},
    /* 7396 : VCVTBF162IBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50566, 644, 7574},
    /* 7397 : VCVTBF162IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50575, 644, 7575},
    /* 7398 : VCVTBF162IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50584, 644, 7576},
    /* 7399 : VCVTBF162IUBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTBF162IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50593, 644, 7577},
    /* 7400 : VCVTTBF162IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50602, 644, 7578},
    /* 7401 : VCVTTBF162IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50611, 644, 7579},
    /* 7402 : VCVTTBF162IBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50620, 644, 7580},
    /* 7403 : VCVTTBF162IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50629, 644, 7581},
    /* 7404 : VCVTTBF162IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50638, 644, 7582},
    /* 7405 : VCVTTBF162IUBS zmmreg|mask|z,zmmrm512|b16 [rm:fv:    evex.512.f2.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTBF162IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50647, 644, 7583},
    /* 7406 : VCVTTPD2DQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50656, 644, 7584},
    /* 7407 : VCVTTPD2DQS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50665, 644, 7585},
    /* 7408 : VCVTTPD2DQS ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.np.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2DQS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50674, 644, 7586},
    /* 7409 : VCVTTPD2QQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50683, 644, 7587},
    /* 7410 : VCVTTPD2QQS ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50692, 644, 7588},
    /* 7411 : VCVTTPD2QQS zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.map5.w1 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2QQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50701, 644, 7589},
    /* 7412 : VCVTTPD2UDQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50710, 644, 7590},
    /* 7413 : VCVTTPD2UDQS xmmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {XMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50719, 644, 7591},
    /* 7414 : VCVTTPD2UDQS ymmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.np.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fc 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UDQS, 2, {YMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50728, 644, 7592},
    /* 7415 : VCVTTPD2UQQS xmmreg|mask|z,xmmrm128|b64 [rm:fv:    evex.128.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50737, 644, 7593},
    /* 7416 : VCVTTPD2UQQS ymmreg|mask|z,ymmrm256|b64 [rm:fv:    evex.256.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B64,0,0,0}, nasm_bytecodes+50746, 644, 7594},
    /* 7417 : VCVTTPD2UQQS zmmreg|mask|z,zmmrm512|b64|sae [rm:fv:    evex.512.66.map5.w1 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 fd 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPD2UQQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B64|SAE,0,0,0}, nasm_bytecodes+50755, 644, 7595},
    /* 7418 : VCVTPH2IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50764, 644, 7596},
    /* 7419 : VCVTPH2IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50773, 644, 7597},
    /* 7420 : VCVTPH2IBS zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+50782, 644, 7598},
    /* 7421 : VCVTPH2IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50791, 644, 7599},
    /* 7422 : VCVTPH2IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50800, 644, 7600},
    /* 7423 : VCVTPH2IUBS zmmreg|mask|z,zmmrm512|b16|er [rm:fv:    evex.512.np.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPH2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|ER,0,0,0}, nasm_bytecodes+50809, 644, 7601},
    /* 7424 : VCVTTPH2IBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50818, 644, 7602},
    /* 7425 : VCVTTPH2IBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50827, 644, 7603},
    /* 7426 : VCVTTPH2IBS zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+50836, 644, 7604},
    /* 7427 : VCVTTPH2IUBS xmmreg|mask|z,xmmrm128|b16 [rm:fv:    evex.128.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50845, 644, 7605},
    /* 7428 : VCVTTPH2IUBS ymmreg|mask|z,ymmrm256|b16 [rm:fv:    evex.256.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B16,0,0,0}, nasm_bytecodes+50854, 644, 7606},
    /* 7429 : VCVTTPH2IUBS zmmreg|mask|z,zmmrm512|b16|sae [rm:fv:    evex.512.np.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPH2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B16|SAE,0,0,0}, nasm_bytecodes+50863, 644, 7607},
    /* 7430 : VCVTTPS2DQS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50872, 644, 7608},
    /* 7431 : VCVTTPS2DQS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50881, 644, 7609},
    /* 7432 : VCVTTPS2DQS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.np.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2DQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50890, 644, 7610},
    /* 7433 : VCVTPS2IBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50899, 644, 7611},
    /* 7434 : VCVTPS2IBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50908, 644, 7612},
    /* 7435 : VCVTPS2IBS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 69 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(69) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+50917, 644, 7613},
    /* 7436 : VCVTPS2IUBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50926, 644, 7614},
    /* 7437 : VCVTPS2IUBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50935, 644, 7615},
    /* 7438 : VCVTPS2IUBS zmmreg|mask|z,zmmrm512|b32|er [rm:fv:    evex.512.66.map5.w0 6b /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(6b) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTPS2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|ER,0,0,0}, nasm_bytecodes+50944, 644, 7616},
    /* 7439 : VCVTTPS2IBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50953, 644, 7617},
    /* 7440 : VCVTTPS2IBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50962, 644, 7618},
    /* 7441 : VCVTTPS2IBS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.map5.w0 68 /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(68) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50971, 644, 7619},
    /* 7442 : VCVTTPS2IUBS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50980, 644, 7620},
    /* 7443 : VCVTTPS2IUBS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+50989, 644, 7621},
    /* 7444 : VCVTTPS2IUBS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.66.map5.w0 6a /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 301 001(6a) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2IUBS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+50998, 644, 7622},
    /* 7445 : VCVTTPS2QQS xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51007, 644, 7623},
    /* 7446 : VCVTTPS2QQS ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51016, 644, 7624},
    /* 7447 : VCVTTPS2QQS zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 302 001(6d) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2QQS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+51025, 644, 7625},
    /* 7448 : VCVTTPS2UDQS xmmreg|mask|z,xmmrm128|b32 [rm:fv:    evex.128.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 08) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51034, 644, 7626},
    /* 7449 : VCVTTPS2UDQS ymmreg|mask|z,ymmrm256|b32 [rm:fv:    evex.256.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 28) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51043, 644, 7627},
    /* 7450 : VCVTTPS2UDQS zmmreg|mask|z,zmmrm512|b32|sae [rm:fv:    evex.512.np.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7c 48) 301 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UDQS, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+51052, 644, 7628},
    /* 7451 : VCVTTPS2UQQS xmmreg|mask|z,xmmrm64|b32 [rm:hv:    evex.128.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 08) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {XMMREG,RM_XMM|BITS64,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51061, 644, 7629},
    /* 7452 : VCVTTPS2UQQS ymmreg|mask|z,xmmrm128|b32 [rm:hv:    evex.256.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 28) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {YMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,B32,0,0,0}, nasm_bytecodes+51070, 644, 7630},
    /* 7453 : VCVTTPS2UQQS zmmreg|mask|z,ymmrm256|b32|sae [rm:hv:    evex.512.66.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7d 48) 302 001(6c) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTPS2UQQS, 2, {ZMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,B32|SAE,0,0,0}, nasm_bytecodes+51079, 644, 7631},
    /* 7454 : VCVTTSD2SIS reg32,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 306 001(6d) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSD2SIS, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51088, 645, 7632},
    /* 7455 : VCVTTSD2SIS reg64,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w1 6d /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 ff 08) 306 001(6d) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSD2SIS, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51097, 647, 7633},
    /* 7456 : VCVTTSD2USIS reg32,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7f 08) 306 001(6c) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSD2USIS, 2, {REG_GPR|BITS32,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51106, 645, 7634},
    /* 7457 : VCVTTSD2USIS reg64,xmmrm64|sae [rm:t1s:   evex.lig.f2.map5.w1 6c /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 ff 08) 306 001(6c) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSD2USIS, 2, {REG_GPR|BITS64,RM_XMM|BITS64,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51115, 647, 7635},
    /* 7458 : VCVTTSS2SIS reg32,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w0 6d /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6d) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSS2SIS, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51124, 645, 7636},
    /* 7459 : VCVTTSS2SIS reg64,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w1 6d /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 fe 08) 306 001(6d) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSS2SIS, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51133, 647, 7637},
    /* 7460 : VCVTTSS2USIS reg32,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w0 6c /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6c) 110 : LIG,EVEX,PROT,AVX10_2,FUTURE */
        {I_VCVTTSS2USIS, 2, {REG_GPR|BITS32,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51142, 645, 7638},
    /* 7461 : VCVTTSS2USIS reg64,xmmrm32|sae [rm:t1s:   evex.lig.f3.map5.w1 6c /r         ] LONG,PROT,AVX10_2,X86_64 */
        /* 250(f5 fe 08) 306 001(6c) 110 : LIG,LONG,EVEX,PROT,AVX10_2,X86_64 */
        {I_VCVTTSS2USIS, 2, {REG_GPR|BITS64,RM_XMM|BITS32,0,0,0}, {0,SAE,0,0,0}, nasm_bytecodes+51151, 647, 7639},
    /* 7462 : VMOVD xmmreg,xmmrm32 [rm:t1s:   evex.128.f3.0f.w0 7e /r           ] AVX10_2,FUTURE */
        /* 250(f1 7e 08) 306 001(7e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVD, 2, {XMMREG,RM_XMM|BITS32,0,0,0}, NO_DECORATOR, nasm_bytecodes+51160, 644, 7642},
    /* 7463 : VMOVD xmmrm32,xmmreg [mr:t1s:   evex.128.66.0f.w0 d6 /r           ] AVX10_2,FUTURE */
        /* 250(f1 7d 08) 306 001(d6) 101 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVD, 2, {RM_XMM|BITS32,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+51169, 644, 7643},
    /* 7464 : VMOVW xmmreg,xmmrm16 [rm:t1s:   evex.128.f3.map5.w0 6e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(6e) 110 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVW, 2, {XMMREG,RM_XMM|BITS16,0,0,0}, NO_DECORATOR, nasm_bytecodes+51178, 644, 7644},
    /* 7465 : VMOVW xmmrm16,xmmreg [mr:t1s:   evex.128.f3.map5.w0 7e /r         ] AVX10_2,FUTURE */
        /* 250(f5 7e 08) 306 001(7e) 101 : EVEX,PROT,AVX10_2,FUTURE */
        {I_VMOVW, 2, {RM_XMM|BITS16,XMMREG,0,0,0}, NO_DECORATOR, nasm_bytecodes+51187, 644, 7645},
    /* 7466 : VBMACOR16X16X16 ymmreg,ymmreg,ymmrm256 [rvm:fv:   evex.256.np.map6.w0 80 /r         ] AVX512BMM,FUTURE */
        /* 241(f6 7c 28) 301 001(80) 120 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBMACOR16X16X16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+51196, 648, 7648},
    /* 7467 : VBMACOR16X16X16 zmmreg,zmmreg,zmmrm512 [rvm:fv:   evex.512.np.map6.w0 80 /r         ] AVX512BMM,FUTURE */
        /* 241(f6 7c 48) 301 001(80) 120 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBMACOR16X16X16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+51205, 648, 7649},
    /* 7468 : VBMACXOR16X16X16 ymmreg,ymmreg,ymmrm256 [rvm:fv:   evex.256.np.map6.w1 80 /r         ] AVX512BMM,FUTURE */
        /* 241(f6 fc 28) 301 001(80) 120 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBMACXOR16X16X16, 3, {YMMREG,YMMREG,RM_YMM|BITS256,0,0}, NO_DECORATOR, nasm_bytecodes+51214, 648, 7650},
    /* 7469 : VBMACXOR16X16X16 zmmreg,zmmreg,zmmrm512 [rvm:fv:   evex.512.np.map6.w1 80 /r         ] AVX512BMM,FUTURE */
        /* 241(f6 fc 48) 301 001(80) 120 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBMACXOR16X16X16, 3, {ZMMREG,ZMMREG,RM_ZMM|BITS512,0,0}, NO_DECORATOR, nasm_bytecodes+51223, 648, 7651},
    /* 7470 : VBITREV xmmreg|mask|z,xmmrm128 [rm:fv:    evex.128.np.map6.w0 81 /r         ] AVX512BMM,FUTURE */
        /* 250(f6 7c 08) 301 001(81) 110 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBITREV, 2, {XMMREG,RM_XMM|BITS128,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+51232, 648, 7652},
    /* 7471 : VBITREV ymmreg|mask|z,ymmrm256 [rm:fv:    evex.256.np.map6.w0 81 /r         ] AVX512BMM,FUTURE */
        /* 250(f6 7c 28) 301 001(81) 110 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBITREV, 2, {YMMREG,RM_YMM|BITS256,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+51241, 648, 7653},
    /* 7472 : VBITREV zmmreg|mask|z,zmmrm512 [rm:fv:    evex.512.np.map6.w0 81 /r         ] AVX512BMM,FUTURE */
        /* 250(f6 7c 48) 301 001(81) 110 : EVEX,PROT,AVX512BMM,FUTURE */
        {I_VBITREV, 2, {ZMMREG,RM_ZMM|BITS512,0,0,0}, {MASK|Z,0,0,0,0}, nasm_bytecodes+51250, 648, 7654},
    /* 7473 : NOP imm,reg16,rm16 [irm:      o16 0f ibn /r                     ] SM1-2,UNDOC,P6 */
        /* 320 355 300 121 : SM1,SM2,UNDOC,P6 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS16,RM_GPR|BITS16,0,0}, NO_DECORATOR, nasm_bytecodes+72569, 649, 7658},
    /* 7474 : NOP imm,reg32,rm32 [irm:      o32 0f ibn /r                     ] SM1-2,UNDOC,P6 */
        /* 321 355 300 121 : SM1,SM2,UNDOC,P6 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS32,RM_GPR|BITS32,0,0}, NO_DECORATOR, nasm_bytecodes+72574, 649, 7659},
    /* 7475 : NOP imm,reg64,rm64 [irm:      o64 0f ibn /r                     ] SM1-2,LONG,PROT,UNDOC,X86_64 */
        /* 324 355 300 121 : SM1,SM2,LONG,PROT,UNDOC,X86_64 */
        {I_NOP, 3, {IMM_NORMAL,REG_GPR|BITS64,RM_GPR|BITS64,0,0}, NO_DECORATOR, nasm_bytecodes+72579, 650, 7660},
};

static const struct itemplate * const itbl_novex_map0_00[] = {
    instrux + 39,
    NULL
};

static const struct itemplate * const itbl_novex_map0_01[] = {
    instrux + 40,
    instrux + 41,
    instrux + 42,
    NULL
};

static const struct itemplate * const itbl_novex_map0_02[] = {
    instrux + 43,
    NULL
};

static const struct itemplate * const itbl_novex_map0_03[] = {
    instrux + 44,
    instrux + 45,
    instrux + 46,
    NULL
};

static const struct itemplate * const itbl_novex_map0_04[] = {
    instrux + 47,
    NULL
};

static const struct itemplate * const itbl_novex_map0_05[] = {
    instrux + 50,
    instrux + 53,
    instrux + 56,
    NULL
};

static const struct itemplate * const itbl_novex_map0_06[] = {
    instrux + 1401,
    NULL
};

static const struct itemplate * const itbl_novex_map0_07[] = {
    instrux + 1407,
    NULL
};

static const struct itemplate * const itbl_novex_map0_08[] = {
    instrux + 88,
    NULL
};

static const struct itemplate * const itbl_novex_map0_09[] = {
    instrux + 89,
    instrux + 90,
    instrux + 91,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0a[] = {
    instrux + 92,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0b[] = {
    instrux + 93,
    instrux + 94,
    instrux + 95,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0c[] = {
    instrux + 96,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0d[] = {
    instrux + 99,
    instrux + 102,
    instrux + 105,
    NULL
};

static const struct itemplate * const itbl_novex_map0_0e[] = {
    instrux + 1402,
    NULL
};

static const struct itemplate * const itbl_novex_map0_10[] = {
    instrux + 137,
    NULL
};

static const struct itemplate * const itbl_novex_map0_11[] = {
    instrux + 138,
    instrux + 139,
    instrux + 140,
    NULL
};

static const struct itemplate * const itbl_novex_map0_12[] = {
    instrux + 141,
    NULL
};

static const struct itemplate * const itbl_novex_map0_13[] = {
    instrux + 142,
    instrux + 143,
    instrux + 144,
    NULL
};

static const struct itemplate * const itbl_novex_map0_14[] = {
    instrux + 145,
    NULL
};

static const struct itemplate * const itbl_novex_map0_15[] = {
    instrux + 148,
    instrux + 151,
    instrux + 154,
    NULL
};

static const struct itemplate * const itbl_novex_map0_16[] = {
    instrux + 1403,
    NULL
};

static const struct itemplate * const itbl_novex_map0_17[] = {
    instrux + 1408,
    NULL
};

static const struct itemplate * const itbl_novex_map0_18[] = {
    instrux + 186,
    NULL
};

static const struct itemplate * const itbl_novex_map0_19[] = {
    instrux + 187,
    instrux + 188,
    instrux + 189,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1a[] = {
    instrux + 190,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1b[] = {
    instrux + 191,
    instrux + 192,
    instrux + 193,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1c[] = {
    instrux + 194,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1d[] = {
    instrux + 197,
    instrux + 200,
    instrux + 203,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1e[] = {
    instrux + 1404,
    NULL
};

static const struct itemplate * const itbl_novex_map0_1f[] = {
    instrux + 1409,
    NULL
};

static const struct itemplate * const itbl_novex_map0_20[] = {
    instrux + 235,
    NULL
};

static const struct itemplate * const itbl_novex_map0_21[] = {
    instrux + 236,
    instrux + 237,
    instrux + 238,
    NULL
};

static const struct itemplate * const itbl_novex_map0_22[] = {
    instrux + 239,
    NULL
};

static const struct itemplate * const itbl_novex_map0_23[] = {
    instrux + 240,
    instrux + 241,
    instrux + 242,
    NULL
};

static const struct itemplate * const itbl_novex_map0_24[] = {
    instrux + 243,
    NULL
};

static const struct itemplate * const itbl_novex_map0_25[] = {
    instrux + 246,
    instrux + 249,
    instrux + 252,
    NULL
};

static const struct itemplate * const itbl_novex_map0_27[] = {
    instrux + 974,
    NULL
};

static const struct itemplate * const itbl_novex_map0_28[] = {
    instrux + 284,
    NULL
};

static const struct itemplate * const itbl_novex_map0_29[] = {
    instrux + 285,
    instrux + 286,
    instrux + 287,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2a[] = {
    instrux + 288,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2b[] = {
    instrux + 289,
    instrux + 290,
    instrux + 291,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2c[] = {
    instrux + 292,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2d[] = {
    instrux + 295,
    instrux + 298,
    instrux + 301,
    NULL
};

static const struct itemplate * const itbl_novex_map0_2f[] = {
    instrux + 975,
    NULL
};

static const struct itemplate * const itbl_novex_map0_30[] = {
    instrux + 333,
    NULL
};

static const struct itemplate * const itbl_novex_map0_31[] = {
    instrux + 334,
    instrux + 335,
    instrux + 336,
    NULL
};

static const struct itemplate * const itbl_novex_map0_32[] = {
    instrux + 337,
    NULL
};

static const struct itemplate * const itbl_novex_map0_33[] = {
    instrux + 338,
    instrux + 339,
    instrux + 340,
    NULL
};

static const struct itemplate * const itbl_novex_map0_34[] = {
    instrux + 341,
    NULL
};

static const struct itemplate * const itbl_novex_map0_35[] = {
    instrux + 344,
    instrux + 347,
    instrux + 350,
    NULL
};

static const struct itemplate * const itbl_novex_map0_37[] = {
    instrux + 968,
    NULL
};

static const struct itemplate * const itbl_novex_map0_38[] = {
    instrux + 382,
    NULL
};

static const struct itemplate * const itbl_novex_map0_39[] = {
    instrux + 383,
    instrux + 384,
    instrux + 385,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3a[] = {
    instrux + 386,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3b[] = {
    instrux + 387,
    instrux + 388,
    instrux + 389,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3c[] = {
    instrux + 390,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3d[] = {
    instrux + 393,
    instrux + 396,
    instrux + 399,
    NULL
};

static const struct itemplate * const itbl_novex_map0_3f[] = {
    instrux + 973,
    NULL
};

static const struct itemplate * const itbl_novex_map0_40[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_41[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_42[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_43[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_44[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_45[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_46[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_47[] = {
    instrux + 717,
    instrux + 718,
    NULL
};

static const struct itemplate * const itbl_novex_map0_48[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_49[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4a[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4b[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4c[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4d[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4e[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_4f[] = {
    instrux + 731,
    instrux + 732,
    NULL
};

static const struct itemplate * const itbl_novex_map0_50[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_51[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_52[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_53[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_54[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_55[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_56[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_57[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    NULL
};

static const struct itemplate * const itbl_novex_map0_58[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_59[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5a[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5b[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5c[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5d[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5e[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_5f[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    NULL
};

static const struct itemplate * const itbl_novex_map0_60[] = {
    instrux + 1755,
    instrux + 1756,
    instrux + 1757,
    NULL
};

static const struct itemplate * const itbl_novex_map0_61[] = {
    instrux + 1758,
    instrux + 1759,
    instrux + 1760,
    NULL
};

static const struct itemplate * const itbl_novex_map0_62[] = {
    instrux + 1769,
    instrux + 1770,
    NULL
};

static const struct itemplate * const itbl_novex_map0_63[] = {
    instrux + 1002,
    instrux + 1003,
    instrux + 1004,
    instrux + 1420,
    instrux + 1421,
    NULL
};

static const struct itemplate * const itbl_novex_map0_68[] = {
    instrux + 1746,
    instrux + 1747,
    instrux + 1748,
    NULL
};

static const struct itemplate * const itbl_novex_map0_69[] = {
    instrux + 759,
    instrux + 760,
    instrux + 761,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6a[] = {
    instrux + 1743,
    instrux + 1744,
    instrux + 1745,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6b[] = {
    instrux + 756,
    instrux + 757,
    instrux + 758,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6c[] = {
    instrux + 1280,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6d[] = {
    instrux + 1281,
    instrux + 1282,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6e[] = {
    instrux + 1283,
    NULL
};

static const struct itemplate * const itbl_novex_map0_6f[] = {
    instrux + 1284,
    instrux + 1285,
    NULL
};

static const struct itemplate * const itbl_novex_map0_70[] = {
    instrux + 1057,
    instrux + 1073,
    instrux + 1089,
    NULL
};

static const struct itemplate * const itbl_novex_map0_71[] = {
    instrux + 1054,
    instrux + 1070,
    instrux + 1086,
    NULL
};

static const struct itemplate * const itbl_novex_map0_72[] = {
    instrux + 1047,
    instrux + 1063,
    instrux + 1079,
    NULL
};

static const struct itemplate * const itbl_novex_map0_73[] = {
    instrux + 1051,
    instrux + 1067,
    instrux + 1083,
    NULL
};

static const struct itemplate * const itbl_novex_map0_74[] = {
    instrux + 1061,
    instrux + 1077,
    instrux + 1093,
    NULL
};

static const struct itemplate * const itbl_novex_map0_75[] = {
    instrux + 1056,
    instrux + 1072,
    instrux + 1088,
    NULL
};

static const struct itemplate * const itbl_novex_map0_76[] = {
    instrux + 1050,
    instrux + 1066,
    instrux + 1082,
    NULL
};

static const struct itemplate * const itbl_novex_map0_77[] = {
    instrux + 1046,
    instrux + 1062,
    instrux + 1078,
    NULL
};

static const struct itemplate * const itbl_novex_map0_78[] = {
    instrux + 1060,
    instrux + 1076,
    instrux + 1092,
    NULL
};

static const struct itemplate * const itbl_novex_map0_79[] = {
    instrux + 1055,
    instrux + 1071,
    instrux + 1087,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7a[] = {
    instrux + 1058,
    instrux + 1074,
    instrux + 1090,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7b[] = {
    instrux + 1059,
    instrux + 1075,
    instrux + 1091,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7c[] = {
    instrux + 1049,
    instrux + 1065,
    instrux + 1081,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7d[] = {
    instrux + 1053,
    instrux + 1069,
    instrux + 1085,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7e[] = {
    instrux + 1052,
    instrux + 1068,
    instrux + 1084,
    NULL
};

static const struct itemplate * const itbl_novex_map0_7f[] = {
    instrux + 1048,
    instrux + 1064,
    instrux + 1080,
    NULL
};

static const struct itemplate * const itbl_novex_map0_80[] = {
    instrux + 48,
    instrux + 97,
    instrux + 146,
    instrux + 195,
    instrux + 244,
    instrux + 293,
    instrux + 342,
    instrux + 391,
    NULL
};

static const struct itemplate * const itbl_novex_map0_81[] = {
    instrux + 51,
    instrux + 54,
    instrux + 57,
    instrux + 100,
    instrux + 103,
    instrux + 106,
    instrux + 149,
    instrux + 152,
    instrux + 155,
    instrux + 198,
    instrux + 201,
    instrux + 204,
    instrux + 247,
    instrux + 250,
    instrux + 253,
    instrux + 296,
    instrux + 299,
    instrux + 302,
    instrux + 345,
    instrux + 348,
    instrux + 351,
    instrux + 394,
    instrux + 397,
    instrux + 400,
    NULL
};

static const struct itemplate * const itbl_novex_map0_83[] = {
    instrux + 49,
    instrux + 52,
    instrux + 55,
    instrux + 98,
    instrux + 101,
    instrux + 104,
    instrux + 147,
    instrux + 150,
    instrux + 153,
    instrux + 196,
    instrux + 199,
    instrux + 202,
    instrux + 245,
    instrux + 248,
    instrux + 251,
    instrux + 294,
    instrux + 297,
    instrux + 300,
    instrux + 343,
    instrux + 346,
    instrux + 349,
    instrux + 392,
    instrux + 395,
    instrux + 398,
    NULL
};

static const struct itemplate * const itbl_novex_map0_84[] = {
    instrux + 401,
    NULL
};

static const struct itemplate * const itbl_novex_map0_85[] = {
    instrux + 402,
    instrux + 403,
    instrux + 404,
    NULL
};

static const struct itemplate * const itbl_novex_map0_86[] = {
    instrux + 1024,
    NULL
};

static const struct itemplate * const itbl_novex_map0_87[] = {
    instrux + 1025,
    instrux + 1026,
    instrux + 1027,
    NULL
};

static const struct itemplate * const itbl_novex_map0_88[] = {
    instrux + 12,
    NULL
};

static const struct itemplate * const itbl_novex_map0_89[] = {
    instrux + 13,
    instrux + 14,
    instrux + 15,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8a[] = {
    instrux + 16,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8b[] = {
    instrux + 17,
    instrux + 18,
    instrux + 19,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8c[] = {
    instrux + 1380,
    instrux + 1381,
    instrux + 1382,
    instrux + 1383,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8d[] = {
    instrux + 36,
    instrux + 37,
    instrux + 38,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8e[] = {
    instrux + 1384,
    instrux + 1385,
    instrux + 1386,
    instrux + 1387,
    NULL
};

static const struct itemplate * const itbl_novex_map0_8f[] = {
    instrux + 1752,
    instrux + 1753,
    instrux + 1754,
    NULL
};

static const struct itemplate * const itbl_novex_map0_90[] = {
    instrux + 0,
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    instrux + 1352,
    NULL
};

static const struct itemplate * const itbl_novex_map0_91[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_92[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_93[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_94[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_95[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_96[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_97[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_novex_map0_98[] = {
    instrux + 990,
    instrux + 992,
    instrux + 995,
    NULL
};

static const struct itemplate * const itbl_novex_map0_99[] = {
    instrux + 991,
    instrux + 993,
    instrux + 994,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9a[] = {
    instrux + 1202,
    instrux + 1203,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9c[] = {
    instrux + 1252,
    instrux + 1253,
    instrux + 1254,
    instrux + 1255,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9d[] = {
    instrux + 1256,
    instrux + 1257,
    instrux + 1258,
    instrux + 1259,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9e[] = {
    instrux + 1250,
    NULL
};

static const struct itemplate * const itbl_novex_map0_9f[] = {
    instrux + 1249,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a0[] = {
    instrux + 4,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a1[] = {
    instrux + 5,
    instrux + 6,
    instrux + 7,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a2[] = {
    instrux + 8,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a3[] = {
    instrux + 9,
    instrux + 10,
    instrux + 11,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a4[] = {
    instrux + 1268,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a5[] = {
    instrux + 1269,
    instrux + 1270,
    instrux + 1271,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a6[] = {
    instrux + 1260,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a7[] = {
    instrux + 1261,
    instrux + 1262,
    instrux + 1263,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a8[] = {
    instrux + 405,
    NULL
};

static const struct itemplate * const itbl_novex_map0_a9[] = {
    instrux + 406,
    instrux + 407,
    instrux + 408,
    NULL
};

static const struct itemplate * const itbl_novex_map0_aa[] = {
    instrux + 1272,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ab[] = {
    instrux + 1273,
    instrux + 1274,
    instrux + 1275,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ac[] = {
    instrux + 1264,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ad[] = {
    instrux + 1265,
    instrux + 1266,
    instrux + 1267,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ae[] = {
    instrux + 1276,
    NULL
};

static const struct itemplate * const itbl_novex_map0_af[] = {
    instrux + 1277,
    instrux + 1278,
    instrux + 1279,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b0[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b1[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b2[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b3[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b4[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b5[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b6[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b7[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b8[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_b9[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ba[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bb[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bc[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bd[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_be[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_bf[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c0[] = {
    instrux + 421,
    instrux + 433,
    instrux + 445,
    instrux + 457,
    instrux + 469,
    instrux + 481,
    instrux + 493,
    instrux + 505,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c1[] = {
    instrux + 422,
    instrux + 423,
    instrux + 424,
    instrux + 434,
    instrux + 435,
    instrux + 436,
    instrux + 446,
    instrux + 447,
    instrux + 448,
    instrux + 458,
    instrux + 459,
    instrux + 460,
    instrux + 470,
    instrux + 471,
    instrux + 472,
    instrux + 482,
    instrux + 483,
    instrux + 484,
    instrux + 494,
    instrux + 495,
    instrux + 496,
    instrux + 506,
    instrux + 507,
    instrux + 508,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c2[] = {
    instrux + 1214,
    instrux + 1215,
    instrux + 1216,
    instrux + 1217,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c3[] = {
    instrux + 1210,
    instrux + 1211,
    instrux + 1212,
    instrux + 1213,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c4[] = {
    instrux + 1390,
    instrux + 1391,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c5[] = {
    instrux + 1388,
    instrux + 1389,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c6[] = {
    instrux + 24,
    instrux + 4372,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c7[] = {
    instrux + 25,
    instrux + 26,
    instrux + 27,
    instrux + 4373,
    instrux + 4374,
    instrux + 4375,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c8[] = {
    instrux + 1761,
    instrux + 1762,
    instrux + 1763,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_novex_map0_c9[] = {
    instrux + 1765,
    instrux + 1766,
    instrux + 1767,
    instrux + 1768,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ca[] = {
    instrux + 1222,
    instrux + 1223,
    instrux + 1224,
    instrux + 1225,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cb[] = {
    instrux + 1218,
    instrux + 1219,
    instrux + 1220,
    instrux + 1221,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cc[] = {
    instrux + 1228,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cd[] = {
    instrux + 1226,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ce[] = {
    instrux + 1229,
    NULL
};

static const struct itemplate * const itbl_novex_map0_cf[] = {
    instrux + 1234,
    instrux + 1235,
    instrux + 1236,
    instrux + 1237,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d0[] = {
    instrux + 413,
    instrux + 425,
    instrux + 437,
    instrux + 449,
    instrux + 461,
    instrux + 473,
    instrux + 485,
    instrux + 497,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d1[] = {
    instrux + 414,
    instrux + 415,
    instrux + 416,
    instrux + 426,
    instrux + 427,
    instrux + 428,
    instrux + 438,
    instrux + 439,
    instrux + 440,
    instrux + 450,
    instrux + 451,
    instrux + 452,
    instrux + 462,
    instrux + 463,
    instrux + 464,
    instrux + 474,
    instrux + 475,
    instrux + 476,
    instrux + 486,
    instrux + 487,
    instrux + 488,
    instrux + 498,
    instrux + 499,
    instrux + 500,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d2[] = {
    instrux + 417,
    instrux + 429,
    instrux + 441,
    instrux + 453,
    instrux + 465,
    instrux + 477,
    instrux + 489,
    instrux + 501,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d3[] = {
    instrux + 418,
    instrux + 419,
    instrux + 420,
    instrux + 430,
    instrux + 431,
    instrux + 432,
    instrux + 442,
    instrux + 443,
    instrux + 444,
    instrux + 454,
    instrux + 455,
    instrux + 456,
    instrux + 466,
    instrux + 467,
    instrux + 468,
    instrux + 478,
    instrux + 479,
    instrux + 480,
    instrux + 490,
    instrux + 491,
    instrux + 492,
    instrux + 502,
    instrux + 503,
    instrux + 504,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d4[] = {
    instrux + 971,
    instrux + 972,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d5[] = {
    instrux + 969,
    instrux + 970,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d6[] = {
    instrux + 1251,
    instrux + 1788,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d7[] = {
    instrux + 1791,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d8[] = {
    instrux + 1462,
    instrux + 1465,
    instrux + 1467,
    instrux + 1492,
    instrux + 1494,
    instrux + 1495,
    instrux + 1500,
    instrux + 1502,
    instrux + 1503,
    instrux + 1508,
    instrux + 1511,
    instrux + 1513,
    instrux + 1516,
    instrux + 1520,
    instrux + 1521,
    instrux + 1572,
    instrux + 1576,
    instrux + 1577,
    instrux + 1613,
    instrux + 1617,
    instrux + 1618,
    instrux + 1621,
    instrux + 1625,
    instrux + 1626,
    NULL
};

static const struct itemplate * const itbl_novex_map0_d9[] = {
    instrux + 1460,
    instrux + 1461,
    instrux + 1474,
    instrux + 1505,
    instrux + 1506,
    instrux + 1545,
    instrux + 1559,
    instrux + 1562,
    instrux + 1563,
    instrux + 1564,
    instrux + 1565,
    instrux + 1566,
    instrux + 1567,
    instrux + 1568,
    instrux + 1569,
    instrux + 1570,
    instrux + 1571,
    instrux + 1584,
    instrux + 1586,
    instrux + 1587,
    instrux + 1590,
    instrux + 1591,
    instrux + 1592,
    instrux + 1593,
    instrux + 1594,
    instrux + 1597,
    instrux + 1599,
    instrux + 1600,
    instrux + 1601,
    instrux + 1602,
    instrux + 1605,
    instrux + 1606,
    instrux + 1607,
    instrux + 1629,
    instrux + 1639,
    instrux + 1640,
    instrux + 1641,
    instrux + 1642,
    instrux + 1643,
    instrux + 1644,
    instrux + 1645,
    NULL
};

static const struct itemplate * const itbl_novex_map0_da[] = {
    instrux + 1476,
    instrux + 1477,
    instrux + 1478,
    instrux + 1479,
    instrux + 1480,
    instrux + 1481,
    instrux + 1490,
    instrux + 1491,
    instrux + 1530,
    instrux + 1532,
    instrux + 1534,
    instrux + 1536,
    instrux + 1538,
    instrux + 1543,
    instrux + 1555,
    instrux + 1557,
    instrux + 1638,
    NULL
};

static const struct itemplate * const itbl_novex_map0_db[] = {
    instrux + 1475,
    instrux + 1482,
    instrux + 1483,
    instrux + 1484,
    instrux + 1485,
    instrux + 1486,
    instrux + 1487,
    instrux + 1488,
    instrux + 1489,
    instrux + 1496,
    instrux + 1497,
    instrux + 1507,
    instrux + 1525,
    instrux + 1540,
    instrux + 1546,
    instrux + 1547,
    instrux + 1549,
    instrux + 1553,
    instrux + 1561,
    instrux + 1580,
    instrux + 1581,
    instrux + 1582,
    instrux + 1583,
    instrux + 1598,
    instrux + 1609,
    instrux + 1632,
    instrux + 1633,
    NULL
};

static const struct itemplate * const itbl_novex_map0_dc[] = {
    instrux + 1463,
    instrux + 1464,
    instrux + 1466,
    instrux + 1493,
    instrux + 1501,
    instrux + 1509,
    instrux + 1510,
    instrux + 1512,
    instrux + 1517,
    instrux + 1518,
    instrux + 1519,
    instrux + 1573,
    instrux + 1574,
    instrux + 1575,
    instrux + 1614,
    instrux + 1615,
    instrux + 1616,
    instrux + 1622,
    instrux + 1623,
    instrux + 1624,
    NULL
};

static const struct itemplate * const itbl_novex_map0_dd[] = {
    instrux + 1526,
    instrux + 1527,
    instrux + 1554,
    instrux + 1560,
    instrux + 1585,
    instrux + 1588,
    instrux + 1595,
    instrux + 1596,
    instrux + 1603,
    instrux + 1604,
    instrux + 1608,
    instrux + 1610,
    instrux + 1611,
    instrux + 1630,
    instrux + 1631,
    instrux + 1636,
    instrux + 1637,
    NULL
};

static const struct itemplate * const itbl_novex_map0_de[] = {
    instrux + 1468,
    instrux + 1469,
    instrux + 1504,
    instrux + 1514,
    instrux + 1515,
    instrux + 1522,
    instrux + 1523,
    instrux + 1531,
    instrux + 1533,
    instrux + 1535,
    instrux + 1537,
    instrux + 1539,
    instrux + 1544,
    instrux + 1556,
    instrux + 1558,
    instrux + 1578,
    instrux + 1579,
    instrux + 1619,
    instrux + 1620,
    instrux + 1627,
    instrux + 1628,
    NULL
};

static const struct itemplate * const itbl_novex_map0_df[] = {
    instrux + 1470,
    instrux + 1471,
    instrux + 1472,
    instrux + 1473,
    instrux + 1498,
    instrux + 1499,
    instrux + 1528,
    instrux + 1529,
    instrux + 1541,
    instrux + 1542,
    instrux + 1548,
    instrux + 1550,
    instrux + 1551,
    instrux + 1552,
    instrux + 1589,
    instrux + 1612,
    instrux + 1634,
    instrux + 1635,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e0[] = {
    instrux + 1154,
    instrux + 1155,
    instrux + 1156,
    instrux + 1169,
    instrux + 1170,
    instrux + 1171,
    instrux + 1172,
    instrux + 1173,
    instrux + 1174,
    instrux + 1181,
    instrux + 1182,
    instrux + 1183,
    instrux + 1184,
    instrux + 1185,
    instrux + 1186,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e1[] = {
    instrux + 1151,
    instrux + 1152,
    instrux + 1153,
    instrux + 1163,
    instrux + 1164,
    instrux + 1165,
    instrux + 1166,
    instrux + 1167,
    instrux + 1168,
    instrux + 1175,
    instrux + 1176,
    instrux + 1177,
    instrux + 1178,
    instrux + 1179,
    instrux + 1180,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e2[] = {
    instrux + 1148,
    instrux + 1149,
    instrux + 1150,
    instrux + 1157,
    instrux + 1158,
    instrux + 1159,
    instrux + 1160,
    instrux + 1161,
    instrux + 1162,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e3[] = {
    instrux + 1142,
    instrux + 1143,
    instrux + 1144,
    instrux + 1145,
    instrux + 1146,
    instrux + 1147,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e4[] = {
    instrux + 1368,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e5[] = {
    instrux + 1369,
    instrux + 1370,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e6[] = {
    instrux + 1374,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e7[] = {
    instrux + 1375,
    instrux + 1376,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e8[] = {
    instrux + 1196,
    instrux + 1197,
    instrux + 1198,
    NULL
};

static const struct itemplate * const itbl_novex_map0_e9[] = {
    instrux + 1032,
    instrux + 1033,
    instrux + 1034,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ea[] = {
    instrux + 1038,
    instrux + 1039,
    NULL
};

static const struct itemplate * const itbl_novex_map0_eb[] = {
    instrux + 1029,
    instrux + 1030,
    instrux + 1031,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ec[] = {
    instrux + 1371,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ed[] = {
    instrux + 1372,
    instrux + 1373,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ee[] = {
    instrux + 1377,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ef[] = {
    instrux + 1378,
    instrux + 1379,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f1[] = {
    instrux + 1227,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f4[] = {
    instrux + 1351,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f5[] = {
    instrux + 1248,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f6[] = {
    instrux + 409,
    instrux + 745,
    instrux + 774,
    instrux + 786,
    instrux + 794,
    instrux + 802,
    instrux + 814,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f7[] = {
    instrux + 410,
    instrux + 411,
    instrux + 412,
    instrux + 746,
    instrux + 747,
    instrux + 748,
    instrux + 775,
    instrux + 776,
    instrux + 777,
    instrux + 787,
    instrux + 788,
    instrux + 789,
    instrux + 795,
    instrux + 796,
    instrux + 797,
    instrux + 803,
    instrux + 804,
    instrux + 805,
    instrux + 815,
    instrux + 816,
    instrux + 817,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f8[] = {
    instrux + 1240,
    NULL
};

static const struct itemplate * const itbl_novex_map0_f9[] = {
    instrux + 1244,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fa[] = {
    instrux + 1242,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fb[] = {
    instrux + 1246,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fc[] = {
    instrux + 1241,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fd[] = {
    instrux + 1245,
    NULL
};

static const struct itemplate * const itbl_novex_map0_fe[] = {
    instrux + 719,
    instrux + 733,
    NULL
};

static const struct itemplate * const itbl_novex_map0_ff[] = {
    instrux + 720,
    instrux + 721,
    instrux + 722,
    instrux + 734,
    instrux + 735,
    instrux + 736,
    instrux + 1035,
    instrux + 1036,
    instrux + 1037,
    instrux + 1040,
    instrux + 1041,
    instrux + 1042,
    instrux + 1043,
    instrux + 1044,
    instrux + 1045,
    instrux + 1199,
    instrux + 1200,
    instrux + 1201,
    instrux + 1204,
    instrux + 1205,
    instrux + 1206,
    instrux + 1207,
    instrux + 1208,
    instrux + 1209,
    instrux + 1740,
    instrux + 1741,
    instrux + 1742,
    instrux + 1789,
    NULL
};

static const struct itemplate * const * const itbl_novex_map0[256] = {
    itbl_novex_map0_00,
    itbl_novex_map0_01,
    itbl_novex_map0_02,
    itbl_novex_map0_03,
    itbl_novex_map0_04,
    itbl_novex_map0_05,
    itbl_novex_map0_06,
    itbl_novex_map0_07,
    itbl_novex_map0_08,
    itbl_novex_map0_09,
    itbl_novex_map0_0a,
    itbl_novex_map0_0b,
    itbl_novex_map0_0c,
    itbl_novex_map0_0d,
    itbl_novex_map0_0e,
    NULL,
    itbl_novex_map0_10,
    itbl_novex_map0_11,
    itbl_novex_map0_12,
    itbl_novex_map0_13,
    itbl_novex_map0_14,
    itbl_novex_map0_15,
    itbl_novex_map0_16,
    itbl_novex_map0_17,
    itbl_novex_map0_18,
    itbl_novex_map0_19,
    itbl_novex_map0_1a,
    itbl_novex_map0_1b,
    itbl_novex_map0_1c,
    itbl_novex_map0_1d,
    itbl_novex_map0_1e,
    itbl_novex_map0_1f,
    itbl_novex_map0_20,
    itbl_novex_map0_21,
    itbl_novex_map0_22,
    itbl_novex_map0_23,
    itbl_novex_map0_24,
    itbl_novex_map0_25,
    NULL,
    itbl_novex_map0_27,
    itbl_novex_map0_28,
    itbl_novex_map0_29,
    itbl_novex_map0_2a,
    itbl_novex_map0_2b,
    itbl_novex_map0_2c,
    itbl_novex_map0_2d,
    NULL,
    itbl_novex_map0_2f,
    itbl_novex_map0_30,
    itbl_novex_map0_31,
    itbl_novex_map0_32,
    itbl_novex_map0_33,
    itbl_novex_map0_34,
    itbl_novex_map0_35,
    NULL,
    itbl_novex_map0_37,
    itbl_novex_map0_38,
    itbl_novex_map0_39,
    itbl_novex_map0_3a,
    itbl_novex_map0_3b,
    itbl_novex_map0_3c,
    itbl_novex_map0_3d,
    NULL,
    itbl_novex_map0_3f,
    itbl_novex_map0_40,
    itbl_novex_map0_41,
    itbl_novex_map0_42,
    itbl_novex_map0_43,
    itbl_novex_map0_44,
    itbl_novex_map0_45,
    itbl_novex_map0_46,
    itbl_novex_map0_47,
    itbl_novex_map0_48,
    itbl_novex_map0_49,
    itbl_novex_map0_4a,
    itbl_novex_map0_4b,
    itbl_novex_map0_4c,
    itbl_novex_map0_4d,
    itbl_novex_map0_4e,
    itbl_novex_map0_4f,
    itbl_novex_map0_50,
    itbl_novex_map0_51,
    itbl_novex_map0_52,
    itbl_novex_map0_53,
    itbl_novex_map0_54,
    itbl_novex_map0_55,
    itbl_novex_map0_56,
    itbl_novex_map0_57,
    itbl_novex_map0_58,
    itbl_novex_map0_59,
    itbl_novex_map0_5a,
    itbl_novex_map0_5b,
    itbl_novex_map0_5c,
    itbl_novex_map0_5d,
    itbl_novex_map0_5e,
    itbl_novex_map0_5f,
    itbl_novex_map0_60,
    itbl_novex_map0_61,
    itbl_novex_map0_62,
    itbl_novex_map0_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map0_68,
    itbl_novex_map0_69,
    itbl_novex_map0_6a,
    itbl_novex_map0_6b,
    itbl_novex_map0_6c,
    itbl_novex_map0_6d,
    itbl_novex_map0_6e,
    itbl_novex_map0_6f,
    itbl_novex_map0_70,
    itbl_novex_map0_71,
    itbl_novex_map0_72,
    itbl_novex_map0_73,
    itbl_novex_map0_74,
    itbl_novex_map0_75,
    itbl_novex_map0_76,
    itbl_novex_map0_77,
    itbl_novex_map0_78,
    itbl_novex_map0_79,
    itbl_novex_map0_7a,
    itbl_novex_map0_7b,
    itbl_novex_map0_7c,
    itbl_novex_map0_7d,
    itbl_novex_map0_7e,
    itbl_novex_map0_7f,
    itbl_novex_map0_80,
    itbl_novex_map0_81,
    NULL,
    itbl_novex_map0_83,
    itbl_novex_map0_84,
    itbl_novex_map0_85,
    itbl_novex_map0_86,
    itbl_novex_map0_87,
    itbl_novex_map0_88,
    itbl_novex_map0_89,
    itbl_novex_map0_8a,
    itbl_novex_map0_8b,
    itbl_novex_map0_8c,
    itbl_novex_map0_8d,
    itbl_novex_map0_8e,
    itbl_novex_map0_8f,
    itbl_novex_map0_90,
    itbl_novex_map0_91,
    itbl_novex_map0_92,
    itbl_novex_map0_93,
    itbl_novex_map0_94,
    itbl_novex_map0_95,
    itbl_novex_map0_96,
    itbl_novex_map0_97,
    itbl_novex_map0_98,
    itbl_novex_map0_99,
    itbl_novex_map0_9a,
    NULL,
    itbl_novex_map0_9c,
    itbl_novex_map0_9d,
    itbl_novex_map0_9e,
    itbl_novex_map0_9f,
    itbl_novex_map0_a0,
    itbl_novex_map0_a1,
    itbl_novex_map0_a2,
    itbl_novex_map0_a3,
    itbl_novex_map0_a4,
    itbl_novex_map0_a5,
    itbl_novex_map0_a6,
    itbl_novex_map0_a7,
    itbl_novex_map0_a8,
    itbl_novex_map0_a9,
    itbl_novex_map0_aa,
    itbl_novex_map0_ab,
    itbl_novex_map0_ac,
    itbl_novex_map0_ad,
    itbl_novex_map0_ae,
    itbl_novex_map0_af,
    itbl_novex_map0_b0,
    itbl_novex_map0_b1,
    itbl_novex_map0_b2,
    itbl_novex_map0_b3,
    itbl_novex_map0_b4,
    itbl_novex_map0_b5,
    itbl_novex_map0_b6,
    itbl_novex_map0_b7,
    itbl_novex_map0_b8,
    itbl_novex_map0_b9,
    itbl_novex_map0_ba,
    itbl_novex_map0_bb,
    itbl_novex_map0_bc,
    itbl_novex_map0_bd,
    itbl_novex_map0_be,
    itbl_novex_map0_bf,
    itbl_novex_map0_c0,
    itbl_novex_map0_c1,
    itbl_novex_map0_c2,
    itbl_novex_map0_c3,
    itbl_novex_map0_c4,
    itbl_novex_map0_c5,
    itbl_novex_map0_c6,
    itbl_novex_map0_c7,
    itbl_novex_map0_c8,
    itbl_novex_map0_c9,
    itbl_novex_map0_ca,
    itbl_novex_map0_cb,
    itbl_novex_map0_cc,
    itbl_novex_map0_cd,
    itbl_novex_map0_ce,
    itbl_novex_map0_cf,
    itbl_novex_map0_d0,
    itbl_novex_map0_d1,
    itbl_novex_map0_d2,
    itbl_novex_map0_d3,
    itbl_novex_map0_d4,
    itbl_novex_map0_d5,
    itbl_novex_map0_d6,
    itbl_novex_map0_d7,
    itbl_novex_map0_d8,
    itbl_novex_map0_d9,
    itbl_novex_map0_da,
    itbl_novex_map0_db,
    itbl_novex_map0_dc,
    itbl_novex_map0_dd,
    itbl_novex_map0_de,
    itbl_novex_map0_df,
    itbl_novex_map0_e0,
    itbl_novex_map0_e1,
    itbl_novex_map0_e2,
    itbl_novex_map0_e3,
    itbl_novex_map0_e4,
    itbl_novex_map0_e5,
    itbl_novex_map0_e6,
    itbl_novex_map0_e7,
    itbl_novex_map0_e8,
    itbl_novex_map0_e9,
    itbl_novex_map0_ea,
    itbl_novex_map0_eb,
    itbl_novex_map0_ec,
    itbl_novex_map0_ed,
    itbl_novex_map0_ee,
    itbl_novex_map0_ef,
    NULL,
    itbl_novex_map0_f1,
    NULL,
    NULL,
    itbl_novex_map0_f4,
    itbl_novex_map0_f5,
    itbl_novex_map0_f6,
    itbl_novex_map0_f7,
    itbl_novex_map0_f8,
    itbl_novex_map0_f9,
    itbl_novex_map0_fa,
    itbl_novex_map0_fb,
    itbl_novex_map0_fc,
    itbl_novex_map0_fd,
    itbl_novex_map0_fe,
    itbl_novex_map0_ff,
};

static const struct itemplate * const itbl_novex_map1_00[] = {
    instrux + 1190,
    instrux + 1191,
    instrux + 1192,
    instrux + 1193,
    instrux + 1194,
    instrux + 1195,
    instrux + 1428,
    instrux + 1429,
    instrux + 1431,
    instrux + 1438,
    instrux + 1439,
    instrux + 1440,
    instrux + 1441,
    instrux + 1442,
    instrux + 1443,
    instrux + 1444,
    instrux + 1445,
    instrux + 1452,
    instrux + 1453,
    instrux + 1454,
    instrux + 1455,
    instrux + 1456,
    instrux + 1457,
    instrux + 1458,
    instrux + 1459,
    NULL
};

static const struct itemplate * const itbl_novex_map1_01[] = {
    instrux + 1238,
    instrux + 1239,
    instrux + 1243,
    instrux + 1247,
    instrux + 1289,
    instrux + 1294,
    instrux + 1295,
    instrux + 1296,
    instrux + 1303,
    instrux + 1304,
    instrux + 1305,
    instrux + 1306,
    instrux + 1309,
    instrux + 1320,
    instrux + 1321,
    instrux + 1322,
    instrux + 1323,
    instrux + 1324,
    instrux + 1337,
    instrux + 1340,
    instrux + 1341,
    instrux + 1353,
    instrux + 1354,
    instrux + 1355,
    instrux + 1356,
    instrux + 1357,
    instrux + 1358,
    instrux + 1359,
    instrux + 1360,
    instrux + 1361,
    instrux + 1362,
    instrux + 1430,
    instrux + 1432,
    instrux + 1433,
    instrux + 1434,
    instrux + 1435,
    instrux + 1436,
    instrux + 1437,
    instrux + 1446,
    instrux + 1447,
    instrux + 1448,
    instrux + 1449,
    instrux + 1450,
    instrux + 1451,
    instrux + 1778,
    instrux + 2699,
    instrux + 2700,
    instrux + 2927,
    instrux + 2928,
    instrux + 2929,
    instrux + 2931,
    instrux + 2932,
    instrux + 2933,
    instrux + 2934,
    instrux + 2939,
    instrux + 2940,
    instrux + 2941,
    instrux + 2944,
    instrux + 2952,
    instrux + 2953,
    instrux + 2954,
    instrux + 2955,
    instrux + 4376,
    instrux + 4377,
    instrux + 6704,
    instrux + 6705,
    instrux + 6717,
    instrux + 6718,
    instrux + 6825,
    instrux + 6826,
    instrux + 6827,
    instrux + 6835,
    instrux + 6836,
    instrux + 6837,
    instrux + 6854,
    instrux + 6855,
    instrux + 6856,
    instrux + 7198,
    instrux + 7200,
    instrux + 7201,
    instrux + 7202,
    NULL
};

static const struct itemplate * const itbl_novex_map1_02[] = {
    instrux + 1422,
    instrux + 1423,
    instrux + 1424,
    NULL
};

static const struct itemplate * const itbl_novex_map1_03[] = {
    instrux + 1425,
    instrux + 1426,
    instrux + 1427,
    NULL
};

static const struct itemplate * const itbl_novex_map1_05[] = {
    instrux + 1230,
    NULL
};

static const struct itemplate * const itbl_novex_map1_06[] = {
    instrux + 1318,
    NULL
};

static const struct itemplate * const itbl_novex_map1_07[] = {
    instrux + 1233,
    NULL
};

static const struct itemplate * const itbl_novex_map1_08[] = {
    instrux + 1297,
    NULL
};

static const struct itemplate * const itbl_novex_map1_09[] = {
    instrux + 1298,
    instrux + 1299,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0b[] = {
    instrux + 1787,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0d[] = {
    instrux + 1705,
    instrux + 1706,
    instrux + 4378,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0e[] = {
    instrux + 1524,
    NULL
};

static const struct itemplate * const itbl_novex_map1_0f[] = {
    instrux + 1667,
    instrux + 1675,
    instrux + 1676,
    instrux + 1677,
    instrux + 1678,
    instrux + 1679,
    instrux + 1680,
    instrux + 1681,
    instrux + 1682,
    instrux + 1683,
    instrux + 1684,
    instrux + 1685,
    instrux + 1686,
    instrux + 1687,
    instrux + 1688,
    instrux + 1689,
    instrux + 1690,
    instrux + 1691,
    instrux + 1696,
    instrux + 2736,
    instrux + 2737,
    instrux + 2738,
    instrux + 2739,
    instrux + 2740,
    instrux + 3081,
    instrux + 3082,
    NULL
};

static const struct itemplate * const itbl_novex_map1_10[] = {
    instrux + 2674,
    instrux + 2676,
    instrux + 2901,
    instrux + 2903,
    NULL
};

static const struct itemplate * const itbl_novex_map1_11[] = {
    instrux + 2675,
    instrux + 2677,
    instrux + 2902,
    instrux + 2904,
    NULL
};

static const struct itemplate * const itbl_novex_map1_12[] = {
    instrux + 2668,
    instrux + 2670,
    instrux + 2898,
    instrux + 2924,
    instrux + 2926,
    NULL
};

static const struct itemplate * const itbl_novex_map1_13[] = {
    instrux + 2669,
    instrux + 2897,
    NULL
};

static const struct itemplate * const itbl_novex_map1_14[] = {
    instrux + 2693,
    instrux + 2915,
    NULL
};

static const struct itemplate * const itbl_novex_map1_15[] = {
    instrux + 2692,
    instrux + 2914,
    NULL
};

static const struct itemplate * const itbl_novex_map1_16[] = {
    instrux + 2665,
    instrux + 2667,
    instrux + 2896,
    instrux + 2925,
    NULL
};

static const struct itemplate * const itbl_novex_map1_17[] = {
    instrux + 2666,
    instrux + 2895,
    NULL
};

static const struct itemplate * const itbl_novex_map1_18[] = {
    instrux + 2713,
    instrux + 2714,
    instrux + 2715,
    instrux + 2716,
    instrux + 2717,
    instrux + 2718,
    instrux + 6719,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_19[] = {
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1a[] = {
    instrux + 4380,
    instrux + 4381,
    instrux + 4382,
    instrux + 4383,
    instrux + 4384,
    instrux + 4385,
    instrux + 4389,
    instrux + 4390,
    instrux + 4393,
    instrux + 4394,
    instrux + 4395,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1b[] = {
    instrux + 4379,
    instrux + 4386,
    instrux + 4387,
    instrux + 4388,
    instrux + 4391,
    instrux + 4392,
    instrux + 4396,
    instrux + 4397,
    instrux + 4398,
    instrux + 4399,
    instrux + 4400,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1c[] = {
    instrux + 6708,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1d[] = {
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1e[] = {
    instrux + 6829,
    instrux + 6830,
    instrux + 6833,
    instrux + 6834,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_1f[] = {
    instrux + 1,
    instrux + 2,
    instrux + 3,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_novex_map1_20[] = {
    instrux + 1325,
    instrux + 1327,
    NULL
};

static const struct itemplate * const itbl_novex_map1_21[] = {
    instrux + 1329,
    instrux + 1331,
    NULL
};

static const struct itemplate * const itbl_novex_map1_22[] = {
    instrux + 1326,
    instrux + 1328,
    NULL
};

static const struct itemplate * const itbl_novex_map1_23[] = {
    instrux + 1330,
    instrux + 1332,
    NULL
};

static const struct itemplate * const itbl_novex_map1_28[] = {
    instrux + 2663,
    instrux + 2893,
    NULL
};

static const struct itemplate * const itbl_novex_map1_29[] = {
    instrux + 2664,
    instrux + 2894,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2a[] = {
    instrux + 2647,
    instrux + 2649,
    instrux + 2650,
    instrux + 2873,
    instrux + 2879,
    instrux + 2880,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2b[] = {
    instrux + 2673,
    instrux + 2745,
    instrux + 2992,
    instrux + 2993,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2c[] = {
    instrux + 2653,
    instrux + 2654,
    instrux + 2655,
    instrux + 2882,
    instrux + 2885,
    instrux + 2886,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2d[] = {
    instrux + 2648,
    instrux + 2651,
    instrux + 2652,
    instrux + 2871,
    instrux + 2876,
    instrux + 2877,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2e[] = {
    instrux + 2691,
    instrux + 2913,
    NULL
};

static const struct itemplate * const itbl_novex_map1_2f[] = {
    instrux + 2646,
    instrux + 2867,
    NULL
};

static const struct itemplate * const itbl_novex_map1_30[] = {
    instrux + 1333,
    NULL
};

static const struct itemplate * const itbl_novex_map1_31[] = {
    instrux + 1308,
    NULL
};

static const struct itemplate * const itbl_novex_map1_32[] = {
    instrux + 1334,
    NULL
};

static const struct itemplate * const itbl_novex_map1_33[] = {
    instrux + 1307,
    NULL
};

static const struct itemplate * const itbl_novex_map1_34[] = {
    instrux + 1231,
    NULL
};

static const struct itemplate * const itbl_novex_map1_35[] = {
    instrux + 1232,
    NULL
};

static const struct itemplate * const itbl_novex_map1_37[] = {
    instrux + 3080,
    NULL
};

static const struct itemplate * const itbl_novex_map1_38[] = {
    instrux + 1342,
    instrux + 1346,
    NULL
};

static const struct itemplate * const itbl_novex_map1_40[] = {
    instrux + 2235,
    instrux + 2251,
    instrux + 2267,
    NULL
};

static const struct itemplate * const itbl_novex_map1_41[] = {
    instrux + 2232,
    instrux + 2248,
    instrux + 2264,
    NULL
};

static const struct itemplate * const itbl_novex_map1_42[] = {
    instrux + 2225,
    instrux + 2241,
    instrux + 2257,
    NULL
};

static const struct itemplate * const itbl_novex_map1_43[] = {
    instrux + 2229,
    instrux + 2245,
    instrux + 2261,
    NULL
};

static const struct itemplate * const itbl_novex_map1_44[] = {
    instrux + 2239,
    instrux + 2255,
    instrux + 2271,
    NULL
};

static const struct itemplate * const itbl_novex_map1_45[] = {
    instrux + 2234,
    instrux + 2250,
    instrux + 2266,
    NULL
};

static const struct itemplate * const itbl_novex_map1_46[] = {
    instrux + 2228,
    instrux + 2244,
    instrux + 2260,
    NULL
};

static const struct itemplate * const itbl_novex_map1_47[] = {
    instrux + 2224,
    instrux + 2240,
    instrux + 2256,
    NULL
};

static const struct itemplate * const itbl_novex_map1_48[] = {
    instrux + 2238,
    instrux + 2254,
    instrux + 2270,
    NULL
};

static const struct itemplate * const itbl_novex_map1_49[] = {
    instrux + 2233,
    instrux + 2249,
    instrux + 2265,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4a[] = {
    instrux + 2236,
    instrux + 2252,
    instrux + 2268,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4b[] = {
    instrux + 2237,
    instrux + 2253,
    instrux + 2269,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4c[] = {
    instrux + 2227,
    instrux + 2243,
    instrux + 2259,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4d[] = {
    instrux + 2231,
    instrux + 2247,
    instrux + 2263,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4e[] = {
    instrux + 2230,
    instrux + 2246,
    instrux + 2262,
    NULL
};

static const struct itemplate * const itbl_novex_map1_4f[] = {
    instrux + 2226,
    instrux + 2242,
    instrux + 2258,
    NULL
};

static const struct itemplate * const itbl_novex_map1_50[] = {
    instrux + 1666,
    instrux + 2671,
    instrux + 2672,
    instrux + 2899,
    instrux + 2900,
    NULL
};

static const struct itemplate * const itbl_novex_map1_51[] = {
    instrux + 1659,
    instrux + 2686,
    instrux + 2687,
    instrux + 2909,
    instrux + 2910,
    NULL
};

static const struct itemplate * const itbl_novex_map1_52[] = {
    instrux + 1694,
    instrux + 2683,
    instrux + 2684,
    NULL
};

static const struct itemplate * const itbl_novex_map1_53[] = {
    instrux + 2681,
    instrux + 2682,
    NULL
};

static const struct itemplate * const itbl_novex_map1_54[] = {
    instrux + 1674,
    instrux + 2627,
    instrux + 2848,
    NULL
};

static const struct itemplate * const itbl_novex_map1_55[] = {
    instrux + 1726,
    instrux + 2626,
    instrux + 2847,
    NULL
};

static const struct itemplate * const itbl_novex_map1_56[] = {
    instrux + 2680,
    instrux + 2907,
    NULL
};

static const struct itemplate * const itbl_novex_map1_57[] = {
    instrux + 2694,
    instrux + 2916,
    NULL
};

static const struct itemplate * const itbl_novex_map1_58[] = {
    instrux + 1703,
    instrux + 2624,
    instrux + 2625,
    instrux + 2845,
    instrux + 2846,
    NULL
};

static const struct itemplate * const itbl_novex_map1_59[] = {
    instrux + 1697,
    instrux + 2678,
    instrux + 2679,
    instrux + 2905,
    instrux + 2906,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5a[] = {
    instrux + 1702,
    instrux + 2872,
    instrux + 2875,
    instrux + 2878,
    instrux + 2881,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5b[] = {
    instrux + 1701,
    instrux + 2869,
    instrux + 2874,
    instrux + 2884,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5c[] = {
    instrux + 1700,
    instrux + 2689,
    instrux + 2690,
    instrux + 2911,
    instrux + 2912,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5d[] = {
    instrux + 1695,
    instrux + 2661,
    instrux + 2662,
    instrux + 2891,
    instrux + 2892,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5e[] = {
    instrux + 1692,
    instrux + 2656,
    instrux + 2657,
    instrux + 2887,
    instrux + 2888,
    NULL
};

static const struct itemplate * const itbl_novex_map1_5f[] = {
    instrux + 2659,
    instrux + 2660,
    instrux + 2889,
    instrux + 2890,
    NULL
};

static const struct itemplate * const itbl_novex_map1_60[] = {
    instrux + 1734,
    instrux + 2840,
    NULL
};

static const struct itemplate * const itbl_novex_map1_61[] = {
    instrux + 1736,
    instrux + 2841,
    NULL
};

static const struct itemplate * const itbl_novex_map1_62[] = {
    instrux + 1735,
    instrux + 2842,
    NULL
};

static const struct itemplate * const itbl_novex_map1_63[] = {
    instrux + 1654,
    instrux + 2764,
    NULL
};

static const struct itemplate * const itbl_novex_map1_64[] = {
    instrux + 1671,
    instrux + 2783,
    NULL
};

static const struct itemplate * const itbl_novex_map1_65[] = {
    instrux + 1673,
    instrux + 2784,
    NULL
};

static const struct itemplate * const itbl_novex_map1_66[] = {
    instrux + 1672,
    instrux + 2785,
    NULL
};

static const struct itemplate * const itbl_novex_map1_67[] = {
    instrux + 1655,
    instrux + 2766,
    NULL
};

static const struct itemplate * const itbl_novex_map1_68[] = {
    instrux + 1731,
    instrux + 2836,
    NULL
};

static const struct itemplate * const itbl_novex_map1_69[] = {
    instrux + 1733,
    instrux + 2837,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6a[] = {
    instrux + 1732,
    instrux + 2838,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6b[] = {
    instrux + 1653,
    instrux + 2765,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6c[] = {
    instrux + 2843,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6d[] = {
    instrux + 2839,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6e[] = {
    instrux + 1647,
    instrux + 1651,
    instrux + 2749,
    instrux + 2750,
    instrux + 2761,
    NULL
};

static const struct itemplate * const itbl_novex_map1_6f[] = {
    instrux + 1649,
    instrux + 2752,
    instrux + 2754,
    NULL
};

static const struct itemplate * const itbl_novex_map1_70[] = {
    instrux + 2735,
    instrux + 2803,
    instrux + 2804,
    instrux + 2805,
    instrux + 2806,
    instrux + 2807,
    instrux + 2808,
    NULL
};

static const struct itemplate * const itbl_novex_map1_71[] = {
    instrux + 1712,
    instrux + 1716,
    instrux + 1722,
    instrux + 2811,
    instrux + 2817,
    instrux + 2822,
    NULL
};

static const struct itemplate * const itbl_novex_map1_72[] = {
    instrux + 1708,
    instrux + 1714,
    instrux + 1718,
    instrux + 2813,
    instrux + 2819,
    instrux + 2824,
    NULL
};

static const struct itemplate * const itbl_novex_map1_73[] = {
    instrux + 1710,
    instrux + 1720,
    instrux + 2809,
    instrux + 2815,
    instrux + 2820,
    instrux + 2826,
    NULL
};

static const struct itemplate * const itbl_novex_map1_74[] = {
    instrux + 1668,
    instrux + 2780,
    NULL
};

static const struct itemplate * const itbl_novex_map1_75[] = {
    instrux + 1670,
    instrux + 2781,
    NULL
};

static const struct itemplate * const itbl_novex_map1_76[] = {
    instrux + 1669,
    instrux + 2782,
    NULL
};

static const struct itemplate * const itbl_novex_map1_77[] = {
    instrux + 1646,
    NULL
};

static const struct itemplate * const itbl_novex_map1_78[] = {
    instrux + 2937,
    instrux + 2938,
    instrux + 2988,
    instrux + 2990,
    NULL
};

static const struct itemplate * const itbl_novex_map1_79[] = {
    instrux + 2942,
    instrux + 2943,
    instrux + 2989,
    instrux + 2991,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7c[] = {
    instrux + 2919,
    instrux + 2920,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7d[] = {
    instrux + 2921,
    instrux + 2922,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7e[] = {
    instrux + 1648,
    instrux + 1652,
    instrux + 2748,
    instrux + 2751,
    instrux + 2757,
    instrux + 2760,
    instrux + 2762,
    NULL
};

static const struct itemplate * const itbl_novex_map1_7f[] = {
    instrux + 1650,
    instrux + 2753,
    instrux + 2755,
    NULL
};

static const struct itemplate * const itbl_novex_map1_80[] = {
    instrux + 1105,
    instrux + 1121,
    instrux + 1137,
    NULL
};

static const struct itemplate * const itbl_novex_map1_81[] = {
    instrux + 1102,
    instrux + 1118,
    instrux + 1134,
    NULL
};

static const struct itemplate * const itbl_novex_map1_82[] = {
    instrux + 1095,
    instrux + 1111,
    instrux + 1127,
    NULL
};

static const struct itemplate * const itbl_novex_map1_83[] = {
    instrux + 1099,
    instrux + 1115,
    instrux + 1131,
    NULL
};

static const struct itemplate * const itbl_novex_map1_84[] = {
    instrux + 1109,
    instrux + 1125,
    instrux + 1141,
    NULL
};

static const struct itemplate * const itbl_novex_map1_85[] = {
    instrux + 1104,
    instrux + 1120,
    instrux + 1136,
    NULL
};

static const struct itemplate * const itbl_novex_map1_86[] = {
    instrux + 1098,
    instrux + 1114,
    instrux + 1130,
    NULL
};

static const struct itemplate * const itbl_novex_map1_87[] = {
    instrux + 1094,
    instrux + 1110,
    instrux + 1126,
    NULL
};

static const struct itemplate * const itbl_novex_map1_88[] = {
    instrux + 1108,
    instrux + 1124,
    instrux + 1140,
    NULL
};

static const struct itemplate * const itbl_novex_map1_89[] = {
    instrux + 1103,
    instrux + 1119,
    instrux + 1135,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8a[] = {
    instrux + 1106,
    instrux + 1122,
    instrux + 1138,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8b[] = {
    instrux + 1107,
    instrux + 1123,
    instrux + 1139,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8c[] = {
    instrux + 1097,
    instrux + 1113,
    instrux + 1129,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8d[] = {
    instrux + 1101,
    instrux + 1117,
    instrux + 1133,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8e[] = {
    instrux + 1100,
    instrux + 1116,
    instrux + 1132,
    NULL
};

static const struct itemplate * const itbl_novex_map1_8f[] = {
    instrux + 1096,
    instrux + 1112,
    instrux + 1128,
    NULL
};

static const struct itemplate * const itbl_novex_map1_90[] = {
    instrux + 2523,
    NULL
};

static const struct itemplate * const itbl_novex_map1_91[] = {
    instrux + 2520,
    NULL
};

static const struct itemplate * const itbl_novex_map1_92[] = {
    instrux + 2513,
    NULL
};

static const struct itemplate * const itbl_novex_map1_93[] = {
    instrux + 2517,
    NULL
};

static const struct itemplate * const itbl_novex_map1_94[] = {
    instrux + 2527,
    NULL
};

static const struct itemplate * const itbl_novex_map1_95[] = {
    instrux + 2522,
    NULL
};

static const struct itemplate * const itbl_novex_map1_96[] = {
    instrux + 2516,
    NULL
};

static const struct itemplate * const itbl_novex_map1_97[] = {
    instrux + 2512,
    NULL
};

static const struct itemplate * const itbl_novex_map1_98[] = {
    instrux + 2526,
    NULL
};

static const struct itemplate * const itbl_novex_map1_99[] = {
    instrux + 2521,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9a[] = {
    instrux + 2524,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9b[] = {
    instrux + 2525,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9c[] = {
    instrux + 2515,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9d[] = {
    instrux + 2519,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9e[] = {
    instrux + 2518,
    NULL
};

static const struct itemplate * const itbl_novex_map1_9f[] = {
    instrux + 2514,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a0[] = {
    instrux + 1405,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a1[] = {
    instrux + 1410,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a2[] = {
    instrux + 1319,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a3[] = {
    instrux + 874,
    instrux + 875,
    instrux + 876,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a4[] = {
    instrux + 838,
    instrux + 839,
    instrux + 840,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a5[] = {
    instrux + 841,
    instrux + 842,
    instrux + 843,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a6[] = {
    instrux + 4036,
    instrux + 4037,
    instrux + 4038,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a7[] = {
    instrux + 4030,
    instrux + 4031,
    instrux + 4032,
    instrux + 4033,
    instrux + 4034,
    instrux + 4035,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a8[] = {
    instrux + 1406,
    NULL
};

static const struct itemplate * const itbl_novex_map1_a9[] = {
    instrux + 1411,
    NULL
};

static const struct itemplate * const itbl_novex_map1_aa[] = {
    instrux + 1350,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ab[] = {
    instrux + 892,
    instrux + 893,
    instrux + 894,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ac[] = {
    instrux + 856,
    instrux + 857,
    instrux + 858,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ad[] = {
    instrux + 859,
    instrux + 860,
    instrux + 861,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ae[] = {
    instrux + 1286,
    instrux + 1287,
    instrux + 1288,
    instrux + 1290,
    instrux + 1291,
    instrux + 1292,
    instrux + 1293,
    instrux + 1363,
    instrux + 1364,
    instrux + 1365,
    instrux + 1366,
    instrux + 1367,
    instrux + 1412,
    instrux + 1413,
    instrux + 1414,
    instrux + 1415,
    instrux + 1416,
    instrux + 1417,
    instrux + 1418,
    instrux + 1419,
    instrux + 2658,
    instrux + 2688,
    instrux + 2695,
    instrux + 2696,
    instrux + 2697,
    instrux + 2698,
    instrux + 2701,
    instrux + 2702,
    instrux + 2705,
    instrux + 2706,
    instrux + 2709,
    instrux + 2710,
    instrux + 2719,
    instrux + 2746,
    instrux + 2747,
    instrux + 6706,
    instrux + 6707,
    instrux + 6828,
    instrux + 6831,
    instrux + 6832,
    NULL
};

static const struct itemplate * const itbl_novex_map1_af[] = {
    instrux + 753,
    instrux + 754,
    instrux + 755,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b0[] = {
    instrux + 1011,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b1[] = {
    instrux + 1012,
    instrux + 1013,
    instrux + 1014,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b2[] = {
    instrux + 1398,
    instrux + 1399,
    instrux + 1400,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b3[] = {
    instrux + 886,
    instrux + 887,
    instrux + 888,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b4[] = {
    instrux + 1392,
    instrux + 1393,
    instrux + 1394,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b5[] = {
    instrux + 1395,
    instrux + 1396,
    instrux + 1397,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b6[] = {
    instrux + 1005,
    instrux + 1006,
    instrux + 1007,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b7[] = {
    instrux + 1008,
    instrux + 1009,
    instrux + 1010,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b8[] = {
    instrux + 1187,
    instrux + 1188,
    instrux + 1189,
    instrux + 3074,
    instrux + 3075,
    instrux + 3076,
    NULL
};

static const struct itemplate * const itbl_novex_map1_b9[] = {
    instrux + 1783,
    instrux + 1784,
    instrux + 1785,
    instrux + 1786,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ba[] = {
    instrux + 877,
    instrux + 878,
    instrux + 879,
    instrux + 883,
    instrux + 884,
    instrux + 885,
    instrux + 889,
    instrux + 890,
    instrux + 891,
    instrux + 895,
    instrux + 896,
    instrux + 897,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bb[] = {
    instrux + 880,
    instrux + 881,
    instrux + 882,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bc[] = {
    instrux + 898,
    instrux + 899,
    instrux + 900,
    instrux + 910,
    instrux + 911,
    instrux + 912,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bd[] = {
    instrux + 901,
    instrux + 902,
    instrux + 903,
    instrux + 904,
    instrux + 905,
    instrux + 906,
    NULL
};

static const struct itemplate * const itbl_novex_map1_be[] = {
    instrux + 996,
    instrux + 997,
    instrux + 998,
    NULL
};

static const struct itemplate * const itbl_novex_map1_bf[] = {
    instrux + 999,
    instrux + 1000,
    instrux + 1001,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c0[] = {
    instrux + 1017,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c1[] = {
    instrux + 1018,
    instrux + 1019,
    instrux + 1020,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c2[] = {
    instrux + 2628,
    instrux + 2629,
    instrux + 2630,
    instrux + 2631,
    instrux + 2632,
    instrux + 2633,
    instrux + 2634,
    instrux + 2635,
    instrux + 2636,
    instrux + 2637,
    instrux + 2638,
    instrux + 2639,
    instrux + 2640,
    instrux + 2641,
    instrux + 2642,
    instrux + 2643,
    instrux + 2644,
    instrux + 2645,
    instrux + 2849,
    instrux + 2850,
    instrux + 2851,
    instrux + 2852,
    instrux + 2853,
    instrux + 2854,
    instrux + 2855,
    instrux + 2856,
    instrux + 2857,
    instrux + 2858,
    instrux + 2859,
    instrux + 2860,
    instrux + 2861,
    instrux + 2862,
    instrux + 2863,
    instrux + 2864,
    instrux + 2865,
    instrux + 2866,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c3[] = {
    instrux + 2743,
    instrux + 2744,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c4[] = {
    instrux + 2725,
    instrux + 2726,
    instrux + 2727,
    instrux + 2787,
    instrux + 2788,
    instrux + 2789,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c5[] = {
    instrux + 2724,
    instrux + 2786,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c6[] = {
    instrux + 2685,
    instrux + 2908,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c7[] = {
    instrux + 1015,
    instrux + 1016,
    instrux + 1310,
    instrux + 1311,
    instrux + 1312,
    instrux + 1313,
    instrux + 1314,
    instrux + 1315,
    instrux + 1316,
    instrux + 1317,
    instrux + 2703,
    instrux + 2704,
    instrux + 2707,
    instrux + 2708,
    instrux + 2711,
    instrux + 2712,
    instrux + 2930,
    instrux + 2935,
    instrux + 2936,
    instrux + 2945,
    instrux + 7199,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c8[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_c9[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ca[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cb[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cc[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cd[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ce[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_cf[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d0[] = {
    instrux + 2917,
    instrux + 2918,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d1[] = {
    instrux + 1721,
    instrux + 2821,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d2[] = {
    instrux + 1717,
    instrux + 2823,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d3[] = {
    instrux + 1719,
    instrux + 2825,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d4[] = {
    instrux + 2770,
    instrux + 2771,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d5[] = {
    instrux + 1699,
    instrux + 2798,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d6[] = {
    instrux + 2756,
    instrux + 2758,
    instrux + 2759,
    instrux + 2763,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d7[] = {
    instrux + 2732,
    instrux + 2795,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d8[] = {
    instrux + 1728,
    instrux + 2834,
    NULL
};

static const struct itemplate * const itbl_novex_map1_d9[] = {
    instrux + 1729,
    instrux + 2835,
    NULL
};

static const struct itemplate * const itbl_novex_map1_da[] = {
    instrux + 2731,
    instrux + 2794,
    NULL
};

static const struct itemplate * const itbl_novex_map1_db[] = {
    instrux + 1664,
    instrux + 2776,
    NULL
};

static const struct itemplate * const itbl_novex_map1_dc[] = {
    instrux + 1661,
    instrux + 2774,
    NULL
};

static const struct itemplate * const itbl_novex_map1_dd[] = {
    instrux + 1662,
    instrux + 2775,
    NULL
};

static const struct itemplate * const itbl_novex_map1_de[] = {
    instrux + 2729,
    instrux + 2792,
    NULL
};

static const struct itemplate * const itbl_novex_map1_df[] = {
    instrux + 1665,
    instrux + 2777,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e0[] = {
    instrux + 2722,
    instrux + 2778,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e1[] = {
    instrux + 1715,
    instrux + 2816,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e2[] = {
    instrux + 1713,
    instrux + 2818,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e3[] = {
    instrux + 2723,
    instrux + 2779,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e4[] = {
    instrux + 2733,
    instrux + 2796,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e5[] = {
    instrux + 1698,
    instrux + 2797,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e6[] = {
    instrux + 2868,
    instrux + 2870,
    instrux + 2883,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e7[] = {
    instrux + 2721,
    instrux + 2742,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e8[] = {
    instrux + 1725,
    instrux + 2832,
    NULL
};

static const struct itemplate * const itbl_novex_map1_e9[] = {
    instrux + 1727,
    instrux + 2833,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ea[] = {
    instrux + 2730,
    instrux + 2793,
    NULL
};

static const struct itemplate * const itbl_novex_map1_eb[] = {
    instrux + 1704,
    instrux + 2801,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ec[] = {
    instrux + 1658,
    instrux + 2772,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ed[] = {
    instrux + 1660,
    instrux + 2773,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ee[] = {
    instrux + 2728,
    instrux + 2791,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ef[] = {
    instrux + 1777,
    instrux + 2844,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f0[] = {
    instrux + 2923,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f1[] = {
    instrux + 1711,
    instrux + 2810,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f2[] = {
    instrux + 1707,
    instrux + 2812,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f3[] = {
    instrux + 1709,
    instrux + 2814,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f4[] = {
    instrux + 2799,
    instrux + 2800,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f5[] = {
    instrux + 1693,
    instrux + 2790,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f6[] = {
    instrux + 2734,
    instrux + 2802,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f7[] = {
    instrux + 2720,
    instrux + 2741,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f8[] = {
    instrux + 1723,
    instrux + 2827,
    NULL
};

static const struct itemplate * const itbl_novex_map1_f9[] = {
    instrux + 1730,
    instrux + 2828,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fa[] = {
    instrux + 1724,
    instrux + 2829,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fb[] = {
    instrux + 2830,
    instrux + 2831,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fc[] = {
    instrux + 1656,
    instrux + 2767,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fd[] = {
    instrux + 1663,
    instrux + 2768,
    NULL
};

static const struct itemplate * const itbl_novex_map1_fe[] = {
    instrux + 1657,
    instrux + 2769,
    NULL
};

static const struct itemplate * const itbl_novex_map1_ff[] = {
    instrux + 1779,
    instrux + 1780,
    instrux + 1781,
    instrux + 1782,
    NULL
};

static const struct itemplate * const * const itbl_novex_map1[256] = {
    itbl_novex_map1_00,
    itbl_novex_map1_01,
    itbl_novex_map1_02,
    itbl_novex_map1_03,
    NULL,
    itbl_novex_map1_05,
    itbl_novex_map1_06,
    itbl_novex_map1_07,
    itbl_novex_map1_08,
    itbl_novex_map1_09,
    NULL,
    itbl_novex_map1_0b,
    NULL,
    itbl_novex_map1_0d,
    itbl_novex_map1_0e,
    itbl_novex_map1_0f,
    itbl_novex_map1_10,
    itbl_novex_map1_11,
    itbl_novex_map1_12,
    itbl_novex_map1_13,
    itbl_novex_map1_14,
    itbl_novex_map1_15,
    itbl_novex_map1_16,
    itbl_novex_map1_17,
    itbl_novex_map1_18,
    itbl_novex_map1_19,
    itbl_novex_map1_1a,
    itbl_novex_map1_1b,
    itbl_novex_map1_1c,
    itbl_novex_map1_1d,
    itbl_novex_map1_1e,
    itbl_novex_map1_1f,
    itbl_novex_map1_20,
    itbl_novex_map1_21,
    itbl_novex_map1_22,
    itbl_novex_map1_23,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map1_28,
    itbl_novex_map1_29,
    itbl_novex_map1_2a,
    itbl_novex_map1_2b,
    itbl_novex_map1_2c,
    itbl_novex_map1_2d,
    itbl_novex_map1_2e,
    itbl_novex_map1_2f,
    itbl_novex_map1_30,
    itbl_novex_map1_31,
    itbl_novex_map1_32,
    itbl_novex_map1_33,
    itbl_novex_map1_34,
    itbl_novex_map1_35,
    NULL,
    itbl_novex_map1_37,
    itbl_novex_map1_38,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map1_40,
    itbl_novex_map1_41,
    itbl_novex_map1_42,
    itbl_novex_map1_43,
    itbl_novex_map1_44,
    itbl_novex_map1_45,
    itbl_novex_map1_46,
    itbl_novex_map1_47,
    itbl_novex_map1_48,
    itbl_novex_map1_49,
    itbl_novex_map1_4a,
    itbl_novex_map1_4b,
    itbl_novex_map1_4c,
    itbl_novex_map1_4d,
    itbl_novex_map1_4e,
    itbl_novex_map1_4f,
    itbl_novex_map1_50,
    itbl_novex_map1_51,
    itbl_novex_map1_52,
    itbl_novex_map1_53,
    itbl_novex_map1_54,
    itbl_novex_map1_55,
    itbl_novex_map1_56,
    itbl_novex_map1_57,
    itbl_novex_map1_58,
    itbl_novex_map1_59,
    itbl_novex_map1_5a,
    itbl_novex_map1_5b,
    itbl_novex_map1_5c,
    itbl_novex_map1_5d,
    itbl_novex_map1_5e,
    itbl_novex_map1_5f,
    itbl_novex_map1_60,
    itbl_novex_map1_61,
    itbl_novex_map1_62,
    itbl_novex_map1_63,
    itbl_novex_map1_64,
    itbl_novex_map1_65,
    itbl_novex_map1_66,
    itbl_novex_map1_67,
    itbl_novex_map1_68,
    itbl_novex_map1_69,
    itbl_novex_map1_6a,
    itbl_novex_map1_6b,
    itbl_novex_map1_6c,
    itbl_novex_map1_6d,
    itbl_novex_map1_6e,
    itbl_novex_map1_6f,
    itbl_novex_map1_70,
    itbl_novex_map1_71,
    itbl_novex_map1_72,
    itbl_novex_map1_73,
    itbl_novex_map1_74,
    itbl_novex_map1_75,
    itbl_novex_map1_76,
    itbl_novex_map1_77,
    itbl_novex_map1_78,
    itbl_novex_map1_79,
    NULL,
    NULL,
    itbl_novex_map1_7c,
    itbl_novex_map1_7d,
    itbl_novex_map1_7e,
    itbl_novex_map1_7f,
    itbl_novex_map1_80,
    itbl_novex_map1_81,
    itbl_novex_map1_82,
    itbl_novex_map1_83,
    itbl_novex_map1_84,
    itbl_novex_map1_85,
    itbl_novex_map1_86,
    itbl_novex_map1_87,
    itbl_novex_map1_88,
    itbl_novex_map1_89,
    itbl_novex_map1_8a,
    itbl_novex_map1_8b,
    itbl_novex_map1_8c,
    itbl_novex_map1_8d,
    itbl_novex_map1_8e,
    itbl_novex_map1_8f,
    itbl_novex_map1_90,
    itbl_novex_map1_91,
    itbl_novex_map1_92,
    itbl_novex_map1_93,
    itbl_novex_map1_94,
    itbl_novex_map1_95,
    itbl_novex_map1_96,
    itbl_novex_map1_97,
    itbl_novex_map1_98,
    itbl_novex_map1_99,
    itbl_novex_map1_9a,
    itbl_novex_map1_9b,
    itbl_novex_map1_9c,
    itbl_novex_map1_9d,
    itbl_novex_map1_9e,
    itbl_novex_map1_9f,
    itbl_novex_map1_a0,
    itbl_novex_map1_a1,
    itbl_novex_map1_a2,
    itbl_novex_map1_a3,
    itbl_novex_map1_a4,
    itbl_novex_map1_a5,
    itbl_novex_map1_a6,
    itbl_novex_map1_a7,
    itbl_novex_map1_a8,
    itbl_novex_map1_a9,
    itbl_novex_map1_aa,
    itbl_novex_map1_ab,
    itbl_novex_map1_ac,
    itbl_novex_map1_ad,
    itbl_novex_map1_ae,
    itbl_novex_map1_af,
    itbl_novex_map1_b0,
    itbl_novex_map1_b1,
    itbl_novex_map1_b2,
    itbl_novex_map1_b3,
    itbl_novex_map1_b4,
    itbl_novex_map1_b5,
    itbl_novex_map1_b6,
    itbl_novex_map1_b7,
    itbl_novex_map1_b8,
    itbl_novex_map1_b9,
    itbl_novex_map1_ba,
    itbl_novex_map1_bb,
    itbl_novex_map1_bc,
    itbl_novex_map1_bd,
    itbl_novex_map1_be,
    itbl_novex_map1_bf,
    itbl_novex_map1_c0,
    itbl_novex_map1_c1,
    itbl_novex_map1_c2,
    itbl_novex_map1_c3,
    itbl_novex_map1_c4,
    itbl_novex_map1_c5,
    itbl_novex_map1_c6,
    itbl_novex_map1_c7,
    itbl_novex_map1_c8,
    itbl_novex_map1_c9,
    itbl_novex_map1_ca,
    itbl_novex_map1_cb,
    itbl_novex_map1_cc,
    itbl_novex_map1_cd,
    itbl_novex_map1_ce,
    itbl_novex_map1_cf,
    itbl_novex_map1_d0,
    itbl_novex_map1_d1,
    itbl_novex_map1_d2,
    itbl_novex_map1_d3,
    itbl_novex_map1_d4,
    itbl_novex_map1_d5,
    itbl_novex_map1_d6,
    itbl_novex_map1_d7,
    itbl_novex_map1_d8,
    itbl_novex_map1_d9,
    itbl_novex_map1_da,
    itbl_novex_map1_db,
    itbl_novex_map1_dc,
    itbl_novex_map1_dd,
    itbl_novex_map1_de,
    itbl_novex_map1_df,
    itbl_novex_map1_e0,
    itbl_novex_map1_e1,
    itbl_novex_map1_e2,
    itbl_novex_map1_e3,
    itbl_novex_map1_e4,
    itbl_novex_map1_e5,
    itbl_novex_map1_e6,
    itbl_novex_map1_e7,
    itbl_novex_map1_e8,
    itbl_novex_map1_e9,
    itbl_novex_map1_ea,
    itbl_novex_map1_eb,
    itbl_novex_map1_ec,
    itbl_novex_map1_ed,
    itbl_novex_map1_ee,
    itbl_novex_map1_ef,
    itbl_novex_map1_f0,
    itbl_novex_map1_f1,
    itbl_novex_map1_f2,
    itbl_novex_map1_f3,
    itbl_novex_map1_f4,
    itbl_novex_map1_f5,
    itbl_novex_map1_f6,
    itbl_novex_map1_f7,
    itbl_novex_map1_f8,
    itbl_novex_map1_f9,
    itbl_novex_map1_fa,
    itbl_novex_map1_fb,
    itbl_novex_map1_fc,
    itbl_novex_map1_fd,
    itbl_novex_map1_fe,
    itbl_novex_map1_ff,
};

static const struct itemplate * const itbl_novex_map2_00[] = {
    instrux + 2980,
    instrux + 2981,
    NULL
};

static const struct itemplate * const itbl_novex_map2_01[] = {
    instrux + 2964,
    instrux + 2965,
    NULL
};

static const struct itemplate * const itbl_novex_map2_02[] = {
    instrux + 2966,
    instrux + 2967,
    NULL
};

static const struct itemplate * const itbl_novex_map2_03[] = {
    instrux + 2968,
    instrux + 2969,
    NULL
};

static const struct itemplate * const itbl_novex_map2_04[] = {
    instrux + 2976,
    instrux + 2977,
    NULL
};

static const struct itemplate * const itbl_novex_map2_05[] = {
    instrux + 2970,
    instrux + 2971,
    NULL
};

static const struct itemplate * const itbl_novex_map2_06[] = {
    instrux + 2972,
    instrux + 2973,
    NULL
};

static const struct itemplate * const itbl_novex_map2_07[] = {
    instrux + 2974,
    instrux + 2975,
    NULL
};

static const struct itemplate * const itbl_novex_map2_08[] = {
    instrux + 2982,
    instrux + 2983,
    NULL
};

static const struct itemplate * const itbl_novex_map2_09[] = {
    instrux + 2984,
    instrux + 2985,
    NULL
};

static const struct itemplate * const itbl_novex_map2_0a[] = {
    instrux + 2986,
    instrux + 2987,
    NULL
};

static const struct itemplate * const itbl_novex_map2_0b[] = {
    instrux + 2978,
    instrux + 2979,
    NULL
};

static const struct itemplate * const itbl_novex_map2_10[] = {
    instrux + 3008,
    instrux + 3009,
    NULL
};

static const struct itemplate * const itbl_novex_map2_14[] = {
    instrux + 2998,
    instrux + 2999,
    NULL
};

static const struct itemplate * const itbl_novex_map2_15[] = {
    instrux + 2996,
    instrux + 2997,
    NULL
};

static const struct itemplate * const itbl_novex_map2_17[] = {
    instrux + 3048,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1c[] = {
    instrux + 2956,
    instrux + 2957,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1d[] = {
    instrux + 2958,
    instrux + 2959,
    NULL
};

static const struct itemplate * const itbl_novex_map2_1e[] = {
    instrux + 2960,
    instrux + 2961,
    NULL
};

static const struct itemplate * const itbl_novex_map2_20[] = {
    instrux + 3034,
    NULL
};

static const struct itemplate * const itbl_novex_map2_21[] = {
    instrux + 3035,
    NULL
};

static const struct itemplate * const itbl_novex_map2_22[] = {
    instrux + 3036,
    NULL
};

static const struct itemplate * const itbl_novex_map2_23[] = {
    instrux + 3037,
    NULL
};

static const struct itemplate * const itbl_novex_map2_24[] = {
    instrux + 3038,
    NULL
};

static const struct itemplate * const itbl_novex_map2_25[] = {
    instrux + 3039,
    NULL
};

static const struct itemplate * const itbl_novex_map2_28[] = {
    instrux + 3046,
    NULL
};

static const struct itemplate * const itbl_novex_map2_29[] = {
    instrux + 3011,
    NULL
};

static const struct itemplate * const itbl_novex_map2_2a[] = {
    instrux + 3005,
    NULL
};

static const struct itemplate * const itbl_novex_map2_2b[] = {
    instrux + 3007,
    NULL
};

static const struct itemplate * const itbl_novex_map2_30[] = {
    instrux + 3040,
    NULL
};

static const struct itemplate * const itbl_novex_map2_31[] = {
    instrux + 3041,
    NULL
};

static const struct itemplate * const itbl_novex_map2_32[] = {
    instrux + 3042,
    NULL
};

static const struct itemplate * const itbl_novex_map2_33[] = {
    instrux + 3043,
    NULL
};

static const struct itemplate * const itbl_novex_map2_34[] = {
    instrux + 3044,
    NULL
};

static const struct itemplate * const itbl_novex_map2_35[] = {
    instrux + 3045,
    NULL
};

static const struct itemplate * const itbl_novex_map2_37[] = {
    instrux + 3073,
    NULL
};

static const struct itemplate * const itbl_novex_map2_38[] = {
    instrux + 3030,
    NULL
};

static const struct itemplate * const itbl_novex_map2_39[] = {
    instrux + 3031,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3a[] = {
    instrux + 3033,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3b[] = {
    instrux + 3032,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3c[] = {
    instrux + 3026,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3d[] = {
    instrux + 3027,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3e[] = {
    instrux + 3029,
    NULL
};

static const struct itemplate * const itbl_novex_map2_3f[] = {
    instrux + 3028,
    NULL
};

static const struct itemplate * const itbl_novex_map2_40[] = {
    instrux + 3047,
    NULL
};

static const struct itemplate * const itbl_novex_map2_41[] = {
    instrux + 3020,
    NULL
};

static const struct itemplate * const itbl_novex_map2_80[] = {
    instrux + 2946,
    instrux + 2947,
    NULL
};

static const struct itemplate * const itbl_novex_map2_81[] = {
    instrux + 2949,
    instrux + 2950,
    NULL
};

static const struct itemplate * const itbl_novex_map2_82[] = {
    instrux + 1300,
    instrux + 1301,
    NULL
};

static const struct itemplate * const itbl_novex_map2_8a[] = {
    instrux + 28,
    NULL
};

static const struct itemplate * const itbl_novex_map2_8b[] = {
    instrux + 29,
    instrux + 30,
    instrux + 31,
    NULL
};

static const struct itemplate * const itbl_novex_map2_c8[] = {
    instrux + 4403,
    NULL
};

static const struct itemplate * const itbl_novex_map2_c9[] = {
    instrux + 4401,
    NULL
};

static const struct itemplate * const itbl_novex_map2_ca[] = {
    instrux + 4402,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cb[] = {
    instrux + 4407,
    instrux + 4408,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cc[] = {
    instrux + 4405,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cd[] = {
    instrux + 4406,
    NULL
};

static const struct itemplate * const itbl_novex_map2_cf[] = {
    instrux + 6732,
    NULL
};

static const struct itemplate * const itbl_novex_map2_d8[] = {
    instrux + 3102,
    instrux + 3103,
    instrux + 3104,
    instrux + 3105,
    NULL
};

static const struct itemplate * const itbl_novex_map2_db[] = {
    instrux + 3087,
    NULL
};

static const struct itemplate * const itbl_novex_map2_dc[] = {
    instrux + 3083,
    instrux + 3097,
    instrux + 3101,
    NULL
};

static const struct itemplate * const itbl_novex_map2_dd[] = {
    instrux + 3084,
    instrux + 3095,
    NULL
};

static const struct itemplate * const itbl_novex_map2_de[] = {
    instrux + 3085,
    instrux + 3098,
    NULL
};

static const struct itemplate * const itbl_novex_map2_df[] = {
    instrux + 3086,
    instrux + 3096,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f0[] = {
    instrux + 978,
    instrux + 979,
    instrux + 980,
    instrux + 3053,
    instrux + 3059,
    instrux + 3067,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f1[] = {
    instrux + 981,
    instrux + 982,
    instrux + 983,
    instrux + 3054,
    instrux + 3055,
    instrux + 3060,
    instrux + 3061,
    instrux + 3062,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f5[] = {
    instrux + 6838,
    instrux + 6840,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f6[] = {
    instrux + 826,
    instrux + 827,
    instrux + 832,
    instrux + 833,
    instrux + 6842,
    instrux + 6844,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f8[] = {
    instrux + 6713,
    instrux + 6714,
    instrux + 6715,
    instrux + 6846,
    instrux + 6847,
    instrux + 6848,
    instrux + 6850,
    instrux + 6851,
    instrux + 6852,
    NULL
};

static const struct itemplate * const itbl_novex_map2_f9[] = {
    instrux + 6709,
    instrux + 6710,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fa[] = {
    instrux + 3099,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fb[] = {
    instrux + 3100,
    NULL
};

static const struct itemplate * const itbl_novex_map2_fc[] = {
    instrux + 7182,
    instrux + 7183,
    instrux + 7186,
    instrux + 7187,
    instrux + 7190,
    instrux + 7191,
    instrux + 7194,
    instrux + 7195,
    NULL
};

static const struct itemplate * const * const itbl_novex_map2[256] = {
    itbl_novex_map2_00,
    itbl_novex_map2_01,
    itbl_novex_map2_02,
    itbl_novex_map2_03,
    itbl_novex_map2_04,
    itbl_novex_map2_05,
    itbl_novex_map2_06,
    itbl_novex_map2_07,
    itbl_novex_map2_08,
    itbl_novex_map2_09,
    itbl_novex_map2_0a,
    itbl_novex_map2_0b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_10,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_14,
    itbl_novex_map2_15,
    NULL,
    itbl_novex_map2_17,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_1c,
    itbl_novex_map2_1d,
    itbl_novex_map2_1e,
    NULL,
    itbl_novex_map2_20,
    itbl_novex_map2_21,
    itbl_novex_map2_22,
    itbl_novex_map2_23,
    itbl_novex_map2_24,
    itbl_novex_map2_25,
    NULL,
    NULL,
    itbl_novex_map2_28,
    itbl_novex_map2_29,
    itbl_novex_map2_2a,
    itbl_novex_map2_2b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_30,
    itbl_novex_map2_31,
    itbl_novex_map2_32,
    itbl_novex_map2_33,
    itbl_novex_map2_34,
    itbl_novex_map2_35,
    NULL,
    itbl_novex_map2_37,
    itbl_novex_map2_38,
    itbl_novex_map2_39,
    itbl_novex_map2_3a,
    itbl_novex_map2_3b,
    itbl_novex_map2_3c,
    itbl_novex_map2_3d,
    itbl_novex_map2_3e,
    itbl_novex_map2_3f,
    itbl_novex_map2_40,
    itbl_novex_map2_41,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_80,
    itbl_novex_map2_81,
    itbl_novex_map2_82,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_8a,
    itbl_novex_map2_8b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_c8,
    itbl_novex_map2_c9,
    itbl_novex_map2_ca,
    itbl_novex_map2_cb,
    itbl_novex_map2_cc,
    itbl_novex_map2_cd,
    NULL,
    itbl_novex_map2_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_d8,
    NULL,
    NULL,
    itbl_novex_map2_db,
    itbl_novex_map2_dc,
    itbl_novex_map2_dd,
    itbl_novex_map2_de,
    itbl_novex_map2_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_f0,
    itbl_novex_map2_f1,
    NULL,
    NULL,
    NULL,
    itbl_novex_map2_f5,
    itbl_novex_map2_f6,
    NULL,
    itbl_novex_map2_f8,
    itbl_novex_map2_f9,
    itbl_novex_map2_fa,
    itbl_novex_map2_fb,
    itbl_novex_map2_fc,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_novex_map3_08[] = {
    instrux + 3050,
    NULL
};

static const struct itemplate * const itbl_novex_map3_09[] = {
    instrux + 3049,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0a[] = {
    instrux + 3052,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0b[] = {
    instrux + 3051,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0c[] = {
    instrux + 2995,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0d[] = {
    instrux + 2994,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0e[] = {
    instrux + 3010,
    NULL
};

static const struct itemplate * const itbl_novex_map3_0f[] = {
    instrux + 2962,
    instrux + 2963,
    NULL
};

static const struct itemplate * const itbl_novex_map3_14[] = {
    instrux + 3012,
    instrux + 3013,
    instrux + 3014,
    NULL
};

static const struct itemplate * const itbl_novex_map3_15[] = {
    instrux + 3017,
    instrux + 3018,
    instrux + 3019,
    NULL
};

static const struct itemplate * const itbl_novex_map3_16[] = {
    instrux + 3015,
    instrux + 3016,
    NULL
};

static const struct itemplate * const itbl_novex_map3_17[] = {
    instrux + 3002,
    instrux + 3003,
    NULL
};

static const struct itemplate * const itbl_novex_map3_20[] = {
    instrux + 3021,
    instrux + 3022,
    instrux + 3023,
    NULL
};

static const struct itemplate * const itbl_novex_map3_21[] = {
    instrux + 3004,
    NULL
};

static const struct itemplate * const itbl_novex_map3_22[] = {
    instrux + 3024,
    instrux + 3025,
    NULL
};

static const struct itemplate * const itbl_novex_map3_40[] = {
    instrux + 3001,
    NULL
};

static const struct itemplate * const itbl_novex_map3_41[] = {
    instrux + 3000,
    NULL
};

static const struct itemplate * const itbl_novex_map3_42[] = {
    instrux + 3006,
    NULL
};

static const struct itemplate * const itbl_novex_map3_44[] = {
    instrux + 3804,
    instrux + 3805,
    instrux + 3806,
    instrux + 3807,
    instrux + 3808,
    NULL
};

static const struct itemplate * const itbl_novex_map3_60[] = {
    instrux + 3070,
    NULL
};

static const struct itemplate * const itbl_novex_map3_61[] = {
    instrux + 3069,
    NULL
};

static const struct itemplate * const itbl_novex_map3_62[] = {
    instrux + 3072,
    NULL
};

static const struct itemplate * const itbl_novex_map3_63[] = {
    instrux + 3071,
    NULL
};

static const struct itemplate * const itbl_novex_map3_cc[] = {
    instrux + 4404,
    NULL
};

static const struct itemplate * const itbl_novex_map3_ce[] = {
    instrux + 6726,
    NULL
};

static const struct itemplate * const itbl_novex_map3_cf[] = {
    instrux + 6720,
    NULL
};

static const struct itemplate * const itbl_novex_map3_df[] = {
    instrux + 3088,
    NULL
};

static const struct itemplate * const itbl_novex_map3_f0[] = {
    instrux + 7203,
    NULL
};

static const struct itemplate * const * const itbl_novex_map3[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_08,
    itbl_novex_map3_09,
    itbl_novex_map3_0a,
    itbl_novex_map3_0b,
    itbl_novex_map3_0c,
    itbl_novex_map3_0d,
    itbl_novex_map3_0e,
    itbl_novex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_14,
    itbl_novex_map3_15,
    itbl_novex_map3_16,
    itbl_novex_map3_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_20,
    itbl_novex_map3_21,
    itbl_novex_map3_22,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_40,
    itbl_novex_map3_41,
    itbl_novex_map3_42,
    NULL,
    itbl_novex_map3_44,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_60,
    itbl_novex_map3_61,
    itbl_novex_map3_62,
    itbl_novex_map3_63,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_cc,
    NULL,
    itbl_novex_map3_ce,
    itbl_novex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_novex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map1_10[] = {
    instrux + 3572,
    instrux + 3573,
    instrux + 3580,
    instrux + 3581,
    instrux + 3584,
    instrux + 3586,
    instrux + 3588,
    instrux + 3590,
    NULL
};

static const struct itemplate * const itbl_vex_map1_11[] = {
    instrux + 3574,
    instrux + 3575,
    instrux + 3582,
    instrux + 3583,
    instrux + 3585,
    instrux + 3587,
    instrux + 3589,
    instrux + 3591,
    NULL
};

static const struct itemplate * const itbl_vex_map1_12[] = {
    instrux + 3532,
    instrux + 3533,
    instrux + 3546,
    instrux + 3552,
    instrux + 3554,
    instrux + 3578,
    instrux + 3579,
    NULL
};

static const struct itemplate * const itbl_vex_map1_13[] = {
    instrux + 3553,
    instrux + 3555,
    NULL
};

static const struct itemplate * const itbl_vex_map1_14[] = {
    instrux + 3794,
    instrux + 3795,
    instrux + 3796,
    instrux + 3797,
    NULL
};

static const struct itemplate * const itbl_vex_map1_15[] = {
    instrux + 3790,
    instrux + 3791,
    instrux + 3792,
    instrux + 3793,
    NULL
};

static const struct itemplate * const itbl_vex_map1_16[] = {
    instrux + 3547,
    instrux + 3549,
    instrux + 3551,
    instrux + 3576,
    instrux + 3577,
    NULL
};

static const struct itemplate * const itbl_vex_map1_17[] = {
    instrux + 3548,
    instrux + 3550,
    NULL
};

static const struct itemplate * const itbl_vex_map1_28[] = {
    instrux + 3518,
    instrux + 3520,
    instrux + 3522,
    instrux + 3524,
    NULL
};

static const struct itemplate * const itbl_vex_map1_29[] = {
    instrux + 3519,
    instrux + 3521,
    instrux + 3523,
    instrux + 3525,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2a[] = {
    instrux + 3455,
    instrux + 3456,
    instrux + 3457,
    instrux + 3458,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2b[] = {
    instrux + 3568,
    instrux + 3569,
    instrux + 3570,
    instrux + 3571,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2c[] = {
    instrux + 3468,
    instrux + 3469,
    instrux + 3470,
    instrux + 3471,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2d[] = {
    instrux + 3452,
    instrux + 3453,
    instrux + 3460,
    instrux + 3461,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2e[] = {
    instrux + 3788,
    instrux + 3789,
    NULL
};

static const struct itemplate * const itbl_vex_map1_2f[] = {
    instrux + 3434,
    instrux + 3435,
    NULL
};

static const struct itemplate * const itbl_vex_map1_41[] = {
    instrux + 4517,
    instrux + 4519,
    instrux + 4521,
    instrux + 4523,
    NULL
};

static const struct itemplate * const itbl_vex_map1_42[] = {
    instrux + 4525,
    instrux + 4527,
    instrux + 4529,
    instrux + 4531,
    NULL
};

static const struct itemplate * const itbl_vex_map1_44[] = {
    instrux + 4533,
    instrux + 4535,
    instrux + 4537,
    instrux + 4539,
    NULL
};

static const struct itemplate * const itbl_vex_map1_45[] = {
    instrux + 4541,
    instrux + 4543,
    instrux + 4545,
    instrux + 4547,
    NULL
};

static const struct itemplate * const itbl_vex_map1_46[] = {
    instrux + 4587,
    instrux + 4589,
    instrux + 4591,
    instrux + 4593,
    NULL
};

static const struct itemplate * const itbl_vex_map1_47[] = {
    instrux + 4595,
    instrux + 4597,
    instrux + 4599,
    instrux + 4601,
    NULL
};

static const struct itemplate * const itbl_vex_map1_4a[] = {
    instrux + 4509,
    instrux + 4511,
    instrux + 4513,
    instrux + 4515,
    NULL
};

static const struct itemplate * const itbl_vex_map1_4b[] = {
    instrux + 4581,
    instrux + 4583,
    instrux + 4585,
    NULL
};

static const struct itemplate * const itbl_vex_map1_50[] = {
    instrux + 3556,
    instrux + 3557,
    instrux + 3558,
    instrux + 3559,
    instrux + 3560,
    instrux + 3561,
    instrux + 3562,
    instrux + 3563,
    NULL
};

static const struct itemplate * const itbl_vex_map1_51[] = {
    instrux + 3771,
    instrux + 3772,
    instrux + 3773,
    instrux + 3774,
    instrux + 3775,
    instrux + 3776,
    NULL
};

static const struct itemplate * const itbl_vex_map1_52[] = {
    instrux + 3758,
    instrux + 3759,
    instrux + 3760,
    NULL
};

static const struct itemplate * const itbl_vex_map1_53[] = {
    instrux + 3755,
    instrux + 3756,
    instrux + 3757,
    NULL
};

static const struct itemplate * const itbl_vex_map1_54[] = {
    instrux + 3132,
    instrux + 3133,
    instrux + 3134,
    instrux + 3135,
    NULL
};

static const struct itemplate * const itbl_vex_map1_55[] = {
    instrux + 3136,
    instrux + 3137,
    instrux + 3138,
    instrux + 3139,
    NULL
};

static const struct itemplate * const itbl_vex_map1_56[] = {
    instrux + 3599,
    instrux + 3600,
    instrux + 3601,
    instrux + 3602,
    NULL
};

static const struct itemplate * const itbl_vex_map1_57[] = {
    instrux + 3798,
    instrux + 3799,
    instrux + 3800,
    instrux + 3801,
    NULL
};

static const struct itemplate * const itbl_vex_map1_58[] = {
    instrux + 3122,
    instrux + 3123,
    instrux + 3124,
    instrux + 3125,
    instrux + 3126,
    instrux + 3127,
    NULL
};

static const struct itemplate * const itbl_vex_map1_59[] = {
    instrux + 3593,
    instrux + 3594,
    instrux + 3595,
    instrux + 3596,
    instrux + 3597,
    instrux + 3598,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5a[] = {
    instrux + 3444,
    instrux + 3445,
    instrux + 3446,
    instrux + 3447,
    instrux + 3450,
    instrux + 3451,
    instrux + 3454,
    instrux + 3459,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5b[] = {
    instrux + 3438,
    instrux + 3439,
    instrux + 3448,
    instrux + 3449,
    instrux + 3466,
    instrux + 3467,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5c[] = {
    instrux + 3778,
    instrux + 3779,
    instrux + 3780,
    instrux + 3781,
    instrux + 3782,
    instrux + 3783,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5d[] = {
    instrux + 3512,
    instrux + 3513,
    instrux + 3514,
    instrux + 3515,
    instrux + 3516,
    instrux + 3517,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5e[] = {
    instrux + 3472,
    instrux + 3473,
    instrux + 3474,
    instrux + 3475,
    instrux + 3476,
    instrux + 3477,
    NULL
};

static const struct itemplate * const itbl_vex_map1_5f[] = {
    instrux + 3506,
    instrux + 3507,
    instrux + 3508,
    instrux + 3509,
    instrux + 3510,
    instrux + 3511,
    NULL
};

static const struct itemplate * const itbl_vex_map1_60[] = {
    instrux + 3750,
    instrux + 4303,
    NULL
};

static const struct itemplate * const itbl_vex_map1_61[] = {
    instrux + 3751,
    instrux + 4304,
    NULL
};

static const struct itemplate * const itbl_vex_map1_62[] = {
    instrux + 3752,
    instrux + 4305,
    NULL
};

static const struct itemplate * const itbl_vex_map1_63[] = {
    instrux + 3606,
    instrux + 4190,
    NULL
};

static const struct itemplate * const itbl_vex_map1_64[] = {
    instrux + 3633,
    instrux + 4213,
    NULL
};

static const struct itemplate * const itbl_vex_map1_65[] = {
    instrux + 3634,
    instrux + 4214,
    NULL
};

static const struct itemplate * const itbl_vex_map1_66[] = {
    instrux + 3635,
    instrux + 4215,
    NULL
};

static const struct itemplate * const itbl_vex_map1_67[] = {
    instrux + 3608,
    instrux + 4193,
    NULL
};

static const struct itemplate * const itbl_vex_map1_68[] = {
    instrux + 3746,
    instrux + 4299,
    NULL
};

static const struct itemplate * const itbl_vex_map1_69[] = {
    instrux + 3747,
    instrux + 4300,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6a[] = {
    instrux + 3748,
    instrux + 4301,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6b[] = {
    instrux + 3607,
    instrux + 4191,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6c[] = {
    instrux + 3753,
    instrux + 4306,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6d[] = {
    instrux + 3749,
    instrux + 4302,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6e[] = {
    instrux + 3526,
    instrux + 3530,
    NULL
};

static const struct itemplate * const itbl_vex_map1_6f[] = {
    instrux + 3534,
    instrux + 3536,
    instrux + 3538,
    instrux + 3540,
    instrux + 3542,
    instrux + 3544,
    NULL
};

static const struct itemplate * const itbl_vex_map1_70[] = {
    instrux + 3712,
    instrux + 3713,
    instrux + 3714,
    instrux + 4267,
    instrux + 4268,
    instrux + 4269,
    NULL
};

static const struct itemplate * const itbl_vex_map1_71[] = {
    instrux + 3721,
    instrux + 3727,
    instrux + 3731,
    instrux + 4275,
    instrux + 4281,
    instrux + 4286,
    NULL
};

static const struct itemplate * const itbl_vex_map1_72[] = {
    instrux + 3723,
    instrux + 3729,
    instrux + 3733,
    instrux + 4277,
    instrux + 4283,
    instrux + 4288,
    NULL
};

static const struct itemplate * const itbl_vex_map1_73[] = {
    instrux + 3718,
    instrux + 3719,
    instrux + 3725,
    instrux + 3735,
    instrux + 4273,
    instrux + 4279,
    instrux + 4284,
    instrux + 4290,
    NULL
};

static const struct itemplate * const itbl_vex_map1_74[] = {
    instrux + 3629,
    instrux + 4209,
    NULL
};

static const struct itemplate * const itbl_vex_map1_75[] = {
    instrux + 3630,
    instrux + 4210,
    NULL
};

static const struct itemplate * const itbl_vex_map1_76[] = {
    instrux + 3631,
    instrux + 4211,
    NULL
};

static const struct itemplate * const itbl_vex_map1_77[] = {
    instrux + 3802,
    instrux + 3803,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7c[] = {
    instrux + 3483,
    instrux + 3484,
    instrux + 3485,
    instrux + 3486,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7d[] = {
    instrux + 3487,
    instrux + 3488,
    instrux + 3489,
    instrux + 3490,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7e[] = {
    instrux + 3527,
    instrux + 3528,
    instrux + 3531,
    NULL
};

static const struct itemplate * const itbl_vex_map1_7f[] = {
    instrux + 3535,
    instrux + 3537,
    instrux + 3539,
    instrux + 3541,
    instrux + 3543,
    instrux + 3545,
    NULL
};

static const struct itemplate * const itbl_vex_map1_90[] = {
    instrux + 4477,
    instrux + 4479,
    instrux + 4481,
    instrux + 4483,
    NULL
};

static const struct itemplate * const itbl_vex_map1_91[] = {
    instrux + 4485,
    instrux + 4487,
    instrux + 4489,
    instrux + 4491,
    NULL
};

static const struct itemplate * const itbl_vex_map1_92[] = {
    instrux + 4493,
    instrux + 4495,
    instrux + 4501,
    instrux + 4503,
    NULL
};

static const struct itemplate * const itbl_vex_map1_93[] = {
    instrux + 4497,
    instrux + 4499,
    instrux + 4505,
    instrux + 4507,
    NULL
};

static const struct itemplate * const itbl_vex_map1_98[] = {
    instrux + 4549,
    instrux + 4551,
    instrux + 4553,
    instrux + 4555,
    NULL
};

static const struct itemplate * const itbl_vex_map1_99[] = {
    instrux + 4573,
    instrux + 4575,
    instrux + 4577,
    instrux + 4579,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ae[] = {
    instrux + 3496,
    instrux + 3777,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c2[] = {
    instrux + 3152,
    instrux + 3153,
    instrux + 3154,
    instrux + 3155,
    instrux + 3156,
    instrux + 3157,
    instrux + 3158,
    instrux + 3159,
    instrux + 3160,
    instrux + 3161,
    instrux + 3162,
    instrux + 3163,
    instrux + 3164,
    instrux + 3165,
    instrux + 3166,
    instrux + 3167,
    instrux + 3168,
    instrux + 3169,
    instrux + 3170,
    instrux + 3171,
    instrux + 3172,
    instrux + 3173,
    instrux + 3174,
    instrux + 3175,
    instrux + 3176,
    instrux + 3177,
    instrux + 3178,
    instrux + 3179,
    instrux + 3180,
    instrux + 3181,
    instrux + 3182,
    instrux + 3183,
    instrux + 3184,
    instrux + 3185,
    instrux + 3186,
    instrux + 3187,
    instrux + 3188,
    instrux + 3189,
    instrux + 3190,
    instrux + 3191,
    instrux + 3192,
    instrux + 3193,
    instrux + 3194,
    instrux + 3195,
    instrux + 3196,
    instrux + 3197,
    instrux + 3198,
    instrux + 3199,
    instrux + 3200,
    instrux + 3201,
    instrux + 3202,
    instrux + 3203,
    instrux + 3204,
    instrux + 3205,
    instrux + 3206,
    instrux + 3207,
    instrux + 3208,
    instrux + 3209,
    instrux + 3210,
    instrux + 3211,
    instrux + 3212,
    instrux + 3213,
    instrux + 3214,
    instrux + 3215,
    instrux + 3216,
    instrux + 3217,
    instrux + 3218,
    instrux + 3219,
    instrux + 3220,
    instrux + 3221,
    instrux + 3222,
    instrux + 3223,
    instrux + 3224,
    instrux + 3225,
    instrux + 3226,
    instrux + 3227,
    instrux + 3228,
    instrux + 3229,
    instrux + 3230,
    instrux + 3231,
    instrux + 3232,
    instrux + 3233,
    instrux + 3234,
    instrux + 3235,
    instrux + 3236,
    instrux + 3237,
    instrux + 3238,
    instrux + 3239,
    instrux + 3240,
    instrux + 3241,
    instrux + 3242,
    instrux + 3243,
    instrux + 3244,
    instrux + 3245,
    instrux + 3246,
    instrux + 3247,
    instrux + 3248,
    instrux + 3249,
    instrux + 3250,
    instrux + 3251,
    instrux + 3252,
    instrux + 3253,
    instrux + 3254,
    instrux + 3255,
    instrux + 3256,
    instrux + 3257,
    instrux + 3258,
    instrux + 3259,
    instrux + 3260,
    instrux + 3261,
    instrux + 3262,
    instrux + 3263,
    instrux + 3264,
    instrux + 3265,
    instrux + 3266,
    instrux + 3267,
    instrux + 3268,
    instrux + 3269,
    instrux + 3270,
    instrux + 3271,
    instrux + 3272,
    instrux + 3273,
    instrux + 3274,
    instrux + 3275,
    instrux + 3276,
    instrux + 3277,
    instrux + 3278,
    instrux + 3279,
    instrux + 3280,
    instrux + 3281,
    instrux + 3282,
    instrux + 3283,
    instrux + 3284,
    instrux + 3285,
    instrux + 3286,
    instrux + 3287,
    instrux + 3288,
    instrux + 3289,
    instrux + 3290,
    instrux + 3291,
    instrux + 3292,
    instrux + 3293,
    instrux + 3294,
    instrux + 3295,
    instrux + 3296,
    instrux + 3297,
    instrux + 3298,
    instrux + 3299,
    instrux + 3300,
    instrux + 3301,
    instrux + 3302,
    instrux + 3303,
    instrux + 3304,
    instrux + 3305,
    instrux + 3306,
    instrux + 3307,
    instrux + 3308,
    instrux + 3309,
    instrux + 3310,
    instrux + 3311,
    instrux + 3312,
    instrux + 3313,
    instrux + 3314,
    instrux + 3315,
    instrux + 3316,
    instrux + 3317,
    instrux + 3318,
    instrux + 3319,
    instrux + 3320,
    instrux + 3321,
    instrux + 3322,
    instrux + 3323,
    instrux + 3324,
    instrux + 3325,
    instrux + 3326,
    instrux + 3327,
    instrux + 3328,
    instrux + 3329,
    instrux + 3330,
    instrux + 3331,
    instrux + 3332,
    instrux + 3333,
    instrux + 3334,
    instrux + 3335,
    instrux + 3336,
    instrux + 3337,
    instrux + 3338,
    instrux + 3339,
    instrux + 3340,
    instrux + 3341,
    instrux + 3342,
    instrux + 3343,
    instrux + 3344,
    instrux + 3345,
    instrux + 3346,
    instrux + 3347,
    instrux + 3348,
    instrux + 3349,
    instrux + 3350,
    instrux + 3351,
    instrux + 3352,
    instrux + 3353,
    instrux + 3354,
    instrux + 3355,
    instrux + 3356,
    instrux + 3357,
    instrux + 3358,
    instrux + 3359,
    instrux + 3360,
    instrux + 3361,
    instrux + 3362,
    instrux + 3363,
    instrux + 3364,
    instrux + 3365,
    instrux + 3366,
    instrux + 3367,
    instrux + 3368,
    instrux + 3369,
    instrux + 3370,
    instrux + 3371,
    instrux + 3372,
    instrux + 3373,
    instrux + 3374,
    instrux + 3375,
    instrux + 3376,
    instrux + 3377,
    instrux + 3378,
    instrux + 3379,
    instrux + 3380,
    instrux + 3381,
    instrux + 3382,
    instrux + 3383,
    instrux + 3384,
    instrux + 3385,
    instrux + 3386,
    instrux + 3387,
    instrux + 3388,
    instrux + 3389,
    instrux + 3390,
    instrux + 3391,
    instrux + 3392,
    instrux + 3393,
    instrux + 3394,
    instrux + 3395,
    instrux + 3396,
    instrux + 3397,
    instrux + 3398,
    instrux + 3399,
    instrux + 3400,
    instrux + 3401,
    instrux + 3402,
    instrux + 3403,
    instrux + 3404,
    instrux + 3405,
    instrux + 3406,
    instrux + 3407,
    instrux + 3408,
    instrux + 3409,
    instrux + 3410,
    instrux + 3411,
    instrux + 3412,
    instrux + 3413,
    instrux + 3414,
    instrux + 3415,
    instrux + 3416,
    instrux + 3417,
    instrux + 3418,
    instrux + 3419,
    instrux + 3420,
    instrux + 3421,
    instrux + 3422,
    instrux + 3423,
    instrux + 3424,
    instrux + 3425,
    instrux + 3426,
    instrux + 3427,
    instrux + 3428,
    instrux + 3429,
    instrux + 3430,
    instrux + 3431,
    instrux + 3432,
    instrux + 3433,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c4[] = {
    instrux + 3667,
    instrux + 3668,
    instrux + 3669,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c5[] = {
    instrux + 3649,
    instrux + 3650,
    NULL
};

static const struct itemplate * const itbl_vex_map1_c6[] = {
    instrux + 3767,
    instrux + 3768,
    instrux + 3769,
    instrux + 3770,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d0[] = {
    instrux + 3128,
    instrux + 3129,
    instrux + 3130,
    instrux + 3131,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d1[] = {
    instrux + 3730,
    instrux + 4285,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d2[] = {
    instrux + 3732,
    instrux + 4287,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d3[] = {
    instrux + 3734,
    instrux + 4289,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d4[] = {
    instrux + 3613,
    instrux + 4197,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d5[] = {
    instrux + 3705,
    instrux + 4261,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d6[] = {
    instrux + 3529,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d7[] = {
    instrux + 3688,
    instrux + 3689,
    instrux + 4237,
    instrux + 4238,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d8[] = {
    instrux + 3744,
    instrux + 4297,
    NULL
};

static const struct itemplate * const itbl_vex_map1_d9[] = {
    instrux + 3745,
    instrux + 4298,
    NULL
};

static const struct itemplate * const itbl_vex_map1_da[] = {
    instrux + 3685,
    instrux + 4234,
    NULL
};

static const struct itemplate * const itbl_vex_map1_db[] = {
    instrux + 3619,
    instrux + 4203,
    NULL
};

static const struct itemplate * const itbl_vex_map1_dc[] = {
    instrux + 3616,
    instrux + 4200,
    NULL
};

static const struct itemplate * const itbl_vex_map1_dd[] = {
    instrux + 3617,
    instrux + 4201,
    NULL
};

static const struct itemplate * const itbl_vex_map1_de[] = {
    instrux + 3679,
    instrux + 4228,
    NULL
};

static const struct itemplate * const itbl_vex_map1_df[] = {
    instrux + 3620,
    instrux + 4204,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e0[] = {
    instrux + 3621,
    instrux + 4205,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e1[] = {
    instrux + 3726,
    instrux + 4280,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e2[] = {
    instrux + 3728,
    instrux + 4282,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e3[] = {
    instrux + 3622,
    instrux + 4206,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e4[] = {
    instrux + 3702,
    instrux + 4259,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e5[] = {
    instrux + 3704,
    instrux + 4260,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e6[] = {
    instrux + 3436,
    instrux + 3437,
    instrux + 3440,
    instrux + 3441,
    instrux + 3442,
    instrux + 3443,
    instrux + 3462,
    instrux + 3463,
    instrux + 3464,
    instrux + 3465,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e7[] = {
    instrux + 3564,
    instrux + 3565,
    instrux + 3566,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e8[] = {
    instrux + 3742,
    instrux + 4295,
    NULL
};

static const struct itemplate * const itbl_vex_map1_e9[] = {
    instrux + 3743,
    instrux + 4296,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ea[] = {
    instrux + 3683,
    instrux + 4232,
    NULL
};

static const struct itemplate * const itbl_vex_map1_eb[] = {
    instrux + 3709,
    instrux + 4264,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ec[] = {
    instrux + 3614,
    instrux + 4198,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ed[] = {
    instrux + 3615,
    instrux + 4199,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ee[] = {
    instrux + 3677,
    instrux + 4226,
    NULL
};

static const struct itemplate * const itbl_vex_map1_ef[] = {
    instrux + 3754,
    instrux + 4307,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f0[] = {
    instrux + 3493,
    instrux + 3494,
    instrux + 3495,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f1[] = {
    instrux + 3720,
    instrux + 4274,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f2[] = {
    instrux + 3722,
    instrux + 4276,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f3[] = {
    instrux + 3724,
    instrux + 4278,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f4[] = {
    instrux + 3707,
    instrux + 4263,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f5[] = {
    instrux + 3674,
    instrux + 4224,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f6[] = {
    instrux + 3710,
    instrux + 4265,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f7[] = {
    instrux + 3497,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f8[] = {
    instrux + 3738,
    instrux + 4291,
    NULL
};

static const struct itemplate * const itbl_vex_map1_f9[] = {
    instrux + 3739,
    instrux + 4292,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fa[] = {
    instrux + 3740,
    instrux + 4293,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fb[] = {
    instrux + 3741,
    instrux + 4294,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fc[] = {
    instrux + 3610,
    instrux + 4194,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fd[] = {
    instrux + 3611,
    instrux + 4195,
    NULL
};

static const struct itemplate * const itbl_vex_map1_fe[] = {
    instrux + 3612,
    instrux + 4196,
    NULL
};

static const struct itemplate * const * const itbl_vex_map1[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_10,
    itbl_vex_map1_11,
    itbl_vex_map1_12,
    itbl_vex_map1_13,
    itbl_vex_map1_14,
    itbl_vex_map1_15,
    itbl_vex_map1_16,
    itbl_vex_map1_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_28,
    itbl_vex_map1_29,
    itbl_vex_map1_2a,
    itbl_vex_map1_2b,
    itbl_vex_map1_2c,
    itbl_vex_map1_2d,
    itbl_vex_map1_2e,
    itbl_vex_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_41,
    itbl_vex_map1_42,
    NULL,
    itbl_vex_map1_44,
    itbl_vex_map1_45,
    itbl_vex_map1_46,
    itbl_vex_map1_47,
    NULL,
    NULL,
    itbl_vex_map1_4a,
    itbl_vex_map1_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_50,
    itbl_vex_map1_51,
    itbl_vex_map1_52,
    itbl_vex_map1_53,
    itbl_vex_map1_54,
    itbl_vex_map1_55,
    itbl_vex_map1_56,
    itbl_vex_map1_57,
    itbl_vex_map1_58,
    itbl_vex_map1_59,
    itbl_vex_map1_5a,
    itbl_vex_map1_5b,
    itbl_vex_map1_5c,
    itbl_vex_map1_5d,
    itbl_vex_map1_5e,
    itbl_vex_map1_5f,
    itbl_vex_map1_60,
    itbl_vex_map1_61,
    itbl_vex_map1_62,
    itbl_vex_map1_63,
    itbl_vex_map1_64,
    itbl_vex_map1_65,
    itbl_vex_map1_66,
    itbl_vex_map1_67,
    itbl_vex_map1_68,
    itbl_vex_map1_69,
    itbl_vex_map1_6a,
    itbl_vex_map1_6b,
    itbl_vex_map1_6c,
    itbl_vex_map1_6d,
    itbl_vex_map1_6e,
    itbl_vex_map1_6f,
    itbl_vex_map1_70,
    itbl_vex_map1_71,
    itbl_vex_map1_72,
    itbl_vex_map1_73,
    itbl_vex_map1_74,
    itbl_vex_map1_75,
    itbl_vex_map1_76,
    itbl_vex_map1_77,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_7c,
    itbl_vex_map1_7d,
    itbl_vex_map1_7e,
    itbl_vex_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_90,
    itbl_vex_map1_91,
    itbl_vex_map1_92,
    itbl_vex_map1_93,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_98,
    itbl_vex_map1_99,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_ae,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_c2,
    NULL,
    itbl_vex_map1_c4,
    itbl_vex_map1_c5,
    itbl_vex_map1_c6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map1_d0,
    itbl_vex_map1_d1,
    itbl_vex_map1_d2,
    itbl_vex_map1_d3,
    itbl_vex_map1_d4,
    itbl_vex_map1_d5,
    itbl_vex_map1_d6,
    itbl_vex_map1_d7,
    itbl_vex_map1_d8,
    itbl_vex_map1_d9,
    itbl_vex_map1_da,
    itbl_vex_map1_db,
    itbl_vex_map1_dc,
    itbl_vex_map1_dd,
    itbl_vex_map1_de,
    itbl_vex_map1_df,
    itbl_vex_map1_e0,
    itbl_vex_map1_e1,
    itbl_vex_map1_e2,
    itbl_vex_map1_e3,
    itbl_vex_map1_e4,
    itbl_vex_map1_e5,
    itbl_vex_map1_e6,
    itbl_vex_map1_e7,
    itbl_vex_map1_e8,
    itbl_vex_map1_e9,
    itbl_vex_map1_ea,
    itbl_vex_map1_eb,
    itbl_vex_map1_ec,
    itbl_vex_map1_ed,
    itbl_vex_map1_ee,
    itbl_vex_map1_ef,
    itbl_vex_map1_f0,
    itbl_vex_map1_f1,
    itbl_vex_map1_f2,
    itbl_vex_map1_f3,
    itbl_vex_map1_f4,
    itbl_vex_map1_f5,
    itbl_vex_map1_f6,
    itbl_vex_map1_f7,
    itbl_vex_map1_f8,
    itbl_vex_map1_f9,
    itbl_vex_map1_fa,
    itbl_vex_map1_fb,
    itbl_vex_map1_fc,
    itbl_vex_map1_fd,
    itbl_vex_map1_fe,
    NULL,
};

static const struct itemplate * const itbl_vex_map2_00[] = {
    instrux + 3711,
    instrux + 4266,
    NULL
};

static const struct itemplate * const itbl_vex_map2_01[] = {
    instrux + 3657,
    instrux + 4217,
    NULL
};

static const struct itemplate * const itbl_vex_map2_02[] = {
    instrux + 3658,
    instrux + 4218,
    NULL
};

static const struct itemplate * const itbl_vex_map2_03[] = {
    instrux + 3659,
    instrux + 4219,
    NULL
};

static const struct itemplate * const itbl_vex_map2_04[] = {
    instrux + 3675,
    instrux + 4223,
    NULL
};

static const struct itemplate * const itbl_vex_map2_05[] = {
    instrux + 3661,
    instrux + 4220,
    NULL
};

static const struct itemplate * const itbl_vex_map2_06[] = {
    instrux + 3662,
    instrux + 4221,
    NULL
};

static const struct itemplate * const itbl_vex_map2_07[] = {
    instrux + 3663,
    instrux + 4222,
    NULL
};

static const struct itemplate * const itbl_vex_map2_08[] = {
    instrux + 3715,
    instrux + 4270,
    NULL
};

static const struct itemplate * const itbl_vex_map2_09[] = {
    instrux + 3716,
    instrux + 4271,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0a[] = {
    instrux + 3717,
    instrux + 4272,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0b[] = {
    instrux + 3703,
    instrux + 4258,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0c[] = {
    instrux + 3641,
    instrux + 3642,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0d[] = {
    instrux + 3637,
    instrux + 3638,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0e[] = {
    instrux + 3784,
    instrux + 3785,
    NULL
};

static const struct itemplate * const itbl_vex_map2_0f[] = {
    instrux + 3786,
    instrux + 3787,
    NULL
};

static const struct itemplate * const itbl_vex_map2_13[] = {
    instrux + 4026,
    instrux + 4027,
    NULL
};

static const struct itemplate * const itbl_vex_map2_16[] = {
    instrux + 4333,
    NULL
};

static const struct itemplate * const itbl_vex_map2_17[] = {
    instrux + 3736,
    instrux + 3737,
    NULL
};

static const struct itemplate * const itbl_vex_map2_18[] = {
    instrux + 3148,
    instrux + 3149,
    instrux + 4309,
    instrux + 4310,
    NULL
};

static const struct itemplate * const itbl_vex_map2_19[] = {
    instrux + 3150,
    instrux + 4311,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1a[] = {
    instrux + 3151,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1c[] = {
    instrux + 3603,
    instrux + 4187,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1d[] = {
    instrux + 3604,
    instrux + 4188,
    NULL
};

static const struct itemplate * const itbl_vex_map2_1e[] = {
    instrux + 3605,
    instrux + 4189,
    NULL
};

static const struct itemplate * const itbl_vex_map2_20[] = {
    instrux + 3690,
    instrux + 4239,
    NULL
};

static const struct itemplate * const itbl_vex_map2_21[] = {
    instrux + 3691,
    instrux + 4240,
    instrux + 4241,
    NULL
};

static const struct itemplate * const itbl_vex_map2_22[] = {
    instrux + 3692,
    instrux + 4242,
    instrux + 4243,
    NULL
};

static const struct itemplate * const itbl_vex_map2_23[] = {
    instrux + 3693,
    instrux + 4244,
    NULL
};

static const struct itemplate * const itbl_vex_map2_24[] = {
    instrux + 3694,
    instrux + 4245,
    instrux + 4246,
    NULL
};

static const struct itemplate * const itbl_vex_map2_25[] = {
    instrux + 3695,
    instrux + 4247,
    NULL
};

static const struct itemplate * const itbl_vex_map2_28[] = {
    instrux + 3708,
    instrux + 4257,
    NULL
};

static const struct itemplate * const itbl_vex_map2_29[] = {
    instrux + 3632,
    instrux + 4212,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2a[] = {
    instrux + 3567,
    instrux + 4308,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2b[] = {
    instrux + 3609,
    instrux + 4192,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2c[] = {
    instrux + 3498,
    instrux + 3499,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2d[] = {
    instrux + 3502,
    instrux + 3503,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2e[] = {
    instrux + 3500,
    instrux + 3501,
    NULL
};

static const struct itemplate * const itbl_vex_map2_2f[] = {
    instrux + 3504,
    instrux + 3505,
    NULL
};

static const struct itemplate * const itbl_vex_map2_30[] = {
    instrux + 3696,
    instrux + 4248,
    NULL
};

static const struct itemplate * const itbl_vex_map2_31[] = {
    instrux + 3697,
    instrux + 4249,
    instrux + 4250,
    NULL
};

static const struct itemplate * const itbl_vex_map2_32[] = {
    instrux + 3698,
    instrux + 4251,
    instrux + 4252,
    NULL
};

static const struct itemplate * const itbl_vex_map2_33[] = {
    instrux + 3699,
    instrux + 4253,
    NULL
};

static const struct itemplate * const itbl_vex_map2_34[] = {
    instrux + 3700,
    instrux + 4254,
    instrux + 4255,
    NULL
};

static const struct itemplate * const itbl_vex_map2_35[] = {
    instrux + 3701,
    instrux + 4256,
    NULL
};

static const struct itemplate * const itbl_vex_map2_36[] = {
    instrux + 4331,
    NULL
};

static const struct itemplate * const itbl_vex_map2_37[] = {
    instrux + 3636,
    instrux + 4216,
    NULL
};

static const struct itemplate * const itbl_vex_map2_38[] = {
    instrux + 3682,
    instrux + 4231,
    NULL
};

static const struct itemplate * const itbl_vex_map2_39[] = {
    instrux + 3684,
    instrux + 4233,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3a[] = {
    instrux + 3686,
    instrux + 4235,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3b[] = {
    instrux + 3687,
    instrux + 4236,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3c[] = {
    instrux + 3676,
    instrux + 4225,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3d[] = {
    instrux + 3678,
    instrux + 4227,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3e[] = {
    instrux + 3680,
    instrux + 4229,
    NULL
};

static const struct itemplate * const itbl_vex_map2_3f[] = {
    instrux + 3681,
    instrux + 4230,
    NULL
};

static const struct itemplate * const itbl_vex_map2_40[] = {
    instrux + 3706,
    instrux + 4262,
    NULL
};

static const struct itemplate * const itbl_vex_map2_41[] = {
    instrux + 3660,
    NULL
};

static const struct itemplate * const itbl_vex_map2_45[] = {
    instrux + 4352,
    instrux + 4353,
    instrux + 4354,
    instrux + 4355,
    NULL
};

static const struct itemplate * const itbl_vex_map2_46[] = {
    instrux + 4350,
    instrux + 4351,
    NULL
};

static const struct itemplate * const itbl_vex_map2_47[] = {
    instrux + 4346,
    instrux + 4347,
    instrux + 4348,
    instrux + 4349,
    NULL
};

static const struct itemplate * const itbl_vex_map2_48[] = {
    instrux + 6922,
    instrux + 6927,
    NULL
};

static const struct itemplate * const itbl_vex_map2_49[] = {
    instrux + 6872,
    instrux + 6874,
    instrux + 6888,
    instrux + 6891,
    NULL
};

static const struct itemplate * const itbl_vex_map2_4a[] = {
    instrux + 6892,
    instrux + 6894,
    NULL
};

static const struct itemplate * const itbl_vex_map2_4b[] = {
    instrux + 6884,
    instrux + 6886,
    instrux + 6889,
    NULL
};

static const struct itemplate * const itbl_vex_map2_50[] = {
    instrux + 4441,
    instrux + 4442,
    instrux + 4449,
    instrux + 4450,
    instrux + 4453,
    instrux + 4454,
    instrux + 4457,
    instrux + 4458,
    NULL
};

static const struct itemplate * const itbl_vex_map2_51[] = {
    instrux + 4443,
    instrux + 4444,
    instrux + 4451,
    instrux + 4452,
    instrux + 4455,
    instrux + 4456,
    instrux + 4459,
    instrux + 4460,
    NULL
};

static const struct itemplate * const itbl_vex_map2_52[] = {
    instrux + 4445,
    instrux + 4446,
    NULL
};

static const struct itemplate * const itbl_vex_map2_53[] = {
    instrux + 4447,
    instrux + 4448,
    NULL
};

static const struct itemplate * const itbl_vex_map2_58[] = {
    instrux + 4323,
    instrux + 4324,
    instrux + 4325,
    instrux + 4326,
    NULL
};

static const struct itemplate * const itbl_vex_map2_59[] = {
    instrux + 4327,
    instrux + 4328,
    instrux + 4329,
    instrux + 4330,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5a[] = {
    instrux + 4312,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5c[] = {
    instrux + 6876,
    instrux + 6877,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5e[] = {
    instrux + 6880,
    instrux + 6881,
    instrux + 6882,
    instrux + 6883,
    NULL
};

static const struct itemplate * const itbl_vex_map2_5f[] = {
    instrux + 6928,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6b[] = {
    instrux + 6904,
    instrux + 6905,
    instrux + 6923,
    instrux + 6924,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6c[] = {
    instrux + 6878,
    instrux + 6879,
    instrux + 6925,
    instrux + 6926,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6e[] = {
    instrux + 6896,
    instrux + 6898,
    NULL
};

static const struct itemplate * const itbl_vex_map2_6f[] = {
    instrux + 6897,
    instrux + 6899,
    NULL
};

static const struct itemplate * const itbl_vex_map2_72[] = {
    instrux + 4439,
    instrux + 4440,
    NULL
};

static const struct itemplate * const itbl_vex_map2_78[] = {
    instrux + 4315,
    instrux + 4316,
    instrux + 4317,
    instrux + 4318,
    NULL
};

static const struct itemplate * const itbl_vex_map2_79[] = {
    instrux + 4319,
    instrux + 4320,
    instrux + 4321,
    instrux + 4322,
    NULL
};

static const struct itemplate * const itbl_vex_map2_8c[] = {
    instrux + 4338,
    instrux + 4339,
    instrux + 4340,
    instrux + 4341,
    NULL
};

static const struct itemplate * const itbl_vex_map2_8e[] = {
    instrux + 4342,
    instrux + 4343,
    instrux + 4344,
    instrux + 4345,
    NULL
};

static const struct itemplate * const itbl_vex_map2_90[] = {
    instrux + 4364,
    instrux + 4366,
    instrux + 4368,
    instrux + 4370,
    NULL
};

static const struct itemplate * const itbl_vex_map2_91[] = {
    instrux + 4365,
    instrux + 4367,
    instrux + 4369,
    instrux + 4371,
    NULL
};

static const struct itemplate * const itbl_vex_map2_92[] = {
    instrux + 4356,
    instrux + 4358,
    instrux + 4360,
    instrux + 4362,
    NULL
};

static const struct itemplate * const itbl_vex_map2_93[] = {
    instrux + 4357,
    instrux + 4359,
    instrux + 4361,
    instrux + 4363,
    NULL
};

static const struct itemplate * const itbl_vex_map2_96[] = {
    instrux + 3858,
    instrux + 3859,
    instrux + 3860,
    instrux + 3861,
    instrux + 3862,
    instrux + 3863,
    instrux + 3864,
    instrux + 3865,
    NULL
};

static const struct itemplate * const itbl_vex_map2_97[] = {
    instrux + 3906,
    instrux + 3907,
    instrux + 3908,
    instrux + 3909,
    instrux + 3910,
    instrux + 3911,
    instrux + 3912,
    instrux + 3913,
    NULL
};

static const struct itemplate * const itbl_vex_map2_98[] = {
    instrux + 3834,
    instrux + 3835,
    instrux + 3836,
    instrux + 3837,
    instrux + 3838,
    instrux + 3839,
    instrux + 3840,
    instrux + 3841,
    NULL
};

static const struct itemplate * const itbl_vex_map2_99[] = {
    instrux + 3978,
    instrux + 3979,
    instrux + 3980,
    instrux + 3981,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9a[] = {
    instrux + 3882,
    instrux + 3883,
    instrux + 3884,
    instrux + 3885,
    instrux + 3886,
    instrux + 3887,
    instrux + 3888,
    instrux + 3889,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9b[] = {
    instrux + 3990,
    instrux + 3991,
    instrux + 3992,
    instrux + 3993,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9c[] = {
    instrux + 3930,
    instrux + 3931,
    instrux + 3932,
    instrux + 3933,
    instrux + 3934,
    instrux + 3935,
    instrux + 3936,
    instrux + 3937,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9d[] = {
    instrux + 4002,
    instrux + 4003,
    instrux + 4004,
    instrux + 4005,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9e[] = {
    instrux + 3954,
    instrux + 3955,
    instrux + 3956,
    instrux + 3957,
    instrux + 3958,
    instrux + 3959,
    instrux + 3960,
    instrux + 3961,
    NULL
};

static const struct itemplate * const itbl_vex_map2_9f[] = {
    instrux + 4014,
    instrux + 4015,
    instrux + 4016,
    instrux + 4017,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a6[] = {
    instrux + 3866,
    instrux + 3867,
    instrux + 3868,
    instrux + 3869,
    instrux + 3870,
    instrux + 3871,
    instrux + 3872,
    instrux + 3873,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a7[] = {
    instrux + 3914,
    instrux + 3915,
    instrux + 3916,
    instrux + 3917,
    instrux + 3918,
    instrux + 3919,
    instrux + 3920,
    instrux + 3921,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a8[] = {
    instrux + 3842,
    instrux + 3843,
    instrux + 3844,
    instrux + 3845,
    instrux + 3846,
    instrux + 3847,
    instrux + 3848,
    instrux + 3849,
    NULL
};

static const struct itemplate * const itbl_vex_map2_a9[] = {
    instrux + 3982,
    instrux + 3983,
    instrux + 3984,
    instrux + 3985,
    NULL
};

static const struct itemplate * const itbl_vex_map2_aa[] = {
    instrux + 3890,
    instrux + 3891,
    instrux + 3892,
    instrux + 3893,
    instrux + 3894,
    instrux + 3895,
    instrux + 3896,
    instrux + 3897,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ab[] = {
    instrux + 3994,
    instrux + 3995,
    instrux + 3996,
    instrux + 3997,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ac[] = {
    instrux + 3938,
    instrux + 3939,
    instrux + 3940,
    instrux + 3941,
    instrux + 3942,
    instrux + 3943,
    instrux + 3944,
    instrux + 3945,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ad[] = {
    instrux + 4006,
    instrux + 4007,
    instrux + 4008,
    instrux + 4009,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ae[] = {
    instrux + 3962,
    instrux + 3963,
    instrux + 3964,
    instrux + 3965,
    instrux + 3966,
    instrux + 3967,
    instrux + 3968,
    instrux + 3969,
    NULL
};

static const struct itemplate * const itbl_vex_map2_af[] = {
    instrux + 4018,
    instrux + 4019,
    instrux + 4020,
    instrux + 4021,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b0[] = {
    instrux + 4431,
    instrux + 4432,
    instrux + 4433,
    instrux + 4434,
    instrux + 4435,
    instrux + 4436,
    instrux + 4437,
    instrux + 4438,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b1[] = {
    instrux + 4425,
    instrux + 4426,
    instrux + 4427,
    instrux + 4428,
    instrux + 4429,
    instrux + 4430,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b4[] = {
    instrux + 4475,
    instrux + 4476,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b5[] = {
    instrux + 4473,
    instrux + 4474,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b6[] = {
    instrux + 3874,
    instrux + 3875,
    instrux + 3876,
    instrux + 3877,
    instrux + 3878,
    instrux + 3879,
    instrux + 3880,
    instrux + 3881,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b7[] = {
    instrux + 3922,
    instrux + 3923,
    instrux + 3924,
    instrux + 3925,
    instrux + 3926,
    instrux + 3927,
    instrux + 3928,
    instrux + 3929,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b8[] = {
    instrux + 3850,
    instrux + 3851,
    instrux + 3852,
    instrux + 3853,
    instrux + 3854,
    instrux + 3855,
    instrux + 3856,
    instrux + 3857,
    NULL
};

static const struct itemplate * const itbl_vex_map2_b9[] = {
    instrux + 3986,
    instrux + 3987,
    instrux + 3988,
    instrux + 3989,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ba[] = {
    instrux + 3898,
    instrux + 3899,
    instrux + 3900,
    instrux + 3901,
    instrux + 3902,
    instrux + 3903,
    instrux + 3904,
    instrux + 3905,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bb[] = {
    instrux + 3998,
    instrux + 3999,
    instrux + 4000,
    instrux + 4001,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bc[] = {
    instrux + 3946,
    instrux + 3947,
    instrux + 3948,
    instrux + 3949,
    instrux + 3950,
    instrux + 3951,
    instrux + 3952,
    instrux + 3953,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bd[] = {
    instrux + 4010,
    instrux + 4011,
    instrux + 4012,
    instrux + 4013,
    NULL
};

static const struct itemplate * const itbl_vex_map2_be[] = {
    instrux + 3970,
    instrux + 3971,
    instrux + 3972,
    instrux + 3973,
    instrux + 3974,
    instrux + 3975,
    instrux + 3976,
    instrux + 3977,
    NULL
};

static const struct itemplate * const itbl_vex_map2_bf[] = {
    instrux + 4022,
    instrux + 4023,
    instrux + 4024,
    instrux + 4025,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cb[] = {
    instrux + 4411,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cc[] = {
    instrux + 4409,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cd[] = {
    instrux + 4410,
    NULL
};

static const struct itemplate * const itbl_vex_map2_cf[] = {
    instrux + 6733,
    instrux + 6734,
    NULL
};

static const struct itemplate * const itbl_vex_map2_d2[] = {
    instrux + 4461,
    instrux + 4462,
    instrux + 4465,
    instrux + 4466,
    instrux + 4469,
    instrux + 4470,
    NULL
};

static const struct itemplate * const itbl_vex_map2_d3[] = {
    instrux + 4463,
    instrux + 4464,
    instrux + 4467,
    instrux + 4468,
    instrux + 4471,
    instrux + 4472,
    NULL
};

static const struct itemplate * const itbl_vex_map2_da[] = {
    instrux + 4412,
    instrux + 4413,
    instrux + 4415,
    instrux + 4416,
    instrux + 4420,
    instrux + 4421,
    NULL
};

static const struct itemplate * const itbl_vex_map2_db[] = {
    instrux + 3093,
    NULL
};

static const struct itemplate * const itbl_vex_map2_dc[] = {
    instrux + 3089,
    instrux + 3106,
    NULL
};

static const struct itemplate * const itbl_vex_map2_dd[] = {
    instrux + 3090,
    instrux + 3107,
    NULL
};

static const struct itemplate * const itbl_vex_map2_de[] = {
    instrux + 3091,
    instrux + 3108,
    NULL
};

static const struct itemplate * const itbl_vex_map2_df[] = {
    instrux + 3092,
    instrux + 3109,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e0[] = {
    instrux + 2571,
    instrux + 2587,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e1[] = {
    instrux + 2568,
    instrux + 2584,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e2[] = {
    instrux + 2561,
    instrux + 2577,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e3[] = {
    instrux + 2565,
    instrux + 2581,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e4[] = {
    instrux + 2575,
    instrux + 2591,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e5[] = {
    instrux + 2570,
    instrux + 2586,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e6[] = {
    instrux + 2564,
    instrux + 2580,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e7[] = {
    instrux + 2560,
    instrux + 2576,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e8[] = {
    instrux + 2574,
    instrux + 2590,
    NULL
};

static const struct itemplate * const itbl_vex_map2_e9[] = {
    instrux + 2569,
    instrux + 2585,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ea[] = {
    instrux + 2572,
    instrux + 2588,
    NULL
};

static const struct itemplate * const itbl_vex_map2_eb[] = {
    instrux + 2573,
    instrux + 2589,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ec[] = {
    instrux + 2563,
    instrux + 2579,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ed[] = {
    instrux + 2567,
    instrux + 2583,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ee[] = {
    instrux + 2566,
    instrux + 2582,
    NULL
};

static const struct itemplate * const itbl_vex_map2_ef[] = {
    instrux + 2562,
    instrux + 2578,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f2[] = {
    instrux + 916,
    instrux + 917,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f3[] = {
    instrux + 924,
    instrux + 925,
    instrux + 928,
    instrux + 929,
    instrux + 932,
    instrux + 933,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f5[] = {
    instrux + 936,
    instrux + 937,
    instrux + 940,
    instrux + 942,
    instrux + 944,
    instrux + 946,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f6[] = {
    instrux + 778,
    instrux + 780,
    NULL
};

static const struct itemplate * const itbl_vex_map2_f7[] = {
    instrux + 510,
    instrux + 511,
    instrux + 512,
    instrux + 514,
    instrux + 515,
    instrux + 516,
    instrux + 920,
    instrux + 921,
    NULL
};

static const struct itemplate * const * const itbl_vex_map2[256] = {
    itbl_vex_map2_00,
    itbl_vex_map2_01,
    itbl_vex_map2_02,
    itbl_vex_map2_03,
    itbl_vex_map2_04,
    itbl_vex_map2_05,
    itbl_vex_map2_06,
    itbl_vex_map2_07,
    itbl_vex_map2_08,
    itbl_vex_map2_09,
    itbl_vex_map2_0a,
    itbl_vex_map2_0b,
    itbl_vex_map2_0c,
    itbl_vex_map2_0d,
    itbl_vex_map2_0e,
    itbl_vex_map2_0f,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_13,
    NULL,
    NULL,
    itbl_vex_map2_16,
    itbl_vex_map2_17,
    itbl_vex_map2_18,
    itbl_vex_map2_19,
    itbl_vex_map2_1a,
    NULL,
    itbl_vex_map2_1c,
    itbl_vex_map2_1d,
    itbl_vex_map2_1e,
    NULL,
    itbl_vex_map2_20,
    itbl_vex_map2_21,
    itbl_vex_map2_22,
    itbl_vex_map2_23,
    itbl_vex_map2_24,
    itbl_vex_map2_25,
    NULL,
    NULL,
    itbl_vex_map2_28,
    itbl_vex_map2_29,
    itbl_vex_map2_2a,
    itbl_vex_map2_2b,
    itbl_vex_map2_2c,
    itbl_vex_map2_2d,
    itbl_vex_map2_2e,
    itbl_vex_map2_2f,
    itbl_vex_map2_30,
    itbl_vex_map2_31,
    itbl_vex_map2_32,
    itbl_vex_map2_33,
    itbl_vex_map2_34,
    itbl_vex_map2_35,
    itbl_vex_map2_36,
    itbl_vex_map2_37,
    itbl_vex_map2_38,
    itbl_vex_map2_39,
    itbl_vex_map2_3a,
    itbl_vex_map2_3b,
    itbl_vex_map2_3c,
    itbl_vex_map2_3d,
    itbl_vex_map2_3e,
    itbl_vex_map2_3f,
    itbl_vex_map2_40,
    itbl_vex_map2_41,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_45,
    itbl_vex_map2_46,
    itbl_vex_map2_47,
    itbl_vex_map2_48,
    itbl_vex_map2_49,
    itbl_vex_map2_4a,
    itbl_vex_map2_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_50,
    itbl_vex_map2_51,
    itbl_vex_map2_52,
    itbl_vex_map2_53,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_58,
    itbl_vex_map2_59,
    itbl_vex_map2_5a,
    NULL,
    itbl_vex_map2_5c,
    NULL,
    itbl_vex_map2_5e,
    itbl_vex_map2_5f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_6b,
    itbl_vex_map2_6c,
    NULL,
    itbl_vex_map2_6e,
    itbl_vex_map2_6f,
    NULL,
    NULL,
    itbl_vex_map2_72,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_78,
    itbl_vex_map2_79,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_8c,
    NULL,
    itbl_vex_map2_8e,
    NULL,
    itbl_vex_map2_90,
    itbl_vex_map2_91,
    itbl_vex_map2_92,
    itbl_vex_map2_93,
    NULL,
    NULL,
    itbl_vex_map2_96,
    itbl_vex_map2_97,
    itbl_vex_map2_98,
    itbl_vex_map2_99,
    itbl_vex_map2_9a,
    itbl_vex_map2_9b,
    itbl_vex_map2_9c,
    itbl_vex_map2_9d,
    itbl_vex_map2_9e,
    itbl_vex_map2_9f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_a6,
    itbl_vex_map2_a7,
    itbl_vex_map2_a8,
    itbl_vex_map2_a9,
    itbl_vex_map2_aa,
    itbl_vex_map2_ab,
    itbl_vex_map2_ac,
    itbl_vex_map2_ad,
    itbl_vex_map2_ae,
    itbl_vex_map2_af,
    itbl_vex_map2_b0,
    itbl_vex_map2_b1,
    NULL,
    NULL,
    itbl_vex_map2_b4,
    itbl_vex_map2_b5,
    itbl_vex_map2_b6,
    itbl_vex_map2_b7,
    itbl_vex_map2_b8,
    itbl_vex_map2_b9,
    itbl_vex_map2_ba,
    itbl_vex_map2_bb,
    itbl_vex_map2_bc,
    itbl_vex_map2_bd,
    itbl_vex_map2_be,
    itbl_vex_map2_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_cb,
    itbl_vex_map2_cc,
    itbl_vex_map2_cd,
    NULL,
    itbl_vex_map2_cf,
    NULL,
    NULL,
    itbl_vex_map2_d2,
    itbl_vex_map2_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map2_da,
    itbl_vex_map2_db,
    itbl_vex_map2_dc,
    itbl_vex_map2_dd,
    itbl_vex_map2_de,
    itbl_vex_map2_df,
    itbl_vex_map2_e0,
    itbl_vex_map2_e1,
    itbl_vex_map2_e2,
    itbl_vex_map2_e3,
    itbl_vex_map2_e4,
    itbl_vex_map2_e5,
    itbl_vex_map2_e6,
    itbl_vex_map2_e7,
    itbl_vex_map2_e8,
    itbl_vex_map2_e9,
    itbl_vex_map2_ea,
    itbl_vex_map2_eb,
    itbl_vex_map2_ec,
    itbl_vex_map2_ed,
    itbl_vex_map2_ee,
    itbl_vex_map2_ef,
    NULL,
    NULL,
    itbl_vex_map2_f2,
    itbl_vex_map2_f3,
    NULL,
    itbl_vex_map2_f5,
    itbl_vex_map2_f6,
    itbl_vex_map2_f7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map3_00[] = {
    instrux + 4334,
    NULL
};

static const struct itemplate * const itbl_vex_map3_01[] = {
    instrux + 4332,
    NULL
};

static const struct itemplate * const itbl_vex_map3_02[] = {
    instrux + 4313,
    instrux + 4314,
    NULL
};

static const struct itemplate * const itbl_vex_map3_04[] = {
    instrux + 3643,
    instrux + 3644,
    NULL
};

static const struct itemplate * const itbl_vex_map3_05[] = {
    instrux + 3639,
    instrux + 3640,
    NULL
};

static const struct itemplate * const itbl_vex_map3_06[] = {
    instrux + 3645,
    NULL
};

static const struct itemplate * const itbl_vex_map3_08[] = {
    instrux + 3763,
    instrux + 3764,
    NULL
};

static const struct itemplate * const itbl_vex_map3_09[] = {
    instrux + 3761,
    instrux + 3762,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0a[] = {
    instrux + 3766,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0b[] = {
    instrux + 3765,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0c[] = {
    instrux + 3142,
    instrux + 3143,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0d[] = {
    instrux + 3140,
    instrux + 3141,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0e[] = {
    instrux + 3624,
    instrux + 4208,
    NULL
};

static const struct itemplate * const itbl_vex_map3_0f[] = {
    instrux + 3618,
    instrux + 4202,
    NULL
};

static const struct itemplate * const itbl_vex_map3_14[] = {
    instrux + 3646,
    instrux + 3647,
    instrux + 3648,
    NULL
};

static const struct itemplate * const itbl_vex_map3_15[] = {
    instrux + 3651,
    instrux + 3652,
    instrux + 3653,
    NULL
};

static const struct itemplate * const itbl_vex_map3_16[] = {
    instrux + 3654,
    instrux + 3655,
    instrux + 3656,
    NULL
};

static const struct itemplate * const itbl_vex_map3_17[] = {
    instrux + 3482,
    NULL
};

static const struct itemplate * const itbl_vex_map3_18[] = {
    instrux + 3491,
    NULL
};

static const struct itemplate * const itbl_vex_map3_19[] = {
    instrux + 3481,
    NULL
};

static const struct itemplate * const itbl_vex_map3_1d[] = {
    instrux + 4028,
    instrux + 4029,
    NULL
};

static const struct itemplate * const itbl_vex_map3_20[] = {
    instrux + 3664,
    instrux + 3665,
    instrux + 3666,
    NULL
};

static const struct itemplate * const itbl_vex_map3_21[] = {
    instrux + 3492,
    NULL
};

static const struct itemplate * const itbl_vex_map3_22[] = {
    instrux + 3670,
    instrux + 3671,
    instrux + 3672,
    instrux + 3673,
    NULL
};

static const struct itemplate * const itbl_vex_map3_30[] = {
    instrux + 4565,
    instrux + 4567,
    NULL
};

static const struct itemplate * const itbl_vex_map3_31[] = {
    instrux + 4569,
    instrux + 4571,
    NULL
};

static const struct itemplate * const itbl_vex_map3_32[] = {
    instrux + 4557,
    instrux + 4559,
    NULL
};

static const struct itemplate * const itbl_vex_map3_33[] = {
    instrux + 4561,
    instrux + 4563,
    NULL
};

static const struct itemplate * const itbl_vex_map3_38[] = {
    instrux + 4337,
    NULL
};

static const struct itemplate * const itbl_vex_map3_39[] = {
    instrux + 4336,
    NULL
};

static const struct itemplate * const itbl_vex_map3_40[] = {
    instrux + 3479,
    instrux + 3480,
    NULL
};

static const struct itemplate * const itbl_vex_map3_41[] = {
    instrux + 3478,
    NULL
};

static const struct itemplate * const itbl_vex_map3_42[] = {
    instrux + 3592,
    instrux + 4186,
    NULL
};

static const struct itemplate * const itbl_vex_map3_44[] = {
    instrux + 3809,
    instrux + 3810,
    instrux + 3811,
    instrux + 3812,
    instrux + 3813,
    instrux + 3814,
    instrux + 3815,
    instrux + 3816,
    instrux + 3817,
    instrux + 3818,
    NULL
};

static const struct itemplate * const itbl_vex_map3_46[] = {
    instrux + 4335,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4a[] = {
    instrux + 3146,
    instrux + 3147,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4b[] = {
    instrux + 3144,
    instrux + 3145,
    NULL
};

static const struct itemplate * const itbl_vex_map3_4c[] = {
    instrux + 3623,
    instrux + 4207,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5c[] = {
    instrux + 4063,
    instrux + 4064,
    instrux + 4065,
    instrux + 4066,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5d[] = {
    instrux + 4059,
    instrux + 4060,
    instrux + 4061,
    instrux + 4062,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5e[] = {
    instrux + 4071,
    instrux + 4072,
    instrux + 4073,
    instrux + 4074,
    NULL
};

static const struct itemplate * const itbl_vex_map3_5f[] = {
    instrux + 4067,
    instrux + 4068,
    instrux + 4069,
    instrux + 4070,
    NULL
};

static const struct itemplate * const itbl_vex_map3_60[] = {
    instrux + 3626,
    NULL
};

static const struct itemplate * const itbl_vex_map3_61[] = {
    instrux + 3625,
    NULL
};

static const struct itemplate * const itbl_vex_map3_62[] = {
    instrux + 3628,
    NULL
};

static const struct itemplate * const itbl_vex_map3_63[] = {
    instrux + 3627,
    NULL
};

static const struct itemplate * const itbl_vex_map3_68[] = {
    instrux + 4051,
    instrux + 4052,
    instrux + 4053,
    instrux + 4054,
    NULL
};

static const struct itemplate * const itbl_vex_map3_69[] = {
    instrux + 4047,
    instrux + 4048,
    instrux + 4049,
    instrux + 4050,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6a[] = {
    instrux + 4057,
    instrux + 4058,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6b[] = {
    instrux + 4055,
    instrux + 4056,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6c[] = {
    instrux + 4079,
    instrux + 4080,
    instrux + 4081,
    instrux + 4082,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6d[] = {
    instrux + 4075,
    instrux + 4076,
    instrux + 4077,
    instrux + 4078,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6e[] = {
    instrux + 4085,
    instrux + 4086,
    NULL
};

static const struct itemplate * const itbl_vex_map3_6f[] = {
    instrux + 4083,
    instrux + 4084,
    NULL
};

static const struct itemplate * const itbl_vex_map3_78[] = {
    instrux + 4091,
    instrux + 4092,
    instrux + 4093,
    instrux + 4094,
    NULL
};

static const struct itemplate * const itbl_vex_map3_79[] = {
    instrux + 4087,
    instrux + 4088,
    instrux + 4089,
    instrux + 4090,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7a[] = {
    instrux + 4097,
    instrux + 4098,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7b[] = {
    instrux + 4095,
    instrux + 4096,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7c[] = {
    instrux + 4103,
    instrux + 4104,
    instrux + 4105,
    instrux + 4106,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7d[] = {
    instrux + 4099,
    instrux + 4100,
    instrux + 4101,
    instrux + 4102,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7e[] = {
    instrux + 4109,
    instrux + 4110,
    NULL
};

static const struct itemplate * const itbl_vex_map3_7f[] = {
    instrux + 4107,
    instrux + 4108,
    NULL
};

static const struct itemplate * const itbl_vex_map3_ce[] = {
    instrux + 6727,
    instrux + 6728,
    NULL
};

static const struct itemplate * const itbl_vex_map3_cf[] = {
    instrux + 6721,
    instrux + 6722,
    NULL
};

static const struct itemplate * const itbl_vex_map3_de[] = {
    instrux + 4414,
    NULL
};

static const struct itemplate * const itbl_vex_map3_df[] = {
    instrux + 3094,
    NULL
};

static const struct itemplate * const itbl_vex_map3_f0[] = {
    instrux + 509,
    instrux + 513,
    NULL
};

static const struct itemplate * const * const itbl_vex_map3[256] = {
    itbl_vex_map3_00,
    itbl_vex_map3_01,
    itbl_vex_map3_02,
    NULL,
    itbl_vex_map3_04,
    itbl_vex_map3_05,
    itbl_vex_map3_06,
    NULL,
    itbl_vex_map3_08,
    itbl_vex_map3_09,
    itbl_vex_map3_0a,
    itbl_vex_map3_0b,
    itbl_vex_map3_0c,
    itbl_vex_map3_0d,
    itbl_vex_map3_0e,
    itbl_vex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_14,
    itbl_vex_map3_15,
    itbl_vex_map3_16,
    itbl_vex_map3_17,
    itbl_vex_map3_18,
    itbl_vex_map3_19,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_1d,
    NULL,
    NULL,
    itbl_vex_map3_20,
    itbl_vex_map3_21,
    itbl_vex_map3_22,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_30,
    itbl_vex_map3_31,
    itbl_vex_map3_32,
    itbl_vex_map3_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_38,
    itbl_vex_map3_39,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_40,
    itbl_vex_map3_41,
    itbl_vex_map3_42,
    NULL,
    itbl_vex_map3_44,
    NULL,
    itbl_vex_map3_46,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_4a,
    itbl_vex_map3_4b,
    itbl_vex_map3_4c,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_5c,
    itbl_vex_map3_5d,
    itbl_vex_map3_5e,
    itbl_vex_map3_5f,
    itbl_vex_map3_60,
    itbl_vex_map3_61,
    itbl_vex_map3_62,
    itbl_vex_map3_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_68,
    itbl_vex_map3_69,
    itbl_vex_map3_6a,
    itbl_vex_map3_6b,
    itbl_vex_map3_6c,
    itbl_vex_map3_6d,
    itbl_vex_map3_6e,
    itbl_vex_map3_6f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_78,
    itbl_vex_map3_79,
    itbl_vex_map3_7a,
    itbl_vex_map3_7b,
    itbl_vex_map3_7c,
    itbl_vex_map3_7d,
    itbl_vex_map3_7e,
    itbl_vex_map3_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_ce,
    itbl_vex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_de,
    itbl_vex_map3_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map5_f8[] = {
    instrux + 6900,
    instrux + 6902,
    NULL
};

static const struct itemplate * const itbl_vex_map5_f9[] = {
    instrux + 6901,
    instrux + 6903,
    NULL
};

static const struct itemplate * const itbl_vex_map5_fd[] = {
    instrux + 6916,
    instrux + 6917,
    instrux + 6918,
    instrux + 6919,
    NULL
};

static const struct itemplate * const * const itbl_vex_map5[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map5_f8,
    itbl_vex_map5_f9,
    NULL,
    NULL,
    NULL,
    itbl_vex_map5_fd,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_vex_map7_f6[] = {
    instrux + 1335,
    instrux + 1338,
    NULL
};

static const struct itemplate * const itbl_vex_map7_f8[] = {
    instrux + 1344,
    instrux + 1348,
    NULL
};

static const struct itemplate * const * const itbl_vex_map7[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_vex_map7_f6,
    NULL,
    itbl_vex_map7_f8,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map8_85[] = {
    instrux + 4151,
    NULL
};

static const struct itemplate * const itbl_xop_map8_86[] = {
    instrux + 4150,
    NULL
};

static const struct itemplate * const itbl_xop_map8_87[] = {
    instrux + 4149,
    NULL
};

static const struct itemplate * const itbl_xop_map8_8e[] = {
    instrux + 4147,
    NULL
};

static const struct itemplate * const itbl_xop_map8_8f[] = {
    instrux + 4148,
    NULL
};

static const struct itemplate * const itbl_xop_map8_95[] = {
    instrux + 4153,
    NULL
};

static const struct itemplate * const itbl_xop_map8_96[] = {
    instrux + 4152,
    NULL
};

static const struct itemplate * const itbl_xop_map8_97[] = {
    instrux + 4146,
    NULL
};

static const struct itemplate * const itbl_xop_map8_9e[] = {
    instrux + 4144,
    NULL
};

static const struct itemplate * const itbl_xop_map8_9f[] = {
    instrux + 4145,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a2[] = {
    instrux + 4117,
    instrux + 4118,
    instrux + 4119,
    instrux + 4120,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a3[] = {
    instrux + 4156,
    instrux + 4157,
    NULL
};

static const struct itemplate * const itbl_xop_map8_a6[] = {
    instrux + 4154,
    NULL
};

static const struct itemplate * const itbl_xop_map8_b6[] = {
    instrux + 4155,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c0[] = {
    instrux + 4160,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c1[] = {
    instrux + 4169,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c2[] = {
    instrux + 4163,
    NULL
};

static const struct itemplate * const itbl_xop_map8_c3[] = {
    instrux + 4166,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cc[] = {
    instrux + 4121,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cd[] = {
    instrux + 4128,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ce[] = {
    instrux + 4122,
    NULL
};

static const struct itemplate * const itbl_xop_map8_cf[] = {
    instrux + 4123,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ec[] = {
    instrux + 4124,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ed[] = {
    instrux + 4127,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ee[] = {
    instrux + 4125,
    NULL
};

static const struct itemplate * const itbl_xop_map8_ef[] = {
    instrux + 4126,
    NULL
};

static const struct itemplate * const * const itbl_xop_map8[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_85,
    itbl_xop_map8_86,
    itbl_xop_map8_87,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_8e,
    itbl_xop_map8_8f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_95,
    itbl_xop_map8_96,
    itbl_xop_map8_97,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_9e,
    itbl_xop_map8_9f,
    NULL,
    NULL,
    itbl_xop_map8_a2,
    itbl_xop_map8_a3,
    NULL,
    NULL,
    itbl_xop_map8_a6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_b6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_c0,
    itbl_xop_map8_c1,
    itbl_xop_map8_c2,
    itbl_xop_map8_c3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_cc,
    itbl_xop_map8_cd,
    itbl_xop_map8_ce,
    itbl_xop_map8_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map8_ec,
    itbl_xop_map8_ed,
    itbl_xop_map8_ee,
    itbl_xop_map8_ef,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map9_01[] = {
    instrux + 952,
    instrux + 953,
    instrux + 954,
    instrux + 955,
    instrux + 956,
    instrux + 957,
    instrux + 958,
    instrux + 959,
    instrux + 962,
    instrux + 963,
    instrux + 964,
    instrux + 965,
    instrux + 966,
    instrux + 967,
    NULL
};

static const struct itemplate * const itbl_xop_map9_02[] = {
    instrux + 950,
    instrux + 951,
    instrux + 960,
    instrux + 961,
    NULL
};

static const struct itemplate * const itbl_xop_map9_12[] = {
    instrux + 4039,
    instrux + 4040,
    instrux + 4041,
    instrux + 4042,
    NULL
};

static const struct itemplate * const itbl_xop_map9_80[] = {
    instrux + 4113,
    instrux + 4114,
    NULL
};

static const struct itemplate * const itbl_xop_map9_81[] = {
    instrux + 4111,
    instrux + 4112,
    NULL
};

static const struct itemplate * const itbl_xop_map9_82[] = {
    instrux + 4116,
    NULL
};

static const struct itemplate * const itbl_xop_map9_83[] = {
    instrux + 4115,
    NULL
};

static const struct itemplate * const itbl_xop_map9_90[] = {
    instrux + 4158,
    instrux + 4159,
    NULL
};

static const struct itemplate * const itbl_xop_map9_91[] = {
    instrux + 4167,
    instrux + 4168,
    NULL
};

static const struct itemplate * const itbl_xop_map9_92[] = {
    instrux + 4161,
    instrux + 4162,
    NULL
};

static const struct itemplate * const itbl_xop_map9_93[] = {
    instrux + 4164,
    instrux + 4165,
    NULL
};

static const struct itemplate * const itbl_xop_map9_94[] = {
    instrux + 4178,
    instrux + 4179,
    NULL
};

static const struct itemplate * const itbl_xop_map9_95[] = {
    instrux + 4184,
    instrux + 4185,
    NULL
};

static const struct itemplate * const itbl_xop_map9_96[] = {
    instrux + 4180,
    instrux + 4181,
    NULL
};

static const struct itemplate * const itbl_xop_map9_97[] = {
    instrux + 4182,
    instrux + 4183,
    NULL
};

static const struct itemplate * const itbl_xop_map9_98[] = {
    instrux + 4170,
    instrux + 4171,
    NULL
};

static const struct itemplate * const itbl_xop_map9_99[] = {
    instrux + 4176,
    instrux + 4177,
    NULL
};

static const struct itemplate * const itbl_xop_map9_9a[] = {
    instrux + 4172,
    instrux + 4173,
    NULL
};

static const struct itemplate * const itbl_xop_map9_9b[] = {
    instrux + 4174,
    instrux + 4175,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c1[] = {
    instrux + 4131,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c2[] = {
    instrux + 4129,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c3[] = {
    instrux + 4130,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c6[] = {
    instrux + 4139,
    NULL
};

static const struct itemplate * const itbl_xop_map9_c7[] = {
    instrux + 4140,
    NULL
};

static const struct itemplate * const itbl_xop_map9_cb[] = {
    instrux + 4132,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d1[] = {
    instrux + 4135,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d2[] = {
    instrux + 4133,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d3[] = {
    instrux + 4134,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d6[] = {
    instrux + 4137,
    NULL
};

static const struct itemplate * const itbl_xop_map9_d7[] = {
    instrux + 4138,
    NULL
};

static const struct itemplate * const itbl_xop_map9_db[] = {
    instrux + 4136,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e1[] = {
    instrux + 4141,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e2[] = {
    instrux + 4143,
    NULL
};

static const struct itemplate * const itbl_xop_map9_e3[] = {
    instrux + 4142,
    NULL
};

static const struct itemplate * const * const itbl_xop_map9[256] = {
    NULL,
    itbl_xop_map9_01,
    itbl_xop_map9_02,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_12,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_80,
    itbl_xop_map9_81,
    itbl_xop_map9_82,
    itbl_xop_map9_83,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_90,
    itbl_xop_map9_91,
    itbl_xop_map9_92,
    itbl_xop_map9_93,
    itbl_xop_map9_94,
    itbl_xop_map9_95,
    itbl_xop_map9_96,
    itbl_xop_map9_97,
    itbl_xop_map9_98,
    itbl_xop_map9_99,
    itbl_xop_map9_9a,
    itbl_xop_map9_9b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_c1,
    itbl_xop_map9_c2,
    itbl_xop_map9_c3,
    NULL,
    NULL,
    itbl_xop_map9_c6,
    itbl_xop_map9_c7,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_cb,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_d1,
    itbl_xop_map9_d2,
    itbl_xop_map9_d3,
    NULL,
    NULL,
    itbl_xop_map9_d6,
    itbl_xop_map9_d7,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_db,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map9_e1,
    itbl_xop_map9_e2,
    itbl_xop_map9_e3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_xop_map10_10[] = {
    instrux + 948,
    instrux + 949,
    NULL
};

static const struct itemplate * const itbl_xop_map10_12[] = {
    instrux + 4043,
    instrux + 4044,
    instrux + 4045,
    instrux + 4046,
    NULL
};

static const struct itemplate * const * const itbl_xop_map10[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_xop_map10_10,
    NULL,
    itbl_xop_map10_12,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map1_10[] = {
    instrux + 5515,
    instrux + 5517,
    instrux + 5525,
    instrux + 5527,
    instrux + 5529,
    instrux + 5530,
    instrux + 5531,
    instrux + 5538,
    instrux + 5539,
    instrux + 5540,
    NULL
};

static const struct itemplate * const itbl_evex_map1_11[] = {
    instrux + 5516,
    instrux + 5518,
    instrux + 5526,
    instrux + 5528,
    instrux + 5532,
    instrux + 5533,
    instrux + 5534,
    instrux + 5535,
    instrux + 5536,
    instrux + 5537,
    instrux + 5541,
    instrux + 5542,
    instrux + 5543,
    instrux + 5544,
    instrux + 5545,
    instrux + 5546,
    NULL
};

static const struct itemplate * const itbl_evex_map1_12[] = {
    instrux + 5450,
    instrux + 5451,
    instrux + 5452,
    instrux + 5489,
    instrux + 5495,
    instrux + 5497,
    instrux + 5522,
    instrux + 5523,
    instrux + 5524,
    NULL
};

static const struct itemplate * const itbl_evex_map1_13[] = {
    instrux + 5496,
    instrux + 5498,
    NULL
};

static const struct itemplate * const itbl_evex_map1_14[] = {
    instrux + 6692,
    instrux + 6693,
    instrux + 6694,
    instrux + 6695,
    instrux + 6696,
    instrux + 6697,
    NULL
};

static const struct itemplate * const itbl_evex_map1_15[] = {
    instrux + 6686,
    instrux + 6687,
    instrux + 6688,
    instrux + 6689,
    instrux + 6690,
    instrux + 6691,
    NULL
};

static const struct itemplate * const itbl_evex_map1_16[] = {
    instrux + 5490,
    instrux + 5492,
    instrux + 5494,
    instrux + 5519,
    instrux + 5520,
    instrux + 5521,
    NULL
};

static const struct itemplate * const itbl_evex_map1_17[] = {
    instrux + 5491,
    instrux + 5493,
    NULL
};

static const struct itemplate * const itbl_evex_map1_28[] = {
    instrux + 5430,
    instrux + 5431,
    instrux + 5432,
    instrux + 5439,
    instrux + 5440,
    instrux + 5441,
    NULL
};

static const struct itemplate * const itbl_evex_map1_29[] = {
    instrux + 5433,
    instrux + 5434,
    instrux + 5435,
    instrux + 5436,
    instrux + 5437,
    instrux + 5438,
    instrux + 5442,
    instrux + 5443,
    instrux + 5444,
    instrux + 5445,
    instrux + 5446,
    instrux + 5447,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2a[] = {
    instrux + 5108,
    instrux + 5109,
    instrux + 5110,
    instrux + 5111,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2b[] = {
    instrux + 5505,
    instrux + 5506,
    instrux + 5507,
    instrux + 5508,
    instrux + 5509,
    instrux + 5510,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2c[] = {
    instrux + 5141,
    instrux + 5142,
    instrux + 5145,
    instrux + 5146,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2d[] = {
    instrux + 5103,
    instrux + 5104,
    instrux + 5113,
    instrux + 5114,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2e[] = {
    instrux + 6684,
    instrux + 6685,
    instrux + 7292,
    instrux + 7294,
    NULL
};

static const struct itemplate * const itbl_evex_map1_2f[] = {
    instrux + 5038,
    instrux + 5039,
    instrux + 7289,
    instrux + 7291,
    NULL
};

static const struct itemplate * const itbl_evex_map1_41[] = {
    instrux + 4518,
    instrux + 4520,
    instrux + 4522,
    instrux + 4524,
    NULL
};

static const struct itemplate * const itbl_evex_map1_42[] = {
    instrux + 4526,
    instrux + 4528,
    instrux + 4530,
    instrux + 4532,
    NULL
};

static const struct itemplate * const itbl_evex_map1_44[] = {
    instrux + 4534,
    instrux + 4536,
    instrux + 4538,
    instrux + 4540,
    NULL
};

static const struct itemplate * const itbl_evex_map1_45[] = {
    instrux + 4542,
    instrux + 4544,
    instrux + 4546,
    instrux + 4548,
    NULL
};

static const struct itemplate * const itbl_evex_map1_46[] = {
    instrux + 4588,
    instrux + 4590,
    instrux + 4592,
    instrux + 4594,
    NULL
};

static const struct itemplate * const itbl_evex_map1_47[] = {
    instrux + 4596,
    instrux + 4598,
    instrux + 4600,
    instrux + 4602,
    NULL
};

static const struct itemplate * const itbl_evex_map1_4a[] = {
    instrux + 4510,
    instrux + 4512,
    instrux + 4514,
    instrux + 4516,
    NULL
};

static const struct itemplate * const itbl_evex_map1_4b[] = {
    instrux + 4582,
    instrux + 4584,
    instrux + 4586,
    NULL
};

static const struct itemplate * const itbl_evex_map1_51[] = {
    instrux + 6668,
    instrux + 6669,
    instrux + 6670,
    instrux + 6671,
    instrux + 6672,
    instrux + 6673,
    instrux + 6674,
    instrux + 6675,
    NULL
};

static const struct itemplate * const itbl_evex_map1_54[] = {
    instrux + 4623,
    instrux + 4624,
    instrux + 4625,
    instrux + 4626,
    instrux + 4627,
    instrux + 4628,
    NULL
};

static const struct itemplate * const itbl_evex_map1_55[] = {
    instrux + 4617,
    instrux + 4618,
    instrux + 4619,
    instrux + 4620,
    instrux + 4621,
    instrux + 4622,
    NULL
};

static const struct itemplate * const itbl_evex_map1_56[] = {
    instrux + 5555,
    instrux + 5556,
    instrux + 5557,
    instrux + 5558,
    instrux + 5559,
    instrux + 5560,
    NULL
};

static const struct itemplate * const itbl_evex_map1_57[] = {
    instrux + 6698,
    instrux + 6699,
    instrux + 6700,
    instrux + 6701,
    instrux + 6702,
    instrux + 6703,
    NULL
};

static const struct itemplate * const itbl_evex_map1_58[] = {
    instrux + 4603,
    instrux + 4604,
    instrux + 4605,
    instrux + 4606,
    instrux + 4607,
    instrux + 4608,
    instrux + 4609,
    instrux + 4610,
    NULL
};

static const struct itemplate * const itbl_evex_map1_59[] = {
    instrux + 5547,
    instrux + 5548,
    instrux + 5549,
    instrux + 5550,
    instrux + 5551,
    instrux + 5552,
    instrux + 5553,
    instrux + 5554,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5a[] = {
    instrux + 5061,
    instrux + 5062,
    instrux + 5063,
    instrux + 5079,
    instrux + 5080,
    instrux + 5081,
    instrux + 5105,
    instrux + 5112,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5b[] = {
    instrux + 5055,
    instrux + 5056,
    instrux + 5057,
    instrux + 5076,
    instrux + 5077,
    instrux + 5078,
    instrux + 5100,
    instrux + 5101,
    instrux + 5102,
    instrux + 5129,
    instrux + 5130,
    instrux + 5131,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5c[] = {
    instrux + 6676,
    instrux + 6677,
    instrux + 6678,
    instrux + 6679,
    instrux + 6680,
    instrux + 6681,
    instrux + 6682,
    instrux + 6683,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5d[] = {
    instrux + 5422,
    instrux + 5423,
    instrux + 5424,
    instrux + 5425,
    instrux + 5426,
    instrux + 5427,
    instrux + 5428,
    instrux + 5429,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5e[] = {
    instrux + 5168,
    instrux + 5169,
    instrux + 5170,
    instrux + 5171,
    instrux + 5172,
    instrux + 5173,
    instrux + 5174,
    instrux + 5175,
    NULL
};

static const struct itemplate * const itbl_evex_map1_5f[] = {
    instrux + 5414,
    instrux + 5415,
    instrux + 5416,
    instrux + 5417,
    instrux + 5418,
    instrux + 5419,
    instrux + 5420,
    instrux + 5421,
    NULL
};

static const struct itemplate * const itbl_evex_map1_60[] = {
    instrux + 6556,
    instrux + 6557,
    instrux + 6558,
    NULL
};

static const struct itemplate * const itbl_evex_map1_61[] = {
    instrux + 6565,
    instrux + 6566,
    instrux + 6567,
    NULL
};

static const struct itemplate * const itbl_evex_map1_62[] = {
    instrux + 6559,
    instrux + 6560,
    instrux + 6561,
    NULL
};

static const struct itemplate * const itbl_evex_map1_63[] = {
    instrux + 5576,
    instrux + 5577,
    instrux + 5578,
    NULL
};

static const struct itemplate * const itbl_evex_map1_64[] = {
    instrux + 5705,
    instrux + 5706,
    instrux + 5707,
    NULL
};

static const struct itemplate * const itbl_evex_map1_65[] = {
    instrux + 5714,
    instrux + 5715,
    instrux + 5716,
    NULL
};

static const struct itemplate * const itbl_evex_map1_66[] = {
    instrux + 5708,
    instrux + 5709,
    instrux + 5710,
    NULL
};

static const struct itemplate * const itbl_evex_map1_67[] = {
    instrux + 5582,
    instrux + 5583,
    instrux + 5584,
    NULL
};

static const struct itemplate * const itbl_evex_map1_68[] = {
    instrux + 6544,
    instrux + 6545,
    instrux + 6546,
    NULL
};

static const struct itemplate * const itbl_evex_map1_69[] = {
    instrux + 6553,
    instrux + 6554,
    instrux + 6555,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6a[] = {
    instrux + 6547,
    instrux + 6548,
    instrux + 6549,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6b[] = {
    instrux + 5573,
    instrux + 5574,
    instrux + 5575,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6c[] = {
    instrux + 6562,
    instrux + 6563,
    instrux + 6564,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6d[] = {
    instrux + 6550,
    instrux + 6551,
    instrux + 6552,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6e[] = {
    instrux + 5448,
    instrux + 5511,
    NULL
};

static const struct itemplate * const itbl_evex_map1_6f[] = {
    instrux + 5453,
    instrux + 5454,
    instrux + 5455,
    instrux + 5459,
    instrux + 5460,
    instrux + 5461,
    instrux + 5465,
    instrux + 5466,
    instrux + 5467,
    instrux + 5471,
    instrux + 5472,
    instrux + 5473,
    instrux + 5477,
    instrux + 5478,
    instrux + 5479,
    instrux + 5483,
    instrux + 5484,
    instrux + 5485,
    NULL
};

static const struct itemplate * const itbl_evex_map1_70[] = {
    instrux + 6394,
    instrux + 6395,
    instrux + 6396,
    instrux + 6397,
    instrux + 6398,
    instrux + 6399,
    instrux + 6400,
    instrux + 6401,
    instrux + 6402,
    NULL
};

static const struct itemplate * const itbl_evex_map1_71[] = {
    instrux + 6430,
    instrux + 6431,
    instrux + 6432,
    instrux + 6457,
    instrux + 6458,
    instrux + 6459,
    instrux + 6487,
    instrux + 6488,
    instrux + 6489,
    NULL
};

static const struct itemplate * const itbl_evex_map1_72[] = {
    instrux + 6352,
    instrux + 6353,
    instrux + 6354,
    instrux + 6355,
    instrux + 6356,
    instrux + 6357,
    instrux + 6364,
    instrux + 6365,
    instrux + 6366,
    instrux + 6367,
    instrux + 6368,
    instrux + 6369,
    instrux + 6406,
    instrux + 6407,
    instrux + 6408,
    instrux + 6436,
    instrux + 6437,
    instrux + 6438,
    instrux + 6442,
    instrux + 6443,
    instrux + 6444,
    instrux + 6463,
    instrux + 6464,
    instrux + 6465,
    NULL
};

static const struct itemplate * const itbl_evex_map1_73[] = {
    instrux + 6409,
    instrux + 6410,
    instrux + 6411,
    instrux + 6415,
    instrux + 6416,
    instrux + 6417,
    instrux + 6466,
    instrux + 6467,
    instrux + 6468,
    instrux + 6472,
    instrux + 6473,
    instrux + 6474,
    NULL
};

static const struct itemplate * const itbl_evex_map1_74[] = {
    instrux + 5693,
    instrux + 5694,
    instrux + 5695,
    NULL
};

static const struct itemplate * const itbl_evex_map1_75[] = {
    instrux + 5702,
    instrux + 5703,
    instrux + 5704,
    NULL
};

static const struct itemplate * const itbl_evex_map1_76[] = {
    instrux + 5696,
    instrux + 5697,
    instrux + 5698,
    NULL
};

static const struct itemplate * const itbl_evex_map1_78[] = {
    instrux + 5123,
    instrux + 5124,
    instrux + 5125,
    instrux + 5126,
    instrux + 5127,
    instrux + 5128,
    instrux + 5135,
    instrux + 5136,
    instrux + 5137,
    instrux + 5138,
    instrux + 5139,
    instrux + 5140,
    instrux + 5143,
    instrux + 5144,
    instrux + 5147,
    instrux + 5148,
    NULL
};

static const struct itemplate * const itbl_evex_map1_79[] = {
    instrux + 5067,
    instrux + 5068,
    instrux + 5069,
    instrux + 5070,
    instrux + 5071,
    instrux + 5072,
    instrux + 5091,
    instrux + 5092,
    instrux + 5093,
    instrux + 5094,
    instrux + 5095,
    instrux + 5096,
    instrux + 5106,
    instrux + 5107,
    instrux + 5115,
    instrux + 5116,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7a[] = {
    instrux + 5120,
    instrux + 5121,
    instrux + 5122,
    instrux + 5132,
    instrux + 5133,
    instrux + 5134,
    instrux + 5149,
    instrux + 5150,
    instrux + 5151,
    instrux + 5152,
    instrux + 5153,
    instrux + 5154,
    instrux + 5155,
    instrux + 5156,
    instrux + 5157,
    instrux + 5158,
    instrux + 5159,
    instrux + 5160,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7b[] = {
    instrux + 5064,
    instrux + 5065,
    instrux + 5066,
    instrux + 5088,
    instrux + 5089,
    instrux + 5090,
    instrux + 5161,
    instrux + 5162,
    instrux + 5163,
    instrux + 5164,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7e[] = {
    instrux + 5449,
    instrux + 5512,
    instrux + 5513,
    instrux + 7462,
    NULL
};

static const struct itemplate * const itbl_evex_map1_7f[] = {
    instrux + 5456,
    instrux + 5457,
    instrux + 5458,
    instrux + 5462,
    instrux + 5463,
    instrux + 5464,
    instrux + 5468,
    instrux + 5469,
    instrux + 5470,
    instrux + 5474,
    instrux + 5475,
    instrux + 5476,
    instrux + 5480,
    instrux + 5481,
    instrux + 5482,
    instrux + 5486,
    instrux + 5487,
    instrux + 5488,
    NULL
};

static const struct itemplate * const itbl_evex_map1_90[] = {
    instrux + 4478,
    instrux + 4480,
    instrux + 4482,
    instrux + 4484,
    NULL
};

static const struct itemplate * const itbl_evex_map1_91[] = {
    instrux + 4486,
    instrux + 4488,
    instrux + 4490,
    instrux + 4492,
    NULL
};

static const struct itemplate * const itbl_evex_map1_92[] = {
    instrux + 4494,
    instrux + 4496,
    instrux + 4502,
    instrux + 4504,
    NULL
};

static const struct itemplate * const itbl_evex_map1_93[] = {
    instrux + 4498,
    instrux + 4500,
    instrux + 4506,
    instrux + 4508,
    NULL
};

static const struct itemplate * const itbl_evex_map1_98[] = {
    instrux + 4550,
    instrux + 4552,
    instrux + 4554,
    instrux + 4556,
    NULL
};

static const struct itemplate * const itbl_evex_map1_99[] = {
    instrux + 4574,
    instrux + 4576,
    instrux + 4578,
    instrux + 4580,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c2[] = {
    instrux + 4662,
    instrux + 4663,
    instrux + 4664,
    instrux + 4665,
    instrux + 4666,
    instrux + 4667,
    instrux + 4668,
    instrux + 4669,
    instrux + 4670,
    instrux + 4671,
    instrux + 4672,
    instrux + 4673,
    instrux + 4674,
    instrux + 4675,
    instrux + 4676,
    instrux + 4677,
    instrux + 4678,
    instrux + 4679,
    instrux + 4680,
    instrux + 4681,
    instrux + 4682,
    instrux + 4683,
    instrux + 4684,
    instrux + 4685,
    instrux + 4686,
    instrux + 4687,
    instrux + 4688,
    instrux + 4689,
    instrux + 4690,
    instrux + 4691,
    instrux + 4692,
    instrux + 4693,
    instrux + 4694,
    instrux + 4695,
    instrux + 4696,
    instrux + 4697,
    instrux + 4698,
    instrux + 4699,
    instrux + 4700,
    instrux + 4701,
    instrux + 4702,
    instrux + 4703,
    instrux + 4704,
    instrux + 4705,
    instrux + 4706,
    instrux + 4707,
    instrux + 4708,
    instrux + 4709,
    instrux + 4710,
    instrux + 4711,
    instrux + 4712,
    instrux + 4713,
    instrux + 4714,
    instrux + 4715,
    instrux + 4716,
    instrux + 4717,
    instrux + 4718,
    instrux + 4719,
    instrux + 4720,
    instrux + 4721,
    instrux + 4722,
    instrux + 4723,
    instrux + 4724,
    instrux + 4725,
    instrux + 4726,
    instrux + 4727,
    instrux + 4728,
    instrux + 4729,
    instrux + 4730,
    instrux + 4731,
    instrux + 4732,
    instrux + 4733,
    instrux + 4734,
    instrux + 4735,
    instrux + 4736,
    instrux + 4737,
    instrux + 4738,
    instrux + 4739,
    instrux + 4740,
    instrux + 4741,
    instrux + 4742,
    instrux + 4743,
    instrux + 4744,
    instrux + 4745,
    instrux + 4746,
    instrux + 4747,
    instrux + 4748,
    instrux + 4749,
    instrux + 4750,
    instrux + 4751,
    instrux + 4752,
    instrux + 4753,
    instrux + 4754,
    instrux + 4755,
    instrux + 4756,
    instrux + 4757,
    instrux + 4758,
    instrux + 4759,
    instrux + 4760,
    instrux + 4761,
    instrux + 4762,
    instrux + 4763,
    instrux + 4764,
    instrux + 4765,
    instrux + 4766,
    instrux + 4767,
    instrux + 4768,
    instrux + 4769,
    instrux + 4770,
    instrux + 4771,
    instrux + 4772,
    instrux + 4773,
    instrux + 4774,
    instrux + 4775,
    instrux + 4776,
    instrux + 4777,
    instrux + 4778,
    instrux + 4779,
    instrux + 4780,
    instrux + 4781,
    instrux + 4782,
    instrux + 4783,
    instrux + 4784,
    instrux + 4785,
    instrux + 4786,
    instrux + 4787,
    instrux + 4788,
    instrux + 4789,
    instrux + 4790,
    instrux + 4791,
    instrux + 4792,
    instrux + 4793,
    instrux + 4794,
    instrux + 4795,
    instrux + 4796,
    instrux + 4797,
    instrux + 4798,
    instrux + 4799,
    instrux + 4800,
    instrux + 4801,
    instrux + 4802,
    instrux + 4803,
    instrux + 4804,
    instrux + 4805,
    instrux + 4806,
    instrux + 4807,
    instrux + 4808,
    instrux + 4809,
    instrux + 4810,
    instrux + 4811,
    instrux + 4812,
    instrux + 4813,
    instrux + 4814,
    instrux + 4815,
    instrux + 4816,
    instrux + 4817,
    instrux + 4818,
    instrux + 4819,
    instrux + 4820,
    instrux + 4821,
    instrux + 4822,
    instrux + 4823,
    instrux + 4824,
    instrux + 4825,
    instrux + 4826,
    instrux + 4827,
    instrux + 4828,
    instrux + 4829,
    instrux + 4830,
    instrux + 4831,
    instrux + 4832,
    instrux + 4833,
    instrux + 4834,
    instrux + 4835,
    instrux + 4836,
    instrux + 4837,
    instrux + 4838,
    instrux + 4839,
    instrux + 4840,
    instrux + 4841,
    instrux + 4842,
    instrux + 4843,
    instrux + 4844,
    instrux + 4845,
    instrux + 4846,
    instrux + 4847,
    instrux + 4848,
    instrux + 4849,
    instrux + 4850,
    instrux + 4851,
    instrux + 4852,
    instrux + 4853,
    instrux + 4854,
    instrux + 4855,
    instrux + 4856,
    instrux + 4857,
    instrux + 4858,
    instrux + 4859,
    instrux + 4860,
    instrux + 4861,
    instrux + 4862,
    instrux + 4863,
    instrux + 4864,
    instrux + 4865,
    instrux + 4866,
    instrux + 4867,
    instrux + 4868,
    instrux + 4869,
    instrux + 4870,
    instrux + 4871,
    instrux + 4872,
    instrux + 4873,
    instrux + 4874,
    instrux + 4875,
    instrux + 4876,
    instrux + 4877,
    instrux + 4878,
    instrux + 4879,
    instrux + 4880,
    instrux + 4881,
    instrux + 4882,
    instrux + 4883,
    instrux + 4884,
    instrux + 4885,
    instrux + 4886,
    instrux + 4887,
    instrux + 4888,
    instrux + 4889,
    instrux + 4890,
    instrux + 4891,
    instrux + 4892,
    instrux + 4893,
    instrux + 4894,
    instrux + 4895,
    instrux + 4896,
    instrux + 4897,
    instrux + 4898,
    instrux + 4899,
    instrux + 4900,
    instrux + 4901,
    instrux + 4902,
    instrux + 4903,
    instrux + 4904,
    instrux + 4905,
    instrux + 4906,
    instrux + 4907,
    instrux + 4908,
    instrux + 4909,
    instrux + 4910,
    instrux + 4911,
    instrux + 4912,
    instrux + 4913,
    instrux + 4914,
    instrux + 4915,
    instrux + 4916,
    instrux + 4917,
    instrux + 4918,
    instrux + 4919,
    instrux + 4920,
    instrux + 4921,
    instrux + 4922,
    instrux + 4923,
    instrux + 4924,
    instrux + 4925,
    instrux + 4926,
    instrux + 4927,
    instrux + 4928,
    instrux + 4929,
    instrux + 4930,
    instrux + 4931,
    instrux + 4932,
    instrux + 4933,
    instrux + 4934,
    instrux + 4935,
    instrux + 4936,
    instrux + 4937,
    instrux + 4938,
    instrux + 4939,
    instrux + 4940,
    instrux + 4941,
    instrux + 4942,
    instrux + 4943,
    instrux + 4944,
    instrux + 4945,
    instrux + 4946,
    instrux + 4947,
    instrux + 4948,
    instrux + 4949,
    instrux + 4950,
    instrux + 4951,
    instrux + 4952,
    instrux + 4953,
    instrux + 4954,
    instrux + 4955,
    instrux + 4956,
    instrux + 4957,
    instrux + 4958,
    instrux + 4959,
    instrux + 4960,
    instrux + 4961,
    instrux + 4962,
    instrux + 4963,
    instrux + 4964,
    instrux + 4965,
    instrux + 4966,
    instrux + 4967,
    instrux + 4968,
    instrux + 4969,
    instrux + 4970,
    instrux + 4971,
    instrux + 4972,
    instrux + 4973,
    instrux + 4974,
    instrux + 4975,
    instrux + 4976,
    instrux + 4977,
    instrux + 4978,
    instrux + 4979,
    instrux + 4980,
    instrux + 4981,
    instrux + 4982,
    instrux + 4983,
    instrux + 4984,
    instrux + 4985,
    instrux + 4986,
    instrux + 4987,
    instrux + 4988,
    instrux + 4989,
    instrux + 4990,
    instrux + 4991,
    instrux + 4992,
    instrux + 4993,
    instrux + 4994,
    instrux + 4995,
    instrux + 4996,
    instrux + 4997,
    instrux + 4998,
    instrux + 4999,
    instrux + 5000,
    instrux + 5001,
    instrux + 5002,
    instrux + 5003,
    instrux + 5004,
    instrux + 5005,
    instrux + 5006,
    instrux + 5007,
    instrux + 5008,
    instrux + 5009,
    instrux + 5010,
    instrux + 5011,
    instrux + 5012,
    instrux + 5013,
    instrux + 5014,
    instrux + 5015,
    instrux + 5016,
    instrux + 5017,
    instrux + 5018,
    instrux + 5019,
    instrux + 5020,
    instrux + 5021,
    instrux + 5022,
    instrux + 5023,
    instrux + 5024,
    instrux + 5025,
    instrux + 5026,
    instrux + 5027,
    instrux + 5028,
    instrux + 5029,
    instrux + 5030,
    instrux + 5031,
    instrux + 5032,
    instrux + 5033,
    instrux + 5034,
    instrux + 5035,
    instrux + 5036,
    instrux + 5037,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c4[] = {
    instrux + 6083,
    instrux + 6084,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c5[] = {
    instrux + 6064,
    instrux + 6065,
    instrux + 6066,
    NULL
};

static const struct itemplate * const itbl_evex_map1_c6[] = {
    instrux + 6662,
    instrux + 6663,
    instrux + 6664,
    instrux + 6665,
    instrux + 6666,
    instrux + 6667,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d1[] = {
    instrux + 6484,
    instrux + 6485,
    instrux + 6486,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d2[] = {
    instrux + 6460,
    instrux + 6461,
    instrux + 6462,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d3[] = {
    instrux + 6469,
    instrux + 6470,
    instrux + 6471,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d4[] = {
    instrux + 5591,
    instrux + 5592,
    instrux + 5593,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d5[] = {
    instrux + 6337,
    instrux + 6338,
    instrux + 6339,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d6[] = {
    instrux + 5514,
    instrux + 7463,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d8[] = {
    instrux + 6505,
    instrux + 6506,
    instrux + 6507,
    NULL
};

static const struct itemplate * const itbl_evex_map1_d9[] = {
    instrux + 6508,
    instrux + 6509,
    instrux + 6510,
    NULL
};

static const struct itemplate * const itbl_evex_map1_da[] = {
    instrux + 6139,
    instrux + 6140,
    instrux + 6141,
    NULL
};

static const struct itemplate * const itbl_evex_map1_db[] = {
    instrux + 5612,
    instrux + 5613,
    instrux + 5614,
    instrux + 5621,
    instrux + 5622,
    instrux + 5623,
    NULL
};

static const struct itemplate * const itbl_evex_map1_dc[] = {
    instrux + 5600,
    instrux + 5601,
    instrux + 5602,
    NULL
};

static const struct itemplate * const itbl_evex_map1_dd[] = {
    instrux + 5603,
    instrux + 5604,
    instrux + 5605,
    NULL
};

static const struct itemplate * const itbl_evex_map1_de[] = {
    instrux + 6115,
    instrux + 6116,
    instrux + 6117,
    NULL
};

static const struct itemplate * const itbl_evex_map1_df[] = {
    instrux + 5615,
    instrux + 5616,
    instrux + 5617,
    instrux + 5618,
    instrux + 5619,
    instrux + 5620,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e0[] = {
    instrux + 5624,
    instrux + 5625,
    instrux + 5626,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e1[] = {
    instrux + 6454,
    instrux + 6455,
    instrux + 6456,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e2[] = {
    instrux + 6433,
    instrux + 6434,
    instrux + 6435,
    instrux + 6439,
    instrux + 6440,
    instrux + 6441,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e3[] = {
    instrux + 5627,
    instrux + 5628,
    instrux + 5629,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e4[] = {
    instrux + 6325,
    instrux + 6326,
    instrux + 6327,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e5[] = {
    instrux + 6328,
    instrux + 6329,
    instrux + 6330,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e6[] = {
    instrux + 5052,
    instrux + 5053,
    instrux + 5054,
    instrux + 5058,
    instrux + 5059,
    instrux + 5060,
    instrux + 5097,
    instrux + 5098,
    instrux + 5099,
    instrux + 5117,
    instrux + 5118,
    instrux + 5119,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e7[] = {
    instrux + 5499,
    instrux + 5500,
    instrux + 5501,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e8[] = {
    instrux + 6499,
    instrux + 6500,
    instrux + 6501,
    NULL
};

static const struct itemplate * const itbl_evex_map1_e9[] = {
    instrux + 6502,
    instrux + 6503,
    instrux + 6504,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ea[] = {
    instrux + 6136,
    instrux + 6137,
    instrux + 6138,
    NULL
};

static const struct itemplate * const itbl_evex_map1_eb[] = {
    instrux + 6346,
    instrux + 6347,
    instrux + 6348,
    instrux + 6349,
    instrux + 6350,
    instrux + 6351,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ec[] = {
    instrux + 5594,
    instrux + 5595,
    instrux + 5596,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ed[] = {
    instrux + 5597,
    instrux + 5598,
    instrux + 5599,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ee[] = {
    instrux + 6112,
    instrux + 6113,
    instrux + 6114,
    NULL
};

static const struct itemplate * const itbl_evex_map1_ef[] = {
    instrux + 6568,
    instrux + 6569,
    instrux + 6570,
    instrux + 6571,
    instrux + 6572,
    instrux + 6573,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f1[] = {
    instrux + 6427,
    instrux + 6428,
    instrux + 6429,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f2[] = {
    instrux + 6403,
    instrux + 6404,
    instrux + 6405,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f3[] = {
    instrux + 6412,
    instrux + 6413,
    instrux + 6414,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f4[] = {
    instrux + 6343,
    instrux + 6344,
    instrux + 6345,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f5[] = {
    instrux + 6100,
    instrux + 6101,
    instrux + 6102,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f6[] = {
    instrux + 6376,
    instrux + 6377,
    instrux + 6378,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f8[] = {
    instrux + 6490,
    instrux + 6491,
    instrux + 6492,
    NULL
};

static const struct itemplate * const itbl_evex_map1_f9[] = {
    instrux + 6511,
    instrux + 6512,
    instrux + 6513,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fa[] = {
    instrux + 6493,
    instrux + 6494,
    instrux + 6495,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fb[] = {
    instrux + 6496,
    instrux + 6497,
    instrux + 6498,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fc[] = {
    instrux + 5585,
    instrux + 5586,
    instrux + 5587,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fd[] = {
    instrux + 5606,
    instrux + 5607,
    instrux + 5608,
    NULL
};

static const struct itemplate * const itbl_evex_map1_fe[] = {
    instrux + 5588,
    instrux + 5589,
    instrux + 5590,
    NULL
};

static const struct itemplate * const * const itbl_evex_map1[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_10,
    itbl_evex_map1_11,
    itbl_evex_map1_12,
    itbl_evex_map1_13,
    itbl_evex_map1_14,
    itbl_evex_map1_15,
    itbl_evex_map1_16,
    itbl_evex_map1_17,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_28,
    itbl_evex_map1_29,
    itbl_evex_map1_2a,
    itbl_evex_map1_2b,
    itbl_evex_map1_2c,
    itbl_evex_map1_2d,
    itbl_evex_map1_2e,
    itbl_evex_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_41,
    itbl_evex_map1_42,
    NULL,
    itbl_evex_map1_44,
    itbl_evex_map1_45,
    itbl_evex_map1_46,
    itbl_evex_map1_47,
    NULL,
    NULL,
    itbl_evex_map1_4a,
    itbl_evex_map1_4b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_51,
    NULL,
    NULL,
    itbl_evex_map1_54,
    itbl_evex_map1_55,
    itbl_evex_map1_56,
    itbl_evex_map1_57,
    itbl_evex_map1_58,
    itbl_evex_map1_59,
    itbl_evex_map1_5a,
    itbl_evex_map1_5b,
    itbl_evex_map1_5c,
    itbl_evex_map1_5d,
    itbl_evex_map1_5e,
    itbl_evex_map1_5f,
    itbl_evex_map1_60,
    itbl_evex_map1_61,
    itbl_evex_map1_62,
    itbl_evex_map1_63,
    itbl_evex_map1_64,
    itbl_evex_map1_65,
    itbl_evex_map1_66,
    itbl_evex_map1_67,
    itbl_evex_map1_68,
    itbl_evex_map1_69,
    itbl_evex_map1_6a,
    itbl_evex_map1_6b,
    itbl_evex_map1_6c,
    itbl_evex_map1_6d,
    itbl_evex_map1_6e,
    itbl_evex_map1_6f,
    itbl_evex_map1_70,
    itbl_evex_map1_71,
    itbl_evex_map1_72,
    itbl_evex_map1_73,
    itbl_evex_map1_74,
    itbl_evex_map1_75,
    itbl_evex_map1_76,
    NULL,
    itbl_evex_map1_78,
    itbl_evex_map1_79,
    itbl_evex_map1_7a,
    itbl_evex_map1_7b,
    NULL,
    NULL,
    itbl_evex_map1_7e,
    itbl_evex_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_90,
    itbl_evex_map1_91,
    itbl_evex_map1_92,
    itbl_evex_map1_93,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_98,
    itbl_evex_map1_99,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_c2,
    NULL,
    itbl_evex_map1_c4,
    itbl_evex_map1_c5,
    itbl_evex_map1_c6,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map1_d1,
    itbl_evex_map1_d2,
    itbl_evex_map1_d3,
    itbl_evex_map1_d4,
    itbl_evex_map1_d5,
    itbl_evex_map1_d6,
    NULL,
    itbl_evex_map1_d8,
    itbl_evex_map1_d9,
    itbl_evex_map1_da,
    itbl_evex_map1_db,
    itbl_evex_map1_dc,
    itbl_evex_map1_dd,
    itbl_evex_map1_de,
    itbl_evex_map1_df,
    itbl_evex_map1_e0,
    itbl_evex_map1_e1,
    itbl_evex_map1_e2,
    itbl_evex_map1_e3,
    itbl_evex_map1_e4,
    itbl_evex_map1_e5,
    itbl_evex_map1_e6,
    itbl_evex_map1_e7,
    itbl_evex_map1_e8,
    itbl_evex_map1_e9,
    itbl_evex_map1_ea,
    itbl_evex_map1_eb,
    itbl_evex_map1_ec,
    itbl_evex_map1_ed,
    itbl_evex_map1_ee,
    itbl_evex_map1_ef,
    NULL,
    itbl_evex_map1_f1,
    itbl_evex_map1_f2,
    itbl_evex_map1_f3,
    itbl_evex_map1_f4,
    itbl_evex_map1_f5,
    itbl_evex_map1_f6,
    NULL,
    itbl_evex_map1_f8,
    itbl_evex_map1_f9,
    itbl_evex_map1_fa,
    itbl_evex_map1_fb,
    itbl_evex_map1_fc,
    itbl_evex_map1_fd,
    itbl_evex_map1_fe,
    NULL,
};

static const struct itemplate * const itbl_evex_map2_00[] = {
    instrux + 6391,
    instrux + 6392,
    instrux + 6393,
    NULL
};

static const struct itemplate * const itbl_evex_map2_04[] = {
    instrux + 6097,
    instrux + 6098,
    instrux + 6099,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0b[] = {
    instrux + 6322,
    instrux + 6323,
    instrux + 6324,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0c[] = {
    instrux + 6007,
    instrux + 6008,
    instrux + 6009,
    NULL
};

static const struct itemplate * const itbl_evex_map2_0d[] = {
    instrux + 6001,
    instrux + 6002,
    instrux + 6003,
    NULL
};

static const struct itemplate * const itbl_evex_map2_10[] = {
    instrux + 6286,
    instrux + 6287,
    instrux + 6288,
    instrux + 6289,
    instrux + 6290,
    instrux + 6291,
    instrux + 6481,
    instrux + 6482,
    instrux + 6483,
    NULL
};

static const struct itemplate * const itbl_evex_map2_11[] = {
    instrux + 6256,
    instrux + 6257,
    instrux + 6258,
    instrux + 6259,
    instrux + 6260,
    instrux + 6261,
    instrux + 6451,
    instrux + 6452,
    instrux + 6453,
    NULL
};

static const struct itemplate * const itbl_evex_map2_12[] = {
    instrux + 6268,
    instrux + 6269,
    instrux + 6270,
    instrux + 6271,
    instrux + 6272,
    instrux + 6273,
    instrux + 6424,
    instrux + 6425,
    instrux + 6426,
    NULL
};

static const struct itemplate * const itbl_evex_map2_13[] = {
    instrux + 5073,
    instrux + 5074,
    instrux + 5075,
    instrux + 6262,
    instrux + 6263,
    instrux + 6264,
    instrux + 6265,
    instrux + 6266,
    instrux + 6267,
    instrux + 6950,
    instrux + 6951,
    instrux + 6952,
    NULL
};

static const struct itemplate * const itbl_evex_map2_14[] = {
    instrux + 6280,
    instrux + 6281,
    instrux + 6282,
    instrux + 6283,
    instrux + 6284,
    instrux + 6285,
    instrux + 6370,
    instrux + 6371,
    instrux + 6372,
    instrux + 6373,
    instrux + 6374,
    instrux + 6375,
    NULL
};

static const struct itemplate * const itbl_evex_map2_15[] = {
    instrux + 6274,
    instrux + 6275,
    instrux + 6276,
    instrux + 6277,
    instrux + 6278,
    instrux + 6279,
    instrux + 6358,
    instrux + 6359,
    instrux + 6360,
    instrux + 6361,
    instrux + 6362,
    instrux + 6363,
    NULL
};

static const struct itemplate * const itbl_evex_map2_16[] = {
    instrux + 6012,
    instrux + 6013,
    instrux + 6014,
    instrux + 6015,
    NULL
};

static const struct itemplate * const itbl_evex_map2_18[] = {
    instrux + 4656,
    instrux + 4657,
    instrux + 4658,
    instrux + 4659,
    instrux + 4660,
    instrux + 4661,
    NULL
};

static const struct itemplate * const itbl_evex_map2_19[] = {
    instrux + 4635,
    instrux + 4636,
    instrux + 4652,
    instrux + 4653,
    instrux + 4654,
    instrux + 4655,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1a[] = {
    instrux + 4637,
    instrux + 4638,
    instrux + 4640,
    instrux + 4641,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1b[] = {
    instrux + 4639,
    instrux + 4642,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1c[] = {
    instrux + 5561,
    instrux + 5562,
    instrux + 5563,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1d[] = {
    instrux + 5570,
    instrux + 5571,
    instrux + 5572,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1e[] = {
    instrux + 5564,
    instrux + 5565,
    instrux + 5566,
    NULL
};

static const struct itemplate * const itbl_evex_map2_1f[] = {
    instrux + 5567,
    instrux + 5568,
    instrux + 5569,
    NULL
};

static const struct itemplate * const itbl_evex_map2_20[] = {
    instrux + 6232,
    instrux + 6233,
    instrux + 6234,
    instrux + 6235,
    instrux + 6236,
    instrux + 6237,
    instrux + 6244,
    instrux + 6245,
    instrux + 6246,
    NULL
};

static const struct itemplate * const itbl_evex_map2_21[] = {
    instrux + 6202,
    instrux + 6203,
    instrux + 6204,
    instrux + 6205,
    instrux + 6206,
    instrux + 6207,
    instrux + 6238,
    instrux + 6239,
    instrux + 6240,
    NULL
};

static const struct itemplate * const itbl_evex_map2_22[] = {
    instrux + 6214,
    instrux + 6215,
    instrux + 6216,
    instrux + 6217,
    instrux + 6218,
    instrux + 6219,
    instrux + 6241,
    instrux + 6242,
    instrux + 6243,
    NULL
};

static const struct itemplate * const itbl_evex_map2_23[] = {
    instrux + 6208,
    instrux + 6209,
    instrux + 6210,
    instrux + 6211,
    instrux + 6212,
    instrux + 6213,
    instrux + 6250,
    instrux + 6251,
    instrux + 6252,
    NULL
};

static const struct itemplate * const itbl_evex_map2_24[] = {
    instrux + 6226,
    instrux + 6227,
    instrux + 6228,
    instrux + 6229,
    instrux + 6230,
    instrux + 6231,
    instrux + 6253,
    instrux + 6254,
    instrux + 6255,
    NULL
};

static const struct itemplate * const itbl_evex_map2_25[] = {
    instrux + 6220,
    instrux + 6221,
    instrux + 6222,
    instrux + 6223,
    instrux + 6224,
    instrux + 6225,
    instrux + 6247,
    instrux + 6248,
    instrux + 6249,
    NULL
};

static const struct itemplate * const itbl_evex_map2_26[] = {
    instrux + 6520,
    instrux + 6521,
    instrux + 6522,
    instrux + 6529,
    instrux + 6530,
    instrux + 6531,
    instrux + 6532,
    instrux + 6533,
    instrux + 6534,
    instrux + 6541,
    instrux + 6542,
    instrux + 6543,
    NULL
};

static const struct itemplate * const itbl_evex_map2_27[] = {
    instrux + 6523,
    instrux + 6524,
    instrux + 6525,
    instrux + 6526,
    instrux + 6527,
    instrux + 6528,
    instrux + 6535,
    instrux + 6536,
    instrux + 6537,
    instrux + 6538,
    instrux + 6539,
    instrux + 6540,
    NULL
};

static const struct itemplate * const itbl_evex_map2_28[] = {
    instrux + 6169,
    instrux + 6170,
    instrux + 6171,
    instrux + 6178,
    instrux + 6179,
    instrux + 6180,
    instrux + 6319,
    instrux + 6320,
    instrux + 6321,
    NULL
};

static const struct itemplate * const itbl_evex_map2_29[] = {
    instrux + 5699,
    instrux + 5700,
    instrux + 5701,
    instrux + 6151,
    instrux + 6152,
    instrux + 6153,
    instrux + 6292,
    instrux + 6293,
    instrux + 6294,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2a[] = {
    instrux + 5502,
    instrux + 5503,
    instrux + 5504,
    instrux + 5666,
    instrux + 5667,
    instrux + 5668,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2b[] = {
    instrux + 5579,
    instrux + 5580,
    instrux + 5581,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2c[] = {
    instrux + 6626,
    instrux + 6627,
    instrux + 6628,
    instrux + 6629,
    instrux + 6630,
    instrux + 6631,
    NULL
};

static const struct itemplate * const itbl_evex_map2_2d[] = {
    instrux + 6632,
    instrux + 6633,
    NULL
};

static const struct itemplate * const itbl_evex_map2_30[] = {
    instrux + 6295,
    instrux + 6296,
    instrux + 6297,
    instrux + 6298,
    instrux + 6299,
    instrux + 6300,
    instrux + 6307,
    instrux + 6308,
    instrux + 6309,
    NULL
};

static const struct itemplate * const itbl_evex_map2_31[] = {
    instrux + 6157,
    instrux + 6158,
    instrux + 6159,
    instrux + 6160,
    instrux + 6161,
    instrux + 6162,
    instrux + 6301,
    instrux + 6302,
    instrux + 6303,
    NULL
};

static const struct itemplate * const itbl_evex_map2_32[] = {
    instrux + 6184,
    instrux + 6185,
    instrux + 6186,
    instrux + 6187,
    instrux + 6188,
    instrux + 6189,
    instrux + 6304,
    instrux + 6305,
    instrux + 6306,
    NULL
};

static const struct itemplate * const itbl_evex_map2_33[] = {
    instrux + 6163,
    instrux + 6164,
    instrux + 6165,
    instrux + 6166,
    instrux + 6167,
    instrux + 6168,
    instrux + 6313,
    instrux + 6314,
    instrux + 6315,
    NULL
};

static const struct itemplate * const itbl_evex_map2_34[] = {
    instrux + 6196,
    instrux + 6197,
    instrux + 6198,
    instrux + 6199,
    instrux + 6200,
    instrux + 6201,
    instrux + 6316,
    instrux + 6317,
    instrux + 6318,
    NULL
};

static const struct itemplate * const itbl_evex_map2_35[] = {
    instrux + 6190,
    instrux + 6191,
    instrux + 6192,
    instrux + 6193,
    instrux + 6194,
    instrux + 6195,
    instrux + 6310,
    instrux + 6311,
    instrux + 6312,
    NULL
};

static const struct itemplate * const itbl_evex_map2_36[] = {
    instrux + 5978,
    instrux + 5979,
    instrux + 6018,
    instrux + 6019,
    NULL
};

static const struct itemplate * const itbl_evex_map2_37[] = {
    instrux + 5711,
    instrux + 5712,
    instrux + 5713,
    NULL
};

static const struct itemplate * const itbl_evex_map2_38[] = {
    instrux + 6127,
    instrux + 6128,
    instrux + 6129,
    instrux + 6172,
    instrux + 6173,
    instrux + 6174,
    instrux + 6175,
    instrux + 6176,
    instrux + 6177,
    NULL
};

static const struct itemplate * const itbl_evex_map2_39[] = {
    instrux + 6130,
    instrux + 6131,
    instrux + 6132,
    instrux + 6133,
    instrux + 6134,
    instrux + 6135,
    instrux + 6154,
    instrux + 6155,
    instrux + 6156,
    instrux + 6181,
    instrux + 6182,
    instrux + 6183,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3a[] = {
    instrux + 5669,
    instrux + 5670,
    instrux + 5671,
    instrux + 6148,
    instrux + 6149,
    instrux + 6150,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3b[] = {
    instrux + 6142,
    instrux + 6143,
    instrux + 6144,
    instrux + 6145,
    instrux + 6146,
    instrux + 6147,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3c[] = {
    instrux + 6103,
    instrux + 6104,
    instrux + 6105,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3d[] = {
    instrux + 6106,
    instrux + 6107,
    instrux + 6108,
    instrux + 6109,
    instrux + 6110,
    instrux + 6111,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3e[] = {
    instrux + 6124,
    instrux + 6125,
    instrux + 6126,
    NULL
};

static const struct itemplate * const itbl_evex_map2_3f[] = {
    instrux + 6118,
    instrux + 6119,
    instrux + 6120,
    instrux + 6121,
    instrux + 6122,
    instrux + 6123,
    NULL
};

static const struct itemplate * const itbl_evex_map2_40[] = {
    instrux + 6331,
    instrux + 6332,
    instrux + 6333,
    instrux + 6334,
    instrux + 6335,
    instrux + 6336,
    NULL
};

static const struct itemplate * const itbl_evex_map2_42[] = {
    instrux + 5385,
    instrux + 5386,
    instrux + 5387,
    instrux + 5388,
    instrux + 5389,
    instrux + 5390,
    NULL
};

static const struct itemplate * const itbl_evex_map2_43[] = {
    instrux + 5391,
    instrux + 5392,
    NULL
};

static const struct itemplate * const itbl_evex_map2_44[] = {
    instrux + 6085,
    instrux + 6086,
    instrux + 6087,
    instrux + 6088,
    instrux + 6089,
    instrux + 6090,
    NULL
};

static const struct itemplate * const itbl_evex_map2_45[] = {
    instrux + 6475,
    instrux + 6476,
    instrux + 6477,
    instrux + 6478,
    instrux + 6479,
    instrux + 6480,
    NULL
};

static const struct itemplate * const itbl_evex_map2_46[] = {
    instrux + 6445,
    instrux + 6446,
    instrux + 6447,
    instrux + 6448,
    instrux + 6449,
    instrux + 6450,
    NULL
};

static const struct itemplate * const itbl_evex_map2_47[] = {
    instrux + 6418,
    instrux + 6419,
    instrux + 6420,
    instrux + 6421,
    instrux + 6422,
    instrux + 6423,
    NULL
};

static const struct itemplate * const itbl_evex_map2_49[] = {
    instrux + 6873,
    instrux + 6875,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4a[] = {
    instrux + 6893,
    instrux + 6895,
    instrux + 6906,
    instrux + 6921,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4b[] = {
    instrux + 6885,
    instrux + 6887,
    instrux + 6890,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4c[] = {
    instrux + 6582,
    instrux + 6583,
    instrux + 6584,
    instrux + 6585,
    instrux + 6586,
    instrux + 6587,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4d[] = {
    instrux + 6588,
    instrux + 6589,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4e[] = {
    instrux + 6614,
    instrux + 6615,
    instrux + 6616,
    instrux + 6617,
    instrux + 6618,
    instrux + 6619,
    NULL
};

static const struct itemplate * const itbl_evex_map2_4f[] = {
    instrux + 6620,
    instrux + 6621,
    NULL
};

static const struct itemplate * const itbl_evex_map2_50[] = {
    instrux + 6792,
    instrux + 6793,
    instrux + 6794,
    instrux + 7343,
    instrux + 7344,
    instrux + 7345,
    instrux + 7349,
    instrux + 7350,
    instrux + 7351,
    instrux + 7355,
    instrux + 7356,
    instrux + 7357,
    NULL
};

static const struct itemplate * const itbl_evex_map2_51[] = {
    instrux + 6795,
    instrux + 6796,
    instrux + 6797,
    instrux + 7346,
    instrux + 7347,
    instrux + 7348,
    instrux + 7352,
    instrux + 7353,
    instrux + 7354,
    instrux + 7358,
    instrux + 7359,
    instrux + 7360,
    NULL
};

static const struct itemplate * const itbl_evex_map2_52[] = {
    instrux + 6798,
    instrux + 6799,
    instrux + 6800,
    instrux + 6824,
    instrux + 6863,
    instrux + 6864,
    instrux + 6865,
    instrux + 7337,
    instrux + 7338,
    instrux + 7339,
    NULL
};

static const struct itemplate * const itbl_evex_map2_53[] = {
    instrux + 6801,
    instrux + 6802,
    instrux + 6803,
    instrux + 6823,
    NULL
};

static const struct itemplate * const itbl_evex_map2_54[] = {
    instrux + 6804,
    instrux + 6805,
    instrux + 6806,
    instrux + 6807,
    instrux + 6808,
    instrux + 6809,
    NULL
};

static const struct itemplate * const itbl_evex_map2_55[] = {
    instrux + 6810,
    instrux + 6811,
    instrux + 6812,
    instrux + 6813,
    instrux + 6814,
    instrux + 6815,
    NULL
};

static const struct itemplate * const itbl_evex_map2_58[] = {
    instrux + 5657,
    instrux + 5658,
    instrux + 5659,
    instrux + 5660,
    instrux + 5661,
    instrux + 5662,
    NULL
};

static const struct itemplate * const itbl_evex_map2_59[] = {
    instrux + 4643,
    instrux + 4644,
    instrux + 4645,
    instrux + 5672,
    instrux + 5673,
    instrux + 5674,
    instrux + 5675,
    instrux + 5676,
    instrux + 5677,
    NULL
};

static const struct itemplate * const itbl_evex_map2_5a[] = {
    instrux + 4646,
    instrux + 4647,
    instrux + 4649,
    instrux + 4650,
    NULL
};

static const struct itemplate * const itbl_evex_map2_5b[] = {
    instrux + 4648,
    instrux + 4651,
    NULL
};

static const struct itemplate * const itbl_evex_map2_62[] = {
    instrux + 6750,
    instrux + 6751,
    instrux + 6752,
    instrux + 6753,
    instrux + 6754,
    instrux + 6755,
    NULL
};

static const struct itemplate * const itbl_evex_map2_63[] = {
    instrux + 6738,
    instrux + 6739,
    instrux + 6740,
    instrux + 6741,
    instrux + 6742,
    instrux + 6743,
    instrux + 6744,
    instrux + 6745,
    instrux + 6746,
    instrux + 6747,
    instrux + 6748,
    instrux + 6749,
    NULL
};

static const struct itemplate * const itbl_evex_map2_64[] = {
    instrux + 5633,
    instrux + 5634,
    instrux + 5635,
    instrux + 5636,
    instrux + 5637,
    instrux + 5638,
    NULL
};

static const struct itemplate * const itbl_evex_map2_65[] = {
    instrux + 4629,
    instrux + 4630,
    instrux + 4631,
    instrux + 4632,
    instrux + 4633,
    instrux + 4634,
    NULL
};

static const struct itemplate * const itbl_evex_map2_66[] = {
    instrux + 5630,
    instrux + 5631,
    instrux + 5632,
    instrux + 5639,
    instrux + 5640,
    instrux + 5641,
    NULL
};

static const struct itemplate * const itbl_evex_map2_67[] = {
    instrux + 7319,
    instrux + 7320,
    instrux + 7321,
    NULL
};

static const struct itemplate * const itbl_evex_map2_68[] = {
    instrux + 6866,
    instrux + 6867,
    instrux + 6868,
    instrux + 6869,
    instrux + 6870,
    instrux + 6871,
    NULL
};

static const struct itemplate * const itbl_evex_map2_6d[] = {
    instrux + 6908,
    instrux + 6910,
    instrux + 6912,
    instrux + 6914,
    NULL
};

static const struct itemplate * const itbl_evex_map2_70[] = {
    instrux + 6765,
    instrux + 6766,
    instrux + 6767,
    NULL
};

static const struct itemplate * const itbl_evex_map2_71[] = {
    instrux + 6768,
    instrux + 6769,
    instrux + 6770,
    instrux + 6771,
    instrux + 6772,
    instrux + 6773,
    NULL
};

static const struct itemplate * const itbl_evex_map2_72[] = {
    instrux + 6783,
    instrux + 6784,
    instrux + 6785,
    instrux + 6857,
    instrux + 6858,
    instrux + 6859,
    instrux + 6860,
    instrux + 6861,
    instrux + 6862,
    NULL
};

static const struct itemplate * const itbl_evex_map2_73[] = {
    instrux + 6786,
    instrux + 6787,
    instrux + 6788,
    instrux + 6789,
    instrux + 6790,
    instrux + 6791,
    NULL
};

static const struct itemplate * const itbl_evex_map2_74[] = {
    instrux + 7295,
    instrux + 7296,
    instrux + 7297,
    instrux + 7307,
    instrux + 7308,
    instrux + 7309,
    instrux + 7322,
    instrux + 7323,
    instrux + 7324,
    NULL
};

static const struct itemplate * const itbl_evex_map2_75[] = {
    instrux + 5980,
    instrux + 5981,
    instrux + 5982,
    instrux + 5995,
    instrux + 5996,
    instrux + 5997,
    NULL
};

static const struct itemplate * const itbl_evex_map2_76[] = {
    instrux + 5983,
    instrux + 5984,
    instrux + 5985,
    instrux + 5992,
    instrux + 5993,
    instrux + 5994,
    NULL
};

static const struct itemplate * const itbl_evex_map2_77[] = {
    instrux + 5986,
    instrux + 5987,
    instrux + 5988,
    instrux + 5989,
    instrux + 5990,
    instrux + 5991,
    NULL
};

static const struct itemplate * const itbl_evex_map2_78[] = {
    instrux + 5642,
    instrux + 5643,
    instrux + 5644,
    NULL
};

static const struct itemplate * const itbl_evex_map2_79[] = {
    instrux + 5681,
    instrux + 5682,
    instrux + 5683,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7a[] = {
    instrux + 5645,
    instrux + 5646,
    instrux + 5647,
    instrux + 5648,
    instrux + 5649,
    instrux + 5650,
    instrux + 5651,
    instrux + 5652,
    instrux + 5653,
    instrux + 5654,
    instrux + 5655,
    instrux + 5656,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7b[] = {
    instrux + 5684,
    instrux + 5685,
    instrux + 5686,
    instrux + 5687,
    instrux + 5688,
    instrux + 5689,
    instrux + 5690,
    instrux + 5691,
    instrux + 5692,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7c[] = {
    instrux + 5663,
    instrux + 5664,
    instrux + 5665,
    instrux + 5678,
    instrux + 5679,
    instrux + 5680,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7d[] = {
    instrux + 6020,
    instrux + 6021,
    instrux + 6022,
    instrux + 6035,
    instrux + 6036,
    instrux + 6037,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7e[] = {
    instrux + 6023,
    instrux + 6024,
    instrux + 6025,
    instrux + 6032,
    instrux + 6033,
    instrux + 6034,
    NULL
};

static const struct itemplate * const itbl_evex_map2_7f[] = {
    instrux + 6026,
    instrux + 6027,
    instrux + 6028,
    instrux + 6029,
    instrux + 6030,
    instrux + 6031,
    NULL
};

static const struct itemplate * const itbl_evex_map2_83[] = {
    instrux + 6340,
    instrux + 6341,
    instrux + 6342,
    NULL
};

static const struct itemplate * const itbl_evex_map2_88[] = {
    instrux + 5178,
    instrux + 5179,
    instrux + 5180,
    instrux + 5181,
    instrux + 5182,
    instrux + 5183,
    instrux + 5184,
    instrux + 5185,
    instrux + 5186,
    instrux + 5187,
    instrux + 5188,
    instrux + 5189,
    NULL
};

static const struct itemplate * const itbl_evex_map2_89[] = {
    instrux + 6041,
    instrux + 6042,
    instrux + 6043,
    instrux + 6044,
    instrux + 6045,
    instrux + 6046,
    instrux + 6047,
    instrux + 6048,
    instrux + 6049,
    instrux + 6050,
    instrux + 6051,
    instrux + 6052,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8a[] = {
    instrux + 5040,
    instrux + 5041,
    instrux + 5042,
    instrux + 5043,
    instrux + 5044,
    instrux + 5045,
    instrux + 5046,
    instrux + 5047,
    instrux + 5048,
    instrux + 5049,
    instrux + 5050,
    instrux + 5051,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8b[] = {
    instrux + 5957,
    instrux + 5958,
    instrux + 5959,
    instrux + 5960,
    instrux + 5961,
    instrux + 5962,
    instrux + 5963,
    instrux + 5964,
    instrux + 5965,
    instrux + 5966,
    instrux + 5967,
    instrux + 5968,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8d[] = {
    instrux + 5975,
    instrux + 5976,
    instrux + 5977,
    instrux + 6038,
    instrux + 6039,
    instrux + 6040,
    NULL
};

static const struct itemplate * const itbl_evex_map2_8f[] = {
    instrux + 6816,
    instrux + 6817,
    instrux + 6818,
    NULL
};

static const struct itemplate * const itbl_evex_map2_90[] = {
    instrux + 6067,
    instrux + 6068,
    instrux + 6069,
    instrux + 6070,
    instrux + 6071,
    instrux + 6072,
    NULL
};

static const struct itemplate * const itbl_evex_map2_91[] = {
    instrux + 6073,
    instrux + 6074,
    instrux + 6075,
    instrux + 6076,
    instrux + 6077,
    instrux + 6078,
    NULL
};

static const struct itemplate * const itbl_evex_map2_92[] = {
    instrux + 5365,
    instrux + 5366,
    instrux + 5367,
    instrux + 5368,
    instrux + 5369,
    instrux + 5370,
    NULL
};

static const struct itemplate * const itbl_evex_map2_93[] = {
    instrux + 5379,
    instrux + 5380,
    instrux + 5381,
    instrux + 5382,
    instrux + 5383,
    instrux + 5384,
    NULL
};

static const struct itemplate * const itbl_evex_map2_96[] = {
    instrux + 5249,
    instrux + 5250,
    instrux + 5251,
    instrux + 5252,
    instrux + 5253,
    instrux + 5254,
    NULL
};

static const struct itemplate * const itbl_evex_map2_97[] = {
    instrux + 5291,
    instrux + 5292,
    instrux + 5293,
    instrux + 5294,
    instrux + 5295,
    instrux + 5296,
    NULL
};

static const struct itemplate * const itbl_evex_map2_98[] = {
    instrux + 5225,
    instrux + 5226,
    instrux + 5227,
    instrux + 5228,
    instrux + 5229,
    instrux + 5230,
    NULL
};

static const struct itemplate * const itbl_evex_map2_99[] = {
    instrux + 5231,
    instrux + 5232,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9a[] = {
    instrux + 5267,
    instrux + 5268,
    instrux + 5269,
    instrux + 5270,
    instrux + 5271,
    instrux + 5272,
    instrux + 6819,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9b[] = {
    instrux + 5273,
    instrux + 5274,
    instrux + 6821,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9c[] = {
    instrux + 5309,
    instrux + 5310,
    instrux + 5311,
    instrux + 5312,
    instrux + 5313,
    instrux + 5314,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9d[] = {
    instrux + 5315,
    instrux + 5316,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9e[] = {
    instrux + 5333,
    instrux + 5334,
    instrux + 5335,
    instrux + 5336,
    instrux + 5337,
    instrux + 5338,
    NULL
};

static const struct itemplate * const itbl_evex_map2_9f[] = {
    instrux + 5339,
    instrux + 5340,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a0[] = {
    instrux + 6379,
    instrux + 6380,
    instrux + 6381,
    instrux + 6382,
    instrux + 6383,
    instrux + 6384,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a1[] = {
    instrux + 6385,
    instrux + 6386,
    instrux + 6387,
    instrux + 6388,
    instrux + 6389,
    instrux + 6390,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a2[] = {
    instrux + 6634,
    instrux + 6635,
    instrux + 6636,
    instrux + 6637,
    instrux + 6638,
    instrux + 6639,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a3[] = {
    instrux + 6648,
    instrux + 6649,
    instrux + 6650,
    instrux + 6651,
    instrux + 6652,
    instrux + 6653,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a6[] = {
    instrux + 5255,
    instrux + 5256,
    instrux + 5257,
    instrux + 5258,
    instrux + 5259,
    instrux + 5260,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a7[] = {
    instrux + 5297,
    instrux + 5298,
    instrux + 5299,
    instrux + 5300,
    instrux + 5301,
    instrux + 5302,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a8[] = {
    instrux + 5233,
    instrux + 5234,
    instrux + 5235,
    instrux + 5236,
    instrux + 5237,
    instrux + 5238,
    NULL
};

static const struct itemplate * const itbl_evex_map2_a9[] = {
    instrux + 5239,
    instrux + 5240,
    NULL
};

static const struct itemplate * const itbl_evex_map2_aa[] = {
    instrux + 5275,
    instrux + 5276,
    instrux + 5277,
    instrux + 5278,
    instrux + 5279,
    instrux + 5280,
    instrux + 6820,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ab[] = {
    instrux + 5281,
    instrux + 5282,
    instrux + 6822,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ac[] = {
    instrux + 5317,
    instrux + 5318,
    instrux + 5319,
    instrux + 5320,
    instrux + 5321,
    instrux + 5322,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ad[] = {
    instrux + 5323,
    instrux + 5324,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ae[] = {
    instrux + 5341,
    instrux + 5342,
    instrux + 5343,
    instrux + 5344,
    instrux + 5345,
    instrux + 5346,
    NULL
};

static const struct itemplate * const itbl_evex_map2_af[] = {
    instrux + 5347,
    instrux + 5348,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b4[] = {
    instrux + 6094,
    instrux + 6095,
    instrux + 6096,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b5[] = {
    instrux + 6091,
    instrux + 6092,
    instrux + 6093,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b6[] = {
    instrux + 5261,
    instrux + 5262,
    instrux + 5263,
    instrux + 5264,
    instrux + 5265,
    instrux + 5266,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b7[] = {
    instrux + 5303,
    instrux + 5304,
    instrux + 5305,
    instrux + 5306,
    instrux + 5307,
    instrux + 5308,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b8[] = {
    instrux + 5241,
    instrux + 5242,
    instrux + 5243,
    instrux + 5244,
    instrux + 5245,
    instrux + 5246,
    NULL
};

static const struct itemplate * const itbl_evex_map2_b9[] = {
    instrux + 5247,
    instrux + 5248,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ba[] = {
    instrux + 5283,
    instrux + 5284,
    instrux + 5285,
    instrux + 5286,
    instrux + 5287,
    instrux + 5288,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bb[] = {
    instrux + 5289,
    instrux + 5290,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bc[] = {
    instrux + 5325,
    instrux + 5326,
    instrux + 5327,
    instrux + 5328,
    instrux + 5329,
    instrux + 5330,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bd[] = {
    instrux + 5331,
    instrux + 5332,
    NULL
};

static const struct itemplate * const itbl_evex_map2_be[] = {
    instrux + 5349,
    instrux + 5350,
    instrux + 5351,
    instrux + 5352,
    instrux + 5353,
    instrux + 5354,
    NULL
};

static const struct itemplate * const itbl_evex_map2_bf[] = {
    instrux + 5355,
    instrux + 5356,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c4[] = {
    instrux + 5969,
    instrux + 5970,
    instrux + 5971,
    instrux + 5972,
    instrux + 5973,
    instrux + 5974,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c6[] = {
    instrux + 5371,
    instrux + 5372,
    instrux + 5375,
    instrux + 5376,
    instrux + 6640,
    instrux + 6641,
    instrux + 6644,
    instrux + 6645,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c7[] = {
    instrux + 5373,
    instrux + 5374,
    instrux + 5377,
    instrux + 5378,
    instrux + 6642,
    instrux + 6643,
    instrux + 6646,
    instrux + 6647,
    NULL
};

static const struct itemplate * const itbl_evex_map2_c8[] = {
    instrux + 5176,
    instrux + 5177,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ca[] = {
    instrux + 6590,
    instrux + 6591,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cb[] = {
    instrux + 6592,
    instrux + 6593,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cc[] = {
    instrux + 6622,
    instrux + 6623,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cd[] = {
    instrux + 6624,
    instrux + 6625,
    NULL
};

static const struct itemplate * const itbl_evex_map2_cf[] = {
    instrux + 6735,
    instrux + 6736,
    instrux + 6737,
    NULL
};

static const struct itemplate * const itbl_evex_map2_d2[] = {
    instrux + 7361,
    instrux + 7362,
    instrux + 7363,
    instrux + 7367,
    instrux + 7368,
    instrux + 7369,
    instrux + 7373,
    instrux + 7374,
    instrux + 7375,
    NULL
};

static const struct itemplate * const itbl_evex_map2_d3[] = {
    instrux + 7364,
    instrux + 7365,
    instrux + 7366,
    instrux + 7370,
    instrux + 7371,
    instrux + 7372,
    instrux + 7376,
    instrux + 7377,
    instrux + 7378,
    NULL
};

static const struct itemplate * const itbl_evex_map2_da[] = {
    instrux + 4417,
    instrux + 4418,
    instrux + 4419,
    instrux + 4422,
    instrux + 4423,
    instrux + 4424,
    NULL
};

static const struct itemplate * const itbl_evex_map2_dc[] = {
    instrux + 3110,
    instrux + 3111,
    instrux + 3118,
    NULL
};

static const struct itemplate * const itbl_evex_map2_dd[] = {
    instrux + 3112,
    instrux + 3113,
    instrux + 3119,
    NULL
};

static const struct itemplate * const itbl_evex_map2_de[] = {
    instrux + 3114,
    instrux + 3115,
    instrux + 3120,
    NULL
};

static const struct itemplate * const itbl_evex_map2_df[] = {
    instrux + 3116,
    instrux + 3117,
    instrux + 3121,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e0[] = {
    instrux + 2603,
    instrux + 2619,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e1[] = {
    instrux + 2600,
    instrux + 2616,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e2[] = {
    instrux + 2593,
    instrux + 2609,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e3[] = {
    instrux + 2597,
    instrux + 2613,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e4[] = {
    instrux + 2607,
    instrux + 2623,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e5[] = {
    instrux + 2602,
    instrux + 2618,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e6[] = {
    instrux + 2596,
    instrux + 2612,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e7[] = {
    instrux + 2592,
    instrux + 2608,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e8[] = {
    instrux + 2606,
    instrux + 2622,
    NULL
};

static const struct itemplate * const itbl_evex_map2_e9[] = {
    instrux + 2601,
    instrux + 2617,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ea[] = {
    instrux + 2604,
    instrux + 2620,
    NULL
};

static const struct itemplate * const itbl_evex_map2_eb[] = {
    instrux + 2605,
    instrux + 2621,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ec[] = {
    instrux + 2595,
    instrux + 2611,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ed[] = {
    instrux + 2599,
    instrux + 2615,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ee[] = {
    instrux + 2598,
    instrux + 2614,
    NULL
};

static const struct itemplate * const itbl_evex_map2_ef[] = {
    instrux + 2594,
    instrux + 2610,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f2[] = {
    instrux + 918,
    instrux + 919,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f3[] = {
    instrux + 926,
    instrux + 927,
    instrux + 930,
    instrux + 931,
    instrux + 934,
    instrux + 935,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f5[] = {
    instrux + 938,
    instrux + 939,
    instrux + 941,
    instrux + 943,
    instrux + 945,
    instrux + 947,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f6[] = {
    instrux + 779,
    instrux + 781,
    NULL
};

static const struct itemplate * const itbl_evex_map2_f7[] = {
    instrux + 710,
    instrux + 711,
    instrux + 712,
    instrux + 714,
    instrux + 715,
    instrux + 716,
    instrux + 922,
    instrux + 923,
    NULL
};

static const struct itemplate * const * const itbl_evex_map2[256] = {
    itbl_evex_map2_00,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_04,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_0b,
    itbl_evex_map2_0c,
    itbl_evex_map2_0d,
    NULL,
    NULL,
    itbl_evex_map2_10,
    itbl_evex_map2_11,
    itbl_evex_map2_12,
    itbl_evex_map2_13,
    itbl_evex_map2_14,
    itbl_evex_map2_15,
    itbl_evex_map2_16,
    NULL,
    itbl_evex_map2_18,
    itbl_evex_map2_19,
    itbl_evex_map2_1a,
    itbl_evex_map2_1b,
    itbl_evex_map2_1c,
    itbl_evex_map2_1d,
    itbl_evex_map2_1e,
    itbl_evex_map2_1f,
    itbl_evex_map2_20,
    itbl_evex_map2_21,
    itbl_evex_map2_22,
    itbl_evex_map2_23,
    itbl_evex_map2_24,
    itbl_evex_map2_25,
    itbl_evex_map2_26,
    itbl_evex_map2_27,
    itbl_evex_map2_28,
    itbl_evex_map2_29,
    itbl_evex_map2_2a,
    itbl_evex_map2_2b,
    itbl_evex_map2_2c,
    itbl_evex_map2_2d,
    NULL,
    NULL,
    itbl_evex_map2_30,
    itbl_evex_map2_31,
    itbl_evex_map2_32,
    itbl_evex_map2_33,
    itbl_evex_map2_34,
    itbl_evex_map2_35,
    itbl_evex_map2_36,
    itbl_evex_map2_37,
    itbl_evex_map2_38,
    itbl_evex_map2_39,
    itbl_evex_map2_3a,
    itbl_evex_map2_3b,
    itbl_evex_map2_3c,
    itbl_evex_map2_3d,
    itbl_evex_map2_3e,
    itbl_evex_map2_3f,
    itbl_evex_map2_40,
    NULL,
    itbl_evex_map2_42,
    itbl_evex_map2_43,
    itbl_evex_map2_44,
    itbl_evex_map2_45,
    itbl_evex_map2_46,
    itbl_evex_map2_47,
    NULL,
    itbl_evex_map2_49,
    itbl_evex_map2_4a,
    itbl_evex_map2_4b,
    itbl_evex_map2_4c,
    itbl_evex_map2_4d,
    itbl_evex_map2_4e,
    itbl_evex_map2_4f,
    itbl_evex_map2_50,
    itbl_evex_map2_51,
    itbl_evex_map2_52,
    itbl_evex_map2_53,
    itbl_evex_map2_54,
    itbl_evex_map2_55,
    NULL,
    NULL,
    itbl_evex_map2_58,
    itbl_evex_map2_59,
    itbl_evex_map2_5a,
    itbl_evex_map2_5b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_62,
    itbl_evex_map2_63,
    itbl_evex_map2_64,
    itbl_evex_map2_65,
    itbl_evex_map2_66,
    itbl_evex_map2_67,
    itbl_evex_map2_68,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_6d,
    NULL,
    NULL,
    itbl_evex_map2_70,
    itbl_evex_map2_71,
    itbl_evex_map2_72,
    itbl_evex_map2_73,
    itbl_evex_map2_74,
    itbl_evex_map2_75,
    itbl_evex_map2_76,
    itbl_evex_map2_77,
    itbl_evex_map2_78,
    itbl_evex_map2_79,
    itbl_evex_map2_7a,
    itbl_evex_map2_7b,
    itbl_evex_map2_7c,
    itbl_evex_map2_7d,
    itbl_evex_map2_7e,
    itbl_evex_map2_7f,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_83,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_88,
    itbl_evex_map2_89,
    itbl_evex_map2_8a,
    itbl_evex_map2_8b,
    NULL,
    itbl_evex_map2_8d,
    NULL,
    itbl_evex_map2_8f,
    itbl_evex_map2_90,
    itbl_evex_map2_91,
    itbl_evex_map2_92,
    itbl_evex_map2_93,
    NULL,
    NULL,
    itbl_evex_map2_96,
    itbl_evex_map2_97,
    itbl_evex_map2_98,
    itbl_evex_map2_99,
    itbl_evex_map2_9a,
    itbl_evex_map2_9b,
    itbl_evex_map2_9c,
    itbl_evex_map2_9d,
    itbl_evex_map2_9e,
    itbl_evex_map2_9f,
    itbl_evex_map2_a0,
    itbl_evex_map2_a1,
    itbl_evex_map2_a2,
    itbl_evex_map2_a3,
    NULL,
    NULL,
    itbl_evex_map2_a6,
    itbl_evex_map2_a7,
    itbl_evex_map2_a8,
    itbl_evex_map2_a9,
    itbl_evex_map2_aa,
    itbl_evex_map2_ab,
    itbl_evex_map2_ac,
    itbl_evex_map2_ad,
    itbl_evex_map2_ae,
    itbl_evex_map2_af,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_b4,
    itbl_evex_map2_b5,
    itbl_evex_map2_b6,
    itbl_evex_map2_b7,
    itbl_evex_map2_b8,
    itbl_evex_map2_b9,
    itbl_evex_map2_ba,
    itbl_evex_map2_bb,
    itbl_evex_map2_bc,
    itbl_evex_map2_bd,
    itbl_evex_map2_be,
    itbl_evex_map2_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_c4,
    NULL,
    itbl_evex_map2_c6,
    itbl_evex_map2_c7,
    itbl_evex_map2_c8,
    NULL,
    itbl_evex_map2_ca,
    itbl_evex_map2_cb,
    itbl_evex_map2_cc,
    itbl_evex_map2_cd,
    NULL,
    itbl_evex_map2_cf,
    NULL,
    NULL,
    itbl_evex_map2_d2,
    itbl_evex_map2_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map2_da,
    NULL,
    itbl_evex_map2_dc,
    itbl_evex_map2_dd,
    itbl_evex_map2_de,
    itbl_evex_map2_df,
    itbl_evex_map2_e0,
    itbl_evex_map2_e1,
    itbl_evex_map2_e2,
    itbl_evex_map2_e3,
    itbl_evex_map2_e4,
    itbl_evex_map2_e5,
    itbl_evex_map2_e6,
    itbl_evex_map2_e7,
    itbl_evex_map2_e8,
    itbl_evex_map2_e9,
    itbl_evex_map2_ea,
    itbl_evex_map2_eb,
    itbl_evex_map2_ec,
    itbl_evex_map2_ed,
    itbl_evex_map2_ee,
    itbl_evex_map2_ef,
    NULL,
    NULL,
    itbl_evex_map2_f2,
    itbl_evex_map2_f3,
    NULL,
    itbl_evex_map2_f5,
    itbl_evex_map2_f6,
    itbl_evex_map2_f7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map3_00[] = {
    instrux + 6016,
    instrux + 6017,
    NULL
};

static const struct itemplate * const itbl_evex_map3_01[] = {
    instrux + 6010,
    instrux + 6011,
    NULL
};

static const struct itemplate * const itbl_evex_map3_03[] = {
    instrux + 4611,
    instrux + 4612,
    instrux + 4613,
    instrux + 4614,
    instrux + 4615,
    instrux + 4616,
    NULL
};

static const struct itemplate * const itbl_evex_map3_04[] = {
    instrux + 6004,
    instrux + 6005,
    instrux + 6006,
    NULL
};

static const struct itemplate * const itbl_evex_map3_05[] = {
    instrux + 5998,
    instrux + 5999,
    instrux + 6000,
    NULL
};

static const struct itemplate * const itbl_evex_map3_07[] = {
    instrux + 6907,
    instrux + 6909,
    instrux + 6913,
    instrux + 6920,
    NULL
};

static const struct itemplate * const itbl_evex_map3_08[] = {
    instrux + 6605,
    instrux + 6606,
    instrux + 6607,
    instrux + 6608,
    instrux + 6609,
    instrux + 6610,
    instrux + 7153,
    instrux + 7154,
    instrux + 7155,
    instrux + 7274,
    instrux + 7275,
    instrux + 7276,
    NULL
};

static const struct itemplate * const itbl_evex_map3_09[] = {
    instrux + 6602,
    instrux + 6603,
    instrux + 6604,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0a[] = {
    instrux + 6612,
    instrux + 6613,
    instrux + 7156,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0b[] = {
    instrux + 6611,
    NULL
};

static const struct itemplate * const itbl_evex_map3_0f[] = {
    instrux + 5609,
    instrux + 5610,
    instrux + 5611,
    NULL
};

static const struct itemplate * const itbl_evex_map3_14[] = {
    instrux + 6053,
    instrux + 6054,
    instrux + 6055,
    instrux + 6056,
    instrux + 6057,
    NULL
};

static const struct itemplate * const itbl_evex_map3_15[] = {
    instrux + 6060,
    instrux + 6061,
    instrux + 6062,
    instrux + 6063,
    NULL
};

static const struct itemplate * const itbl_evex_map3_16[] = {
    instrux + 6058,
    instrux + 6059,
    NULL
};

static const struct itemplate * const itbl_evex_map3_17[] = {
    instrux + 5214,
    instrux + 5215,
    instrux + 5216,
    NULL
};

static const struct itemplate * const itbl_evex_map3_18[] = {
    instrux + 5401,
    instrux + 5402,
    instrux + 5404,
    instrux + 5405,
    NULL
};

static const struct itemplate * const itbl_evex_map3_19[] = {
    instrux + 5190,
    instrux + 5191,
    instrux + 5192,
    instrux + 5193,
    instrux + 5196,
    instrux + 5197,
    instrux + 5198,
    instrux + 5199,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1a[] = {
    instrux + 5403,
    instrux + 5406,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1b[] = {
    instrux + 5194,
    instrux + 5195,
    instrux + 5200,
    instrux + 5201,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1d[] = {
    instrux + 5082,
    instrux + 5083,
    instrux + 5084,
    instrux + 5085,
    instrux + 5086,
    instrux + 5087,
    instrux + 6971,
    instrux + 6972,
    instrux + 6973,
    instrux + 6974,
    instrux + 6975,
    instrux + 6976,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1e[] = {
    instrux + 5729,
    instrux + 5730,
    instrux + 5731,
    instrux + 5732,
    instrux + 5733,
    instrux + 5734,
    instrux + 5753,
    instrux + 5754,
    instrux + 5755,
    instrux + 5756,
    instrux + 5757,
    instrux + 5758,
    instrux + 5777,
    instrux + 5778,
    instrux + 5779,
    instrux + 5780,
    instrux + 5781,
    instrux + 5782,
    instrux + 5801,
    instrux + 5802,
    instrux + 5803,
    instrux + 5804,
    instrux + 5805,
    instrux + 5806,
    instrux + 5825,
    instrux + 5826,
    instrux + 5827,
    instrux + 5828,
    instrux + 5829,
    instrux + 5830,
    instrux + 5849,
    instrux + 5850,
    instrux + 5851,
    instrux + 5852,
    instrux + 5853,
    instrux + 5854,
    instrux + 5873,
    instrux + 5874,
    instrux + 5875,
    instrux + 5876,
    instrux + 5877,
    instrux + 5878,
    instrux + 5897,
    instrux + 5898,
    instrux + 5899,
    instrux + 5900,
    instrux + 5901,
    instrux + 5902,
    instrux + 5921,
    instrux + 5922,
    instrux + 5923,
    instrux + 5924,
    instrux + 5925,
    instrux + 5926,
    instrux + 5945,
    instrux + 5946,
    instrux + 5947,
    instrux + 5948,
    instrux + 5949,
    instrux + 5950,
    NULL
};

static const struct itemplate * const itbl_evex_map3_1f[] = {
    instrux + 5720,
    instrux + 5721,
    instrux + 5722,
    instrux + 5723,
    instrux + 5724,
    instrux + 5725,
    instrux + 5744,
    instrux + 5745,
    instrux + 5746,
    instrux + 5747,
    instrux + 5748,
    instrux + 5749,
    instrux + 5768,
    instrux + 5769,
    instrux + 5770,
    instrux + 5771,
    instrux + 5772,
    instrux + 5773,
    instrux + 5792,
    instrux + 5793,
    instrux + 5794,
    instrux + 5795,
    instrux + 5796,
    instrux + 5797,
    instrux + 5816,
    instrux + 5817,
    instrux + 5818,
    instrux + 5819,
    instrux + 5820,
    instrux + 5821,
    instrux + 5840,
    instrux + 5841,
    instrux + 5842,
    instrux + 5843,
    instrux + 5844,
    instrux + 5845,
    instrux + 5864,
    instrux + 5865,
    instrux + 5866,
    instrux + 5867,
    instrux + 5868,
    instrux + 5869,
    instrux + 5888,
    instrux + 5889,
    instrux + 5890,
    instrux + 5891,
    instrux + 5892,
    instrux + 5893,
    instrux + 5912,
    instrux + 5913,
    instrux + 5914,
    instrux + 5915,
    instrux + 5916,
    instrux + 5917,
    instrux + 5936,
    instrux + 5937,
    instrux + 5938,
    instrux + 5939,
    instrux + 5940,
    instrux + 5941,
    NULL
};

static const struct itemplate * const itbl_evex_map3_20[] = {
    instrux + 6079,
    instrux + 6080,
    NULL
};

static const struct itemplate * const itbl_evex_map3_21[] = {
    instrux + 5413,
    NULL
};

static const struct itemplate * const itbl_evex_map3_22[] = {
    instrux + 6081,
    instrux + 6082,
    NULL
};

static const struct itemplate * const itbl_evex_map3_23[] = {
    instrux + 6654,
    instrux + 6655,
    instrux + 6656,
    instrux + 6657,
    NULL
};

static const struct itemplate * const itbl_evex_map3_25[] = {
    instrux + 6514,
    instrux + 6515,
    instrux + 6516,
    instrux + 6517,
    instrux + 6518,
    instrux + 6519,
    NULL
};

static const struct itemplate * const itbl_evex_map3_26[] = {
    instrux + 5393,
    instrux + 5394,
    instrux + 5395,
    instrux + 5396,
    instrux + 5397,
    instrux + 5398,
    instrux + 7123,
    instrux + 7124,
    instrux + 7125,
    instrux + 7256,
    instrux + 7257,
    instrux + 7258,
    NULL
};

static const struct itemplate * const itbl_evex_map3_27[] = {
    instrux + 5399,
    instrux + 5400,
    instrux + 7126,
    NULL
};

static const struct itemplate * const itbl_evex_map3_30[] = {
    instrux + 4566,
    instrux + 4568,
    NULL
};

static const struct itemplate * const itbl_evex_map3_31[] = {
    instrux + 4570,
    instrux + 4572,
    NULL
};

static const struct itemplate * const itbl_evex_map3_32[] = {
    instrux + 4558,
    instrux + 4560,
    NULL
};

static const struct itemplate * const itbl_evex_map3_33[] = {
    instrux + 4562,
    instrux + 4564,
    NULL
};

static const struct itemplate * const itbl_evex_map3_38[] = {
    instrux + 5407,
    instrux + 5408,
    instrux + 5410,
    instrux + 5411,
    NULL
};

static const struct itemplate * const itbl_evex_map3_39[] = {
    instrux + 5202,
    instrux + 5203,
    instrux + 5204,
    instrux + 5205,
    instrux + 5208,
    instrux + 5209,
    instrux + 5210,
    instrux + 5211,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3a[] = {
    instrux + 5409,
    instrux + 5412,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3b[] = {
    instrux + 5206,
    instrux + 5207,
    instrux + 5212,
    instrux + 5213,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3e[] = {
    instrux + 5726,
    instrux + 5727,
    instrux + 5728,
    instrux + 5735,
    instrux + 5736,
    instrux + 5737,
    instrux + 5750,
    instrux + 5751,
    instrux + 5752,
    instrux + 5759,
    instrux + 5760,
    instrux + 5761,
    instrux + 5774,
    instrux + 5775,
    instrux + 5776,
    instrux + 5783,
    instrux + 5784,
    instrux + 5785,
    instrux + 5798,
    instrux + 5799,
    instrux + 5800,
    instrux + 5807,
    instrux + 5808,
    instrux + 5809,
    instrux + 5822,
    instrux + 5823,
    instrux + 5824,
    instrux + 5831,
    instrux + 5832,
    instrux + 5833,
    instrux + 5846,
    instrux + 5847,
    instrux + 5848,
    instrux + 5855,
    instrux + 5856,
    instrux + 5857,
    instrux + 5870,
    instrux + 5871,
    instrux + 5872,
    instrux + 5879,
    instrux + 5880,
    instrux + 5881,
    instrux + 5894,
    instrux + 5895,
    instrux + 5896,
    instrux + 5903,
    instrux + 5904,
    instrux + 5905,
    instrux + 5918,
    instrux + 5919,
    instrux + 5920,
    instrux + 5927,
    instrux + 5928,
    instrux + 5929,
    instrux + 5942,
    instrux + 5943,
    instrux + 5944,
    instrux + 5951,
    instrux + 5952,
    instrux + 5953,
    NULL
};

static const struct itemplate * const itbl_evex_map3_3f[] = {
    instrux + 5717,
    instrux + 5718,
    instrux + 5719,
    instrux + 5738,
    instrux + 5739,
    instrux + 5740,
    instrux + 5741,
    instrux + 5742,
    instrux + 5743,
    instrux + 5762,
    instrux + 5763,
    instrux + 5764,
    instrux + 5765,
    instrux + 5766,
    instrux + 5767,
    instrux + 5786,
    instrux + 5787,
    instrux + 5788,
    instrux + 5789,
    instrux + 5790,
    instrux + 5791,
    instrux + 5810,
    instrux + 5811,
    instrux + 5812,
    instrux + 5813,
    instrux + 5814,
    instrux + 5815,
    instrux + 5834,
    instrux + 5835,
    instrux + 5836,
    instrux + 5837,
    instrux + 5838,
    instrux + 5839,
    instrux + 5858,
    instrux + 5859,
    instrux + 5860,
    instrux + 5861,
    instrux + 5862,
    instrux + 5863,
    instrux + 5882,
    instrux + 5883,
    instrux + 5884,
    instrux + 5885,
    instrux + 5886,
    instrux + 5887,
    instrux + 5906,
    instrux + 5907,
    instrux + 5908,
    instrux + 5909,
    instrux + 5910,
    instrux + 5911,
    instrux + 5930,
    instrux + 5931,
    instrux + 5932,
    instrux + 5933,
    instrux + 5934,
    instrux + 5935,
    instrux + 5954,
    instrux + 5955,
    instrux + 5956,
    NULL
};

static const struct itemplate * const itbl_evex_map3_42[] = {
    instrux + 5165,
    instrux + 5166,
    instrux + 5167,
    instrux + 7340,
    instrux + 7341,
    instrux + 7342,
    NULL
};

static const struct itemplate * const itbl_evex_map3_43[] = {
    instrux + 6658,
    instrux + 6659,
    instrux + 6660,
    instrux + 6661,
    NULL
};

static const struct itemplate * const itbl_evex_map3_44[] = {
    instrux + 3819,
    instrux + 3820,
    instrux + 3821,
    instrux + 3822,
    instrux + 3823,
    instrux + 3824,
    instrux + 3825,
    instrux + 3826,
    instrux + 3827,
    instrux + 3828,
    instrux + 3829,
    instrux + 3830,
    instrux + 3831,
    instrux + 3832,
    instrux + 3833,
    NULL
};

static const struct itemplate * const itbl_evex_map3_50[] = {
    instrux + 6574,
    instrux + 6575,
    instrux + 6576,
    instrux + 6577,
    instrux + 6578,
    instrux + 6579,
    NULL
};

static const struct itemplate * const itbl_evex_map3_51[] = {
    instrux + 6580,
    instrux + 6581,
    NULL
};

static const struct itemplate * const itbl_evex_map3_52[] = {
    instrux + 7379,
    instrux + 7380,
    instrux + 7381,
    instrux + 7382,
    instrux + 7383,
    instrux + 7384,
    instrux + 7385,
    instrux + 7386,
    instrux + 7387,
    instrux + 7388,
    instrux + 7389,
    instrux + 7390,
    NULL
};

static const struct itemplate * const itbl_evex_map3_53[] = {
    instrux + 7391,
    instrux + 7392,
    instrux + 7393,
    NULL
};

static const struct itemplate * const itbl_evex_map3_54[] = {
    instrux + 5217,
    instrux + 5218,
    instrux + 5219,
    instrux + 5220,
    instrux + 5221,
    instrux + 5222,
    NULL
};

static const struct itemplate * const itbl_evex_map3_55[] = {
    instrux + 5223,
    instrux + 5224,
    NULL
};

static const struct itemplate * const itbl_evex_map3_56[] = {
    instrux + 6594,
    instrux + 6595,
    instrux + 6596,
    instrux + 6597,
    instrux + 6598,
    instrux + 6599,
    instrux + 7149,
    instrux + 7150,
    instrux + 7151,
    instrux + 7271,
    instrux + 7272,
    instrux + 7273,
    NULL
};

static const struct itemplate * const itbl_evex_map3_57[] = {
    instrux + 6600,
    instrux + 6601,
    instrux + 7152,
    NULL
};

static const struct itemplate * const itbl_evex_map3_66[] = {
    instrux + 5357,
    instrux + 5358,
    instrux + 5359,
    instrux + 5360,
    instrux + 5361,
    instrux + 5362,
    instrux + 7115,
    instrux + 7116,
    instrux + 7117,
    instrux + 7250,
    instrux + 7251,
    instrux + 7252,
    NULL
};

static const struct itemplate * const itbl_evex_map3_67[] = {
    instrux + 5363,
    instrux + 5364,
    instrux + 7118,
    NULL
};

static const struct itemplate * const itbl_evex_map3_70[] = {
    instrux + 6756,
    instrux + 6757,
    instrux + 6758,
    NULL
};

static const struct itemplate * const itbl_evex_map3_71[] = {
    instrux + 6759,
    instrux + 6760,
    instrux + 6761,
    instrux + 6762,
    instrux + 6763,
    instrux + 6764,
    NULL
};

static const struct itemplate * const itbl_evex_map3_72[] = {
    instrux + 6774,
    instrux + 6775,
    instrux + 6776,
    NULL
};

static const struct itemplate * const itbl_evex_map3_73[] = {
    instrux + 6777,
    instrux + 6778,
    instrux + 6779,
    instrux + 6780,
    instrux + 6781,
    instrux + 6782,
    NULL
};

static const struct itemplate * const itbl_evex_map3_77[] = {
    instrux + 6911,
    instrux + 6915,
    NULL
};

static const struct itemplate * const itbl_evex_map3_c2[] = {
    instrux + 6933,
    instrux + 6934,
    instrux + 6935,
    instrux + 6936,
    instrux + 7207,
    instrux + 7208,
    instrux + 7209,
    NULL
};

static const struct itemplate * const itbl_evex_map3_ce[] = {
    instrux + 6729,
    instrux + 6730,
    instrux + 6731,
    NULL
};

static const struct itemplate * const itbl_evex_map3_cf[] = {
    instrux + 6723,
    instrux + 6724,
    instrux + 6725,
    NULL
};

static const struct itemplate * const itbl_evex_map3_f0[] = {
    instrux + 709,
    instrux + 713,
    NULL
};

static const struct itemplate * const * const itbl_evex_map3[256] = {
    itbl_evex_map3_00,
    itbl_evex_map3_01,
    NULL,
    itbl_evex_map3_03,
    itbl_evex_map3_04,
    itbl_evex_map3_05,
    NULL,
    itbl_evex_map3_07,
    itbl_evex_map3_08,
    itbl_evex_map3_09,
    itbl_evex_map3_0a,
    itbl_evex_map3_0b,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_0f,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_14,
    itbl_evex_map3_15,
    itbl_evex_map3_16,
    itbl_evex_map3_17,
    itbl_evex_map3_18,
    itbl_evex_map3_19,
    itbl_evex_map3_1a,
    itbl_evex_map3_1b,
    NULL,
    itbl_evex_map3_1d,
    itbl_evex_map3_1e,
    itbl_evex_map3_1f,
    itbl_evex_map3_20,
    itbl_evex_map3_21,
    itbl_evex_map3_22,
    itbl_evex_map3_23,
    NULL,
    itbl_evex_map3_25,
    itbl_evex_map3_26,
    itbl_evex_map3_27,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_30,
    itbl_evex_map3_31,
    itbl_evex_map3_32,
    itbl_evex_map3_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_38,
    itbl_evex_map3_39,
    itbl_evex_map3_3a,
    itbl_evex_map3_3b,
    NULL,
    NULL,
    itbl_evex_map3_3e,
    itbl_evex_map3_3f,
    NULL,
    NULL,
    itbl_evex_map3_42,
    itbl_evex_map3_43,
    itbl_evex_map3_44,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_50,
    itbl_evex_map3_51,
    itbl_evex_map3_52,
    itbl_evex_map3_53,
    itbl_evex_map3_54,
    itbl_evex_map3_55,
    itbl_evex_map3_56,
    itbl_evex_map3_57,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_66,
    itbl_evex_map3_67,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_70,
    itbl_evex_map3_71,
    itbl_evex_map3_72,
    itbl_evex_map3_73,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_77,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_c2,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_ce,
    itbl_evex_map3_cf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map3_f0,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map4_00[] = {
    instrux + 66,
    instrux + 67,
    NULL
};

static const struct itemplate * const itbl_evex_map4_01[] = {
    instrux + 68,
    instrux + 69,
    instrux + 70,
    instrux + 71,
    instrux + 72,
    instrux + 73,
    NULL
};

static const struct itemplate * const itbl_evex_map4_02[] = {
    instrux + 58,
    instrux + 59,
    NULL
};

static const struct itemplate * const itbl_evex_map4_03[] = {
    instrux + 60,
    instrux + 61,
    instrux + 62,
    instrux + 63,
    instrux + 64,
    instrux + 65,
    NULL
};

static const struct itemplate * const itbl_evex_map4_08[] = {
    instrux + 115,
    instrux + 116,
    NULL
};

static const struct itemplate * const itbl_evex_map4_09[] = {
    instrux + 117,
    instrux + 118,
    instrux + 119,
    instrux + 120,
    instrux + 121,
    instrux + 122,
    NULL
};

static const struct itemplate * const itbl_evex_map4_0a[] = {
    instrux + 107,
    instrux + 108,
    NULL
};

static const struct itemplate * const itbl_evex_map4_0b[] = {
    instrux + 109,
    instrux + 110,
    instrux + 111,
    instrux + 112,
    instrux + 113,
    instrux + 114,
    NULL
};

static const struct itemplate * const itbl_evex_map4_10[] = {
    instrux + 164,
    instrux + 165,
    NULL
};

static const struct itemplate * const itbl_evex_map4_11[] = {
    instrux + 166,
    instrux + 167,
    instrux + 168,
    instrux + 169,
    instrux + 170,
    instrux + 171,
    NULL
};

static const struct itemplate * const itbl_evex_map4_12[] = {
    instrux + 156,
    instrux + 157,
    NULL
};

static const struct itemplate * const itbl_evex_map4_13[] = {
    instrux + 158,
    instrux + 159,
    instrux + 160,
    instrux + 161,
    instrux + 162,
    instrux + 163,
    NULL
};

static const struct itemplate * const itbl_evex_map4_18[] = {
    instrux + 213,
    instrux + 214,
    NULL
};

static const struct itemplate * const itbl_evex_map4_19[] = {
    instrux + 215,
    instrux + 216,
    instrux + 217,
    instrux + 218,
    instrux + 219,
    instrux + 220,
    NULL
};

static const struct itemplate * const itbl_evex_map4_1a[] = {
    instrux + 205,
    instrux + 206,
    NULL
};

static const struct itemplate * const itbl_evex_map4_1b[] = {
    instrux + 207,
    instrux + 208,
    instrux + 209,
    instrux + 210,
    instrux + 211,
    instrux + 212,
    NULL
};

static const struct itemplate * const itbl_evex_map4_20[] = {
    instrux + 262,
    instrux + 263,
    NULL
};

static const struct itemplate * const itbl_evex_map4_21[] = {
    instrux + 264,
    instrux + 265,
    instrux + 266,
    instrux + 267,
    instrux + 268,
    instrux + 269,
    NULL
};

static const struct itemplate * const itbl_evex_map4_22[] = {
    instrux + 254,
    instrux + 255,
    NULL
};

static const struct itemplate * const itbl_evex_map4_23[] = {
    instrux + 256,
    instrux + 257,
    instrux + 258,
    instrux + 259,
    instrux + 260,
    instrux + 261,
    NULL
};

static const struct itemplate * const itbl_evex_map4_24[] = {
    instrux + 844,
    instrux + 845,
    instrux + 846,
    instrux + 847,
    instrux + 848,
    instrux + 849,
    NULL
};

static const struct itemplate * const itbl_evex_map4_28[] = {
    instrux + 311,
    instrux + 312,
    NULL
};

static const struct itemplate * const itbl_evex_map4_29[] = {
    instrux + 313,
    instrux + 314,
    instrux + 315,
    instrux + 316,
    instrux + 317,
    instrux + 318,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2a[] = {
    instrux + 303,
    instrux + 304,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2b[] = {
    instrux + 305,
    instrux + 306,
    instrux + 307,
    instrux + 308,
    instrux + 309,
    instrux + 310,
    NULL
};

static const struct itemplate * const itbl_evex_map4_2c[] = {
    instrux + 862,
    instrux + 863,
    instrux + 864,
    instrux + 865,
    instrux + 866,
    instrux + 867,
    NULL
};

static const struct itemplate * const itbl_evex_map4_30[] = {
    instrux + 360,
    instrux + 361,
    NULL
};

static const struct itemplate * const itbl_evex_map4_31[] = {
    instrux + 362,
    instrux + 363,
    instrux + 364,
    instrux + 365,
    instrux + 366,
    instrux + 367,
    NULL
};

static const struct itemplate * const itbl_evex_map4_32[] = {
    instrux + 352,
    instrux + 353,
    NULL
};

static const struct itemplate * const itbl_evex_map4_33[] = {
    instrux + 354,
    instrux + 355,
    instrux + 356,
    instrux + 357,
    instrux + 358,
    instrux + 359,
    NULL
};

static const struct itemplate * const itbl_evex_map4_38[] = {
    instrux + 1792,
    instrux + 1793,
    instrux + 1794,
    instrux + 1795,
    instrux + 1796,
    instrux + 1797,
    instrux + 1798,
    instrux + 1799,
    instrux + 1800,
    instrux + 1801,
    instrux + 1802,
    instrux + 1803,
    instrux + 1804,
    instrux + 1805,
    instrux + 1806,
    instrux + 1807,
    NULL
};

static const struct itemplate * const itbl_evex_map4_39[] = {
    instrux + 1808,
    instrux + 1809,
    instrux + 1810,
    instrux + 1811,
    instrux + 1812,
    instrux + 1813,
    instrux + 1814,
    instrux + 1815,
    instrux + 1816,
    instrux + 1817,
    instrux + 1818,
    instrux + 1819,
    instrux + 1820,
    instrux + 1821,
    instrux + 1822,
    instrux + 1823,
    instrux + 1824,
    instrux + 1825,
    instrux + 1826,
    instrux + 1827,
    instrux + 1828,
    instrux + 1829,
    instrux + 1830,
    instrux + 1831,
    instrux + 1832,
    instrux + 1833,
    instrux + 1834,
    instrux + 1835,
    instrux + 1836,
    instrux + 1837,
    instrux + 1838,
    instrux + 1839,
    instrux + 1840,
    instrux + 1841,
    instrux + 1842,
    instrux + 1843,
    instrux + 1844,
    instrux + 1845,
    instrux + 1846,
    instrux + 1847,
    instrux + 1848,
    instrux + 1849,
    instrux + 1850,
    instrux + 1851,
    instrux + 1852,
    instrux + 1853,
    instrux + 1854,
    instrux + 1855,
    NULL
};

static const struct itemplate * const itbl_evex_map4_3a[] = {
    instrux + 1856,
    instrux + 1857,
    instrux + 1858,
    instrux + 1859,
    instrux + 1860,
    instrux + 1861,
    instrux + 1862,
    instrux + 1863,
    instrux + 1864,
    instrux + 1865,
    instrux + 1866,
    instrux + 1867,
    instrux + 1868,
    instrux + 1869,
    instrux + 1870,
    instrux + 1871,
    NULL
};

static const struct itemplate * const itbl_evex_map4_3b[] = {
    instrux + 1872,
    instrux + 1873,
    instrux + 1874,
    instrux + 1875,
    instrux + 1876,
    instrux + 1877,
    instrux + 1878,
    instrux + 1879,
    instrux + 1880,
    instrux + 1881,
    instrux + 1882,
    instrux + 1883,
    instrux + 1884,
    instrux + 1885,
    instrux + 1886,
    instrux + 1887,
    instrux + 1888,
    instrux + 1889,
    instrux + 1890,
    instrux + 1891,
    instrux + 1892,
    instrux + 1893,
    instrux + 1894,
    instrux + 1895,
    instrux + 1896,
    instrux + 1897,
    instrux + 1898,
    instrux + 1899,
    instrux + 1900,
    instrux + 1901,
    instrux + 1902,
    instrux + 1903,
    instrux + 1904,
    instrux + 1905,
    instrux + 1906,
    instrux + 1907,
    instrux + 1908,
    instrux + 1909,
    instrux + 1910,
    instrux + 1911,
    instrux + 1912,
    instrux + 1913,
    instrux + 1914,
    instrux + 1915,
    instrux + 1916,
    instrux + 1917,
    instrux + 1918,
    instrux + 1919,
    NULL
};

static const struct itemplate * const itbl_evex_map4_40[] = {
    instrux + 2283,
    instrux + 2299,
    instrux + 2315,
    instrux + 2331,
    instrux + 2347,
    instrux + 2363,
    instrux + 2379,
    instrux + 2395,
    instrux + 2411,
    instrux + 2427,
    instrux + 2443,
    instrux + 2459,
    instrux + 2475,
    instrux + 2491,
    instrux + 2507,
    instrux + 2539,
    instrux + 2555,
    NULL
};

static const struct itemplate * const itbl_evex_map4_41[] = {
    instrux + 2280,
    instrux + 2296,
    instrux + 2312,
    instrux + 2328,
    instrux + 2344,
    instrux + 2360,
    instrux + 2376,
    instrux + 2392,
    instrux + 2408,
    instrux + 2424,
    instrux + 2440,
    instrux + 2456,
    instrux + 2472,
    instrux + 2488,
    instrux + 2504,
    instrux + 2536,
    instrux + 2552,
    NULL
};

static const struct itemplate * const itbl_evex_map4_42[] = {
    instrux + 2273,
    instrux + 2289,
    instrux + 2305,
    instrux + 2321,
    instrux + 2337,
    instrux + 2353,
    instrux + 2369,
    instrux + 2385,
    instrux + 2401,
    instrux + 2417,
    instrux + 2433,
    instrux + 2449,
    instrux + 2465,
    instrux + 2481,
    instrux + 2497,
    instrux + 2529,
    instrux + 2545,
    NULL
};

static const struct itemplate * const itbl_evex_map4_43[] = {
    instrux + 2277,
    instrux + 2293,
    instrux + 2309,
    instrux + 2325,
    instrux + 2341,
    instrux + 2357,
    instrux + 2373,
    instrux + 2389,
    instrux + 2405,
    instrux + 2421,
    instrux + 2437,
    instrux + 2453,
    instrux + 2469,
    instrux + 2485,
    instrux + 2501,
    instrux + 2533,
    instrux + 2549,
    NULL
};

static const struct itemplate * const itbl_evex_map4_44[] = {
    instrux + 2287,
    instrux + 2303,
    instrux + 2319,
    instrux + 2335,
    instrux + 2351,
    instrux + 2367,
    instrux + 2383,
    instrux + 2399,
    instrux + 2415,
    instrux + 2431,
    instrux + 2447,
    instrux + 2463,
    instrux + 2479,
    instrux + 2495,
    instrux + 2511,
    instrux + 2543,
    instrux + 2559,
    NULL
};

static const struct itemplate * const itbl_evex_map4_45[] = {
    instrux + 2282,
    instrux + 2298,
    instrux + 2314,
    instrux + 2330,
    instrux + 2346,
    instrux + 2362,
    instrux + 2378,
    instrux + 2394,
    instrux + 2410,
    instrux + 2426,
    instrux + 2442,
    instrux + 2458,
    instrux + 2474,
    instrux + 2490,
    instrux + 2506,
    instrux + 2538,
    instrux + 2554,
    NULL
};

static const struct itemplate * const itbl_evex_map4_46[] = {
    instrux + 2276,
    instrux + 2292,
    instrux + 2308,
    instrux + 2324,
    instrux + 2340,
    instrux + 2356,
    instrux + 2372,
    instrux + 2388,
    instrux + 2404,
    instrux + 2420,
    instrux + 2436,
    instrux + 2452,
    instrux + 2468,
    instrux + 2484,
    instrux + 2500,
    instrux + 2532,
    instrux + 2548,
    NULL
};

static const struct itemplate * const itbl_evex_map4_47[] = {
    instrux + 2272,
    instrux + 2288,
    instrux + 2304,
    instrux + 2320,
    instrux + 2336,
    instrux + 2352,
    instrux + 2368,
    instrux + 2384,
    instrux + 2400,
    instrux + 2416,
    instrux + 2432,
    instrux + 2448,
    instrux + 2464,
    instrux + 2480,
    instrux + 2496,
    instrux + 2528,
    instrux + 2544,
    NULL
};

static const struct itemplate * const itbl_evex_map4_48[] = {
    instrux + 2286,
    instrux + 2302,
    instrux + 2318,
    instrux + 2334,
    instrux + 2350,
    instrux + 2366,
    instrux + 2382,
    instrux + 2398,
    instrux + 2414,
    instrux + 2430,
    instrux + 2446,
    instrux + 2462,
    instrux + 2478,
    instrux + 2494,
    instrux + 2510,
    instrux + 2542,
    instrux + 2558,
    NULL
};

static const struct itemplate * const itbl_evex_map4_49[] = {
    instrux + 2281,
    instrux + 2297,
    instrux + 2313,
    instrux + 2329,
    instrux + 2345,
    instrux + 2361,
    instrux + 2377,
    instrux + 2393,
    instrux + 2409,
    instrux + 2425,
    instrux + 2441,
    instrux + 2457,
    instrux + 2473,
    instrux + 2489,
    instrux + 2505,
    instrux + 2537,
    instrux + 2553,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4a[] = {
    instrux + 2284,
    instrux + 2300,
    instrux + 2316,
    instrux + 2332,
    instrux + 2348,
    instrux + 2364,
    instrux + 2380,
    instrux + 2396,
    instrux + 2412,
    instrux + 2428,
    instrux + 2444,
    instrux + 2460,
    instrux + 2476,
    instrux + 2492,
    instrux + 2508,
    instrux + 2540,
    instrux + 2556,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4b[] = {
    instrux + 2285,
    instrux + 2301,
    instrux + 2317,
    instrux + 2333,
    instrux + 2349,
    instrux + 2365,
    instrux + 2381,
    instrux + 2397,
    instrux + 2413,
    instrux + 2429,
    instrux + 2445,
    instrux + 2461,
    instrux + 2477,
    instrux + 2493,
    instrux + 2509,
    instrux + 2541,
    instrux + 2557,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4c[] = {
    instrux + 2275,
    instrux + 2291,
    instrux + 2307,
    instrux + 2323,
    instrux + 2339,
    instrux + 2355,
    instrux + 2371,
    instrux + 2387,
    instrux + 2403,
    instrux + 2419,
    instrux + 2435,
    instrux + 2451,
    instrux + 2467,
    instrux + 2483,
    instrux + 2499,
    instrux + 2531,
    instrux + 2547,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4d[] = {
    instrux + 2279,
    instrux + 2295,
    instrux + 2311,
    instrux + 2327,
    instrux + 2343,
    instrux + 2359,
    instrux + 2375,
    instrux + 2391,
    instrux + 2407,
    instrux + 2423,
    instrux + 2439,
    instrux + 2455,
    instrux + 2471,
    instrux + 2487,
    instrux + 2503,
    instrux + 2535,
    instrux + 2551,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4e[] = {
    instrux + 2278,
    instrux + 2294,
    instrux + 2310,
    instrux + 2326,
    instrux + 2342,
    instrux + 2358,
    instrux + 2374,
    instrux + 2390,
    instrux + 2406,
    instrux + 2422,
    instrux + 2438,
    instrux + 2454,
    instrux + 2470,
    instrux + 2486,
    instrux + 2502,
    instrux + 2534,
    instrux + 2550,
    NULL
};

static const struct itemplate * const itbl_evex_map4_4f[] = {
    instrux + 2274,
    instrux + 2290,
    instrux + 2306,
    instrux + 2322,
    instrux + 2338,
    instrux + 2354,
    instrux + 2370,
    instrux + 2386,
    instrux + 2402,
    instrux + 2418,
    instrux + 2434,
    instrux + 2450,
    instrux + 2466,
    instrux + 2482,
    instrux + 2498,
    instrux + 2530,
    instrux + 2546,
    NULL
};

static const struct itemplate * const itbl_evex_map4_60[] = {
    instrux + 984,
    instrux + 985,
    instrux + 986,
    NULL
};

static const struct itemplate * const itbl_evex_map4_61[] = {
    instrux + 987,
    instrux + 988,
    instrux + 989,
    NULL
};

static const struct itemplate * const itbl_evex_map4_65[] = {
    instrux + 6839,
    instrux + 6841,
    NULL
};

static const struct itemplate * const itbl_evex_map4_66[] = {
    instrux + 828,
    instrux + 829,
    instrux + 830,
    instrux + 831,
    instrux + 834,
    instrux + 835,
    instrux + 836,
    instrux + 837,
    instrux + 6843,
    instrux + 6845,
    NULL
};

static const struct itemplate * const itbl_evex_map4_69[] = {
    instrux + 771,
    instrux + 772,
    instrux + 773,
    NULL
};

static const struct itemplate * const itbl_evex_map4_6b[] = {
    instrux + 768,
    instrux + 769,
    instrux + 770,
    NULL
};

static const struct itemplate * const itbl_evex_map4_80[] = {
    instrux + 80,
    instrux + 81,
    instrux + 129,
    instrux + 130,
    instrux + 178,
    instrux + 179,
    instrux + 227,
    instrux + 228,
    instrux + 276,
    instrux + 277,
    instrux + 325,
    instrux + 326,
    instrux + 374,
    instrux + 375,
    instrux + 1968,
    instrux + 1969,
    instrux + 1970,
    instrux + 1971,
    instrux + 1972,
    instrux + 1973,
    instrux + 1974,
    instrux + 1975,
    instrux + 1976,
    instrux + 1977,
    instrux + 1978,
    instrux + 1979,
    instrux + 1980,
    instrux + 1981,
    instrux + 1982,
    instrux + 1983,
    NULL
};

static const struct itemplate * const itbl_evex_map4_81[] = {
    instrux + 82,
    instrux + 83,
    instrux + 84,
    instrux + 85,
    instrux + 86,
    instrux + 87,
    instrux + 131,
    instrux + 132,
    instrux + 133,
    instrux + 134,
    instrux + 135,
    instrux + 136,
    instrux + 180,
    instrux + 181,
    instrux + 182,
    instrux + 183,
    instrux + 184,
    instrux + 185,
    instrux + 229,
    instrux + 230,
    instrux + 231,
    instrux + 232,
    instrux + 233,
    instrux + 234,
    instrux + 278,
    instrux + 279,
    instrux + 280,
    instrux + 281,
    instrux + 282,
    instrux + 283,
    instrux + 327,
    instrux + 328,
    instrux + 329,
    instrux + 330,
    instrux + 331,
    instrux + 332,
    instrux + 376,
    instrux + 377,
    instrux + 378,
    instrux + 379,
    instrux + 380,
    instrux + 381,
    instrux + 1984,
    instrux + 1985,
    instrux + 1986,
    instrux + 1987,
    instrux + 1988,
    instrux + 1989,
    instrux + 1990,
    instrux + 1991,
    instrux + 1992,
    instrux + 1993,
    instrux + 1994,
    instrux + 1995,
    instrux + 1996,
    instrux + 1997,
    instrux + 1998,
    instrux + 1999,
    instrux + 2000,
    instrux + 2001,
    instrux + 2002,
    instrux + 2003,
    instrux + 2004,
    instrux + 2005,
    instrux + 2006,
    instrux + 2007,
    instrux + 2008,
    instrux + 2009,
    instrux + 2010,
    instrux + 2011,
    instrux + 2012,
    instrux + 2013,
    instrux + 2014,
    instrux + 2015,
    instrux + 2016,
    instrux + 2017,
    instrux + 2018,
    instrux + 2019,
    instrux + 2020,
    instrux + 2021,
    instrux + 2022,
    instrux + 2023,
    instrux + 2024,
    instrux + 2025,
    instrux + 2026,
    instrux + 2027,
    instrux + 2028,
    instrux + 2029,
    instrux + 2030,
    instrux + 2031,
    NULL
};

static const struct itemplate * const itbl_evex_map4_83[] = {
    instrux + 74,
    instrux + 75,
    instrux + 76,
    instrux + 77,
    instrux + 78,
    instrux + 79,
    instrux + 123,
    instrux + 124,
    instrux + 125,
    instrux + 126,
    instrux + 127,
    instrux + 128,
    instrux + 172,
    instrux + 173,
    instrux + 174,
    instrux + 175,
    instrux + 176,
    instrux + 177,
    instrux + 221,
    instrux + 222,
    instrux + 223,
    instrux + 224,
    instrux + 225,
    instrux + 226,
    instrux + 270,
    instrux + 271,
    instrux + 272,
    instrux + 273,
    instrux + 274,
    instrux + 275,
    instrux + 319,
    instrux + 320,
    instrux + 321,
    instrux + 322,
    instrux + 323,
    instrux + 324,
    instrux + 368,
    instrux + 369,
    instrux + 370,
    instrux + 371,
    instrux + 372,
    instrux + 373,
    instrux + 1920,
    instrux + 1921,
    instrux + 1922,
    instrux + 1923,
    instrux + 1924,
    instrux + 1925,
    instrux + 1926,
    instrux + 1927,
    instrux + 1928,
    instrux + 1929,
    instrux + 1930,
    instrux + 1931,
    instrux + 1932,
    instrux + 1933,
    instrux + 1934,
    instrux + 1935,
    instrux + 1936,
    instrux + 1937,
    instrux + 1938,
    instrux + 1939,
    instrux + 1940,
    instrux + 1941,
    instrux + 1942,
    instrux + 1943,
    instrux + 1944,
    instrux + 1945,
    instrux + 1946,
    instrux + 1947,
    instrux + 1948,
    instrux + 1949,
    instrux + 1950,
    instrux + 1951,
    instrux + 1952,
    instrux + 1953,
    instrux + 1954,
    instrux + 1955,
    instrux + 1956,
    instrux + 1957,
    instrux + 1958,
    instrux + 1959,
    instrux + 1960,
    instrux + 1961,
    instrux + 1962,
    instrux + 1963,
    instrux + 1964,
    instrux + 1965,
    instrux + 1966,
    instrux + 1967,
    NULL
};

static const struct itemplate * const itbl_evex_map4_84[] = {
    instrux + 2032,
    instrux + 2033,
    instrux + 2034,
    instrux + 2035,
    instrux + 2036,
    instrux + 2037,
    instrux + 2038,
    instrux + 2039,
    instrux + 2040,
    instrux + 2041,
    instrux + 2042,
    instrux + 2043,
    instrux + 2044,
    instrux + 2045,
    instrux + 2046,
    instrux + 2047,
    NULL
};

static const struct itemplate * const itbl_evex_map4_85[] = {
    instrux + 2048,
    instrux + 2049,
    instrux + 2050,
    instrux + 2051,
    instrux + 2052,
    instrux + 2053,
    instrux + 2054,
    instrux + 2055,
    instrux + 2056,
    instrux + 2057,
    instrux + 2058,
    instrux + 2059,
    instrux + 2060,
    instrux + 2061,
    instrux + 2062,
    instrux + 2063,
    instrux + 2064,
    instrux + 2065,
    instrux + 2066,
    instrux + 2067,
    instrux + 2068,
    instrux + 2069,
    instrux + 2070,
    instrux + 2071,
    instrux + 2072,
    instrux + 2073,
    instrux + 2074,
    instrux + 2075,
    instrux + 2076,
    instrux + 2077,
    instrux + 2078,
    instrux + 2079,
    instrux + 2080,
    instrux + 2081,
    instrux + 2082,
    instrux + 2083,
    instrux + 2084,
    instrux + 2085,
    instrux + 2086,
    instrux + 2087,
    instrux + 2088,
    instrux + 2089,
    instrux + 2090,
    instrux + 2091,
    instrux + 2092,
    instrux + 2093,
    instrux + 2094,
    instrux + 2095,
    NULL
};

static const struct itemplate * const itbl_evex_map4_88[] = {
    instrux + 3077,
    instrux + 3078,
    instrux + 3079,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8a[] = {
    instrux + 32,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8b[] = {
    instrux + 33,
    instrux + 34,
    instrux + 35,
    NULL
};

static const struct itemplate * const itbl_evex_map4_8f[] = {
    instrux + 1775,
    instrux + 1776,
    NULL
};

static const struct itemplate * const itbl_evex_map4_a5[] = {
    instrux + 850,
    instrux + 851,
    instrux + 852,
    instrux + 853,
    instrux + 854,
    instrux + 855,
    NULL
};

static const struct itemplate * const itbl_evex_map4_ad[] = {
    instrux + 868,
    instrux + 869,
    instrux + 870,
    instrux + 871,
    instrux + 872,
    instrux + 873,
    NULL
};

static const struct itemplate * const itbl_evex_map4_af[] = {
    instrux + 762,
    instrux + 763,
    instrux + 764,
    instrux + 765,
    instrux + 766,
    instrux + 767,
    NULL
};

static const struct itemplate * const itbl_evex_map4_c0[] = {
    instrux + 533,
    instrux + 534,
    instrux + 557,
    instrux + 558,
    instrux + 581,
    instrux + 582,
    instrux + 605,
    instrux + 606,
    instrux + 629,
    instrux + 630,
    instrux + 653,
    instrux + 654,
    instrux + 677,
    instrux + 678,
    instrux + 701,
    instrux + 702,
    NULL
};

static const struct itemplate * const itbl_evex_map4_c1[] = {
    instrux + 535,
    instrux + 536,
    instrux + 537,
    instrux + 538,
    instrux + 539,
    instrux + 540,
    instrux + 559,
    instrux + 560,
    instrux + 561,
    instrux + 562,
    instrux + 563,
    instrux + 564,
    instrux + 583,
    instrux + 584,
    instrux + 585,
    instrux + 586,
    instrux + 587,
    instrux + 588,
    instrux + 607,
    instrux + 608,
    instrux + 609,
    instrux + 610,
    instrux + 611,
    instrux + 612,
    instrux + 631,
    instrux + 632,
    instrux + 633,
    instrux + 634,
    instrux + 635,
    instrux + 636,
    instrux + 655,
    instrux + 656,
    instrux + 657,
    instrux + 658,
    instrux + 659,
    instrux + 660,
    instrux + 679,
    instrux + 680,
    instrux + 681,
    instrux + 682,
    instrux + 683,
    instrux + 684,
    instrux + 703,
    instrux + 704,
    instrux + 705,
    instrux + 706,
    instrux + 707,
    instrux + 708,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d0[] = {
    instrux + 517,
    instrux + 518,
    instrux + 541,
    instrux + 542,
    instrux + 565,
    instrux + 566,
    instrux + 589,
    instrux + 590,
    instrux + 613,
    instrux + 614,
    instrux + 637,
    instrux + 638,
    instrux + 661,
    instrux + 662,
    instrux + 685,
    instrux + 686,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d1[] = {
    instrux + 519,
    instrux + 520,
    instrux + 521,
    instrux + 522,
    instrux + 523,
    instrux + 524,
    instrux + 543,
    instrux + 544,
    instrux + 545,
    instrux + 546,
    instrux + 547,
    instrux + 548,
    instrux + 567,
    instrux + 568,
    instrux + 569,
    instrux + 570,
    instrux + 571,
    instrux + 572,
    instrux + 591,
    instrux + 592,
    instrux + 593,
    instrux + 594,
    instrux + 595,
    instrux + 596,
    instrux + 615,
    instrux + 616,
    instrux + 617,
    instrux + 618,
    instrux + 619,
    instrux + 620,
    instrux + 639,
    instrux + 640,
    instrux + 641,
    instrux + 642,
    instrux + 643,
    instrux + 644,
    instrux + 663,
    instrux + 664,
    instrux + 665,
    instrux + 666,
    instrux + 667,
    instrux + 668,
    instrux + 687,
    instrux + 688,
    instrux + 689,
    instrux + 690,
    instrux + 691,
    instrux + 692,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d2[] = {
    instrux + 525,
    instrux + 526,
    instrux + 549,
    instrux + 550,
    instrux + 573,
    instrux + 574,
    instrux + 597,
    instrux + 598,
    instrux + 621,
    instrux + 622,
    instrux + 645,
    instrux + 646,
    instrux + 669,
    instrux + 670,
    instrux + 693,
    instrux + 694,
    NULL
};

static const struct itemplate * const itbl_evex_map4_d3[] = {
    instrux + 527,
    instrux + 528,
    instrux + 529,
    instrux + 530,
    instrux + 531,
    instrux + 532,
    instrux + 551,
    instrux + 552,
    instrux + 553,
    instrux + 554,
    instrux + 555,
    instrux + 556,
    instrux + 575,
    instrux + 576,
    instrux + 577,
    instrux + 578,
    instrux + 579,
    instrux + 580,
    instrux + 599,
    instrux + 600,
    instrux + 601,
    instrux + 602,
    instrux + 603,
    instrux + 604,
    instrux + 623,
    instrux + 624,
    instrux + 625,
    instrux + 626,
    instrux + 627,
    instrux + 628,
    instrux + 647,
    instrux + 648,
    instrux + 649,
    instrux + 650,
    instrux + 651,
    instrux + 652,
    instrux + 671,
    instrux + 672,
    instrux + 673,
    instrux + 674,
    instrux + 675,
    instrux + 676,
    instrux + 695,
    instrux + 696,
    instrux + 697,
    instrux + 698,
    instrux + 699,
    instrux + 700,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f0[] = {
    instrux + 2948,
    instrux + 3056,
    instrux + 3063,
    instrux + 3068,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f1[] = {
    instrux + 2951,
    instrux + 3057,
    instrux + 3058,
    instrux + 3064,
    instrux + 3065,
    instrux + 3066,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f2[] = {
    instrux + 1302,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f4[] = {
    instrux + 913,
    instrux + 914,
    instrux + 915,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f5[] = {
    instrux + 907,
    instrux + 908,
    instrux + 909,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f6[] = {
    instrux + 749,
    instrux + 782,
    instrux + 790,
    instrux + 798,
    instrux + 806,
    instrux + 807,
    instrux + 818,
    instrux + 819,
    instrux + 2096,
    instrux + 2097,
    instrux + 2098,
    instrux + 2099,
    instrux + 2100,
    instrux + 2101,
    instrux + 2102,
    instrux + 2103,
    instrux + 2104,
    instrux + 2105,
    instrux + 2106,
    instrux + 2107,
    instrux + 2108,
    instrux + 2109,
    instrux + 2110,
    instrux + 2111,
    instrux + 2160,
    instrux + 2161,
    instrux + 2162,
    instrux + 2163,
    instrux + 2164,
    instrux + 2165,
    instrux + 2166,
    instrux + 2167,
    instrux + 2168,
    instrux + 2169,
    instrux + 2170,
    instrux + 2171,
    instrux + 2172,
    instrux + 2173,
    instrux + 2174,
    instrux + 2175,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f7[] = {
    instrux + 750,
    instrux + 751,
    instrux + 752,
    instrux + 783,
    instrux + 784,
    instrux + 785,
    instrux + 791,
    instrux + 792,
    instrux + 793,
    instrux + 799,
    instrux + 800,
    instrux + 801,
    instrux + 808,
    instrux + 809,
    instrux + 810,
    instrux + 811,
    instrux + 812,
    instrux + 813,
    instrux + 820,
    instrux + 821,
    instrux + 822,
    instrux + 823,
    instrux + 824,
    instrux + 825,
    instrux + 2112,
    instrux + 2113,
    instrux + 2114,
    instrux + 2115,
    instrux + 2116,
    instrux + 2117,
    instrux + 2118,
    instrux + 2119,
    instrux + 2120,
    instrux + 2121,
    instrux + 2122,
    instrux + 2123,
    instrux + 2124,
    instrux + 2125,
    instrux + 2126,
    instrux + 2127,
    instrux + 2128,
    instrux + 2129,
    instrux + 2130,
    instrux + 2131,
    instrux + 2132,
    instrux + 2133,
    instrux + 2134,
    instrux + 2135,
    instrux + 2136,
    instrux + 2137,
    instrux + 2138,
    instrux + 2139,
    instrux + 2140,
    instrux + 2141,
    instrux + 2142,
    instrux + 2143,
    instrux + 2144,
    instrux + 2145,
    instrux + 2146,
    instrux + 2147,
    instrux + 2148,
    instrux + 2149,
    instrux + 2150,
    instrux + 2151,
    instrux + 2152,
    instrux + 2153,
    instrux + 2154,
    instrux + 2155,
    instrux + 2156,
    instrux + 2157,
    instrux + 2158,
    instrux + 2159,
    instrux + 2176,
    instrux + 2177,
    instrux + 2178,
    instrux + 2179,
    instrux + 2180,
    instrux + 2181,
    instrux + 2182,
    instrux + 2183,
    instrux + 2184,
    instrux + 2185,
    instrux + 2186,
    instrux + 2187,
    instrux + 2188,
    instrux + 2189,
    instrux + 2190,
    instrux + 2191,
    instrux + 2192,
    instrux + 2193,
    instrux + 2194,
    instrux + 2195,
    instrux + 2196,
    instrux + 2197,
    instrux + 2198,
    instrux + 2199,
    instrux + 2200,
    instrux + 2201,
    instrux + 2202,
    instrux + 2203,
    instrux + 2204,
    instrux + 2205,
    instrux + 2206,
    instrux + 2207,
    instrux + 2208,
    instrux + 2209,
    instrux + 2210,
    instrux + 2211,
    instrux + 2212,
    instrux + 2213,
    instrux + 2214,
    instrux + 2215,
    instrux + 2216,
    instrux + 2217,
    instrux + 2218,
    instrux + 2219,
    instrux + 2220,
    instrux + 2221,
    instrux + 2222,
    instrux + 2223,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f8[] = {
    instrux + 1343,
    instrux + 1347,
    instrux + 6716,
    instrux + 6849,
    instrux + 6853,
    NULL
};

static const struct itemplate * const itbl_evex_map4_f9[] = {
    instrux + 6711,
    instrux + 6712,
    NULL
};

static const struct itemplate * const itbl_evex_map4_fc[] = {
    instrux + 7184,
    instrux + 7185,
    instrux + 7188,
    instrux + 7189,
    instrux + 7192,
    instrux + 7193,
    instrux + 7196,
    instrux + 7197,
    NULL
};

static const struct itemplate * const itbl_evex_map4_fe[] = {
    instrux + 723,
    instrux + 724,
    instrux + 737,
    instrux + 738,
    NULL
};

static const struct itemplate * const itbl_evex_map4_ff[] = {
    instrux + 725,
    instrux + 726,
    instrux + 727,
    instrux + 728,
    instrux + 729,
    instrux + 730,
    instrux + 739,
    instrux + 740,
    instrux + 741,
    instrux + 742,
    instrux + 743,
    instrux + 744,
    instrux + 1773,
    instrux + 1774,
    NULL
};

static const struct itemplate * const * const itbl_evex_map4[256] = {
    itbl_evex_map4_00,
    itbl_evex_map4_01,
    itbl_evex_map4_02,
    itbl_evex_map4_03,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_08,
    itbl_evex_map4_09,
    itbl_evex_map4_0a,
    itbl_evex_map4_0b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_10,
    itbl_evex_map4_11,
    itbl_evex_map4_12,
    itbl_evex_map4_13,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_18,
    itbl_evex_map4_19,
    itbl_evex_map4_1a,
    itbl_evex_map4_1b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_20,
    itbl_evex_map4_21,
    itbl_evex_map4_22,
    itbl_evex_map4_23,
    itbl_evex_map4_24,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_28,
    itbl_evex_map4_29,
    itbl_evex_map4_2a,
    itbl_evex_map4_2b,
    itbl_evex_map4_2c,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_30,
    itbl_evex_map4_31,
    itbl_evex_map4_32,
    itbl_evex_map4_33,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_38,
    itbl_evex_map4_39,
    itbl_evex_map4_3a,
    itbl_evex_map4_3b,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_40,
    itbl_evex_map4_41,
    itbl_evex_map4_42,
    itbl_evex_map4_43,
    itbl_evex_map4_44,
    itbl_evex_map4_45,
    itbl_evex_map4_46,
    itbl_evex_map4_47,
    itbl_evex_map4_48,
    itbl_evex_map4_49,
    itbl_evex_map4_4a,
    itbl_evex_map4_4b,
    itbl_evex_map4_4c,
    itbl_evex_map4_4d,
    itbl_evex_map4_4e,
    itbl_evex_map4_4f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_60,
    itbl_evex_map4_61,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_65,
    itbl_evex_map4_66,
    NULL,
    NULL,
    itbl_evex_map4_69,
    NULL,
    itbl_evex_map4_6b,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_80,
    itbl_evex_map4_81,
    NULL,
    itbl_evex_map4_83,
    itbl_evex_map4_84,
    itbl_evex_map4_85,
    NULL,
    NULL,
    itbl_evex_map4_88,
    NULL,
    itbl_evex_map4_8a,
    itbl_evex_map4_8b,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_8f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_a5,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_ad,
    NULL,
    itbl_evex_map4_af,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_c0,
    itbl_evex_map4_c1,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_d0,
    itbl_evex_map4_d1,
    itbl_evex_map4_d2,
    itbl_evex_map4_d3,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map4_f0,
    itbl_evex_map4_f1,
    itbl_evex_map4_f2,
    NULL,
    itbl_evex_map4_f4,
    itbl_evex_map4_f5,
    itbl_evex_map4_f6,
    itbl_evex_map4_f7,
    itbl_evex_map4_f8,
    itbl_evex_map4_f9,
    NULL,
    NULL,
    itbl_evex_map4_fc,
    NULL,
    itbl_evex_map4_fe,
    itbl_evex_map4_ff,
};

static const struct itemplate * const itbl_evex_map5_10[] = {
    instrux + 7135,
    instrux + 7137,
    NULL
};

static const struct itemplate * const itbl_evex_map5_11[] = {
    instrux + 7136,
    instrux + 7138,
    NULL
};

static const struct itemplate * const itbl_evex_map5_18[] = {
    instrux + 7301,
    instrux + 7302,
    instrux + 7303,
    instrux + 7313,
    instrux + 7314,
    instrux + 7315,
    instrux + 7328,
    instrux + 7329,
    instrux + 7330,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1b[] = {
    instrux + 7304,
    instrux + 7305,
    instrux + 7306,
    instrux + 7316,
    instrux + 7317,
    instrux + 7318,
    instrux + 7331,
    instrux + 7332,
    instrux + 7333,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1d[] = {
    instrux + 6977,
    instrux + 6978,
    instrux + 6979,
    instrux + 6992,
    NULL
};

static const struct itemplate * const itbl_evex_map5_1e[] = {
    instrux + 7334,
    instrux + 7335,
    instrux + 7336,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2a[] = {
    instrux + 6990,
    instrux + 6991,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2c[] = {
    instrux + 7011,
    instrux + 7012,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2d[] = {
    instrux + 6985,
    instrux + 6986,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2e[] = {
    instrux + 7173,
    instrux + 7293,
    NULL
};

static const struct itemplate * const itbl_evex_map5_2f[] = {
    instrux + 6937,
    instrux + 7210,
    instrux + 7290,
    NULL
};

static const struct itemplate * const itbl_evex_map5_51[] = {
    instrux + 7165,
    instrux + 7166,
    instrux + 7167,
    instrux + 7168,
    instrux + 7283,
    instrux + 7284,
    instrux + 7285,
    NULL
};

static const struct itemplate * const itbl_evex_map5_57[] = {
    instrux + 7178,
    instrux + 7179,
    instrux + 7180,
    instrux + 7181,
    NULL
};

static const struct itemplate * const itbl_evex_map5_58[] = {
    instrux + 6929,
    instrux + 6930,
    instrux + 6931,
    instrux + 6932,
    instrux + 7204,
    instrux + 7205,
    instrux + 7206,
    NULL
};

static const struct itemplate * const itbl_evex_map5_59[] = {
    instrux + 7141,
    instrux + 7142,
    instrux + 7143,
    instrux + 7144,
    instrux + 7265,
    instrux + 7266,
    instrux + 7267,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5a[] = {
    instrux + 6941,
    instrux + 6942,
    instrux + 6943,
    instrux + 6947,
    instrux + 6948,
    instrux + 6949,
    instrux + 6983,
    instrux + 6984,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5b[] = {
    instrux + 6938,
    instrux + 6939,
    instrux + 6940,
    instrux + 6944,
    instrux + 6945,
    instrux + 6946,
    instrux + 6980,
    instrux + 6981,
    instrux + 6982,
    instrux + 6993,
    instrux + 6994,
    instrux + 6995,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5c[] = {
    instrux + 7169,
    instrux + 7170,
    instrux + 7171,
    instrux + 7172,
    instrux + 7286,
    instrux + 7287,
    instrux + 7288,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5d[] = {
    instrux + 7131,
    instrux + 7132,
    instrux + 7133,
    instrux + 7134,
    instrux + 7174,
    instrux + 7175,
    instrux + 7176,
    instrux + 7177,
    instrux + 7262,
    instrux + 7263,
    instrux + 7264,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5e[] = {
    instrux + 7029,
    instrux + 7030,
    instrux + 7031,
    instrux + 7032,
    instrux + 7211,
    instrux + 7212,
    instrux + 7213,
    NULL
};

static const struct itemplate * const itbl_evex_map5_5f[] = {
    instrux + 7127,
    instrux + 7128,
    instrux + 7129,
    instrux + 7130,
    instrux + 7259,
    instrux + 7260,
    instrux + 7261,
    NULL
};

static const struct itemplate * const itbl_evex_map5_68[] = {
    instrux + 7400,
    instrux + 7401,
    instrux + 7402,
    instrux + 7424,
    instrux + 7425,
    instrux + 7426,
    instrux + 7439,
    instrux + 7440,
    instrux + 7441,
    NULL
};

static const struct itemplate * const itbl_evex_map5_69[] = {
    instrux + 7394,
    instrux + 7395,
    instrux + 7396,
    instrux + 7418,
    instrux + 7419,
    instrux + 7420,
    instrux + 7433,
    instrux + 7434,
    instrux + 7435,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6a[] = {
    instrux + 7403,
    instrux + 7404,
    instrux + 7405,
    instrux + 7427,
    instrux + 7428,
    instrux + 7429,
    instrux + 7442,
    instrux + 7443,
    instrux + 7444,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6b[] = {
    instrux + 7397,
    instrux + 7398,
    instrux + 7399,
    instrux + 7421,
    instrux + 7422,
    instrux + 7423,
    instrux + 7436,
    instrux + 7437,
    instrux + 7438,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6c[] = {
    instrux + 7412,
    instrux + 7413,
    instrux + 7414,
    instrux + 7415,
    instrux + 7416,
    instrux + 7417,
    instrux + 7448,
    instrux + 7449,
    instrux + 7450,
    instrux + 7451,
    instrux + 7452,
    instrux + 7453,
    instrux + 7456,
    instrux + 7457,
    instrux + 7460,
    instrux + 7461,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6d[] = {
    instrux + 7406,
    instrux + 7407,
    instrux + 7408,
    instrux + 7409,
    instrux + 7410,
    instrux + 7411,
    instrux + 7430,
    instrux + 7431,
    instrux + 7432,
    instrux + 7445,
    instrux + 7446,
    instrux + 7447,
    instrux + 7454,
    instrux + 7455,
    instrux + 7458,
    instrux + 7459,
    NULL
};

static const struct itemplate * const itbl_evex_map5_6e[] = {
    instrux + 7139,
    instrux + 7464,
    NULL
};

static const struct itemplate * const itbl_evex_map5_74[] = {
    instrux + 7298,
    instrux + 7299,
    instrux + 7300,
    instrux + 7310,
    instrux + 7311,
    instrux + 7312,
    instrux + 7325,
    instrux + 7326,
    instrux + 7327,
    NULL
};

static const struct itemplate * const itbl_evex_map5_78[] = {
    instrux + 6999,
    instrux + 7000,
    instrux + 7001,
    instrux + 7002,
    instrux + 7003,
    instrux + 7004,
    instrux + 7013,
    instrux + 7014,
    NULL
};

static const struct itemplate * const itbl_evex_map5_79[] = {
    instrux + 6959,
    instrux + 6960,
    instrux + 6961,
    instrux + 6962,
    instrux + 6963,
    instrux + 6964,
    instrux + 6988,
    instrux + 6989,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7a[] = {
    instrux + 6996,
    instrux + 6997,
    instrux + 6998,
    instrux + 7015,
    instrux + 7016,
    instrux + 7017,
    instrux + 7018,
    instrux + 7019,
    instrux + 7020,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7b[] = {
    instrux + 6956,
    instrux + 6957,
    instrux + 6958,
    instrux + 7021,
    instrux + 7022,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7c[] = {
    instrux + 7005,
    instrux + 7006,
    instrux + 7007,
    instrux + 7008,
    instrux + 7009,
    instrux + 7010,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7d[] = {
    instrux + 6965,
    instrux + 6966,
    instrux + 6967,
    instrux + 6968,
    instrux + 6969,
    instrux + 6970,
    instrux + 7023,
    instrux + 7024,
    instrux + 7025,
    instrux + 7026,
    instrux + 7027,
    instrux + 7028,
    NULL
};

static const struct itemplate * const itbl_evex_map5_7e[] = {
    instrux + 7140,
    instrux + 7465,
    NULL
};

static const struct itemplate * const * const itbl_evex_map5[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_10,
    itbl_evex_map5_11,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_18,
    NULL,
    NULL,
    itbl_evex_map5_1b,
    NULL,
    itbl_evex_map5_1d,
    itbl_evex_map5_1e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_2a,
    NULL,
    itbl_evex_map5_2c,
    itbl_evex_map5_2d,
    itbl_evex_map5_2e,
    itbl_evex_map5_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_51,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_57,
    itbl_evex_map5_58,
    itbl_evex_map5_59,
    itbl_evex_map5_5a,
    itbl_evex_map5_5b,
    itbl_evex_map5_5c,
    itbl_evex_map5_5d,
    itbl_evex_map5_5e,
    itbl_evex_map5_5f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_68,
    itbl_evex_map5_69,
    itbl_evex_map5_6a,
    itbl_evex_map5_6b,
    itbl_evex_map5_6c,
    itbl_evex_map5_6d,
    itbl_evex_map5_6e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_74,
    NULL,
    NULL,
    NULL,
    itbl_evex_map5_78,
    itbl_evex_map5_79,
    itbl_evex_map5_7a,
    itbl_evex_map5_7b,
    itbl_evex_map5_7c,
    itbl_evex_map5_7d,
    itbl_evex_map5_7e,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map6_13[] = {
    instrux + 6953,
    instrux + 6954,
    instrux + 6955,
    instrux + 6987,
    NULL
};

static const struct itemplate * const itbl_evex_map6_2c[] = {
    instrux + 7161,
    instrux + 7162,
    instrux + 7163,
    instrux + 7280,
    instrux + 7281,
    instrux + 7282,
    NULL
};

static const struct itemplate * const itbl_evex_map6_2d[] = {
    instrux + 7164,
    NULL
};

static const struct itemplate * const itbl_evex_map6_42[] = {
    instrux + 7119,
    instrux + 7120,
    instrux + 7121,
    instrux + 7253,
    instrux + 7254,
    instrux + 7255,
    NULL
};

static const struct itemplate * const itbl_evex_map6_43[] = {
    instrux + 7122,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4c[] = {
    instrux + 7145,
    instrux + 7146,
    instrux + 7147,
    instrux + 7268,
    instrux + 7269,
    instrux + 7270,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4d[] = {
    instrux + 7148,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4e[] = {
    instrux + 7157,
    instrux + 7158,
    instrux + 7159,
    instrux + 7277,
    instrux + 7278,
    instrux + 7279,
    NULL
};

static const struct itemplate * const itbl_evex_map6_4f[] = {
    instrux + 7160,
    NULL
};

static const struct itemplate * const itbl_evex_map6_56[] = {
    instrux + 7033,
    instrux + 7034,
    instrux + 7035,
    instrux + 7036,
    instrux + 7037,
    instrux + 7038,
    NULL
};

static const struct itemplate * const itbl_evex_map6_57[] = {
    instrux + 7039,
    instrux + 7040,
    NULL
};

static const struct itemplate * const itbl_evex_map6_80[] = {
    instrux + 7466,
    instrux + 7467,
    instrux + 7468,
    instrux + 7469,
    NULL
};

static const struct itemplate * const itbl_evex_map6_81[] = {
    instrux + 7470,
    instrux + 7471,
    instrux + 7472,
    NULL
};

static const struct itemplate * const itbl_evex_map6_96[] = {
    instrux + 7049,
    instrux + 7050,
    instrux + 7051,
    NULL
};

static const struct itemplate * const itbl_evex_map6_97[] = {
    instrux + 7058,
    instrux + 7059,
    instrux + 7060,
    NULL
};

static const struct itemplate * const itbl_evex_map6_98[] = {
    instrux + 7067,
    instrux + 7068,
    instrux + 7069,
    instrux + 7214,
    instrux + 7215,
    instrux + 7216,
    NULL
};

static const struct itemplate * const itbl_evex_map6_99[] = {
    instrux + 7085,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9a[] = {
    instrux + 7091,
    instrux + 7092,
    instrux + 7093,
    instrux + 7223,
    instrux + 7224,
    instrux + 7225,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9b[] = {
    instrux + 7109,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9c[] = {
    instrux + 7076,
    instrux + 7077,
    instrux + 7078,
    instrux + 7232,
    instrux + 7233,
    instrux + 7234,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9d[] = {
    instrux + 7088,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9e[] = {
    instrux + 7100,
    instrux + 7101,
    instrux + 7102,
    instrux + 7241,
    instrux + 7242,
    instrux + 7243,
    NULL
};

static const struct itemplate * const itbl_evex_map6_9f[] = {
    instrux + 7112,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a6[] = {
    instrux + 7052,
    instrux + 7053,
    instrux + 7054,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a7[] = {
    instrux + 7061,
    instrux + 7062,
    instrux + 7063,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a8[] = {
    instrux + 7070,
    instrux + 7071,
    instrux + 7072,
    instrux + 7217,
    instrux + 7218,
    instrux + 7219,
    NULL
};

static const struct itemplate * const itbl_evex_map6_a9[] = {
    instrux + 7086,
    NULL
};

static const struct itemplate * const itbl_evex_map6_aa[] = {
    instrux + 7094,
    instrux + 7095,
    instrux + 7096,
    instrux + 7226,
    instrux + 7227,
    instrux + 7228,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ab[] = {
    instrux + 7110,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ac[] = {
    instrux + 7079,
    instrux + 7080,
    instrux + 7081,
    instrux + 7235,
    instrux + 7236,
    instrux + 7237,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ad[] = {
    instrux + 7089,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ae[] = {
    instrux + 7103,
    instrux + 7104,
    instrux + 7105,
    instrux + 7244,
    instrux + 7245,
    instrux + 7246,
    NULL
};

static const struct itemplate * const itbl_evex_map6_af[] = {
    instrux + 7113,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b6[] = {
    instrux + 7055,
    instrux + 7056,
    instrux + 7057,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b7[] = {
    instrux + 7064,
    instrux + 7065,
    instrux + 7066,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b8[] = {
    instrux + 7073,
    instrux + 7074,
    instrux + 7075,
    instrux + 7220,
    instrux + 7221,
    instrux + 7222,
    NULL
};

static const struct itemplate * const itbl_evex_map6_b9[] = {
    instrux + 7087,
    NULL
};

static const struct itemplate * const itbl_evex_map6_ba[] = {
    instrux + 7097,
    instrux + 7098,
    instrux + 7099,
    instrux + 7229,
    instrux + 7230,
    instrux + 7231,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bb[] = {
    instrux + 7111,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bc[] = {
    instrux + 7082,
    instrux + 7083,
    instrux + 7084,
    instrux + 7238,
    instrux + 7239,
    instrux + 7240,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bd[] = {
    instrux + 7090,
    NULL
};

static const struct itemplate * const itbl_evex_map6_be[] = {
    instrux + 7106,
    instrux + 7107,
    instrux + 7108,
    instrux + 7247,
    instrux + 7248,
    instrux + 7249,
    NULL
};

static const struct itemplate * const itbl_evex_map6_bf[] = {
    instrux + 7114,
    NULL
};

static const struct itemplate * const itbl_evex_map6_d6[] = {
    instrux + 7041,
    instrux + 7042,
    instrux + 7043,
    instrux + 7044,
    instrux + 7045,
    instrux + 7046,
    NULL
};

static const struct itemplate * const itbl_evex_map6_d7[] = {
    instrux + 7047,
    instrux + 7048,
    NULL
};

static const struct itemplate * const * const itbl_evex_map6[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_13,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_2c,
    itbl_evex_map6_2d,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_42,
    itbl_evex_map6_43,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_4c,
    itbl_evex_map6_4d,
    itbl_evex_map6_4e,
    itbl_evex_map6_4f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_56,
    itbl_evex_map6_57,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_80,
    itbl_evex_map6_81,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_96,
    itbl_evex_map6_97,
    itbl_evex_map6_98,
    itbl_evex_map6_99,
    itbl_evex_map6_9a,
    itbl_evex_map6_9b,
    itbl_evex_map6_9c,
    itbl_evex_map6_9d,
    itbl_evex_map6_9e,
    itbl_evex_map6_9f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_a6,
    itbl_evex_map6_a7,
    itbl_evex_map6_a8,
    itbl_evex_map6_a9,
    itbl_evex_map6_aa,
    itbl_evex_map6_ab,
    itbl_evex_map6_ac,
    itbl_evex_map6_ad,
    itbl_evex_map6_ae,
    itbl_evex_map6_af,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_b6,
    itbl_evex_map6_b7,
    itbl_evex_map6_b8,
    itbl_evex_map6_b9,
    itbl_evex_map6_ba,
    itbl_evex_map6_bb,
    itbl_evex_map6_bc,
    itbl_evex_map6_bd,
    itbl_evex_map6_be,
    itbl_evex_map6_bf,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map6_d6,
    itbl_evex_map6_d7,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_evex_map7_f6[] = {
    instrux + 1336,
    instrux + 1339,
    NULL
};

static const struct itemplate * const itbl_evex_map7_f8[] = {
    instrux + 1345,
    instrux + 1349,
    NULL
};

static const struct itemplate * const * const itbl_evex_map7[256] = {
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_evex_map7_f6,
    NULL,
    itbl_evex_map7_f8,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
};

static const struct itemplate * const itbl_rex2_map0_00[] = {
    instrux + 39,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_01[] = {
    instrux + 40,
    instrux + 41,
    instrux + 42,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_02[] = {
    instrux + 43,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_03[] = {
    instrux + 44,
    instrux + 45,
    instrux + 46,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_04[] = {
    instrux + 47,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_05[] = {
    instrux + 50,
    instrux + 53,
    instrux + 56,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_08[] = {
    instrux + 88,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_09[] = {
    instrux + 89,
    instrux + 90,
    instrux + 91,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0a[] = {
    instrux + 92,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0b[] = {
    instrux + 93,
    instrux + 94,
    instrux + 95,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0c[] = {
    instrux + 96,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_0d[] = {
    instrux + 99,
    instrux + 102,
    instrux + 105,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_10[] = {
    instrux + 137,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_11[] = {
    instrux + 138,
    instrux + 139,
    instrux + 140,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_12[] = {
    instrux + 141,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_13[] = {
    instrux + 142,
    instrux + 143,
    instrux + 144,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_14[] = {
    instrux + 145,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_15[] = {
    instrux + 148,
    instrux + 151,
    instrux + 154,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_18[] = {
    instrux + 186,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_19[] = {
    instrux + 187,
    instrux + 188,
    instrux + 189,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1a[] = {
    instrux + 190,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1b[] = {
    instrux + 191,
    instrux + 192,
    instrux + 193,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1c[] = {
    instrux + 194,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_1d[] = {
    instrux + 197,
    instrux + 200,
    instrux + 203,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_20[] = {
    instrux + 235,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_21[] = {
    instrux + 236,
    instrux + 237,
    instrux + 238,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_22[] = {
    instrux + 239,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_23[] = {
    instrux + 240,
    instrux + 241,
    instrux + 242,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_24[] = {
    instrux + 243,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_25[] = {
    instrux + 246,
    instrux + 249,
    instrux + 252,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_28[] = {
    instrux + 284,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_29[] = {
    instrux + 285,
    instrux + 286,
    instrux + 287,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2a[] = {
    instrux + 288,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2b[] = {
    instrux + 289,
    instrux + 290,
    instrux + 291,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2c[] = {
    instrux + 292,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_2d[] = {
    instrux + 295,
    instrux + 298,
    instrux + 301,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_30[] = {
    instrux + 333,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_31[] = {
    instrux + 334,
    instrux + 335,
    instrux + 336,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_32[] = {
    instrux + 337,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_33[] = {
    instrux + 338,
    instrux + 339,
    instrux + 340,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_34[] = {
    instrux + 341,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_35[] = {
    instrux + 344,
    instrux + 347,
    instrux + 350,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_38[] = {
    instrux + 382,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_39[] = {
    instrux + 383,
    instrux + 384,
    instrux + 385,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3a[] = {
    instrux + 386,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3b[] = {
    instrux + 387,
    instrux + 388,
    instrux + 389,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3c[] = {
    instrux + 390,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_3d[] = {
    instrux + 393,
    instrux + 396,
    instrux + 399,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_50[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_51[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_52[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_53[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_54[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_55[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_56[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_57[] = {
    instrux + 1737,
    instrux + 1738,
    instrux + 1739,
    instrux + 1771,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_58[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_59[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5a[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5b[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5c[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5d[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5e[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_5f[] = {
    instrux + 1749,
    instrux + 1750,
    instrux + 1751,
    instrux + 1772,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_63[] = {
    instrux + 1002,
    instrux + 1003,
    instrux + 1004,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_68[] = {
    instrux + 1746,
    instrux + 1748,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_69[] = {
    instrux + 759,
    instrux + 760,
    instrux + 761,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6a[] = {
    instrux + 1743,
    instrux + 1744,
    instrux + 1745,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6b[] = {
    instrux + 756,
    instrux + 757,
    instrux + 758,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6c[] = {
    instrux + 1280,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6d[] = {
    instrux + 1281,
    instrux + 1282,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6e[] = {
    instrux + 1283,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_6f[] = {
    instrux + 1284,
    instrux + 1285,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_80[] = {
    instrux + 48,
    instrux + 97,
    instrux + 146,
    instrux + 195,
    instrux + 244,
    instrux + 293,
    instrux + 342,
    instrux + 391,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_81[] = {
    instrux + 51,
    instrux + 54,
    instrux + 57,
    instrux + 100,
    instrux + 103,
    instrux + 106,
    instrux + 149,
    instrux + 152,
    instrux + 155,
    instrux + 198,
    instrux + 201,
    instrux + 204,
    instrux + 247,
    instrux + 250,
    instrux + 253,
    instrux + 296,
    instrux + 299,
    instrux + 302,
    instrux + 345,
    instrux + 348,
    instrux + 351,
    instrux + 394,
    instrux + 397,
    instrux + 400,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_83[] = {
    instrux + 49,
    instrux + 52,
    instrux + 55,
    instrux + 98,
    instrux + 101,
    instrux + 104,
    instrux + 147,
    instrux + 150,
    instrux + 153,
    instrux + 196,
    instrux + 199,
    instrux + 202,
    instrux + 245,
    instrux + 248,
    instrux + 251,
    instrux + 294,
    instrux + 297,
    instrux + 300,
    instrux + 343,
    instrux + 346,
    instrux + 349,
    instrux + 392,
    instrux + 395,
    instrux + 398,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_84[] = {
    instrux + 401,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_85[] = {
    instrux + 402,
    instrux + 403,
    instrux + 404,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_86[] = {
    instrux + 1024,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_87[] = {
    instrux + 1025,
    instrux + 1026,
    instrux + 1027,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_88[] = {
    instrux + 12,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_89[] = {
    instrux + 13,
    instrux + 14,
    instrux + 15,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8a[] = {
    instrux + 16,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8b[] = {
    instrux + 17,
    instrux + 18,
    instrux + 19,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8c[] = {
    instrux + 1380,
    instrux + 1381,
    instrux + 1382,
    instrux + 1383,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8d[] = {
    instrux + 36,
    instrux + 37,
    instrux + 38,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8e[] = {
    instrux + 1384,
    instrux + 1385,
    instrux + 1386,
    instrux + 1387,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_8f[] = {
    instrux + 1752,
    instrux + 1753,
    instrux + 1754,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_90[] = {
    instrux + 0,
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    instrux + 1352,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_91[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_92[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_93[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_94[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_95[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_96[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_97[] = {
    instrux + 1021,
    instrux + 1022,
    instrux + 1023,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_98[] = {
    instrux + 990,
    instrux + 992,
    instrux + 995,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_99[] = {
    instrux + 991,
    instrux + 993,
    instrux + 994,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9c[] = {
    instrux + 1252,
    instrux + 1253,
    instrux + 1254,
    instrux + 1255,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9d[] = {
    instrux + 1256,
    instrux + 1257,
    instrux + 1258,
    instrux + 1259,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9e[] = {
    instrux + 1250,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_9f[] = {
    instrux + 1249,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_a1[] = {
    instrux + 1028,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b0[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b1[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b2[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b3[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b4[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b5[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b6[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b7[] = {
    instrux + 20,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b8[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_b9[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ba[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bb[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bc[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bd[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_be[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_bf[] = {
    instrux + 21,
    instrux + 22,
    instrux + 23,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c0[] = {
    instrux + 421,
    instrux + 433,
    instrux + 445,
    instrux + 457,
    instrux + 469,
    instrux + 481,
    instrux + 493,
    instrux + 505,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c1[] = {
    instrux + 422,
    instrux + 423,
    instrux + 424,
    instrux + 434,
    instrux + 435,
    instrux + 436,
    instrux + 446,
    instrux + 447,
    instrux + 448,
    instrux + 458,
    instrux + 459,
    instrux + 460,
    instrux + 470,
    instrux + 471,
    instrux + 472,
    instrux + 482,
    instrux + 483,
    instrux + 484,
    instrux + 494,
    instrux + 495,
    instrux + 496,
    instrux + 506,
    instrux + 507,
    instrux + 508,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c2[] = {
    instrux + 1214,
    instrux + 1215,
    instrux + 1217,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c3[] = {
    instrux + 1210,
    instrux + 1211,
    instrux + 1213,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c6[] = {
    instrux + 24,
    instrux + 4372,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c7[] = {
    instrux + 25,
    instrux + 26,
    instrux + 27,
    instrux + 4375,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c8[] = {
    instrux + 1761,
    instrux + 1762,
    instrux + 1763,
    instrux + 1764,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_c9[] = {
    instrux + 1765,
    instrux + 1766,
    instrux + 1767,
    instrux + 1768,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ca[] = {
    instrux + 1222,
    instrux + 1223,
    instrux + 1224,
    instrux + 1225,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cb[] = {
    instrux + 1218,
    instrux + 1219,
    instrux + 1220,
    instrux + 1221,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cc[] = {
    instrux + 1228,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cd[] = {
    instrux + 1226,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_cf[] = {
    instrux + 1234,
    instrux + 1235,
    instrux + 1236,
    instrux + 1237,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d0[] = {
    instrux + 413,
    instrux + 425,
    instrux + 437,
    instrux + 449,
    instrux + 461,
    instrux + 473,
    instrux + 485,
    instrux + 497,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d1[] = {
    instrux + 414,
    instrux + 415,
    instrux + 416,
    instrux + 426,
    instrux + 427,
    instrux + 428,
    instrux + 438,
    instrux + 439,
    instrux + 440,
    instrux + 450,
    instrux + 451,
    instrux + 452,
    instrux + 462,
    instrux + 463,
    instrux + 464,
    instrux + 474,
    instrux + 475,
    instrux + 476,
    instrux + 486,
    instrux + 487,
    instrux + 488,
    instrux + 498,
    instrux + 499,
    instrux + 500,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d2[] = {
    instrux + 417,
    instrux + 429,
    instrux + 441,
    instrux + 453,
    instrux + 465,
    instrux + 477,
    instrux + 489,
    instrux + 501,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d3[] = {
    instrux + 418,
    instrux + 419,
    instrux + 420,
    instrux + 430,
    instrux + 431,
    instrux + 432,
    instrux + 442,
    instrux + 443,
    instrux + 444,
    instrux + 454,
    instrux + 455,
    instrux + 456,
    instrux + 466,
    instrux + 467,
    instrux + 468,
    instrux + 478,
    instrux + 479,
    instrux + 480,
    instrux + 490,
    instrux + 491,
    instrux + 492,
    instrux + 502,
    instrux + 503,
    instrux + 504,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d6[] = {
    instrux + 1788,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d7[] = {
    instrux + 1791,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d8[] = {
    instrux + 1462,
    instrux + 1465,
    instrux + 1467,
    instrux + 1492,
    instrux + 1494,
    instrux + 1495,
    instrux + 1500,
    instrux + 1502,
    instrux + 1503,
    instrux + 1508,
    instrux + 1511,
    instrux + 1513,
    instrux + 1516,
    instrux + 1520,
    instrux + 1521,
    instrux + 1572,
    instrux + 1576,
    instrux + 1577,
    instrux + 1613,
    instrux + 1617,
    instrux + 1618,
    instrux + 1621,
    instrux + 1625,
    instrux + 1626,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_d9[] = {
    instrux + 1460,
    instrux + 1461,
    instrux + 1474,
    instrux + 1505,
    instrux + 1506,
    instrux + 1545,
    instrux + 1559,
    instrux + 1562,
    instrux + 1563,
    instrux + 1564,
    instrux + 1565,
    instrux + 1566,
    instrux + 1567,
    instrux + 1568,
    instrux + 1569,
    instrux + 1570,
    instrux + 1571,
    instrux + 1584,
    instrux + 1586,
    instrux + 1587,
    instrux + 1590,
    instrux + 1591,
    instrux + 1592,
    instrux + 1593,
    instrux + 1594,
    instrux + 1597,
    instrux + 1599,
    instrux + 1600,
    instrux + 1601,
    instrux + 1602,
    instrux + 1605,
    instrux + 1606,
    instrux + 1607,
    instrux + 1629,
    instrux + 1639,
    instrux + 1640,
    instrux + 1641,
    instrux + 1642,
    instrux + 1643,
    instrux + 1644,
    instrux + 1645,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_da[] = {
    instrux + 1476,
    instrux + 1477,
    instrux + 1478,
    instrux + 1479,
    instrux + 1480,
    instrux + 1481,
    instrux + 1490,
    instrux + 1491,
    instrux + 1530,
    instrux + 1532,
    instrux + 1534,
    instrux + 1536,
    instrux + 1538,
    instrux + 1543,
    instrux + 1555,
    instrux + 1557,
    instrux + 1638,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_db[] = {
    instrux + 1475,
    instrux + 1482,
    instrux + 1483,
    instrux + 1484,
    instrux + 1485,
    instrux + 1486,
    instrux + 1487,
    instrux + 1488,
    instrux + 1489,
    instrux + 1496,
    instrux + 1497,
    instrux + 1507,
    instrux + 1525,
    instrux + 1540,
    instrux + 1546,
    instrux + 1547,
    instrux + 1549,
    instrux + 1553,
    instrux + 1561,
    instrux + 1580,
    instrux + 1581,
    instrux + 1582,
    instrux + 1583,
    instrux + 1598,
    instrux + 1609,
    instrux + 1632,
    instrux + 1633,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_dc[] = {
    instrux + 1463,
    instrux + 1464,
    instrux + 1466,
    instrux + 1493,
    instrux + 1501,
    instrux + 1509,
    instrux + 1510,
    instrux + 1512,
    instrux + 1517,
    instrux + 1518,
    instrux + 1519,
    instrux + 1573,
    instrux + 1574,
    instrux + 1575,
    instrux + 1614,
    instrux + 1615,
    instrux + 1616,
    instrux + 1622,
    instrux + 1623,
    instrux + 1624,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_dd[] = {
    instrux + 1526,
    instrux + 1527,
    instrux + 1554,
    instrux + 1560,
    instrux + 1585,
    instrux + 1588,
    instrux + 1595,
    instrux + 1596,
    instrux + 1603,
    instrux + 1604,
    instrux + 1608,
    instrux + 1610,
    instrux + 1611,
    instrux + 1630,
    instrux + 1631,
    instrux + 1636,
    instrux + 1637,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_de[] = {
    instrux + 1468,
    instrux + 1469,
    instrux + 1504,
    instrux + 1514,
    instrux + 1515,
    instrux + 1522,
    instrux + 1523,
    instrux + 1531,
    instrux + 1533,
    instrux + 1535,
    instrux + 1537,
    instrux + 1539,
    instrux + 1544,
    instrux + 1556,
    instrux + 1558,
    instrux + 1578,
    instrux + 1579,
    instrux + 1619,
    instrux + 1620,
    instrux + 1627,
    instrux + 1628,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_df[] = {
    instrux + 1470,
    instrux + 1471,
    instrux + 1472,
    instrux + 1473,
    instrux + 1498,
    instrux + 1499,
    instrux + 1528,
    instrux + 1529,
    instrux + 1541,
    instrux + 1542,
    instrux + 1548,
    instrux + 1550,
    instrux + 1551,
    instrux + 1552,
    instrux + 1589,
    instrux + 1612,
    instrux + 1634,
    instrux + 1635,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_e9[] = {
    instrux + 1034,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f1[] = {
    instrux + 1227,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f4[] = {
    instrux + 1351,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f5[] = {
    instrux + 1248,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f6[] = {
    instrux + 409,
    instrux + 745,
    instrux + 774,
    instrux + 786,
    instrux + 794,
    instrux + 802,
    instrux + 814,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f7[] = {
    instrux + 410,
    instrux + 411,
    instrux + 412,
    instrux + 746,
    instrux + 747,
    instrux + 748,
    instrux + 775,
    instrux + 776,
    instrux + 777,
    instrux + 787,
    instrux + 788,
    instrux + 789,
    instrux + 795,
    instrux + 796,
    instrux + 797,
    instrux + 803,
    instrux + 804,
    instrux + 805,
    instrux + 815,
    instrux + 816,
    instrux + 817,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f8[] = {
    instrux + 1240,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_f9[] = {
    instrux + 1244,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fa[] = {
    instrux + 1242,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fb[] = {
    instrux + 1246,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fc[] = {
    instrux + 1241,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fd[] = {
    instrux + 1245,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_fe[] = {
    instrux + 719,
    instrux + 733,
    NULL
};

static const struct itemplate * const itbl_rex2_map0_ff[] = {
    instrux + 720,
    instrux + 721,
    instrux + 722,
    instrux + 734,
    instrux + 735,
    instrux + 736,
    instrux + 1037,
    instrux + 1041,
    instrux + 1042,
    instrux + 1045,
    instrux + 1201,
    instrux + 1205,
    instrux + 1206,
    instrux + 1209,
    instrux + 1740,
    instrux + 1741,
    instrux + 1742,
    instrux + 1789,
    NULL
};

static const struct itemplate * const * const itbl_rex2_map0[256] = {
    itbl_rex2_map0_00,
    itbl_rex2_map0_01,
    itbl_rex2_map0_02,
    itbl_rex2_map0_03,
    itbl_rex2_map0_04,
    itbl_rex2_map0_05,
    NULL,
    NULL,
    itbl_rex2_map0_08,
    itbl_rex2_map0_09,
    itbl_rex2_map0_0a,
    itbl_rex2_map0_0b,
    itbl_rex2_map0_0c,
    itbl_rex2_map0_0d,
    NULL,
    NULL,
    itbl_rex2_map0_10,
    itbl_rex2_map0_11,
    itbl_rex2_map0_12,
    itbl_rex2_map0_13,
    itbl_rex2_map0_14,
    itbl_rex2_map0_15,
    NULL,
    NULL,
    itbl_rex2_map0_18,
    itbl_rex2_map0_19,
    itbl_rex2_map0_1a,
    itbl_rex2_map0_1b,
    itbl_rex2_map0_1c,
    itbl_rex2_map0_1d,
    NULL,
    NULL,
    itbl_rex2_map0_20,
    itbl_rex2_map0_21,
    itbl_rex2_map0_22,
    itbl_rex2_map0_23,
    itbl_rex2_map0_24,
    itbl_rex2_map0_25,
    NULL,
    NULL,
    itbl_rex2_map0_28,
    itbl_rex2_map0_29,
    itbl_rex2_map0_2a,
    itbl_rex2_map0_2b,
    itbl_rex2_map0_2c,
    itbl_rex2_map0_2d,
    NULL,
    NULL,
    itbl_rex2_map0_30,
    itbl_rex2_map0_31,
    itbl_rex2_map0_32,
    itbl_rex2_map0_33,
    itbl_rex2_map0_34,
    itbl_rex2_map0_35,
    NULL,
    NULL,
    itbl_rex2_map0_38,
    itbl_rex2_map0_39,
    itbl_rex2_map0_3a,
    itbl_rex2_map0_3b,
    itbl_rex2_map0_3c,
    itbl_rex2_map0_3d,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_50,
    itbl_rex2_map0_51,
    itbl_rex2_map0_52,
    itbl_rex2_map0_53,
    itbl_rex2_map0_54,
    itbl_rex2_map0_55,
    itbl_rex2_map0_56,
    itbl_rex2_map0_57,
    itbl_rex2_map0_58,
    itbl_rex2_map0_59,
    itbl_rex2_map0_5a,
    itbl_rex2_map0_5b,
    itbl_rex2_map0_5c,
    itbl_rex2_map0_5d,
    itbl_rex2_map0_5e,
    itbl_rex2_map0_5f,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_63,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_68,
    itbl_rex2_map0_69,
    itbl_rex2_map0_6a,
    itbl_rex2_map0_6b,
    itbl_rex2_map0_6c,
    itbl_rex2_map0_6d,
    itbl_rex2_map0_6e,
    itbl_rex2_map0_6f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_80,
    itbl_rex2_map0_81,
    NULL,
    itbl_rex2_map0_83,
    itbl_rex2_map0_84,
    itbl_rex2_map0_85,
    itbl_rex2_map0_86,
    itbl_rex2_map0_87,
    itbl_rex2_map0_88,
    itbl_rex2_map0_89,
    itbl_rex2_map0_8a,
    itbl_rex2_map0_8b,
    itbl_rex2_map0_8c,
    itbl_rex2_map0_8d,
    itbl_rex2_map0_8e,
    itbl_rex2_map0_8f,
    itbl_rex2_map0_90,
    itbl_rex2_map0_91,
    itbl_rex2_map0_92,
    itbl_rex2_map0_93,
    itbl_rex2_map0_94,
    itbl_rex2_map0_95,
    itbl_rex2_map0_96,
    itbl_rex2_map0_97,
    itbl_rex2_map0_98,
    itbl_rex2_map0_99,
    NULL,
    NULL,
    itbl_rex2_map0_9c,
    itbl_rex2_map0_9d,
    itbl_rex2_map0_9e,
    itbl_rex2_map0_9f,
    NULL,
    itbl_rex2_map0_a1,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_b0,
    itbl_rex2_map0_b1,
    itbl_rex2_map0_b2,
    itbl_rex2_map0_b3,
    itbl_rex2_map0_b4,
    itbl_rex2_map0_b5,
    itbl_rex2_map0_b6,
    itbl_rex2_map0_b7,
    itbl_rex2_map0_b8,
    itbl_rex2_map0_b9,
    itbl_rex2_map0_ba,
    itbl_rex2_map0_bb,
    itbl_rex2_map0_bc,
    itbl_rex2_map0_bd,
    itbl_rex2_map0_be,
    itbl_rex2_map0_bf,
    itbl_rex2_map0_c0,
    itbl_rex2_map0_c1,
    itbl_rex2_map0_c2,
    itbl_rex2_map0_c3,
    NULL,
    NULL,
    itbl_rex2_map0_c6,
    itbl_rex2_map0_c7,
    itbl_rex2_map0_c8,
    itbl_rex2_map0_c9,
    itbl_rex2_map0_ca,
    itbl_rex2_map0_cb,
    itbl_rex2_map0_cc,
    itbl_rex2_map0_cd,
    NULL,
    itbl_rex2_map0_cf,
    itbl_rex2_map0_d0,
    itbl_rex2_map0_d1,
    itbl_rex2_map0_d2,
    itbl_rex2_map0_d3,
    NULL,
    NULL,
    itbl_rex2_map0_d6,
    itbl_rex2_map0_d7,
    itbl_rex2_map0_d8,
    itbl_rex2_map0_d9,
    itbl_rex2_map0_da,
    itbl_rex2_map0_db,
    itbl_rex2_map0_dc,
    itbl_rex2_map0_dd,
    itbl_rex2_map0_de,
    itbl_rex2_map0_df,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_e9,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map0_f1,
    NULL,
    NULL,
    itbl_rex2_map0_f4,
    itbl_rex2_map0_f5,
    itbl_rex2_map0_f6,
    itbl_rex2_map0_f7,
    itbl_rex2_map0_f8,
    itbl_rex2_map0_f9,
    itbl_rex2_map0_fa,
    itbl_rex2_map0_fb,
    itbl_rex2_map0_fc,
    itbl_rex2_map0_fd,
    itbl_rex2_map0_fe,
    itbl_rex2_map0_ff,
};

static const struct itemplate * const itbl_rex2_map1_00[] = {
    instrux + 1190,
    instrux + 1191,
    instrux + 1192,
    instrux + 1193,
    instrux + 1194,
    instrux + 1195,
    instrux + 1428,
    instrux + 1429,
    instrux + 1431,
    instrux + 1438,
    instrux + 1439,
    instrux + 1440,
    instrux + 1441,
    instrux + 1442,
    instrux + 1443,
    instrux + 1444,
    instrux + 1445,
    instrux + 1452,
    instrux + 1453,
    instrux + 1454,
    instrux + 1455,
    instrux + 1456,
    instrux + 1457,
    instrux + 1458,
    instrux + 1459,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_01[] = {
    instrux + 1238,
    instrux + 1239,
    instrux + 1243,
    instrux + 1247,
    instrux + 1289,
    instrux + 1294,
    instrux + 1295,
    instrux + 1296,
    instrux + 1303,
    instrux + 1304,
    instrux + 1305,
    instrux + 1306,
    instrux + 1309,
    instrux + 1320,
    instrux + 1321,
    instrux + 1322,
    instrux + 1323,
    instrux + 1324,
    instrux + 1337,
    instrux + 1340,
    instrux + 1341,
    instrux + 1353,
    instrux + 1354,
    instrux + 1355,
    instrux + 1356,
    instrux + 1357,
    instrux + 1358,
    instrux + 1359,
    instrux + 1360,
    instrux + 1361,
    instrux + 1362,
    instrux + 1430,
    instrux + 1432,
    instrux + 1433,
    instrux + 1434,
    instrux + 1435,
    instrux + 1436,
    instrux + 1437,
    instrux + 1446,
    instrux + 1447,
    instrux + 1448,
    instrux + 1449,
    instrux + 1450,
    instrux + 1451,
    instrux + 1778,
    instrux + 2699,
    instrux + 2700,
    instrux + 2927,
    instrux + 2928,
    instrux + 2929,
    instrux + 2931,
    instrux + 2932,
    instrux + 2933,
    instrux + 2934,
    instrux + 2939,
    instrux + 2940,
    instrux + 2941,
    instrux + 2944,
    instrux + 2952,
    instrux + 2953,
    instrux + 2954,
    instrux + 2955,
    instrux + 4376,
    instrux + 4377,
    instrux + 6704,
    instrux + 6705,
    instrux + 6717,
    instrux + 6718,
    instrux + 6825,
    instrux + 6826,
    instrux + 6827,
    instrux + 6835,
    instrux + 6836,
    instrux + 6837,
    instrux + 6854,
    instrux + 6855,
    instrux + 6856,
    instrux + 7198,
    instrux + 7200,
    instrux + 7201,
    instrux + 7202,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_02[] = {
    instrux + 1422,
    instrux + 1423,
    instrux + 1424,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_03[] = {
    instrux + 1425,
    instrux + 1426,
    instrux + 1427,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_05[] = {
    instrux + 1230,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_06[] = {
    instrux + 1318,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_07[] = {
    instrux + 1233,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_08[] = {
    instrux + 1297,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_09[] = {
    instrux + 1298,
    instrux + 1299,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0b[] = {
    instrux + 1787,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0d[] = {
    instrux + 1705,
    instrux + 1706,
    instrux + 4378,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0e[] = {
    instrux + 1524,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_0f[] = {
    instrux + 1667,
    instrux + 1675,
    instrux + 1676,
    instrux + 1677,
    instrux + 1678,
    instrux + 1679,
    instrux + 1680,
    instrux + 1681,
    instrux + 1682,
    instrux + 1683,
    instrux + 1684,
    instrux + 1685,
    instrux + 1686,
    instrux + 1687,
    instrux + 1688,
    instrux + 1689,
    instrux + 1690,
    instrux + 1691,
    instrux + 1696,
    instrux + 2736,
    instrux + 2737,
    instrux + 2738,
    instrux + 2739,
    instrux + 2740,
    instrux + 3081,
    instrux + 3082,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_10[] = {
    instrux + 2674,
    instrux + 2676,
    instrux + 2901,
    instrux + 2903,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_11[] = {
    instrux + 2675,
    instrux + 2677,
    instrux + 2902,
    instrux + 2904,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_12[] = {
    instrux + 2668,
    instrux + 2670,
    instrux + 2898,
    instrux + 2924,
    instrux + 2926,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_13[] = {
    instrux + 2669,
    instrux + 2897,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_14[] = {
    instrux + 2693,
    instrux + 2915,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_15[] = {
    instrux + 2692,
    instrux + 2914,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_16[] = {
    instrux + 2665,
    instrux + 2667,
    instrux + 2896,
    instrux + 2925,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_17[] = {
    instrux + 2666,
    instrux + 2895,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_18[] = {
    instrux + 2713,
    instrux + 2714,
    instrux + 2715,
    instrux + 2716,
    instrux + 2717,
    instrux + 2718,
    instrux + 6719,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_19[] = {
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1a[] = {
    instrux + 4380,
    instrux + 4382,
    instrux + 4383,
    instrux + 4385,
    instrux + 4389,
    instrux + 4390,
    instrux + 4393,
    instrux + 4395,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1b[] = {
    instrux + 4379,
    instrux + 4386,
    instrux + 4388,
    instrux + 4391,
    instrux + 4392,
    instrux + 4396,
    instrux + 4398,
    instrux + 4400,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1c[] = {
    instrux + 6708,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1d[] = {
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1e[] = {
    instrux + 6829,
    instrux + 6830,
    instrux + 6833,
    instrux + 6834,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_1f[] = {
    instrux + 1,
    instrux + 2,
    instrux + 3,
    instrux + 7473,
    instrux + 7474,
    instrux + 7475,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_20[] = {
    instrux + 1327,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_21[] = {
    instrux + 1331,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_22[] = {
    instrux + 1328,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_23[] = {
    instrux + 1332,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_28[] = {
    instrux + 2663,
    instrux + 2893,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_29[] = {
    instrux + 2664,
    instrux + 2894,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2a[] = {
    instrux + 2647,
    instrux + 2649,
    instrux + 2650,
    instrux + 2873,
    instrux + 2879,
    instrux + 2880,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2b[] = {
    instrux + 2673,
    instrux + 2745,
    instrux + 2992,
    instrux + 2993,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2c[] = {
    instrux + 2653,
    instrux + 2654,
    instrux + 2655,
    instrux + 2882,
    instrux + 2885,
    instrux + 2886,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2d[] = {
    instrux + 2648,
    instrux + 2651,
    instrux + 2652,
    instrux + 2871,
    instrux + 2876,
    instrux + 2877,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2e[] = {
    instrux + 2691,
    instrux + 2913,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_2f[] = {
    instrux + 2646,
    instrux + 2867,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_40[] = {
    instrux + 2235,
    instrux + 2251,
    instrux + 2267,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_41[] = {
    instrux + 2232,
    instrux + 2248,
    instrux + 2264,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_42[] = {
    instrux + 2225,
    instrux + 2241,
    instrux + 2257,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_43[] = {
    instrux + 2229,
    instrux + 2245,
    instrux + 2261,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_44[] = {
    instrux + 2239,
    instrux + 2255,
    instrux + 2271,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_45[] = {
    instrux + 2234,
    instrux + 2250,
    instrux + 2266,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_46[] = {
    instrux + 2228,
    instrux + 2244,
    instrux + 2260,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_47[] = {
    instrux + 2224,
    instrux + 2240,
    instrux + 2256,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_48[] = {
    instrux + 2238,
    instrux + 2254,
    instrux + 2270,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_49[] = {
    instrux + 2233,
    instrux + 2249,
    instrux + 2265,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4a[] = {
    instrux + 2236,
    instrux + 2252,
    instrux + 2268,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4b[] = {
    instrux + 2237,
    instrux + 2253,
    instrux + 2269,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4c[] = {
    instrux + 2227,
    instrux + 2243,
    instrux + 2259,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4d[] = {
    instrux + 2231,
    instrux + 2247,
    instrux + 2263,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4e[] = {
    instrux + 2230,
    instrux + 2246,
    instrux + 2262,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_4f[] = {
    instrux + 2226,
    instrux + 2242,
    instrux + 2258,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_50[] = {
    instrux + 1666,
    instrux + 2671,
    instrux + 2672,
    instrux + 2899,
    instrux + 2900,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_51[] = {
    instrux + 1659,
    instrux + 2686,
    instrux + 2687,
    instrux + 2909,
    instrux + 2910,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_52[] = {
    instrux + 1694,
    instrux + 2683,
    instrux + 2684,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_53[] = {
    instrux + 2681,
    instrux + 2682,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_54[] = {
    instrux + 1674,
    instrux + 2627,
    instrux + 2848,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_55[] = {
    instrux + 1726,
    instrux + 2626,
    instrux + 2847,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_56[] = {
    instrux + 2680,
    instrux + 2907,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_57[] = {
    instrux + 2694,
    instrux + 2916,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_58[] = {
    instrux + 1703,
    instrux + 2624,
    instrux + 2625,
    instrux + 2845,
    instrux + 2846,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_59[] = {
    instrux + 1697,
    instrux + 2678,
    instrux + 2679,
    instrux + 2905,
    instrux + 2906,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5a[] = {
    instrux + 1702,
    instrux + 2872,
    instrux + 2875,
    instrux + 2878,
    instrux + 2881,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5b[] = {
    instrux + 1701,
    instrux + 2869,
    instrux + 2874,
    instrux + 2884,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5c[] = {
    instrux + 1700,
    instrux + 2689,
    instrux + 2690,
    instrux + 2911,
    instrux + 2912,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5d[] = {
    instrux + 1695,
    instrux + 2661,
    instrux + 2662,
    instrux + 2891,
    instrux + 2892,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5e[] = {
    instrux + 1692,
    instrux + 2656,
    instrux + 2657,
    instrux + 2887,
    instrux + 2888,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_5f[] = {
    instrux + 2659,
    instrux + 2660,
    instrux + 2889,
    instrux + 2890,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_60[] = {
    instrux + 1734,
    instrux + 2840,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_61[] = {
    instrux + 1736,
    instrux + 2841,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_62[] = {
    instrux + 1735,
    instrux + 2842,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_63[] = {
    instrux + 1654,
    instrux + 2764,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_64[] = {
    instrux + 1671,
    instrux + 2783,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_65[] = {
    instrux + 1673,
    instrux + 2784,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_66[] = {
    instrux + 1672,
    instrux + 2785,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_67[] = {
    instrux + 1655,
    instrux + 2766,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_68[] = {
    instrux + 1731,
    instrux + 2836,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_69[] = {
    instrux + 1733,
    instrux + 2837,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6a[] = {
    instrux + 1732,
    instrux + 2838,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6b[] = {
    instrux + 1653,
    instrux + 2765,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6c[] = {
    instrux + 2843,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6d[] = {
    instrux + 2839,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6e[] = {
    instrux + 1647,
    instrux + 1651,
    instrux + 2749,
    instrux + 2750,
    instrux + 2761,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_6f[] = {
    instrux + 1649,
    instrux + 2752,
    instrux + 2754,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_70[] = {
    instrux + 2735,
    instrux + 2803,
    instrux + 2804,
    instrux + 2805,
    instrux + 2806,
    instrux + 2807,
    instrux + 2808,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_71[] = {
    instrux + 1712,
    instrux + 1716,
    instrux + 1722,
    instrux + 2811,
    instrux + 2817,
    instrux + 2822,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_72[] = {
    instrux + 1708,
    instrux + 1714,
    instrux + 1718,
    instrux + 2813,
    instrux + 2819,
    instrux + 2824,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_73[] = {
    instrux + 1710,
    instrux + 1720,
    instrux + 2809,
    instrux + 2815,
    instrux + 2820,
    instrux + 2826,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_74[] = {
    instrux + 1668,
    instrux + 2780,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_75[] = {
    instrux + 1670,
    instrux + 2781,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_76[] = {
    instrux + 1669,
    instrux + 2782,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_77[] = {
    instrux + 1646,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_78[] = {
    instrux + 2938,
    instrux + 2988,
    instrux + 2990,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_79[] = {
    instrux + 2943,
    instrux + 2989,
    instrux + 2991,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7c[] = {
    instrux + 2919,
    instrux + 2920,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7d[] = {
    instrux + 2921,
    instrux + 2922,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7e[] = {
    instrux + 1648,
    instrux + 1652,
    instrux + 2748,
    instrux + 2751,
    instrux + 2757,
    instrux + 2760,
    instrux + 2762,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_7f[] = {
    instrux + 1650,
    instrux + 2753,
    instrux + 2755,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_90[] = {
    instrux + 2523,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_91[] = {
    instrux + 2520,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_92[] = {
    instrux + 2513,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_93[] = {
    instrux + 2517,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_94[] = {
    instrux + 2527,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_95[] = {
    instrux + 2522,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_96[] = {
    instrux + 2516,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_97[] = {
    instrux + 2512,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_98[] = {
    instrux + 2526,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_99[] = {
    instrux + 2521,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9a[] = {
    instrux + 2524,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9b[] = {
    instrux + 2525,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9c[] = {
    instrux + 2515,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9d[] = {
    instrux + 2519,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9e[] = {
    instrux + 2518,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_9f[] = {
    instrux + 2514,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a0[] = {
    instrux + 1405,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a1[] = {
    instrux + 1410,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a2[] = {
    instrux + 1319,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a3[] = {
    instrux + 874,
    instrux + 875,
    instrux + 876,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a4[] = {
    instrux + 838,
    instrux + 839,
    instrux + 840,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a5[] = {
    instrux + 841,
    instrux + 842,
    instrux + 843,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a6[] = {
    instrux + 4036,
    instrux + 4037,
    instrux + 4038,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a7[] = {
    instrux + 4030,
    instrux + 4031,
    instrux + 4032,
    instrux + 4033,
    instrux + 4034,
    instrux + 4035,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a8[] = {
    instrux + 1406,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_a9[] = {
    instrux + 1411,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_aa[] = {
    instrux + 1350,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ab[] = {
    instrux + 892,
    instrux + 893,
    instrux + 894,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ac[] = {
    instrux + 856,
    instrux + 857,
    instrux + 858,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ad[] = {
    instrux + 859,
    instrux + 860,
    instrux + 861,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ae[] = {
    instrux + 1286,
    instrux + 1287,
    instrux + 1288,
    instrux + 1290,
    instrux + 1291,
    instrux + 1292,
    instrux + 1293,
    instrux + 1363,
    instrux + 1364,
    instrux + 1365,
    instrux + 1366,
    instrux + 1367,
    instrux + 1412,
    instrux + 1413,
    instrux + 1414,
    instrux + 1415,
    instrux + 1416,
    instrux + 1417,
    instrux + 1418,
    instrux + 1419,
    instrux + 2658,
    instrux + 2688,
    instrux + 2695,
    instrux + 2696,
    instrux + 2697,
    instrux + 2698,
    instrux + 2719,
    instrux + 2746,
    instrux + 2747,
    instrux + 6706,
    instrux + 6707,
    instrux + 6828,
    instrux + 6831,
    instrux + 6832,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_af[] = {
    instrux + 753,
    instrux + 754,
    instrux + 755,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b0[] = {
    instrux + 1011,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b1[] = {
    instrux + 1012,
    instrux + 1013,
    instrux + 1014,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b2[] = {
    instrux + 1398,
    instrux + 1399,
    instrux + 1400,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b3[] = {
    instrux + 886,
    instrux + 887,
    instrux + 888,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b4[] = {
    instrux + 1392,
    instrux + 1393,
    instrux + 1394,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b5[] = {
    instrux + 1395,
    instrux + 1396,
    instrux + 1397,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b6[] = {
    instrux + 1005,
    instrux + 1006,
    instrux + 1007,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b7[] = {
    instrux + 1008,
    instrux + 1009,
    instrux + 1010,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b8[] = {
    instrux + 1187,
    instrux + 1188,
    instrux + 1189,
    instrux + 3074,
    instrux + 3075,
    instrux + 3076,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_b9[] = {
    instrux + 1783,
    instrux + 1784,
    instrux + 1785,
    instrux + 1786,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ba[] = {
    instrux + 877,
    instrux + 878,
    instrux + 879,
    instrux + 883,
    instrux + 884,
    instrux + 885,
    instrux + 889,
    instrux + 890,
    instrux + 891,
    instrux + 895,
    instrux + 896,
    instrux + 897,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bb[] = {
    instrux + 880,
    instrux + 881,
    instrux + 882,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bc[] = {
    instrux + 898,
    instrux + 899,
    instrux + 900,
    instrux + 910,
    instrux + 911,
    instrux + 912,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bd[] = {
    instrux + 901,
    instrux + 902,
    instrux + 903,
    instrux + 904,
    instrux + 905,
    instrux + 906,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_be[] = {
    instrux + 996,
    instrux + 997,
    instrux + 998,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_bf[] = {
    instrux + 999,
    instrux + 1000,
    instrux + 1001,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c0[] = {
    instrux + 1017,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c1[] = {
    instrux + 1018,
    instrux + 1019,
    instrux + 1020,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c2[] = {
    instrux + 2628,
    instrux + 2629,
    instrux + 2630,
    instrux + 2631,
    instrux + 2632,
    instrux + 2633,
    instrux + 2634,
    instrux + 2635,
    instrux + 2636,
    instrux + 2637,
    instrux + 2638,
    instrux + 2639,
    instrux + 2640,
    instrux + 2641,
    instrux + 2642,
    instrux + 2643,
    instrux + 2644,
    instrux + 2645,
    instrux + 2849,
    instrux + 2850,
    instrux + 2851,
    instrux + 2852,
    instrux + 2853,
    instrux + 2854,
    instrux + 2855,
    instrux + 2856,
    instrux + 2857,
    instrux + 2858,
    instrux + 2859,
    instrux + 2860,
    instrux + 2861,
    instrux + 2862,
    instrux + 2863,
    instrux + 2864,
    instrux + 2865,
    instrux + 2866,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c3[] = {
    instrux + 2743,
    instrux + 2744,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c4[] = {
    instrux + 2725,
    instrux + 2726,
    instrux + 2727,
    instrux + 2787,
    instrux + 2788,
    instrux + 2789,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c5[] = {
    instrux + 2724,
    instrux + 2786,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c6[] = {
    instrux + 2685,
    instrux + 2908,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c7[] = {
    instrux + 1015,
    instrux + 1016,
    instrux + 1310,
    instrux + 1311,
    instrux + 1312,
    instrux + 1313,
    instrux + 1314,
    instrux + 1315,
    instrux + 1316,
    instrux + 1317,
    instrux + 2930,
    instrux + 2935,
    instrux + 2936,
    instrux + 2945,
    instrux + 7199,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c8[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_c9[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ca[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cb[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cc[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cd[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ce[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_cf[] = {
    instrux + 976,
    instrux + 977,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d0[] = {
    instrux + 2917,
    instrux + 2918,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d1[] = {
    instrux + 1721,
    instrux + 2821,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d2[] = {
    instrux + 1717,
    instrux + 2823,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d3[] = {
    instrux + 1719,
    instrux + 2825,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d4[] = {
    instrux + 2770,
    instrux + 2771,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d5[] = {
    instrux + 1699,
    instrux + 2798,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d6[] = {
    instrux + 2756,
    instrux + 2758,
    instrux + 2759,
    instrux + 2763,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d7[] = {
    instrux + 2732,
    instrux + 2795,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d8[] = {
    instrux + 1728,
    instrux + 2834,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_d9[] = {
    instrux + 1729,
    instrux + 2835,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_da[] = {
    instrux + 2731,
    instrux + 2794,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_db[] = {
    instrux + 1664,
    instrux + 2776,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_dc[] = {
    instrux + 1661,
    instrux + 2774,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_dd[] = {
    instrux + 1662,
    instrux + 2775,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_de[] = {
    instrux + 2729,
    instrux + 2792,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_df[] = {
    instrux + 1665,
    instrux + 2777,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e0[] = {
    instrux + 2722,
    instrux + 2778,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e1[] = {
    instrux + 1715,
    instrux + 2816,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e2[] = {
    instrux + 1713,
    instrux + 2818,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e3[] = {
    instrux + 2723,
    instrux + 2779,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e4[] = {
    instrux + 2733,
    instrux + 2796,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e5[] = {
    instrux + 1698,
    instrux + 2797,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e6[] = {
    instrux + 2868,
    instrux + 2870,
    instrux + 2883,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e7[] = {
    instrux + 2721,
    instrux + 2742,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e8[] = {
    instrux + 1725,
    instrux + 2832,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_e9[] = {
    instrux + 1727,
    instrux + 2833,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ea[] = {
    instrux + 2730,
    instrux + 2793,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_eb[] = {
    instrux + 1704,
    instrux + 2801,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ec[] = {
    instrux + 1658,
    instrux + 2772,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ed[] = {
    instrux + 1660,
    instrux + 2773,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ee[] = {
    instrux + 2728,
    instrux + 2791,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ef[] = {
    instrux + 1777,
    instrux + 2844,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f0[] = {
    instrux + 2923,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f1[] = {
    instrux + 1711,
    instrux + 2810,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f2[] = {
    instrux + 1707,
    instrux + 2812,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f3[] = {
    instrux + 1709,
    instrux + 2814,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f4[] = {
    instrux + 2799,
    instrux + 2800,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f5[] = {
    instrux + 1693,
    instrux + 2790,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f6[] = {
    instrux + 2734,
    instrux + 2802,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f7[] = {
    instrux + 2720,
    instrux + 2741,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f8[] = {
    instrux + 1723,
    instrux + 2827,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_f9[] = {
    instrux + 1730,
    instrux + 2828,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fa[] = {
    instrux + 1724,
    instrux + 2829,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fb[] = {
    instrux + 2830,
    instrux + 2831,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fc[] = {
    instrux + 1656,
    instrux + 2767,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fd[] = {
    instrux + 1663,
    instrux + 2768,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_fe[] = {
    instrux + 1657,
    instrux + 2769,
    NULL
};

static const struct itemplate * const itbl_rex2_map1_ff[] = {
    instrux + 1779,
    instrux + 1780,
    instrux + 1781,
    instrux + 1782,
    NULL
};

static const struct itemplate * const * const itbl_rex2_map1[256] = {
    itbl_rex2_map1_00,
    itbl_rex2_map1_01,
    itbl_rex2_map1_02,
    itbl_rex2_map1_03,
    NULL,
    itbl_rex2_map1_05,
    itbl_rex2_map1_06,
    itbl_rex2_map1_07,
    itbl_rex2_map1_08,
    itbl_rex2_map1_09,
    NULL,
    itbl_rex2_map1_0b,
    NULL,
    itbl_rex2_map1_0d,
    itbl_rex2_map1_0e,
    itbl_rex2_map1_0f,
    itbl_rex2_map1_10,
    itbl_rex2_map1_11,
    itbl_rex2_map1_12,
    itbl_rex2_map1_13,
    itbl_rex2_map1_14,
    itbl_rex2_map1_15,
    itbl_rex2_map1_16,
    itbl_rex2_map1_17,
    itbl_rex2_map1_18,
    itbl_rex2_map1_19,
    itbl_rex2_map1_1a,
    itbl_rex2_map1_1b,
    itbl_rex2_map1_1c,
    itbl_rex2_map1_1d,
    itbl_rex2_map1_1e,
    itbl_rex2_map1_1f,
    itbl_rex2_map1_20,
    itbl_rex2_map1_21,
    itbl_rex2_map1_22,
    itbl_rex2_map1_23,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_28,
    itbl_rex2_map1_29,
    itbl_rex2_map1_2a,
    itbl_rex2_map1_2b,
    itbl_rex2_map1_2c,
    itbl_rex2_map1_2d,
    itbl_rex2_map1_2e,
    itbl_rex2_map1_2f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_40,
    itbl_rex2_map1_41,
    itbl_rex2_map1_42,
    itbl_rex2_map1_43,
    itbl_rex2_map1_44,
    itbl_rex2_map1_45,
    itbl_rex2_map1_46,
    itbl_rex2_map1_47,
    itbl_rex2_map1_48,
    itbl_rex2_map1_49,
    itbl_rex2_map1_4a,
    itbl_rex2_map1_4b,
    itbl_rex2_map1_4c,
    itbl_rex2_map1_4d,
    itbl_rex2_map1_4e,
    itbl_rex2_map1_4f,
    itbl_rex2_map1_50,
    itbl_rex2_map1_51,
    itbl_rex2_map1_52,
    itbl_rex2_map1_53,
    itbl_rex2_map1_54,
    itbl_rex2_map1_55,
    itbl_rex2_map1_56,
    itbl_rex2_map1_57,
    itbl_rex2_map1_58,
    itbl_rex2_map1_59,
    itbl_rex2_map1_5a,
    itbl_rex2_map1_5b,
    itbl_rex2_map1_5c,
    itbl_rex2_map1_5d,
    itbl_rex2_map1_5e,
    itbl_rex2_map1_5f,
    itbl_rex2_map1_60,
    itbl_rex2_map1_61,
    itbl_rex2_map1_62,
    itbl_rex2_map1_63,
    itbl_rex2_map1_64,
    itbl_rex2_map1_65,
    itbl_rex2_map1_66,
    itbl_rex2_map1_67,
    itbl_rex2_map1_68,
    itbl_rex2_map1_69,
    itbl_rex2_map1_6a,
    itbl_rex2_map1_6b,
    itbl_rex2_map1_6c,
    itbl_rex2_map1_6d,
    itbl_rex2_map1_6e,
    itbl_rex2_map1_6f,
    itbl_rex2_map1_70,
    itbl_rex2_map1_71,
    itbl_rex2_map1_72,
    itbl_rex2_map1_73,
    itbl_rex2_map1_74,
    itbl_rex2_map1_75,
    itbl_rex2_map1_76,
    itbl_rex2_map1_77,
    itbl_rex2_map1_78,
    itbl_rex2_map1_79,
    NULL,
    NULL,
    itbl_rex2_map1_7c,
    itbl_rex2_map1_7d,
    itbl_rex2_map1_7e,
    itbl_rex2_map1_7f,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    NULL,
    itbl_rex2_map1_90,
    itbl_rex2_map1_91,
    itbl_rex2_map1_92,
    itbl_rex2_map1_93,
    itbl_rex2_map1_94,
    itbl_rex2_map1_95,
    itbl_rex2_map1_96,
    itbl_rex2_map1_97,
    itbl_rex2_map1_98,
    itbl_rex2_map1_99,
    itbl_rex2_map1_9a,
    itbl_rex2_map1_9b,
    itbl_rex2_map1_9c,
    itbl_rex2_map1_9d,
    itbl_rex2_map1_9e,
    itbl_rex2_map1_9f,
    itbl_rex2_map1_a0,
    itbl_rex2_map1_a1,
    itbl_rex2_map1_a2,
    itbl_rex2_map1_a3,
    itbl_rex2_map1_a4,
    itbl_rex2_map1_a5,
    itbl_rex2_map1_a6,
    itbl_rex2_map1_a7,
    itbl_rex2_map1_a8,
    itbl_rex2_map1_a9,
    itbl_rex2_map1_aa,
    itbl_rex2_map1_ab,
    itbl_rex2_map1_ac,
    itbl_rex2_map1_ad,
    itbl_rex2_map1_ae,
    itbl_rex2_map1_af,
    itbl_rex2_map1_b0,
    itbl_rex2_map1_b1,
    itbl_rex2_map1_b2,
    itbl_rex2_map1_b3,
    itbl_rex2_map1_b4,
    itbl_rex2_map1_b5,
    itbl_rex2_map1_b6,
    itbl_rex2_map1_b7,
    itbl_rex2_map1_b8,
    itbl_rex2_map1_b9,
    itbl_rex2_map1_ba,
    itbl_rex2_map1_bb,
    itbl_rex2_map1_bc,
    itbl_rex2_map1_bd,
    itbl_rex2_map1_be,
    itbl_rex2_map1_bf,
    itbl_rex2_map1_c0,
    itbl_rex2_map1_c1,
    itbl_rex2_map1_c2,
    itbl_rex2_map1_c3,
    itbl_rex2_map1_c4,
    itbl_rex2_map1_c5,
    itbl_rex2_map1_c6,
    itbl_rex2_map1_c7,
    itbl_rex2_map1_c8,
    itbl_rex2_map1_c9,
    itbl_rex2_map1_ca,
    itbl_rex2_map1_cb,
    itbl_rex2_map1_cc,
    itbl_rex2_map1_cd,
    itbl_rex2_map1_ce,
    itbl_rex2_map1_cf,
    itbl_rex2_map1_d0,
    itbl_rex2_map1_d1,
    itbl_rex2_map1_d2,
    itbl_rex2_map1_d3,
    itbl_rex2_map1_d4,
    itbl_rex2_map1_d5,
    itbl_rex2_map1_d6,
    itbl_rex2_map1_d7,
    itbl_rex2_map1_d8,
    itbl_rex2_map1_d9,
    itbl_rex2_map1_da,
    itbl_rex2_map1_db,
    itbl_rex2_map1_dc,
    itbl_rex2_map1_dd,
    itbl_rex2_map1_de,
    itbl_rex2_map1_df,
    itbl_rex2_map1_e0,
    itbl_rex2_map1_e1,
    itbl_rex2_map1_e2,
    itbl_rex2_map1_e3,
    itbl_rex2_map1_e4,
    itbl_rex2_map1_e5,
    itbl_rex2_map1_e6,
    itbl_rex2_map1_e7,
    itbl_rex2_map1_e8,
    itbl_rex2_map1_e9,
    itbl_rex2_map1_ea,
    itbl_rex2_map1_eb,
    itbl_rex2_map1_ec,
    itbl_rex2_map1_ed,
    itbl_rex2_map1_ee,
    itbl_rex2_map1_ef,
    itbl_rex2_map1_f0,
    itbl_rex2_map1_f1,
    itbl_rex2_map1_f2,
    itbl_rex2_map1_f3,
    itbl_rex2_map1_f4,
    itbl_rex2_map1_f5,
    itbl_rex2_map1_f6,
    itbl_rex2_map1_f7,
    itbl_rex2_map1_f8,
    itbl_rex2_map1_f9,
    itbl_rex2_map1_fa,
    itbl_rex2_map1_fb,
    itbl_rex2_map1_fc,
    itbl_rex2_map1_fd,
    itbl_rex2_map1_fe,
    itbl_rex2_map1_ff,
};

const struct itemplate * const * const * const
ndisasm_itable[] = {
    /* ---- novex ---- */
    /*  0 */ itbl_novex_map0,
    /*  1 */ itbl_novex_map1,
    /*  2 */ itbl_novex_map2,
    /*  3 */ itbl_novex_map3,
    /* ---- vex ---- */
    /*  0 */ NULL,
    /*  1 */ itbl_vex_map1,
    /*  2 */ itbl_vex_map2,
    /*  3 */ itbl_vex_map3,
    /*  4 */ NULL,
    /*  5 */ itbl_vex_map5,
    /*  6 */ NULL,
    /*  7 */ itbl_vex_map7,
    /*  8 */ NULL,
    /*  9 */ NULL,
    /* 10 */ NULL,
    /* 11 */ NULL,
    /* 12 */ NULL,
    /* 13 */ NULL,
    /* 14 */ NULL,
    /* 15 */ NULL,
    /* 16 */ NULL,
    /* 17 */ NULL,
    /* 18 */ NULL,
    /* 19 */ NULL,
    /* 20 */ NULL,
    /* 21 */ NULL,
    /* 22 */ NULL,
    /* 23 */ NULL,
    /* 24 */ NULL,
    /* 25 */ NULL,
    /* 26 */ NULL,
    /* 27 */ NULL,
    /* 28 */ NULL,
    /* 29 */ NULL,
    /* 30 */ NULL,
    /* 31 */ NULL,
    /* ---- xop ---- */
    /*  8 */ itbl_xop_map8,
    /*  9 */ itbl_xop_map9,
    /* 10 */ itbl_xop_map10,
    /* 11 */ NULL,
    /* 12 */ NULL,
    /* 13 */ NULL,
    /* 14 */ NULL,
    /* 15 */ NULL,
    /* 16 */ NULL,
    /* 17 */ NULL,
    /* 18 */ NULL,
    /* 19 */ NULL,
    /* 20 */ NULL,
    /* 21 */ NULL,
    /* 22 */ NULL,
    /* 23 */ NULL,
    /* 24 */ NULL,
    /* 25 */ NULL,
    /* 26 */ NULL,
    /* 27 */ NULL,
    /* 28 */ NULL,
    /* 29 */ NULL,
    /* 30 */ NULL,
    /* 31 */ NULL,
    /* ---- evex ---- */
    /*  0 */ NULL,
    /*  1 */ itbl_evex_map1,
    /*  2 */ itbl_evex_map2,
    /*  3 */ itbl_evex_map3,
    /*  4 */ itbl_evex_map4,
    /*  5 */ itbl_evex_map5,
    /*  6 */ itbl_evex_map6,
    /*  7 */ itbl_evex_map7,
    /* ---- rex2 ---- */
    /*  0 */ itbl_rex2_map0,
    /*  1 */ itbl_rex2_map1,
};
